// Copyright (c) 2026 The Chromium Embedded Framework Authors. All rights
// reserved. Use of this source code is governed by a BSD-style license that
// can be found in the LICENSE file.
//
// ---------------------------------------------------------------------------
//
// This file was generated by the CEF translator tool. If making changes by
// hand only do so within the body of existing method and function
// implementations. See the translator.README.txt file in the tools directory
// for more information.
//
// $hash=48639f64214f366e6028b8276f1081caa6f4b880$
//

#ifndef CEF_LIBCEF_DLL_CTOCPP_TEST_API_VERSION_TEST_REF_PTR_LIBRARY_CHILD_CHILD_V2_CTOCPP_H_
#define CEF_LIBCEF_DLL_CTOCPP_TEST_API_VERSION_TEST_REF_PTR_LIBRARY_CHILD_CHILD_V2_CTOCPP_H_
#pragma once

#if !defined(WRAPPING_CEF_SHARED)
#error This file can be included wrapper-side only
#endif

#include "include/capi/test/cef_api_version_test_capi.h"
#include "include/test/cef_api_version_test.h"
#include "libcef_dll/ctocpp/ctocpp_ref_counted.h"

#if CEF_API_ADDED(13302)

// Wrap a C structure with a C++ class.
// This class may be instantiated and accessed wrapper-side only.
class CefApiVersionTestRefPtrLibraryChildChildV2CToCpp
    : public CefCToCppRefCounted<
          CefApiVersionTestRefPtrLibraryChildChildV2CToCpp,
          CefApiVersionTestRefPtrLibraryChildChildV2,
          cef_api_version_test_ref_ptr_library_child_child_v2_t> {
public:
  CefApiVersionTestRefPtrLibraryChildChildV2CToCpp();
  virtual ~CefApiVersionTestRefPtrLibraryChildChildV2CToCpp();

  // CefApiVersionTestRefPtrLibraryChildChildV2 methods.
  int GetOtherOtherValue() override;
  void SetOtherOtherValue(int value) override;

  // CefApiVersionTestRefPtrLibraryChild methods.
  int GetOtherValue() override;
  void SetOtherValue(int value) override;

  // CefApiVersionTestRefPtrLibrary methods.
  int GetValueLegacy() override;
  void SetValueLegacy(int value) override;
#if CEF_API_ADDED(CEF_EXPERIMENTAL)
  int GetValueExp() override;
#endif
#if CEF_API_ADDED(CEF_EXPERIMENTAL)
  void SetValueExp(int value) override;
#endif
#if CEF_API_REMOVED(13301)
  int GetValue() override;
#endif
#if CEF_API_REMOVED(13301)
  void SetValue(int value) override;
#endif
#if CEF_API_RANGE(13301, 13302)
  int GetValueV1() override;
#endif
#if CEF_API_RANGE(13301, 13302)
  void SetValueV1(int value) override;
#endif
#if CEF_API_ADDED(13302)
  int GetValueV2() override;
#endif
#if CEF_API_ADDED(13302)
  void SetValueV2(int value) override;
#endif
};

constexpr auto CefApiVersionTestRefPtrLibraryChildChildV2CToCpp_Wrap =
    CefApiVersionTestRefPtrLibraryChildChildV2CToCpp::Wrap;
constexpr auto CefApiVersionTestRefPtrLibraryChildChildV2CToCpp_Unwrap =
    CefApiVersionTestRefPtrLibraryChildChildV2CToCpp::Unwrap;

#endif // CEF_API_ADDED(13302)

#endif // CEF_LIBCEF_DLL_CTOCPP_TEST_API_VERSION_TEST_REF_PTR_LIBRARY_CHILD_CHILD_V2_CTOCPP_H_
