// Copyright (c) 2026 The Chromium Embedded Framework Authors. All rights
// reserved. Use of this source code is governed by a BSD-style license that
// can be found in the LICENSE file.
//
// ---------------------------------------------------------------------------
//
// This file was generated by the CEF translator tool. If making changes by
// hand only do so within the body of existing method and function
// implementations. See the translator.README.txt file in the tools directory
// for more information.
//
// $hash=8ef24e07dd12e22bfdccf2f73cb7d230357d1e5b$
//

#ifndef CEF_LIBCEF_DLL_CTOCPP_COMPONENT_CTOCPP_H_
#define CEF_LIBCEF_DLL_CTOCPP_COMPONENT_CTOCPP_H_
#pragma once

#if !defined(WRAPPING_CEF_SHARED)
#error This file can be included wrapper-side only
#endif

#include "include/capi/cef_component_updater_capi.h"
#include "include/cef_component_updater.h"
#include "libcef_dll/ctocpp/ctocpp_ref_counted.h"

#if CEF_API_ADDED(14600)

// Wrap a C structure with a C++ class.
// This class may be instantiated and accessed wrapper-side only.
class CefComponentCToCpp
    : public CefCToCppRefCounted<CefComponentCToCpp, CefComponent,
                                 cef_component_t> {
public:
  CefComponentCToCpp();
  virtual ~CefComponentCToCpp();

  // CefComponent methods.
  CefString GetID() override;
  CefString GetName() override;
  CefString GetVersion() override;
  cef_component_state_t GetState() override;
};

constexpr auto CefComponentCToCpp_Wrap = CefComponentCToCpp::Wrap;
constexpr auto CefComponentCToCpp_Unwrap = CefComponentCToCpp::Unwrap;

#endif // CEF_API_ADDED(14600)

#endif // CEF_LIBCEF_DLL_CTOCPP_COMPONENT_CTOCPP_H_
