// Copyright (c) 2026 The Chromium Embedded Framework Authors. All rights
// reserved. Use of this source code is governed by a BSD-style license that
// can be found in the LICENSE file.
//
// ---------------------------------------------------------------------------
//
// This file was generated by the CEF translator tool. If making changes by
// hand only do so within the body of existing method and function
// implementations. See the translator.README.txt file in the tools directory
// for more information.
//
// $hash=03efc42e6c4f72e04cae2f4b375707cf1d213037$
//

#ifndef CEF_LIBCEF_DLL_CTOCPP_VIEWS_LAYOUT_CTOCPP_H_
#define CEF_LIBCEF_DLL_CTOCPP_VIEWS_LAYOUT_CTOCPP_H_
#pragma once

#if !defined(WRAPPING_CEF_SHARED)
#error This file can be included wrapper-side only
#endif

#include "include/capi/views/cef_box_layout_capi.h"
#include "include/capi/views/cef_fill_layout_capi.h"
#include "include/capi/views/cef_layout_capi.h"
#include "include/views/cef_box_layout.h"
#include "include/views/cef_fill_layout.h"
#include "include/views/cef_layout.h"
#include "libcef_dll/ctocpp/ctocpp_ref_counted.h"

// Wrap a C structure with a C++ class.
// This class may be instantiated and accessed wrapper-side only.
class CefLayoutCToCpp
    : public CefCToCppRefCounted<CefLayoutCToCpp, CefLayout, cef_layout_t> {
public:
  CefLayoutCToCpp();
  virtual ~CefLayoutCToCpp();

  // CefLayout methods.
  CefRefPtr<CefBoxLayout> AsBoxLayout() override;
  CefRefPtr<CefFillLayout> AsFillLayout() override;
  bool IsValid() override;
};

constexpr auto CefLayoutCToCpp_Wrap = CefLayoutCToCpp::Wrap;
constexpr auto CefLayoutCToCpp_Unwrap = CefLayoutCToCpp::Unwrap;

#endif // CEF_LIBCEF_DLL_CTOCPP_VIEWS_LAYOUT_CTOCPP_H_
