// Copyright (c) 2026 Marshall A. Greenblatt. All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are
// met:
//
//    * Redistributions of source code must retain the above copyright
// notice, this list of conditions and the following disclaimer.
//    * Redistributions in binary form must reproduce the above
// copyright notice, this list of conditions and the following disclaimer
// in the documentation and/or other materials provided with the
// distribution.
//    * Neither the name of Google Inc. nor the name Chromium Embedded
// Framework nor the names of its contributors may be used to endorse
// or promote products derived from this software without specific prior
// written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// ---------------------------------------------------------------------------
//
// This file is generated by the make_pack_header.py tool.
//

#ifndef CEF_INCLUDE_CEF_PACK_STRINGS_H_
#define CEF_INCLUDE_CEF_PACK_STRINGS_H_
#pragma once

// ---------------------------------------------------------------------------
// From app_locale_settings.h:

#define IDS_WEB_FONT_FAMILY 49563
#define IDS_WEB_FONT_SIZE 49564

// ---------------------------------------------------------------------------
// From auto_image_annotation_strings.h:

#define IDS_AX_UNLABELED_IMAGE_ROLE_DESCRIPTION 49570
#define IDS_AX_IMAGE_ELIGIBLE_FOR_ANNOTATION 49572
#define IDS_AX_IMAGE_ANNOTATION_PENDING 49573
#define IDS_AX_IMAGE_ANNOTATION_ADULT 49574
#define IDS_AX_IMAGE_ANNOTATION_NO_DESCRIPTION 49575
#define IDS_AX_IMAGE_ANNOTATION_OCR_CONTEXT 49576
#define IDS_AX_IMAGE_ANNOTATION_DESCRIPTION_CONTEXT 49577
#define IDS_AX_IMAGE_ANNOTATION_ICON_PLUS 49578
#define IDS_AX_IMAGE_ANNOTATION_ICON_ARROW_BACKWARD 49579
#define IDS_AX_IMAGE_ANNOTATION_ICON_ARROW_FORWARD 49580
#define IDS_AX_IMAGE_ANNOTATION_ICON_CALL 49581
#define IDS_AX_IMAGE_ANNOTATION_ICON_CHAT 49582
#define IDS_AX_IMAGE_ANNOTATION_ICON_CHECK 49583
#define IDS_AX_IMAGE_ANNOTATION_ICON_X 49584
#define IDS_AX_IMAGE_ANNOTATION_ICON_DELETE 49585
#define IDS_AX_IMAGE_ANNOTATION_ICON_EDIT 49586
#define IDS_AX_IMAGE_ANNOTATION_ICON_EMOJI 49587
#define IDS_AX_IMAGE_ANNOTATION_ICON_END_CALL 49588
#define IDS_AX_IMAGE_ANNOTATION_ICON_V_DOWNWARD 49589
#define IDS_AX_IMAGE_ANNOTATION_ICON_HEART 49590
#define IDS_AX_IMAGE_ANNOTATION_ICON_HOME 49591
#define IDS_AX_IMAGE_ANNOTATION_ICON_INFO 49592
#define IDS_AX_IMAGE_ANNOTATION_ICON_LAUNCH_APPS 49593
#define IDS_AX_IMAGE_ANNOTATION_ICON_THUMBS_UP 49594
#define IDS_AX_IMAGE_ANNOTATION_ICON_THREE_BARS 49595
#define IDS_AX_IMAGE_ANNOTATION_ICON_THREE_DOTS 49596
#define IDS_AX_IMAGE_ANNOTATION_ICON_NOTIFICATIONS 49597
#define IDS_AX_IMAGE_ANNOTATION_ICON_PAUSE 49598
#define IDS_AX_IMAGE_ANNOTATION_ICON_PLAY 49599
#define IDS_AX_IMAGE_ANNOTATION_ICON_REFRESH 49600
#define IDS_AX_IMAGE_ANNOTATION_ICON_MAGNIFYING_GLASS 49601
#define IDS_AX_IMAGE_ANNOTATION_ICON_SEND 49602
#define IDS_AX_IMAGE_ANNOTATION_ICON_SETTINGS 49603
#define IDS_AX_IMAGE_ANNOTATION_ICON_SHARE 49604
#define IDS_AX_IMAGE_ANNOTATION_ICON_STAR 49605
#define IDS_AX_IMAGE_ANNOTATION_ICON_TAKE_PHOTO 49606
#define IDS_AX_IMAGE_ANNOTATION_ICON_TIME 49607
#define IDS_AX_IMAGE_ANNOTATION_ICON_VIDEOCAM 49608
#define IDS_AX_IMAGE_ANNOTATION_ICON_EXPAND 49609
#define IDS_AX_IMAGE_ANNOTATION_ICON_CONTRACT 49610
#define IDS_AX_IMAGE_ANNOTATION_ICON_GOOGLE 49611
#define IDS_AX_IMAGE_ANNOTATION_ICON_TWITTER 49612
#define IDS_AX_IMAGE_ANNOTATION_ICON_FACEBOOK 49613
#define IDS_AX_IMAGE_ANNOTATION_ICON_ASSISTANT 49614
#define IDS_AX_IMAGE_ANNOTATION_ICON_WEATHER 49615
#define IDS_AX_IMAGE_ANNOTATION_ICON_SHOPPING_CART 49616
#define IDS_AX_IMAGE_ANNOTATION_ICON_UPLOAD 49617
#define IDS_AX_IMAGE_ANNOTATION_ICON_QUESTION 49618
#define IDS_AX_IMAGE_ANNOTATION_ICON_MIC 49619
#define IDS_AX_IMAGE_ANNOTATION_ICON_MIC_MUTE 49620
#define IDS_AX_IMAGE_ANNOTATION_ICON_GALLERY 49621
#define IDS_AX_IMAGE_ANNOTATION_ICON_COMPASS 49622
#define IDS_AX_IMAGE_ANNOTATION_ICON_PEOPLE 49623
#define IDS_AX_IMAGE_ANNOTATION_ICON_ARROW_UPWARD 49624
#define IDS_AX_IMAGE_ANNOTATION_ICON_ENVELOPE 49625
#define IDS_AX_IMAGE_ANNOTATION_ICON_EMOJI_FACE 49626
#define IDS_AX_IMAGE_ANNOTATION_ICON_PAPERCLIP 49627
#define IDS_AX_IMAGE_ANNOTATION_ICON_CAST 49628
#define IDS_AX_IMAGE_ANNOTATION_ICON_VOLUME_UP 49629
#define IDS_AX_IMAGE_ANNOTATION_ICON_VOLUME_DOWN 49630
#define IDS_AX_IMAGE_ANNOTATION_ICON_VOLUME_STATE 49631
#define IDS_AX_IMAGE_ANNOTATION_ICON_VOLUME_MUTE 49632
#define IDS_AX_IMAGE_ANNOTATION_ICON_STOP 49633
#define IDS_AX_IMAGE_ANNOTATION_ICON_SHOPPING_BAG 49634
#define IDS_AX_IMAGE_ANNOTATION_ICON_LIST 49635
#define IDS_AX_IMAGE_ANNOTATION_ICON_LOCATION 49636
#define IDS_AX_IMAGE_ANNOTATION_ICON_CALENDAR 49637
#define IDS_AX_IMAGE_ANNOTATION_ICON_THUMBS_DOWN 49638
#define IDS_AX_IMAGE_ANNOTATION_ICON_HEADSET 49639
#define IDS_AX_IMAGE_ANNOTATION_ICON_REDO 49640
#define IDS_AX_IMAGE_ANNOTATION_ICON_UNDO 49641
#define IDS_AX_IMAGE_ANNOTATION_ICON_DOWNLOAD 49642
#define IDS_AX_IMAGE_ANNOTATION_ICON_ARROW_DOWNWARD 49643
#define IDS_AX_IMAGE_ANNOTATION_ICON_V_UPWARD 49644
#define IDS_AX_IMAGE_ANNOTATION_ICON_V_FORWARD 49645
#define IDS_AX_IMAGE_ANNOTATION_ICON_V_BACKWARD 49646
#define IDS_AX_IMAGE_ANNOTATION_ICON_HISTORY 49647
#define IDS_AX_IMAGE_ANNOTATION_ICON_PERSON 49648
#define IDS_AX_IMAGE_ANNOTATION_ICON_HAPPY_FACE 49649
#define IDS_AX_IMAGE_ANNOTATION_ICON_SAD_FACE 49650
#define IDS_AX_IMAGE_ANNOTATION_ICON_MOON 49651
#define IDS_AX_IMAGE_ANNOTATION_ICON_CLOUD 49652
#define IDS_AX_IMAGE_ANNOTATION_ICON_SUN 49653

// ---------------------------------------------------------------------------
// From blink_strings.h:

#define IDS_DETAILS_WITHOUT_SUMMARY_LABEL 46000
#define IDS_FORM_CALENDAR_CLEAR 46001
#define IDS_FORM_CALENDAR_TODAY 46002
#define IDS_FORM_SUBMIT_LABEL 46003
#define IDS_FORM_INPUT_ALT 46004
#define IDS_FORM_RESET_LABEL 46005
#define IDS_FORM_FILE_BUTTON_LABEL 46006
#define IDS_FORM_MULTIPLE_FILES_BUTTON_LABEL 46007
#define IDS_FORM_FILE_NO_FILE_LABEL 46008
#define IDS_FORM_FILE_MULTIPLE_UPLOAD 46009
#define IDS_FORM_OTHER_COLOR_LABEL 46010
#define IDS_FORM_OTHER_DATE_LABEL 46011
#define IDS_FORM_OTHER_MONTH_LABEL 46012
#define IDS_FORM_OTHER_WEEK_LABEL 46013
#define IDS_FORM_PLACEHOLDER_FOR_DAY_OF_MONTH_FIELD 46014
#define IDS_FORM_PLACEHOLDER_FOR_MONTH_FIELD 46015
#define IDS_FORM_PLACEHOLDER_FOR_YEAR_FIELD 46016
#define IDS_FORM_SELECT_MENU_LIST_TEXT 46017
#define IDS_FORM_THIS_MONTH_LABEL 46018
#define IDS_FORM_THIS_WEEK_LABEL 46019
#define IDS_FORM_WEEK_NUMBER_LABEL 46020
#define IDS_DEBUGGER_PAUSED_IN_ANOTHER_TAB 46021
#define IDS_VIEW_SOURCE_LINE_WRAP 46022
#define IDS_FORM_INPUT_WEEK_TEMPLATE 46023
#define IDS_FORM_VALIDATION_VALUE_MISSING_MULTIPLE_FILE 46024
#define IDS_FORM_VALIDATION_TYPE_MISMATCH 46025
#define IDS_FORM_VALIDATION_TYPE_MISMATCH_EMAIL_EMPTY 46026
#define IDS_FORM_VALIDATION_TYPE_MISMATCH_EMAIL_EMPTY_DOMAIN 46027
#define IDS_FORM_VALIDATION_TYPE_MISMATCH_EMAIL_EMPTY_LOCAL 46028
#define IDS_FORM_VALIDATION_TYPE_MISMATCH_EMAIL_INVALID_DOMAIN 46029
#define IDS_FORM_VALIDATION_TYPE_MISMATCH_EMAIL_INVALID_DOTS 46030
#define IDS_FORM_VALIDATION_TYPE_MISMATCH_EMAIL_INVALID_LOCAL 46031
#define IDS_FORM_VALIDATION_TYPE_MISMATCH_EMAIL_NO_AT_SIGN 46032
#define IDS_FORM_VALIDATION_TYPE_MISMATCH_MULTIPLE_EMAIL 46033
#define IDS_FORM_VALIDATION_VALUE_NOT_EQUAL 46034
#define IDS_FORM_VALIDATION_VALUE_NOT_EQUAL_DATETIME 46035
#define IDS_FORM_VALIDATION_RANGE_UNDERFLOW 46036
#define IDS_FORM_VALIDATION_RANGE_UNDERFLOW_DATETIME 46037
#define IDS_FORM_VALIDATION_RANGE_OVERFLOW 46038
#define IDS_FORM_VALIDATION_RANGE_OVERFLOW_DATETIME 46039
#define IDS_FORM_VALIDATION_REVERSED_RANGE_OUT_OF_RANGE_TIME 46040
#define IDS_FORM_VALIDATION_RANGE_REVERSED 46041
#define IDS_FORM_VALIDATION_RANGE_INVALID_DATETIME 46042
#define IDS_FORM_VALIDATION_BAD_INPUT_DATETIME 46043
#define IDS_FORM_VALIDATION_BAD_INPUT_NUMBER 46044
#define IDS_PRETTY_PRINT_JSON 46045
#define IDS_FORM_VALIDATION_VALUE_MISSING 46046
#define IDS_FORM_VALIDATION_VALUE_MISSING_CHECKBOX 46047
#define IDS_FORM_VALIDATION_VALUE_MISSING_FILE 46048
#define IDS_FORM_VALIDATION_VALUE_MISSING_RADIO 46049
#define IDS_FORM_VALIDATION_VALUE_MISSING_SELECT 46050
#define IDS_FORM_VALIDATION_TYPE_MISMATCH_EMAIL 46051
#define IDS_FORM_VALIDATION_TYPE_MISMATCH_URL 46052
#define IDS_FORM_VALIDATION_PATTERN_MISMATCH 46053
#define IDS_FORM_VALIDATION_STEP_MISMATCH 46054
#define IDS_FORM_VALIDATION_STEP_MISMATCH_CLOSE_TO_LIMIT 46055
#define IDS_FORM_VALIDATION_TOO_LONG 46056
#define IDS_FORM_VALIDATION_TOO_SHORT 46057
#define IDS_FORM_VALIDATION_TOO_SHORT_PLURAL 46058
#define IDS_MEDIA_SESSION_DATA_SOURCE 46059
#define IDS_MEDIA_SESSION_FILE_SOURCE 46060
#define IDS_MEDIA_OVERFLOW_MENU_CLOSED_CAPTIONS 46061
#define IDS_MEDIA_OVERFLOW_MENU_CLOSED_CAPTIONS_SUBMENU_TITLE 46062
#define IDS_MEDIA_TRACK_IDENTIFIER_VERBOSE_TITLE_NUMBERED 46063
#define IDS_MEDIA_OVERFLOW_MENU_SHOW_AUDIO_TRACK_MENU_BUTTON 46064
#define IDS_MEDIA_OVERFLOW_MENU_SHOW_VIDEO_TRACK_MENU_BUTTON 46065
#define IDS_MEDIA_OVERFLOW_MENU_TRACK_SELECTION_SUBMENU_TITLE 46066
#define IDS_MEDIA_OVERFLOW_MENU_PLAYBACK_SPEED 46067
#define IDS_MEDIA_OVERFLOW_MENU_PLAYBACK_SPEED_SUBMENU_TITLE 46068
#define IDS_MEDIA_OVERFLOW_MENU_PLAYBACK_SPEED_0_25X_TITLE 46069
#define IDS_MEDIA_OVERFLOW_MENU_PLAYBACK_SPEED_0_5X_TITLE 46070
#define IDS_MEDIA_OVERFLOW_MENU_PLAYBACK_SPEED_0_75X_TITLE 46071
#define IDS_MEDIA_OVERFLOW_MENU_PLAYBACK_SPEED_NORMAL_TITLE 46072
#define IDS_MEDIA_OVERFLOW_MENU_PLAYBACK_SPEED_1_25X_TITLE 46073
#define IDS_MEDIA_OVERFLOW_MENU_PLAYBACK_SPEED_1_5X_TITLE 46074
#define IDS_MEDIA_OVERFLOW_MENU_PLAYBACK_SPEED_1_75X_TITLE 46075
#define IDS_MEDIA_OVERFLOW_MENU_PLAYBACK_SPEED_2X_TITLE 46076
#define IDS_MEDIA_OVERFLOW_MENU_CAST 46077
#define IDS_MEDIA_OVERFLOW_MENU_ENTER_FULLSCREEN 46078
#define IDS_MEDIA_OVERFLOW_MENU_EXIT_FULLSCREEN 46079
#define IDS_MEDIA_OVERFLOW_MENU_MUTE 46080
#define IDS_MEDIA_OVERFLOW_MENU_UNMUTE 46081
#define IDS_MEDIA_OVERFLOW_MENU_PLAY 46082
#define IDS_MEDIA_OVERFLOW_MENU_PAUSE 46083
#define IDS_MEDIA_OVERFLOW_MENU_DOWNLOAD 46084
#define IDS_MEDIA_OVERFLOW_MENU_ENTER_PICTURE_IN_PICTURE 46085
#define IDS_MEDIA_OVERFLOW_MENU_EXIT_PICTURE_IN_PICTURE 46086
#define IDS_MEDIA_PICTURE_IN_PICTURE_INTERSTITIAL_TEXT 46087
#define IDS_MEDIA_REMOTING_CAST_TEXT 46088
#define IDS_MEDIA_REMOTING_CAST_TO_UNKNOWN_DEVICE_TEXT 46089
#define IDS_MEDIA_REMOTING_STOP_TEXT 46090
#define IDS_MEDIA_REMOTING_STOP_BY_PLAYBACK_QUALITY_TEXT 46091
#define IDS_MEDIA_REMOTING_STOP_BY_ERROR_TEXT 46092
#define IDS_MEDIA_SCRUBBING_MESSAGE_TEXT 46093
#define IDS_MEDIA_TRACKS_NO_LABEL 46094
#define IDS_MEDIA_TRACKS_OFF 46095
#define IDS_PLUGIN_INITIALIZATION_ERROR 46096
#define IDS_MEDIA_PLAYBACK_ERROR 46097
#define IDS_UNITS_KIBIBYTES 46098
#define IDS_UNITS_MEBIBYTES 46099
#define IDS_UNITS_GIBIBYTES 46100
#define IDS_UNITS_TEBIBYTES 46101
#define IDS_UNITS_PEBIBYTES 46102

// ---------------------------------------------------------------------------
// From branded_strings.h:

#define IDS_RELAUNCH_CONFIRMATION_DIALOG_TITLE 800
#define IDS_SETTINGS_ABOUT_PROGRAM 801
#define IDS_SETTINGS_GET_HELP_USING_CHROME 802
#define IDS_SETTINGS_UPGRADE_UPDATING 803
#define IDS_SETTINGS_UPGRADE_UPDATING_PERCENT 804
#define IDS_SETTINGS_UPGRADE_SUCCESSFUL_RELAUNCH 805
#define IDS_SETTINGS_UPGRADE_UP_TO_DATE 806
#define IDS_SETTINGS_SIGNED_OUT_USER_LABEL 809
#define IDS_SETTINGS_AUTOFILL_AI_WHEN_ON_USE_TO_FILL 810
#define IDS_SETTINGS_DEFAULT_BROWSER_DEFAULT 811
#define IDS_SETTINGS_DEFAULT_BROWSER_DEFAULT_THANK_YOU 812
#define IDS_SETTINGS_DEFAULT_BROWSER_MAKE_DEFAULT 813
#define IDS_SETTINGS_DEFAULT_BROWSER_MAKE_DEFAULT_USER_VALUE 814
#define IDS_SETTINGS_DEFAULT_BROWSER_MAKE_DEFAULT_AND_PIN 815
#define IDS_SETTINGS_DEFAULT_BROWSER_MAKE_DEFAULT_AND_PIN_USER_VALUE 816
#define IDS_SETTINGS_DEFAULT_BROWSER_ERROR 817
#define IDS_SETTINGS_DEFAULT_BROWSER_SECONDARY 818
#define IDS_SETTINGS_PRELOAD_PAGES_STANDARD_PRELOADING_WHEN_ON_BULLET_TWO 819
#define IDS_SETTINGS_PRELOAD_PAGES_EXTENDED_PRELOADING_WHEN_ON_BULLET_TWO 820
#define IDS_SETTINGS_PRELOAD_PAGES_EXTENDED_PRELOADING_THINGS_TO_CONSIDER_BULLET_TWO 821
#define IDS_SETTINGS_PRELOAD_PAGES_THINGS_TO_CONSIDER_BULLET_ONE 822
#define IDS_SETTINGS_SPELLING_PREF_DESC 823
#define IDS_SETTINGS_RESTART_TO_APPLY_CHANGES 824
#define IDS_SETTINGS_SIGNIN_ALLOWED 825
#define IDS_SETTINGS_SIGNIN_ALLOWED_DESC 826
#define IDS_SETTINGS_SIGNIN_ALLOWED_DESC_2 827
#define IDS_SETTINGS_SITE_SETTINGS_PDFS_BLOCKED 828
#define IDS_SETTINGS_SITE_SITE_DETAILS_CAMERA_BLOCKED_BY_OS 829
#define IDS_SETTINGS_SITE_SITE_DETAILS_MICROPHONE_BLOCKED_BY_OS 830
#define IDS_SETTINGS_SITE_SITE_DETAILS_LOCATION_BLOCKED_BY_OS 831
#define IDS_SETTINGS_SITE_SETTINGS_CAMERA_BLOCKED_BY_OS 832
#define IDS_SETTINGS_SITE_SETTINGS_LOCATION_BLOCKED_BY_OS 833
#define IDS_SETTINGS_SITE_SETTINGS_MIC_BLOCKED_BY_OS 834
#define IDS_SETTINGS_SITE_DATA_PAGE_CLEAR_ON_EXIT_WITH_EXCEPTION_RADIO_SUBLABEL 835
#define IDS_SETTINGS_SITE_DATA_PAGE_DELETE_ON_EXIT_EXCEPTIONS_SUB_HEADING 836
#define IDS_SETTINGS_PRIVACY_GUIDE_PROMO_BODY 837
#define IDS_SETTINGS_PRIVACY_GUIDE_SAFE_BROWSING_CARD_STANDARD_PROTECTION_FEATURE_DESCRIPTION2 838
#define IDS_SETTINGS_PRIVACY_GUIDE_SAFE_BROWSING_CARD_STANDARD_PROTECTION_PRIVACY_DESCRIPTION1 839
#define IDS_SETTINGS_PRIVACY_GUIDE_COMPLETION_CARD_WAA_SUB_LABEL 840
#define IDS_SETTINGS_PRIVACY_GUIDE_MSBB_FEATURE_DESCRIPTION3 841
#define IDS_SETTINGS_PRIVACY_GUIDE_MSBB_PRIVACY_DESCRIPTION2 842
#define IDS_SETTINGS_PRIVACY_SANDBOX_AD_MEASUREMENT_DIALOG_DESCRIPTION 843
#define IDS_SETTINGS_SAFETY_CHECK_UNUSED_SITE_PERMISSIONS_SETTING_SUBLABEL 844
#define IDS_SETTINGS_SAFETY_CHECK_UNUSED_SITE_PERMISSIONS_AND_NOTIFICATIONS_SETTING_SUBLABEL 845
#define IDS_SETTINGS_SAFEBROWSING_ENHANCED_BULLET_TWO 846
#define IDS_SETTINGS_SAFEBROWSING_STANDARD_BULLET_TWO 847
#define IDS_SETTINGS_SAFEBROWSING_ENHANCED_WHEN_ON_BULLET_ONE 848
#define IDS_SETTINGS_SAFEBROWSING_ENHANCED_LEARN_MORE_LABEL 849
#define IDS_SETTINGS_SECURE_DNS_DESCRIPTION 850
#define IDS_SETTINGS_SYNC_DISCONNECT_DELETE_PROFILE_WARNING_WITH_COUNTS_SINGULAR 851
#define IDS_SETTINGS_SYNC_DISCONNECT_DELETE_PROFILE_WARNING_WITH_COUNTS_PLURAL 852
#define IDS_SETTINGS_SYNC_DISCONNECT_DELETE_PROFILE_WARNING_WITHOUT_COUNTS 853
#define IDS_SETTINGS_CUSTOMIZE_YOUR_CHROME_PROFILE 854
#define IDS_SETTING_NAME_YOUR_PROFILE 855
#define IDS_SETTINGS_PEOPLE_SIGN_IN_PROMPT_SECONDARY_WITH_ACCOUNT 856
#define IDS_SETTINGS_PEOPLE_EXPLICIT_SIGN_IN_PROMPT_SECONDARY_WITH_NO_ACCOUNT 857
#define IDS_SETTINGS_PEOPLE_EXPLICIT_SIGN_IN_PROMPT_SECONDARY_WITH_NO_ACCOUNT_WITH_BOOKMARKS 858
#define IDS_SETTINGS_PEOPLE_ACCOUNT_AWARE_SIGNIN_ACCOUNT_ROW_TITLE 859
#define IDS_SETTINGS_SIGNIN_CHROME_SIGNIN_OPTIONS_TITLE 861
#define IDS_SETTINGS_SIGNIN_CHROME_SIGNIN_OPTIONS_DESC 862
#define IDS_SETTINGS_PEOPLE_SIGNIN_SECTION_SIGNED_OUT_TITLE 863
#define IDS_SETTINGS_SYNC_DATA_ENCRYPTED_TEXT 864
#define IDS_SETTINGS_SYNC_DISCONNECT_TITLE 865
#define IDS_SETTINGS_SUGGEST_PREF_DESC 866
#define IDS_SETTINGS_SEARCH_AGGREGATOR_PREF_DESC 867
#define IDS_SETTINGS_PERSONALIZE_GOOGLE_SERVICES_DESC 868
#define IDS_SETTINGS_PERSONALIZE_GOOGLE_SERVICES_DESC_WITH_LINKED_SERVICES 869
#define IDS_SETTINGS_PERSONALIZE_GOOGLE_SERVICES_LINKED_SERVICES_DESC 870
#define IDS_SETTINGS_ERROR_PASSPHRASE_USER_ERROR_DESCRIPTION 871
#define IDS_SETTINGS_ERROR_TRUSTED_VAULT_USER_ERROR_DESCRIPTION 872
#define IDS_SETTINGS_ERROR_UPGRADE_CLIENT_USER_ERROR_DESCRIPTION 873
#define IDS_SETTINGS_PENDING_STATE_DESCRIPTION 874
#define IDS_SETTINGS_ERROR_PASSPHRASE_USER_ERROR_DESCRIPTION_WITH_EMAIL 875
#define IDS_SETTINGS_SIGN_OUT 876
#define IDS_SETTINGS_ACCOUNT_DATA_DASHBOARD 877
#define IDS_SETTINGS_ENCRYPT_ACCOUNT_DATA_WITH_PASSPHRASE_LABEL 878
#define IDS_SETTINGS_ENCRYPT_ACCOUNT_DATA_WITH_PASSPHRASE_EXPLANATION_TEXT 879
#define IDS_SETTINGS_PASSPHRASE_RESET_HINT_ACCOUNT_DATA_ENCRYPTION 880
#define IDS_SETTINGS_PERFORMANCE_MEMORY_SAVER_MODE_SETTING_DESCRIPTION 881
#define IDS_SETTINGS_PERFORMANCE_BATTERY_SAVER_MODE_SETTING_DESCRIPTION 882
#define IDS_SETTINGS_PERFORMANCE_PRELOAD_TOGGLE_SUMMARY 883
#define IDS_SETTINGS_SYSTEM_BACKGROUND_APPS_LABEL 886
#define IDS_SETTINGS_RESET_PROFILE_FEEDBACK 887
#define IDS_SETTINGS_SAFETY_HUB_ENTRY_POINT_NOTHING_TO_DO 888
#define IDS_SETTINGS_SAFETY_HUB_ENTRY_POINT_HEADER 889
#define IDS_SETTINGS_SAFETY_HUB_EMPTY_STATE_MODULE_SUBHEADER 890
#define IDS_SETTINGS_SAFETY_HUB_VERSION_MODULE_UPPERCASE_NAME 891
#define IDS_SETTINGS_SAFETY_HUB_VERSION_MODULE_LOWERCASE_NAME 892
#define IDS_SETTINGS_SAFETY_HUB_PASSWORD_CHECK_SUBHEADER_NO_PASSWORDS 893
#define IDS_SETTINGS_SAFETY_HUB_USER_EDU_MODULE_HEADER 894
#define IDS_SETTINGS_SAFETY_HUB_USER_EDU_DATA_HEADER 895
#define IDS_SETTINGS_SAFETY_HUB_USER_EDU_DATA_SUBHEADER 896
#define IDS_SETTINGS_SAFETY_HUB_USER_EDU_INCOGNITO_HEADER 897
#define IDS_SETTINGS_SAFETY_HUB_USER_EDU_SAFE_BROWSING_SUBHEADER 898
#define IDS_SETTINGS_SAFETY_HUB_VERSION_CARD_HEADER_RESTART 899
#define IDS_SETTINGS_SAFETY_HUB_VERSION_CARD_HEADER_UPDATED 900
#define IDS_SETTINGS_SAFETY_HUB_VERSION_NAVIGATION_ARIA_LABEL 901
#define IDS_SETTINGS_SAFETY_HUB_VERSION_RELAUNCH_ARIA_LABEL 902
#define IDS_SETTINGS_SAFETY_HUB_UNUSED_SITE_PERMISSIONS_REMOVED_ONE_PERMISSION_LABEL 903
#define IDS_SETTINGS_SAFETY_HUB_UNUSED_SITE_PERMISSIONS_REMOVED_TWO_PERMISSIONS_LABEL 904
#define IDS_SETTINGS_SAFETY_HUB_UNUSED_SITE_PERMISSIONS_REMOVED_THREE_PERMISSIONS_LABEL 905
#define IDS_SETTINGS_SAFETY_HUB_UNUSED_SITE_PERMISSIONS_REMOVED_FOUR_OR_MORE_PERMISSIONS_LABEL 906
#define IDS_SETTINGS_SAFETY_HUB_ABUSIVE_NOTIFICATION_PERMISSIONS_SETTING_SUBLABEL 907
#define IDS_SETTINGS_SECURITY_PASSWORDS_LEAK_DETECTION_DESC 908
#define IDS_SETTINGS_SECURITY_SAFE_BROWSING_STANDARD_HOW_IT_WORKS_CONTENT 909
#define IDS_SETTINGS_SECURITY_JAVASCRIPT_GUARDRAILS_UNRECOGNIZED_SITES_DESCRIPTION 910
#define IDS_SETTINGS_SECURITY_BUNDLE_ENHANCED_BULLET_ONE 911
#define IDS_SETTINGS_SECURE_CONNECTIONS_SECTION_DESCRIPTION 912
#define IDS_SETTINGS_HTTPS_FIRST_MODE_SUBTITLE 913
#define IDS_SETTINGS_HTTPS_FIRST_MODE_ENABLED_FULL_SUBLABEL 914
#define IDS_SETTINGS_HTTPS_FIRST_MODE_ENABLED_INCOGNITO_SUBLABEL 915
#define IDS_SETTINGS_HTTPS_FIRST_MODE_DISABLED_SUBLABEL 916
#define IDS_PRODUCT_NAME 917
#define IDS_SHORT_PRODUCT_NAME 918
#define IDS_FIRST_RUN_DIALOG_WINDOW_TITLE 924
#define IDS_TERMS_DIALOG_ACCEPT 925
#define IDS_TERMS_DIALOG_CANCEL 926
#define IDS_TERMS_DIALOG_TITLE 927
#define IDS_TERMS_DIALOG_ACCESSIBILITY_LABEL 928
#define IDS_PRODUCT_LOGO_ENTERPRISE_ALT_TEXT 932
#define IDS_TASK_MANAGER_TITLE 933
#define IDS_TASK_MANAGER_SEARCH_PLACEHOLDER 934
#define IDS_TASK_MANAGER_SEARCH_ACCESSIBILITY_NAME 935
#define IDS_TASK_MANAGER_CLEAR_SEARCH_BUTTON_ACCESSIBILITY_NAME 936
#define IDS_TASK_MANAGER_KILL_ACCESSIBILITY_NAME 937
#define IDS_TASK_MANAGER_TASK_KILL_SUCCESS_ACCESSIBILITY_MESSAGE 938
#define IDS_TASK_MANAGER_TASK_KILL_FAIL_ACCESSIBILITY_MESSAGE 939
#define IDS_TASK_MANAGER_CATEGORY_TABS_AND_EXTENSIONS_NAME 940
#define IDS_TASK_MANAGER_CATEGORY_SYSTEM_NAME 941
#define IDS_TASK_MANAGER_CATEGORY_BROWSER_NAME 942
#define IDS_TASK_MANAGER_CATEGORY_ALL_NAME 943
#define IDS_SESSION_CRASHED_VIEW_UMA_OPTIN 944
#define IDS_CHROME_SIGNOUT_CONFIRMATION_PROMPT_TOO_MANY_BOOKMARKS_BODY 945
#define IDS_RESET_PROFILE_SETTINGS_EXPLANATION_IN_BULLET_POINTS 946
#define IDS_BROWSER_WINDOW_TITLE_FORMAT 947
#define IDS_CAPTIVE_PORTAL_BROWSER_WINDOW_TITLE_FORMAT 948
#define IDS_ACCESSIBLE_BROWSER_WINDOW_TITLE_FORMAT 949
#define IDS_ACCESSIBLE_BETA_BROWSER_WINDOW_TITLE_FORMAT 950
#define IDS_ACCESSIBLE_DEV_BROWSER_WINDOW_TITLE_FORMAT 951
#define IDS_ACCESSIBLE_CANARY_BROWSER_WINDOW_TITLE_FORMAT 952
#define IDS_ABOUT_VERSION_COMPANY_NAME 953
#define IDS_ABOUT_VERSION_COPYRIGHT 954
#define IDS_ABOUT_TERMS_OF_SERVICE 958
#define IDS_LINUX_OBSOLETE 964
#define IDS_ACCNAME_APP 965
#define IDS_COULDNT_STARTUP_PROFILE_ERROR 967
#define IDS_BROWSER_SHARING_CLICK_TO_CALL_DIALOG_HELP_TEXT_NO_DEVICES 968
#define IDS_BROWSER_SHARING_CLICK_TO_CALL_DIALOG_HELP_TEXT_NO_DEVICES_ORIGIN 969
#define IDS_BROWSER_SHARING_ERROR_DIALOG_TEXT_DEVICE_NOT_FOUND 970
#define IDS_FR_CUSTOMIZE_DEFAULT_BROWSER 974
#define IDS_DEFAULT_BROWSER_CHANGED_MESSAGE 975
#define IDS_STATUS_TRAY_KEEP_CHROME_RUNNING_IN_BACKGROUND 981
#define IDS_STATUS_TRAY_KEEP_CHROME_RUNNING_IN_BACKGROUND_SETTING 982
#define IDS_CANT_WRITE_USER_DIRECTORY_SUMMARY 983
#define IDS_PROFILE_TOO_NEW_ERROR 985
#define IDS_PREFERENCES_UNREADABLE_ERROR 986
#define IDS_PREFERENCES_CORRUPT_ERROR 987
#define IDS_DESKTOP_SHORTCUT_COMMENT 988
#define IDS_PASSWORD_MANAGER_ONBOARDING_DETAILS_C 989
#define IDS_PASSWORD_MANAGER_TITLE_BRAND 990
#define IDS_PASSWORD_BUBBLES_PASSWORD_MANAGER_LINK_TEXT_SAVING_ON_DEVICE 991
#define IDS_PASSWORD_MANAGER_SAVE_PASSWORD_SIGNED_OUT_MESSAGE_DESCRIPTION 992
#define IDS_PASSWORD_MANAGER_UPDATE_PASSWORD_SIGNED_OUT_MESSAGE_DESCRIPTION 993
#define IDS_PASSWORD_MANAGER_IMPORT_DIALOG_TITLE 1014
#define IDS_PASSWORD_MANAGER_EXPORT_DIALOG_TITLE 1015
#define IDS_DEFAULT_BROWSER_INFOBAR_TEXT 1023
#define IDS_DEFAULT_BROWSER_PIN_INFOBAR_TEXT 1024
#define IDS_DEFAULT_BROWSER_MODAL_TITLE_WITH_SETTINGS_ILLUSTRATION 1025
#define IDS_DEFAULT_BROWSER_MODAL_BODY_WITH_SETTINGS_ILLUSTRATION 1026
#define IDS_DEFAULT_BROWSER_MODAL_BODY_WITH_SETTINGS_ILLUSTRATION_UNPINNED 1027
#define IDS_DEFAULT_BROWSER_MODAL_TITLE_WITHOUT_SETTINGS_ILLUSTRATION 1028
#define IDS_DEFAULT_BROWSER_MODAL_BODY_WITHOUT_SETTINGS_ILLUSTRATION 1029
#define IDS_DEFAULT_BROWSER_MODAL_BODY_WITHOUT_SETTINGS_ILLUSTRATION_UNPINNED 1030
#define IDS_DEFAULT_BROWSER_BUBBLE_DIALOG_TITLE 1031
#define IDS_DEFAULT_BROWSER_BUBBLE_DIALOG_BODY 1032
#define IDS_DEFAULT_BROWSER_SUCCESS_TOAST_BODY 1033
#define IDS_DEFAULT_BROWSER_BUBBLE_DIALOG_STEPS_PINNED_COMBINED 1034
#define IDS_PDF_GLIC_SUMMARIZE_IPH_TEXT 1036
#define IDS_PDF_GLIC_SUMMARIZE_IPH_TEXT_NEW 1037
#define IDS_PDF_GLIC_SUMMARIZE_IPH_TEXT_NEW_SCREENREADER 1038
#define IDS_DOWNLOAD_BUBBLE_SUBPAGE_SUMMARY_UNKNOWN_SOURCE 1039
#define IDS_DOWNLOAD_BUBBLE_SUBPAGE_SUMMARY_WARNING_BLOCKED_LEARN_MORE_LINK 1040
#define IDS_DOWNLOAD_BUBBLE_SUBPAGE_SUMMARY_DEEP_SCANNING_PROMPT_LOCAL_DECRYPTION 1041
#define IDS_DOWNLOAD_STATUS_CRX_INSTALL_RUNNING 1042
#define IDS_PROMPT_DOWNLOAD_CHANGES_SETTINGS 1043
#define IDS_PROMPT_MALICIOUS_DOWNLOAD_URL 1044
#define IDS_PROMPT_MALICIOUS_DOWNLOAD_CONTENT 1045
#define IDS_BLOCK_REASON_PROMPT_FOR_SCANNING 1046
#define IDS_BLOCK_DOWNLOAD_REASON_DANGEROUS_FILETYPE 1047
#define IDS_BLOCK_DOWNLOAD_REASON_DANGEROUS 1048
#define IDS_BLOCK_DOWNLOAD_REASON_DANGEROUS_COOKIE_THEFT 1049
#define IDS_BLOCK_DOWNLOAD_REASON_UNCOMMON 1050
#define IDS_BLOCK_DOWNLOAD_REASON_UNCOMMON_SUSPICIOUS_ARCHIVE 1051
#define IDS_BLOCK_DOWNLOAD_REASON_POTENTIALLY_UNWANTED 1052
#define IDS_BLOCK_DOWNLOAD_REASON_INSECURE 1053
#define IDS_BLOCK_DOWNLOAD_REASON_UNVERIFIED_NO_SAFE_BROWSING 1054
#define IDS_DOWNLOAD_WARNING_BYPASS_PROMPT_LEARN_MORE_LINK 1055
#define IDS_DOWNLOAD_WARNING_BYPASS_PROMPT_LEARN_MORE_LINK_ACCESSIBLE 1056
#define IDS_ABANDON_DOWNLOAD_DIALOG_BROWSER_MESSAGE 1057
#define IDS_MISSING_GOOGLE_API_KEYS 1059
#define IDS_EXTENSION_INSTALLED_HEADING 1060
#define IDS_EXTENSION_UNINSTALL_PROMPT_REMOVE_DATA_CHECKBOX 1062
#define IDS_EXTENSION_ALERT_ITEM_BLOCKLISTED_MALWARE 1063
#define IDS_EXTENSIONS_ALERT_ITEM_BLOCKLISTED_MALWARE_TITLE 1064
#define IDS_EXTENSIONS_MANIFEST_V2_DEPRECATION_KEEP_DIALOG_DESCRIPTION 1065
#define IDS_EXTENSIONS_MANIFEST_V2_DEPRECATION_DISABLED_DIALOG_DESCRIPTION 1066
#define IDS_EXTENSIONS_MV2_DEPRECATION_MESSAGE_DISABLED_SUBTITLE 1067
#define IDS_EXTENSIONS_MV2_DEPRECATION_PANEL_DISABLED_SUBTITLE 1068
#define IDS_EXTENSIONS_MV2_DEPRECATION_PANEL_REMOVE_BUTTON_ACC_LABEL 1069
#define IDS_EXTENSIONS_MENU_ZERO_STATE_DESC 1070
#define IDS_OPEN_WEB_STORE_BUTTON_TEXT 1071
#define IDS_SAFETY_CHECK_EXTENSIONS_MALWARE 1072
#define IDS_SAFETY_CHECK_EXTENSIONS_POLICY_VIOLATION 1073
#define IDS_SAFETY_CHECK_EXTENSIONS_UNPUBLISHED 1074
#define IDS_EXTENSIONS_SAFETY_CHECK_PRIMARY_LABEL 1075
#define IDS_EXTENSIONS_SAFETY_CHECK_OFFSTORE 1076
#define IDS_EXTENSIONS_SAFETY_CHECK_OFFSTORE_ON 1077
#define IDS_EXTENSIONS_SAFETY_CHECK_OFFSTORE_OFF 1078
#define IDS_EXTENSIONS_SAFETY_CHECK_NO_PRIVACY_PRACTICES 1079
#define IDS_EXTENSIONS_SC_DESCRIPTION 1080
#define IDS_EXTENSIONS_ITEM_SHOW_ACCESS_REQUESTS_IN_TOOLBAR 1081
#define IDS_EXTENSIONS_INCOGNITO_WARNING 1082
#define IDS_EXTENSIONS_UNINSTALL 1083
#define IDS_EXTENSIONS_SHORTCUT_SCOPE_IN_CHROME 1084
#define IDS_PENDING_CHANGE_WARNING 1085
#define IDS_EXTENSIONS_MULTIPLE_UNSUPPORTED_DISABLED_BODY 1086
#define IDS_EXTENSIONS_SINGLE_UNSUPPORTED_DISABLED_BODY 1087
#define IDS_ANCHORED_MESSAGE_MENU_TOOLTIP 1088
#define IDS_APPMENU_TOOLTIP 1089
#define IDS_APPMENU_TOOLTIP_UPDATE_AVAILABLE 1090
#define IDS_APPMENU_TOOLTIP_ALERT 1091
#define IDS_OPEN_IN_CHROME 1092
#define IDS_ABOUT 1093
#define IDS_RELAUNCH_TO_UPDATE 1094
#define IDS_CUSTOMIZE_CHROME 1098
#define IDS_OTHER_CHROME_PROFILES_TITLE 1099
#define IDS_MANAGE_CHROME_PROFILES 1100
#define IDS_VIEW_PASSWORDS 1101
#define IDS_CHROME_SIGNIN_TITLE 1102
#define IDS_PROFILES_DICE_SYNC_PROMO 1103
#define IDS_SYNC_WRONG_EMAIL 1104
#define IDS_SYNC_USED_PROFILE_ERROR 1105
#define IDS_SESSION_RESTORE_TURN_OFF_RESTORE_FROM_SESSION 1106
#define IDS_SESSION_RESTORE_TURN_OFF_RESTORE_FROM_RESTART 1107
#define IDS_SESSION_RESTORE_TURN_ON 1108
#define IDS_SESSION_RESTORE_LINK 1109
#define IDS_SESSION_RESTORE_LINK_ARIA_LABEL 1110
#define IDS_ENTERPRISE_VALUE_PROPOSITION_PROFILE_REQUIRED_BY_ORG_TITLE 1111
#define IDS_AI_SIGNIN_PROMO_SUBTITLE 1112
#define IDS_AI_SIGNIN_PROMO_AVATAR_PILL_TEXT 1113
#define IDS_ENTERPRISE_SIGNIN_TITLE 1114
#define IDS_ENTERPRISE_SIGNIN_EXPLANATION_WITHOUT_PROFILE_CREATION 1115
#define IDS_ENTERPRISE_SIGNIN_EXPLANATION_WITH_PROFILE_CREATION 1116
#define IDS_ENTERPRISE_VALUE_PROPOSITION_PROFILE_REQUIRED_BY_ORG_KNOWN_DOMAIN_TITLE 1117
#define IDS_ENTERPRISE_VALUE_PROPOSITION_PROFILE_SUGGESTED_TITLE 1118
#define IDS_MANAGEMENT_DIALOG_BROWSER_MANAGED 1119
#define IDS_MANAGEMENT_DIALOG_BROWSER_MANAGED_BY 1120
#define IDS_MANAGEMENT_DIALOG_BROWSER_MANAGED_BY_MULTIPLE_ORGANIZATIONS 1121
#define IDS_MANAGED_PROFILE_INTERSTITIAL_PRIMARY_PARAGRAPH 1122
#define IDS_EXTENSION_ASKS_IDENTITY_WHILE_SIGNED_IN_WEB_ONLY_TITLE 1123
#define IDS_EXTENSION_ASKS_IDENTITY_WHILE_SIGNED_IN_WEB_ONLY_BODY_PART_1 1124
#define IDS_EXTENSION_ASKS_IDENTITY_WHILE_SIGNED_IN_WEB_ONLY_TITLE_FALLBACK 1125
#define IDS_ABOUT_BROWSER_SWITCH_DESCRIPTION_UNKNOWN_BROWSER 1126
#define IDS_ABOUT_BROWSER_SWITCH_DESCRIPTION_KNOWN_BROWSER 1127
#define IDS_NTP_CUSTOMIZE_TOOLBAR_CATEGORY_YOUR_CHROME 1128
#define IDS_NTP_CUSTOMIZE_BUTTON_LABEL 1129
#define IDS_NTP_MODULES_SETUP_LIST_TITLE 1130
#define IDS_NTP_CUSTOMIZE_CHROME_RESET_TO_CLASSIC_CHROME_LABEL 1131
#define IDS_NTP_CUSTOMIZATION_PROMO 1132
#define IDS_NTP_MOBILE_PROMO_TITLE 1133
#define IDS_NTP_MOBILE_PROMO_PASSWORDS_TITLE 1134
#define IDS_IPH_CUSTOMIZE_CHROME_AUTO_OPEN_BODY 1135
#define IDS_IPH_CUSTOMIZE_CHROME_AUTO_OPEN_SCREENREADER 1136
#define IDS_IPH_REALBOX_CONTEXTUAL_ENTRYPOINT 1137
#define IDS_IPH_REALBOX_CONTEXTUAL_ENTRYPOINT_SCREENREADER 1138
#define IDS_SIGNIN_EMAIL_CONFIRMATION_TITLE 1139
#define IDS_AVATAR_BUTTON_INTERCEPT_BUBBLE_CHROME_SIGNIN_TEXT 1140
#define IDS_SIGNIN_DICE_WEB_INTERCEPT_BUBBLE_CHROME_SIGNIN_TITLE 1141
#define IDS_SIGNIN_DICE_WEB_INTERCEPT_BUBBLE_CHROME_SIGNIN_DECLINE_TEXT 1142
#define IDS_SIGNIN_DICE_WEB_INTERCEPT_CREATE_BUBBLE_TITLE_V2 1143
#define IDS_SIGNIN_DICE_WEB_INTERCEPT_SWITCH_BUBBLE_TITLE 1144
#define IDS_SIGNIN_DICE_WEB_INTERCEPT_SWITCH_BUBBLE_DESC_V2 1145
#define IDS_SIGNIN_DICE_WEB_INTERCEPT_SWITCH_BUBBLE_DESC_V2_SUPERVISED 1146
#define IDS_SIGNIN_DICE_WEB_INTERCEPT_CONSUMER_BUBBLE_DESC 1147
#define IDS_SIGNIN_DICE_WEB_INTERCEPT_CREATE_BUBBLE_DESC 1148
#define IDS_SIGNIN_DICE_WEB_INTERCEPT_BUBBLE_CHROME_SIGNIN_SUBTITLE 1149
#define IDS_SIGNIN_DICE_WEB_INTERCEPT_BUBBLE_CHROME_SIGNIN_SUBTITLE_WITH_BOOKMARKS 1150
#define IDS_SIGNIN_DICE_WEB_INTERCEPT_BUBBLE_CHROME_SIGNIN_SUBTITLE_SUPERVISED 1151
#define IDS_SIGNIN_DICE_WEB_INTERCEPT_BUBBLE_CHROME_SIGNIN_IPH_TEXT_SIGNIN_REMINDER 1152
#define IDS_SIGNIN_DICE_WEB_INTERCEPT_BUBBLE_CHROME_SIGNIN_IPH_TEXT_SIGNIN_REMINDER_SCREENREADER 1153
#define IDS_CHROME_SIGNOUT_CONFIRMATION_PROMPT_NO_UNSYNCED_TITLE 1154
#define IDS_CHROME_SIGNOUT_CONFIRMATION_PROMPT_NO_UNSYNCED_BODY 1155
#define IDS_CHROME_SIGNOUT_CONFIRMATION_PROMPT_NO_UNSYNCED_DATA_BODY 1156
#define IDS_CHROME_SIGNOUT_CONFIRMATION_PROMPT_VERIFY_BODY 1157
#define IDS_CHROME_SIGNOUT_CONFIRMATION_PROMPT_VERIFY_BODY_WITH_COUNT 1158
#define IDS_CHROME_SIGNOUT_CONFIRMATION_PROMPT_UNSYNCED_BODY 1159
#define IDS_CHROME_SIGNOUT_CONFIRMATION_PROMPT_UNSYNCED_BODY_WITH_COUNT 1160
#define IDS_CLOSE_WARNING_FOR_CLEAR_COOKIES_ON_EXIT_TEXT 1161
#define IDS_AVATAR_BUTTON_INTERCEPT_BUBBLE_CHROME_SIGNIN_ACCESSIBILITY_LABEL 1162
#define IDS_AVATAR_BUTTON_SIGNIN_PENDING_ACCESSIBILITY_LABEL 1163
#define IDS_AUTOFILL_SIGNIN_PROMO_SUBTITLE_PASSWORD 1164
#define IDS_AUTOFILL_SIGNIN_PROMO_SUBTITLE_ADDRESS 1165
#define IDS_SYNC_PASSPHRASE_DIALOG_FOOTER 1166
#define IDS_SYNC_PASSPHRASE_DIALOG_FOOTER_LINK_ACC 1167
#define IDS_SYNC_PASSPHRASE_DIALOG_BODY 1168
#define IDS_PROFILE_CUSTOMIZATION_TEXT 1169
#define IDS_PROFILE_CUSTOMIZATION_TITLE_V2 1170
#define IDS_PROFILE_MENU_SIGNIN_PROMO_BUTTON 1180
#define IDS_PROFILE_MENU_PLACEHOLDER_PROFILE_NAME 1181
#define IDS_PROFILE_MENU_SIGN_OUT_WHEN_SIGNIN_PENDING 1182
#define IDS_PROFILE_MENU_PROFILES_LIST_TITLE 1183
#define IDS_PROFILE_MENU_MANAGE_PROFILES 1184
#define IDS_PROFILE_MENU_SIGN_OUT 1185
#define IDS_PROFILE_MENU_ADD_PROFILE 1186
#define IDS_APP_ALSO_DELETE_APPS_DATA 1188
#define IDS_APP_SHORTCUTS_SUBDIR_NAME 1189
#define IDS_APP_SHORTCUTS_SUBDIR_NAME_CANARY 1190
#define IDS_MEDIA_STREAM_STATUS_TRAY_TEXT_AUDIO_AND_VIDEO 1193
#define IDS_MEDIA_STREAM_STATUS_TRAY_TEXT_AUDIO_ONLY 1194
#define IDS_MEDIA_STREAM_STATUS_TRAY_TEXT_VIDEO_ONLY 1195
#define IDS_WEBHID_SYSTEM_TRAY_ICON_TITLE 1196
#define IDS_WEBUSB_SYSTEM_TRAY_ICON_TITLE 1197
#define IDS_PROFILE_IN_USE_POSIX 1198
#define IDS_LOGIN_POD_USER_REMOVE_WARNING_SYNC 1200
#define IDS_USER_MANAGER_TUTORIAL_SLIDE_INTRO_TITLE 1201
#define IDS_USER_MANAGER_TUTORIAL_SLIDE_INTRO_TEXT 1202
#define IDS_USER_MANAGER_TUTORIAL_SLIDE_YOUR_CHROME_TITLE 1203
#define IDS_USER_MANAGER_TUTORIAL_SLIDE_YOUR_CHROME_TEXT 1204
#define IDS_USER_MANAGER_TUTORIAL_SLIDE_GUEST_TEXT 1205
#define IDS_USER_MANAGER_TUTORIAL_SLIDE_FRIENDS_TEXT 1206
#define IDS_USER_MANAGER_TUTORIAL_SLIDE_OUTRO_TEXT 1207
#define IDS_USER_MANAGER_TUTORIAL_SLIDE_OUTRO_ADD_USER 1208
#define IDS_TUTORIAL_PASSWORD_MANAGER_OPEN_APP_MENU 1209
#define IDS_TUTORIAL_PASSWORD_MANAGER_CLICK_PASSWORD_MANAGER 1210
#define IDS_TUTORIAL_PASSWORD_MANAGER_SUCCESS_BODY 1211
#define IDS_PASSWORD_MANAGER_IPH_CREATE_SHORTCUT_BODY 1212
#define IDS_EXTENSIONS_SETTINGS_API_SECOND_LINE_START_PAGES 1213
#define IDS_EXTENSIONS_SETTINGS_API_SECOND_LINE_START_AND_SEARCH 1214
#define IDS_SETTINGS_SEARCH_EXTENSION_ENGINES_ACTIVE_SHORTCUTS_EXPLANATION 1215
#define IDS_SETTINGS_SEARCH_EXTENSION_ENGINES_INACTIVE_SHORTCUTS_EXPLANATION 1216
#define IDS_WEBSTORE_APP_DESCRIPTION 1217
#define IDS_CONTENT_CONTEXT_ACCESSIBILITY_LABELS_BUBBLE_TEXT 1226
#define IDS_CONTENT_CONTEXT_ACCESSIBILITY_LABELS_BUBBLE_TEXT_ONCE 1227
#define IDS_CONTENT_CONTEXT_SPELLING_BUBBLE_TEXT 1228
#define IDS_CONTENT_CONTEXT_OPENLINKNEWTAB_INAPP 1229
#define IDS_CONTENT_CONTEXT_OPENLINKOFFTHERECORD_INAPP 1230
#define IDS_CONTENT_CONTEXT_OPENLINKOFFTHERECORD_INAPP_V2 1231
#define IDS_AVATAR_BUTTON_MAKING_CHROME_YOURS 1232
#define IDS_AVATAR_BUTTON_SIGNIN_PROMO 1233
#define IDS_UPDATE_RECOMMENDED_DIALOG_TITLE 1234
#define IDS_UPDATE_RECOMMENDED 1235
#define IDS_RELAUNCH_AND_UPDATE 1236
#define IDS_UPDATE_RECOMMENDED_DIALOG_TITLE_ALT 1237
#define IDS_UPDATE_RECOMMENDED_ALT 1238
#define IDS_REINSTALL_APP 1239
#define IDS_UPGRADE_BUBBLE_MENU_ITEM 1240
#define IDS_UPGRADE_BUBBLE_TITLE 1241
#define IDS_UPGRADE_BUBBLE_TEXT 1242
#define IDS_SYNC_ERROR_USER_MENU_UPGRADE_BUTTON 1243
#define IDS_SYNC_ERROR_PASSPHRASE_USER_MENU_ERROR_DESCRIPTION 1244
#define IDS_SYNC_ERROR_TRUSTED_VAULT_USER_MENU_ERROR_DESCRIPTION 1245
#define IDS_SYNC_ERROR_UPGRADE_CLIENT_USER_MENU_TITLE 1246
#define IDS_SYNC_ERROR_UPGRADE_CLIENT_USER_MENU_ERROR_DESCRIPTION 1247
#define IDS_SYNC_UPGRADE_CLIENT 1248
#define IDS_SYNC_UPGRADE_CLIENT_BUTTON 1249
#define IDS_RECOVERY_BUBBLE_TITLE 1250
#define IDS_RUN_RECOVERY 1251
#define IDS_RECOVERY_BUBBLE_TEXT 1252
#define IDS_CRITICAL_NOTIFICATION_TITLE 1253
#define IDS_CRITICAL_NOTIFICATION_TITLE_ALTERNATE 1254
#define IDS_CRITICAL_NOTIFICATION_TEXT 1255
#define IDS_DESKTOP_MEDIA_PICKER_SOURCE_TYPE_TAB 1256
#define IDS_DESKTOP_MEDIA_PICKER_SCREEN_PERMISSION_TEXT_MAC 1257
#define IDS_DESKTOP_MEDIA_PICKER_SCREEN_PERMISSION_TEXT_PERIOD_MAC 1258
#define IDS_DESKTOP_MEDIA_PICKER_WINDOW_PERMISSION_TEXT_MAC 1259
#define IDS_DESKTOP_MEDIA_PICKER_WINDOW_PERMISSION_TEXT_PERIOD_MAC 1260
#define IDS_DESKTOP_MEDIA_PICKER_PERMISSION_RESTART_TEXT_MAC 1261
#define IDS_DESKTOP_MEDIA_PICKER_SYSTEM_AUDIO_PERMISSION_TEXT_MAC 1262
#define IDS_SIDE_PANEL_CUSTOMIZE_CHROME_TITLE 1280
#define IDS_HISTORY_HISTORY_MENU_ITEM 1281
#define IDS_LENS_SEND_FEEDBACK 1282
#define IDS_LENS_SEND_FEEDBACK_PLACEHOLDER 1283
#define IDS_SIDE_PANEL_LENS_OVERLAY_PINNABLE_IPH 1284
#define IDS_SIDE_PANEL_LENS_OVERLAY_PINNABLE_IPH_SCREENREADER 1285
#define IDS_SIDE_PANEL_LENS_OVERLAY_PINNABLE_FOLLOWUP_IPH 1286
#define IDS_SIDE_PANEL_LENS_OVERLAY_PINNABLE_FOLLOWUP_IPH_SCREENREADER 1287
#define IDS_LENS_OVERLAY_COPY 1288
#define IDS_LENS_OVERLAY_COPY_TEXT 1289
#define IDS_LENS_OVERLAY_COPY_TOAST_MESSAGE 1290
#define IDS_LENS_OVERLAY_COPY_AS_IMAGE_TOAST_MESSAGE 1291
#define IDS_LENS_OVERLAY_FEEDBACK_TOAST_MESSAGE 1292
#define IDS_LENS_OVERLAY_FEEDBACK_TOAST_MESSAGE_ALT 1293
#define IDS_LENS_OVERLAY_SEND_FEEDBACK_BUTTON_LABEL 1294
#define IDS_LENS_OVERLAY_CLOSE_FEEDBACK_TOAST_ACCESSIBILITY_LABEL 1295
#define IDS_LENS_OVERLAY_MORE_OPTIONS_BUTTON_LABEL 1296
#define IDS_LENS_OVERLAY_MY_ACTIVITY 1297
#define IDS_LENS_OVERLAY_TOAST_PAGE_CONTENT_NOT_FOUND_MESSAGE 1298
#define IDS_LENS_OVERLAY_TOAST_DISMISS_MESSAGE 1299
#define IDS_LENS_OVERLAY_TRANSLATE 1300
#define IDS_LENS_OVERLAY_SELECT_TEXT 1301
#define IDS_LENS_OVERLAY_COPY_AS_IMAGE 1302
#define IDS_LENS_OVERLAY_SAVE_AS_IMAGE 1303
#define IDS_LENS_OVERLAY_LEARN_MORE 1304
#define IDS_LENS_OVERLAY_INITIAL_TOAST_LABEL 1305
#define IDS_LENS_OVERLAY_INITIAL_TOAST_MESSAGE 1306
#define IDS_LENS_OVERLAY_INITIAL_TOAST_LABEL_SIMPLIFIED 1307
#define IDS_LENS_OVERLAY_INITIAL_TOAST_MESSAGE_SIMPLIFIED 1308
#define IDS_LENS_OVERLAY_INITIAL_TOAST_MESSAGE_SELECT_TEXT 1309
#define IDS_LENS_OVERLAY_INITIAL_TOAST_ERROR_MESSAGE 1310
#define IDS_LENS_OVERLAY_INITIAL_TOAST_ERROR_EXIT_BUTTON_TEXT 1311
#define IDS_LENS_OVERLAY_CURSOR_TOOLTIP_DRAG_MESSAGE 1312
#define IDS_LENS_OVERLAY_CURSOR_TOOLTIP_TEXT_HIGHLIGHT_MESSAGE 1313
#define IDS_LENS_OVERLAY_CURSOR_TOOLTIP_CLICK_MESSAGE 1314
#define IDS_LENS_OVERLAY_CURSOR_TOOLTIP_LIVE_PAGE_MESSAGE 1315
#define IDS_LENS_OVERLAY_AUTO_DETECT_LANGUAGE_LABEL 1316
#define IDS_LENS_OVERLAY_TRANSLATE_BUTTON_LABEL 1317
#define IDS_LENS_OVERLAY_SOURCE_LANGUAGE_PICKER_MENU_TITLE 1318
#define IDS_LENS_OVERLAY_TARGET_LANGUAGE_PICKER_MENU_TITLE 1319
#define IDS_LENS_OVERLAY_RECENT_LANGUAGES_LABEL 1320
#define IDS_LENS_OVERLAY_ALL_LANGUAGES_LABEL 1321
#define IDS_LENS_OVERLAY_DETECTED_LABEL 1322
#define IDS_LENS_OVERLAY_DETECT_LANGUAGE_LABEL 1323
#define IDS_LENS_OVERLAY_RENDERER_LABEL 1324
#define IDS_LENS_OVERLAY_CLOSE_TRANSLATE_SCREEN_LABEL 1325
#define IDS_LENS_OVERLAY_LANGUAGE_PICKER_LABEL 1326
#define IDS_LENS_OVERLAY_SOURCE_LANGUAGE_ACCESSIBILITY_LABEL 1327
#define IDS_LENS_OVERLAY_TARGET_LANGUAGE_ACCESSIBILITY_LABEL 1328
#define IDS_LENS_OVERLAY_SEARCH_LANGUAGE_PICKER_LABEL 1329
#define IDS_LENS_OVERLAY_TOP_LEFT_CORNER_SLIDER_ACCESSIBILITY_LABEL 1330
#define IDS_LENS_OVERLAY_TOP_RIGHT_CORNER_SLIDER_ACCESSIBILITY_LABEL 1331
#define IDS_LENS_OVERLAY_BOTTOM_RIGHT_CORNER_SLIDER_ACCESSIBILITY_LABEL 1332
#define IDS_LENS_OVERLAY_BOTTOM_LEFT_CORNER_SLIDER_ACCESSIBILITY_LABEL 1333
#define IDS_LENS_OVERLAY_SEARCH_SCREENSHOT_ACCESSIBILITY_LABEL 1334
#define IDS_LENS_OVERLAY_TRANSLATE_BUTTON_IPH 1335
#define IDS_LENS_OVERLAY_TRANSLATE_BUTTON_IPH_SCREENREADER 1336
#define IDS_CONTENT_CONTEXT_LENS_OVERLAY 1337
#define IDS_CONTENT_LENS_OVERLAY_ENTRYPOINT_LABEL 1338
#define IDS_CONTENT_LENS_OVERLAY_HOMEWORK_ENTRYPOINT_LABEL 1339
#define IDS_CONTENT_LENS_OVERLAY_ASK_GOOGLE_ENTRYPOINT_LABEL 1340
#define IDS_SIDE_PANEL_LENS_OVERLAY_TOOLBAR_TOOLTIP 1341
#define IDS_LENS_OVERLAY_ENTRYPOINT_LABEL_ALT1 1342
#define IDS_LENS_OVERLAY_ENTRYPOINT_LABEL_ALT2 1343
#define IDS_LENS_OVERLAY_ENTRYPOINT_LABEL_ALT3 1344
#define IDS_LENS_OVERLAY_IMAGE_ENTRYPOINT_LABEL_ALT1 1345
#define IDS_LENS_OVERLAY_IMAGE_ENTRYPOINT_LABEL_ALT2 1346
#define IDS_LENS_OVERLAY_IMAGE_ENTRYPOINT_LABEL_ALT3 1347
#define IDS_LENS_OVERLAY_TAB_ENTRYPOINT_LABEL 1348
#define IDS_LENS_OVERLAY_TAB_ENTRYPOINT_LABEL_V2 1349
#define IDS_LENS_OVERLAY_VIDEO_ENTRYPOINT_LABEL_ALT1 1350
#define IDS_LENS_OVERLAY_VIDEO_ENTRYPOINT_LABEL_ALT2 1351
#define IDS_LENS_OVERLAY_VIDEO_ENTRYPOINT_LABEL_ALT3 1352
#define IDS_AI_MODE_ENTRYPOINT_LABEL 1353
#define IDS_SIDE_PANEL_LENS_OVERLAY_GENERIC_ERROR_PAGE_FIRST_LINE 1354
#define IDS_SIDE_PANEL_LENS_OVERLAY_GENERIC_ERROR_PAGE_SECOND_LINE 1355
#define IDS_SIDE_PANEL_LENS_OVERLAY_PROTECTED_PAGE_ERROR_FIRST_LINE 1356
#define IDS_SIDE_PANEL_LENS_OVERLAY_PROTECTED_PAGE_ERROR_SECOND_LINE 1357
#define IDS_LENS_PERMISSION_BUBBLE_DIALOG_CANCEL_BUTTON 1358
#define IDS_LENS_PERMISSION_BUBBLE_DIALOG_CONTINUE_BUTTON 1359
#define IDS_LENS_PERMISSION_BUBBLE_DIALOG_CSB_DESCRIPTION 1360
#define IDS_LENS_PERMISSION_BUBBLE_DIALOG_NON_BLOCKING_DESCRIPTION 1361
#define IDS_LENS_PERMISSION_BUBBLE_DIALOG_DESCRIPTION 1362
#define IDS_LENS_PERMISSION_BUBBLE_DIALOG_LEARN_MORE_LINK 1363
#define IDS_LENS_PERMISSION_BUBBLE_DIALOG_LEARN_MORE_ABOUT_GOOGLE_LENS_LINK 1364
#define IDS_LENS_PERMISSION_BUBBLE_DIALOG_TITLE 1365
#define IDS_LENS_PERMISSION_BUBBLE_DIALOG_TAB_TO_CONTINUE 1366
#define IDS_LENS_PERMISSION_BUBBLE_DIALOG_GOT_IT 1367
#define IDS_LENS_PERMISSION_BUBBLE_DIALOG_REMIND_ME_LATER 1368
#define IDS_SHOW_LENS_OVERLAY 1370
#define IDS_CONTEXT_MENU_SHOW_GOOGLE_LENS_SHORTCUT 1369
#define IDS_LENS_CONTEXTUAL_SEARCH_DEFAULT_QUERY 1371
#define IDS_LENS_CONTEXTUAL_SEARCH_ZERO_STATE_QUERY 1372
#define IDS_LENS_COMPOSEBOX_HINT_TEXT 1373
#define IDS_LENS_COMPOSEBOX_HINT_TEXT_SEARCH_PAGE 1374
#define IDS_LENS_COMPOSEBOX_HINT_TEXT_SELECT_PAGE 1375
#define IDS_LENS_COMPOSEBOX_HINT_TEXT_ASK_ANYTHING 1376
#define IDS_AI_MODE_ASK_ABOUT_THIS_TAB 1377
#define IDS_AI_MODE_NEW_SEARCH 1378
#define IDS_CONTEXT_MENU_SHOW_AI_MODE_OMNIBOX_BUTTON 1379
#define IDS_RELAUNCH_RECOMMENDED_TITLE 1383
#define IDS_RELAUNCH_RECOMMENDED_BODY 1384
#define IDS_RELAUNCH_REQUIRED_TITLE_DAYS 1385
#define IDS_RELAUNCH_REQUIRED_TITLE_HOURS 1386
#define IDS_RELAUNCH_REQUIRED_TITLE_MINUTES 1387
#define IDS_RELAUNCH_REQUIRED_TITLE_SECONDS 1388
#define IDS_RELAUNCH_REQUIRED_BODY 1389
#define IDS_ADVANCED_PROTECTION_RELAUNCH_REQUIRED_BODY 1390
#define IDS_ENTERPRISE_STARTUP_CLOUD_POLICY_ENROLLMENT_TOOLTIP 1391
#define IDS_ENTERPRISE_STARTUP_CLOUD_POLICY_ENROLLMENT_ERROR 1392
#define IDS_ENTERPRISE_STARTUP_RELAUNCH_BUTTON 1393
#define IDS_DESKTOP_MEDIA_PICKER_TITLE_WEB_CONTENTS_ONLY 1394
#define IDS_IDLE_BUBBLE_TITLE_CLOSE 1395
#define IDS_IDLE_BUBBLE_BODY_CLOSE_AND_CLEAR 1396
#define IDS_IDLE_BUBBLE_BODY_CLEAR 1397
#define IDS_IDLE_BUBBLE_BODY_CLOSE 1398
#define IDS_IDLE_TIMEOUT_CLOSE_TITLE 1399
#define IDS_IDLE_TIMEOUT_CLEAR_TITLE 1400
#define IDS_IDLE_TIMEOUT_CLOSE_AND_CLEAR_TITLE 1401
#define IDS_IDLE_TIMEOUT_CLOSE_BODY 1402
#define IDS_IDLE_TIMEOUT_CLEAR_BODY 1403
#define IDS_IDLE_TIMEOUT_CLOSE_AND_CLEAR_BODY 1404
#define IDS_IDLE_DISMISS_BUTTON 1405
#define IDS_HATS_BUBBLE_TITLE 1406
#define IDS_EXTENSION_PERMISSIONS_BLOCKED_BY_PARENT_PROMPT_MESSAGE 1408
#define IDS_EXTENSION_EXTERNAL_INSTALL_PROMPT_TITLE_APP 1409
#define IDS_EXTENSION_EXTERNAL_INSTALL_PROMPT_TITLE_EXTENSION 1410
#define IDS_EXTENSION_EXTERNAL_INSTALL_PROMPT_TITLE_THEME 1411
#define IDS_EXTENSION_EXTERNAL_INITIAL_INSTALL_PROMPT_TITLE_EXTENSION 1412
#define IDS_EXTENSION_EXTERNAL_INSTALL_PROMPT_ABORT_BUTTON 1413
#define IDS_ENTERPRISE_WELCOME_PROFILE_INFORMATION_DETAILS 1414
#define IDS_PROFILE_PICKER_MAIN_VIEW_TITLE 1415
#define IDS_PROFILE_PICKER_MAIN_VIEW_TITLE_V2 1416
#define IDS_PROFILE_PICKER_MAIN_VIEW_SUBTITLE 1417
#define IDS_PROFILE_PICKER_PROFILE_CREATION_FLOW_PROFILE_TYPE_CHOICE_TITLE 1418
#define IDS_PROFILE_PICKER_IPH_FOR_PROFILES_TEXT 1419
#define IDS_PROFILE_PICKER_IPH_FOR_ADD_PROFILE_TEXT 1420
#define IDS_PROFILE_PICKER_PROFILE_SWITCH_TITLE 1421
#define IDS_PROFILE_PICKER_PROFILE_SWITCH_SUBTITLE 1422
#define IDS_PROFILE_PICKER_FORCE_SIGN_IN_ERROR_DIALOG_NOT_SUPPORTED_BY_GLIC_FLOW_BODY 1423
#define IDS_FRE_SIGN_IN_TITLE_0 1424
#define IDS_FRE_GET_YOUR_BROWSER_READY_TITLE 1425
#define IDS_FRE_DEFAULT_BROWSER_TITLE 1426
#define IDS_FRE_DEFAULT_BROWSER_AND_PINNING_TITLE 1427
#define IDS_FRE_DEFAULT_BROWSER_SUBTITLE_NEW 1428
#define IDS_FRE_DEFAULT_BROWSER_AND_PINNING_SUBTITLE 1429
#define IDS_FRE_DEFAULT_BROWSER_ILLUSTRATION_ALT_TEXT 1430
#define IDS_APP_MENU_TOOLTIP_DEFAULT_PROMPT 1431
#define IDS_APP_MENU_BUTTON_DEFAULT_PROMPT 1432
#define IDS_SET_BROWSER_AS_DEFAULT_MENU_ITEM 1433
#define IDS_FRE_REFRESH_DEFAULT_BROWSER_TITLE 1434
#define IDS_FRE_REFRESH_DEFAULT_BROWSER_SUBTITLE 1435
#define IDS_PROFILE_SWITCH_PROMO 1436
#define IDS_PROFILE_SWITCH_PROMO_SCREENREADER 1437
#define IDS_PASSWORD_MANAGER_IPH_BODY_WEB_APP_PROFILE_SWITCH 1438
#define IDS_SIGNOUT_DICE_WEB_INTERCEPT_BUBBLE_CHROME_SIGNOUT_IPH_TEXT 1439
#define IDS_SIGNOUT_DICE_WEB_INTERCEPT_BUBBLE_CHROME_SIGNOUT_IPH_TEXT_SCREENREADER 1440
#define IDS_QUIET_NOTIFICATION_IPH_TEXT 1441
#define IDS_QUIET_NOTIFICATION_IPH_TEXT_SCREENREADER 1442
#define IDS_CHROMELABS_RELAUNCH_FOOTER_MESSAGE 1443
#define IDS_SERIAL_DEVICE_CHOOSER_AUTHORIZE_BLUETOOTH 1444
#define IDS_TAILORED_SECURITY_UNCONSENTED_PROMOTION_NOTIFICATION_TITLE 1445
#define IDS_TAILORED_SECURITY_UNCONSENTED_PROMOTION_MESSAGE_TITLE 1446
#define IDS_TAILORED_SECURITY_UNCONSENTED_PROMOTION_MESSAGE_DESCRIPTION 1447
#define IDS_TAILORED_SECURITY_UNCONSENTED_PROMOTION_MESSAGE_DESCRIPTION_UPDATED 1448
#define IDS_TAILORED_SECURITY_UNCONSENTED_PROMOTION_MESSAGE_ACCEPT 1449
#define IDS_SETTINGS_SECURITY_JAVASCRIPT_OPTIMIZATION_LINK_ROW_ENABLED 1454
#define IDS_SETTINGS_SECURITY_JAVASCRIPT_OPTIMIZATION_LINK_ROW_DISABLED 1455
#define IDS_SETTINGS_SITE_SETTINGS_JAVASCRIPT_OPTIMIZER_DESCRIPTION 1456
#define IDS_SETTINGS_SITE_SETTINGS_JAVASCRIPT_OPTIMIZER_ALLOWED_SUB_LABEL 1457
#define IDS_SETTINGS_SITE_SETTINGS_JAVASCRIPT_OPTIMIZER_BLOCKED_SUB_LABEL 1458
#define IDS_MEMORY_SAVER_DIALOG_BODY 1524
#define IDS_MEMORY_SAVER_MODE_PROMO_TITLE 1525
#define IDS_PERFORMANCE_INTERVENTION_DIALOG_BODY_V1 1526
#define IDS_PERFORMANCE_INTERVENTION_DIALOG_BODY_SINGULAR_V1 1527
#define IDS_PERFORMANCE_INTERVENTION_DIALOG_TITLE_UPDATED_V3 1528
#define IDS_SETTINGS_CLEAR_BROWSING_DATA_SIGNED_IN 1529
#define IDS_CREATE_SHORTCUT_NOT_APPS_DIALOG_SUBTITLE 1530
#define IDS_CLEAR_BROWSING_DATA_TOAST_BODY 1532
#define IDS_SKILL_SAVED_TOAST_BUTTON 1533
#define IDS_GLIC_PROMO_TITLE 1534
#define IDS_GLIC_TAB_STRIP_BUTTON_TOOLTIP 1535
#define IDS_GLIC_TAB_STRIP_BUTTON_TOOLTIP_CLOSE 1536
#define IDS_GLIC_THREE_DOT_MENU_ITEM 1537
#define IDS_GLIC_STATUS_ICON_TOOLTIP 1538
#define IDS_GLIC_STATUS_ICON_TOOLTIP_CANARY 1539
#define IDS_GLIC_STATUS_ICON_TOOLTIP_DEV 1540
#define IDS_GLIC_STATUS_ICON_TOOLTIP_BETA 1541
#define IDS_GLIC_STATUS_ICON_TOOLTIP_CLOSE 1542
#define IDS_GLIC_STATUS_ICON_TOOLTIP_CLOSE_CANARY 1543
#define IDS_GLIC_STATUS_ICON_TOOLTIP_CLOSE_DEV 1544
#define IDS_GLIC_STATUS_ICON_TOOLTIP_CLOSE_BETA 1545
#define IDS_GLIC_STATUS_ICON_TOOLTIP_TOGGLE 1546
#define IDS_GLIC_STATUS_ICON_TOOLTIP_TOGGLE_CANARY 1547
#define IDS_GLIC_STATUS_ICON_TOOLTIP_TOGGLE_DEV 1548
#define IDS_GLIC_STATUS_ICON_TOOLTIP_TOGGLE_BETA 1549
#define IDS_GLIC_STATUS_ICON_MENU_SHOW 1550
#define IDS_GLIC_STATUS_ICON_MENU_CLOSE 1551
#define IDS_GLIC_STATUS_ICON_MENU_TOGGLE 1552
#define IDS_GLIC_INELIGIBLE_PROFILE_NOTICE 1553
#define IDS_SETTINGS_GLIC_PAGE_TITLE 1554
#define IDS_SETTINGS_GLIC_SECTION_TITLE 1555
#define IDS_SETTINGS_GLIC_ROW_LABEL 1556
#define IDS_SETTINGS_GLIC_BUTTON_TOGGLE_SUBLABEL 1557
#define IDS_SETTINGS_GLIC_OS_WIDGET_TOGGLE 1558
#define IDS_SETTINGS_GLIC_OS_WIDGET_TOGGLE_SUBLABEL 1559
#define IDS_SETTINGS_GLIC_KEYBOARD_SHORTCUT_SUBLABEL 1560
#define IDS_SETTINGS_GLIC_NAVIGATION_SHORTCUT_SUBLABEL 1561
#define IDS_SETTINGS_GLIC_KEEP_SIDEPANEL_OPEN_ON_NEW_TABS_SUBLABEL 1562
#define IDS_SETTINGS_GLIC_SELECTION_SHORTCUT_SUBLABEL 1563
#define IDS_PROFILE_PICKER_MAIN_VIEW_TITLE_GLIC 1564
#define IDS_PROFILE_PICKER_MAIN_VIEW_TITLE_GLIC_NO_PROFILE 1565
#define IDS_PROFILE_PICKER_MAIN_VIEW_SUBTITLE_GLIC 1566
#define IDS_PROFILE_PICKER_MAIN_VIEW_SUBTITLE_GLIC_NO_PROFILE 1567
#define IDS_PROFILE_PICKER_ADD_PROFILE_HELPER_GLIC 1568
#define IDS_GLIC_CONTEXTUAL_CUEING_ANNOUNCEMENT 1569
#define IDS_EXTENSION_INSTALLED_PROMO_EXPLICIT_SIGNIN_MESSAGE 1570
#define IDS_BOOKMARK_INSTALLED_PROMO_EXPLICIT_SIGNIN_MESSAGE 1571
#define IDS_BOOKMARK_INSTALLED_BUBBLE_PROMO_EXPLICIT_SIGNIN_MESSAGE 1572
#define IDS_DICE_MIGRATION_CONFIRMATION_TOAST_MESSAGE 1573
#define IDS_DICE_MIGRATION_CONFIRMATION_TOAST_BUTTON 1574
#define IDS_PROFILE_MENU_ACCOUNT_SETTINGS_BUTTON 1575
#define IDS_IOS_DESKTOP_ESB_PROMO_BUBBLE_DESCRIPTION_REMINDER 1576
#define IDS_IOS_DESKTOP_ESB_PROMO_BUBBLE_TITLE_QR 1577
#define IDS_IOS_DESKTOP_PASSWORD_PROMO_BUBBLE_FOOTER_DESCRIPTION_REMINDER 1578
#define IDS_IOS_DESKTOP_PASSWORD_PROMO_BUBBLE_FOOTER_TITLE 1579
#define IDS_IOS_DESKTOP_PROMO_BUBBLE_BUTTON_ACCEPT_QR 1580
#define IDS_IOS_DESKTOP_PROMO_QR_CODE_ALT_TEXT 1581
#define IDS_IOS_DESKTOP_PROMO_REMINDER_CONFIRMATION 1582
#define IDS_IOS_DESKTOP_PASSWORD_PROMO_REMINDER_CONFIRMATION 1583
#define IDS_IOS_DESKTOP_LENS_PROMO_BUBBLE_DESCRIPTION_REMINDER 1584
#define IDS_IOS_DESKTOP_LENS_PROMO_BUBBLE_TITLE_QR 1585
#define IDS_IOS_DESKTOP_PASSWORD_PROMO_BUBBLE_FOOTER_DESCRIPTION_QR 1586
#define IDS_IOS_DESKTOP_PRICE_TRACKING_DESCRIPTION_QR 1587
#define IDS_IOS_DESKTOP_PRICE_TRACKING_REMINDER_CONFIRMATION 1588
#define IDS_IOS_DESKTOP_TAB_GROUPS_PROMO_BUBBLE_DESCRIPTION_QR 1589
#define IDS_IOS_DESKTOP_TAB_GROUPS_REMINDER_CONFIRMATION 1590
#define IDS_DEFAULT_SEARCH_ENGINE_RESET_NOTIFICATION_TITLE 1591
#define IDS_DEFAULT_SEARCH_ENGINE_RESET_NOTIFICATION_BODY 1592
#define IDS_DEFAULT_SEARCH_ENGINE_RESET_NOTIFICATION_GOT_IT_BUTTON 1593
#define IDS_DEFAULT_SEARCH_ENGINE_RESET_NOTIFICATION_LEARN_MORE_BUTTON 1594
#define IDS_DEFAULT_SEARCH_ENGINE_RESET_NOTIFICATION_SEARCH_SETTINGS_BUTTON 1595
#define IDS_REPORT_UNSAFE_SITE_DIALOG_TITLE 1596
#define IDS_REPORT_UNSAFE_SITE_DIALOG_DESCRIPTION 1597
#define IDS_REPORT_UNSAFE_SITE_DIALOG_FOOTER 1598
#define IDS_JS_OPTIMIZATION_BUBBLE_BODY_NON_POLICY_BODY 1599
#define IDS_SEND_TAB_TO_SELF_SIGN_IN_PROMO_TITLE 1600
#define IDS_SEND_TAB_TO_SELF_SIGN_IN_PROMO_BUTTON_LABEL 1601

// ---------------------------------------------------------------------------
// From cef_strings.h:

#define IDS_TEXT_FILES 58630
#define IDS_CONTENT_CONTEXT_NO_SPELLING_SUGGESTIONS 58631

// ---------------------------------------------------------------------------
// From components_branded_strings.h:

#define IDS_ERRORPAGES_SUGGESTION_NETWORK_PREDICTION_BODY 34390
#define IDS_ERRORPAGES_SUGGESTION_FIREWALL_CONFIG_HEADER 34391
#define IDS_ERRORPAGES_SUMMARY_BLOCKED_BY_CLIENT 34392
#define IDS_ERRORPAGES_SUGGESTION_PROXY_DISABLE_PLATFORM 34393
#define IDS_FLAGS_UI_RELAUNCH_NOTICE 34396
#define IDS_CRASH_DISABLED_MESSAGE 34397
#define IDS_SHORT_PRODUCT_LOGO_ALT_TEXT 34398
#define IDS_VERSION_UI_LICENSE 34399
#define IDS_PAGE_INFO_INTERNAL_PAGE 34402
#define IDS_SESSION_CRASHED_VIEW_MESSAGE 34403
#define IDS_BLUETOOTH_DEVICE_CHOOSER_AUTHORIZE_BLUETOOTH 34404
#define IDS_INTERNAL_DEBUG_PAGES_DISABLED_BODY 34405
#define IDS_IOS_DESKTOP_PASSWORD_PROMO_NOTIFICATION_TITLE 34406
#define IDS_IOS_DESKTOP_SAFE_BROWSING_PROMO_NOTIFICATION_TITLE 34407
#define IDS_IOS_DESKTOP_LENS_PROMO_NOTIFICATION_TITLE 34408

// ---------------------------------------------------------------------------
// From components_strings.h:

#define IDS_ACCESSIBILITY_ANNOTATOR_INFO_TITLE 34420
#define IDS_ACCESSIBILITY_ANNOTATOR_INFO_DESCRIPTION_ANDROID 34421
#define IDS_ACCESSIBILITY_ANNOTATOR_INFO_CARD_1_ANDROID 34422
#define IDS_ACCESSIBILITY_ANNOTATOR_INFO_CARD_2_ANDROID 34423
#define IDS_ACCESSIBILITY_ANNOTATOR_INFO_LEARN_MORE_ANDROID 34424
#define IDS_ACCESSIBILITY_ANNOTATOR_INFO_DESCRIPTION_DESKTOP 34425
#define IDS_ACCESSIBILITY_ANNOTATOR_INFO_CARD_1_DESKTOP 34426
#define IDS_ACCESSIBILITY_ANNOTATOR_INFO_CARD_2_DESKTOP 34427
#define IDS_ACCESSIBILITY_ANNOTATOR_INFO_LEARN_MORE_DESKTOP 34428
#define IDS_ACCESSIBILITY_ANNOTATOR_INFO_PRIMARY_BUTTON 34429
#define IDS_ACCESSIBILITY_ANNOTATOR_INFO_SECONDARY_BUTTON 34430
#define IDS_ACCESSIBILITY_ANNOTATOR_SETTINGS_TITLE 34431
#define IDS_ACCESSIBILITY_ANNOTATOR_SETTINGS_DESCRIPTION_ANDROID 34432
#define IDS_ACCESSIBILITY_ANNOTATOR_SETTINGS_DESCRIPTION_DESKTOP 34433
#define IDS_ASH_ARC_APP_COMPAT_DISABLED_COMPAT_MODE_BUTTON_TOOLTIP_PHONE 34434
#define IDS_ASH_ARC_APP_COMPAT_RESIZE_CONFIRM_TITLE 34435
#define IDS_ASH_ARC_APP_COMPAT_RESIZE_CONFIRM_BODY 34436
#define IDS_ASH_ARC_APP_COMPAT_RESIZE_CONFIRM_ACCEPT 34437
#define IDS_ASH_ARC_APP_COMPAT_RESIZE_CONFIRM_DONT_ASK_ME 34438
#define IDS_ASH_ARC_NEARBY_SHARE_FILE_PREPARATION_PROGRESS 34439
#define IDS_ASH_ARC_NEARBY_SHARE_FILES_PREPARATION_PROGRESS 34440
#define IDS_ASH_ARC_NEARBY_SHARE_ERROR_DIALOG_MESSAGE 34441
#define IDS_ASH_ARC_NEARBY_SHARE_ERROR_DIALOG_MESSAGE_PH 34442
#define IDS_ASH_ARC_NEARBY_SHARE_LOW_DISK_SPACE_DIALOG_TITLE 34443
#define IDS_ASH_ARC_NEARBY_SHARE_LOW_DISK_SPACE_DIALOG_MESSAGE 34444
#define IDS_ASH_ARC_NEARBY_SHARE_LOW_DISK_SPACE_DIALOG_MESSAGE_PH 34445
#define IDS_ASH_ARC_NEARBY_SHARE_LOW_DISK_SPACE_DIALOG_STORAGE_BUTTON 34446
#define IDS_ARC_COMPAT_MODE_RESIZE_TOGGLE_MENU_RESIZE_SETTINGS 34447
#define IDS_ARC_COMPAT_MODE_RESIZE_TOGGLE_MENU_TITLE 34448
#define IDS_ARC_COMPAT_MODE_SPLASH_SCREEN_TITLE 34449
#define IDS_ARC_COMPAT_MODE_SPLASH_SCREEN_BODY 34450
#define IDS_ARC_COMPAT_MODE_SPLASH_SCREEN_BODY_UNRESIZABLE 34451
#define IDS_ARC_COMPAT_MODE_SPLASH_SCREEN_CLOSE 34452
#define IDS_ARC_COMPAT_MODE_SPLASH_SCREEN_LINK 34453
#define IDS_ARC_COMPAT_MODE_DISABLE_RESIZE_LOCK_TOAST 34454
#define IDS_ARC_GHOST_WINDOW_APP_LAUNCHING_ICON 34455
#define IDS_ARC_GHOST_WINDOW_APP_LAUNCHING_THROBBER 34456
#define IDS_ARC_GHOST_WINDOW_APP_LAUNCHING_MESSAGE 34457
#define IDS_ARC_GHOST_WINDOW_APP_LAUNCHING_AOD_MESSAGE 34458
#define IDS_ARC_GHOST_WINDOW_APP_FIXUP_MESSAGE 34459
#define IDS_ASH_ARC_PASSPOINT_APP_APPROVAL_TITLE 34460
#define IDS_ASH_ARC_PASSPOINT_APP_APPROVAL_BODY 34461
#define IDS_ASH_ARC_PASSPOINT_APP_APPROVAL_BODY_WITH_LEARN_MORE 34462
#define IDS_ASH_ARC_PASSPOINT_APP_APPROVAL_SUBSCRIPTION_BODY_WITH_LEARN_MORE 34463
#define IDS_ASH_ARC_PASSPOINT_APP_APPROVAL_LEARN_MORE_LABEL 34464
#define IDS_ASH_ARC_PASSPOINT_APP_APPROVAL_ALLOW_BUTTON 34465
#define IDS_ASH_ARC_PASSPOINT_APP_APPROVAL_DONT_ALLOW_BUTTON 34466
#define IDS_ARC_VM_DATA_MIGRATION_NOTIFICATION_TITLE 34467
#define IDS_ARC_VM_DATA_MIGRATION_NOTIFICATION_DAYS_UNTIL_DEADLINE 34468
#define IDS_ARC_VM_DATA_MIGRATION_NOTIFICATION_ACCEPT_BUTTON_LABEL 34469
#define IDS_ARC_VM_DATA_MIGRATION_DIALOG_TITLE 34470
#define IDS_ARC_VM_DATA_MIGRATION_DIALOG_SAVE_WORK_MESSAGE 34471
#define IDS_ARC_VM_DATA_MIGRATION_DIALOG_DAYS_UNTIL_DEADLINE 34472
#define IDS_ARC_VM_DATA_MIGRATION_DIALOG_UPDATE_NOW_BUTTON_LABEL 34473
#define IDS_ARC_VM_DATA_MIGRATION_DIALOG_SKIP_BUTTON_LABEL 34474
#define IDS_ARC_VM_DATA_MIGRATION_DIALOG_UPDATE_BUTTON_LABEL 34475
#define IDS_ARC_VM_DATA_MIGRATION_LOADING_SCREEN_MESSAGE 34476
#define IDS_ARC_VM_DATA_MIGRATION_WELCOME_SCREEN_TITLE 34477
#define IDS_ARC_VM_DATA_MIGRATION_WHAT_TO_EXPECT_TITLE 34478
#define IDS_ARC_VM_DATA_MIGRATION_UPDATE_DESCRIPTION 34479
#define IDS_ARC_VM_DATA_MIGRATION_BLOCKING_BEHAVIOR_DESCRIPTION 34480
#define IDS_ARC_VM_DATA_MIGRATION_CONNECT_TO_CHARGER_MESSAGE 34481
#define IDS_ARC_VM_DATA_MIGRATION_NOT_ENOUGH_FREE_DISK_SPACE_MESSAGE 34482
#define IDS_ARC_VM_DATA_MIGRATION_NOT_ENOUGH_BATTERY_MESSAGE 34483
#define IDS_ARC_VM_DATA_MIGRATION_SCREEN_UPDATE_BUTTON_LABEL 34484
#define IDS_ARC_VM_DATA_MIGRATION_SCREEN_SKIP_BUTTON_LABEL 34485
#define IDS_ARC_VM_DATA_MIGRATION_RESUME_SCREEN_TITLE 34486
#define IDS_ARC_VM_DATA_MIGRATION_RESUME_DESCRIPTION 34487
#define IDS_ARC_VM_DATA_MIGRATION_SCREEN_RESUME_BUTTON_LABEL 34488
#define IDS_ARC_VM_DATA_MIGRATION_PROGRESS_SCREEN_TITLE 34489
#define IDS_ARC_VM_DATA_MIGRATION_PROGRESS_DESCRIPTION 34490
#define IDS_ARC_VM_DATA_MIGRATION_SUCCESS_SCREEN_TITLE 34491
#define IDS_ARC_VM_DATA_MIGRATION_SCREEN_FINISH_BUTTON_LABEL 34492
#define IDS_ARC_VM_DATA_MIGRATION_FAILURE_SCREEN_TITLE 34493
#define IDS_ARC_VM_DATA_MIGRATION_FAILURE_DESCRIPTION 34494
#define IDS_ARC_VM_DATA_MIGRATION_SEND_FEEDBACK_MESSAGE 34495
#define IDS_ARC_VM_DATA_MIGRATION_SCREEN_REPORT_BUTTON_LABEL 34496
#define IDS_ARC_VM_PRELOAD_NOTIFICATION_TITLE 34497
#define IDS_ARC_VM_PRELOAD_STARTED_MESSAGE 34498
#define IDS_ARC_VM_PRELOAD_SUCCEEDED_MESSAGE 34499
#define IDS_ARC_VM_PRELOAD_FAILED_MESSAGE 34500
#define IDS_AUTOFILL_NO_THANKS_DESKTOP_LOCAL_SAVE 34501
#define IDS_AUTOFILL_NO_THANKS_DESKTOP_UPLOAD_SAVE 34502
#define IDS_AUTOFILL_SAVE_IBAN_BUBBLE_SAVE_NO_THANKS 34503
#define IDS_AUTOFILL_MANDATORY_REAUTH_OPT_IN_NO_THANKS 34504
#define IDS_AUTOFILL_MANDATORY_REAUTH_OPT_IN_ACCEPT 34505
#define IDS_AUTOFILL_GOOGLE_PAY_LOGO_ACCESSIBLE_NAME 34508
#define IDS_AUTOFILL_GOOGLE_WALLET_LOGO_ACCESSIBLE_NAME 34509
#define IDS_AUTOFILL_CHROME_LOGO_ACCESSIBLE_NAME 34510
#define IDS_AUTOFILL_FIELD_LABEL_PHONE 34511
#define IDS_AUTOFILL_SAVE_CARD_BUBBLE_LOCAL_SAVE_ACCEPT 34513
#define IDS_AUTOFILL_SAVE_CARD_BUBBLE_UPLOAD_SAVE_ACCEPT 34514
#define IDS_AUTOFILL_SAVE_CARD_PROMPT_CONTINUE 34516
#define IDS_AUTOFILL_SAVE_CARD_PROMPT_TITLE_LOCAL 34517
#define IDS_AUTOFILL_SAVE_CARD_ONLY_PROMPT_EXPLANATION_LOCAL 34519
#define IDS_AUTOFILL_SAVE_CARD_WITH_CVC_PROMPT_EXPLANATION_LOCAL 34520
#define IDS_AUTOFILL_SAVE_CVC_PROMPT_TITLE_LOCAL 34521
#define IDS_AUTOFILL_SAVE_CVC_PROMPT_EXPLANATION_LOCAL 34522
#define IDS_AUTOFILL_FIX_FLOW_PROMPT_SAVE_CARD_LABEL 34523
#define IDS_AUTOFILL_SAVE_CARD_PROMPT_TITLE_TO_CLOUD 34524
#define IDS_AUTOFILL_SAVE_CARD_PROMPT_TITLE_TO_CLOUD_SECURITY 34526
#define IDS_AUTOFILL_SAVE_CARD_IN_GOOGLE_WALLET_PROMPT_TITLE 34527
#define IDS_AUTOFILL_SAVE_CVC_PROMPT_TITLE_TO_CLOUD 34528
#define IDS_AUTOFILL_SAVE_CARD_PROMPT_CARD_DESCRIPTION 34529
#define IDS_AUTOFILL_SAVE_CARD_PROMPT_CARD_DESCRIPTION_WITH_NICKNAME 34530
#define IDS_AUTOFILL_CARD_SAVED 34534
#define IDS_AUTOFILL_CVC_SAVED 34535
#define IDS_AUTOFILL_DONE 34536
#define IDS_AUTOFILL_SAVE_CARD_PROMPT_UPLOAD_TO_WALLET_EXPLANATION 34538
#define IDS_AUTOFILL_SAVE_CVC_PROMPT_EXPLANATION_UPLOAD 34539
#define IDS_AUTOFILL_SAVE_CVC_TO_WALLET_PROMPT_EXPLANATION_UPLOAD 34540
#define IDS_AUTOFILL_SAVE_CARD_PROMPT_UPLOAD_EXPLANATION_SECURITY 34541
#define IDS_AUTOFILL_SAVE_CARD_PROMPT_UPLOAD_TO_WALLET_EXPLANATION_SECURITY 34542
#define IDS_AUTOFILL_SAVE_CARD_PROMPT_UPLOAD_TO_WALLET_V2_EXPLANATION 34543
#define IDS_AUTOFILL_SAVE_CARD_PROMPT_CARDHOLDER_NAME 34548
#define IDS_AUTOFILL_SAVE_CARD_PROMPT_CARDHOLDER_NAME_TOOLTIP 34549
#define IDS_AUTOFILL_SAVE_CARD_CARDHOLDER_NAME_FIX_FLOW_HEADER 34550
#define IDS_AUTOFILL_SAVE_CARD_UPDATE_EXPIRATION_DATE_TITLE 34551
#define IDS_AUTOFILL_SAVE_CARD_PROMPT_UPLOAD_EXPLANATION_TOOLTIP 34554
#define IDS_AUTOFILL_SAVE_CARD_PROMPT_UPLOAD_EXPLANATION_AND_CARDHOLDER_NAME_TOOLTIP 34555
#define IDS_AUTOFILL_SAVE_CARD_PROMPT_LOADING_THROBBER_ACCESSIBLE_NAME 34556
#define IDS_AUTOFILL_SAVE_IBAN_PROMPT_LOADING_THROBBER_ACCESSIBLE_NAME 34557
#define IDS_AUTOFILL_SAVE_AND_FILL_DIALOG_TITLE 34558
#define IDS_AUTOFILL_SAVE_AND_FILL_DIALOG_EXPLANATION_LOCAL 34559
#define IDS_AUTOFILL_SAVE_AND_FILL_DIALOG_EXPLANATION_UPLOAD 34560
#define IDS_AUTOFILL_SAVE_AND_FILL_TO_WALLET_DIALOG_EXPLANATION_UPLOAD 34561
#define IDS_AUTOFILL_SAVE_AND_FILL_DIALOG_CARD_NUMBER_LABEL 34562
#define IDS_AUTOFILL_SAVE_AND_FILL_DIALOG_CVC_LABEL 34563
#define IDS_AUTOFILL_SAVE_AND_FILL_DIALOG_CVC_PLACEHOLDER 34564
#define IDS_AUTOFILL_SAVE_AND_FILL_DIALOG_EXPIRATION_DATE_LABEL 34565
#define IDS_AUTOFILL_SAVE_AND_FILL_DIALOG_EXPIRATION_DATE_PLACEHOLDER 34566
#define IDS_AUTOFILL_SAVE_AND_FILL_DIALOG_NAME_ON_CARD_LABEL 34567
#define IDS_AUTOFILL_SAVE_AND_FILL_DIALOG_ACCEPT 34568
#define IDS_AUTOFILL_SAVE_AND_FILL_DIALOG_INVALID_CARD_NUMBER 34569
#define IDS_AUTOFILL_SAVE_AND_FILL_DIALOG_INVALID_CVC 34570
#define IDS_AUTOFILL_SAVE_AND_FILL_DIALOG_INVALID_EXPIRATION_DATE 34571
#define IDS_AUTOFILL_SAVE_AND_FILL_DIALOG_INVALID_NAME_ON_CARD 34572
#define IDS_AUTOFILL_PENDING_DIALOG_LOADING_ACCESSIBILITY_DESCRIPTION 34573
#define IDS_AUTOFILL_MASKED_SERVER_CARD_RISK_BASED_UNMASKING_ERROR_TITLE 34574
#define IDS_AUTOFILL_VIRTUAL_CARD_TEMPORARY_ERROR_TITLE 34575
#define IDS_AUTOFILL_VIRTUAL_CARD_PERMANENT_ERROR_TITLE 34576
#define IDS_AUTOFILL_VIRTUAL_CARD_TEMPORARY_ERROR_DESCRIPTION 34577
#define IDS_AUTOFILL_VIRTUAL_CARD_PERMANENT_ERROR_DESCRIPTION 34578
#define IDS_AUTOFILL_VIRTUAL_CARD_NOT_ELIGIBLE_ERROR_TITLE 34579
#define IDS_AUTOFILL_VIRTUAL_CARD_NOT_ELIGIBLE_ERROR_DESCRIPTION 34580
#define IDS_AUTOFILL_CARD_INFO_RETRIEVAL_TEMPORARY_ERROR_TITLE 34581
#define IDS_AUTOFILL_CARD_INFO_RETRIEVAL_PERMANENT_ERROR_TITLE 34582
#define IDS_AUTOFILL_CARD_INFO_RETRIEVAL_TEMPORARY_ERROR_DESCRIPTION 34583
#define IDS_AUTOFILL_CARD_INFO_RETRIEVAL_PERMANENT_ERROR_DESCRIPTION 34584
#define IDS_AUTOFILL_ERROR_DIALOG_NEGATIVE_BUTTON_LABEL 34585
#define IDS_AUTOFILL_CARD_UNMASK_PROMPT_ERROR_TRY_AGAIN_CVC 34586
#define IDS_AUTOFILL_CARD_UNMASK_PROMPT_ERROR_TRY_AGAIN_SECURITY_CODE 34587
#define IDS_AUTOFILL_CARD_UNMASK_PROMPT_ERROR_PERMANENT 34588
#define IDS_AUTOFILL_CARD_UNMASK_PROMPT_ERROR_NETWORK 34589
#define IDS_AUTOFILL_CARD_UNMASK_PROMPT_SECURITY_CODE_POSITION_BACK_OF_CARD 34590
#define IDS_AUTOFILL_CARD_UNMASK_PROMPT_SECURITY_CODE_POSITION_FRONT_OF_CARD 34591
#define IDS_AUTOFILL_CARD_UNMASK_CVC_IMAGE_DESCRIPTION 34592
#define IDS_AUTOFILL_CARD_UNMASK_CVC_IMAGE_DESCRIPTION_FOR_AMEX 34593
#define IDS_AUTOFILL_CARD_UNMASK_PROMPT_INSTRUCTIONS_DEFAULT 34594
#define IDS_AUTOFILL_CARD_UNMASK_PROMPT_INSTRUCTIONS_EXPIRED_CARD 34595
#define IDS_AUTOFILL_CARD_UNMASK_PROMPT_INSTRUCTIONS_VIRTUAL_CARD 34596
#define IDS_AUTOFILL_CARD_UNMASK_VERIFICATION_TITLE 34597
#define IDS_AUTOFILL_CARD_UNMASK_PROMPT_TITLE 34601
#define IDS_AUTOFILL_CARD_UNMASK_PROMPT_TITLE_SECURITY_CODE 34602
#define IDS_AUTOFILL_CARD_UNMASK_PROMPT_EXPIRED_TITLE 34603
#define IDS_AUTOFILL_CARD_UNMASK_CONFIRM_BUTTON 34617
#define IDS_AUTOFILL_CARD_UNMASK_EXPIRATION_MONTH 34618
#define IDS_AUTOFILL_CARD_UNMASK_EXPIRATION_YEAR 34619
#define IDS_AUTOFILL_CARD_UNMASK_VERIFICATION_IN_PROGRESS 34620
#define IDS_AUTOFILL_CARD_UNMASK_VERIFICATION_SUCCESS 34621
#define IDS_AUTOFILL_CARD_UNMASK_INVALID_EXPIRATION_DATE 34622
#define IDS_AUTOFILL_EXPIRATION_DATE_SEPARATOR 34623
#define IDS_AUTOFILL_CARD_UNMASK_NEW_CARD_LINK 34624
#define IDS_AUTOFILL_DIALOG_PLACEHOLDER_CVC 34625
#define IDS_AUTOFILL_DIALOG_ACCESSIBLE_NAME_SECURITY_CODE 34626
#define IDS_AUTOFILL_CARD_UNMASK_PROGRESS_BAR_MESSAGE 34627
#define IDS_AUTOFILL_MASKED_SERVER_CARD_RISK_BASED_UNMASK_PROGRESS_BAR_MESSAGE 34628
#define IDS_AUTOFILL_CARD_UNMASK_PROGRESS_DIALOG_TITLE 34629
#define IDS_AUTOFILL_IBAN_UNMASK_PROGRESS_DIALOG_TITLE 34630
#define IDS_AUTOFILL_IBAN_UNMASK_PROGRESS_BAR_MESSAGE 34631
#define IDS_AUTOFILL_CARD_UNMASK_CONFIRMATION_DIALOG_TITLE 34632
#define IDS_AUTOFILL_CARD_UNMASK_CANCEL_BUTTON_LABEL 34633
#define IDS_AUTOFILL_CARD_UNMASK_CONFIRMATION_MESSAGE 34634
#define IDS_AUTOFILL_3DS_FETCH_VCN_PROGRESS_DIALOG_LOADING_MESSAGE 34636
#define IDS_AUTOFILL_3DS_FETCH_VCN_PROGRESS_DIALOG_CONFIRMATION_MESSAGE 34637
#define IDS_AUTOFILL_IBAN_UNMASK_ERROR_DIALOG_TITLE 34638
#define IDS_AUTOFILL_IBAN_UNMASK_ERROR_DIALOG_MESSAGE 34639
#define IDS_AUTOFILL_WEBAUTHN_OPT_IN_DIALOG_TITLE 34649
#define IDS_AUTOFILL_WEBAUTHN_OPT_IN_DIALOG_TITLE_ERROR 34650
#define IDS_AUTOFILL_WEBAUTHN_OPT_IN_DIALOG_INSTRUCTION 34651
#define IDS_AUTOFILL_WEBAUTHN_OPT_IN_DIALOG_OK_BUTTON_LABEL 34652
#define IDS_AUTOFILL_WEBAUTHN_OPT_IN_DIALOG_INSTRUCTION_ERROR 34653
#define IDS_AUTOFILL_WEBAUTHN_OPT_IN_DIALOG_CANCEL_BUTTON_LABEL 34654
#define IDS_AUTOFILL_WEBAUTHN_OPT_IN_DIALOG_CANCEL_BUTTON_LABEL_ERROR 34655
#define IDS_AUTOFILL_WEBAUTHN_VERIFY_PENDING_DIALOG_TITLE 34656
#define IDS_AUTOFILL_WEBAUTHN_VERIFY_PENDING_DIALOG_CANCEL_BUTTON_LABEL 34657
#define IDS_AUTOFILL_WALLET_MANAGEMENT_LINK_TEXT 34658
#define IDS_AUTOFILL_FROM_GOOGLE_ACCOUNT_LONG 34659
#define IDS_AUTOFILL_SAVE_IBAN_PROMPT_TITLE_LOCAL 34660
#define IDS_AUTOFILL_SAVE_IBAN_LABEL 34661
#define IDS_AUTOFILL_IBANS_TITLE 34662
#define IDS_AUTOFILL_SAVE_IBAN_BUBBLE_SAVE_ACCEPT 34663
#define IDS_AUTOFILL_SAVE_IBAN_PROMPT_NICKNAME 34664
#define IDS_AUTOFILL_SAVE_IBAN_PLACEHOLDER 34665
#define IDS_IBAN_NICKNAME_COUNT_BY 34666
#define IDS_AUTOFILL_IBAN_SAVED 34667
#define IDS_AUTOFILL_MANAGE_SAVED_PAYMENT_METHODS 34668
#define IDS_AUTOFILL_SAVE_IBAN_PROMPT_TITLE_SERVER 34669
#define IDS_AUTOFILL_SAVE_IBAN_TO_WALLET_PROMPT_TITLE 34670
#define IDS_AUTOFILL_UPLOAD_IBAN_PROMPT_EXPLANATION 34671
#define IDS_AUTOFILL_UPLOAD_IBAN_TO_WALLET_PROMPT_EXPLANATION 34672
#define IDS_AUTOFILL_MANDATORY_REAUTH_OPT_IN_TITLE 34678
#define IDS_AUTOFILL_MANDATORY_REAUTH_OPT_IN_EXPLANATION 34679
#define IDS_AUTOFILL_MANDATORY_REAUTH_ICON_TOOLTIP 34680
#define IDS_AUTOFILL_MANDATORY_REAUTH_CONFIRMATION_TITLE 34681
#define IDS_AUTOFILL_MANDATORY_REAUTH_CONFIRMATION_EXPLANATION 34682
#define IDS_AUTOFILL_MANDATORY_REAUTH_CONFIRMATION_SETTINGS_LINK 34683
#define IDS_AUTOFILL_FILLED_CARD_INFORMATION_BUBBLE_TITLE 34687
#define IDS_AUTOFILL_FILLED_CARD_INFORMATION_BUBBLE_EDUCATIONAL_BODY_LABEL 34688
#define IDS_AUTOFILL_FILLED_CARD_INFORMATION_BUBBLE_LEARN_MORE_LINK_LABEL 34689
#define IDS_AUTOFILL_FILLED_CARD_INFORMATION_BUBBLE_CARD_NUMBER_LABEL 34690
#define IDS_AUTOFILL_CARD_INFO_RETRIEVAL_ENROLLED_CARD_UNMASK_PROGRESS_DIALOG_TITLE 34691
#define IDS_AUTOFILL_VIRTUAL_CARD_ENTRY_PREFIX 34692
#define IDS_AUTOFILL_VIRTUAL_CARD_ENROLLMENT_FALLBACK_ICON_TOOLTIP 34693
#define IDS_AUTOFILL_FILLED_CARD_INFORMATION_BUBBLE_TITLE_VIRTUAL_CARD 34694
#define IDS_AUTOFILL_FILLED_CARD_INFORMATION_BUBBLE_EDUCATIONAL_BODY_LABEL_VIRTUAL_CARD 34695
#define IDS_AUTOFILL_FILLED_CARD_INFORMATION_BUBBLE_LEARN_MORE_LINK_LABEL_VIRTUAL_CARD 34696
#define IDS_AUTOFILL_FILLED_CARD_INFORMATION_BUBBLE_CARD_NUMBER_LABEL_VIRTUAL_CARD 34697
#define IDS_AUTOFILL_FILLED_CARD_INFORMATION_BUBBLE_EXP_DATE_LABEL_VIRTUAL_CARD 34698
#define IDS_AUTOFILL_FILLED_CARD_INFORMATION_BUBBLE_CARDHOLDER_NAME_LABEL_VIRTUAL_CARD 34699
#define IDS_AUTOFILL_FILLED_CARD_INFORMATION_BUBBLE_CVC_LABEL_VIRTUAL_CARD 34700
#define IDS_AUTOFILL_FILLED_CARD_INFORMATION_BUBBLE_VIRTUAL_CARD_LABEL 34701
#define IDS_AUTOFILL_FILLED_CARD_INFORMATION_ICON_TOOLTIP_VIRTUAL_CARD 34702
#define IDS_AUTOFILL_FILLED_CARD_INFORMATION_BUBBLE_BUTTON_TOOLTIP_NORMAL_VIRTUAL_CARD 34703
#define IDS_AUTOFILL_FILLED_CARD_INFORMATION_BUBBLE_BUTTON_TOOLTIP_CLICKED_VIRTUAL_CARD 34704
#define IDS_AUTOFILL_VIRTUAL_CARD_SUGGESTION_IPH_BUBBLE_LABEL 34705
#define IDS_AUTOFILL_VIRTUAL_CARD_ENROLLMENT_ACCEPT_BUTTON_LABEL 34706
#define IDS_AUTOFILL_VIRTUAL_CARD_ENROLLMENT_DIALOG_TITLE_LABEL 34707
#define IDS_AUTOFILL_VIRTUAL_CARD_ENROLLMENT_DIALOG_TITLE_LABEL_V2 34708
#define IDS_AUTOFILL_VIRTUAL_CARD_ENROLLMENT_DIALOG_CONTENT_LABEL 34709
#define IDS_AUTOFILL_VIRTUAL_CARD_ENROLLMENT_DIALOG_CONTENT_LABEL_V2 34710
#define IDS_AUTOFILL_VIRTUAL_CARD_ENROLLMENT_DECLINE_BUTTON_LABEL_SKIP 34711
#define IDS_AUTOFILL_VIRTUAL_CARD_ENROLLMENT_DECLINE_BUTTON_LABEL_NO_THANKS 34712
#define IDS_AUTOFILL_VIRTUAL_CARD_ENROLLMENT_LEARN_MORE_LINK_LABEL 34713
#define IDS_AUTOFILL_VIRTUAL_CARD_ENROLLMENT_CARD_IMAGE_TOOLTIP 34714
#define IDS_AUTOFILL_VIRTUAL_CARD_SUGGESTION_OPTION_VALUE 34715
#define IDS_AUTOFILL_IBAN_SUGGESTION_OPTION_VALUE 34716
#define IDS_AUTOFILL_VIRTUAL_CARD_NUMBER_SWITCH_LABEL 34717
#define IDS_AUTOFILL_VIRTUAL_CARD_ENROLL_LOADING_THROBBER_ACCESSIBLE_NAME 34718
#define IDS_AUTOFILL_VIRTUAL_CARD_STANDALONE_CVC_SUGGESTION_IPH_BUBBLE_LABEL 34730
#define IDS_AUTOFILL_VIRTUAL_CARD_STANDALONE_CVC_SUGGESTION_IPH_BUBBLE_LABEL_SCREENREADER 34731
#define IDS_AUTOFILL_VIRTUAL_CARD_STANDALONE_CVC_SUGGESTION_TITLE 34732
#define IDS_AUTOFILL_FILLED_CARD_INFORMATION_BUBBLE_CARD_NUMBER_LABEL_VIRTUAL_CARD_IOS 34733
#define IDS_AUTOFILL_FILLED_CARD_INFORMATION_BUBBLE_EXP_DATE_LABEL_VIRTUAL_CARD_IOS 34734
#define IDS_AUTOFILL_FILLED_CARD_INFORMATION_BUBBLE_NAME_ON_CARD_LABEL_VIRTUAL_CARD_IOS 34735
#define IDS_AUTOFILL_FILLED_CARD_INFORMATION_BUBBLE_CVC_LABEL_VIRTUAL_CARD_IOS 34736
#define IDS_AUTOFILL_VIRTUAL_CARD_DISABLED_SUGGESTION_OPTION_VALUE 34737
#define IDS_AUTOFILL_CARD_INFO_RETRIEVAL_SUGGESTION_IPH_BUBBLE_LABEL 34738
#define IDS_AUTOFILL_CARD_INFO_RETRIEVAL_SUGGESTION_IPH_BUBBLE_LABEL_SCREENREADER 34739
#define IDS_AUTOFILL_CARD_BNPL_AFFIRM_OR_ZIP_SUGGESTION_IPH_BUBBLE_LABEL_DESKTOP 34740
#define IDS_AUTOFILL_CARD_BNPL_AFFIRM_OR_ZIP_SUGGESTION_IPH_BUBBLE_LABEL_DESKTOP_SCREENREADER 34741
#define IDS_AUTOFILL_CARD_BNPL_AFFIRM_ZIP_OR_KLARNA_SUGGESTION_IPH_BUBBLE_LABEL_DESKTOP 34742
#define IDS_AUTOFILL_CARD_BNPL_AFFIRM_ZIP_OR_KLARNA_SUGGESTION_IPH_BUBBLE_LABEL_DESKTOP_SCREENREADER 34743
#define IDS_AUTOFILL_CARD_BNPL_SUGGESTION_WITH_GOOGLE_PAY_IPH_BUBBLE_LABEL_DESKTOP 34744
#define IDS_AUTOFILL_CARD_BNPL_SUGGESTION_WITH_GOOGLE_PAY_IPH_BUBBLE_LABEL_DESKTOP_SCREENREADER 34745
#define IDS_AUTOFILL_CARD_BNPL_SELECT_PROVIDER_PAYMENT_OPTION_AFFIRM_AND_AFTERPAY 34746
#define IDS_AUTOFILL_CARD_BNPL_SELECT_PROVIDER_PAYMENT_OPTION_ZIP 34747
#define IDS_AUTOFILL_CARD_BNPL_SELECT_PROVIDER_PAYMENT_OPTION_KLARNA 34748
#define IDS_AUTOFILL_CARD_BNPL_PAY_LATER_PAYMENT_OPTION_AFFIRM_AND_AFTERPAY 34749
#define IDS_AUTOFILL_CARD_BNPL_PAY_LATER_PAYMENT_OPTION_ZIP 34750
#define IDS_AUTOFILL_CARD_BNPL_PAY_LATER_PAYMENT_OPTION_KLARNA 34751
#define IDS_AUTOFILL_CARD_BNPL_PAY_LATER_CLEAR_FORM_TO_ENABLE 34752
#define IDS_AUTOFILL_CARD_BNPL_PAY_LATER_PAYMENT_OPTION_NOT_AVAILABLE_RIGHT_NOW 34753
#define IDS_AUTOFILL_CARD_BNPL_PAY_LATER_PAYMENT_OPTION_CURRENCY_NOT_SUPPORTED 34754
#define IDS_AUTOFILL_CARD_BNPL_PAY_LATER_PAYMENT_OPTION_NOT_AVAILABLE_FOR_MERCHANT 34755
#define IDS_AUTOFILL_CARD_BNPL_SELECT_PROVIDER_PAYMENT_OPTION_NOT_SUPPORTED_BY_MERCHANT 34756
#define IDS_AUTOFILL_CARD_BNPL_SELECT_PROVIDER_PAYMENT_OPTION_CHECKOUT_AMOUNT_TOO_LOW 34757
#define IDS_AUTOFILL_CARD_BNPL_SELECT_PROVIDER_PAYMENT_OPTION_CHECKOUT_AMOUNT_TOO_HIGH 34758
#define IDS_AUTOFILL_CARD_BNPL_PAY_LATER_PAYMENT_OPTION_CHECKOUT_AMOUNT_TOO_LOW 34759
#define IDS_AUTOFILL_CARD_BNPL_PAY_LATER_PAYMENT_OPTION_CHECKOUT_AMOUNT_TOO_HIGH 34760
#define IDS_AUTOFILL_CARD_BNPL_SELECT_PROVIDER_AI_FOOTNOTE 34761
#define IDS_AUTOFILL_CARD_BNPL_PAY_LATER_OPTIONS_AI_FOOTNOTE 34762
#define IDS_AUTOFILL_CARD_BNPL_SELECT_PROVIDER_FOOTNOTE_FOR_AI_AMOUNT_EXTRACTION_NOTE 34763
#define IDS_AUTOFILL_CARD_BNPL_SELECT_PROVIDER_FOOTNOTE_HIDE_OPTION 34764
#define IDS_AUTOFILL_CARD_BNPL_SELECT_PROVIDER_FOOTNOTE_HIDE_OPTION_PAYMENT_SETTINGS_LINK_TEXT 34765
#define IDS_AUTOFILL_CARD_BNPL_SELECT_PROVIDER_TITLE 34766
#define IDS_AUTOFILL_BNPL_SELECT_PROVIDER_TITLE_DESCRIPTION 34767
#define IDS_AUTOFILL_CARD_BNPL_LINKED_ISSUER_PILL_LABEL 34768
#define IDS_AUTOFILL_PAY_NOW 34769
#define IDS_AUTOFILL_PAY_LATER 34770
#define IDS_AUTOFILL_PAY_NOW_PAY_LATER_TAB_ACCESSIBILITY_ANNOUNCEMENT 34771
#define IDS_AUTOFILL_A11Y_ANNOUNCEMENT_CONCATENATE_TWO_STRINGS 34772
#define IDS_AUTOFILL_CARD_INFO_RETRIEVAL_SUGGESTION_IPH_BUBBLE_FALLBACK_LABEL 34773
#define IDS_AUTOFILL_DISABLED_VIRTUAL_CARD_SUGGESTION_IPH_BUBBLE_LABEL_DESKTOP 34774
#define IDS_AUTOFILL_DISABLED_VIRTUAL_CARD_SUGGESTION_IPH_BUBBLE_LABEL_DESKTOP_SCREENREADER 34775
#define IDS_AUTOFILL_DOWNSTREAM_CARD_AWARENESS_IPH_BUBBLE_LABEL 34776
#define IDS_AUTOFILL_DOWNSTREAM_CARD_AWARENESS_IPH_BUBBLE_LABEL_SCREENREADER 34777
#define IDS_AUTOFILL_CARD_AUTH_SELECTION_DIALOG_TITLE_MULTIPLE_OPTIONS 34778
#define IDS_AUTOFILL_CARD_UNMASK_AUTHENTICATION_SELECTION_DIALOG_ISSUER_CONFIRMATION_TEXT 34779
#define IDS_AUTOFILL_AUTHENTICATION_MODE_SECURITY_CODE 34780
#define IDS_AUTOFILL_AUTHENTICATION_MODE_GET_TEXT_MESSAGE 34781
#define IDS_AUTOFILL_AUTHENTICATION_MODE_TEXT_MESSAGE 34782
#define IDS_AUTOFILL_AUTHENTICATION_MODE_GET_EMAIL 34783
#define IDS_AUTOFILL_AUTHENTICATION_MODE_THREE_DOMAIN_SECURE 34784
#define IDS_AUTOFILL_CARD_UNMASK_AUTHENTICATION_SELECTION_DIALOG_CURRENT_INFO_NOT_SEEN_TEXT 34785
#define IDS_AUTOFILL_CARD_UNMASK_AUTHENTICATION_SELECTION_DIALOG_OK_BUTTON_LABEL_CONTINUE 34786
#define IDS_AUTOFILL_CARD_UNMASK_AUTHENTICATION_SELECTION_DIALOG_OK_BUTTON_LABEL_SEND 34787
#define IDS_AUTOFILL_CARD_UNMASK_AUTHENTICATION_SELECTION_DIALOG_CVC_CHALLENGE_INFO 34788
#define IDS_AUTOFILL_CARD_UNMASK_AUTHENTICATION_SELECTION_DIALOG_THREE_DOMAIN_SECURE_CHALLENGE_INFO 34789
#define IDS_AUTOFILL_CARD_UNMASK_OTP_INPUT_DIALOG_TITLE 34790
#define IDS_AUTOFILL_CARD_UNMASK_OTP_INPUT_DIALOG_FOOTER_MESSAGE 34791
#define IDS_AUTOFILL_CARD_UNMASK_OTP_INPUT_DIALOG_NEW_CODE_MESSAGE 34792
#define IDS_AUTOFILL_CARD_UNMASK_OTP_INPUT_DIALOG_TEXTFIELD_PLACEHOLDER_MESSAGE 34793
#define IDS_AUTOFILL_CARD_UNMASK_OTP_INPUT_DIALOG_PENDING_MESSAGE 34794
#define IDS_AUTOFILL_CARD_UNMASK_OTP_INPUT_DIALOG_VERIFICATION_CODE_EXPIRED_LABEL 34795
#define IDS_AUTOFILL_CARD_UNMASK_OTP_INPUT_DIALOG_ENTER_CORRECT_CODE_LABEL 34796
#define IDS_AUTOFILL_LOADING_AND_CONSENT_DIALOG_TITLE_VCN_3DS 34797
#define IDS_AUTOFILL_PAYMENTS_WINDOW_USER_CONSENT_DIALOG_DESCRIPTION_VCN_3DS 34798
#define IDS_AUTOFILL_PAYMENTS_WINDOW_USER_CONSENT_DIALOG_OK_BUTTON_LABEL_VCN_3DS 34799
#define IDS_AUTOFILL_PAYMENTS_WINDOW_USER_CONSENT_DIALOG_ACCEPTANCE_ACCESSIBILITY_ANNOUNCEMENT 34800
#define IDS_AUTOFILL_REGULAR_CARD_MANUAL_FALLBACK_BUBBLE_CARD_NUMBER_LABEL_IOS 34801
#define IDS_AUTOFILL_BNPL_PAY_LATER_OPTIONS_TEXT 34802
#define IDS_AUTOFILL_BNPL_CREDIT_CARD_SUGGESTION_LABEL 34803
#define IDS_AUTOFILL_BNPL_CREDIT_CARD_SUGGESTION_LABEL_TWO_ISSUERS 34804
#define IDS_AUTOFILL_BNPL_CREDIT_CARD_SUGGESTION_LABEL_THREE_ISSUERS 34805
#define IDS_AUTOFILL_BNPL_SUGGESTION_LABEL_FOR_UNAVAILABLE_PURCHASE 34806
#define IDS_AUTOFILL_BNPL_SETTINGS_LABEL 34807
#define IDS_AUTOFILL_BNPL_SETTINGS_SUBLABEL 34808
#define IDS_AUTOFILL_BNPL_AFFIRM 34809
#define IDS_AUTOFILL_BNPL_ZIP 34810
#define IDS_AUTOFILL_BNPL_AFTER_PAY 34811
#define IDS_AUTOFILL_BNPL_KLARNA 34812
#define IDS_AUTOFILL_GOOGLE_PAY_AND_AFFIRM_LOGO_ACCESSIBLE_NAME 34813
#define IDS_AUTOFILL_GOOGLE_PAY_AND_ZIP_LOGO_ACCESSIBLE_NAME 34814
#define IDS_AUTOFILL_GOOGLE_PAY_AND_AFTERPAY_LOGO_ACCESSIBLE_NAME 34815
#define IDS_AUTOFILL_GOOGLE_PAY_AND_KLARNA_LOGO_ACCESSIBLE_NAME 34816
#define IDS_AUTOFILL_BNPL_FILLED_CARD_INFORMATION_BUBBLE_TITLE 34817
#define IDS_AUTOFILL_BNPL_FILLED_CARD_INFORMATION_BUBBLE_EDUCATIONAL_BODY_LABEL 34818
#define IDS_AUTOFILL_BNPL_FILLED_CARD_SNACKBAR_MESSAGE_TEXT 34819
#define IDS_AUTOFILL_BNPL_TOS_OK_BUTTON_LABEL 34820
#define IDS_AUTOFILL_BNPL_TOS_CANCEL_BUTTON_LABEL 34821
#define IDS_AUTOFILL_BNPL_TOS_TITLE_DESCRIPTION 34822
#define IDS_AUTOFILL_BNPL_TOS_UNLINKED_TITLE 34824
#define IDS_AUTOFILL_BNPL_TOS_LINKED_TITLE 34825
#define IDS_AUTOFILL_BNPL_TOS_REVIEW_TEXT 34826
#define IDS_AUTOFILL_BNPL_TOS_REVIEW_TEXT_WALLET_BRANDING 34827
#define IDS_AUTOFILL_BNPL_TOS_APPROVE_TEXT 34828
#define IDS_AUTOFILL_BNPL_TOS_LINK_TEXT 34829
#define IDS_AUTOFILL_BNPL_FETCH_VCN_PROGRESS_DIALOG_TITLE 34830
#define IDS_AUTOFILL_BNPL_PROGRESS_DIALOG_LOADING_MESSAGE 34831
#define IDS_AUTOFILL_BNPL_FETCH_VCN_PROGRESS_DIALOG_CONFIRMATION_MESSAGE 34832
#define IDS_AUTOFILL_BNPL_ERROR_DIALOG_TITLE 34833
#define IDS_AUTOFILL_BNPL_TEMPORARY_ERROR_DESCRIPTION 34834
#define IDS_AUTOFILL_BNPL_PERMANENT_ERROR_DESCRIPTION 34835
#define IDS_AUTOFILL_BNPL_ERROR_OK_BUTTON 34836
#define IDS_AUTOFILL_BNPL_UNSUPPORTED_CURRENCY_ERROR_DIALOG_TITLE 34838
#define IDS_AUTOFILL_BNPL_UNSUPPORTED_CURRENCY_ERROR_DESCRIPTION 34839
#define IDS_AUTOFILL_CREDIT_CARD_BENEFIT_TEXT_FOR_SUGGESTIONS 34840
#define IDS_AUTOFILL_CREDIT_CARD_BENEFIT_IPH_BUBBLE_LABEL 34841
#define IDS_AUTOFILL_CREDIT_CARD_BENEFIT_IPH_BUBBLE_LABEL_SCREENREADER 34842
#define IDS_AUTOFILL_SETTINGS_PAGE_CARD_BENEFITS_LABEL 34843
#define IDS_AUTOFILL_SETTINGS_PAGE_CARD_BENEFITS_TOGGLE_SUBLABEL_WITH_LEARN_LINK 34844
#define IDS_AUTOFILL_SETTINGS_PAGE_CARD_BENEFITS_TOGGLE_SUBLABEL_WITH_ISSUER_TERMS_APPLY_TEXT_AND_LEARN_LINK 34845
#define IDS_AUTOFILL_SETTINGS_PAGE_BENEFITS_TERMS_TAG_FOR_CREDIT_CARD_LIST_ENTRY 34846
#define IDS_AUTOFILL_SETTINGS_PAGE_BENEFITS_TERMS_ARIA_LABEL 34847
#define IDS_AUTOFILL_CARD_LINKED_OFFER_REMINDER_TITLE 34854
#define IDS_AUTOFILL_OFFERS_REMINDER_POSITIVE_BUTTON_LABEL 34855
#define IDS_AUTOFILL_PROMO_CODE_SUGGESTIONS_FOOTER_TEXT 34856
#define IDS_AUTOFILL_OFFERS_REMINDER_ICON_TOOLTIP_TEXT 34857
#define IDS_AUTOFILL_OFFERS_REMINDER_DESCRIPTION_TEXT 34858
#define IDS_AUTOFILL_GPAY_PROMO_CODE_OFFERS_REMINDER_TITLE 34861
#define IDS_AUTOFILL_PROMO_CODE_OFFER_BUTTON_TOOLTIP_NORMAL 34862
#define IDS_AUTOFILL_PROMO_CODE_OFFER_BUTTON_TOOLTIP_CLICKED 34863
#define IDS_AUTOFILL_GPAY_PROMO_CODE_OFFERS_REMINDER_VALUE_PROP_TEXT 34864
#define IDS_AUTOFILL_PAYMENTS_OTP_VERIFICATION_DIALOG_CANT_FIND_CODE_MESSAGE 34865
#define IDS_AUTOFILL_PAYMENTS_OTP_VERIFICATION_DIALOG_OTP_INPUT_HINT 34866
#define IDS_AUTOFILL_PAYMENTS_OTP_VERIFICATION_DIALOG_POSITIVE_BUTTON_LABEL 34867
#define IDS_AUTOFILL_PAYMENTS_OTP_VERIFICATION_DIALOG_NEGATIVE_BUTTON_LABEL 34868
#define IDS_AUTOFILL_SETTINGS_PAGE_ENABLE_PAYMENT_METHOD_MANDATORY_REAUTH_LABEL 34924
#define IDS_AUTOFILL_SETTINGS_PAGE_ENABLE_PAYMENT_METHOD_MANDATORY_REAUTH_SUBLABEL 34925
#define IDS_AUTOFILL_SETTINGS_PAGE_ENABLE_CVC_STORAGE_LABEL 34928
#define IDS_AUTOFILL_SETTINGS_PAGE_ENABLE_CVC_STORAGE_ARIA_LABEL_FOR_NO_CVC_SAVED 34929
#define IDS_AUTOFILL_SETTINGS_PAGE_ENABLE_CVC_STORAGE_SUBLABEL 34930
#define IDS_AUTOFILL_SETTINGS_PAGE_ENABLE_CVC_STORAGE_WITH_DELETE_LINK_SUBLABEL 34931
#define IDS_AUTOFILL_SETTINGS_PAGE_BULK_REMOVE_CVC_TITLE 34932
#define IDS_AUTOFILL_SETTINGS_PAGE_BULK_REMOVE_CVC_DESCRIPTION 34933
#define IDS_AUTOFILL_SETTINGS_PAGE_BULK_REMOVE_CVC_FROM_WALLET_DESCRIPTION 34934
#define IDS_AUTOFILL_SETTINGS_PAGE_CVC_TAG_FOR_CREDIT_CARD_LIST_ENTRY 34935
#define IDS_AUTOFILL_CVC_SUGGESTION_MAIN_TEXT 34938
#define IDS_AUTOFILL_SETTINGS_PAGE_CVC_SAVED_LABEL 34939
#define IDS_AUTOFILL_SETTINGS_PAGE_SUMMARY_SEPARATED_BY_PIPE 34940
#define IDS_AUTOFILL_SAVE_CARD_CONFIRMATION_SUCCESS_TITLE_TEXT 34944
#define IDS_AUTOFILL_SAVE_CARD_CONFIRMATION_FAILURE_TITLE_TEXT 34945
#define IDS_AUTOFILL_SAVE_IBAN_CONFIRMATION_SUCCESS_TITLE_TEXT 34946
#define IDS_AUTOFILL_SAVE_IBAN_CONFIRMATION_FAILURE_TITLE_TEXT 34947
#define IDS_AUTOFILL_VIRTUAL_CARD_ENROLL_CONFIRMATION_SUCCESS_TITLE_TEXT 34948
#define IDS_AUTOFILL_VIRTUAL_CARD_ENROLL_CONFIRMATION_FAILURE_TITLE_TEXT 34949
#define IDS_AUTOFILL_SAVE_CARD_CONFIRMATION_SUCCESS_DESCRIPTION_TEXT 34950
#define IDS_AUTOFILL_SAVE_CARD_TO_WALLET_CONFIRMATION_SUCCESS_DESCRIPTION_TEXT 34951
#define IDS_AUTOFILL_SAVE_CARD_TO_WALLET_CONFIRMATION_SUCCESS_DESCRIPTION_TEXT_V2 34952
#define IDS_AUTOFILL_SAVE_CARD_CONFIRMATION_FAILURE_DESCRIPTION_TEXT 34953
#define IDS_AUTOFILL_SAVE_CARD_TO_WALLET_CONFIRMATION_FAILURE_DESCRIPTION_TEXT 34954
#define IDS_AUTOFILL_SAVE_IBAN_CONFIRMATION_SUCCESS_DESCRIPTION_TEXT 34955
#define IDS_AUTOFILL_SAVE_IBAN_TO_WALLET_CONFIRMATION_SUCCESS_DESCRIPTION_TEXT 34956
#define IDS_AUTOFILL_SAVE_IBAN_CONFIRMATION_FAILURE_DESCRIPTION_TEXT 34957
#define IDS_AUTOFILL_SAVE_IBAN_TO_WALLET_CONFIRMATION_FAILURE_DESCRIPTION_TEXT 34958
#define IDS_AUTOFILL_SAVE_IBAN_CONFIRMATION_FAILURE_HIT_MAX_STRIKE_DESCRIPTION_TEXT 34959
#define IDS_AUTOFILL_SAVE_IBAN_TO_WALLET_CONFIRMATION_FAILURE_HIT_MAX_STRIKE_DESCRIPTION_TEXT 34960
#define IDS_AUTOFILL_VIRTUAL_CARD_ENROLL_CONFIRMATION_SUCCESS_DESCRIPTION_TEXT 34961
#define IDS_AUTOFILL_VIRTUAL_CARD_ENROLL_CONFIRMATION_FAILURE_DESCRIPTION_TEXT 34962
#define IDS_AUTOFILL_SAVE_CARD_AND_VIRTUAL_CARD_ENROLL_CONFIRMATION_BUTTON_TEXT 34963
#define IDS_AUTOFILL_SAVE_CARD_CONFIRMATION_FAILURE_OK_BUTTON_ACCESSIBLE_NAME 34964
#define IDS_AUTOFILL_VIRTUAL_CARD_ENROLL_CONFIRMATION_FAILURE_OK_BUTTON_ACCESSIBLE_NAME 34965
#define IDS_AUTOFILL_PAYMENT_TEXT 34967
#define IDS_AUTOFILL_IPH_OMNIBOX_PAYMENT_CHIP_LABEL 34968
#define IDS_AUTOFILL_IPH_OMNIBOX_PAYMENT_CHIP_LABEL_SCREENREADER 34969
#define IDS_AUTOFILL_CLEAR_FORM_MENU_ITEM 34970
#define IDS_AUTOFILL_UNDO_MENU_ITEM 34971
#define IDS_AUTOFILL_WARNING_INSECURE_CONNECTION 34972
#define IDS_AUTOFILL_WARNING_MIXED_FORM 34973
#define IDS_AUTOFILL_DELETE_AUTOCOMPLETE_SUGGESTION_CONFIRMATION_BODY 34977
#define IDS_AUTOFILL_DELETE_CREDIT_CARD_SUGGESTION_CONFIRMATION_BODY 34978
#define IDS_AUTOFILL_DELETE_PROFILE_SUGGESTION_CONFIRMATION_BODY 34979
#define IDS_AUTOFILL_REMOVE_HOME_PROFILE_SUGGESTION_CONFIRMATION_TITLE 34980
#define IDS_AUTOFILL_REMOVE_HOME_PROFILE_SUGGESTION_CONFIRMATION_BODY 34981
#define IDS_AUTOFILL_REMOVE_WORK_PROFILE_SUGGESTION_CONFIRMATION_TITLE 34982
#define IDS_AUTOFILL_REMOVE_WORK_PROFILE_SUGGESTION_CONFIRMATION_BODY 34983
#define IDS_AUTOFILL_REMOVE_ACCOUNT_NAME_AND_EMAIL_PROFILE_SUGGESTION_CONFIRMATION_TITLE 34984
#define IDS_AUTOFILL_REMOVE_ACCOUNT_NAME_AND_EMAIL_PROFILE_SUGGESTION_CONFIRMATION_BODY 34985
#define IDS_AUTOFILL_REMOVE_SUGGESTION_BUTTON 34986
#define IDS_AUTOFILL_DELETE_SUGGESTION_BUTTON 34987
#define IDS_AUTOFILL_DELETE_AUTOCOMPLETE_SUGGESTION_TOOLTIP 34988
#define IDS_AUTOFILL_AUTOCOMPLETE_ENTRY_DELETED_A11Y_HINT 34989
#define IDS_AUTOFILL_DELETE_AUTOCOMPLETE_SUGGESTION_A11Y_HINT 34990
#define IDS_HOME_SUGGESTION_VOICE_OVER 34991
#define IDS_WORK_SUGGESTION_VOICE_OVER 34992
#define IDS_AUTOFILL_CC_AMEX 34993
#define IDS_AUTOFILL_CC_AMEX_SHORT 34994
#define IDS_AUTOFILL_CC_DINERS 34995
#define IDS_AUTOFILL_CC_DISCOVER 34996
#define IDS_AUTOFILL_CC_ELO 34997
#define IDS_AUTOFILL_CC_JCB 34998
#define IDS_AUTOFILL_CC_MASTERCARD 34999
#define IDS_AUTOFILL_CC_MIR 35000
#define IDS_AUTOFILL_CC_TROY 35001
#define IDS_AUTOFILL_CC_UNION_PAY 35002
#define IDS_AUTOFILL_CC_VERVE 35003
#define IDS_AUTOFILL_CC_VISA 35004
#define IDS_AUTOFILL_CC_GENERIC 35005
#define IDS_AUTOFILL_IBAN_GENERIC 35006
#define IDS_AUTOFILL_LOYALTY_CARD_GENERIC 35007
#define IDS_AUTOFILL_LOYALTY_CARDS_SUBMENU_TITLE 35008
#define IDS_AUTOFILL_LOYALTY_CARDS_ALL_YOUR_CARDS_SUBMENU_TITLE 35009
#define IDS_AUTOFILL_ADDRESS_SUMMARY_SEPARATOR 35010
#define IDS_AUTOFILL_DEVELOPER_TOOLS 35011
#define IDS_AUTOFILL_TEST_ADDRESS_BY_COUNTRY 35012
#define IDS_AUTOFILL_TEST_ADDRESS_SELECTED_A11Y_HINT 35013
#define IDS_AUTOFILL_EMAIL_VERIFIER_PROMPT_TITLE 35014
#define IDS_AUTOFILL_EMAIL_VERIFIER_PROMPT_BODY 35015
#define IDS_AUTOFILL_EMAIL_VERIFIER_PROMPT_VERIFY 35016
#define IDS_AUTOFILL_EMAIL_VERIFIER_PROMPT_NOT_NOW 35017
#define IDS_AUTOFILL_A11Y_ANNOUNCE_FILLED_FORM 35018
#define IDS_AUTOFILL_A11Y_ANNOUNCE_FILLED_CARD_INFORMATION_ENTRY 35019
#define IDS_AUTOFILL_A11Y_ANNOUNCE_CLEARED_FORM 35020
#define IDS_AUTOFILL_IDENTITY_CREDENTIAL_PROVIDED_MINOR_TEXT 35021
#define IDS_AUTOFILL_IDENTITY_CREDENTIAL_VERIFIED_MINOR_TEXT 35022
#define IDS_AUTOFILL_IDENTITY_CREDENTIAL_VERIFIED_EMAIL_LABEL 35023
#define IDS_AUTOFILL_IDENTITY_CREDENTIAL_LABEL_TEXT 35024
#define IDS_AUTOFILL_MANAGE_ADDRESSES 35025
#define IDS_AUTOFILL_MANAGE_PAYMENT_METHODS 35026
#define IDS_AUTOFILL_MANAGE_LOYALTY_CARDS 35027
#define IDS_AUTOFILL_MANAGE_LOYALTY_CARDS_A11Y_HINT 35028
#define IDS_AUTOFILL_SCAN_CREDIT_CARD 35029
#define IDS_AUTOFILL_SAVE_AND_FILL_SUGGESTION_TITLE 35030
#define IDS_AUTOFILL_LOCAL_SAVE_AND_FILL_SUGGESTION_DESCRIPTION 35031
#define IDS_AUTOFILL_SERVER_SAVE_AND_FILL_SUGGESTION_DESCRIPTION 35032
#define IDS_AUTOFILL_POPUP_ACCESSIBLE_NODE_DATA 35033
#define IDS_AUTOFILL_IPH_EXTERNAL_ACCOUNT_PROFILE_SUGGESTION 35034
#define IDS_AUTOFILL_IPH_HOME_AND_WORK_ACCOUNT_PROFILE_SUGGESTION 35035
#define IDS_AUTOFILL_IPH_HOME_AND_WORK_ACCOUNT_PROFILE_SUGGESTION_SCREENREADER 35036
#define IDS_AUTOFILL_IPH_HOME_AND_WORK_ACCOUNT_PROFILE_SUGGESTION_TITLE 35037
#define IDS_AUTOFILL_IPH_ACCOUNT_NAME_EMAIL_SUGGESTION 35038
#define IDS_AUTOFILL_IPH_ACCOUNT_NAME_EMAIL_SUGGESTION_SCREENREADER 35039
#define IDS_AUTOFILL_IPH_ACCOUNT_NAME_EMAIL_SUGGESTION_TITLE 35040
#define IDS_AUTOFILL_CREDIT_CARD_EXPIRATION_DATE_ABBR 35041
#define IDS_AUTOFILL_CREDIT_CARD_EXPIRATION_DATE_ABBR_V2 35042
#define IDS_AUTOFILL_CREDIT_CARD_TWO_LINE_LABEL_FROM_NAME 35043
#define IDS_AUTOFILL_CREDIT_CARD_TWO_LINE_LABEL_FROM_CARD_NUMBER 35044
#define IDS_AUTOFILL_LOADING_REGIONS 35045
#define IDS_AUTOFILL_ADDRESSES 35046
#define IDS_AUTOFILL_ADDRESSES_TABLE_ARIA_LABEL 35047
#define IDS_AUTOFILL_ENABLE_PROFILES_TOGGLE_SUBLABEL 35048
#define IDS_AUTOFILL_LOCAL_PROFILE_ICON_ACCESSIBILITY_LABEL 35049
#define IDS_AUTOFILL_HOME_PROFILE_ICON_ACCESSIBILITY_LABEL 35050
#define IDS_AUTOFILL_WORK_PROFILE_ICON_ACCESSIBILITY_LABEL 35051
#define IDS_AUTOFILL_ENABLE_CREDIT_CARDS_TOGGLE_SUBLABEL 35052
#define IDS_AUTOFILL_DELETE_ACCOUNT_ADDRESS_RECORD_TYPE_NOTICE 35053
#define IDS_AUTOFILL_DELETE_SYNC_ADDRESS_RECORD_TYPE_NOTICE 35054
#define IDS_AUTOFILL_DELETE_LOCAL_ADDRESS_RECORD_TYPE_NOTICE 35055
#define IDS_AUTOFILL_SAVE_IN_ACCOUNT_MESSAGE_ADDRESS_RECORD_TYPE_NOTICE 35056
#define IDS_AUTOFILL_SAVE_IN_ACCOUNT_MESSAGE_ADDRESS_MIGRATION_RECORD_TYPE_NOTICE 35057
#define IDS_AUTOFILL_ADDRESS_ALREADY_SAVED_IN_ACCOUNT_RECORD_TYPE_NOTICE 35058
#define IDS_AUTOFILL_ADDRESS_WILL_BE_SAVED_IN_ACCOUNT_RECORD_TYPE_NOTICE 35059
#define IDS_AUTOFILL_ADDRESS_HOME_RECORD_TYPE_NOTICE 35060
#define IDS_AUTOFILL_ADDRESS_WORK_RECORD_TYPE_NOTICE 35061
#define IDS_AUTOFILL_ADDRESS_HOME_AND_WORK_RECORD_TYPE_NOTICE 35062
#define IDS_AUTOFILL_EDIT_ADDRESS_REQUIRED_FIELD_FORM_ERROR 35063
#define IDS_AUTOFILL_EDIT_ADDRESS_REQUIRED_FIELDS_FORM_ERROR 35064
#define IDS_AUTOFILL_LOCAL_ADDRESS_ICON_A11Y_LABEL 35065
#define IDS_AUTOFILL_HOME_ADDRESS_ICON_A11Y_LABEL 35066
#define IDS_AUTOFILL_WORK_ADDRESS_ICON_A11Y_LABEL 35067
#define IDS_AUTOFILL_HOME_WORK_ADDRESS_EDIT_A11Y_LABEL 35068
#define IDS_AUTOFILL_SYNC_TOGGLE_LABEL 35069
#define IDS_AUTOFILL_ADDRESSES_SETTINGS_TITLE 35070
#define IDS_AUTOFILL_ADDRESSES_SETTINGS_SUBLABEL 35071
#define IDS_AUTOFILL_SETTINGS_EMAIL_VERIFICATION_LABEL 35072
#define IDS_AUTOFILL_SETTINGS_EMAIL_VERIFICATION_SECTION_TITLE 35073
#define IDS_AUTOFILL_SETTINGS_EMAIL_VERIFICATION_EMPTY_LABEL 35074
#define IDS_AUTOFILL_PAYMENT_METHODS 35075
#define IDS_AUTOFILL_PAYMENT_METHODS_KEYWORDS 35076
#define IDS_AUTOFILL_PAYMENT_METHODS_TABLE_ARIA_LABEL 35077
#define IDS_AUTOFILL_ENABLE_PROFILES_TOGGLE_LABEL 35078
#define IDS_AUTOFILL_DISABLE_SETTINGS_EXPLANATION_TITLE 35079
#define IDS_AUTOFILL_DISABLE_SETTINGS_EXPLANATION 35080
#define IDS_AUTOFILL_DISABLE_SETTINGS_EXPLANATION_V2 35081
#define IDS_AUTOFILL_DISABLE_SETTINGS_BUTTON_LABEL 35082
#define IDS_AUTOFILL_ENABLE_CREDIT_CARDS_TOGGLE_LABEL 35083
#define IDS_AUTOFILL_DELETE_ADDRESS_CONFIRMATION_DIALOG_TITLE 35084
#define IDS_SYNC_DATATYPE_PAYMENTS_AND_INFO 35095
#define IDS_AUTOFILL_SAVE_ADDRESS_PROMPT_OK_BUTTON_LABEL 35104
#define IDS_AUTOFILL_SAVE_ADDRESS_WITH_MORE_INFO_ADDRESS_PROMPT_TITLE 35105
#define IDS_AUTOFILL_SAVE_ADDRESS_PROMPT_MIGRATION_OK_BUTTON_LABEL 35106
#define IDS_AUTOFILL_ACCOUNT_MIGRATE_ADDRESS_PROMPT_TITLE 35107
#define IDS_AUTOFILL_UPDATE_ADDRESS_PROMPT_OK_BUTTON_LABEL 35108
#define IDS_AUTOFILL_ADD_NEW_INFO_ADDRESS_PROMPT_TITLE 35109
#define IDS_AUTOFILL_UPDATE_ADDRESS_ADD_NEW_INFO_PROMPT_OK_BUTTON_LABEL 35110
#define IDS_AUTOFILL_ADDRESS_MIGRATION_TO_ACCOUNT_PROMPT_OK_BUTTON_LABEL 35111
#define IDS_AUTOFILL_ADDRESS_MIGRATION_TO_ACCOUNT_PROMPT_CANCEL_BUTTON_LABEL 35112
#define IDS_AUTOFILL_UPDATE_ADDRESS_PROMPT_NEW_VALUES_SECTION_LABEL 35113
#define IDS_AUTOFILL_UPDATE_ADDRESS_PROMPT_OLD_VALUES_SECTION_LABEL 35114
#define IDS_AUTOFILL_MIGRATE_ADDRESS_PROMPT_CANCEL_BUTTON_LABEL 35115
#define IDS_AUTOFILL_SAVE_IN_ACCOUNT_PROMPT_ADDRESS_SOURCE_NOTICE 35116
#define IDS_AUTOFILL_UPDATE_PROMPT_ACCOUNT_ADDRESS_SOURCE_NOTICE 35117
#define IDS_AUTOFILL_SYNCABLE_PROFILE_MIGRATION_PROMPT_NOTICE 35118
#define IDS_AUTOFILL_LOCAL_PROFILE_MIGRATION_PROMPT_NOTICE 35119
#define IDS_AUTOFILL_SAVE_PROMPT_ADDRESS_SECTION_A11Y_LABEL 35120
#define IDS_AUTOFILL_SAVE_PROMPT_PHONE_SECTION_A11Y_LABEL 35121
#define IDS_AUTOFILL_SAVE_PROMPT_EMAIL_SECTION_A11Y_LABEL 35122
#define IDS_AUTOFILL_CONTACT_INFO_TITLE 35143
#define IDS_AUTOFILL_PAYMENTS_TITLE 35144
#define IDS_AUTOFILL_IDENTITY_DOCS_TITLE 35145
#define IDS_AUTOFILL_SHOPPING_TITLE 35146
#define IDS_AUTOFILL_TRAVEL_TITLE 35147
#define IDS_AUTOFILL_TRAVEL_OPT_IN_TOGGLE_LABEL 35148
#define IDS_AUTOFILL_TRAVEL_OPT_IN_TOGGLE_SUB_LABEL 35149
#define IDS_AUTOFILL_IDENTITY_DOCS_OPT_IN_TOGGLE_LABEL 35150
#define IDS_AUTOFILL_IDENTITY_DOCS_OPT_IN_TOGGLE_SUB_LABEL 35151
#define IDS_AUTOFILL_SHOPPING_OPT_IN_TOGGLE_LABEL 35152
#define IDS_AUTOFILL_SHOPPING_OPT_IN_TOGGLE_SUB_LABEL 35153
#define IDS_AUTOFILL_SAVE_ADDRESS_PROMPT_TITLE 35123
#define IDS_AUTOFILL_SAVE_ADDRESS_PROMPT_CANCEL_BUTTON_LABEL 35161
#define IDS_AUTOFILL_SAVE_ADDRESS_PROMPT_EDIT_BUTTON_TOOLTIP 35126
#define IDS_AUTOFILL_UPDATE_ADDRESS_PROMPT_TITLE 35124
#define IDS_AUTOFILL_UPDATE_ADDRESS_ADD_NEW_INFO_PROMPT_CANCEL_BUTTON_LABEL 35162
#define IDS_AUTOFILL_UPDATE_ADDRESS_PROMPT_CANCEL_BUTTON_LABEL 35163
#define IDS_AUTOFILL_SAVE_ADDRESS_PROMPT_NAME_EMAIL_HOME_WORK_MERGE_PROMPT_TITLE 35164
#define IDS_AUTOFILL_SAVE_ADDRESS_PROMPT_NAME_EMAIL_HOME_WORK_MERGE_OK_BUTTON_LABEL 35165
#define IDS_AUTOFILL_SAVE_ADDRESS_PROMPT_NAME_EMAIL_HOME_WORK_MERGE_CANCEL_BUTTON_LABEL 35166
#define IDS_AUTOFILL_EDIT_ADDRESS_DIALOG_TITLE 35127
#define IDS_AUTOFILL_EDIT_ADDRESS_DIALOG_OK_BUTTON_LABEL_SAVE 35167
#define IDS_AUTOFILL_MIGRATE_ADDRESS_DIALOG_OK_BUTTON_LABEL_SAVE 35168
#define IDS_AUTOFILL_EDIT_ADDRESS_DIALOG_OK_BUTTON_LABEL_UPDATE 35169
#define IDS_AUTOFILL_EDIT_ADDRESS_DIALOG_CANCEL_BUTTON_LABEL 35170
#define IDS_AUTOFILL_EXPANDABLE_SUGGESTION_FILL_ADDRESS_A11Y_ADDON 35171
#define IDS_AUTOFILL_EXPANDABLE_SUGGESTION_EXPAND_SHORTCUT 35172
#define IDS_AUTOFILL_EXPANDABLE_SUGGESTION_EXPAND_SHORTCUT_RTL 35173
#define IDS_AUTOFILL_EXPANDABLE_SUGGESTION_SUBMENU_HINT 35174
#define IDS_AUTOFILL_SIGNIN_PROMO_TITLE_PASSWORD 35175
#define IDS_AUTOFILL_VERIFY_PROMO_SUBTITLE_PASSWORD 35176
#define IDS_AUTOFILL_POPUP_SEARCH_BAR_CLEAR_SEARCH_BUTTON_A11Y_NAME 35177
#define IDS_AUTOFILL_POPUP_SEARCH_BAR_PASSWORDS_INPUT_PLACEHOLDER 35178
#define IDS_AUTOFILL_AT_MEMORY_POPUP_SEARCH_BAR_PLACEHOLDER 35179
#define IDS_AUTOFILL_POPUP_SEARCH_BAR_PASSWORDS_NOT_FOUND 35180
#define IDS_AUTOFILL_SIGNIN_PROMO_TITLE_ADDRESS 35181
#define IDS_AUTOFILL_VERIFY_PROMO_SUBTITLE_ADDRESS 35182
#define IDS_AUTOFILL_IPH_LOYALTY_CARD_SUGGESTION_TITLE 35183
#define IDS_AUTOFILL_IPH_LOYALTY_CARD_SUGGESTION_BODY 35184
#define IDS_AUTOFILL_IPH_LOYALTY_CARD_SUGGESTION_SCREENREADER 35185
#define IDS_PASSWORD_MANAGER_UI_PASSWORD_RECOVERY_SHOWN_A11Y_ANNOUNCEMENT 35186
#define IDS_AUTOFILL_AI_SAVE_OR_UPDATE_LOCAL_ENTITY_SOURCE_NOTICE 35187
#define IDS_AUTOFILL_AT_MEMORY_SOURCE_ATTRIBUTION_TITLE 35188
#define IDS_AUTOFILL_AT_MEMORY_SOURCE_ATTRIBUTION_DESCRIPTION 35189
#define IDS_AUTOFILL_AT_MEMORY_SOURCE_GMAIL 35190
#define IDS_AUTOFILL_AT_MEMORY_SOURCE_CALENDAR 35191
#define IDS_AUTOFILL_AT_MEMORY_SOURCE_PHOTOS 35192
#define IDS_AUTOFILL_AT_MEMORY_SOURCE_AMBIENT 35193
#define IDS_AUTOFILL_AT_MEMORY_SOURCE_LIVETABS 35194
#define IDS_AUTOFILL_AT_MEMORY_NO_DATA 35195
#define IDS_AUTOFILL_AT_MEMORY_NO_CONNECTION 35196
#define IDS_AUTOFILL_AT_MEMORY_SEARCH_AFFORDANCE_SUBTITLE 35197
#define IDS_AUTOFILL_AT_MEMORY_UNSUPPORTED_QUERY_TITLE 35198
#define IDS_AUTOFILL_AT_MEMORY_UNSUPPORTED_QUERY_DESCRIPTION 35199
#define IDS_AUTOFILL_AT_MEMORY_MANAGE_SUGGESTION_MAIN_TEXT 35202
#define IDS_AUTOFILL_ADDRESS_EDIT_DIALOG_HISPANIC_MUNICIPIO 35203
#define IDS_AUTOFILL_ADDRESS_EDIT_DIALOG_FRENCH_LOCALITY_2 35204
#define IDS_AUTOFILL_ADDRESS_EDIT_DIALOG_JAPANESE_ALTERNATIVE_NAME 35205
#define IDS_SETTINGS_AUTOFILL_AI_ENTERPRISE_LOGGING_MANAGED_DISABLED 35208
#define IDS_AUTOFILL_AI_DRIVERS_LICENSE_ENTITY_NAME 35209
#define IDS_AUTOFILL_AI_NATIONAL_ID_CARD_ENTITY_NAME 35210
#define IDS_AUTOFILL_AI_ID_CARD_ENTITY_NAME 35211
#define IDS_AUTOFILL_AI_PASSPORT_ENTITY_NAME 35212
#define IDS_AUTOFILL_AI_VEHICLE_ENTITY_NAME 35213
#define IDS_AUTOFILL_AI_KNOWN_TRAVELER_NUMBER_ENTITY_NAME 35214
#define IDS_AUTOFILL_AI_REDRESS_NUMBER_ENTITY_NAME 35215
#define IDS_AUTOFILL_AI_FLIGHT_RESERVATION_ENTITY_NAME 35216
#define IDS_AUTOFILL_AI_ORDER_ENTITY_NAME 35217
#define IDS_AUTOFILL_AI_SHIPMENT_ENTITY_NAME 35218
#define IDS_AUTOFILL_AI_DRIVERS_LICENSES_TITLE 35219
#define IDS_AUTOFILL_AI_NATIONAL_IDS_TITLE 35220
#define IDS_AUTOFILL_AI_NATIONAL_IDS_SHORT_TITLE 35221
#define IDS_AUTOFILL_AI_PASSPORTS_TITLE 35222
#define IDS_AUTOFILL_AI_VEHICLES_TITLE 35223
#define IDS_AUTOFILL_AI_KNOWN_TRAVELER_NUMBER_TITLE 35224
#define IDS_AUTOFILL_AI_REDRESS_NUMBER_TITLE 35225
#define IDS_AUTOFILL_AI_TRAVEL_INFO_TITLE 35226
#define IDS_AUTOFILL_AI_FLIGHT_RESERVATIONS_TITLE 35227
#define IDS_AUTOFILL_AI_ORDERS_TITLE 35228
#define IDS_AUTOFILL_AI_SHIPMENTS_TITLE 35229
#define IDS_AUTOFILL_AI_ADD_DRIVERS_LICENSE_ENTITY 35230
#define IDS_AUTOFILL_AI_ADD_NATIONAL_ID_CARD_ENTITY 35231
#define IDS_AUTOFILL_AI_ADD_PASSPORT_ENTITY 35232
#define IDS_AUTOFILL_AI_ADD_VEHICLE_ENTITY 35233
#define IDS_AUTOFILL_AI_ADD_KNOWN_TRAVELER_NUMBER_ENTITY 35234
#define IDS_AUTOFILL_AI_ADD_REDRESS_NUMBER_ENTITY 35235
#define IDS_AUTOFILL_AI_EDIT_DRIVERS_LICENSE_ENTITY 35236
#define IDS_AUTOFILL_AI_EDIT_NATIONAL_ID_CARD_ENTITY 35237
#define IDS_AUTOFILL_AI_EDIT_PASSPORT_ENTITY 35238
#define IDS_AUTOFILL_AI_EDIT_VEHICLE_ENTITY 35239
#define IDS_AUTOFILL_AI_EDIT_KNOWN_TRAVELER_NUMBER_ENTITY 35240
#define IDS_AUTOFILL_AI_EDIT_REDRESS_NUMBER_ENTITY 35241
#define IDS_AUTOFILL_AI_DELETE_DRIVERS_LICENSE_ENTITY 35242
#define IDS_AUTOFILL_AI_DELETE_NATIONAL_ID_CARD_ENTITY 35243
#define IDS_AUTOFILL_AI_DELETE_PASSPORT_ENTITY 35244
#define IDS_AUTOFILL_AI_DELETE_VEHICLE_ENTITY 35245
#define IDS_AUTOFILL_AI_DELETE_KNOWN_TRAVELER_NUMBER_ENTITY 35246
#define IDS_AUTOFILL_AI_DELETE_REDRESS_NUMBER_ENTITY 35247
#define IDS_AUTOFILL_AI_ORDER_ID_ATTRIBUTE_NAME 35248
#define IDS_AUTOFILL_AI_ORDER_ACCOUNT_ATTRIBUTE_NAME 35249
#define IDS_AUTOFILL_AI_ORDER_DATE_ATTRIBUTE_NAME 35250
#define IDS_AUTOFILL_AI_ORDER_MERCHANT_NAME_ATTRIBUTE_NAME 35251
#define IDS_AUTOFILL_AI_ORDER_MERCHANT_DOMAIN_ATTRIBUTE_NAME 35252
#define IDS_AUTOFILL_AI_ORDER_PRODUCT_NAMES_ATTRIBUTE_NAME 35253
#define IDS_AUTOFILL_AI_SHIPMENT_TRACKING_NUMBER_ATTRIBUTE_NAME 35254
#define IDS_AUTOFILL_AI_SHIPMENT_DELIVERY_ZIP_CODE_ATTRIBUTE_NAME 35255
#define IDS_AUTOFILL_AI_SHIPMENT_CARRIER_NAME_ATTRIBUTE_NAME 35256
#define IDS_AUTOFILL_AI_SHIPMENT_CARRIER_DOMAIN_ATTRIBUTE_NAME 35257
#define IDS_AUTOFILL_AI_SHIPMENT_ESTIMATED_DELIVERY_DATE_ATTRIBUTE_NAME 35258
#define IDS_AUTOFILL_AI_SHIPMENT_ORDER_IDS_ATTRIBUTE_NAME 35259
#define IDS_AUTOFILL_AI_SHIPMENT_ORDER_DATES_ATTRIBUTE_NAME 35260
#define IDS_AUTOFILL_AI_SHIPMENT_MERCHANT_NAME_ATTRIBUTE_NAME 35261
#define IDS_AUTOFILL_AI_SHIPMENT_PRODUCT_NAMES_ATTRIBUTE_NAME 35262
#define IDS_AUTOFILL_AI_DRIVERS_LICENSE_NAME_ATTRIBUTE_NAME 35263
#define IDS_AUTOFILL_AI_DRIVERS_LICENSE_STATE_ATTRIBUTE_NAME 35264
#define IDS_AUTOFILL_AI_DRIVERS_LICENSE_NUMBER_ATTRIBUTE_NAME 35265
#define IDS_AUTOFILL_AI_DRIVERS_LICENSE_NUMBER_ATTRIBUTE_NAME_NEW 35266
#define IDS_AUTOFILL_AI_DRIVERS_LICENSE_EXPIRATION_DATE_ATTRIBUTE_NAME 35267
#define IDS_AUTOFILL_AI_DRIVERS_LICENSE_ISSUE_DATE_ATTRIBUTE_NAME 35268
#define IDS_AUTOFILL_AI_NATIONAL_ID_CARD_NAME_ATTRIBUTE_NAME 35269
#define IDS_AUTOFILL_AI_NATIONAL_ID_CARD_NUMBER_ATTRIBUTE_NAME 35270
#define IDS_AUTOFILL_AI_NATIONAL_ID_CARD_COUNTRY_ATTRIBUTE_NAME 35271
#define IDS_AUTOFILL_AI_NATIONAL_ID_CARD_COUNTRY_ATTRIBUTE_NAME_NEW 35272
#define IDS_AUTOFILL_AI_NATIONAL_ID_CARD_EXPIRATION_DATE_ATTRIBUTE_NAME 35273
#define IDS_AUTOFILL_AI_NATIONAL_ID_CARD_ISSUE_DATE_ATTRIBUTE_NAME 35274
#define IDS_AUTOFILL_AI_NATIONAL_ID_CARD_ISSUE_DATE_ATTRIBUTE_NAME_NEW 35275
#define IDS_AUTOFILL_AI_KNOWN_TRAVELER_NUMBER_NAME_ATTRIBUTE_NAME 35276
#define IDS_AUTOFILL_AI_KNOWN_TRAVELER_NUMBER_ATTRIBUTE_NAME 35277
#define IDS_AUTOFILL_AI_KNOWN_TRAVELER_NUMBER_EXPIRATION_DATE_ATTRIBUTE_NAME 35278
#define IDS_AUTOFILL_AI_REDRESS_NUMBER_NAME_ATTRIBUTE_NAME 35279
#define IDS_AUTOFILL_AI_REDRESS_NUMBER_ATTRIBUTE_NAME 35280
#define IDS_AUTOFILL_AI_PASSPORT_NAME_ATTRIBUTE_NAME 35281
#define IDS_AUTOFILL_AI_PASSPORT_NUMBER_ATTRIBUTE_NAME 35282
#define IDS_AUTOFILL_AI_PASSPORT_COUNTRY_ATTRIBUTE_NAME 35283
#define IDS_AUTOFILL_AI_PASSPORT_EXPIRATION_DATE_ATTRIBUTE_NAME 35284
#define IDS_AUTOFILL_AI_PASSPORT_ISSUE_DATE_ATTRIBUTE_NAME 35285
#define IDS_AUTOFILL_AI_VEHICLE_MAKE_ATTRIBUTE_NAME 35286
#define IDS_AUTOFILL_AI_VEHICLE_MODEL_ATTRIBUTE_NAME 35287
#define IDS_AUTOFILL_AI_VEHICLE_YEAR_ATTRIBUTE_NAME 35288
#define IDS_AUTOFILL_AI_VEHICLE_OWNER_ATTRIBUTE_NAME 35289
#define IDS_AUTOFILL_AI_VEHICLE_PLATE_NUMBER_ATTRIBUTE_NAME 35290
#define IDS_AUTOFILL_AI_VEHICLE_PLATE_STATE_ATTRIBUTE_NAME 35291
#define IDS_AUTOFILL_AI_VEHICLE_VEHICLE_IDENTIFICATION_NUMBER_ATTRIBUTE_NAME 35292
#define IDS_AUTOFILL_AI_MANAGE_SUGGESTION_MAIN_TEXT 35293
#define IDS_AUTOFILL_AI_MANAGE_IDENTITY_DOCS_SUGGESTION_MAIN_TEXT 35294
#define IDS_AUTOFILL_AI_MANAGE_TRAVEL_SUGGESTION_MAIN_TEXT 35295
#define IDS_AUTOFILL_PREDICTION_IMPROVEMENTS_SAVE_DIALOG_NO_THANKS_BUTTON 35296
#define IDS_AUTOFILL_PREDICTION_IMPROVEMENTS_SAVE_DIALOG_SAVE_BUTTON 35297
#define IDS_AUTOFILL_PREDICTION_IMPROVEMENTS_UPDATE_DIALOG_UPDATE_BUTTON 35298
#define IDS_AUTOFILL_AI_OPT_IN_IPH_TITLE 35299
#define IDS_AUTOFILL_AI_OPT_IN_IPH_BODY 35300
#define IDS_AUTOFILL_AI_OPT_IN_IPH_TURN_ON 35301
#define IDS_AUTOFILL_AI_OPT_IN_IPH_MAYBE_LATER 35302
#define IDS_AUTOFILL_AI_VALUABLES_IPH_TITLE 35303
#define IDS_AUTOFILL_AI_VALUABLES_IPH_BODY 35304
#define IDS_AUTOFILL_GOOGLE_WALLET_TITLE 35305
#define IDS_AUTOFILL_AI_UPDATE_ENTITY_TO_WALLET_DIALOG_SUBTITLE 35306
#define IDS_AUTOFILL_AI_SAVE_ENTITY_TO_WALLET_DIALOG_SUBTITLE_NEW 35307
#define IDS_AUTOFILL_AI_SAVE_DRIVERS_LICENSE_ENTITY_DIALOG_TITLE 35308
#define IDS_AUTOFILL_AI_SAVE_NATIONAL_ID_CARD_ENTITY_DIALOG_TITLE 35309
#define IDS_AUTOFILL_AI_SAVE_ID_CARD_ENTITY_DIALOG_TITLE 35310
#define IDS_AUTOFILL_AI_SAVE_PASSPORT_ENTITY_DIALOG_TITLE 35311
#define IDS_AUTOFILL_AI_SAVE_VEHICLE_ENTITY_DIALOG_TITLE 35312
#define IDS_AUTOFILL_AI_SAVE_KNOWN_TRAVELER_NUMBER_ENTITY_DIALOG_TITLE 35313
#define IDS_AUTOFILL_AI_SAVE_REDRESS_NUMBER_ENTITY_DIALOG_TITLE 35314
#define IDS_AUTOFILL_AI_UPDATE_DRIVERS_LICENSE_ENTITY_DIALOG_TITLE 35315
#define IDS_AUTOFILL_AI_UPDATE_NATIONAL_ID_CARD_ENTITY_DIALOG_TITLE 35316
#define IDS_AUTOFILL_AI_UPDATE_ID_CARD_ENTITY_DIALOG_TITLE 35317
#define IDS_AUTOFILL_AI_UPDATE_PASSPORT_ENTITY_DIALOG_TITLE 35318
#define IDS_AUTOFILL_AI_UPDATE_VEHICLE_ENTITY_DIALOG_TITLE 35319
#define IDS_AUTOFILL_AI_UPDATE_KNOWN_TRAVELER_NUMBER_ENTITY_DIALOG_TITLE 35320
#define IDS_AUTOFILL_AI_UPDATE_REDRESS_NUMBER_ENTITY_DIALOG_TITLE 35321
#define IDS_AUTOFILL_AI_UPDATE_ENTITY_DIALOG_SUBTITLE 35322
#define IDS_AUTOFILL_AI_SAVE_ENTITY_DIALOG_SUBTITLE 35323
#define IDS_AUTOFILL_AI_SAVE_ENTITY_TO_WALLET_DIALOG_SUBTITLE 35324
#define IDS_AUTOFILL_MANAGE_YOUR_INFO_LINK 35325
#define IDS_AUTOFILL_AI_UPDATE_ENTITY_DIALOG_UPDATED_ATTRIBUTE_ACCESSIBLE_NAME 35326
#define IDS_AUTOFILL_AI_UPDATE_ENTITY_DIALOG_NEW_ATTRIBUTE_ACCESSIBLE_NAME 35327
#define IDS_AUTOFILL_AI_UPDATE_ENTITY_DIALOG_NEW_ATTRIBUTE_ACCESSIBLE_NAME_V2 35328
#define IDS_AUTOFILL_AI_UPDATE_ENTITY_DIALOG_UPDATED_ATTRIBUTE_ACCESSIBLE_NAME_V2 35329
#define IDS_AUTOFILL_AI_WALLET_UPLOAD_THROBBER_ACCESSIBLE_NAME 35330
#define IDS_AUTOFILL_AI_LOCAL_SAVE_NOTIFICATION_TITLE 35331
#define IDS_AUTOFILL_AI_LOCAL_SAVE_NOTIFICATION_DESCRIPTION 35332
#define IDS_AUTOFILL_AI_LOCAL_SAVE_NOTIFICATION_DISMISS_BUTTON_LABEL 35333
#define IDS_AUTOFILL_AI_WALLET_UPDATE_OR_MIGRATE_FAILURE_NOTIFICATION 35334
#define IDS_AUTOFILL_AI_WALLET_FETCH_FAILURE_NOTIFICATION 35335
#define IDS_AUTOFILL_AI_FFR_WHEN_ON_TITLE 35367
#define IDS_AUTOFILL_AI_FFR_WHEN_ON_DESCRIPTION 35368
#define IDS_AUTOFILL_AI_FFR_CONSIDER_DESCRIPTION 35369
#define IDS_AUTOFILL_AI_FFR_CONSIDER_LEARN_MORE 35370
#define IDS_SETTINGS_AUTOFILL_AI_AUTHENTICATION_TOGGLE_TITLE 35371
#define IDS_AT_MEMORY_PROMO_TITLE 35372
#define IDS_AT_MEMORY_PROMO_DESCRIPTION 35373
#define IDS_AT_MEMORY_PROMO_LEARN_MORE_BUTTON 35374
#define IDS_AT_MEMORY_PROMO_GOT_IT_BUTTON 35375
#define IDS_SETTINGS_AUTOFILL_AI_AUTHENTICATION_TOGGLE_SUBTITLE 35376
#define IDS_SETTINGS_AUTOFILL_AI_PAGE_TITLE 35377
#define IDS_SETTINGS_AUTOFILL_AI_WHEN_ON 35378
#define IDS_SETTINGS_AUTOFILL_AI_THINGS_TO_CONSIDER 35379
#define IDS_SETTINGS_AUTOFILL_SERVICE_PROVIDER 35380
#define IDS_SETTINGS_AUTOFILL_AI_TO_CONSIDER_DATA_USAGE 35381
#define IDS_SETTINGS_AUTOFILL_AI_DESCRIPTION 35382
#define IDS_SETTINGS_AUTOFILL_AI_TOGGLE_SUB_LABEL 35383
#define IDS_SETTINGS_AUTOFILL_AI_WHEN_ON_CAN_FILL_DIFFICULT_FIELDS 35384
#define IDS_AUTOFILL_WALLET_DATA_SHARING_PROMO_TITLE 35385
#define IDS_AUTOFILL_WALLET_DATA_SHARING_PROMO_SUBTITLE 35386
#define IDS_AUTOFILL_WALLET_DATA_SHARING_PROMO_BUTTON_LABEL 35387
#define IDS_BOOKMARK_BAR_FOLDER_NAME 35390
#define IDS_BOOKMARK_BAR_MOBILE_FOLDER_NAME 35391
#define IDS_BOOKMARK_BAR_OTHER_FOLDER_NAME 35392
#define IDS_BOOKMARK_BAR_MANAGED_FOLDER_DOMAIN_NAME 35393
#define IDS_BOOKMARK_BAR_MANAGED_FOLDER_DEFAULT_NAME 35394
#define IDS_BOOKMARK_EDITOR_TITLE 35395
#define IDS_BOOKMARK_MOVE_DIALOG_TITLE 35396
#define IDS_BOOKMARK_EDITOR_NEW_FOLDER_NAME 35397
#define IDS_BOOKMARK_BUBBLE_REMOVE_BOOKMARK 35398
#define IDS_BOOKMARK_MANAGER_NAME_INPUT_PLACE_HOLDER 35399
#define IDS_BOOKMARK_MANAGER_URL_INPUT_PLACE_HOLDER 35400
#define IDS_TOOLTIP_STAR 35401
#define IDS_CLEAR_BROWSING_DATA_CALCULATING 35402
#define IDS_DEL_BROWSING_HISTORY_COUNTER 35403
#define IDS_DEL_BROWSING_HISTORY_COUNTER_SYNCED 35404
#define IDS_DEL_NO_BROWSING_HISTORY_TEXT 35405
#define IDS_DEL_NO_BROWSING_HISTORY_SYNC_TEXT 35406
#define IDS_DEL_BROWSING_HISTORY_DOMAIN_COUNT_TEXT 35407
#define IDS_DEL_BROWSING_HISTORY_DOMAIN_COUNT_SYNC_TEXT 35408
#define IDS_DEL_BROWSING_HISTORY_COUNTER_MULTIPLE_DOMAINS_TEXT 35409
#define IDS_DEL_BROWSING_HISTORY_COUNTER_SINGLE_DOMAIN_TEXT 35410
#define IDS_DEL_BROWSING_HISTORY_COUNTER_SINGLE_DOMAIN_SYNC_TEXT 35411
#define IDS_DEL_CACHE_COUNTER_UPPER_ESTIMATE 35412
#define IDS_DEL_CACHE_COUNTER_ALMOST_EMPTY 35413
#define IDS_DEL_CACHE_COUNTER_BASIC 35414
#define IDS_DEL_CACHE_COUNTER_UPPER_ESTIMATE_BASIC 35415
#define IDS_DEL_CACHE_COUNTER_ALMOST_EMPTY_BASIC 35416
#define IDS_ANDROID_DEL_CACHE_COUNTER_ADVANCED 35417
#define IDS_ANDROID_DEL_CACHE_COUNTER_ADVANCED_UPPER_ESTIMATE 35418
#define IDS_ANDROID_DEL_CACHE_COUNTER_ADVANCED_ALMOST_EMPTY 35419
#define IDS_DEL_PASSWORDS_COUNTER 35420
#define IDS_DEL_PASSWORDS_COUNTER_SYNCED 35421
#define IDS_DEL_ACCOUNT_PASSWORDS_COUNTER 35422
#define IDS_DEL_PASSWORDS_DOMAINS_DISPLAY 35423
#define IDS_DEL_PASSWORDS_COUNTER_AND_X_MORE 35424
#define IDS_DEL_SIGNIN_DATA_COUNTER 35425
#define IDS_DEL_PASSWORDS_AND_SIGNIN_DATA_COUNTER_NONE 35426
#define IDS_DEL_PASSWORDS_AND_SIGNIN_DATA_COUNTER_COMBINATION 35427
#define IDS_DEL_SITE_SETTINGS_COUNTER 35428
#define IDS_DEL_AUTOFILL_COUNTER_EMPTY 35429
#define IDS_DEL_AUTOFILL_COUNTER_PAYMENT_METHODS 35430
#define IDS_DEL_AUTOFILL_COUNTER_ADDRESSES 35431
#define IDS_DEL_AUTOFILL_COUNTER_SUGGESTIONS 35432
#define IDS_DEL_AUTOFILL_COUNTER_USER_ANNOTATION_ENTRIES 35433
#define IDS_DEL_AUTOFILL_COUNTER_SUGGESTIONS_LONG 35434
#define IDS_DEL_AUTOFILL_COUNTER_SUGGESTIONS_SHORT 35435
#define IDS_DEL_AUTOFILL_COUNTER_ONE_TYPE_SYNCED 35436
#define IDS_DEL_AUTOFILL_COUNTER_TWO_TYPES 35437
#define IDS_DEL_AUTOFILL_COUNTER_TWO_TYPES_SYNCED 35438
#define IDS_DEL_AUTOFILL_COUNTER_THREE_TYPES 35439
#define IDS_DEL_AUTOFILL_COUNTER_THREE_TYPES_SYNCED 35440
#define IDS_DEL_AUTOFILL_SYNCABLE_NON_SYNCABLE_COMBINATION 35441
#define IDS_DEL_COOKIES_COUNTER 35442
#define IDS_DEL_COOKIES_COUNTER_ADVANCED 35443
#define IDS_DEL_COOKIES_COUNTER_ADVANCED_WITH_SIGNED_IN_EXCEPTION 35444
#define IDS_ANDROID_DEL_COOKIES_COUNTER_ADVANCED 35445
#define IDS_DEL_DOWNLOADS_COUNTER 35446
#define IDS_DEL_HOSTED_APPS_COUNTER 35447
#define IDS_DEL_HOSTED_APPS_COUNTER_AND_X_MORE 35448
#define IDS_DEL_TABS_COUNTER 35449
#define IDS_TABS_COUNT 35450
#define IDS_WINDOWS_COUNT 35451
#define IDS_DEL_TABS_MULTIWINDOW_COUNTER 35452
#define IDS_DEL_GOOGLE_COOKIES_SIGNOUT_LINK 35453
#define IDS_SETTINGS_MANAGE_IN_GOOGLE_PASSWORD_MANAGER 35454
#define IDS_SETTINGS_MANAGE_IN_YOUR_GOOGLE_ACCOUNT 35455
#define IDS_SETTINGS_MANAGE_OTHER_DATA_LABEL 35456
#define IDS_SETTINGS_MANAGE_OTHER_GOOGLE_DATA_LABEL 35457
#define IDS_SETTINGS_MY_ACTIVITY 35458
#define IDS_SETTINGS_OTHER_DATA_TITLE 35459
#define IDS_SETTINGS_OTHER_GOOGLE_DATA_TITLE 35460
#define IDS_SETTINGS_PASSWORDS_AND_PASSKEYS 35461
#define IDS_SETTINGS_MANAGE_PASSWORDS_SUB_LABEL 35462
#define IDS_SETTINGS_SEARCH_HISTORY 35463
#define IDS_SETTINGS_MANAGE_OTHER_DATA_SUB_LABEL 35464
#define IDS_SETTINGS_OTHER_DATA_DESCRIPTION 35465
#define IDS_SETTINGS_CLEAR_NON_GOOGLE_SEARCH_HISTORY_NON_PREPOPULATED_DSE 35466
#define IDS_SETTINGS_CLEAR_NON_GOOGLE_SEARCH_HISTORY_PREPOPULATED_DSE 35467
#define IDS_OMNIBOX_TRACK_PRICE_DIALOG_TITLE 35468
#define IDS_OMNIBOX_TRACK_PRICE_DIALOG_ACTION_BUTTON 35469
#define IDS_OMNIBOX_TRACK_PRICE_DIALOG_ERROR_DESCRIPTION 35470
#define IDS_OMNIBOX_TRACK_PRICE_DIALOG_TITLE_FIRST_RUN 35471
#define IDS_OMNIBOX_TRACK_PRICE_DIALOG_DESCRIPTION_FIRST_RUN 35472
#define IDS_OMNIBOX_TRACK_PRICE_DIALOG_CANCEL_BUTTON 35473
#define IDS_PRICE_TRACKING_SAVE_DESCRIPTION 35474
#define IDS_PRICE_TRACKING_SAVE_LOCATION 35475
#define IDS_PRICE_TRACKING_EMAIL_CONSENT_BODY 35476
#define IDS_PRICE_TRACKING_EMAIL_CONSENT_TITLE 35477
#define IDS_PRICE_TRACKING_EMAIL_CONSENT_LEARN_MORE_LINK_TEXT 35478
#define IDS_PRICE_TRACKING_NOT_NOW 35479
#define IDS_PRICE_TRACKING_YES_IM_IN 35480
#define IDS_OMNIBOX_TRACKING_PRICE_DIALOG_TITLE 35481
#define IDS_OMNIBOX_TRACKING_PRICE_DIALOG_ACTION_BUTTON 35482
#define IDS_OMNIBOX_TRACKING_PRICE_DIALOG_UNTRACK_BUTTON 35483
#define IDS_BOOKMARK_STAR_DIALOG_TRACK_PRICE_DESCRIPTION 35484
#define IDS_BOOKMARK_STAR_DIALOG_TRACK_PRICE_DESCRIPTION_EMAIL_OFF 35485
#define IDS_PRICE_TRACKING_SIDE_PANEL_IPH 35486
#define IDS_SIDE_PANEL_TRACKED_PRODUCTS 35487
#define IDS_SIDE_PANEL_UNTRACK_BUTTON 35488
#define IDS_SIDE_PANEL_TRACK_BUTTON 35489
#define IDS_PRICE_TRACKING_UNTRACK_PRODUCT_ACCESSIBILITY 35490
#define IDS_PRICE_TRACKING_TRACK_PRODUCT_ACCESSIBILITY 35491
#define IDS_PRICE_TRACKING_SIDE_PANEL_ERROR_BUTTON 35492
#define IDS_PRICE_TRACKING_SETTINGS_TITLE 35493
#define IDS_PRICE_TRACKING_SETTINGS_EMAIL_DESCRIPTION 35494
#define IDS_PRICE_TRACKING_SIDE_PANEL_ERROR_MESSAGE 35495
#define IDS_SHOPPING_COLLECTION_IPH_TITLE 35496
#define IDS_SHOPPING_COLLECTION_IPH_BODY 35497
#define IDS_SHOPPING_INSIGHTS_SIDE_PANEL_TITLE 35498
#define IDS_PRICE_HISTORY_TITLE 35499
#define IDS_PRICE_HISTORY_TITLE_MULTIPLE_OPTIONS 35500
#define IDS_PRICE_HISTORY_MULTIPLE_OPTIONS_LOW_PRICE 35501
#define IDS_PRICE_HISTORY_MULTIPLE_OPTIONS_HIGH_PRICE 35502
#define IDS_PRICE_HISTORY_MULTIPLE_OPTIONS_TYPICAL_PRICE 35503
#define IDS_SHOPPING_INSIGHTS_SIDE_PANEL_TRACK_PRICE_TITLE 35504
#define IDS_SHOPPING_INSIGHTS_ICON_TOOLTIP_TEXT 35505
#define IDS_SHOPPING_INSIGHTS_BUYING_OPTIONS 35506
#define IDS_PRICE_HISTORY_YESTERDAY_PRICE 35507
#define IDS_PRICE_HISTORY_DESCRIPTION 35508
#define IDS_PRICE_HISTORY_GRAPH_ACCESSIBILITY 35509
#define IDS_SHOPPING_INSIGHTS_COLLECT_FEEDBACK 35510
#define IDS_SHOPPING_INSIGHTS_FEEDBACK_FORM_TITLE 35511
#define IDS_PRICE_HISTORY_SINGLE_OPTION_LOW_PRICE 35512
#define IDS_PRICE_HISTORY_SINGLE_OPTION_HIGH_PRICE 35513
#define IDS_PRICE_HISTORY_SINGLE_OPTION_TYPICAL_PRICE 35514
#define IDS_SHOPPING_INSIGHTS_SIDE_PANEL_TRACK_PRICE_DESCRIPTION 35515
#define IDS_SHOPPING_INSIGHTS_SIDE_PANEL_TRACK_PRICE_ERROR 35516
#define IDS_DISCOUNT_USE_THIS_CODE_AT_CHECKOUT_WITH_EXPIRATION_DATE 35517
#define IDS_DISCOUNT_USE_THIS_CODE_AT_CHECKOUT 35518
#define IDS_TWO_STRINGS_CONNECTOR_WITH_SPACE 35519
#define IDS_SELLER_TERMS_AND_CONDITIONS_DIALOG_TITLE 35520
#define IDS_SELLER_TERMS_AND_CONDITIONS_DIALOG_FOOTER 35521
#define IDS_SEE_SELLER_TERMS_AND_CONDITIONS 35522
#define IDS_DISCOUNTS_COUPON_CODE_BUTTON_TOOLTIP 35523
#define IDS_DISCOUNTS_COUPON_CODE_BUTTON_TOOLTIP_CLICKED 35524
#define IDS_PRICE_DROP_POPUP_CONTENT_TITLE 35535
#define IDS_PRICE_DROP_POPUP_CONTENT_TEXT 35536
#define IDS_PRICE_DROP_POPUP_ACTION_BUTTON 35537
#define IDS_PRICE_DROP_POPUP_UNTRACK_BUTTON 35538
#define IDS_PRICE_RANGE_ALL_OPTIONS 35539
#define IDS_PRICE_RANGE_ALL_OPTIONS_ONE_TYPICAL_PRICE 35540
#define IDS_PRICE_RANGE_SINGLE_OPTION 35541
#define IDS_PRICE_RANGE_SINGLE_OPTION_ONE_TYPICAL_PRICE 35542
#define IDS_PRICE_TRACKING_DESCRIPTION 35543
#define IDS_PRICE_INSIGHTS_BUYING_OPTIONS_TITLE 35544
#define IDS_PRICE_INSIGHTS_BUYING_OPTIONS_SUBTITLE 35545
#define IDS_PRICE_INSIGHTS_TRACK_BUTTON_TITLE 35546
#define IDS_PRICE_INSIGHTS_TRACKING_BUTTON_TITLE 35547
#define IDS_INSIGHTS_ICON_EXPANDED_TEXT_GOOD_DEAL 35548
#define IDS_INSIGHTS_RICH_IPH_TEXT 35549
#define IDS_PRICE_INSIGHTS_ACCESSIBILITY 35550
#define IDS_PRICE_INSIGHTS_GRAPH_ACCESSIBILITY_LABEL 35551
#define IDS_PRICE_INSIGHTS_SNACKBAR_BUTTON_TITLE 35552
#define IDS_PRICE_INSIGHTS_SNACKBAR_MESSAGE_TITLE_NOTIFICATION_DISABLED 35553
#define IDS_PRICE_INSIGHTS_SNACKBAR_MESSAGE_TITLE_NOTIFICATION_ENABLED 35554
#define IDS_BUYING_OPTIONS_ACCESSIBILITY_DESCRIPTION 35555
#define IDS_PRICE_TRACKING_DESCRIPTION_TRACKED 35556
#define IDS_PRICE_HISTORY_TITLE_WITH_VARIANTS 35557
#define IDS_IOS_PRICE_INSIGHTS_PRICE_TRACK_PERMISSION_REDIRECT_ALERT_TITLE 35558
#define IDS_IOS_PRICE_INSIGHTS_PRICE_TRACK_PERMISSION_REDIRECT_ALERT_MESSAGE 35559
#define IDS_IOS_PRICE_INSIGHTS_PRICE_TRACK_PERMISSION_REDIRECT_ALERT_CLOSE 35560
#define IDS_IOS_PRICE_INSIGHTS_PRICE_TRACK_PERMISSION_REDIRECT_ALERT_REDIRECT 35561
#define IDS_PRICE_INSIGHTS_UNTRACK_SNACKBAR_MESSAGE 35562
#define IDS_PRICE_INSIGHTS_UNTRACK_SNACKBAR_BUTTON_TITLE 35563
#define IDS_PRICE_HISTORY_TITLE_SINGLE_OPTION 35564
#define IDS_SHOPPING_INSIGHTS_ICON_EXPANDED_TEXT_LOW_PRICE 35565
#define IDS_SHOPPING_INSIGHTS_ICON_EXPANDED_TEXT_HIGH_PRICE 35566
#define IDS_SHOPPING_COLLECTION_FOLDER_NAME 35567
#define IDS_DISCOUNT_CODE_COPY_BUTTON_TEXT 35568
#define IDS_DISCOUNT_ICON_EXPANDED_TEXT 35569
#define IDS_SETTINGS_TITLE 35578
#define IDS_SETTINGS_HIDE_ADVANCED_SETTINGS 35579
#define IDS_SETTINGS_SHOW_ADVANCED_SETTINGS 35580
#define IDS_NETWORK_PREDICTION_ENABLED_DESCRIPTION 35581
#define IDS_SYSTEM_TITLE 35582
#define IDS_OPTIONS_PROXIES_CONFIGURE_BUTTON 35583
#define IDS_COMPOSE_SUGGESTION_MAIN_TEXT 35584
#define IDS_COMPOSE_CONTEXT_MENU_TEXT 35585
#define IDS_COMPOSE_SUGGESTION_LABEL 35586
#define IDS_COMPOSE_SUGGESTION_SAVED_TEXT 35587
#define IDS_COMPOSE_SUGGESTION_SAVED_LABEL 35588
#define IDS_COMPOSE_SUGGESTION_AX_MESSAGE_ON_SHOW_PROACTIVE 35589
#define IDS_COMPOSE_SUGGESTION_AX_MESSAGE_ON_SHOW_RESUME 35590
#define IDS_COMPOSE_DONT_SHOW_ON_THIS_SITE_CHILD_SUGGESTION_TEXT 35591
#define IDS_COMPOSE_DISABLE_HELP_ME_WRITE_CHILD_SUGGESTION_TEXT 35592
#define IDS_COMPOSE_GO_TO_SETTINGS_CHILD_SUGGESTION_TEXT 35593
#define IDS_COMPOSE_FRE_TITLE 35594
#define IDS_COMPOSE_FRE_MAIN_TOP 35595
#define IDS_COMPOSE_FRE_MAIN_MID 35596
#define IDS_COMPOSE_FRE_MAIN_MID_ENTERPRISE 35597
#define IDS_COMPOSE_FRE_OK_BUTTON 35598
#define IDS_COMPOSE_FRE_MSBB_TITLE 35599
#define IDS_COMPOSE_FRE_MSBB_MAIN 35600
#define IDS_COMPOSE_FRE_MSBB_SETTINGS_BUTTON 35601
#define IDS_COMPOSE_MSBB_IPH_BUBBLE_TEXT 35602
#define IDS_COMPOSE_MSBB_IPH_BUBBLE_TEXT_SCREENREADER 35603
#define IDS_COMPOSE_MSBB_IPH_BUBBLE_CLOSE_BUTTON_LABEL_TEXT 35604
#define IDS_COMPOSE_DIALOG_TITLE 35605
#define IDS_COMPOSE_INPUT_PLACEHOLDER_TITLE 35606
#define IDS_COMPOSE_INPUT_PLACEHOLDER_LINE_1 35607
#define IDS_COMPOSE_INPUT_PLACEHOLDER_LINE_2 35608
#define IDS_COMPOSE_INPUT_PLACEHOLDER_LINE_3 35609
#define IDS_COMPOSE_INPUT_FOOTER 35610
#define IDS_COMPOSE_INPUT_FOOTER_ENTERPRISE 35611
#define IDS_COMPOSE_INPUT_MODE_POLISH 35612
#define IDS_COMPOSE_INPUT_MODE_ELABORATE 35613
#define IDS_COMPOSE_INPUT_MODE_FORMALIZE 35614
#define IDS_COMPOSE_SUBMIT_BUTTON 35615
#define IDS_COMPOSE_EDIT_UPDATE_BUTTON 35616
#define IDS_COMPOSE_EDIT 35617
#define IDS_COMPOSE_MODIFIERS_MENU_TITLE 35618
#define IDS_COMPOSE_MODIFIERS_MENU_LABEL 35619
#define IDS_COMPOSE_MENU_RETRY_OPTION 35620
#define IDS_COMPOSE_MENU_LENGTH_TITLE 35621
#define IDS_COMPOSE_MENU_SHORTER_OPTION 35622
#define IDS_COMPOSE_MENU_LONGER_OPTION 35623
#define IDS_COMPOSE_MENU_TONE_TITLE 35624
#define IDS_COMPOSE_MENU_CASUAL_OPTION 35625
#define IDS_COMPOSE_MENU_FORMAL_OPTION 35626
#define IDS_COMPOSE_UNDO_BUTTON_TEXT 35627
#define IDS_COMPOSE_UNDO_LABEL 35628
#define IDS_COMPOSE_REDO_BUTTON_TEXT 35629
#define IDS_COMPOSE_REDO_LABEL 35630
#define IDS_COMPOSE_RESUBMIT 35631
#define IDS_COMPOSE_INSERT_BUTTON 35632
#define IDS_COMPOSE_REPLACE_BUTTON 35633
#define IDS_COMPOSE_THUMBS_DOWN 35634
#define IDS_COMPOSE_THUMBS_UP 35635
#define IDS_COMPOSE_RESULT_TEXT_LABEL 35636
#define IDS_COMPOSE_RESULT_LOADING_A11Y_MESSAGE 35637
#define IDS_COMPOSE_RESULT_UPDATED_A11Y_MESSAGE 35638
#define IDS_COMPOSE_UNDO_RESULT_A11Y_MESSAGE 35639
#define IDS_COMPOSE_REDO_RESULT_A11Y_MESSAGE 35640
#define IDS_COMPOSE_ERROR_TOO_SHORT 35641
#define IDS_COMPOSE_ERROR_TOO_LONG 35642
#define IDS_COMPOSE_ERROR_TRY_AGAIN 35643
#define IDS_COMPOSE_ERROR_TRY_AGAIN_LATER 35644
#define IDS_COMPOSE_ERROR_FILTERED 35645
#define IDS_COMPOSE_ERROR_FILTERED_BACK_BUTTON 35646
#define IDS_COMPOSE_ERROR_UNSUPPORTED_LANGUAGE 35647
#define IDS_COMPOSE_ERROR_PERMISSION_DENIED 35648
#define IDS_COMPOSE_ERROR_REQUEST_THROTTLED 35649
#define IDS_COMPOSE_ERROR_OFFLINE 35650
#define IDS_COMPOSE_FOOTER_FISHFOOD 35651
#define IDS_COMPOSE_EXPERIMENTAL_DISCLAIMER_FOOTER 35652
#define IDS_COMPOSE_REFINEMENTS_EXPERIMENTAL_DISCLAIMER_FOOTER 35653
#define IDS_COMPOSE_FOOTER_FISHFOOD_ON_DEVICE_USED 35654
#define IDS_COMPOSE_FEEDBACK_PLACEHOLDER 35655
#define IDS_CRASH_TITLE 35656
#define IDS_CRASH_CRASH_COUNT_BANNER_FORMAT 35657
#define IDS_CRASH_SHOW_DEVELOPER_DETAILS 35658
#define IDS_CRASH_CAPTURE_TIME_FORMAT 35659
#define IDS_CRASH_REPORT_STATUS 35660
#define IDS_CRASH_REPORT_STATUS_NOT_UPLOADED 35661
#define IDS_CRASH_REPORT_STATUS_PENDING 35662
#define IDS_CRASH_REPORT_STATUS_PENDING_USER_REQUESTED 35663
#define IDS_CRASH_REPORT_STATUS_UPLOADED 35664
#define IDS_CRASH_REPORT_UPLOADED_ID 35665
#define IDS_CRASH_REPORT_UPLOADED_TIME 35666
#define IDS_CRASH_REPORT_LOCAL_ID 35667
#define IDS_CRASH_REPORT_FILE_SIZE 35668
#define IDS_CRASH_BUG_LINK_LABEL 35669
#define IDS_CRASH_NO_CRASHES_MESSAGE 35670
#define IDS_CRASH_DISABLED_HEADER 35671
#define IDS_CRASH_UPLOAD_MESSAGE 35672
#define IDS_CRASH_UPLOAD_NOW_LINK_TEXT 35673
#define IDS_HTTP_POST_WARNING_TITLE 35674
#define IDS_HTTP_POST_WARNING 35675
#define IDS_HTTP_POST_WARNING_RESEND 35676
#define IDS_DOM_DISTILLER_JAVASCRIPT_DISABLED_CONTENT 35677
#define IDS_DOM_DISTILLER_WEBUI_ENTRY_URL 35678
#define IDS_DOM_DISTILLER_WEBUI_ENTRY_ADD 35679
#define IDS_DOM_DISTILLER_WEBUI_ENTRY_ADD_FAILED 35680
#define IDS_DOM_DISTILLER_WEBUI_VIEW_URL 35681
#define IDS_DOM_DISTILLER_WEBUI_VIEW_URL_FAILED 35682
#define IDS_DOM_DISTILLER_WEBUI_REFRESH 35683
#define IDS_DOM_DISTILLER_WEBUI_FETCHING_ENTRIES 35684
#define IDS_DOM_DISTILLER_VIEWER_FAILED_TO_FIND_ARTICLE_TITLE 35685
#define IDS_DOM_DISTILLER_VIEWER_FAILED_TO_FIND_ARTICLE_CONTENT 35686
#define IDS_DOM_DISTILLER_VIEWER_LOADING_TITLE 35687
#define IDS_DOM_DISTILLER_VIEWER_CUSTOMIZE_APPEARANCE 35688
#define IDS_DOM_DISTILLER_VIEWER_FONT_STYLE 35689
#define IDS_DOM_DISTILLER_VIEWER_SANS_SERIF_FONT 35690
#define IDS_DOM_DISTILLER_VIEWER_SERIF_FONT 35691
#define IDS_DOM_DISTILLER_VIEWER_MONOSPACE_FONT 35692
#define IDS_DOM_DISTILLER_VIEWER_PAGE_COLOR 35693
#define IDS_DOM_DISTILLER_VIEWER_PAGE_COLOR_LIGHT 35694
#define IDS_DOM_DISTILLER_VIEWER_PAGE_COLOR_SEPIA 35695
#define IDS_DOM_DISTILLER_VIEWER_PAGE_COLOR_DARK 35696
#define IDS_DOM_DISTILLER_VIEWER_FONT_SIZE 35697
#define IDS_DOM_DISTILLER_VIEWER_FONT_SIZE_SMALL 35698
#define IDS_DOM_DISTILLER_VIEWER_FONT_SIZE_LARGE 35699
#define IDS_DOM_DISTILLER_VIEWER_CLOSE 35700
#define IDS_DOM_DISTILLER_VIEWER_NO_DATA_CONTENT 35701
#define IDS_DOM_DISTILLER_WEBUI_TITLE 35702
#define IDS_DOM_DISTILLER_VIEWER_TITLE_SUFFIX 35703
#define IDS_ENTERPRISE_COPY_PREVENTION_MISSING_LIST_ERROR 35704
#define IDS_ENTERPRISE_COPY_PREVENTION_DISABLE_CONTAINS_WILDCARD_ERROR 35705
#define IDS_ENTERPRISE_COPY_PREVENTION_WARNING_MESSAGE 35706
#define IDS_ENTERPRISE_DATA_CONTROLS_COPY_PREVENTION_WARNING_MESSAGE 35707
#define IDS_DATA_CONTROLS_CLIPBOARD_PASTE_BLOCK_TITLE 35708
#define IDS_DATA_CONTROLS_CLIPBOARD_PASTE_WARN_TITLE 35709
#define IDS_DATA_CONTROLS_CLIPBOARD_COPY_BLOCK_TITLE 35710
#define IDS_DATA_CONTROLS_CLIPBOARD_COPY_WARN_TITLE 35711
#define IDS_DATA_CONTROLS_CLIPBOARD_DRAG_BLOCK_TITLE 35712
#define IDS_DATA_CONTROLS_CLIPBOARD_SHARE_WARN_TITLE 35713
#define IDS_DATA_CONTROLS_CLIPBOARD_ACTION_WARN_TITLE 35714
#define IDS_DATA_CONTROLS_BLOCKED_LABEL 35715
#define IDS_DATA_CONTROLS_WARNED_LABEL 35716
#define IDS_DATA_CONTROLS_WARNED_LABEL_WITH_DOMAIN 35717
#define IDS_DATA_CONTROLS_PASTE_WARN_CONTINUE_BUTTON 35718
#define IDS_DATA_CONTROLS_COPY_WARN_CONTINUE_BUTTON 35719
#define IDS_DATA_CONTROLS_SHARE_WARN_CONTINUE_BUTTON 35720
#define IDS_DATA_CONTROLS_PASTE_WARN_CANCEL_BUTTON 35721
#define IDS_DATA_CONTROLS_COPY_WARN_CANCEL_BUTTON 35722
#define IDS_DATA_CONTROLS_SHARE_WARN_CANCEL_BUTTON 35723
#define IDS_DATA_CONTROLS_BLOCKED_LABEL_WITH_DOMAIN 35724
#define IDS_ERRORPAGE_NET_BUTTON_DETAILS 35725
#define IDS_ERRORPAGE_NET_BUTTON_HIDE_DETAILS 35726
#define IDS_ERRORPAGES_BUTTON_RELOAD 35727
#define IDS_ERRORPAGES_BUTTON_SHOW_SAVED_COPY 35728
#define IDS_ERRORPAGE_FUN_DISABLED 35729
#define IDS_ERRORPAGES_SUGGESTION_RELOAD_REPOST_SUMMARY 35734
#define IDS_ERRORPAGES_SUGGESTION_CHECK_CONNECTION_HEADER 35735
#define IDS_ERRORPAGES_SUGGESTION_CHECK_CONNECTION_BODY 35736
#define IDS_ERRORPAGES_SUGGESTION_SECURE_DNS_CONFIG_HEADER 35737
#define IDS_ERRORPAGES_SUGGESTION_SECURE_DNS_CONFIG_BODY 35738
#define IDS_ERRORPAGES_SUGGESTION_DNS_CONFIG_HEADER 35739
#define IDS_ERRORPAGES_SUGGESTION_DNS_CONFIG_BODY 35740
#define IDS_ERRORPAGES_SUGGESTION_NETWORK_PREDICTION_HEADER 35741
#define IDS_ERRORPAGES_SUGGESTION_FIREWALL_CONFIG_BODY 35742
#define IDS_ERRORPAGES_SUGGESTION_PROXY_CONFIG_HEADER 35743
#define IDS_ERRORPAGES_SUGGESTION_PROXY_CONFIG_BODY 35744
#define IDS_ERRORPAGES_SUGGESTION_CAPTIVE_PORTAL_SIGNIN 35745
#define IDS_ERRORPAGES_SUGGESTION_VIEW_POLICIES_HEADER 35747
#define IDS_ERRORPAGES_SUGGESTION_VIEW_POLICIES_BODY 35748
#define IDS_ERRORPAGES_SUGGESTION_UNSUPPORTED_CIPHER_HEADER 35749
#define IDS_ERRORPAGES_SUGGESTION_UNSUPPORTED_CIPHER_BODY 35750
#define IDS_ERRORPAGES_SUGGESTION_NAVIGATE_TO_ORIGIN 35751
#define IDS_ERRORPAGES_HEADING_NOT_AVAILABLE 35752
#define IDS_ERRORPAGES_HEADING_NETWORK_ACCESS_DENIED 35753
#define IDS_ERRORPAGES_HEADING_INTERNET_DISCONNECTED 35754
#define IDS_ERRORPAGES_HEADING_YOU_ARE_OFFLINE 35755
#define IDS_ERRORPAGES_MESSAGE_IWA_INVALID_WEB_BUNDLE 35756
#define IDS_ERRORPAGES_MESSAGE_IWA_CONNECTION_REFUSED 35757
#define IDS_ERRORPAGES_MESSAGE_IWA_INSTALLED_IN_DEV_MODE 35758
#define IDS_ERRORPAGES_MESSAGE_IWA_PAGE_NOT_FOUND 35759
#define IDS_ERRORPAGES_HEADING_CACHE_READ_FAILURE 35760
#define IDS_ERRORPAGES_HEADING_CONNECTION_INTERRUPTED 35761
#define IDS_ERRORPAGES_HEADING_NOT_FOUND 35762
#define IDS_ERRORPAGES_HEADING_FILE_NOT_FOUND 35763
#define IDS_ERRORPAGES_HEADING_BLOCKED 35764
#define IDS_ERRORPAGES_HEADING_BLOCKED_BY_ADMINISTRATOR 35765
#define IDS_ERRORPAGES_SUMMARY_NOT_AVAILABLE 35766
#define IDS_ERRORPAGES_SUMMARY_TIMED_OUT 35767
#define IDS_ERRORPAGES_SUMMARY_CONNECTION_RESET 35768
#define IDS_ERRORPAGES_SUMMARY_CONNECTION_CLOSED 35769
#define IDS_ERRORPAGES_SUMMARY_CONNECTION_FAILED 35770
#define IDS_ERRORPAGES_SUMMARY_NETWORK_CHANGED 35771
#define IDS_ERRORPAGES_SUMMARY_CONNECTION_REFUSED 35772
#define IDS_ERRORPAGES_SUMMARY_NAME_NOT_RESOLVED 35773
#define IDS_ERRORPAGES_SUMMARY_ICANN_NAME_COLLISION 35774
#define IDS_ERRORPAGES_SUMMARY_ADDRESS_UNREACHABLE 35775
#define IDS_ERRORPAGES_SUMMARY_FILE_ACCESS_DENIED 35776
#define IDS_ERRORPAGES_SUMMARY_NETWORK_ACCESS_DENIED 35777
#define IDS_ERRORPAGES_SUMMARY_PROXY_CONNECTION_FAILED 35778
#define IDS_ERRORPAGES_SUMMARY_CACHE_READ_FAILURE 35779
#define IDS_ERRORPAGES_SUMMARY_NETWORK_IO_SUSPENDED 35780
#define IDS_ERRORPAGES_SUMMARY_NOT_FOUND 35781
#define IDS_ERRORPAGES_SUMMARY_FILE_NOT_FOUND 35782
#define IDS_ERRORPAGES_SUMMARY_TOO_MANY_REDIRECTS 35783
#define IDS_ERRORPAGES_SUMMARY_EMPTY_RESPONSE 35784
#define IDS_ERRORPAGES_SUMMARY_INVALID_RESPONSE 35785
#define IDS_ERRORPAGES_SUMMARY_DNS_PROBE_RUNNING 35786
#define IDS_ERRORPAGES_HEADING_ACCESS_DENIED 35787
#define IDS_ERRORPAGES_HEADING_FILE_ACCESS_DENIED 35788
#define IDS_ERRORPAGES_SUMMARY_FORBIDDEN 35789
#define IDS_ERRORPAGES_SUMMARY_GONE 35790
#define IDS_ERRORPAGES_HEADING_PAGE_NOT_WORKING 35791
#define IDS_ERRORPAGES_SUMMARY_CONTACT_SITE_OWNER 35792
#define IDS_ERRORPAGES_SUMMARY_WEBSITE_CANNOT_HANDLE_REQUEST 35793
#define IDS_ERRORPAGES_SUMMARY_GATEWAY_TIMEOUT 35794
#define IDS_ERRORPAGES_SUMMARY_SSL_SECURITY_ERROR 35795
#define IDS_ERRORPAGES_SUMMARY_SSL_VERSION_OR_CIPHER_MISMATCH 35796
#define IDS_ERRORPAGES_HEADING_INSECURE_CONNECTION 35797
#define IDS_ERRORPAGES_SUMMARY_BAD_SSL_CLIENT_AUTH_CERT 35798
#define IDS_ERRORPAGES_SUMMARY_SSL_CLIENT_AUTH_SIGNATURE_FAILED 35799
#define IDS_ERRORPAGES_SUMMARY_BLOCKED_BY_EXTENSION 35800
#define IDS_ERRORPAGES_SUMMARY_BLOCKED_BY_ADMINISTRATOR 35801
#define IDS_ERRORPAGES_SUMMARY_BLOCKED_BY_SECURITY 35802
#define IDS_ERRORPAGES_SUMMARY_BLOCKED_BY_LOCAL_NETWORK_ACCESS_CHECKS 35803
#define IDS_ERRORPAGES_HTTP_POST_WARNING 35804
#define IDS_ERRORPAGES_SUGGESTION_LIST_HEADER 35805
#define IDS_ERRORPAGES_SUGGESTION_CHECK_CONNECTION_SUMMARY 35806
#define IDS_ERRORPAGES_SUGGESTION_CHECK_PROXY_FIREWALL_DNS_SUMMARY 35808
#define IDS_ERRORPAGES_SUGGESTION_CHECK_PROXY_FIREWALL_SECURE_DNS_SUMMARY 35809
#define IDS_ERRORPAGES_SUGGESTION_CHECK_FIREWALL_ANTIVIRUS_SUMMARY 35810
#define IDS_ERRORPAGES_SUGGESTION_CHECK_PROXY_FIREWALL_SUMMARY 35811
#define IDS_ERRORPAGES_SUGGESTION_CHECK_PROXY_ADDRESS_SUMMARY 35812
#define IDS_ERRORPAGES_SUGGESTION_CONTACT_ADMIN_SUMMARY 35813
#define IDS_ERRORPAGES_SUGGESTION_CONTACT_ADMIN_SUMMARY_STANDALONE 35814
#define IDS_ERRORPAGES_SUGGESTION_LEARNMORE_SUMMARY 35815
#define IDS_ERRORPAGES_SUGGESTION_LEARNMORE_SUMMARY_STANDALONE 35816
#define IDS_ERRORPAGES_SUGGESTION_DELETE_COOKIES_SUMMARY 35817
#define IDS_ERRORPAGES_SUGGESTION_CHECK_HARDWARE_SUMMARY 35821
#define IDS_ERRORPAGES_SUGGESTION_CHECK_WIFI_SUMMARY 35822
#define IDS_ERRORPAGES_SUGGESTION_DISABLE_EXTENSION_SUMMARY 35823
#define IDS_ERRORPAGES_CHECK_TYPO_SUMMARY 35824
#define IDS_ERRORPAGES_GAME_INSTRUCTIONS 35828
#define IDS_ERRORPAGE_DINO_GAME_DESCRIPTION 35829
#define IDS_ERRORPAGE_DINO_ARIA_LABEL 35830
#define IDS_ERRORPAGE_DINO_GAME_START 35831
#define IDS_ERRORPAGE_DINO_GAME_OVER 35832
#define IDS_ERRORPAGE_DINO_HIGH_SCORE 35833
#define IDS_ERRORPAGE_DINO_JUMP 35834
#define IDS_ERRORPAGE_DINO_SLOW_SPEED_TOGGLE 35835
#define IDS_ERRORPAGES_GAME_INSTRUCTIONS_TOUCH 35836
#define IDS_ERRORPAGES_GAME_INSTRUCTIONS_KEYBOARD 35837
#define IDS_ERRORPAGES_GAME_INSTRUCTIONS_HYBRID 35838
#define IDS_ERRORPAGE_DINO_ARIA_LABEL_TOUCH 35839
#define IDS_ERRORPAGE_DINO_ARIA_LABEL_KEYBOARD 35840
#define IDS_ERRORPAGE_DINO_ARIA_LABEL_HYBRID 35841
#define IDS_ERRORPAGES_HEADING_BLOCKED_IN_INCOGNITO_BY_ADMINISTRATOR 35842
#define IDS_ERRORPAGES_SUMMARY_BLOCKED_IN_INCOGNITO_BY_ADMINISTRATOR 35843
#define IDS_FIND_IN_PAGE_ACCESSIBLE_TITLE 35844
#define IDS_FIND_IN_PAGE_COUNT 35845
#define IDS_ACCESSIBLE_FIND_IN_PAGE_COUNT 35846
#define IDS_ACCESSIBLE_FIND_IN_PAGE_NO_RESULTS 35847
#define IDS_FIND_IN_PAGE_PREVIOUS_TOOLTIP 35848
#define IDS_FIND_IN_PAGE_NEXT_TOOLTIP 35849
#define IDS_FIND_IN_PAGE_CLOSE_TOOLTIP 35850
#define IDS_FLAGS_UI_SEARCH_PLACEHOLDER 35851
#define IDS_FLAGS_UI_SEARCH_LABEL 35852
#define IDS_FLAGS_UI_TITLE 35853
#define IDS_FLAGS_UI_PAGE_RESET 35854
#define IDS_FLAGS_UI_PAGE_WARNING 35855
#define IDS_FLAGS_UI_PAGE_WARNING_EXPLANATION 35856
#define IDS_FLAGS_UI_OWNER_WARNING 35857
#define IDS_FLAGS_UI_AVAILABLE_FEATURE 35858
#define IDS_FLAGS_UI_UNAVAILABLE_FEATURE 35859
#define IDS_FLAGS_UI_ENABLED_FEATURE 35860
#define IDS_FLAGS_UI_DISABLED_FEATURE 35861
#define IDS_FLAGS_UI_NO_RESULTS 35862
#define IDS_FLAGS_UI_NOT_AVAILABLE_ON_PLATFORM 35863
#define IDS_FLAGS_UI_RELAUNCH 35864
#define IDS_FLAGS_UI_CLEAR_SEARCH 35865
#define IDS_FLAGS_UI_RESET_ACKNOWLEDGED 35866
#define IDS_FLAGS_UI_EXPERIMENT_ENABLED 35867
#define IDS_FLAGS_UI_SEARCH_RESULTS_SINGULAR 35868
#define IDS_FLAGS_UI_SEARCH_RESULTS_PLURAL 35869
#define IDS_DEPRECATED_FEATURES_SEARCH_PLACEHOLDER 35870
#define IDS_DEPRECATED_FEATURES_TITLE 35871
#define IDS_DEPRECATED_FEATURES_HEADING 35872
#define IDS_DEPRECATED_FEATURES_PAGE_WARNING_EXPLANATION 35873
#define IDS_DEPRECATED_FEATURES_NO_RESULTS 35874
#define IDS_EXIT_FULLSCREEN_MODE 35875
#define IDS_FULLSCREEN_HOLD_TO_EXIT_FULLSCREEN 35876
#define IDS_FULLSCREEN_HOLD_TO_EXIT_FULLSCREEN_WITH_ORIGIN 35877
#define IDS_FULLSCREEN_PRESS_TO_EXIT_FULLSCREEN 35878
#define IDS_FULLSCREEN_PRESS_TO_EXIT_FULLSCREEN_WITH_ORIGIN 35879
#define IDS_FULLSCREEN_PRESS_TO_EXIT_FULLSCREEN_TWO_KEYS 35880
#define IDS_PRESS_TO_EXIT_MOUSELOCK 35881
#define IDS_PRESS_TO_EXIT_MOUSELOCK_WITH_ORIGIN 35882
#define IDS_PRESS_TO_EXIT_MOUSELOCK_TWO_KEYS 35883
#define IDS_FULLSCREEN_HOLD_TO_SEE_DOWNLOADS 35884
#define IDS_FULLSCREEN_PRESS_TO_SEE_DOWNLOADS 35885
#define IDS_FULLSCREEN_PRESS_TO_SEE_DOWNLOADS_AND_EXIT 35886
#define IDS_FULLSCREEN_HOLD_TO_SEE_DOWNLOADS_AND_EXIT 35887
#define IDS_FULLSCREEN_TOUCH_BASED_EXIT_FULLSCREEN_WITH_ORIGIN 35888
#define IDS_FULLSCREEN_TOUCH_BASED_EXIT_FULLSCREEN 35889
#define IDS_TOUCH_BASED_EXIT_MOUSELOCK_WITH_ORIGIN 35890
#define IDS_TOUCH_BASED_EXIT_MOUSELOCK 35891
#define IDS_FULLSCREEN_TOUCH_BASED_INSTRUCTIONS_TO_SEE_DOWNLOADS 35892
#define IDS_FULLSCREEN_TOUCH_BASED_INSTRUCTIONS_TO_SEE_DOWNLOADS_AND_EXIT 35893
#define IDS_GLOBAL_MEDIA_CONTROLS_BACK_TO_TAB 35894
#define IDS_GLOBAL_MEDIA_CONTROLS_DISMISS_ICON_TOOLTIP_TEXT 35895
#define IDS_GLOBAL_MEDIA_CONTROLS_CAST_HEADER_TEXT 35896
#define IDS_GLOBAL_MEDIA_CONTROLS_CLOSE_DEVICE_LIST_TEXT 35897
#define IDS_GLOBAL_MEDIA_CONTROLS_UNKNOWN_SOURCE_TEXT 35898
#define IDS_GLOBAL_MEDIA_CONTROLS_UNKNOWN_DEVICE_TEXT 35899
#define IDS_GLOBAL_MEDIA_CONTROLS_MEDIA_LIVE_TEXT 35900
#define IDS_HEAVY_AD_INTERVENTION_BUTTON_DETAILS 35901
#define IDS_HEAVY_AD_INTERVENTION_HEADING 35902
#define IDS_HEAVY_AD_INTERVENTION_SUMMARY 35903
#define IDS_HEAVY_AD_INTERVENTION_BUTTON_RELOAD 35904
#define IDS_HISTORY_CLUSTERS_ACTION_MENU_DESCRIPTION 35905
#define IDS_HISTORY_CLUSTERS_DELETE_PRIOR_VISITS_CONFIRM_BUTTON 35906
#define IDS_HISTORY_CLUSTERS_DELETE_PRIOR_VISITS_WARNING 35907
#define IDS_HISTORY_CLUSTERS_BY_GROUP_TAB_LABEL 35908
#define IDS_HISTORY_CLUSTERS_BY_DATE_TAB_LABEL 35909
#define IDS_HISTORY_CLUSTERS_CLUSTER_LABEL_SEARCH_TERMS 35910
#define IDS_HISTORY_CLUSTERS_CLUSTER_LABEL_MULTIPLE_HOSTNAMES 35911
#define IDS_HISTORY_CLUSTERS_CLUSTER_LABEL_OTHER_MATCHING_VISITS 35912
#define IDS_HISTORY_CLUSTERS_HIDE_PAGE 35913
#define IDS_HISTORY_CLUSTERS_HIDE_VISITS 35914
#define IDS_HISTORY_CLUSTERS_LOAD_MORE_BUTTON_LABEL 35915
#define IDS_HISTORY_CLUSTERS_NO_RESULTS 35916
#define IDS_HISTORY_CLUSTERS_NO_SEARCH_RESULTS 35917
#define IDS_HISTORY_CLUSTERS_SHOW_SIDE_PANEL 35918
#define IDS_HISTORY_CLUSTERS_OPEN_ALL_IN_TABGROUP 35919
#define IDS_HISTORY_CLUSTERS_OPEN_INCOGNITO 35920
#define IDS_HISTORY_CLUSTERS_OPEN_IN_NEW_TAB 35921
#define IDS_HISTORY_CLUSTERS_OPEN_IN_NEW_WINDOW 35922
#define IDS_HISTORY_CLUSTERS_RELATED_SEARCHES_HEADER 35923
#define IDS_HISTORY_CLUSTERS_REMOVE_ALL_ITEMS 35924
#define IDS_HISTORY_CLUSTERS_REMOVE_PAGE 35925
#define IDS_HISTORY_CLUSTERS_REMOVE_ITEM_TOAST 35926
#define IDS_HISTORY_CLUSTERS_REMOVE_SELECTED_ITEMS 35927
#define IDS_HISTORY_CLUSTERS_SAVED_IN_TABGROUP_LABEL 35928
#define IDS_HISTORY_CLUSTERS_CUT 35929
#define IDS_HISTORY_CLUSTERS_COPY 35930
#define IDS_HISTORY_CLUSTERS_PASTE 35931
#define IDS_HISTORY_CLUSTERS_COPY_LINK 35932
#define IDS_HISTORY_CLUSTERS_SHOW_LESS_BUTTON_LABEL 35933
#define IDS_HISTORY_CLUSTERS_SHOW_MORE_BUTTON_LABEL 35934
#define IDS_HISTORY_ACTION_MENU_DESCRIPTION 35935
#define IDS_HISTORY_ACTOR_TASK_TOOLTIP 35936
#define IDS_HISTORY_ARIA_ROLE_DESCRIPTION 35937
#define IDS_HISTORY_CANCEL_EDITING_BUTTON 35938
#define IDS_HISTORY_DATE_WITH_RELATIVE_TIME 35939
#define IDS_HISTORY_DELETE_PRIOR_VISITS_CONFIRM_BUTTON 35940
#define IDS_HISTORY_DELETE_PRIOR_VISITS_WARNING 35941
#define IDS_HISTORY_DELETE_SELECTED_ENTRIES_BUTTON 35942
#define IDS_HISTORY_ENTRY_ACCESSIBILITY_DELETE 35943
#define IDS_HISTORY_ENTRY_ACCESSIBILITY_LABEL 35944
#define IDS_HISTORY_ENTRY_BOOKMARKED 35945
#define IDS_HISTORY_ENTRY_SUMMARY 35946
#define IDS_HISTORY_FOUND_SEARCH_RESULTS 35947
#define IDS_HISTORY_OTHER_FORMS_OF_HISTORY_GMA_ONLY 35948
#define IDS_HISTORY_OTHER_FORMS_OF_HISTORY_GAA_ONLY 35949
#define IDS_HISTORY_OTHER_FORMS_OF_HISTORY_GMA_AND_GAA 35950
#define IDS_HISTORY_LOADING 35951
#define IDS_HISTORY_MORE_FROM_SITE 35952
#define IDS_HISTORY_NO_RESULTS 35953
#define IDS_HISTORY_NO_SEARCH_RESULTS 35954
#define IDS_HISTORY_OPEN_CLEAR_BROWSING_DATA_DIALOG 35955
#define IDS_HISTORY_OTHER_SESSIONS_COLLAPSE_SESSION 35956
#define IDS_HISTORY_OTHER_SESSIONS_EXPAND_SESSION 35957
#define IDS_HISTORY_OTHER_SESSIONS_HIDE_FOR_NOW 35958
#define IDS_HISTORY_OTHER_SESSIONS_OPEN_ALL 35959
#define IDS_HISTORY_REMOVE_BOOKMARK 35960
#define IDS_HISTORY_REMOVE_PAGE 35961
#define IDS_HISTORY_REMOVE_PAGE_SUCCESS 35962
#define IDS_HISTORY_REMOVE_SELECTED_ITEMS 35963
#define IDS_HISTORY_SEARCH_BUTTON 35964
#define IDS_HISTORY_SEARCH_RESULT 35965
#define IDS_HISTORY_SEARCH_RESULTS 35966
#define IDS_HISTORY_SEARCH_EXACT_MATCH_RESULT 35967
#define IDS_HISTORY_SEARCH_EXACT_MATCH_RESULTS 35968
#define IDS_HISTORY_SEARCH_EMBEDDINGS_MATCH_RESULT 35969
#define IDS_HISTORY_SEARCH_EMBEDDINGS_MATCH_RESULTS 35970
#define IDS_HISTORY_SHOW_HISTORY 35971
#define IDS_HISTORY_SHOW_SIDE_PANEL 35972
#define IDS_HISTORY_SHOWFULLHISTORY_LINK 35973
#define IDS_HISTORY_SOURCE_FILTER_CHIPS_ARIA_LABEL 35974
#define IDS_HISTORY_SOURCE_FILTER_CHIP_ACTOR 35975
#define IDS_HISTORY_SOURCE_FILTER_CHIP_USER 35976
#define IDS_HISTORY_CLOSE_AND_RETURN_TO_PREFIX 35977
#define IDS_HISTORY_START_EDITING_BUTTON 35978
#define IDS_HISTORY_TITLE 35979
#define IDS_HISTORY_UNKNOWN_DEVICE 35980
#define IDS_HISTORY_SYNC_PROMO_TITLE 35981
#define IDS_HISTORY_SYNC_PROMO_BODY_SIGNED_OUT 35982
#define IDS_HISTORY_SYNC_PROMO_BODY_SIGN_IN_PENDING 35983
#define IDS_HISTORY_SYNC_PROMO_BODY_SIGN_IN_PENDING_SYNC_HISTORY_ON 35984
#define IDS_HISTORY_SYNC_PROMO_BODY_SIGNED_IN 35985
#define IDS_HISTORY_EMBEDDINGS_DISCLAIMER 35986
#define IDS_HISTORY_EMBEDDINGS_DISCLAIMER_LOGGING_OFF 35987
#define IDS_HISTORY_EMBEDDINGS_PROMO_LABEL 35988
#define IDS_HISTORY_EMBEDDINGS_PROMO_CLOSE 35989
#define IDS_HISTORY_EMBEDDINGS_PROMO_HEADING 35990
#define IDS_HISTORY_EMBEDDINGS_PROMO_BODY 35991
#define IDS_HISTORY_EMBEDDINGS_ANSWERS_PROMO_HEADING 35992
#define IDS_HISTORY_EMBEDDINGS_ANSWERS_PROMO_BODY 35993
#define IDS_HISTORY_EMBEDDIGNS_PROMO_SETTINGS_LINK_TEXT 35994
#define IDS_HISTORY_EMBEDDINGS_SHOW_BY_ARIA_LABEL 35995
#define IDS_HISTORY_EMBEDDINGS_SHOW_BY_DATE 35996
#define IDS_HISTORY_EMBEDDINGS_SHOW_BY_GROUP 35997
#define IDS_HISTORY_EMBEDDINGS_SUGGESTION_1 35998
#define IDS_HISTORY_EMBEDDINGS_SUGGESTION_2 35999
#define IDS_HISTORY_EMBEDDINGS_SUGGESTION_3 36000
#define IDS_HISTORY_EMBEDDINGS_SUGGESTION_1_ARIA_LABEL 36001
#define IDS_HISTORY_EMBEDDINGS_SUGGESTION_2_ARIA_LABEL 36002
#define IDS_HISTORY_EMBEDDINGS_SUGGESTION_3_ARIA_LABEL 36003
#define IDS_HISTORY_EMBEDDINGS_HEADING 36004
#define IDS_HISTORY_EMBEDDINGS_HEADING_LOADING 36005
#define IDS_HISTORY_EMBEDDINGS_WITH_ANSWERS_RESULTS_HEADING 36006
#define IDS_HISTORY_EMBEDDINGS_FOOTER 36007
#define IDS_HISTORY_EMBEDDINGS_FEEDBACK_PLACEHOLDER 36008
#define IDS_HISTORY_EMBEDDINGS_IPH_BODY 36009
#define IDS_HISTORY_EMBEDDINGS_IPH_ACTION 36010
#define IDS_HISTORY_EMBEDDINGS_SEARCH_PROMPT 36011
#define IDS_HISTORY_EMBEDDINGS_SEARCH_ANSWERS_ALTERNATIVE_PROMPT_1 36012
#define IDS_HISTORY_EMBEDDINGS_SEARCH_ANSWERS_ALTERNATIVE_PROMPT_2 36013
#define IDS_HISTORY_EMBEDDINGS_SEARCH_ANSWERS_ALTERNATIVE_PROMPT_3 36014
#define IDS_HISTORY_EMBEDDINGS_SEARCH_ANSWERS_ALTERNATIVE_PROMPT_4 36015
#define IDS_HISTORY_EMBEDDINGS_ANSWER_HEADING 36016
#define IDS_HISTORY_EMBEDDINGS_ANSWER_LOADING_HEADING 36017
#define IDS_HISTORY_EMBEDDINGS_ANSWER_SOURCE_VISIT_DATE_LABEL 36018
#define IDS_HISTORY_EMBEDDINGS_ANSWERER_ERROR_UNANSWERABLE 36019
#define IDS_HISTORY_EMBEDDINGS_ANSWERER_ERROR_MODEL_UNAVAILABLE 36020
#define IDS_HISTORY_EMBEDDINGS_ANSWERER_ERROR_TRY_AGAIN 36021
#define IDS_COMPARE_HISTORY_EMPTY 36022
#define IDS_COMPARE_HISTORY_HEADER 36023
#define IDS_COMPARE_HISTORY_INFO 36024
#define IDS_COMPARE_HISTORY_MENU_ARIA_LABEL 36025
#define IDS_COMPARE_HISTORY_MENU_ITEM 36026
#define IDS_COMPARE_HISTORY_REMOVE 36027
#define IDS_COMPARE_HISTORY_ROW 36028
#define IDS_JAVASCRIPT_MESSAGEBOX_TITLE 36029
#define IDS_JAVASCRIPT_MESSAGEBOX_TITLE_IFRAME 36030
#define IDS_JAVASCRIPT_MESSAGEBOX_TITLE_NONSTANDARD_URL 36031
#define IDS_JAVASCRIPT_MESSAGEBOX_TITLE_NONSTANDARD_URL_IFRAME 36032
#define IDS_JAVASCRIPT_MESSAGEBOX_TITLE_EXTENSION 36033
#define IDS_JAVASCRIPT_MESSAGEBOX_SUPPRESS_OPTION 36034
#define IDS_BEFOREUNLOAD_MESSAGEBOX_TITLE 36035
#define IDS_BEFOREUNLOAD_APP_MESSAGEBOX_TITLE 36036
#define IDS_BEFOREUNLOAD_MESSAGEBOX_OK_BUTTON_LABEL 36037
#define IDS_BEFOREUNLOAD_MESSAGEBOX_MESSAGE 36038
#define IDS_BEFORERELOAD_MESSAGEBOX_TITLE 36039
#define IDS_BEFORERELOAD_APP_MESSAGEBOX_TITLE 36040
#define IDS_BEFORERELOAD_MESSAGEBOX_OK_BUTTON_LABEL 36041
#define IDS_LIVE_CAPTION_BUBBLE_TITLE 36042
#define IDS_LIVE_CAPTION_BUBBLE_CLOSE 36043
#define IDS_LIVE_CAPTION_BUBBLE_EXPAND 36044
#define IDS_LIVE_CAPTION_BUBBLE_COLLAPSE 36045
#define IDS_LIVE_CAPTION_BUBBLE_BACK_TO_TAB 36046
#define IDS_LIVE_CAPTION_BUBBLE_ERROR 36047
#define IDS_LIVE_CAPTION_BUBBLE_CONTENT_SETTINGS 36048
#define IDS_LIVE_CAPTION_BUBBLE_MEDIA_FOUNDATION_RENDERER_ERROR 36049
#define IDS_LIVE_CAPTION_BUBBLE_MEDIA_FOUNDATION_RENDERER_ERROR_CHECKBOX 36050
#define IDS_LIVE_CAPTION_BUBBLE_APPEAR_SCREENREADER_ANNOUNCEMENT 36051
#define IDS_LIVE_CAPTION_CAPTION_LANGUAGE_AUTODETECTED 36052
#define IDS_LIVE_CAPTION_BUBBLE_SCROLL_BUTTON_SCROLLING 36053
#define IDS_LIVE_CAPTION_BUBBLE_SCROLL_BUTTON_LOCKED 36054
#define IDS_LIVE_CAPTION_DOWNLOAD_PROGRESS 36055
#define IDS_LIVE_CAPTION_TRANSLATION_DOWNLOADING_LANGUAGE_PACKAGES_WITH_PROGRESS 36056
#define IDS_LIVE_CAPTION_INSTALLING 36057
#define IDS_LIVE_CAPTION_INSTALLING_LANGUAGE_PACKAGES 36058
#define IDS_LIVE_CAPTION_LANGUAGE_DOWNLOAD_FAILED 36059
#define IDS_LIVE_CAPTION_TRANSLATING 36060
#define IDS_LOGIN_DIALOG_TITLE 36061
#define IDS_LOGIN_DIALOG_OK_BUTTON_LABEL 36062
#define IDS_LOGIN_DIALOG_AUTHORITY 36063
#define IDS_LOGIN_DIALOG_PROXY_AUTHORITY 36064
#define IDS_LOGIN_DIALOG_NOT_PRIVATE 36065
#define IDS_LOGIN_DIALOG_USERNAME_FIELD 36066
#define IDS_LOGIN_DIALOG_PASSWORD_FIELD 36067
#define IDS_MANAGEMENT_TITLE 36068
#define IDS_MANAGEMENT_TOOLBAR_TITLE 36069
#define IDS_MANAGEMENT_SEARCH_PROMPT 36070
#define IDS_MANAGEMENT_SUBTITLE_MANAGED_BY 36072
#define IDS_MANAGEMENT_NOT_MANAGED_SUBTITLE 36073
#define IDS_MANAGEMENT_SUBTITLE 36074
#define IDS_MANAGEMENT_SUBTITLE_BROWSER_AND_PROFILE_DIFFERENT_MANAGED_BY 36075
#define IDS_MANAGEMENT_SUBTITLE_BROWSER_MANAGED_AND_PROFILE_MANAGED_BY 36076
#define IDS_MANAGEMENT_SUBTITLE_BROWSER_AND_PROFILE_SAME_MANAGED_BY 36077
#define IDS_MANAGEMENT_SUBTITLE_PROFILE_MANAGED_BY 36078
#define IDS_MANAGEMENT_BROWSER_NOTICE 36079
#define IDS_MANAGEMENT_NOT_MANAGED_NOTICE 36080
#define IDS_MANAGEMENT_PROFILE_NOTICE 36081
#define IDS_MANAGEMENT_LEARN_MORE_ACCCESSIBILITY_TEXT 36082
#define IDS_MANAGEMENT_EXTENSION_REPORTING 36138
#define IDS_MANAGEMENT_EXTENSIONS_INSTALLED 36139
#define IDS_MANAGEMENT_EXTENSIONS_INSTALLED_BY 36140
#define IDS_MANAGEMENT_EXTENSIONS_NAME 36141
#define IDS_MANAGEMENT_EXTENSIONS_PERMISSIONS 36142
#define IDS_MANAGEMENT_APPLICATION_REPORTING 36143
#define IDS_MANAGEMENT_APPLICATIONS_INSTALLED 36144
#define IDS_MANAGEMENT_APPLICATIONS_INSTALLED_BY 36145
#define IDS_MANAGEMENT_APPLICATIONS_NAME 36146
#define IDS_MANAGEMENT_APPLICATIONS_PERMISSIONS 36147
#define IDS_MANAGEMENT_APPLICATIONS_RUN_ON_OS_LOGIN 36148
#define IDS_MANAGEMENT_MANAGED_WEBSITES 36149
#define IDS_MANAGEMENT_MANAGED_WEBSITES_EXPLANATION 36150
#define IDS_MANAGEMENT_MANAGED_WEBSITES_BY_EXPLANATION 36151
#define IDS_MANAGEMENT_BROWSER_REPORTING_EXPLANATION 36152
#define IDS_MANAGEMENT_EXTENSION_REPORT_MACHINE_NAME 36153
#define IDS_MANAGEMENT_EXTENSION_REPORT_MACHINE_NAME_ADDRESS 36154
#define IDS_MANAGEMENT_EXTENSION_REPORT_USERNAME 36155
#define IDS_MANAGEMENT_EXTENSION_REPORT_VERSION 36156
#define IDS_MANAGEMENT_EXTENSION_REPORT_EXTENSIONS_PLUGINS 36157
#define IDS_MANAGEMENT_EXTENSION_REPORT_USER_BROWSING_DATA 36158
#define IDS_MANAGEMENT_EXTENSION_REPORT_PERF_CRASH 36159
#define IDS_MANAGEMENT_EXTENSION_REPORT_VISITED_URL 36160
#define IDS_MANAGEMENT_LEGACY_TECH_REPORT 36161
#define IDS_MANAGEMENT_LEGACY_TECH_REPORT_NO_LINK 36162
#define IDS_MANAGEMENT_THREAT_PROTECTION_MORE 36163
#define IDS_MANAGEMENT_FILE_ATTACHED_EVENT 36164
#define IDS_MANAGEMENT_TEXT_ENTERED_EVENT 36165
#define IDS_MANAGEMENT_PAGE_PRINTED_EVENT 36166
#define IDS_MANAGEMENT_FILE_TRANSFER_EVENT 36167
#define IDS_MANAGEMENT_SCREEN_CAPTURE_EVENT 36168
#define IDS_MANAGEMENT_EXTENSION_TELEMETRY_EVENT 36169
#define IDS_MANAGEMENT_FILE_ATTACHED_VISIBLE_DATA 36170
#define IDS_MANAGEMENT_TEXT_ENTERED_VISIBLE_DATA 36171
#define IDS_MANAGEMENT_PAGE_PRINTED_VISIBLE_DATA 36172
#define IDS_MANAGEMENT_FILE_TRANSFER_VISIBLE_DATA 36173
#define IDS_MANAGEMENT_SCREEN_CAPTURE_DATA 36174
#define IDS_MANAGEMENT_EXTENSION_TELEMETRY_VISIBLE_DATA 36175
#define IDS_MANAGEMENT_BROWSER_REPORTING 36176
#define IDS_MANAGEMENT_PROFILE_REPORTING_EXPLANATION 36184
#define IDS_MANAGEMENT_PROFILE_REPORTING_OVERVIEW 36185
#define IDS_MANAGEMENT_PROFILE_REPORTING_USERNAME 36186
#define IDS_MANAGEMENT_PROFILE_REPORTING_BROWSER 36187
#define IDS_MANAGEMENT_PROFILE_REPORTING_POLICY 36188
#define IDS_MANAGEMENT_PROFILE_REPORTING_LEARN_MORE 36189
#define IDS_MANAGEMENT_PROFILE_REPORTING_EXTENSION 36190
#define IDS_MANAGEMENT_DEVICE_SIGNALS_DISCLOSURE 36195
#define IDS_MANAGEMENT_THREAT_PROTECTION 36196
#define IDS_MANAGEMENT_THREAT_PROTECTION_DESCRIPTION 36197
#define IDS_MANAGEMENT_THREAT_PROTECTION_DESCRIPTION_BY 36198
#define IDS_MANAGEMENT_CONNECTORS_EVENT 36199
#define IDS_MANAGEMENT_CONNECTORS_VISIBLE_DATA 36200
#define IDS_MANAGEMENT_ENTERPRISE_REPORTING_EVENT 36201
#define IDS_MANAGEMENT_FILE_DOWNLOADED_EVENT 36202
#define IDS_MANAGEMENT_FILE_DOWNLOADED_VISIBLE_DATA 36203
#define IDS_MANAGEMENT_PAGE_VISITED_EVENT 36204
#define IDS_MANAGEMENT_PAGE_VISITED_VISIBLE_DATA 36205
#define IDS_MANAGEMENT_ENTERPRISE_REPORTING_VISIBLE_DATA 36206
#define IDS_MANAGEMENT_DIALOG_BROWSING_DATA_MANAGEMENT 36207
#define IDS_MANAGEMENT_DIALOG_DEVICE_SIGNALS 36208
#define IDS_MANAGEMENT_DIALOG_PROFILE_MANAGED_BY 36209
#define IDS_MANAGEMENT_DIALOG_THINGS_TO_CONSIDER_SUBTITLE 36210
#define IDS_MANAGED_PROFILE_INTERSTITIAL_TAB_TITLE 36211
#define IDS_MANAGEMENT_BANNER_PROMOTION_TITLE 36212
#define IDS_MANAGEMENT_BANNER_PROMOTION_DESC 36213
#define IDS_MANAGEMENT_BANNER_PROMOTION_BTN 36214
#define IDS_MANAGEMENT_BANNER_ARIA_LABEL 36215
#define IDS_MANAGEMENT_BANNER_NEW_TAB_ARIA_DESCRIPTION 36216
#define IDS_MANAGEMENT_BANNER_DISMISS_ARIA_LABEL 36217
#define IDS_MEDIA_MESSAGE_CENTER_MEDIA_NOTIFICATION_ACTION_PREVIOUS_TRACK 36218
#define IDS_MEDIA_MESSAGE_CENTER_MEDIA_NOTIFICATION_ACTION_SEEK_BACKWARD 36219
#define IDS_MEDIA_MESSAGE_CENTER_MEDIA_NOTIFICATION_ACTION_REPLAY_10 36220
#define IDS_MEDIA_MESSAGE_CENTER_MEDIA_NOTIFICATION_ACTION_PLAY 36221
#define IDS_MEDIA_MESSAGE_CENTER_MEDIA_NOTIFICATION_ACTION_PAUSE 36222
#define IDS_MEDIA_MESSAGE_CENTER_MEDIA_NOTIFICATION_ACTION_FORWARD_10 36223
#define IDS_MEDIA_MESSAGE_CENTER_MEDIA_NOTIFICATION_ACTION_SEEK_FORWARD 36224
#define IDS_MEDIA_MESSAGE_CENTER_MEDIA_NOTIFICATION_ACTION_NEXT_TRACK 36225
#define IDS_MEDIA_MESSAGE_CENTER_MEDIA_NOTIFICATION_ACTION_ENTER_PIP 36226
#define IDS_MEDIA_MESSAGE_CENTER_MEDIA_NOTIFICATION_ACTION_EXIT_PIP 36227
#define IDS_MEDIA_MESSAGE_CENTER_MEDIA_NOTIFICATION_ACTION_MUTE 36228
#define IDS_MEDIA_MESSAGE_CENTER_MEDIA_NOTIFICATION_ACTION_UNMUTE 36229
#define IDS_MEDIA_MESSAGE_CENTER_MEDIA_NOTIFICATION_ACTION_SHOW_DEVICE_LIST 36230
#define IDS_MEDIA_MESSAGE_CENTER_MEDIA_NOTIFICATION_ACTION_HIDE_DEVICE_LIST 36231
#define IDS_MEDIA_MESSAGE_CENTER_MEDIA_NOTIFICATION_ACTION_STOP_CASTING 36232
#define IDS_MEDIA_MESSAGE_CENTER_MEDIA_NOTIFICATION_TIME_SCRUBBER 36233
#define IDS_MEDIA_MESSAGE_CENTER_MEDIA_NOTIFICATION_ACCESSIBLE_NAME 36234
#define IDS_MEDIA_MESSAGE_CENTER_MEDIA_NOTIFICATION_LABEL 36235
#define IDS_DEFAULT_TAB_TITLE 36236
#define IDS_DOWNLOAD_TAB_TITLE 36237
#define IDS_SAD_TAB_TITLE 36238
#define IDS_SAD_TAB_MESSAGE 36239
#define IDS_SAD_TAB_HELP_MESSAGE 36240
#define IDS_SAD_TAB_HELP_LINK 36241
#define IDS_SAD_TAB_RELOAD_LABEL 36242
#define IDS_SAD_TAB_RELOAD_TITLE 36245
#define IDS_SAD_TAB_OOM_MESSAGE_TABS 36246
#define IDS_SAD_TAB_OOM_MESSAGE_NOTABS 36247
#define IDS_SAD_TAB_RELOAD_TRY 36248
#define IDS_SAD_TAB_RELOAD_INCOGNITO 36249
#define IDS_SAD_TAB_RELOAD_CLOSE_TABS 36250
#define IDS_SAD_TAB_RELOAD_CLOSE_NOTABS 36251
#define IDS_SAD_TAB_RELOAD_RESTART_BROWSER 36252
#define IDS_SAD_TAB_RELOAD_RESTART_DEVICE 36253
#define IDS_SAD_TAB_ERROR_CODE 36255
#define IDS_NEW_TAB_TITLE 36256
#define IDS_NEW_INCOGNITO_TAB_TITLE 36257
#define IDS_INCOGNITO_TAB_LEARN_MORE_ACCESSIBILITY_LABEL 36258
#define IDS_NEW_TAB_OTR_HEADING 36259
#define IDS_NEW_TAB_OTR_DESCRIPTION 36260
#define IDS_NEW_TAB_OTR_LEARN_MORE_LINK 36261
#define IDS_NEW_TAB_OTR_MESSAGE_WARNING 36262
#define IDS_NEW_TAB_UNDO_THUMBNAIL_REMOVE 36263
#define IDS_NEW_TAB_OTR_TITLE 36264
#define IDS_NEW_TAB_OTR_SUBTITLE 36265
#define IDS_NEW_TAB_OTR_SUBTITLE_WITH_READING_LIST 36266
#define IDS_NEW_TAB_OTR_NOT_SAVED 36267
#define IDS_NEW_TAB_OTR_VISIBLE 36268
#define IDS_NEW_TAB_OPENS_HC_ARTICLE_IN_NEW_TAB 36269
#define IDS_OMNIBOX_PEDAL_CLEAR_BROWSING_DATA_HINT 36270
#define IDS_OMNIBOX_PEDAL_CLEAR_BROWSING_DATA_SUGGESTION_CONTENTS 36271
#define IDS_ACC_OMNIBOX_PEDAL_CLEAR_BROWSING_DATA_SUFFIX 36272
#define IDS_ACC_OMNIBOX_PEDAL_CLEAR_BROWSING_DATA 36273
#define IDS_OMNIBOX_PEDAL_MANAGE_PASSWORDS_HINT 36274
#define IDS_OMNIBOX_PEDAL_MANAGE_PASSWORDS_SUGGESTION_CONTENTS 36275
#define IDS_ACC_OMNIBOX_PEDAL_MANAGE_PASSWORDS_SUFFIX 36276
#define IDS_ACC_OMNIBOX_PEDAL_MANAGE_PASSWORDS 36277
#define IDS_OMNIBOX_PEDAL_UPDATE_CREDIT_CARD_HINT 36278
#define IDS_OMNIBOX_PEDAL_UPDATE_CREDIT_CARD_SUGGESTION_CONTENTS 36279
#define IDS_ACC_OMNIBOX_PEDAL_UPDATE_CREDIT_CARD_SUFFIX 36280
#define IDS_ACC_OMNIBOX_PEDAL_UPDATE_CREDIT_CARD 36281
#define IDS_OMNIBOX_PEDAL_LAUNCH_INCOGNITO_HINT 36282
#define IDS_OMNIBOX_PEDAL_LAUNCH_INCOGNITO_SUGGESTION_CONTENTS 36283
#define IDS_ACC_OMNIBOX_PEDAL_LAUNCH_INCOGNITO_SUFFIX 36284
#define IDS_ACC_OMNIBOX_PEDAL_LAUNCH_INCOGNITO 36285
#define IDS_OMNIBOX_PEDAL_TRANSLATE_HINT 36286
#define IDS_OMNIBOX_PEDAL_TRANSLATE_SUGGESTION_CONTENTS 36287
#define IDS_ACC_OMNIBOX_PEDAL_TRANSLATE_SUFFIX 36288
#define IDS_ACC_OMNIBOX_PEDAL_TRANSLATE 36289
#define IDS_OMNIBOX_PEDAL_UPDATE_CHROME_HINT 36290
#define IDS_OMNIBOX_PEDAL_UPDATE_CHROME_SUGGESTION_CONTENTS 36291
#define IDS_ACC_OMNIBOX_PEDAL_UPDATE_CHROME_SUFFIX 36292
#define IDS_ACC_OMNIBOX_PEDAL_UPDATE_CHROME 36293
#define IDS_OMNIBOX_PEDAL_RUN_CHROME_SAFETY_CHECK_SUGGESTION_CONTENTS 36294
#define IDS_ACC_OMNIBOX_PEDAL_RUN_CHROME_SAFETY_CHECK_SUFFIX 36295
#define IDS_ACC_OMNIBOX_PEDAL_RUN_CHROME_SAFETY_CHECK 36296
#define IDS_OMNIBOX_PEDAL_RUN_CHROME_SAFETY_CHECK_V2_HINT 36297
#define IDS_OMNIBOX_PEDAL_RUN_CHROME_SAFETY_CHECK_V2_SUGGESTION_CONTENTS 36298
#define IDS_ACC_OMNIBOX_PEDAL_RUN_CHROME_SAFETY_CHECK_V2_SUFFIX 36299
#define IDS_ACC_OMNIBOX_PEDAL_RUN_CHROME_SAFETY_CHECK_V2 36300
#define IDS_OMNIBOX_PEDAL_MANAGE_SECURITY_SETTINGS_HINT 36301
#define IDS_OMNIBOX_PEDAL_MANAGE_SECURITY_SETTINGS_SUGGESTION_CONTENTS 36302
#define IDS_ACC_OMNIBOX_PEDAL_MANAGE_SECURITY_SETTINGS_SUFFIX 36303
#define IDS_ACC_OMNIBOX_PEDAL_MANAGE_SECURITY_SETTINGS 36304
#define IDS_OMNIBOX_PEDAL_MANAGE_COOKIES_HINT 36305
#define IDS_OMNIBOX_PEDAL_MANAGE_COOKIES_SUGGESTION_CONTENTS 36306
#define IDS_ACC_OMNIBOX_PEDAL_MANAGE_COOKIES_SUFFIX 36307
#define IDS_ACC_OMNIBOX_PEDAL_MANAGE_COOKIES 36308
#define IDS_OMNIBOX_PEDAL_MANAGE_ADDRESSES_HINT 36309
#define IDS_OMNIBOX_PEDAL_MANAGE_ADDRESSES_SUGGESTION_CONTENTS 36310
#define IDS_ACC_OMNIBOX_PEDAL_MANAGE_ADDRESSES_SUFFIX 36311
#define IDS_ACC_OMNIBOX_PEDAL_MANAGE_ADDRESSES 36312
#define IDS_OMNIBOX_PEDAL_MANAGE_SYNC_HINT 36313
#define IDS_OMNIBOX_PEDAL_MANAGE_SYNC_SUGGESTION_CONTENTS 36314
#define IDS_ACC_OMNIBOX_PEDAL_MANAGE_SYNC_SUFFIX 36315
#define IDS_ACC_OMNIBOX_PEDAL_MANAGE_SYNC 36316
#define IDS_OMNIBOX_PEDAL_MANAGE_SYNC_HINT_UPDATED 36317
#define IDS_OMNIBOX_PEDAL_MANAGE_SYNC_SUGGESTION_CONTENTS_UPDATED 36318
#define IDS_ACC_OMNIBOX_PEDAL_MANAGE_SYNC_SUFFIX_UPDATED 36319
#define IDS_ACC_OMNIBOX_PEDAL_MANAGE_SYNC_UPDATED 36320
#define IDS_OMNIBOX_PEDAL_MANAGE_SITE_SETTINGS_HINT 36321
#define IDS_OMNIBOX_PEDAL_MANAGE_SITE_SETTINGS_SUGGESTION_CONTENTS 36322
#define IDS_ACC_OMNIBOX_PEDAL_MANAGE_SITE_SETTINGS_SUFFIX 36323
#define IDS_ACC_OMNIBOX_PEDAL_MANAGE_SITE_SETTINGS 36324
#define IDS_OMNIBOX_PEDAL_CREATE_GOOGLE_DOC_HINT 36325
#define IDS_OMNIBOX_PEDAL_CREATE_GOOGLE_DOC_SUGGESTION_CONTENTS 36326
#define IDS_ACC_OMNIBOX_PEDAL_CREATE_GOOGLE_DOC_SUFFIX 36327
#define IDS_ACC_OMNIBOX_PEDAL_CREATE_GOOGLE_DOC 36328
#define IDS_OMNIBOX_PEDAL_CREATE_GOOGLE_SHEET_HINT 36329
#define IDS_OMNIBOX_PEDAL_CREATE_GOOGLE_SHEET_SUGGESTION_CONTENTS 36330
#define IDS_ACC_OMNIBOX_PEDAL_CREATE_GOOGLE_SHEET_SUFFIX 36331
#define IDS_ACC_OMNIBOX_PEDAL_CREATE_GOOGLE_SHEET 36332
#define IDS_OMNIBOX_PEDAL_CREATE_GOOGLE_SLIDE_HINT 36333
#define IDS_OMNIBOX_PEDAL_CREATE_GOOGLE_SLIDE_SUGGESTION_CONTENTS 36334
#define IDS_ACC_OMNIBOX_PEDAL_CREATE_GOOGLE_SLIDE_SUFFIX 36335
#define IDS_ACC_OMNIBOX_PEDAL_CREATE_GOOGLE_SLIDE 36336
#define IDS_OMNIBOX_PEDAL_CREATE_GOOGLE_CALENDAR_EVENT_HINT 36337
#define IDS_OMNIBOX_PEDAL_CREATE_GOOGLE_CALENDAR_EVENT_SUGGESTION_CONTENTS 36338
#define IDS_ACC_OMNIBOX_PEDAL_CREATE_GOOGLE_CALENDAR_EVENT_SUFFIX 36339
#define IDS_ACC_OMNIBOX_PEDAL_CREATE_GOOGLE_CALENDAR_EVENT 36340
#define IDS_OMNIBOX_PEDAL_CREATE_GOOGLE_SITE_HINT 36341
#define IDS_OMNIBOX_PEDAL_CREATE_GOOGLE_SITE_SUGGESTION_CONTENTS 36342
#define IDS_ACC_OMNIBOX_PEDAL_CREATE_GOOGLE_SITE_SUFFIX 36343
#define IDS_ACC_OMNIBOX_PEDAL_CREATE_GOOGLE_SITE 36344
#define IDS_OMNIBOX_PEDAL_CREATE_GOOGLE_KEEP_NOTE_HINT 36345
#define IDS_OMNIBOX_PEDAL_CREATE_GOOGLE_KEEP_NOTE_SUGGESTION_CONTENTS 36346
#define IDS_ACC_OMNIBOX_PEDAL_CREATE_GOOGLE_KEEP_NOTE_SUFFIX 36347
#define IDS_ACC_OMNIBOX_PEDAL_CREATE_GOOGLE_KEEP_NOTE 36348
#define IDS_OMNIBOX_PEDAL_CREATE_GOOGLE_FORM_HINT 36349
#define IDS_OMNIBOX_PEDAL_CREATE_GOOGLE_FORM_SUGGESTION_CONTENTS 36350
#define IDS_ACC_OMNIBOX_PEDAL_CREATE_GOOGLE_FORM_SUFFIX 36351
#define IDS_ACC_OMNIBOX_PEDAL_CREATE_GOOGLE_FORM 36352
#define IDS_OMNIBOX_PEDAL_SEE_CHROME_TIPS_HINT 36353
#define IDS_OMNIBOX_PEDAL_SEE_CHROME_TIPS_SUGGESTION_CONTENTS 36354
#define IDS_ACC_OMNIBOX_PEDAL_SEE_CHROME_TIPS_SUFFIX 36355
#define IDS_ACC_OMNIBOX_PEDAL_SEE_CHROME_TIPS 36356
#define IDS_OMNIBOX_PEDAL_MANAGE_GOOGLE_ACCOUNT_HINT 36357
#define IDS_OMNIBOX_PEDAL_MANAGE_GOOGLE_ACCOUNT_SUGGESTION_CONTENTS 36358
#define IDS_ACC_OMNIBOX_PEDAL_MANAGE_GOOGLE_ACCOUNT_SUFFIX 36359
#define IDS_ACC_OMNIBOX_PEDAL_MANAGE_GOOGLE_ACCOUNT 36360
#define IDS_OMNIBOX_PEDAL_CHANGE_GOOGLE_PASSWORD_HINT 36361
#define IDS_OMNIBOX_PEDAL_CHANGE_GOOGLE_PASSWORD_SUGGESTION_CONTENTS 36362
#define IDS_ACC_OMNIBOX_PEDAL_CHANGE_GOOGLE_PASSWORD_SUFFIX 36363
#define IDS_ACC_OMNIBOX_PEDAL_CHANGE_GOOGLE_PASSWORD 36364
#define IDS_OMNIBOX_PEDAL_CLOSE_INCOGNITO_WINDOWS_HINT 36365
#define IDS_OMNIBOX_PEDAL_CLOSE_INCOGNITO_WINDOWS_SUGGESTION_CONTENTS 36366
#define IDS_ACC_OMNIBOX_PEDAL_CLOSE_INCOGNITO_WINDOWS_SUFFIX 36367
#define IDS_ACC_OMNIBOX_PEDAL_CLOSE_INCOGNITO_WINDOWS 36368
#define IDS_OMNIBOX_PEDAL_PLAY_CHROME_DINO_GAME_HINT 36369
#define IDS_OMNIBOX_PEDAL_PLAY_CHROME_DINO_GAME_SUGGESTION_CONTENTS 36370
#define IDS_ACC_OMNIBOX_PEDAL_PLAY_CHROME_DINO_GAME_SUFFIX 36371
#define IDS_ACC_OMNIBOX_PEDAL_PLAY_CHROME_DINO_GAME 36372
#define IDS_OMNIBOX_PEDAL_FIND_MY_PHONE_HINT 36373
#define IDS_OMNIBOX_PEDAL_FIND_MY_PHONE_SUGGESTION_CONTENTS 36374
#define IDS_ACC_OMNIBOX_PEDAL_FIND_MY_PHONE_SUFFIX 36375
#define IDS_ACC_OMNIBOX_PEDAL_FIND_MY_PHONE 36376
#define IDS_OMNIBOX_PEDAL_MANAGE_GOOGLE_PRIVACY_HINT 36377
#define IDS_OMNIBOX_PEDAL_MANAGE_GOOGLE_PRIVACY_SUGGESTION_CONTENTS 36378
#define IDS_ACC_OMNIBOX_PEDAL_MANAGE_GOOGLE_PRIVACY_SUFFIX 36379
#define IDS_ACC_OMNIBOX_PEDAL_MANAGE_GOOGLE_PRIVACY 36380
#define IDS_OMNIBOX_PEDAL_MANAGE_CHROME_SETTINGS_HINT 36381
#define IDS_OMNIBOX_PEDAL_MANAGE_CHROME_SETTINGS_SUGGESTION_CONTENTS 36382
#define IDS_ACC_OMNIBOX_PEDAL_MANAGE_CHROME_SETTINGS_SUFFIX 36383
#define IDS_ACC_OMNIBOX_PEDAL_MANAGE_CHROME_SETTINGS 36384
#define IDS_OMNIBOX_PEDAL_MANAGE_CHROME_DOWNLOADS_HINT 36385
#define IDS_OMNIBOX_PEDAL_MANAGE_CHROME_DOWNLOADS_SUGGESTION_CONTENTS 36386
#define IDS_ACC_OMNIBOX_PEDAL_MANAGE_CHROME_DOWNLOADS_SUFFIX 36387
#define IDS_ACC_OMNIBOX_PEDAL_MANAGE_CHROME_DOWNLOADS 36388
#define IDS_OMNIBOX_PEDAL_VIEW_CHROME_HISTORY_HINT 36389
#define IDS_OMNIBOX_PEDAL_VIEW_CHROME_HISTORY_SUGGESTION_CONTENTS 36390
#define IDS_ACC_OMNIBOX_PEDAL_VIEW_CHROME_HISTORY_SUFFIX 36391
#define IDS_ACC_OMNIBOX_PEDAL_VIEW_CHROME_HISTORY 36392
#define IDS_OMNIBOX_PEDAL_SHARE_THIS_PAGE_HINT 36393
#define IDS_OMNIBOX_PEDAL_SHARE_THIS_PAGE_SUGGESTION_CONTENTS 36394
#define IDS_ACC_OMNIBOX_PEDAL_SHARE_THIS_PAGE_SUFFIX 36395
#define IDS_ACC_OMNIBOX_PEDAL_SHARE_THIS_PAGE 36396
#define IDS_OMNIBOX_PEDAL_MANAGE_CHROME_ACCESSIBILITY_HINT 36397
#define IDS_OMNIBOX_PEDAL_MANAGE_CHROME_ACCESSIBILITY_SUGGESTION_CONTENTS 36398
#define IDS_ACC_OMNIBOX_PEDAL_MANAGE_CHROME_ACCESSIBILITY_SUFFIX 36399
#define IDS_ACC_OMNIBOX_PEDAL_MANAGE_CHROME_ACCESSIBILITY 36400
#define IDS_OMNIBOX_PEDAL_CUSTOMIZE_CHROME_FONTS_HINT 36401
#define IDS_OMNIBOX_PEDAL_CUSTOMIZE_CHROME_FONTS_SUGGESTION_CONTENTS 36402
#define IDS_ACC_OMNIBOX_PEDAL_CUSTOMIZE_CHROME_FONTS_SUFFIX 36403
#define IDS_ACC_OMNIBOX_PEDAL_CUSTOMIZE_CHROME_FONTS 36404
#define IDS_OMNIBOX_PEDAL_MANAGE_CHROME_THEMES_HINT 36405
#define IDS_OMNIBOX_PEDAL_MANAGE_CHROME_THEMES_SUGGESTION_CONTENTS 36406
#define IDS_ACC_OMNIBOX_PEDAL_MANAGE_CHROME_THEMES_SUFFIX 36407
#define IDS_ACC_OMNIBOX_PEDAL_MANAGE_CHROME_THEMES 36408
#define IDS_OMNIBOX_PEDAL_CUSTOMIZE_SEARCH_ENGINES_HINT 36409
#define IDS_OMNIBOX_PEDAL_CUSTOMIZE_SEARCH_ENGINES_SUGGESTION_CONTENTS 36410
#define IDS_ACC_OMNIBOX_PEDAL_CUSTOMIZE_SEARCH_ENGINES_SUFFIX 36411
#define IDS_ACC_OMNIBOX_PEDAL_CUSTOMIZE_SEARCH_ENGINES 36412
#define IDS_OMNIBOX_PEDAL_MANAGE_CHROMEOS_ACCESSIBILITY_HINT 36413
#define IDS_OMNIBOX_PEDAL_MANAGE_CHROMEOS_ACCESSIBILITY_SUGGESTION_CONTENTS 36414
#define IDS_ACC_OMNIBOX_PEDAL_MANAGE_CHROMEOS_ACCESSIBILITY_SUFFIX 36415
#define IDS_ACC_OMNIBOX_PEDAL_MANAGE_CHROMEOS_ACCESSIBILITY 36416
#define IDS_OMNIBOX_PEDAL_SET_CHROME_AS_DEFAULT_BROWSER_HINT 36417
#define IDS_OMNIBOX_PEDAL_SET_CHROME_AS_DEFAULT_BROWSER_SUGGESTION_CONTENTS 36418
#define IDS_ACC_OMNIBOX_PEDAL_SET_CHROME_AS_DEFAULT_BROWSER_SUFFIX 36419
#define IDS_ACC_OMNIBOX_PEDAL_SET_CHROME_AS_DEFAULT_BROWSER 36420
#define IDS_ANDROID_OMNIBOX_PEDAL_LAUNCH_INCOGNITO_HINT 36421
#define IDS_ANDROID_OMNIBOX_PEDAL_LAUNCH_INCOGNITO_SUGGESTION_CONTENTS 36422
#define IDS_ANDROID_ACC_OMNIBOX_PEDAL_LAUNCH_INCOGNITO_SUFFIX 36423
#define IDS_ANDROID_ACC_OMNIBOX_PEDAL_LAUNCH_INCOGNITO 36424
#define IDS_IOS_OMNIBOX_PEDAL_CLEAR_BROWSING_DATA_HINT 36425
#define IDS_IOS_OMNIBOX_PEDAL_SET_CHROME_AS_DEFAULT_BROWSER_HINT 36426
#define IDS_IOS_OMNIBOX_PEDAL_SET_CHROME_AS_DEFAULT_BROWSER_SUGGESTION_CONTENTS 36427
#define IDS_IOS_ACC_OMNIBOX_PEDAL_SET_CHROME_AS_DEFAULT_BROWSER_SUFFIX 36428
#define IDS_IOS_ACC_OMNIBOX_PEDAL_SET_CHROME_AS_DEFAULT_BROWSER 36429
#define IDS_IOS_OMNIBOX_PEDAL_UPDATE_CREDIT_CARD_HINT 36430
#define IDS_IOS_OMNIBOX_PEDAL_LAUNCH_INCOGNITO_HINT 36431
#define IDS_IOS_OMNIBOX_PEDAL_RUN_CHROME_SAFETY_CHECK_HINT 36432
#define IDS_IOS_OMNIBOX_PEDAL_MANAGE_CHROME_SETTINGS_HINT 36433
#define IDS_IOS_OMNIBOX_PEDAL_VIEW_CHROME_HISTORY_HINT 36434
#define IDS_IOS_OMNIBOX_PEDAL_PLAY_CHROME_DINO_GAME_HINT 36435
#define IDS_IOS_OMNIBOX_PEDAL_LAUNCH_INCOGNITO_SUGGESTION_CONTENTS 36436
#define IDS_IOS_OMNIBOX_PEDAL_MANAGE_PASSWORDS_HINT 36437
#define IDS_AUTOCOMPLETE_SEARCH_DESCRIPTION 36438
#define IDS_OMNIBOX_ILLS_USE_APPROXIMATE_LOCATION 36439
#define IDS_OMNIBOX_ILLS_USE_LOCATION 36440
#define IDS_OMNIBOX_ILLS_USE_PRECISE_LOCATION 36441
#define IDS_AUTOCOMPLETE_ASK_DESCRIPTION 36442
#define IDS_CONTEXTUAL_SEARCH_OPEN_LENS_ACTION_LABEL 36443
#define IDS_CONTEXTUAL_SEARCH_OPEN_LENS_ACTION_LABEL_ALT 36444
#define IDS_CONTEXTUAL_SEARCH_OPEN_LENS_ACTION_LABEL_ALT2 36445
#define IDS_OMNIBOX_TOOLBELT_LABEL 36446
#define IDS_CONTEXTUAL_SEARCH_OPEN_LENS_ACTION_HINT 36447
#define IDS_CONTEXTUAL_SEARCH_OPEN_LENS_ACTION_SUGGESTION_CONTENTS 36448
#define IDS_ACC_CONTEXTUAL_SEARCH_OPEN_LENS_ACTION_SUFFIX 36449
#define IDS_ACC_CONTEXTUAL_SEARCH_OPEN_LENS_ACTION 36450
#define IDS_STARTER_PACK_BOOKMARKS_ACTION_HINT 36451
#define IDS_STARTER_PACK_BOOKMARKS_ACTION_SUGGESTION_CONTENTS 36452
#define IDS_ACC_STARTER_PACK_BOOKMARKS_ACTION_SUFFIX 36453
#define IDS_ACC_STARTER_PACK_BOOKMARKS_ACTION 36454
#define IDS_STARTER_PACK_HISTORY_ACTION_HINT 36455
#define IDS_STARTER_PACK_HISTORY_ACTION_SUGGESTION_CONTENTS 36456
#define IDS_ACC_STARTER_PACK_HISTORY_ACTION_SUFFIX 36457
#define IDS_ACC_STARTER_PACK_HISTORY_ACTION 36458
#define IDS_STARTER_PACK_TABS_ACTION_HINT 36459
#define IDS_STARTER_PACK_TABS_ACTION_SUGGESTION_CONTENTS 36460
#define IDS_ACC_STARTER_PACK_TABS_ACTION_SUFFIX 36461
#define IDS_ACC_STARTER_PACK_TABS_ACTION 36462
#define IDS_STARTER_PACK_AI_MODE_ACTION_HINT 36463
#define IDS_STARTER_PACK_AI_MODE_ACTION_SUGGESTION_CONTENTS 36464
#define IDS_ACC_STARTER_PACK_AI_MODE_ACTION_SUFFIX 36465
#define IDS_ACC_STARTER_PACK_AI_MODE_ACTION 36466
#define IDS_EMPTY_KEYWORD_VALUE 36467
#define IDS_EMPTY_STARTER_PACK_AI_MODE_VALUE 36468
#define IDS_LINK_FROM_CLIPBOARD 36469
#define IDS_TEXT_FROM_CLIPBOARD 36470
#define IDS_IMAGE_FROM_CLIPBOARD 36471
#define IDS_SECURE_VERBOSE_STATE 36472
#define IDS_NOT_SECURE_VERBOSE_STATE 36473
#define IDS_DANGEROUS_VERBOSE_STATE 36474
#define IDS_OFFLINE_VERBOSE_STATE 36475
#define IDS_OMNIBOX_EMPTY_HINT 36476
#define IDS_OMNIBOX_EMPTY_HINT_WITH_DSE_NAME 36477
#define IDS_OMNIBOX_EMPTY_ASK_HINT_WITH_DSE_NAME 36478
#define IDS_OMNIBOX_NAVATTACH_GALLERY 36479
#define IDS_OMNIBOX_NAVATTACH_FILES 36480
#define IDS_OMNIBOX_NAVATTACH_TABS 36481
#define IDS_FUSEBOX_ADD_CURRENT_TAB 36482
#define IDS_FUSEBOX_ADD_TAB 36483
#define IDS_FUSEBOX_UPLOAD_FAILED 36484
#define IDS_OMNIBOX_CREATE_IMAGE 36485
#define IDS_OMNIBOX_EMPTY_HINT_FOR_IMAGE_GENERATION 36486
#define IDS_FUSEBOX_RECENT_TABS_HEADER 36487
#define IDS_OMNIBOX_ADDITIONAL_TEXT_DASH_TEMPLATE 36489
#define IDS_OMNIBOX_TAB_SUGGEST_HINT 36490
#define IDS_OMNIBOX_HISTORY_EMBEDDING_HINT 36491
#define IDS_OMNIBOX_FILE 36492
#define IDS_OMNIBOX_READER_MODE 36493
#define IDS_CONTENT_SUGGESTION_DESCRIPTION_BUGANIZER 36494
#define IDS_CONTENT_SUGGESTION_DESCRIPTION_COMMA_SEPARATED_VALUES 36495
#define IDS_CONTENT_SUGGESTION_DESCRIPTION_GOOGLE_APPS_SCRIPT 36496
#define IDS_CONTENT_SUGGESTION_DESCRIPTION_GOOGLE_DOCS 36497
#define IDS_CONTENT_SUGGESTION_DESCRIPTION_GOOGLE_DRAWINGS 36498
#define IDS_CONTENT_SUGGESTION_DESCRIPTION_GOOGLE_DRIVE 36499
#define IDS_CONTENT_SUGGESTION_DESCRIPTION_GOOGLE_DRIVE_FOLDER 36500
#define IDS_CONTENT_SUGGESTION_DESCRIPTION_GOOGLE_FORMS 36501
#define IDS_CONTENT_SUGGESTION_DESCRIPTION_GOOGLE_JAMBOARD 36502
#define IDS_CONTENT_SUGGESTION_DESCRIPTION_GOOGLE_PHOTOS 36503
#define IDS_CONTENT_SUGGESTION_DESCRIPTION_GOOGLE_SITES 36504
#define IDS_CONTENT_SUGGESTION_DESCRIPTION_GOOGLE_SHEETS 36505
#define IDS_CONTENT_SUGGESTION_DESCRIPTION_GOOGLE_SLIDES 36506
#define IDS_CONTENT_SUGGESTION_DESCRIPTION_IMAGE_JPEG 36507
#define IDS_CONTENT_SUGGESTION_DESCRIPTION_IMAGE_PNG 36508
#define IDS_CONTENT_SUGGESTION_DESCRIPTION_IMAGE_SVG 36509
#define IDS_CONTENT_SUGGESTION_DESCRIPTION_JIRA 36510
#define IDS_CONTENT_SUGGESTION_DESCRIPTION_JSON 36511
#define IDS_CONTENT_SUGGESTION_DESCRIPTION_MARKDOWN 36512
#define IDS_CONTENT_SUGGESTION_DESCRIPTION_MS_EXCEL 36513
#define IDS_CONTENT_SUGGESTION_DESCRIPTION_MS_POWERPOINT 36514
#define IDS_CONTENT_SUGGESTION_DESCRIPTION_MS_WORD 36515
#define IDS_CONTENT_SUGGESTION_DESCRIPTION_OPEN_DOCUMENT_PRESENTATION 36516
#define IDS_CONTENT_SUGGESTION_DESCRIPTION_OPEN_DOCUMENT_SPREADSHEET 36517
#define IDS_CONTENT_SUGGESTION_DESCRIPTION_OPEN_DOCUMENT_TEXT 36518
#define IDS_CONTENT_SUGGESTION_DESCRIPTION_PDF 36519
#define IDS_CONTENT_SUGGESTION_DESCRIPTION_PLAIN_TEXT 36520
#define IDS_CONTENT_SUGGESTION_DESCRIPTION_RICH_TEXT_FORMAT 36521
#define IDS_CONTENT_SUGGESTION_DESCRIPTION_SALESFORCE 36522
#define IDS_CONTENT_SUGGESTION_DESCRIPTION_SLACK 36523
#define IDS_CONTENT_SUGGESTION_DESCRIPTION_VIDEO_MP4 36524
#define IDS_CONTENT_SUGGESTION_DESCRIPTION_VIDEO_QUICKTIME 36525
#define IDS_CONTENT_SUGGESTION_DESCRIPTION_VIDEO_WEBM 36526
#define IDS_CONTENT_SUGGESTION_DESCRIPTION_ZIP_FILE 36527
#define IDS_CONTENT_SUGGESTION_DESCRIPTION_TEMPLATE 36528
#define IDS_CONTENT_SUGGESTION_DESCRIPTION_TEMPLATE_WITHOUT_DATE 36529
#define IDS_CONTENT_SUGGESTION_DESCRIPTION_TEMPLATE_WITHOUT_OWNER 36530
#define IDS_CONTENT_SUGGESTION_DESCRIPTION_TEMPLATE_WITHOUT_FILE_TYPE_DESCRIPTION 36531
#define IDS_PERSON_SUGGESTION_DESCRIPTION 36532
#define IDS_ACC_OMNIBOX_SUGGESTION_IN_GROUP 36533
#define IDS_ACC_OMNIBOX_SUGGESTION_IN_GROUP_WITH_DESCRIPTION 36534
#define IDS_OMNIBOX_ACTION_CROSS_DEVICE_TAB_HINT 36535
#define IDS_OMNIBOX_ACTION_CROSS_DEVICE_TAB_SUGGESTION_CONTENTS 36536
#define IDS_ACC_OMNIBOX_ACTION_CROSS_DEVICE_TAB_SUFFIX 36537
#define IDS_ACC_OMNIBOX_ACTION_CROSS_DEVICE_TAB 36538
#define IDS_OMNIBOX_HISTORY_CLUSTERS_SEARCH_HINT 36539
#define IDS_OMNIBOX_ACTION_HISTORY_CLUSTERS_SEARCH_HINT 36540
#define IDS_OMNIBOX_ACTION_HISTORY_CLUSTERS_SEARCH_SUGGESTION_CONTENTS 36541
#define IDS_ACC_OMNIBOX_ACTION_HISTORY_CLUSTERS_SEARCH_SUFFIX 36542
#define IDS_ACC_OMNIBOX_ACTION_HISTORY_CLUSTERS_SEARCH 36543
#define IDS_ACC_OMNIBOX_ACTION_IN_SUGGEST_SUFFIX 36544
#define IDS_OMNIBOX_ACTION_IN_SUGGEST_CALL_HINT 36545
#define IDS_OMNIBOX_ACTION_IN_SUGGEST_CALL_CONTENTS 36546
#define IDS_OMNIBOX_ACTION_IN_SUGGEST_DIRECTIONS_HINT 36547
#define IDS_OMNIBOX_ACTION_IN_SUGGEST_DIRECTIONS_CONTENTS 36548
#define IDS_OMNIBOX_ACTION_IN_SUGGEST_REVIEWS_HINT 36549
#define IDS_OMNIBOX_ACTION_IN_SUGGEST_REVIEWS_CONTENTS 36550
#define IDS_OMNIBOX_ACTION_IN_SUGGEST_AIM_HINT 36551
#define IDS_OMNIBOX_ACTION_IN_SUGGEST_AIM_CONTENTS 36552
#define IDS_OMNIBOX_ACTION_IN_SUGGEST_TAB_SWITCH_HINT 36553
#define IDS_OMNIBOX_ACTION_IN_SUGGEST_TAB_SWITCH_CONTENTS 36554
#define IDS_ACC_OMNIBOX_ACTION_IN_EXTENSION_SUGGEST_SUFFIX 36555
#define IDS_ACC_SEND_BUTTON_SEARCH_OR_NAVIGATE 36556
#define IDS_ACC_SEND_BUTTON_SEND_TO_AI 36557
#define IDS_ACC_SEND_BUTTON_CREATE_IMAGE 36558
#define IDS_ACC_REMOVE_IMAGE_ATTACHMENT 36559
#define IDS_ACC_REMOVE_ATTACHMENT 36560
#define IDS_ACC_FUSEBOX_POPUP_BUTTON_SELECTED 36561
#define IDS_ACC_AUTOCOMPLETE_HISTORY 36562
#define IDS_ACC_AUTOCOMPLETE_SEARCH_HISTORY 36563
#define IDS_ACC_AUTOCOMPLETE_SEARCH 36564
#define IDS_ACC_AUTOCOMPLETE_SUGGESTED_SEARCH 36565
#define IDS_ACC_AUTOCOMPLETE_SUGGESTED_SEARCH_ENTITY 36566
#define IDS_ACC_AUTOCOMPLETE_QUICK_ANSWER 36567
#define IDS_ACC_AUTOCOMPLETE_BOOKMARK 36568
#define IDS_ACC_AUTOCOMPLETE_CLIPBOARD_IMAGE 36569
#define IDS_ACC_AUTOCOMPLETE_CLIPBOARD_TEXT 36570
#define IDS_ACC_AUTOCOMPLETE_CLIPBOARD_URL 36571
#define IDS_ACC_SEARCH_ICON 36572
#define IDS_ACC_AUTOCOMPLETE_N_OF_M 36573
#define IDS_ACC_TAB_SWITCH_SUFFIX 36574
#define IDS_ACC_TAB_SWITCH_BUTTON 36575
#define IDS_ACC_MULTIPLE_ACTIONS_SUFFIX 36576
#define IDS_ACC_KEYWORD_SUFFIX 36577
#define IDS_ACC_KEYWORD_MODE 36578
#define IDS_ACC_ASK_KEYWORD_MODE 36579
#define IDS_ACC_KEYWORD_MODE_WITH_SHORTCUT 36580
#define IDS_ACC_ASK_KEYWORD_MODE_WITH_SHORTCUT 36581
#define IDS_OMNIBOX_KEYWORD_TEXT_MD 36582
#define IDS_ACC_THUMBS_UP_SUGGESTION_FOCUSED_PREFIX 36583
#define IDS_ACC_THUMBS_UP_SUGGESTION_BUTTON 36584
#define IDS_ACC_THUMBS_DOWN_SUGGESTION_FOCUSED_PREFIX 36585
#define IDS_ACC_THUMBS_DOWN_SUGGESTION_BUTTON 36586
#define IDS_ACC_REMOVE_SUGGESTION_SUFFIX 36587
#define IDS_ACC_REMOVE_SUGGESTION_FOCUSED_PREFIX 36588
#define IDS_ACC_REMOVE_SUGGESTION_BUTTON 36589
#define IDS_ACC_AI_MODE_PLACEHOLDER_TEXT 36590
#define IDS_ACC_AI_MODE_BUTTON_FOCUSED 36591
#define IDS_ACC_CHROME_TIP 36592
#define IDS_ACC_DISMISS_CHROME_TIP_SUFFIX 36593
#define IDS_ACC_DISMISS_CHROME_TIP_FOCUSED_PREFIX 36594
#define IDS_ACC_DISMISS_CHROME_TIP_BUTTON 36595
#define IDS_SEARCH_ENGINES_STARTER_PACK_BOOKMARKS_NAME 36596
#define IDS_SEARCH_ENGINES_STARTER_PACK_BOOKMARKS_KEYWORD 36597
#define IDS_SEARCH_ENGINES_STARTER_PACK_HISTORY_NAME 36598
#define IDS_SEARCH_ENGINES_STARTER_PACK_HISTORY_KEYWORD 36599
#define IDS_SEARCH_ENGINES_STARTER_PACK_TABS_NAME 36600
#define IDS_SEARCH_ENGINES_STARTER_PACK_TABS_KEYWORD 36601
#define IDS_SEARCH_ENGINES_STARTER_PACK_GEMINI_NAME 36602
#define IDS_SEARCH_ENGINES_STARTER_PACK_GEMINI_KEYWORD 36603
#define IDS_SEARCH_ENGINES_STARTER_PACK_PAGE_NAME 36604
#define IDS_SEARCH_ENGINES_STARTER_PACK_PAGE_KEYWORD 36605
#define IDS_SEARCH_ENGINES_STARTER_PACK_AI_MODE_NAME 36606
#define IDS_SEARCH_ENGINES_STARTER_PACK_AI_MODE_KEYWORD 36607
#define IDS_STARTER_PACK_PAGE_KEYWORD_TEXT 36608
#define IDS_STARTER_PACK_PAGE_EMPTY_QUERY_MATCH_TEXT 36609
#define IDS_OMNIBOX_INSTANT_KEYWORD_ASK_TEXT 36610
#define IDS_OMNIBOX_INSTANT_KEYWORD_SEARCH_TEXT 36611
#define IDS_OMNIBOX_SELECTED_KEYWORD_ASK_TEXT 36612
#define IDS_OMNIBOX_TAB_SEARCH_NO_RESULTS_FOUND 36613
#define IDS_OMNIBOX_GEMINI_IPH 36614
#define IDS_OMNIBOX_FEATURED_ENTERPRISE_SITE_SEARCH_IPH 36615
#define IDS_OMNIBOX_FEATURED_ENTERPRISE_SEARCH_AGGREGATOR_IPH 36616
#define IDS_OMNIBOX_HISTORY_EMBEDDINGS_SETTINGS_PROMO_IPH 36617
#define IDS_OMNIBOX_HISTORY_EMBEDDINGS_SETTINGS_PROMO_IPH_LINK_TEXT 36618
#define IDS_OMNIBOX_HISTORY_EMBEDDINGS_DISCLAIMER_IPH 36619
#define IDS_OMNIBOX_HISTORY_EMBEDDINGS_DISCLAIMER_IPH_LINK_TEXT 36620
#define IDS_ACC_OMNIBOX_IPH_LINK_SELECTED 36621
#define IDS_ACC_OMNIBOX_TOOLBELT_NEXT_SUFFIX 36622
#define IDS_OMNIBOX_HISTORY_SCOPE_PROMO_IPH 36623
#define IDS_OMNIBOX_HISTORY_EMBEDDINGS_SCOPE_PROMO_IPH 36624
#define IDS_OMNIBOX_ONE_LINE_CALCULATOR_SUGGESTION_TEMPLATE 36625
#define IDS_OMNIBOX_BOOKMARKS_SCOPE_PLACEHOLDER_TEXT 36626
#define IDS_OMNIBOX_HISTORY_SCOPE_PLACEHOLDER_TEXT 36627
#define IDS_OMNIBOX_HISTORY_EMBEDDINGS_SCOPE_PLACEHOLDER_TEXT 36628
#define IDS_OMNIBOX_TABS_SCOPE_PLACEHOLDER_TEXT 36629
#define IDS_OMNIBOX_GEMINI_SCOPE_PLACEHOLDER_TEXT 36630
#define IDS_OMNIBOX_PAGE_SCOPE_PLACEHOLDER_TEXT 36631
#define IDS_OMNIBOX_AI_MODE_SCOPE_PLACEHOLDER_TEXT 36632
#define IDS_OMNIBOX_HUB_OPEN_TABS_HEADER 36633
#define IDS_OMNIBOX_HUB_SEARCH_HEADER 36634
#define IDS_OMNIBOX_HUB_HISTORY_HEADER 36635
#define IDS_OMNIBOX_HUB_TYPED_MATCH_HEADER 36636
#define IDS_PAGE_INFO_SECURE_SUMMARY 36637
#define IDS_PAGE_INFO_MIXED_CONTENT_SUMMARY 36638
#define IDS_PAGE_INFO_MIXED_CONTENT_SUMMARY_SHORT 36639
#define IDS_PAGE_INFO_NOT_SECURE_SUMMARY 36640
#define IDS_PAGE_INFO_NOT_SECURE_SUMMARY_SHORT 36641
#define IDS_PAGE_INFO_SAFE_BROWSING_SUMMARY 36642
#define IDS_PAGE_INFO_EXTENSION_PAGE 36643
#define IDS_PAGE_INFO_VIEW_SOURCE_PAGE 36644
#define IDS_PAGE_INFO_DEVTOOLS_PAGE 36645
#define IDS_PAGE_INFO_READER_MODE_PAGE_SECURE 36646
#define IDS_PAGE_INFO_READER_MODE_PAGE 36647
#define IDS_PAGE_INFO_SAFETY_TIP_MORE_INFO_LINK 36648
#define IDS_PAGE_INFO_SAFETY_TIP_LOOKALIKE_LEAVE_BUTTON 36649
#define IDS_PAGE_INFO_SAFETY_TIP_LOOKALIKE_TITLE 36650
#define IDS_PAGE_INFO_SAFETY_TIP_DESCRIPTION 36651
#define IDS_PAGE_INFO_FILE_PAGE 36652
#define IDS_PAGE_INFO_SECURE_DETAILS 36653
#define IDS_PAGE_INFO_MIXED_CONTENT_DETAILS 36654
#define IDS_PAGE_INFO_NOT_SECURE_DETAILS 36655
#define IDS_PAGE_INFO_MALWARE_DETAILS 36656
#define IDS_PAGE_INFO_SOCIAL_ENGINEERING_DETAILS 36657
#define IDS_PAGE_INFO_UNWANTED_SOFTWARE_DETAILS 36658
#define IDS_PAGE_INFO_SECURITY_TAB_INSECURE_IDENTITY 36659
#define IDS_PAGE_INFO_CHANGE_PASSWORD_SUMMARY 36660
#define IDS_PAGE_INFO_CHANGE_PASSWORD_SAVED_PASSWORD_SUMMARY 36661
#define IDS_PAGE_INFO_CHANGE_PASSWORD_DETAILS_SAVED 36662
#define IDS_PAGE_INFO_CHECK_PASSWORDS_BUTTON 36663
#define IDS_PAGE_INFO_INVALID_CERTIFICATE_DESCRIPTION 36664
#define IDS_PAGE_INFO_RESET_INVALID_CERTIFICATE_DECISIONS_BUTTON 36665
#define IDS_PAGE_INFO_HELP_CENTER_LINK 36666
#define IDS_PAGE_INFO_SECURITY_TAB_DEPRECATED_SIGNATURE_ALGORITHM 36667
#define IDS_PAGE_INFO_SECURITY_TAB_ENCRYPTED_CONNECTION_TEXT 36668
#define IDS_PAGE_INFO_SECURITY_TAB_ENCRYPTED_INSECURE_CONTENT_ERROR 36669
#define IDS_PAGE_INFO_SECURITY_TAB_ENCRYPTED_INSECURE_CONTENT_WARNING 36670
#define IDS_PAGE_INFO_SECURITY_TAB_ENCRYPTED_INSECURE_FORM_WARNING 36671
#define IDS_PAGE_INFO_SECURITY_TAB_ENCRYPTED_SENTENCE_LINK 36672
#define IDS_PAGE_INFO_SECURITY_TAB_ENCRYPTION_DETAILS 36673
#define IDS_PAGE_INFO_SECURITY_TAB_ENCRYPTION_DETAILS_AEAD 36674
#define IDS_PAGE_INFO_SECURITY_TAB_NOT_ENCRYPTED_CONNECTION_TEXT 36675
#define IDS_PAGE_INFO_SECURITY_TAB_SSL_VERSION 36676
#define IDS_PAGE_INFO_SECURITY_TAB_UNKNOWN_PARTY 36677
#define IDS_PAGE_INFO_SECURITY_TAB_WEAK_ENCRYPTION_CONNECTION_TEXT 36678
#define IDS_PAGE_INFO_CERT_INFO_BUTTON 36679
#define IDS_PAGE_INFO_SECURITY_TAB_SECURE_IDENTITY_EV_VERIFIED 36680
#define IDS_PAGE_INFO_CERTIFICATE_IS_VALID 36681
#define IDS_PAGE_INFO_CERTIFICATE_DETAILS 36682
#define IDS_PAGE_INFO_CERTIFICATE_VALID_LINK_TOOLTIP 36683
#define IDS_PAGE_INFO_CERTIFICATE_INVALID_LINK_TOOLTIP 36684
#define IDS_PAGE_INFO_QWAC_STATUS_TITLE 36685
#define IDS_PAGE_INFO_COOKIES 36687
#define IDS_PAGE_INFO_COOKIES_TOOLTIP 36688
#define IDS_PAGE_INFO_BUTTON_TEXT_AUTOMATIC_BY_DEFAULT 36689
#define IDS_PAGE_INFO_BUTTON_TEXT_MUTED_BY_DEFAULT 36690
#define IDS_PAGE_INFO_STATE_TEXT_ALLOWED 36691
#define IDS_PAGE_INFO_STATE_TEXT_ALLOWED_ONCE 36692
#define IDS_PAGE_INFO_STATE_TEXT_NOT_ALLOWED 36693
#define IDS_PAGE_INFO_STATE_TEXT_NOT_ALLOWED_IN_INCOGNITO 36694
#define IDS_PAGE_INFO_STATE_TEXT_NOT_ALLOWED_IN_GUEST 36695
#define IDS_PAGE_INFO_STATE_TEXT_NOT_ALLOWED_INSECURE 36696
#define IDS_PAGE_INFO_STATE_TEXT_MUTED 36697
#define IDS_PAGE_INFO_STATE_TEXT_ALLOWED_BY_DEFAULT 36698
#define IDS_PAGE_INFO_STATE_TEXT_NOT_ALLOWED_BY_DEFAULT 36699
#define IDS_PAGE_INFO_SELECTOR_TOOLTIP 36700
#define IDS_PAGE_INFO_SELECTOR_STORAGE_ACCESS_TOOLTIP 36701
#define IDS_PAGE_INFO_STATE_TEXT_AR_ASK 36702
#define IDS_PAGE_INFO_STATE_TEXT_AUTO_PICTURE_IN_PICTURE_ASK 36703
#define IDS_PAGE_INFO_STATE_TEXT_AUTOMATIC_DOWNLOADS_ASK 36704
#define IDS_PAGE_INFO_STATE_TEXT_BLUETOOTH_DEVICES_ASK 36705
#define IDS_PAGE_INFO_STATE_TEXT_CAMERA_ASK 36706
#define IDS_PAGE_INFO_STATE_TEXT_CAPTURED_SURFACE_CONTROL_ASK 36707
#define IDS_PAGE_INFO_STATE_TEXT_CAMERA_PAN_TILT_ZOOM_ASK 36708
#define IDS_PAGE_INFO_STATE_TEXT_CLIPBOARD_ASK 36709
#define IDS_PAGE_INFO_STATE_TEXT_FILE_SYSTEM_WRITE_ASK 36710
#define IDS_PAGE_INFO_STATE_TEXT_HAND_TRACKING_ASK 36711
#define IDS_PAGE_INFO_STATE_TEXT_HID_DEVICES_ASK 36712
#define IDS_PAGE_INFO_STATE_TEXT_KEYBOARD_LOCK_ASK 36713
#define IDS_PAGE_INFO_STATE_TEXT_LOCATION_ASK 36714
#define IDS_PAGE_INFO_STATE_TEXT_MIC_ASK 36715
#define IDS_PAGE_INFO_STATE_TEXT_MIDI_SYSEX_ASK 36716
#define IDS_PAGE_INFO_STATE_TEXT_NOTIFICATIONS_ASK 36717
#define IDS_PAGE_INFO_STATE_TEXT_POINTER_LOCK_ASK 36718
#define IDS_PAGE_INFO_STATE_TEXT_MOTION_AND_LIGHT_SENSORS_ASK 36719
#define IDS_PAGE_INFO_STATE_TEXT_MOTION_SENSORS_ASK 36720
#define IDS_PAGE_INFO_STATE_TEXT_USB_ASK 36721
#define IDS_PAGE_INFO_STATE_TEXT_VR_ASK 36722
#define IDS_PAGE_INFO_STATE_TEXT_FONT_ACCESS_ASK 36723
#define IDS_PAGE_INFO_STATE_TEXT_SERIAL_ASK 36724
#define IDS_PAGE_INFO_STATE_TEXT_IDLE_DETECTION_ASK 36725
#define IDS_PAGE_INFO_STATE_TEXT_WINDOW_MANAGEMENT_ASK 36726
#define IDS_PAGE_INFO_STATE_TEXT_BLUETOOTH_SCANNING_ASK 36727
#define IDS_PAGE_INFO_STATE_TEXT_STORAGE_ACCESS_ASK 36728
#define IDS_PAGE_INFO_STATE_TEXT_WEB_APP_INSTALLATION_ASK 36729
#define IDS_PAGE_INFO_STATE_TEXT_WEB_PRINTING_ASK 36730
#define IDS_PAGE_INFO_STATE_TEXT_LOCAL_NETWORK_ASK 36731
#define IDS_PAGE_INFO_STATE_TEXT_LOOPBACK_NETWORK_ASK 36732
#define IDS_PAGE_INFO_USB_DEVICE_SECONDARY_LABEL 36733
#define IDS_PAGE_INFO_USB_DEVICE_ALLOWED_BY_POLICY_LABEL 36734
#define IDS_PAGE_INFO_DELETE_USB_DEVICE_WITH_NAME 36735
#define IDS_PAGE_INFO_SERIAL_PORT_SECONDARY_LABEL 36736
#define IDS_PAGE_INFO_SERIAL_PORT_ALLOWED_BY_POLICY_LABEL 36737
#define IDS_PAGE_INFO_DELETE_SERIAL_PORT_WITH_NAME 36738
#define IDS_PAGE_INFO_BLUETOOTH_DEVICE_SECONDARY_LABEL 36739
#define IDS_PAGE_INFO_DELETE_BLUETOOTH_DEVICE_WITH_NAME 36740
#define IDS_PAGE_INFO_HID_DEVICE_SECONDARY_LABEL 36741
#define IDS_PAGE_INFO_HID_DEVICE_ALLOWED_BY_POLICY_LABEL 36742
#define IDS_PAGE_INFO_DELETE_HID_DEVICE_WITH_NAME 36743
#define IDS_PAGE_INFO_SMART_CARD_READER_ALLOWED_BY_POLICY_LABEL 36744
#define IDS_PAGE_INFO_SMART_CARD_READER_SECONDARY_LABEL 36745
#define IDS_PAGE_INFO_DELETE_SMART_CARD_READER_WITH_NAME 36746
#define IDS_PAGE_INFO_SITE_SETTINGS_LINK 36747
#define IDS_PAGE_INFO_SITE_SETTINGS_TOOLTIP 36748
#define IDS_PAGE_INFO_CAMERA_SYSTEM_SETTINGS_DESCRIPTION 36751
#define IDS_PAGE_INFO_MICROPHONE_SYSTEM_SETTINGS_DESCRIPTION 36752
#define IDS_PAGE_INFO_LOCATION_SYSTEM_SETTINGS_DESCRIPTION 36753
#define IDS_PAGE_INFO_CLIPBOARD_SYSTEM_SETTINGS_DESCRIPTION 36754
#define IDS_PAGE_INFO_SETTINGS_OF_A_SYSTEM_LINK 36755
#define IDS_PAGE_INFO_PERMISSION_ALLOWED_BY_POLICY 36756
#define IDS_PAGE_INFO_PERMISSION_BLOCKED_BY_POLICY 36757
#define IDS_PAGE_INFO_PERMISSION_ASK_BY_POLICY 36758
#define IDS_PAGE_INFO_PERMISSION_ALLOWED_BY_EXTENSION 36759
#define IDS_PAGE_INFO_PERMISSION_BLOCKED_BY_EXTENSION 36760
#define IDS_PAGE_INFO_PERMISSION_ASK_BY_EXTENSION 36761
#define IDS_PAGE_INFO_PERMISSION_AUTOMATICALLY_BLOCKED 36762
#define IDS_PAGE_INFO_PERMISSION_ADS_SUBTITLE 36763
#define IDS_PAGE_INFO_PERMISSION_MANAGED_BY_POLICY 36764
#define IDS_PAGE_INFO_PERMISSION_MANAGED_BY_EXTENSION 36765
#define IDS_PAGE_INFO_INFOBAR_TEXT 36766
#define IDS_PAGE_INFO_INFOBAR_BUTTON 36767
#define IDS_PAGE_INFO_PERMISSION_USING_NOW 36768
#define IDS_PAGE_INFO_PERMISSION_RECENTLY_USED 36769
#define IDS_PAGE_INFO_PERMISSION_USED_TIME_AGO 36770
#define IDS_PAGE_INFO_CHANGE_PASSWORD_DETAILS 36771
#define IDS_PAGE_INFO_CHANGE_PASSWORD_DETAILS_SYNC 36772
#define IDS_PAGE_INFO_CHANGE_PASSWORD_DETAILS_SIGNED_IN_NON_SYNC 36773
#define IDS_PAGE_INFO_CHANGE_PASSWORD_DETAILS_ENTERPRISE 36774
#define IDS_PAGE_INFO_CHANGE_PASSWORD_DETAILS_ENTERPRISE_WITH_ORG_NAME 36775
#define IDS_PAGE_INFO_CHANGE_PASSWORD_BUTTON 36776
#define IDS_PAGE_INFO_PROTECT_ACCOUNT_BUTTON 36777
#define IDS_PAGE_INFO_IGNORE_PASSWORD_WARNING_BUTTON 36778
#define IDS_PAGE_INFO_ALLOWLIST_PASSWORD_REUSE_BUTTON 36779
#define IDS_PAGE_INFO_BILLING_SUMMARY 36780
#define IDS_PAGE_INFO_BILLING_DETAILS 36781
#define IDS_PAGE_INFO_VR_PRESENTING_TEXT 36782
#define IDS_PAGE_INFO_VR_TURN_OFF_BUTTON_TEXT 36783
#define IDS_CERT_INFO_SUBJECT_GROUP 36784
#define IDS_CERT_INFO_ISSUER_GROUP 36785
#define IDS_CERT_INFO_COMMON_NAME_LABEL 36786
#define IDS_CERT_INFO_ORGANIZATION_LABEL 36787
#define IDS_CERT_INFO_ORGANIZATIONAL_UNIT_LABEL 36788
#define IDS_CERT_INFO_SERIAL_NUMBER_LABEL 36789
#define IDS_CERT_INFO_VALIDITY_GROUP 36790
#define IDS_CERT_INFO_ISSUED_ON_LABEL 36791
#define IDS_CERT_INFO_EXPIRES_ON_LABEL 36792
#define IDS_CERT_INFO_FINGERPRINTS_GROUP 36793
#define IDS_CERT_INFO_SHA256_FINGERPRINT_LABEL 36794
#define IDS_CERT_INFO_SHA256_SPKI_FINGERPRINT_LABEL 36795
#define IDS_CERT_DETAILS_EXTENSIONS 36796
#define IDS_CERT_X509_SUBJECT_ALT_NAME 36797
#define IDS_CERT_DETAILS_VERSION 36798
#define IDS_CERT_DETAILS_SERIAL_NUMBER 36799
#define IDS_CERT_DETAILS_ISSUER 36800
#define IDS_CERT_DETAILS_VALIDITY 36801
#define IDS_CERT_DETAILS_NOT_BEFORE 36802
#define IDS_CERT_DETAILS_NOT_AFTER 36803
#define IDS_CERT_DETAILS_SUBJECT 36804
#define IDS_CERT_DETAILS_SUBJECT_KEY_INFO 36805
#define IDS_CERT_GENERAL_NAME_RFC822_NAME 36806
#define IDS_CERT_GENERAL_NAME_DNS_NAME 36807
#define IDS_CERT_GENERAL_NAME_X400_ADDRESS 36808
#define IDS_CERT_GENERAL_NAME_DIRECTORY_NAME 36809
#define IDS_CERT_GENERAL_NAME_EDI_PARTY_NAME 36810
#define IDS_CERT_GENERAL_NAME_URI 36811
#define IDS_CERT_GENERAL_NAME_IP_ADDRESS 36812
#define IDS_CERT_GENERAL_NAME_REGISTERED_ID 36813
#define IDS_CERT_EXTENSION_CRITICAL 36814
#define IDS_CERT_EXTENSION_NON_CRITICAL 36815
#define IDS_CERT_X509_SUBJECT_KEYID 36816
#define IDS_CERT_X509_AUTH_KEYID 36817
#define IDS_CERT_X509_AUTH_INFO_ACCESS 36818
#define IDS_CERT_X509_KEY_USAGE 36819
#define IDS_CERT_X509_ISSUER_ALT_NAME 36820
#define IDS_CERT_X509_BASIC_CONSTRAINTS 36821
#define IDS_CERT_X509_CRL_DIST_POINTS 36822
#define IDS_CERT_X509_CERT_POLICIES 36823
#define IDS_CERT_X509_SCT_LIST 36824
#define IDS_CERT_X509_EXT_KEY_USAGE 36825
#define IDS_CERT_QC_STATEMENTS 36826
#define IDS_CERT_OID_PKCS1_RSA_ENCRYPTION 36827
#define IDS_CERT_OID_PKCS1_MD2_WITH_RSA_ENCRYPTION 36828
#define IDS_CERT_OID_PKCS1_MD4_WITH_RSA_ENCRYPTION 36829
#define IDS_CERT_OID_PKCS1_MD5_WITH_RSA_ENCRYPTION 36830
#define IDS_CERT_OID_PKCS1_SHA1_WITH_RSA_ENCRYPTION 36831
#define IDS_CERT_OID_PKCS1_SHA256_WITH_RSA_ENCRYPTION 36832
#define IDS_CERT_OID_PKCS1_SHA384_WITH_RSA_ENCRYPTION 36833
#define IDS_CERT_OID_PKCS1_SHA512_WITH_RSA_ENCRYPTION 36834
#define IDS_CERT_OID_ANSIX962_ECDSA_SHA1_SIGNATURE 36835
#define IDS_CERT_OID_ANSIX962_ECDSA_SHA256_SIGNATURE 36836
#define IDS_CERT_OID_ANSIX962_ECDSA_SHA384_SIGNATURE 36837
#define IDS_CERT_OID_ANSIX962_ECDSA_SHA512_SIGNATURE 36838
#define IDS_CERT_OID_ANSIX962_EC_PUBLIC_KEY 36839
#define IDS_CERT_OID_SECG_EC_SECP256R1 36840
#define IDS_CERT_OID_SECG_EC_SECP384R1 36841
#define IDS_CERT_OID_SECG_EC_SECP521R1 36842
#define IDS_CERT_OID_ML_DSA_44 36843
#define IDS_CERT_OID_ML_DSA_65 36844
#define IDS_CERT_OID_ML_DSA_87 36845
#define IDS_CERT_EKU_ANY_EKU 36846
#define IDS_CERT_EKU_CODE_SIGNING 36847
#define IDS_CERT_EKU_EMAIL_PROTECTION 36848
#define IDS_CERT_EKU_TIME_STAMPING 36849
#define IDS_CERT_EKU_TLS_WEB_SERVER_AUTHENTICATION 36850
#define IDS_CERT_EKU_TLS_WEB_CLIENT_AUTHENTICATION 36851
#define IDS_CERT_EKU_OCSP_SIGNING 36852
#define IDS_CERT_X509_KEY_USAGE_SIGNING 36853
#define IDS_CERT_X509_KEY_USAGE_NONREP 36854
#define IDS_CERT_X509_KEY_USAGE_ENCIPHERMENT 36855
#define IDS_CERT_X509_KEY_USAGE_DATA_ENCIPHERMENT 36856
#define IDS_CERT_X509_KEY_USAGE_KEY_AGREEMENT 36857
#define IDS_CERT_X509_KEY_USAGE_CERT_SIGNER 36858
#define IDS_CERT_X509_KEY_USAGE_CRL_SIGNER 36859
#define IDS_CERT_X509_KEY_USAGE_ENCIPHER_ONLY 36860
#define IDS_CERT_X509_KEY_USAGE_DECIPHER_ONLY 36861
#define IDS_CERT_PKIX_CPS_POINTER_QUALIFIER 36862
#define IDS_CERT_PKIX_USER_NOTICE_QUALIFIER 36863
#define IDS_CERT_QC_ETSI_QCS_QCCOMPLIANCE 36864
#define IDS_CERT_QC_ETSI_QCS_QCTYPE 36865
#define IDS_CERT_QC_ETSI_QCT_WEB 36866
#define IDS_PAGE_INFO_SECURITY_SUBPAGE_BUTTON 36867
#define IDS_PAGE_INFO_SECURITY_SUBPAGE_HEADER 36868
#define IDS_PAGE_INFO_PERMISSIONS_SUBPAGE_BUTTON_TOOLTIP 36869
#define IDS_PAGE_INFO_PERMISSIONS_SUBPAGE_MANAGE_BUTTON 36870
#define IDS_PAGE_INFO_PERMISSIONS_SUBPAGE_MANAGE_BUTTON_TOOLTIP 36871
#define IDS_PAGE_INFO_PERMISSIONS_SUBPAGE_REMEMBER_THIS_SETTING 36872
#define IDS_PAGE_INFO_RESET_PERMISSIONS 36873
#define IDS_PAGE_INFO_ABOUT_THIS_PAGE_TITLE 36874
#define IDS_PAGE_INFO_ABOUT_THIS_PAGE_DESCRIPTION_PLACEHOLDER 36875
#define IDS_PAGE_INFO_ABOUT_THIS_PAGE_TOOLTIP 36876
#define IDS_PAGE_INFO_COOKIES_HEADER 36877
#define IDS_PAGE_INFO_COOKIES_DESCRIPTION 36878
#define IDS_PAGE_INFO_COOKIES_SETTINGS_LINK 36879
#define IDS_PAGE_INFO_BLOCK_THIRD_PARTY_COOKIES_MANAGED_BY_SETTINGS_TOOLTIP 36880
#define IDS_PAGE_INFO_COOKIES_ALLOWED_SITES_COUNT 36881
#define IDS_PAGE_INFO_COOKIES_DIALOG_BUTTON_TITLE 36882
#define IDS_PAGE_INFO_COOKIES_DIALOG_BUTTON_TOOLTIP 36883
#define IDS_PAGE_INFO_RWS_BUTTON_TOOLTIP 36884
#define IDS_PAGE_INFO_RWS_BUTTON_TITLE 36885
#define IDS_PAGE_INFO_RWS_BUTTON_SUBTITLE 36886
#define IDS_PAGE_INFO_COOKIES_SITE_NOT_WORKING_TITLE 36887
#define IDS_PAGE_INFO_TRACKING_PROTECTION_SITE_NOT_WORKING_DESCRIPTION_TEMPORARY 36888
#define IDS_PAGE_INFO_COOKIES_SITE_NOT_WORKING_DESCRIPTION_PERMANENT 36889
#define IDS_PAGE_INFO_COOKIES_THIRD_PARTY_COOKIES_LABEL 36890
#define IDS_PAGE_INFO_TRACKING_PROTECTION_COOKIES_BLOCKED_RESTART_TITLE 36891
#define IDS_PAGE_INFO_TRACKING_PROTECTION_COOKIES_RESTART_DESCRIPTION 36892
#define IDS_PAGE_INFO_TRACKING_PROTECTION_PERMANENT_ALLOWED_TITLE 36893
#define IDS_PAGE_INFO_TRACKING_PROTECTION_COOKIES_PERMANENT_ALLOWED_DESCRIPTION 36894
#define IDS_PAGE_INFO_COOKIE_SYNC_DESCRIPTION 36895
#define IDS_PAGE_INFO_SYNC_SETTINGS_LINK 36896
#define IDS_MERCHANT_TRUST_SIDE_PANEL_TITLE 36897
#define IDS_PAGE_INFO_MERCHANT_TRUST_HEADER 36898
#define IDS_PAGE_INFO_MERCHANT_TRUST_DESCRIPTION 36899
#define IDS_PAGE_INFO_MERCHANT_TRUST_VIEW_ALL_REVIEWS 36900
#define IDS_PAGE_INFO_MERCHANT_TRUST_STAR_RATING_AND_COUNT_A11Y_DESCRIPTION 36901
#define IDS_PAGE_INFO_MERCHANT_TRUST_STAR_RATING_A11Y_DESCRIPTION 36902
#define IDS_PAGE_INFO_MERCHANT_TRUST_REVIEWS_SUMMARY_TITLE 36903
#define IDS_PAGE_INFO_HISTORY 36904
#define IDS_PAGE_INFO_HISTORY_LAST_VISIT_TODAY 36905
#define IDS_PAGE_INFO_HISTORY_LAST_VISIT_YESTERDAY 36906
#define IDS_PAGE_INFO_HISTORY_LAST_VISIT_DAYS 36907
#define IDS_PAGE_INFO_HISTORY_LAST_VISIT_DATE 36908
#define IDS_PAGE_INFO_ENTERPRISE_WARN_SUMMARY 36909
#define IDS_PAGE_INFO_ENTERPRISE_WARN_DETAILS 36910
#define IDS_PAGE_INFO_ENTERPRISE_BLOCK_SUMMARY 36911
#define IDS_PAGE_INFO_ENTERPRISE_BLOCK_DETAILS 36912
#define IDS_PAGE_INFO_AD_PRIVACY_HEADER 36913
#define IDS_PAGE_INFO_AD_PRIVACY_TOOLTIP 36914
#define IDS_PAGE_INFO_AD_PRIVACY_TOPICS_DESCRIPTION 36915
#define IDS_PAGE_INFO_AD_PRIVACY_FLEDGE_DESCRIPTION 36916
#define IDS_PAGE_INFO_AD_PRIVACY_TOPICS_AND_FLEDGE_DESCRIPTION 36917
#define IDS_PAGE_INFO_AD_PRIVACY_SUBPAGE_MANAGE_BUTTON 36918
#define IDS_PAINT_PREVIEW_COMPOSITOR_SERVICE_DISPLAY_NAME 36919
#define IDS_LEAK_CHECK_CREDENTIALS 36920
#define IDS_CREDENTIAL_LEAK_TITLE_CHANGE 36921
#define IDS_CREDENTIAL_LEAK_TITLE_CHECK_GPM 36922
#define IDS_CREDENTIAL_LEAK_CHECK_PASSWORDS_MESSAGE_GPM_BRANDED 36923
#define IDS_CREDENTIAL_LEAK_CHANGE_PASSWORD_MESSAGE_GPM_BRANDED 36924
#define IDS_CREDENTIAL_LEAK_CHANGE_AND_CHECK_PASSWORDS_MESSAGE_GPM_BRANDED 36925
#define IDS_CREDENTIAL_LEAK_CHECK_PASSWORDS_MESSAGE_GPM_NON_BRANDED 36926
#define IDS_CREDENTIAL_LEAK_CHANGE_PASSWORD_MESSAGE_GPM_NON_BRANDED 36927
#define IDS_CREDENTIAL_LEAK_CHANGE_AND_CHECK_PASSWORDS_MESSAGE_GPM_NON_BRANDED 36928
#define IDS_PASSWORD_MANAGER_LEAK_HELP_MESSAGE 36931
#define IDS_MANAGE_PASSWORDS_AUTO_SIGNIN_TITLE 36932
#define IDS_PASSWORD_MANAGER_EMPTY_LOGIN 36933
#define IDS_PASSWORD_MANAGER_MANAGE_PASSWORDS 36934
#define IDS_PASSWORD_MANAGER_MANAGE_PASSWORDS_AND_PASSKEYS 36935
#define IDS_PASSWORD_MANAGER_GENERATE_PASSWORD 36936
#define IDS_PASSWORD_MANAGER_SHOW_PASSWORDS_TAB_TITLE 36937
#define IDS_PASSWORD_MANAGER_DEFAULT_EXPORT_FILENAME 36938
#define IDS_PASSWORD_MANAGER_PASSWORD_FOR_ACCOUNT 36939
#define IDS_PASSWORD_MANAGER_USE_SCREEN_LOCK 36940
#define IDS_PASSWORD_MANAGER_PASSKEY 36941
#define IDS_PASSWORD_MANAGER_USE_GENERIC_DEVICE 36942
#define IDS_PASSWORD_MANAGER_PASSKEY_FROM_CHROME_PROFILE 36943
#define IDS_PASSWORD_MANAGER_PASSKEY_FROM_ICLOUD_KEYCHAIN 36944
#define IDS_PASSWORD_MANAGER_PASSKEY_FROM_GOOGLE_PASSWORD_MANAGER 36945
#define IDS_PASSWORD_MANAGER_PASSKEY_FROM_WINDOWS_HELLO 36946
#define IDS_PASSWORD_MANAGER_PASSKEY_FROM_PROVIDER 36947
#define IDS_PASSWORD_MANAGER_PASSWORD_FROM_GOOGLE_PASSWORD_MANAGER 36948
#define IDS_PASSWORD_MANAGER_USE_DIFFERENT_PASSKEY 36949
#define IDS_PASSWORD_MANAGER_USE_PASSKEY_OTHER_DEVICE 36950
#define IDS_PASSWORD_MANAGER_USE_PASSKEY 36951
#define IDS_PASSWORD_MANAGER_RELAUNCH_CHROME_BUBBLE_TITLE_LINUX_BRANDED 36976
#define IDS_PASSWORD_MANAGER_RELAUNCH_CHROME_BUBBLE_DESCRIPTION_LINUX_BRANDED 36977
#define IDS_PASSWORD_MANAGER_RELAUNCH_CHROME_BUBBLE_TITLE_LINUX_NON_BRANDED 36978
#define IDS_PASSWORD_MANAGER_RELAUNCH_CHROME_BUBBLE_DESCRIPTION_LINUX_NON_BRANDED 36979
#define IDS_PASSWORD_MANAGER_RELAUNCH_CHROME_BUBBLE_ACCEPT_BUTTON_BRANDED 36983
#define IDS_PASSWORD_MANAGER_RELAUNCH_CHROME_BUBBLE_ACCEPT_BUTTON_NON_BRANDED 36984
#define IDS_PASSWORD_MANAGER_RELAUNCH_CHROME_BUBBLE_CANCEL_BUTTON 36985
#define IDS_PASSWORD_MANAGER_RELAUNCH_CHROME_BUBBLE_NEVER_BUTTON 36986
#define IDS_PASSWORD_MANAGER_MANAGEMENT_BUBBLE_FOOTER_MOVE_PASSWORD 36987
#define IDS_PASSWORD_MANAGER_MANAGEMENT_BUBBLE_LINK_TEXT_MOVE_PASSWORD 36988
#define IDS_PASSWORD_MANAGER_MANUAL_FALLBACK_SUGGESTED_PASSWORDS_SECTION_TITLE 36989
#define IDS_PASSWORD_MANAGER_MANUAL_FALLBACK_ALL_PASSWORDS_SECTION_TITLE 36990
#define IDS_PASSWORD_MANAGER_MANUAL_FALLBACK_FILL_PASSWORD_ENTRY 36991
#define IDS_PASSWORD_MANAGER_MANUAL_FALLBACK_VIEW_DETAILS_ENTRY 36992
#define IDS_PASSWORD_MANAGER_PENDING_STATE 36993
#define IDS_PASSWORD_MANAGER_UI_TROUBLE_SIGNING_IN 36994
#define IDS_PASSWORD_MANAGER_UI_PROACTIVE_RECOVERY_SUGGESTION 36995
#define IDS_PASSWORD_MANAGER_UI_PROACTIVE_RECOVERY_FOOTER_BRANDED 36996
#define IDS_PASSWORD_MANAGER_UI_PROACTIVE_RECOVERY_FOOTER_NON_BRANDED 36997
#define IDS_PASSWORD_MANAGER_UI_BACKUP_PASSWORD_TAG 36998
#define IDS_PAYMENTS_TITLE 36999
#define IDS_PAYMENTS_ERROR_MESSAGE_DIALOG_TITLE 37000
#define IDS_PAYMENTS_METHOD_OF_PAYMENT_LABEL 37001
#define IDS_PAYMENTS_CONTACT_DETAILS_LABEL 37002
#define IDS_PAYMENTS_ADD_CONTACT_DETAILS_LABEL 37003
#define IDS_PAYMENTS_EDIT_CONTACT_DETAILS_LABEL 37004
#define IDS_PAYMENTS_ADD_MORE_INFORMATION 37005
#define IDS_PAYMENTS_ADD_PHONE_NUMBER 37006
#define IDS_PAYMENTS_ADD_RECIPIENT 37007
#define IDS_PAYMENTS_ADD_VALID_ADDRESS 37008
#define IDS_PAYMENTS_ADD_EMAIL 37009
#define IDS_PAYMENTS_ADD_NAME 37010
#define IDS_PAYMENTS_ORDER_SUMMARY_LABEL 37011
#define IDS_PAYMENT_REQUEST_PAYMENT_METHOD_SECTION_NAME 37012
#define IDS_PAYMENT_REQUEST_CONTACT_INFO_SECTION_NAME 37013
#define IDS_PAYMENTS_SHIPPING_SUMMARY_LABEL 37014
#define IDS_PAYMENTS_SHIPPING_ADDRESS_LABEL 37015
#define IDS_PAYMENTS_SHIPPING_OPTION_LABEL 37016
#define IDS_PAYMENTS_DELIVERY_SUMMARY_LABEL 37017
#define IDS_PAYMENTS_DELIVERY_ADDRESS_LABEL 37018
#define IDS_PAYMENTS_DELIVERY_OPTION_LABEL 37019
#define IDS_PAYMENTS_PICKUP_SUMMARY_LABEL 37020
#define IDS_PAYMENTS_PICKUP_ADDRESS_LABEL 37021
#define IDS_PAYMENTS_PICKUP_OPTION_LABEL 37022
#define IDS_PAYMENTS_EDIT_BUTTON 37023
#define IDS_PAYMENTS_PAY_BUTTON 37024
#define IDS_PAYMENTS_CONTINUE_BUTTON 37025
#define IDS_PAYMENTS_ADD_CONTACT 37026
#define IDS_PAYMENTS_ADD_CARD 37027
#define IDS_PAYMENTS_ADD_ADDRESS 37028
#define IDS_PAYMENTS_EDIT_ADDRESS 37029
#define IDS_PAYMENTS_CANCEL_PAYMENT 37030
#define IDS_PAYMENTS_CONFIRM_BUTTON 37031
#define IDS_PAYMENTS_NAME_FIELD_IN_CONTACT_DETAILS 37032
#define IDS_PAYMENTS_PHONE_FIELD_IN_CONTACT_DETAILS 37033
#define IDS_PAYMENTS_EMAIL_FIELD_IN_CONTACT_DETAILS 37034
#define IDS_PAYMENTS_LOADING_MESSAGE 37035
#define IDS_PAYMENTS_PROCESSING_MESSAGE 37036
#define IDS_PAYMENTS_CHECKING_OPTION 37037
#define IDS_PAYMENTS_UPDATED_LABEL 37038
#define IDS_PAYMENTS_ERROR_MESSAGE 37039
#define IDS_PAYMENTS_CARD_AND_ADDRESS_SETTINGS 37040
#define IDS_PAYMENTS_CARD_AND_ADDRESS_SETTINGS_SIGNED_IN 37041
#define IDS_PAYMENTS_CARD_AND_ADDRESS_SETTINGS_SIGNED_OUT 37042
#define IDS_SETTINGS_CAN_MAKE_PAYMENT_TOGGLE_LABEL 37043
#define IDS_PAYMENTS_REQUIRED_FIELD_MESSAGE 37044
#define IDS_PAYMENTS_VALIDATION_INVALID_CREDIT_CARD_EXPIRED 37045
#define IDS_PAYMENTS_VALIDATION_UNSUPPORTED_CREDIT_CARD_TYPE 37046
#define IDS_PAYMENTS_PHONE_INVALID_VALIDATION_MESSAGE 37047
#define IDS_PAYMENTS_EMAIL_INVALID_VALIDATION_MESSAGE 37048
#define IDS_PAYMENTS_CARD_NUMBER_INVALID_VALIDATION_MESSAGE 37049
#define IDS_PAYMENTS_INVALID_ADDRESS 37050
#define IDS_PAYMENTS_BILLING_ADDRESS_REQUIRED 37051
#define IDS_PAYMENTS_MORE_INFORMATION_REQUIRED 37052
#define IDS_PAYMENTS_PHONE_NUMBER_REQUIRED 37053
#define IDS_PAYMENTS_RECIPIENT_REQUIRED 37054
#define IDS_PAYMENTS_EMAIL_REQUIRED 37055
#define IDS_PAYMENTS_NAME_REQUIRED 37056
#define IDS_PREF_EDIT_DIALOG_FIELD_REQUIRED_VALIDATION_MESSAGE 37057
#define IDS_PAYMENT_REQUEST_ORDER_SUMMARY_SECTION_TOTAL_FORMAT 37058
#define IDS_PAYMENT_REQUEST_ORDER_SUMMARY_SHEET_TOTAL_FORMAT 37059
#define IDS_PAYMENT_REQUEST_ORDER_SUMMARY_MORE_ITEMS 37060
#define IDS_PAYMENT_REQUEST_ORDER_SUMMARY_MULTIPLE_CURRENCY_INDICATOR 37061
#define IDS_PAYMENTS_SELECT_SHIPPING_ADDRESS_FOR_SHIPPING_METHODS 37062
#define IDS_PAYMENTS_UNSUPPORTED_SHIPPING_ADDRESS 37063
#define IDS_PAYMENTS_UNSUPPORTED_SHIPPING_OPTION 37064
#define IDS_PAYMENTS_SELECT_DELIVERY_ADDRESS_FOR_DELIVERY_METHODS 37065
#define IDS_PAYMENTS_UNSUPPORTED_DELIVERY_ADDRESS 37066
#define IDS_PAYMENTS_SELECT_PICKUP_ADDRESS_FOR_PICKUP_METHODS 37067
#define IDS_PAYMENTS_UNSUPPORTED_PICKUP_ADDRESS 37068
#define IDS_PAYMENTS_ANDROID_APP_ERROR 37069
#define IDS_PAYMENT_REQUEST_PAYMENT_METHODS_PREVIEW 37071
#define IDS_PAYMENT_REQUEST_SHIPPING_ADDRESSES_PREVIEW 37072
#define IDS_PAYMENT_REQUEST_SHIPPING_OPTIONS_PREVIEW 37073
#define IDS_PAYMENT_REQUEST_CONTACTS_PREVIEW 37074
#define IDS_PAYMENTS_BACK 37075
#define IDS_PAYMENTS_CLOSE 37076
#define IDS_PAYMENTS_EDIT 37077
#define IDS_PAYMENTS_ROW_ACCESSIBLE_NAME_FORMAT 37078
#define IDS_PAYMENTS_ROW_ACCESSIBLE_NAME_SELECTED_FORMAT 37079
#define IDS_PAYMENTS_PROFILE_LABELS_ACCESSIBLE_FORMAT 37080
#define IDS_PAYMENTS_ACCESSIBLE_LABEL_WITH_ERROR 37081
#define IDS_PAYMENTS_ORDER_SUMMARY_ACCESSIBLE_LABEL 37082
#define IDS_PAYMENTS_ITEM_ICON 37083
#define IDS_PAYMENT_HANDLER_SHEET_DESCRIPTION 37084
#define IDS_PAYMENT_HANDLER_SHEET_OPENED_HALF 37085
#define IDS_PAYMENT_HANDLER_SHEET_OPENED_FULL 37086
#define IDS_PAYMENT_HANDLER_SHEET_CLOSED 37087
#define IDS_PAYMENT_HANDLER_ICON 37088
#define IDS_SECURE_PAYMENT_CONFIRMATION_TITLE 37089
#define IDS_SECURE_PAYMENT_CONFIRMATION_INFORM_ONLY_TITLE 37090
#define IDS_SECURE_PAYMENT_CONFIRMATION_VERIFY_BUTTON_LABEL 37091
#define IDS_SECURE_PAYMENT_CONFIRMATION_FOOTNOTE_TEXT 37096
#define IDS_SECURE_PAYMENT_CONFIRMATION_FOOTNOTE_LINK_TEXT 37097
#define IDS_SECURE_PAYMENT_CONFIRMATION_OPT_OUT_TEXT 37098
#define IDS_SECURE_PAYMENT_CONFIRMATION_OPT_OUT_LINK_TEXT 37099
#define IDS_SECURE_PAYMENT_CONFIRMATION_CONFIRM_BUTTON_LABEL 37100
#define IDS_SECURE_PAYMENT_CONFIRMATION_OPT_OUT_AUTHENTICATOR_TEXT 37101
#define IDS_PDF_DOWNLOAD_ORIGINAL 37109
#define IDS_PDF_DOWNLOAD_EDITED 37110
#define IDS_PDF_PRESENT 37111
#define IDS_PDF_NEED_PASSWORD 37112
#define IDS_PDF_PASSWORD_DIALOG_TITLE 37113
#define IDS_PDF_PASSWORD_SUBMIT 37114
#define IDS_PDF_PASSWORD_INVALID 37115
#define IDS_PDF_SIDEBAR_LABEL 37116
#define IDS_PDF_PAGE_LOADING 37117
#define IDS_PDF_ERROR_DIALOG_TITLE 37118
#define IDS_PDF_PAGE_LOAD_FAILED 37119
#define IDS_PDF_PAGE_RELOAD_BUTTON 37120
#define IDS_PDF_BOOKMARKS 37121
#define IDS_PDF_BOOKMARK_EXPAND_ICON_ARIA_LABEL 37122
#define IDS_PDF_PROPERTIES_DIALOG_TITLE 37123
#define IDS_PDF_PROPERTIES_FILE_NAME 37124
#define IDS_PDF_PROPERTIES_FILE_SIZE 37125
#define IDS_PDF_PROPERTIES_TITLE 37126
#define IDS_PDF_PROPERTIES_AUTHOR 37127
#define IDS_PDF_PROPERTIES_SUBJECT 37128
#define IDS_PDF_PROPERTIES_KEYWORDS 37129
#define IDS_PDF_PROPERTIES_CREATED 37130
#define IDS_PDF_PROPERTIES_MODIFIED 37131
#define IDS_PDF_PROPERTIES_APPLICATION 37132
#define IDS_PDF_PROPERTIES_PDF_PRODUCER 37133
#define IDS_PDF_PROPERTIES_PDF_VERSION 37134
#define IDS_PDF_PROPERTIES_PAGE_COUNT 37135
#define IDS_PDF_PROPERTIES_PAGE_SIZE 37136
#define IDS_PDF_PROPERTIES_PAGE_SIZE_VALUE_INCH 37137
#define IDS_PDF_PROPERTIES_PAGE_SIZE_VALUE_MM 37138
#define IDS_PDF_PROPERTIES_PAGE_SIZE_PORTRAIT 37139
#define IDS_PDF_PROPERTIES_PAGE_SIZE_LANDSCAPE 37140
#define IDS_PDF_PROPERTIES_PAGE_SIZE_SQUARE 37141
#define IDS_PDF_PROPERTIES_PAGE_SIZE_VARIABLE 37142
#define IDS_PDF_PROPERTIES_FAST_WEB_VIEW 37143
#define IDS_PDF_PROPERTIES_FAST_WEB_VIEW_NO 37144
#define IDS_PDF_PROPERTIES_FAST_WEB_VIEW_YES 37145
#define IDS_PDF_TOOLTIP_ROTATE_CCW 37146
#define IDS_PDF_ROTATION_STATE_LABEL_0 37147
#define IDS_PDF_ROTATION_STATE_LABEL_90 37148
#define IDS_PDF_ROTATION_STATE_LABEL_180 37149
#define IDS_PDF_ROTATION_STATE_LABEL_270 37150
#define IDS_PDF_TOOLTIP_DOWNLOAD 37151
#define IDS_PDF_TOOLTIP_PRINT 37152
#define IDS_PDF_TOOLTIP_FIT_PAGE 37153
#define IDS_PDF_TOOLTIP_FIT_WIDTH 37154
#define IDS_PDF_TWO_UP_VIEW_ENABLE 37155
#define IDS_PDF_ANNOTATIONS_SHOW_TOGGLE 37156
#define IDS_PDF_ZOOM_TEXT_INPUT_ARIA_LABEL 37157
#define IDS_PDF_TOOLTIP_ZOOM_IN 37158
#define IDS_PDF_TOOLTIP_ZOOM_OUT 37159
#define IDS_PDF_TOOLTIP_THUMBNAILS 37160
#define IDS_PDF_TOOLTIP_DOCUMENT_OUTLINE 37161
#define IDS_PDF_TOOLTIP_ATTACHMENTS 37162
#define IDS_PDF_TOOLTIP_DOWNLOAD_ATTACHMENT 37163
#define IDS_PDF_LABEL_PAGE_NUMBER 37164
#define IDS_PDF_LOADING_TO_A11Y_TREE 37165
#define IDS_PDF_LOADED_TO_A11Y_TREE 37166
#define IDS_PDF_OVERSIZE_ATTACHMENT_WARNING 37167
#define IDS_PDF_PAGE_INDEX 37168
#define IDS_PDF_DOCUMENT_PAGE_COUNT 37169
#define IDS_PDF_THUMBNAIL_PAGE_ARIA_LABEL 37170
#define IDS_PDF_ANNOTATION_PEN 37171
#define IDS_PDF_ANNOTATION_HIGHLIGHTER 37172
#define IDS_PDF_ANNOTATION_ERASER 37173
#define IDS_PDF_ANNOTATION_UNDO 37174
#define IDS_PDF_ANNOTATION_REDO 37175
#define IDS_PDF_ANNOTATION_COLOR_BLACK 37176
#define IDS_PDF_ANNOTATION_COLOR_RED 37177
#define IDS_PDF_ANNOTATION_COLOR_YELLOW 37178
#define IDS_PDF_ANNOTATION_COLOR_GREEN 37179
#define IDS_PDF_ANNOTATION_COLOR_WHITE 37180
#define IDS_PDF_ANNOTATION_COLOR_ORANGE 37181
#define IDS_PDF_ANNOTATION_COLOR_BLUE 37182
#define IDS_PDF_ANNOTATION_COLOR_LIGHT_GREY 37183
#define IDS_PDF_ANNOTATION_COLOR_LIGHT_ORANGE 37184
#define IDS_PDF_ANNOTATION_COLOR_LIGHT_GREEN 37185
#define IDS_PDF_ANNOTATION_COLOR_LIGHT_BLUE 37186
#define IDS_PDF_INK2_DRAW 37187
#define IDS_PDF_INK2_ANNOTATION_TOOL 37188
#define IDS_PDF_INK2_ANNOTATION_SIZE 37189
#define IDS_PDF_INK2_ANNOTATION_COLOR 37190
#define IDS_PDF_INK2_ANNOTATION_SIZE_EXTRA_THIN 37191
#define IDS_PDF_INK2_ANNOTATION_SIZE_THIN 37192
#define IDS_PDF_INK2_ANNOTATION_SIZE_MEDIUM 37193
#define IDS_PDF_INK2_ANNOTATION_SIZE_THICK 37194
#define IDS_PDF_INK2_ANNOTATION_SIZE_EXTRA_THICK 37195
#define IDS_PDF_INK2_ANNOTATION_COLOR_LIGHT_RED 37196
#define IDS_PDF_INK2_ANNOTATION_COLOR_LIGHT_YELLOW 37197
#define IDS_PDF_INK2_ANNOTATION_COLOR_DARK_GREY_1 37198
#define IDS_PDF_INK2_ANNOTATION_COLOR_DARK_GREY_2 37199
#define IDS_PDF_INK2_ANNOTATION_COLOR_RED_1 37200
#define IDS_PDF_INK2_ANNOTATION_COLOR_YELLOW_1 37201
#define IDS_PDF_INK2_ANNOTATION_COLOR_GREEN_1 37202
#define IDS_PDF_INK2_ANNOTATION_COLOR_CYAN_1 37203
#define IDS_PDF_INK2_ANNOTATION_COLOR_BLUE_1 37204
#define IDS_PDF_INK2_ANNOTATION_COLOR_TAN_1 37205
#define IDS_PDF_INK2_ANNOTATION_COLOR_RED_2 37206
#define IDS_PDF_INK2_ANNOTATION_COLOR_YELLOW_2 37207
#define IDS_PDF_INK2_ANNOTATION_COLOR_GREEN_2 37208
#define IDS_PDF_INK2_ANNOTATION_COLOR_CYAN_2 37209
#define IDS_PDF_INK2_ANNOTATION_COLOR_BLUE_2 37210
#define IDS_PDF_INK2_ANNOTATION_COLOR_TAN_2 37211
#define IDS_PDF_INK2_ANNOTATION_COLOR_RED_3 37212
#define IDS_PDF_INK2_ANNOTATION_COLOR_YELLOW_3 37213
#define IDS_PDF_INK2_ANNOTATION_COLOR_GREEN_3 37214
#define IDS_PDF_INK2_ANNOTATION_COLOR_CYAN_3 37215
#define IDS_PDF_INK2_ANNOTATION_COLOR_BLUE_3 37216
#define IDS_PDF_INK2_ANNOTATION_COLOR_TAN_3 37217
#define IDS_PDF_INK2_TEXT_ANNOTATION 37218
#define IDS_PDF_INK2_TEXT_FONT 37219
#define IDS_PDF_INK2_TEXT_FONT_SANS_SERIF 37220
#define IDS_PDF_INK2_TEXT_FONT_SERIF 37221
#define IDS_PDF_INK2_TEXT_FONT_MONOSPACE 37222
#define IDS_PDF_INK2_TEXT_FONT_SIZE 37223
#define IDS_PDF_INK2_TEXT_STYLES 37224
#define IDS_PDF_INK2_TEXT_STYLE_BOLD 37225
#define IDS_PDF_INK2_TEXT_STYLE_ITALIC 37226
#define IDS_PDF_INK2_TEXT_ALIGNMENT 37227
#define IDS_PDF_INK2_TEXT_ALIGN_LEFT 37228
#define IDS_PDF_INK2_TEXT_ALIGN_CENTER 37229
#define IDS_PDF_INK2_TEXT_ALIGN_RIGHT 37230
#define IDS_PDF_INK2_TEXT_COLOR 37231
#define IDS_AX_ROLE_DESCRIPTION_PDF_HIGHLIGHT 37232
#define IDS_AX_ROLE_DESCRIPTION_PDF_POPUP_NOTE 37233
#define IDS_PDF_TOOLTIP_SAVE_TO_DRIVE 37234
#define IDS_SAVE_TO_DRIVE_DIALOG_CANCEL_UPLOAD_BUTTON_LABEL 37235
#define IDS_SAVE_TO_DRIVE_DIALOG_CONNECTION_ERROR_MESSAGE 37236
#define IDS_SAVE_TO_DRIVE_DIALOG_ERROR_TITLE 37237
#define IDS_SAVE_TO_DRIVE_DIALOG_MANAGE_STORAGE_BUTTON_LABEL 37238
#define IDS_SAVE_TO_DRIVE_DIALOG_OPEN_IN_DRIVE_BUTTON_LABEL 37239
#define IDS_SAVE_TO_DRIVE_DIALOG_RETRY_BUTTON_LABEL 37240
#define IDS_SAVE_TO_DRIVE_DIALOG_SESSION_TIMEOUT_ERROR_MESSAGE 37241
#define IDS_SAVE_TO_DRIVE_DIALOG_STORAGE_FULL_ERROR_MESSAGE 37242
#define IDS_SAVE_TO_DRIVE_DIALOG_SUCCESS_MESSAGE 37243
#define IDS_SAVE_TO_DRIVE_DIALOG_SUCCESS_TITLE 37244
#define IDS_SAVE_TO_DRIVE_DIALOG_UNKNOWN_ERROR_MESSAGE 37245
#define IDS_SAVE_TO_DRIVE_DIALOG_UPLOAD_STATUS_DONE 37246
#define IDS_SAVE_TO_DRIVE_DIALOG_UPLOAD_STATUS_WITH_SEPARATOR 37247
#define IDS_SAVE_TO_DRIVE_DIALOG_UPLOADING_TITLE 37248
#define IDS_PDF_OCR_FEATURE_ALERT 37249
#define IDS_PDF_OCR_IN_PROGRESS 37250
#define IDS_PDF_OCR_COMPLETED 37251
#define IDS_PDF_OCR_NO_RESULT 37252
#define IDS_PDF_OCR_POSTAMBLE_PAGE 37253
#define IDS_PDF_SEARCHIFY_IN_PROGRESS 37254
#define IDS_GEOLOCATION_INFOBAR_TEXT 37255
#define IDS_GEOLOCATION_INFOBAR_PERMISSION_FRAGMENT 37295
#define IDS_MOTION_AND_LIGHT_SENSORS_PERMISSION_FRAGMENT 37296
#define IDS_MOTION_SENSORS_PERMISSION_FRAGMENT 37297
#define IDS_NOTIFICATION_PERMISSIONS_FRAGMENT 37298
#define IDS_MIDI_SYSEX_PERMISSION_FRAGMENT 37299
#define IDS_MEDIA_CAPTURE_AUDIO_ONLY_PERMISSION_FRAGMENT 37300
#define IDS_MEDIA_CAPTURE_AUDIO_ONLY_PERMISSION_FRAGMENT_WITH_COUNT 37301
#define IDS_MEDIA_CAPTURE_VIDEO_ONLY_PERMISSION_FRAGMENT 37302
#define IDS_MEDIA_CAPTURE_VIDEO_ONLY_PERMISSION_FRAGMENT_WITH_COUNT 37303
#define IDS_MEDIA_CAPTURE_CAMERA_PAN_TILT_ZOOM_PERMISSION_FRAGMENT 37304
#define IDS_MEDIA_CAPTURE_CAMERA_PAN_TILT_ZOOM_PERMISSION_FRAGMENT_WITH_COUNT 37305
#define IDS_ACCESSIBILITY_EVENTS_PERMISSION_FRAGMENT 37306
#define IDS_CLIPBOARD_PERMISSION_FRAGMENT 37307
#define IDS_VR_PERMISSION_FRAGMENT 37308
#define IDS_AR_PERMISSION_FRAGMENT 37309
#define IDS_HAND_TRACKING_PERMISSION_FRAGMENT 37310
#define IDS_KEYBOARD_LOCK_PERMISSIONS_FRAGMENT 37311
#define IDS_POINTER_LOCK_PERMISSIONS_FRAGMENT 37312
#define IDS_STORAGE_ACCESS_PERMISSION_TWO_ORIGIN_PROMPT_TITLE 37313
#define IDS_STORAGE_ACCESS_PERMISSION_FRAGMENT 37314
#define IDS_STORAGE_ACCESS_PERMISSION_EXPLANATION 37315
#define IDS_STORAGE_ACCESS_PERMISSION_TWO_ORIGIN_EXPLANATION 37316
#define IDS_STORAGE_ACCESS_PERMISSION_TWO_ORIGIN_LINK 37317
#define IDS_WINDOW_MANAGEMENT_PERMISSION_FRAGMENT 37318
#define IDS_FONT_ACCESS_PERMISSION_FRAGMENT 37319
#define IDS_IDLE_DETECTION_PERMISSION_FRAGMENT 37320
#define IDS_IDENTITY_PROVIDER_PERMISSION_FRAGMENT 37321
#define IDS_MULTI_DOWNLOAD_PERMISSION_FRAGMENT 37322
#define IDS_CAPTURED_SURFACE_CONTROL_PERMISSION_FRAGMENT 37323
#define IDS_WEB_APP_INSTALLATION_PERMISSION_FRAGMENT 37324
#define IDS_LOCAL_NETWORK_PERMISSION_FRAGMENT 37325
#define IDS_LOOPBACK_NETWORK_PERMISSION_FRAGMENT 37326
#define IDS_PERMISSION_ALLOW 37327
#define IDS_PERMISSION_DENY 37328
#define IDS_PERMISSION_ALLOW_THIS_TIME 37329
#define IDS_PERMISSION_ALLOW_EVERY_VISIT 37330
#define IDS_PERMISSION_ALLOW_WHILE_VISITING 37331
#define IDS_PERMISSION_DONT_ALLOW 37332
#define IDS_PERMISSION_NEVER_ALLOW 37333
#define IDS_PERMISSION_ALLOW_APPROXIMATE_GEO 37334
#define IDS_PERMISSION_ALLOW_PRECISE_GEO 37335
#define IDS_PERMISSION_ALLOW_PRECISE_GEO_DESCRIPTION 37336
#define IDS_PERMISSION_ALLOW_APPROXIMATE_GEO_DESCRIPTION 37337
#define IDS_PERMISSION_ALLOW_PRECISE_GEO_LONG_DESCRIPTION 37338
#define IDS_PERMISSION_ALLOW_APPROXIMATE_GEO_LONG_DESCRIPTION 37339
#define IDS_BLUETOOTH_DEVICE_CHOOSER_PROMPT 37340
#define IDS_BLUETOOTH_DEVICE_CHOOSER_NO_DEVICES_FOUND_PROMPT 37341
#define IDS_BLUETOOTH_DEVICE_CHOOSER_PAIR_BUTTON_TEXT 37342
#define IDS_BLUETOOTH_DEVICE_CHOOSER_SCANNING_LABEL 37343
#define IDS_BLUETOOTH_DEVICE_CHOOSER_SCANNING_LABEL_TOOLTIP 37344
#define IDS_BLUETOOTH_DEVICE_CHOOSER_ADAPTER_OFF 37345
#define IDS_BLUETOOTH_DEVICE_CHOOSER_TURN_ON_BLUETOOTH_LINK_TEXT 37346
#define IDS_BLUETOOTH_DEVICE_CHOOSER_AUTHORIZE_BLUETOOTH_LINK_TEXT 37347
#define IDS_DEVICE_CHOOSER_DEVICE_NAME_WITH_ID 37348
#define IDS_BLUETOOTH_SCANNING_PROMPT 37349
#define IDS_BLUETOOTH_SCANNING_DEVICE_UNKNOWN 37350
#define IDS_BLUETOOTH_SCANNING_PROMPT_NO_DEVICES_FOUND_PROMPT 37351
#define IDS_BLUETOOTH_SCANNING_PROMPT_ALLOW_BUTTON_TEXT 37352
#define IDS_BLUETOOTH_SCANNING_PROMPT_BLOCK_BUTTON_TEXT 37353
#define IDS_DEVICE_CHOOSER_CANCEL_BUTTON_TEXT 37354
#define IDS_DEVICE_CHOOSER_NO_DEVICES_FOUND_PROMPT 37355
#define IDS_USB_DEVICE_CHOOSER_PROMPT 37356
#define IDS_USB_DEVICE_CHOOSER_CONNECT_BUTTON_TEXT 37357
#define IDS_USB_DEVICE_CHOOSER_LOADING_LABEL 37358
#define IDS_USB_DEVICE_CHOOSER_LOADING_LABEL_TOOLTIP 37359
#define IDS_GEOLOCATION_PERMISSION_CHIP 37360
#define IDS_NOTIFICATION_PERMISSIONS_CHIP 37361
#define IDS_MIDI_SYSEX_PERMISSION_CHIP 37362
#define IDS_MEDIA_CAPTURE_AUDIO_ONLY_PERMISSION_CHIP 37363
#define IDS_MEDIA_CAPTURE_VIDEO_ONLY_PERMISSION_CHIP 37364
#define IDS_MEDIA_CAPTURE_VIDEO_AND_AUDIO_PERMISSION_CHIP 37365
#define IDS_CLIPBOARD_PERMISSION_CHIP 37366
#define IDS_SAA_PERMISSION_CHIP 37367
#define IDS_IDLE_DETECTION_PERMISSION_CHIP 37368
#define IDS_KEYBOARD_LOCK_PERMISSION_CHIP 37369
#define IDS_KEYBOARD_AND_POINTER_LOCK_PERMISSION_CHIP 37370
#define IDS_POINTER_LOCK_PERMISSION_CHIP 37371
#define IDS_WEB_APP_INSTALLATION_PERMISSION_CHIP 37372
#define IDS_MOTION_AND_LIGHT_SENSORS_PERMISSION_CHIP 37373
#define IDS_MOTION_SENSORS_PERMISSION_CHIP 37374
#define IDS_AR_PERMISSION_CHIP 37375
#define IDS_PERMISSIONS_AR_ALLOWED_CONFIRMATION_SCREENREADER_ANNOUNCEMENT 37376
#define IDS_PERMISSIONS_AR_ALLOWED_ONCE_CONFIRMATION_SCREENREADER_ANNOUNCEMENT 37377
#define IDS_PERMISSIONS_AR_NOT_ALLOWED_CONFIRMATION_SCREENREADER_ANNOUNCEMENT 37378
#define IDS_VR_PERMISSION_CHIP 37379
#define IDS_PERMISSIONS_VR_ALLOWED_CONFIRMATION_SCREENREADER_ANNOUNCEMENT 37380
#define IDS_PERMISSIONS_VR_ALLOWED_ONCE_CONFIRMATION_SCREENREADER_ANNOUNCEMENT 37381
#define IDS_PERMISSIONS_VR_NOT_ALLOWED_CONFIRMATION_SCREENREADER_ANNOUNCEMENT 37382
#define IDS_HAND_TRACKING_PERMISSION_CHIP 37383
#define IDS_HAND_TRACKING_PERMISSION_BLOCKED_CHIP 37384
#define IDS_PERMISSIONS_HAND_TRACKING_ALLOWED_CONFIRMATION_SCREENREADER_ANNOUNCEMENT 37385
#define IDS_PERMISSIONS_HAND_TRACKING_ALLOWED_ONCE_CONFIRMATION_SCREENREADER_ANNOUNCEMENT 37386
#define IDS_PERMISSIONS_HAND_TRACKING_NOT_ALLOWED_CONFIRMATION_SCREENREADER_ANNOUNCEMENT 37387
#define IDS_GEOLOCATION_PERMISSION_BLOCKED_CHIP 37388
#define IDS_NOTIFICATION_PERMISSIONS_BLOCKED_CHIP 37389
#define IDS_CAPTURED_SURFACE_CONTROL_PERMISSION_CHIP 37390
#define IDS_CAPTURED_SURFACE_CONTROL_PERMISSION_BLOCKED_CHIP 37391
#define IDS_PERMISSIONS_CAPTURED_SURFACE_CONTROL_ALLOWED_CONFIRMATION_SCREENREADER_ANNOUNCEMENT 37392
#define IDS_PERMISSIONS_CAPTURED_SURFACE_CONTROL_NOT_ALLOWED_CONFIRMATION_SCREENREADER_ANNOUNCEMENT 37393
#define IDS_PERMISSIONS_PERMISSION_ALLOWED_CONFIRMATION 37394
#define IDS_PERMISSIONS_PERMISSION_ALLOWED_ONCE_CONFIRMATION 37395
#define IDS_PERMISSIONS_PERMISSION_NOT_ALLOWED_CONFIRMATION 37396
#define IDS_PERMISSIONS_NOTIFICATION_ALLOWED_CONFIRMATION_SCREENREADER_ANNOUNCEMENT 37397
#define IDS_PERMISSIONS_NOTIFICATION_NOT_ALLOWED_CONFIRMATION_SCREENREADER_ANNOUNCEMENT 37398
#define IDS_PERMISSIONS_GEOLOCATION_ALLOWED_CONFIRMATION_SCREENREADER_ANNOUNCEMENT 37399
#define IDS_PERMISSIONS_GEOLOCATION_ALLOWED_ONCE_CONFIRMATION_SCREENREADER_ANNOUNCEMENT 37400
#define IDS_PERMISSIONS_GEOLOCATION_NOT_ALLOWED_CONFIRMATION_SCREENREADER_ANNOUNCEMENT 37401
#define IDS_PERMISSIONS_CAMERA_ALLOWED_CONFIRMATION_SCREENREADER_ANNOUNCEMENT 37402
#define IDS_PERMISSIONS_CAMERA_ALLOWED_ONCE_CONFIRMATION_SCREENREADER_ANNOUNCEMENT 37403
#define IDS_PERMISSIONS_CAMERA_NOT_ALLOWED_CONFIRMATION_SCREENREADER_ANNOUNCEMENT 37404
#define IDS_PERMISSIONS_MICROPHONE_ALLOWED_CONFIRMATION_SCREENREADER_ANNOUNCEMENT 37405
#define IDS_PERMISSIONS_MICROPHONE_ALLOWED_ONCE_CONFIRMATION_SCREENREADER_ANNOUNCEMENT 37406
#define IDS_PERMISSIONS_MICROPHONE_NOT_ALLOWED_CONFIRMATION_SCREENREADER_ANNOUNCEMENT 37407
#define IDS_PERMISSIONS_CAMERA_AND_MICROPHONE_ALLOWED_CONFIRMATION_SCREENREADER_ANNOUNCEMENT 37408
#define IDS_PERMISSIONS_CAMERA_AND_MICROPHONE_ALLOWED_ONCE_CONFIRMATION_SCREENREADER_ANNOUNCEMENT 37409
#define IDS_PERMISSIONS_CAMERA_AND_MICROPHONE_NOT_ALLOWED_CONFIRMATION_SCREENREADER_ANNOUNCEMENT 37410
#define IDS_PERMISSIONS_SAA_ALLOWED_CONFIRMATION_SCREENREADER_ANNOUNCEMENT 37411
#define IDS_PERMISSIONS_SAA_NOT_ALLOWED_CONFIRMATION_SCREENREADER_ANNOUNCEMENT 37412
#define IDS_PERMISSIONS_KEYBOARD_LOCK_ALLOWED_CONFIRMATION_SCREENREADER_ANNOUNCEMENT 37413
#define IDS_PERMISSIONS_KEYBOARD_LOCK_NOT_ALLOWED_CONFIRMATION_SCREENREADER_ANNOUNCEMENT 37414
#define IDS_PERMISSIONS_POINTER_LOCK_ALLOWED_CONFIRMATION_SCREENREADER_ANNOUNCEMENT 37415
#define IDS_PERMISSIONS_POINTER_LOCK_NOT_ALLOWED_CONFIRMATION_SCREENREADER_ANNOUNCEMENT 37416
#define IDS_PERMISSIONS_WEB_INSTALL_ALLOWED_CONFIRMATION_SCREENREADER_ANNOUNCEMENT 37417
#define IDS_PERMISSIONS_WEB_INSTALL_NOT_ALLOWED_CONFIRMATION_SCREENREADER_ANNOUNCEMENT 37418
#define IDS_PERMISSIONS_MOTION_AND_LIGHT_SENSORS_ALLOWED_CONFIRMATION_SCREENREADER_ANNOUNCEMENT 37419
#define IDS_PERMISSIONS_MOTION_AND_LIGHT_SENSORS_ALLOWED_ONCE_CONFIRMATION_SCREENREADER_ANNOUNCEMENT 37420
#define IDS_PERMISSIONS_MOTION_AND_LIGHT_SENSORS_NOT_ALLOWED_CONFIRMATION_SCREENREADER_ANNOUNCEMENT 37421
#define IDS_PERMISSIONS_MOTION_SENSORS_ALLOWED_CONFIRMATION_SCREENREADER_ANNOUNCEMENT 37422
#define IDS_PERMISSIONS_MOTION_SENSORS_ALLOWED_ONCE_CONFIRMATION_SCREENREADER_ANNOUNCEMENT 37423
#define IDS_PERMISSIONS_MOTION_SENSORS_NOT_ALLOWED_CONFIRMATION_SCREENREADER_ANNOUNCEMENT 37424
#define IDS_REQUEST_QUOTA_INFOBAR_TEXT 37428
#define IDS_REQUEST_LARGE_QUOTA_INFOBAR_TEXT 37429
#define IDS_REQUEST_QUOTA_PERMISSION_FRAGMENT 37430
#define IDS_EMBEDDED_PROMPT_ADMIN_BLOCKED 37431
#define IDS_EMBEDDED_PROMPT_ADMIN_ALLOWED 37432
#define IDS_EMBEDDED_PROMPT_PREVIOUSLY_ALLOWED 37433
#define IDS_EMBEDDED_PROMPT_PREVIOUSLY_NOT_ALLOWED 37434
#define IDS_PERMISSION_OFF_FOR_CHROME 37435
#define IDS_PERMISSION_CHROME_NEEDS_PERMISSION 37436
#define IDS_GEOLOCATION_NAME_FRAGMENT 37437
#define IDS_CAMERA_PERMISSION_NAME_FRAGMENT 37438
#define IDS_MICROPHONE_PERMISSION_NAME_FRAGMENT 37439
#define IDS_CAMERA_AND_MICROPHONE_PERMISSION_NAME_FRAGMENT 37440
#define IDS_MACOS_NAME_FRAGMENT 37441
#define IDS_WINDOWS_NAME_FRAGMENT 37442
#define IDS_CHROMEOS_NAME_FRAGMENT 37443
#define IDS_EMBEDDED_PROMPT_OK_LABEL 37445
#define IDS_EMBEDDED_PROMPT_CONTINUE_ALLOWING 37446
#define IDS_EMBEDDED_PROMPT_STOP_ALLOWING 37447
#define IDS_EMBEDDED_PROMPT_OPEN_SYSTEM_SETTINGS 37449
#define IDS_EMBEDDED_PROMPT_CONTINUE_NOT_ALLOWING 37450
#define IDS_PERMISSION_PROMPT_SURVEY_CUSTOM_INVITATION 37451
#define IDS_POLICY_DM_STATUS_SUCCESS 37455
#define IDS_POLICY_DM_STATUS_REQUEST_INVALID 37456
#define IDS_POLICY_DM_STATUS_REQUEST_FAILED 37457
#define IDS_POLICY_DM_STATUS_TEMPORARY_UNAVAILABLE 37458
#define IDS_POLICY_DM_STATUS_HTTP_STATUS_ERROR 37459
#define IDS_POLICY_DM_STATUS_RESPONSE_DECODING_ERROR 37460
#define IDS_POLICY_DM_STATUS_SERVICE_MANAGEMENT_NOT_SUPPORTED 37461
#define IDS_POLICY_DM_STATUS_SERVICE_DEVICE_NOT_FOUND 37462
#define IDS_POLICY_DM_STATUS_SERVICE_MANAGEMENT_TOKEN_INVALID 37463
#define IDS_POLICY_DM_STATUS_SERVICE_ACTIVATION_PENDING 37464
#define IDS_POLICY_DM_STATUS_SERVICE_INVALID_SERIAL_NUMBER 37465
#define IDS_POLICY_DM_STATUS_SERVICE_DEVICE_ID_CONFLICT 37466
#define IDS_POLICY_DM_STATUS_SERVICE_MISSING_LICENSES 37467
#define IDS_POLICY_DM_STATUS_SERVICE_DEPROVISIONED 37468
#define IDS_POLICY_DM_STATUS_SERVICE_POLICY_NOT_FOUND 37469
#define IDS_POLICY_DM_STATUS_UNKNOWN_ERROR 37470
#define IDS_POLICY_DM_STATUS_INVALID_PACKAGED_DEVICE_FOR_KIOSK 37471
#define IDS_POLICY_DM_STATUS_ORG_UNIT_ENROLLMENT_LIMIT_EXCEEDED 37472
#define IDS_POLICY_DM_STATUS_SERVICE_DOMAIN_MISMATCH 37473
#define IDS_POLICY_DM_STATUS_CANNOT_SIGN_REQUEST 37474
#define IDS_POLICY_DM_STATUS_REQUEST_TOO_LARGE 37475
#define IDS_POLICY_DM_STATUS_SERVICE_TOO_MANY_REQUESTS 37476
#define IDS_POLICY_DM_STATUS_SERVICE_DEVICE_NEEDS_RESET 37477
#define IDS_POLICY_DM_STATUS_CONSUMER_ACCOUNT_WITH_PACKAGED_LICENSE 37478
#define IDS_POLICY_DM_STATUS_ENTERPRISE_ACCOUNT_IS_NOT_ELIGIBLE_TO_ENROLL 37479
#define IDS_POLICY_ACTION_BLOCKED_BY_ORGANIZATION 37480
#define IDS_POLICY_ACTION_FORCED_SAVE_TO_CLOUD 37481
#define IDS_POLICY_VALIDATION_OK 37482
#define IDS_POLICY_VALIDATION_BAD_INITIAL_SIGNATURE 37483
#define IDS_POLICY_VALIDATION_BAD_SIGNATURE 37484
#define IDS_POLICY_VALIDATION_ERROR_CODE_PRESENT 37485
#define IDS_POLICY_VALIDATION_PAYLOAD_PARSE_ERROR 37486
#define IDS_POLICY_VALIDATION_WRONG_POLICY_TYPE 37487
#define IDS_POLICY_VALIDATION_WRONG_SETTINGS_ENTITY_ID 37488
#define IDS_POLICY_VALIDATION_BAD_TIMESTAMP 37489
#define IDS_POLICY_VALIDATION_BAD_DM_TOKEN 37490
#define IDS_POLICY_VALIDATION_BAD_DEVICE_ID 37491
#define IDS_POLICY_VALIDATION_BAD_USER 37492
#define IDS_POLICY_VALIDATION_POLICY_PARSE_ERROR 37493
#define IDS_POLICY_VALIDATION_BAD_KEY_VERIFICATION_SIGNATURE 37494
#define IDS_POLICY_VALIDATION_VALUE_WARNING 37495
#define IDS_POLICY_VALIDATION_VALUE_ERROR 37496
#define IDS_POLICY_VALIDATION_UNKNOWN_ERROR 37497
#define IDS_POLICY_STORE_STATUS_OK 37498
#define IDS_POLICY_STORE_STATUS_LOAD_ERROR 37499
#define IDS_POLICY_STORE_STATUS_STORE_ERROR 37500
#define IDS_POLICY_STORE_STATUS_PARSE_ERROR 37501
#define IDS_POLICY_STORE_STATUS_SERIALIZE_ERROR 37502
#define IDS_POLICY_STORE_STATUS_VALIDATION_ERROR 37503
#define IDS_POLICY_STORE_STATUS_BAD_STATE 37504
#define IDS_POLICY_STORE_STATUS_UNKNOWN_ERROR 37505
#define IDS_POLICY_ASSOCIATION_STATE_ACTIVE 37506
#define IDS_POLICY_ASSOCIATION_STATE_UNMANAGED 37507
#define IDS_POLICY_ASSOCIATION_STATE_DEPROVISIONED 37508
#define IDS_POLICY_TYPE_ERROR 37509
#define IDS_POLICY_OUT_OF_RANGE_ERROR 37510
#define IDS_POLICY_VALUE_FORMAT_ERROR 37511
#define IDS_POLICY_INVALID_FILE_EXTENSION_ERROR 37512
#define IDS_POLICY_INVALID_HASH_ERROR 37513
#define IDS_POLICY_INVALID_SELECTION_ERROR 37514
#define IDS_POLICY_PAPER_SIZE_CUSTOM_NO_SIZE_ERROR 37515
#define IDS_POLICY_PAPER_SIZE_NOT_CUSTOM_ERROR 37516
#define IDS_POLICY_INVALID_ACCOUNT_PATTERN_FORMAT_ERROR 37517
#define IDS_POLICY_INVALID_PROTOCOL_ERROR 37518
#define IDS_POLICY_INVALID_ORIGIN_ERROR 37519
#define IDS_POLICY_EMPTY_ORIGIN_LIST_ERROR 37520
#define IDS_POLICY_INVALID_EXTENSION_ID_ERROR 37522
#define IDS_POLICY_INVALID_UPDATE_URL_ERROR 37523
#define IDS_POLICY_URL_PATH_SPECIFIED_ERROR 37524
#define IDS_POLICY_MISSING_VENDOR_ID_ERROR 37525
#define IDS_POLICY_INVALID_NUMBER_OF_URLS_ERROR 37526
#define IDS_POLICY_BRUSCHETTA_UNINSTALLABLE_ERROR 37527
#define IDS_POLICY_CLOUD_SOURCE_ONLY_ERROR 37528
#define IDS_POLICY_CLOUD_USER_ONLY_ERROR 37529
#define IDS_POLICY_CLOUD_MANAGEMENT_ENROLLMENT_ONLY_ERROR 37530
#define IDS_POLICY_DEFAULT_SEARCH_DISABLED 37531
#define IDS_POLICY_NOT_SPECIFIED_ERROR 37532
#define IDS_POLICY_EXTENSION_SETTINGS_ORIGIN_LIMIT_WARNING 37533
#define IDS_POLICY_URL_ALLOW_BLOCK_LIST_MAX_FILTERS_LIMIT_WARNING 37534
#define IDS_POLICY_ERROR_WITH_PATH 37535
#define IDS_POLICY_SCHEMA_VALIDATION_ERROR 37536
#define IDS_POLICY_INVALID_JSON_ERROR 37537
#define IDS_POLICY_INVALID_URL_ERROR 37538
#define IDS_POLICY_URL_NOT_HTTPS_ERROR 37539
#define IDS_POLICY_INVALID_SEARCH_URL_ERROR 37540
#define IDS_POLICY_INVALID_SECURE_DNS_MODE_ERROR 37541
#define IDS_POLICY_SECURE_DNS_TEMPLATES_INVALID_ERROR 37542
#define IDS_POLICY_SECURE_DNS_SALT_INVALID_ERROR 37543
#define IDS_POLICY_SECURE_DNS_SALT_INVALID_SIZE_ERROR 37544
#define IDS_POLICY_SECURE_DNS_TEMPLATES_IRRELEVANT_MODE_ERROR 37545
#define IDS_POLICY_SECURE_DNS_TEMPLATES_INVALID_MODE_ERROR 37546
#define IDS_POLICY_SECURE_DNS_TEMPLATES_UNSET_MODE_ERROR 37547
#define IDS_POLICY_SECURE_DNS_TEMPLATES_NOT_SPECIFIED_ERROR 37548
#define IDS_POLICY_INVALID_PROXY_MODE_ERROR 37549
#define IDS_POLICY_OFF_CWS_URL_ERROR 37551
#define IDS_POLICY_HOMEPAGE_LOCATION_ERROR 37552
#define IDS_POLICY_PROXY_MODE_DISABLED_ERROR 37553
#define IDS_POLICY_PROXY_MODE_AUTO_DETECT_ERROR 37554
#define IDS_POLICY_PROXY_MODE_PAC_URL_ERROR 37555
#define IDS_POLICY_PROXY_MODE_FIXED_SERVERS_ERROR 37556
#define IDS_POLICY_PROXY_MODE_SYSTEM_ERROR 37557
#define IDS_POLICY_PROXY_BOTH_SPECIFIED_ERROR 37558
#define IDS_POLICY_PROXY_NEITHER_SPECIFIED_ERROR 37559
#define IDS_POLICY_PROXY_UNKNOWN_CONDITION 37560
#define IDS_POLICY_PROXY_INVALID_SCHEME_HOST_PORT 37561
#define IDS_POLICY_PROXY_INVALID_PROXY 37562
#define IDS_POLICY_PROXY_INVALID_DESTINATION 37563
#define IDS_POLICY_UNAFFILIATED_USER_ERROR 37564
#define IDS_POLICY_DEVELOPER_TOOLS_EXTENSIONS_CONFLICT_MESSAGE 37565
#define IDS_POLICY_OVERRIDDEN 37566
#define IDS_POLICY_VALUE_DEPRECATED 37567
#define IDS_POLICY_DEPENDENCY_ERROR 37568
#define IDS_POLICY_DEPENDENCY_ERROR_ANY_VALUE 37569
#define IDS_POLICY_IDLE_TIMEOUT_ACTIONS_DEPENDENCY_ERROR 37570
#define IDS_POLICY_USER_IS_NOT_AFFILIATED_ERROR 37571
#define IDS_POLICY_LEVEL_ERROR 37576
#define IDS_POLICY_OK 37577
#define IDS_POLICY_UNSET 37578
#define IDS_POLICY_UNKNOWN 37579
#define IDS_POLICY_PROTO_PARSING_ERROR 37580
#define IDS_POLICY_HEX_COLOR_ERROR 37581
#define IDS_POLICY_SITE_SEARCH_SETTINGS_MAX_PROVIDERS_LIMIT_ERROR 37582
#define IDS_POLICY_SITE_SEARCH_SETTINGS_MAX_FEATURED_PROVIDERS_LIMIT_ERROR 37583
#define IDS_POLICY_SITE_SEARCH_SETTINGS_DUPLICATED_SHORTCUT 37584
#define IDS_SEARCH_POLICY_SETTINGS_SHORTCUT_CONTAINS_SPACE 37585
#define IDS_SEARCH_POLICY_SETTINGS_SHORTCUT_STARTS_WITH_AT 37586
#define IDS_SEARCH_POLICY_SETTINGS_NO_VALID_PROVIDER 37587
#define IDS_SEARCH_POLICY_SETTINGS_SHORTCUT_IS_EMPTY 37588
#define IDS_SEARCH_POLICY_SETTINGS_NAME_IS_EMPTY 37589
#define IDS_SEARCH_POLICY_SETTINGS_URL_IS_EMPTY 37590
#define IDS_SEARCH_POLICY_SETTINGS_URL_DOESNT_SUPPORT_REPLACEMENT 37591
#define IDS_SEARCH_POLICY_SETTINGS_SHORTCUT_EQUALS_DSP_KEYWORD 37592
#define IDS_POLICY_SITE_SEARCH_SETTINGS_SHORTCUT_EQUALS_SEARCH_AGGREGATOR_KEYWORD 37593
#define IDS_SEARCH_POLICY_SETTINGS_URL_NOT_HTTPS 37594
#define IDS_POLICY_DATA_CONTROLS_MUTUALLY_EXCLUSIVE_KEYS 37595
#define IDS_POLICY_DATA_CONTROLS_MUTUALLY_EXCLUSIVE_KEY_SETS 37596
#define IDS_POLICY_DATA_CONTROLS_UNSUPPORTED_CONDITION 37597
#define IDS_POLICY_DATA_CONTROLS_UNSUPPORTED_RESTRICTION 37598
#define IDS_POLICY_DATA_CONTROLS_UNSUPPORTED_LEVEL 37599
#define IDS_POLICY_NTP_SHORTCUTS_MAX_SHORTCUTS_LIMIT_ERROR 37600
#define IDS_POLICY_NTP_SHORTCUTS_NO_VALID_PROVIDER 37601
#define IDS_POLICY_NTP_SHORTCUTS_DUPLICATED_URL 37602
#define IDS_POLICY_NTP_SHORTCUTS_NAME_TOO_LONG 37603
#define IDS_POLICY_NTP_SHORTCUTS_URL_TOO_LONG 37604
#define IDS_POLICY_HTTP_HEADER_INJECTION_MAX_URL_PATTERNS_LIMIT_ERROR 37605
#define IDS_POLICY_HTTP_HEADER_INJECTION_MAX_HEADERS_PER_RULE_LIMIT_ERROR 37606
#define IDS_POLICY_HTTP_HEADER_INJECTION_MAX_HEADER_SIZE_LIMIT_ERROR 37607
#define IDS_POLICY_TITLE 37608
#define IDS_POLICY_FILTER_PLACEHOLDER 37609
#define IDS_POLICY_RELOAD_POLICIES 37610
#define IDS_POLICY_RELOADING_POLICIES 37611
#define IDS_POLICY_RELOAD_POLICIES_DONE 37612
#define IDS_EXPORT_POLICIES_JSON 37613
#define IDS_EXPORT_POLICIES_JSON_DONE 37614
#define IDS_UPLOAD_REPORT 37615
#define IDS_REPORT_UPLOADING 37616
#define IDS_REPORT_UPLOADED 37617
#define IDS_POLICY_STATUS 37618
#define IDS_POLICY_STATUS_DEVICE 37619
#define IDS_POLICY_STATUS_USER 37620
#define IDS_POLICY_STATUS_MACHINE 37621
#define IDS_POLICY_STATUS_ERROR_MANAGED_NO_POLICY 37623
#define IDS_POLICY_STATUS_FLEX_ORG_NO_POLICY 37624
#define IDS_POLICY_LABEL_MACHINE_ENROLLMENT_DOMAIN 37625
#define IDS_POLICY_LABEL_MACHINE_ENROLLMENT_TOKEN 37626
#define IDS_POLICY_LABEL_MACHINE_ENROLLMENT_DEVICE_ID 37627
#define IDS_POLICY_LABEL_MACHINE_ENROLLMENT_MACHINE_NAME 37628
#define IDS_POLICY_LABEL_USERNAME 37629
#define IDS_POLICY_LABEL_GAIA_ID 37630
#define IDS_POLICY_LABEL_CLIENT_ID 37631
#define IDS_POLICY_LABEL_PROFILE_ID 37632
#define IDS_POLICY_LABEL_ASSET_ID 37633
#define IDS_POLICY_LABEL_LOCATION 37634
#define IDS_POLICY_LABEL_DIRECTORY_API_ID 37635
#define IDS_POLICY_LABEL_MANAGED_BY 37636
#define IDS_POLICY_LABEL_TIME_SINCE_LAST_FETCH_ATTEMPT 37637
#define IDS_POLICY_LABEL_TIME_SINCE_LAST_REFRESH 37638
#define IDS_POLICY_LABEL_POLICY_FETCH 37639
#define IDS_POLICY_LABEL_EXTENSION_INSTALL_POLICY_FETCH 37640
#define IDS_POLICY_LABEL_LAST_CLOUD_REPORT_SENT_TIMESTAMP 37641
#define IDS_POLICY_NOT_SPECIFIED 37642
#define IDS_POLICY_LABEL_PUSH_POLICIES 37643
#define IDS_POLICY_PUSH_POLICIES_ON 37644
#define IDS_POLICY_PUSH_POLICIES_OFF 37645
#define IDS_POLICY_NEVER_FETCHED 37646
#define IDS_POLICY_LABEL_REFRESH_INTERVAL 37647
#define IDS_POLICY_LABEL_CONFLICT 37648
#define IDS_POLICY_LABEL_SUPERSEDING 37649
#define IDS_POLICY_LABEL_CONFLICT_VALUE 37650
#define IDS_POLICY_LABEL_SUPERSEDED_VALUE 37651
#define IDS_POLICY_LABEL_ERROR 37652
#define IDS_POLICY_LABEL_DEPRECATED 37653
#define IDS_POLICY_LABEL_FUTURE 37654
#define IDS_POLICY_LABEL_IGNORED 37655
#define IDS_POLICY_LABEL_VALUE 37656
#define IDS_POLICY_LABEL_VERSION 37657
#define IDS_POLICY_LABEL_STATUS 37658
#define IDS_POLICY_LABEL_INFO 37659
#define IDS_POLICY_LABEL_PRECEDENCE 37660
#define IDS_POLICY_SHOW_UNSET 37661
#define IDS_POLICY_NO_POLICIES_SET 37662
#define IDS_POLICY_HEADER_SCOPE 37663
#define IDS_POLICY_HEADER_LEVEL 37664
#define IDS_POLICY_HEADER_NAMESPACE 37665
#define IDS_POLICY_HEADER_NAME 37666
#define IDS_POLICY_HEADER_VALUE 37667
#define IDS_POLICY_HEADER_STATUS 37668
#define IDS_POLICY_HEADER_SOURCE 37669
#define IDS_POLICY_HEADER_WARNING 37670
#define IDS_POLICY_SHOW_MORE 37671
#define IDS_POLICY_SHOW_LESS 37672
#define IDS_POLICY_LEARN_MORE 37673
#define IDS_POLICY_SCOPE_USER 37674
#define IDS_POLICY_SCOPE_ALL_USERS 37675
#define IDS_POLICY_SCOPE_DEVICE 37676
#define IDS_POLICY_LEVEL_RECOMMENDED 37677
#define IDS_POLICY_LEVEL_MANDATORY 37678
#define IDS_POLICY_SOURCE_ENTERPRISE_DEFAULT 37679
#define IDS_POLICY_SOURCE_DEFAULT 37680
#define IDS_POLICY_SOURCE_COMMAND_LINE 37681
#define IDS_POLICY_SOURCE_CLOUD 37682
#define IDS_POLICY_SOURCE_MERGED 37683
#define IDS_POLICY_SOURCE_CLOUD_FROM_ASH 37684
#define IDS_POLICY_SOURCE_RESTRICTED_MANAGED_GUEST_SESSION_OVERRIDE 37685
#define IDS_POLICY_SOURCE_ACTIVE_DIRECTORY 37686
#define IDS_POLICY_SOURCE_PLATFORM 37687
#define IDS_POLICY_SOURCE_DEVICE_LOCAL_ACCOUNT_OVERRIDE 37688
#define IDS_POLICY_RISK_TAG_FULL_ADMIN_ACCESS 37689
#define IDS_POLICY_RISK_TAG_SYSTEM_SECURITY 37690
#define IDS_POLICY_RISK_TAG_WEBSITE_SHARING 37691
#define IDS_POLICY_RISK_TAG_ADMIN_SHARING 37692
#define IDS_POLICY_RISK_TAG_FILTERING 37693
#define IDS_POLICY_RISK_TAG_LOCAL_DATA_ACCESS 37694
#define IDS_POLICY_RISK_TAG_GOOGLE_SHARING 37695
#define IDS_POLICY_SHOW_EXPANDED_STATUS 37696
#define IDS_POLICY_HIDE_EXPANDED_STATUS 37697
#define IDS_POLICY_LIST_MERGING_WRONG_POLICY_TYPE_SPECIFIED 37698
#define IDS_POLICY_DICTIONARY_MERGING_WRONG_POLICY_TYPE_SPECIFIED 37699
#define IDS_POLICY_DICTIONARY_MERGING_POLICY_NOT_ALLOWED 37700
#define IDS_POLICY_CONFLICT_SAME_VALUE 37701
#define IDS_POLICY_CONFLICT_DIFF_VALUE 37702
#define IDS_POLICY_USES_SAME_LOCAL_STATE_AND_PROFILE_PREFS 37703
#define IDS_POLICY_USES_LOCAL_STATE_AND_PROFILE_PREFS 37704
#define IDS_POLICY_MIGRATED_OLD_POLICY 37705
#define IDS_POLICY_MIGRATED_NEW_POLICY 37706
#define IDS_POLICY_BLOCKED 37707
#define IDS_POLICY_INVALID 37708
#define IDS_POLICY_IGNORED_BY_GROUP_MERGING 37709
#define IDS_POLICY_INVALID_VALUE 37710
#define IDS_POLICY_IGNORED_MANDATORY_REPORTING_POLICY 37711
#define IDS_POLICY_IGNORED_CHROME_PROFILE 37712
#define IDS_POLICY_IGNORED_UNAFFILIATED 37713
#define IDS_POLICY_IGNORED_EXTENSION 37714
#define IDS_POLICY_EXTENSION_INSTALL_IGNORED_BY_INSTALLATION_MODE 37715
#define IDS_POLICY_PRECEDENCE_PLATFORM_MACHINE 37716
#define IDS_POLICY_PRECEDENCE_PLATFORM_USER 37717
#define IDS_POLICY_PRECEDENCE_CLOUD_MACHINE 37718
#define IDS_POLICY_PRECEDENCE_CLOUD_USER 37719
#define IDS_POLICY_SPELLCHECK_UNKNOWN_LANGUAGE 37720
#define IDS_POLICY_SPELLCHECK_BLOCKLIST_IGNORE 37721
#define IDS_POLICY_DEFAULT_GEO_POLICY_ARC_CONFLICT 37722
#define IDS_POLICY_LABEL_IS_AFFILIATED 37723
#define IDS_POLICY_IS_AFFILIATED_YES 37724
#define IDS_POLICY_IS_AFFILIATED_NO 37725
#define IDS_POLICY_LABEL_IS_OFFHOURS_ACTIVE 37726
#define IDS_POLICY_OFFHOURS_ACTIVE 37727
#define IDS_POLICY_OFFHOURS_NOT_ACTIVE 37728
#define IDS_POLICY_SIGNIN_PROFILE 37729
#define IDS_POLICY_COPY_VALUE 37730
#define IDS_COPY_POLICIES_JSON 37731
#define IDS_COPY_POLICIES_DONE 37732
#define IDS_VIEW_POLICY_LOGS 37733
#define IDS_POLICY_MORE_ACTIONS 37734
#define IDS_POLICY_TABLE_COLUMN_SORT 37735
#define IDS_POLICY_TABLE_COLUMN_SORT_ASCENDING 37736
#define IDS_POLICY_TABLE_COLUMN_SORT_DESCENDING 37737
#define IDS_POLICY_BANNER_PROMOTION_TITLE 37738
#define IDS_POLICY_BANNER_PROMOTION_DESC 37739
#define IDS_POLICY_BANNER_PROMOTION_BTN 37740
#define IDS_DEVTOOLS_BLOCKED_BY_POLICY 37741
#define IDS_DEVTOOLS_NOT_ALLOWED 37742
#define IDS_POLICY_LOGS_TITLE 37743
#define IDS_POLICY_LOGS_FILTER_PLACEHOLDER 37744
#define IDS_VERSION_INFO 37745
#define IDS_EXPORT_POLICY_LOGS_JSON 37746
#define IDS_REFRESH_POLICY_LOGS 37747
#define IDS_POLICY_TEST_TITLE 37748
#define IDS_POLICY_TEST_RESTART_AND_APPLY 37749
#define IDS_POLICY_TEST_APPLY 37750
#define IDS_POLICY_TEST_IMPORT 37751
#define IDS_POLICY_TEST_DESC 37752
#define IDS_POLICY_TEST_REVERT 37753
#define IDS_POLICY_TEST_TABLE_SCOPE 37754
#define IDS_POLICY_TEST_ADD 37755
#define IDS_POLICY_SELECT_NAME 37756
#define IDS_POLICY_TEST_TABLE_PRESET 37757
#define IDS_LOCAL_TEST_POLICIES_ENABLED 37758
#define IDS_POLICY_TEST_PRESET_CUSTOM 37759
#define IDS_POLICY_TEST_PRESET_LOCAL_MACHINE 37760
#define IDS_POLICY_TEST_PRESET_CLOUD_ACCOUNT 37761
#define IDS_POLICY_TEST_USER_AFFILIATED 37762
#define IDS_CHROME_URLS_DISABLED_PAGE_HEADER 37763
#define IDS_CHROME_URLS_DISABLED_PAGE_TITLE 37764
#define IDS_CHROME_URLS_DISABLED_PAGE_MESSAGE 37765
#define IDS_POLICY_DLP_CLIPBOARD_BLOCKED_ON_PASTE 37766
#define IDS_POLICY_DLP_CLIPBOARD_BLOCKED_ON_COPY_VM 37767
#define IDS_POLICY_DLP_CLIPBOARD_BLOCK_TOAST_BUTTON 37768
#define IDS_POLICY_DLP_ANDROID_APPS 37769
#define IDS_POLICY_DLP_CLIPBOARD_BLOCK_DISMISS_BUTTON 37770
#define IDS_POLICY_DLP_CLIPBOARD_WARN_ON_PASTE 37771
#define IDS_POLICY_DLP_CLIPBOARD_WARN_PROCEED_BUTTON 37772
#define IDS_POLICY_DLP_CLIPBOARD_WARN_ON_COPY_VM 37773
#define IDS_POLICY_DLP_PRINTING_BLOCKED_TITLE 37774
#define IDS_POLICY_DLP_PRINTING_BLOCKED_MESSAGE 37775
#define IDS_POLICY_DLP_PRINTING_WARN_TITLE 37776
#define IDS_POLICY_DLP_PRINTING_WARN_MESSAGE 37777
#define IDS_POLICY_DLP_PRINTING_WARN_CONTINUE_BUTTON 37778
#define IDS_POLICY_DLP_SCREEN_SHARE_BLOCKED_TITLE 37779
#define IDS_POLICY_DLP_SCREEN_SHARE_BLOCKED_MESSAGE 37780
#define IDS_POLICY_DLP_SCREEN_SHARE_PAUSED_TITLE 37781
#define IDS_POLICY_DLP_SCREEN_SHARE_PAUSED_MESSAGE 37782
#define IDS_POLICY_DLP_SCREEN_SHARE_RESUMED_TITLE 37783
#define IDS_POLICY_DLP_SCREEN_SHARE_RESUMED_MESSAGE 37784
#define IDS_POLICY_DLP_SCREEN_CAPTURE_DISABLED_TITLE 37785
#define IDS_POLICY_DLP_SCREEN_CAPTURE_DISABLED_MESSAGE 37786
#define IDS_POLICY_DLP_VIDEO_CAPTURE_STOPPED_TITLE 37787
#define IDS_POLICY_DLP_VIDEO_CAPTURE_STOPPED_MESSAGE 37788
#define IDS_POLICY_DLP_SCREEN_SHARE_WARN_TITLE 37789
#define IDS_POLICY_DLP_SCREEN_SHARE_WARN_MESSAGE 37790
#define IDS_POLICY_DLP_SCREEN_SHARE_WARN_CONTINUE_BUTTON 37791
#define IDS_POLICY_DLP_SCREEN_CAPTURE_WARN_TITLE 37792
#define IDS_POLICY_DLP_SCREEN_CAPTURE_WARN_MESSAGE 37793
#define IDS_POLICY_DLP_SCREEN_CAPTURE_WARN_CONTINUE_BUTTON 37794
#define IDS_POLICY_DLP_VIDEO_CAPTURE_WARN_TITLE 37795
#define IDS_POLICY_DLP_VIDEO_CAPTURE_WARN_MESSAGE 37796
#define IDS_POLICY_DLP_VIDEO_CAPTURE_WARN_CONTINUE_BUTTON 37797
#define IDS_POLICY_DLP_VIDEO_CAPTURE_WARN_CANCEL_BUTTON 37798
#define IDS_POLICY_DLP_FILES_DOWNLOAD_WARN_TITLE 37799
#define IDS_POLICY_DLP_FILES_TRANSFER_WARN_TITLE 37800
#define IDS_POLICY_DLP_FILES_UPLOAD_WARN_TITLE 37801
#define IDS_POLICY_DLP_FILES_MOVE_WARN_TITLE 37802
#define IDS_POLICY_DLP_FILES_COPY_WARN_TITLE 37803
#define IDS_POLICY_DLP_FILES_OPEN_WARN_TITLE 37804
#define IDS_POLICY_DLP_FILES_DOWNLOAD_REVIEW_TITLE 37805
#define IDS_POLICY_DLP_FILES_TRANSFER_REVIEW_TITLE 37806
#define IDS_POLICY_DLP_FILES_UPLOAD_REVIEW_TITLE 37807
#define IDS_POLICY_DLP_FILES_MOVE_REVIEW_TITLE 37808
#define IDS_POLICY_DLP_FILES_COPY_REVIEW_TITLE 37809
#define IDS_POLICY_DLP_FILES_OPEN_REVIEW_TITLE 37810
#define IDS_POLICY_DLP_FILES_DOWNLOAD_WARN_MESSAGE 37811
#define IDS_POLICY_DLP_FILES_TRANSFER_WARN_MESSAGE 37812
#define IDS_POLICY_DLP_FILES_UPLOAD_WARN_MESSAGE 37813
#define IDS_POLICY_DLP_FILES_MOVE_WARN_MESSAGE 37814
#define IDS_POLICY_DLP_FILES_COPY_WARN_MESSAGE 37815
#define IDS_POLICY_DLP_FILES_OPEN_WARN_MESSAGE 37816
#define IDS_POLICY_DLP_FILES_WARN_MESSAGE 37817
#define IDS_POLICY_DLP_FILES_DOWNLOAD_WARN_CONTINUE_BUTTON 37818
#define IDS_POLICY_DLP_FILES_TRANSFER_WARN_CONTINUE_BUTTON 37819
#define IDS_POLICY_DLP_FILES_UPLOAD_WARN_CONTINUE_BUTTON 37820
#define IDS_POLICY_DLP_FILES_MOVE_WARN_CONTINUE_BUTTON 37821
#define IDS_POLICY_DLP_FILES_COPY_WARN_CONTINUE_BUTTON 37822
#define IDS_POLICY_DLP_FILES_OPEN_WARN_CONTINUE_BUTTON 37823
#define IDS_POLICY_DLP_FILES_DESTINATION_REMOVABLE_STORAGE 37824
#define IDS_POLICY_DLP_FILES_REVIEW_BUTTON 37825
#define IDS_POLICY_DLP_FILES_DISMISS_BUTTON 37826
#define IDS_POLICY_DLP_FILES_OK_BUTTON 37827
#define IDS_POLICY_DLP_WARN_CANCEL_BUTTON 37828
#define IDS_POLICY_DLP_CLIPBOARD_BUBBLE_MESSAGE 37829
#define IDS_POLICY_DLP_FROM_YOUR_ADMIN_MESSAGE 37830
#define IDS_POLICY_DLP_FILES_DOWNLOAD_BLOCK_TITLE 37831
#define IDS_POLICY_DLP_FILES_DOWNLOAD_BLOCK_MESSAGE 37832
#define IDS_POLICY_DLP_FILES_UPLOAD_BLOCK_TITLE 37833
#define IDS_POLICY_DLP_FILES_UPLOAD_BLOCK_MESSAGE 37834
#define IDS_POLICY_DLP_FILES_OPEN_BLOCK_TITLE 37835
#define IDS_POLICY_DLP_FILES_OPEN_BLOCK_MESSAGE 37836
#define IDS_POLICY_DLP_FILES_DOWNLOAD_BLOCKED_TITLE 37837
#define IDS_POLICY_DLP_FILES_TRANSFER_BLOCKED_TITLE 37838
#define IDS_POLICY_DLP_FILES_UPLOAD_BLOCKED_TITLE 37839
#define IDS_POLICY_DLP_FILES_MOVE_BLOCKED_TITLE 37840
#define IDS_POLICY_DLP_FILES_COPY_BLOCKED_TITLE 37841
#define IDS_POLICY_DLP_FILES_OPEN_BLOCKED_TITLE 37842
#define IDS_POLICY_DLP_FILES_POLICY_BLOCK_SINGLE_FILE_MESSAGE 37843
#define IDS_POLICY_DLP_FILES_POLICY_BLOCK_MESSAGE 37844
#define IDS_POLICY_DLP_FILES_CONTENT_BLOCK_SINGLE_FILE_MESSAGE 37845
#define IDS_POLICY_DLP_FILES_CONTENT_BLOCK_MESSAGE 37846
#define IDS_POLICY_DLP_FILES_BLOCK_MESSAGE 37847
#define IDS_POLICY_DLP_FILES_DOWNLOAD_JUSTIFICATION_LABEL 37848
#define IDS_POLICY_DLP_FILES_UPLOAD_JUSTIFICATION_LABEL 37849
#define IDS_POLICY_DLP_FILES_COPY_JUSTIFICATION_LABEL 37850
#define IDS_POLICY_DLP_FILES_MOVE_JUSTIFICATION_LABEL 37851
#define IDS_POLICY_DLP_FILES_OPEN_JUSTIFICATION_LABEL 37852
#define IDS_POLICY_DLP_FILES_TRANSFER_JUSTIFICATION_LABEL 37853
#define IDS_POLICY_DLP_FILES_DOWNLOAD_TIMEOUT_TITLE 37854
#define IDS_POLICY_DLP_FILES_TRANSFER_TIMEOUT_TITLE 37855
#define IDS_POLICY_DLP_FILES_UPLOAD_TIMEOUT_TITLE 37856
#define IDS_POLICY_DLP_FILES_MOVE_TIMEOUT_TITLE 37857
#define IDS_POLICY_DLP_FILES_COPY_TIMEOUT_TITLE 37858
#define IDS_POLICY_DLP_FILES_OPEN_TIMEOUT_TITLE 37859
#define IDS_POLICY_DLP_FILES_DOWNLOAD_TIMEOUT_MESSAGE 37860
#define IDS_POLICY_DLP_FILES_TRANSFER_TIMEOUT_MESSAGE 37861
#define IDS_POLICY_DLP_FILES_UPLOAD_TIMEOUT_MESSAGE 37862
#define IDS_POLICY_DLP_FILES_MOVE_TIMEOUT_MESSAGE 37863
#define IDS_POLICY_DLP_FILES_COPY_TIMEOUT_MESSAGE 37864
#define IDS_POLICY_DLP_FILES_OPEN_TIMEOUT_MESSAGE 37865
#define IDS_POLICY_DLP_FILES_LEARN_MORE_ABOUT_DATA_CONTROLS_ACCESSIBLE_NAME 37866
#define IDS_POLICY_DLP_FILES_LEARN_MORE_ABOUT_MALWARE_PROTECTION_ACCESSIBLE_NAME 37867
#define IDS_POLICY_DLP_FILES_LEARN_MORE_ABOUT_SENSITIVE_DATA_PROTECTION_ACCESSIBLE_NAME 37868
#define IDS_POLICY_DLP_FILES_JUSTIFICATION_TEXTAREA_ACCESSIBLE_DESCRIPTION 37869
#define IDS_POLICY_DEVICE_SCHEDULED_REBOOT_TITLE 37870
#define IDS_POLICY_DEVICE_SCHEDULED_REBOOT_MESSAGE 37871
#define IDS_POLICY_REBOOT_BUTTON 37872
#define IDS_POLICY_DEVICE_SCHEDULED_REBOOT_DIALOG_MESSAGE 37873
#define IDS_REBOOT_SCHEDULED_TITLE_MINUTES 37874
#define IDS_REBOOT_SCHEDULED_TITLE_SECONDS 37875
#define IDS_POLICY_DEVICE_POST_REBOOT_TITLE 37876
#define IDS_POLICY_DEVICE_LOCATED_TITLE 37877
#define IDS_POLICY_DEVICE_LOCATED_MESSAGE 37878
#define IDS_POLICY_BROWSING_DATA_DEPENDENCY_APPLIED_INFO 37879
#define IDS_POLICY_BROWSING_DATA_PLATFORM_UNSUPPORTED 37880
#define IDS_POLICY_NOT_SUPPORTED_AS_USER_POLICY_ON_IOS 37881
#define IDS_POLICY_GEN_AI_DEFAULT_SETTINGS_CONTROL_MESSAGE 37882
#define IDS_POLICY_GEN_AI_DEFAULT_SETTINGS_NO_CONTROL_MESSAGE 37883
#define IDS_POLICY_SKYVAULT_CAMERA_SIGN_IN_DISPLAY_SOURCE 37884
#define IDS_POLICY_SKYVAULT_CAMERA_SIGN_IN_TITLE_PHOTO 37885
#define IDS_POLICY_SKYVAULT_CAMERA_SIGN_IN_TITLE_VIDEO 37886
#define IDS_POLICY_SKYVAULT_CAMERA_SIGN_IN_TITLE_SCAN 37887
#define IDS_POLICY_SKYVAULT_CAMERA_SIGN_IN_MESSAGE_PHOTO 37888
#define IDS_POLICY_SKYVAULT_CAMERA_SIGN_IN_MESSAGE_VIDEO 37889
#define IDS_POLICY_SKYVAULT_CAMERA_SIGN_IN_MESSAGE_SCAN 37890
#define IDS_POLICY_SKYVAULT_CAMERA_SIGN_IN_BUTTON 37891
#define IDS_POLICY_SKYVAULT_CAMERA_SIGN_IN_CANCEL_BUTTON 37892
#define IDS_POLICY_SKYVAULT_CAMERA_UPLOAD_DISPLAY_SOURCE 37893
#define IDS_POLICY_SKYVAULT_CAMERA_UPLOAD_CANCEL_BUTTON 37894
#define IDS_POLICY_SKYVAULT_CAMERA_UPLOAD_TITLE 37895
#define IDS_POLICY_SKYVAULT_CAMERA_MULTI_UPLOAD_TITLE 37896
#define IDS_POLICY_SKYVAULT_CAMERA_UPLOAD_ONEDRIVE_MESSAGE 37897
#define IDS_POLICY_SKYVAULT_CAMERA_MULTI_UPLOAD_ONEDRIVE_MESSAGE 37898
#define IDS_POLICY_SKYVAULT_CAMERA_UPLOAD_GOOGLE_DRIVE_MESSAGE 37899
#define IDS_POLICY_SKYVAULT_CAMERA_MULTI_UPLOAD_GOOGLE_DRIVE_MESSAGE 37900
#define IDS_POLICY_SKYVAULT_CAMERA_CANCEL_UPLOAD_TITLE 37901
#define IDS_POLICY_SKYVAULT_CAMERA_CANCEL_UPLOAD_MESSAGE 37902
#define IDS_POLICY_SKYVAULT_CAMERA_CANCEL_UPLOAD_SKIP_CHECKBOX 37903
#define IDS_POLICY_SKYVAULT_CAMERA_CANCEL_UPLOAD_CANCEL_BUTTON 37904
#define IDS_POLICY_SKYVAULT_CAMERA_CANCEL_UPLOAD_CLOSE_BUTTON 37905
#define IDS_POLICY_SKYVAULT_CAMERA_PHOTO_UPLOAD_DONE_TITLE_ONEDRIVE 37906
#define IDS_POLICY_SKYVAULT_CAMERA_VIDEO_UPLOAD_DONE_TITLE_ONEDRIVE 37907
#define IDS_POLICY_SKYVAULT_CAMERA_SCAN_UPLOAD_DONE_TITLE_ONEDRIVE 37908
#define IDS_POLICY_SKYVAULT_CAMERA_PHOTO_UPLOAD_DONE_TITLE_GOOGLE_DRIVE 37909
#define IDS_POLICY_SKYVAULT_CAMERA_VIDEO_UPLOAD_DONE_TITLE_GOOGLE_DRIVE 37910
#define IDS_POLICY_SKYVAULT_CAMERA_SCAN_UPLOAD_DONE_TITLE_GOOGLE_DRIVE 37911
#define IDS_POLICY_SKYVAULT_CAMERA_UPLOAD_DONE_MESSAGE 37912
#define IDS_POLICY_SKYVAULT_CAMERA_UPLOAD_DONE_EDIT_BUTTON 37913
#define IDS_POLICY_SKYVAULT_CAMERA_UPLOAD_DONE_DELETE_BUTTON 37914
#define IDS_POLICY_SKYVAULT_CAMERA_PHOTO_UPLOAD_ERROR_TITLE 37915
#define IDS_POLICY_SKYVAULT_CAMERA_PHOTO_UPLOAD_ERROR_MESSAGE 37916
#define IDS_POLICY_SKYVAULT_CAMERA_PHOTO_UPLOAD_ERROR_RETAKE_BUTTON 37917
#define IDS_POLICY_SKYVAULT_CAMERA_VIDEO_UPLOAD_ERROR_TITLE 37918
#define IDS_POLICY_SKYVAULT_CAMERA_VIDEO_UPLOAD_ERROR_MESSAGE 37919
#define IDS_POLICY_SKYVAULT_CAMERA_VIDEO_UPLOAD_ERROR_RETAKE_BUTTON 37920
#define IDS_POLICY_SKYVAULT_CAMERA_SCAN_UPLOAD_ERROR_TITLE 37921
#define IDS_POLICY_SKYVAULT_CAMERA_SCAN_UPLOAD_ERROR_MESSAGE 37922
#define IDS_POLICY_SKYVAULT_CAMERA_SCAN_UPLOAD_ERROR_RETAKE_BUTTON 37923
#define IDS_POLICY_SKYVAULT_CAMERA_UPLOAD_ERROR_DISMISS_BUTTON 37924
#define IDS_POLICY_SKYVAULT_DOWNLOAD_SIGN_IN_TITLE 37925
#define IDS_POLICY_SKYVAULT_DOWNLOAD_SIGN_IN_BUTTON 37926
#define IDS_POLICY_SKYVAULT_DOWNLOAD_SIGN_IN_CANCEL_BUTTON 37927
#define IDS_POLICY_SKYVAULT_DOWNLOAD_SIGN_IN_DISPLAY_SOURCE 37928
#define IDS_POLICY_SKYVAULT_MIGRATION_BANNER 37929
#define IDS_POLICY_SKYVAULT_MISCONFIGURED_POLICY_MESSAGE 37930
#define IDS_POLICY_SKYVAULT_SCREENCAPTURE_UPLOAD_TITLE 37931
#define IDS_POLICY_SKYVAULT_SCREENCAPTURE_RECORDING_UPLOAD_TITLE 37932
#define IDS_POLICY_SKYVAULT_SCREENCAPTURE_UPLOAD_ONEDRIVE_MESSAGE 37933
#define IDS_POLICY_SKYVAULT_SCREENCAPTURE_UPLOAD_CANCEL_BUTTON 37934
#define IDS_POLICY_SKYVAULT_SCREENCAPTURE_SIGN_IN_TITLE 37935
#define IDS_POLICY_SKYVAULT_SCREENCAPTURE_SIGN_IN_BUTTON 37936
#define IDS_POLICY_SKYVAULT_SCREENCAPTURE_SIGN_IN_CANCEL_BUTTON 37937
#define IDS_POLICY_SKYVAULT_CLOUD_PROVIDER_GOOGLE_DRIVE 37938
#define IDS_POLICY_SKYVAULT_CLOUD_PROVIDER_ONEDRIVE 37939
#define IDS_POLICY_SKYVAULT_MIGRATION_PROGRESS_TITLE 37940
#define IDS_POLICY_SKYVAULT_MIGRATION_PROGRESS_MESSAGE 37941
#define IDS_POLICY_SKYVAULT_MIGRATION_COMPLETED_TITLE 37942
#define IDS_POLICY_SKYVAULT_MIGRATION_COMPLETED_MESSAGE 37943
#define IDS_POLICY_SKYVAULT_MIGRATION_COMPLETED_BUTTON 37944
#define IDS_POLICY_SKYVAULT_MIGRATION_CONFIG_ERROR_TITLE 37945
#define IDS_POLICY_SKYVAULT_MIGRATION_CONFIG_ERROR_MESSAGE 37946
#define IDS_POLICY_SKYVAULT_MIGRATION_ERROR_TITLE 37947
#define IDS_POLICY_SKYVAULT_MIGRATION_ERROR_MESSAGE 37948
#define IDS_POLICY_SKYVAULT_MIGRATION_ERROR_BUTTON 37949
#define IDS_POLICY_SKYVAULT_MIGRATION_SIGN_IN_TITLE 37950
#define IDS_POLICY_SKYVAULT_MIGRATION_SIGN_IN_MESSAGE 37951
#define IDS_POLICY_SKYVAULT_MIGRATION_SIGN_IN_BUTTON 37952
#define IDS_POLICY_SKYVAULT_MIGRATION_DIALOG_TITLE_HOURS 37953
#define IDS_POLICY_SKYVAULT_MIGRATION_DIALOG_TITLE_HOUR 37954
#define IDS_POLICY_SKYVAULT_MIGRATION_DIALOG_TITLE_MINUTES 37955
#define IDS_POLICY_SKYVAULT_MIGRATION_DIALOG_TITLE_MINUTE 37956
#define IDS_POLICY_SKYVAULT_MIGRATION_DIALOG_UPLOAD_START_MESSAGE 37957
#define IDS_POLICY_SKYVAULT_MIGRATION_DIALOG_UPLOAD_START_ON_MESSAGE 37958
#define IDS_POLICY_SKYVAULT_MIGRATION_DIALOG_UPLOAD_DONE_MESSAGE 37959
#define IDS_POLICY_SKYVAULT_MIGRATION_DIALOG_UPLOAD_NOW_BUTTON 37960
#define IDS_POLICY_SKYVAULT_MIGRATION_DIALOG_UPLOAD_IN_HOURS_BUTTON 37961
#define IDS_POLICY_SKYVAULT_MIGRATION_DIALOG_UPLOAD_IN_MINUTES_BUTTON 37962
#define IDS_POLICY_SKYVAULT_DELETION_DIALOG_TITLE_HOURS 37963
#define IDS_POLICY_SKYVAULT_DELETION_DIALOG_TITLE_HOUR 37964
#define IDS_POLICY_SKYVAULT_DELETION_DIALOG_TITLE_MINUTES 37965
#define IDS_POLICY_SKYVAULT_DELETION_DIALOG_TITLE_MINUTE 37966
#define IDS_POLICY_SKYVAULT_DELETION_DIALOG_START_MESSAGE 37967
#define IDS_POLICY_SKYVAULT_DELETION_DIALOG_STORE_MESSAGE 37968
#define IDS_POLICY_SKYVAULT_DELETION_DIALOG_DELETE_NOW_BUTTON 37969
#define IDS_POLICY_SKYVAULT_DELETION_DIALOG_DELETE_IN_HOURS_BUTTON 37970
#define IDS_POLICY_SKYVAULT_DELETION_DIALOG_DELETE_IN_MINUTES_BUTTON 37971
#define IDS_POLICY_SKYVAULT_DELETION_COMPLETED_TITLE 37972
#define IDS_POLICY_SKYVAULT_DELETION_COMPLETED_MESSAGE 37973
#define IDS_POLICY_SKYVAULT_DELETION_BANNER 37974
#define IDS_POLICY_SKYVAULT_MIGRATION_UPLOAD_ERROR_NO_SPACE 37975
#define IDS_POLICY_SKYVAULT_MIGRATION_UPLOAD_ERROR_FILE_NOT_EXIST 37976
#define IDS_POLICY_SKYVAULT_MIGRATION_UPLOAD_ERROR_ODFS_SIGN_IN 37977
#define IDS_POLICY_SKYVAULT_MIGRATION_UPLOAD_ERROR_NO_INTERNET 37978
#define IDS_POLICY_SKYVAULT_MIGRATION_UPLOAD_ERROR_GENERIC 37979
#define PRINT_PREVIEW_MEDIA_ASME_F_28X40IN 37980
#define PRINT_PREVIEW_MEDIA_ISO_2A0_1189X1682MM 37981
#define PRINT_PREVIEW_MEDIA_ISO_A0_841X1189MM 37982
#define PRINT_PREVIEW_MEDIA_ISO_A0X3_1189X2523MM 37983
#define PRINT_PREVIEW_MEDIA_ISO_A10_26X37MM 37984
#define PRINT_PREVIEW_MEDIA_ISO_A1_594X841MM 37985
#define PRINT_PREVIEW_MEDIA_ISO_A1X3_841X1783MM 37986
#define PRINT_PREVIEW_MEDIA_ISO_A1X4_841X2378MM 37987
#define PRINT_PREVIEW_MEDIA_ISO_A2_420X594MM 37988
#define PRINT_PREVIEW_MEDIA_ISO_A2X3_594X1261MM 37989
#define PRINT_PREVIEW_MEDIA_ISO_A2X4_594X1682MM 37990
#define PRINT_PREVIEW_MEDIA_ISO_A2X5_594X2102MM 37991
#define PRINT_PREVIEW_MEDIA_ISO_A3_297X420MM 37992
#define PRINT_PREVIEW_MEDIA_ISO_A3_EXTRA_322X445MM 37993
#define PRINT_PREVIEW_MEDIA_ISO_A3X3_420X891MM 37994
#define PRINT_PREVIEW_MEDIA_ISO_A3X4_420X1189MM 37995
#define PRINT_PREVIEW_MEDIA_ISO_A3X5_420X1486MM 37996
#define PRINT_PREVIEW_MEDIA_ISO_A3X6_420X1783MM 37997
#define PRINT_PREVIEW_MEDIA_ISO_A3X7_420X2080MM 37998
#define PRINT_PREVIEW_MEDIA_ISO_A4_210X297MM 37999
#define PRINT_PREVIEW_MEDIA_ISO_A4_EXTRA_235_5X322_3MM 38000
#define PRINT_PREVIEW_MEDIA_ISO_A4_TAB_225X297MM 38001
#define PRINT_PREVIEW_MEDIA_ISO_A4X3_297X630MM 38002
#define PRINT_PREVIEW_MEDIA_ISO_A4X4_297X841MM 38003
#define PRINT_PREVIEW_MEDIA_ISO_A4X5_297X1051MM 38004
#define PRINT_PREVIEW_MEDIA_ISO_A4X6_297X1261MM 38005
#define PRINT_PREVIEW_MEDIA_ISO_A4X7_297X1471MM 38006
#define PRINT_PREVIEW_MEDIA_ISO_A4X8_297X1682MM 38007
#define PRINT_PREVIEW_MEDIA_ISO_A4X9_297X1892MM 38008
#define PRINT_PREVIEW_MEDIA_ISO_A5_148X210MM 38009
#define PRINT_PREVIEW_MEDIA_ISO_A5_EXTRA_174X235MM 38010
#define PRINT_PREVIEW_MEDIA_ISO_A6_105X148MM 38011
#define PRINT_PREVIEW_MEDIA_ISO_A7_74X105MM 38012
#define PRINT_PREVIEW_MEDIA_ISO_A8_52X74MM 38013
#define PRINT_PREVIEW_MEDIA_ISO_A9_37X52MM 38014
#define PRINT_PREVIEW_MEDIA_ISO_B0_1000X1414MM 38015
#define PRINT_PREVIEW_MEDIA_ISO_B10_31X44MM 38016
#define PRINT_PREVIEW_MEDIA_ISO_B1_707X1000MM 38017
#define PRINT_PREVIEW_MEDIA_ISO_B2_500X707MM 38018
#define PRINT_PREVIEW_MEDIA_ISO_B3_353X500MM 38019
#define PRINT_PREVIEW_MEDIA_ISO_B4_250X353MM 38020
#define PRINT_PREVIEW_MEDIA_ISO_B5_176X250MM 38021
#define PRINT_PREVIEW_MEDIA_ISO_B5_EXTRA_201X276MM 38022
#define PRINT_PREVIEW_MEDIA_ISO_B6C4_125X324MM 38023
#define PRINT_PREVIEW_MEDIA_ISO_B6_125X176MM 38024
#define PRINT_PREVIEW_MEDIA_ISO_B7_88X125MM 38025
#define PRINT_PREVIEW_MEDIA_ISO_B8_62X88MM 38026
#define PRINT_PREVIEW_MEDIA_ISO_B9_44X62MM 38027
#define PRINT_PREVIEW_MEDIA_ISO_C0_917X1297MM 38028
#define PRINT_PREVIEW_MEDIA_ISO_C10_28X40MM 38029
#define PRINT_PREVIEW_MEDIA_ISO_C1_648X917MM 38030
#define PRINT_PREVIEW_MEDIA_ISO_C2_458X648MM 38031
#define PRINT_PREVIEW_MEDIA_ISO_C3_324X458MM 38032
#define PRINT_PREVIEW_MEDIA_ISO_C4_229X324MM 38033
#define PRINT_PREVIEW_MEDIA_ISO_C5_162X229MM 38034
#define PRINT_PREVIEW_MEDIA_ISO_C6C5_114X229MM 38035
#define PRINT_PREVIEW_MEDIA_ISO_C6_114X162MM 38036
#define PRINT_PREVIEW_MEDIA_ISO_C7C6_81X162MM 38037
#define PRINT_PREVIEW_MEDIA_ISO_C7_81X114MM 38038
#define PRINT_PREVIEW_MEDIA_ISO_C8_57X81MM 38039
#define PRINT_PREVIEW_MEDIA_ISO_C9_40X57MM 38040
#define PRINT_PREVIEW_MEDIA_ISO_DL_110X220MM 38041
#define PRINT_PREVIEW_MEDIA_ISO_ID_1_53_98X85_6MM 38042
#define PRINT_PREVIEW_MEDIA_ISO_RA0_860X1220MM 38043
#define PRINT_PREVIEW_MEDIA_ISO_RA1_610X860MM 38044
#define PRINT_PREVIEW_MEDIA_ISO_RA2_430X610MM 38045
#define PRINT_PREVIEW_MEDIA_ISO_RA3_305X430MM 38046
#define PRINT_PREVIEW_MEDIA_ISO_RA4_215X305MM 38047
#define PRINT_PREVIEW_MEDIA_ISO_SRA0_900X1280MM 38048
#define PRINT_PREVIEW_MEDIA_ISO_SRA1_640X900MM 38049
#define PRINT_PREVIEW_MEDIA_ISO_SRA2_450X640MM 38050
#define PRINT_PREVIEW_MEDIA_ISO_SRA3_320X450MM 38051
#define PRINT_PREVIEW_MEDIA_ISO_SRA4_225X320MM 38052
#define PRINT_PREVIEW_MEDIA_JIS_B0_1030X1456MM 38053
#define PRINT_PREVIEW_MEDIA_JIS_B1_728X1030MM 38054
#define PRINT_PREVIEW_MEDIA_JIS_B2_515X728MM 38055
#define PRINT_PREVIEW_MEDIA_JIS_B3_364X515MM 38056
#define PRINT_PREVIEW_MEDIA_JIS_B4_257X364MM 38057
#define PRINT_PREVIEW_MEDIA_JIS_B5_182X257MM 38058
#define PRINT_PREVIEW_MEDIA_JIS_B6_128X182MM 38059
#define PRINT_PREVIEW_MEDIA_JIS_B7_91X128MM 38060
#define PRINT_PREVIEW_MEDIA_JIS_B8_64X91MM 38061
#define PRINT_PREVIEW_MEDIA_JIS_B9_45X64MM 38062
#define PRINT_PREVIEW_MEDIA_JIS_B10_32X45MM 38063
#define PRINT_PREVIEW_MEDIA_JIS_EXEC_216X330MM 38064
#define PRINT_PREVIEW_MEDIA_JPN_CHOU2_111_1X146MM 38065
#define PRINT_PREVIEW_MEDIA_JPN_CHOU3_120X235MM 38066
#define PRINT_PREVIEW_MEDIA_JPN_CHOU4_90X205MM 38067
#define PRINT_PREVIEW_MEDIA_JPN_CHOU40_90X225MM 38068
#define PRINT_PREVIEW_MEDIA_JPN_HAGAKI_100X148MM 38069
#define PRINT_PREVIEW_MEDIA_JPN_KAHU_240X322_1MM 38070
#define PRINT_PREVIEW_MEDIA_JPN_KAKU1_270X382MM 38071
#define PRINT_PREVIEW_MEDIA_JPN_KAKU2_240X332MM 38072
#define PRINT_PREVIEW_MEDIA_JPN_KAKU3_216X277MM 38073
#define PRINT_PREVIEW_MEDIA_JPN_KAKU4_197X267MM 38074
#define PRINT_PREVIEW_MEDIA_JPN_KAKU5_190X240MM 38075
#define PRINT_PREVIEW_MEDIA_JPN_KAKU7_142X205MM 38076
#define PRINT_PREVIEW_MEDIA_JPN_KAKU8_119X197MM 38077
#define PRINT_PREVIEW_MEDIA_JPN_OUFUKU_148X200MM 38078
#define PRINT_PREVIEW_MEDIA_JPN_YOU4_105X235MM 38079
#define PRINT_PREVIEW_MEDIA_JPN_YOU6_98X190MM 38080
#define PRINT_PREVIEW_MEDIA_NA_10X11_10X11IN 38081
#define PRINT_PREVIEW_MEDIA_NA_10X13_10X13IN 38082
#define PRINT_PREVIEW_MEDIA_NA_10X14_10X14IN 38083
#define PRINT_PREVIEW_MEDIA_NA_10X15_10X15IN 38084
#define PRINT_PREVIEW_MEDIA_NA_11X12_11X12IN 38085
#define PRINT_PREVIEW_MEDIA_NA_11X15_11X15IN 38086
#define PRINT_PREVIEW_MEDIA_NA_12X19_12X19IN 38087
#define PRINT_PREVIEW_MEDIA_NA_5X7_5X7IN 38088
#define PRINT_PREVIEW_MEDIA_NA_6X9_6X9IN 38089
#define PRINT_PREVIEW_MEDIA_NA_7X9_7X9IN 38090
#define PRINT_PREVIEW_MEDIA_NA_9X11_9X11IN 38091
#define PRINT_PREVIEW_MEDIA_NA_A2_4_375X5_75IN 38092
#define PRINT_PREVIEW_MEDIA_NA_ARCH_A_9X12IN 38093
#define PRINT_PREVIEW_MEDIA_NA_ARCH_B_12X18IN 38094
#define PRINT_PREVIEW_MEDIA_NA_ARCH_C_18X24IN 38095
#define PRINT_PREVIEW_MEDIA_NA_ARCH_D_24X36IN 38096
#define PRINT_PREVIEW_MEDIA_NA_ARCH_E_36X48IN 38097
#define PRINT_PREVIEW_MEDIA_NA_ARCH_E2_26X38IN 38098
#define PRINT_PREVIEW_MEDIA_NA_ARCH_E3_27X39IN 38099
#define PRINT_PREVIEW_MEDIA_NA_B_PLUS_12X19_17IN 38100
#define PRINT_PREVIEW_MEDIA_NA_C5_6_5X9_5IN 38101
#define PRINT_PREVIEW_MEDIA_NA_C_17X22IN 38102
#define PRINT_PREVIEW_MEDIA_NA_D_22X34IN 38103
#define PRINT_PREVIEW_MEDIA_NA_EDP_11X14IN 38104
#define PRINT_PREVIEW_MEDIA_NA_EUR_EDP_12X14IN 38105
#define PRINT_PREVIEW_MEDIA_NA_E_34X44IN 38106
#define PRINT_PREVIEW_MEDIA_NA_EXECUTIVE_7_25X10_5IN 38107
#define PRINT_PREVIEW_MEDIA_NA_FANFOLD_EUR_8_5X12IN 38108
#define PRINT_PREVIEW_MEDIA_NA_FANFOLD_US_11X14_875IN 38109
#define PRINT_PREVIEW_MEDIA_NA_FOOLSCAP_8_5X13IN 38110
#define PRINT_PREVIEW_MEDIA_NA_F_44X68IN 38111
#define PRINT_PREVIEW_MEDIA_NA_GOVT_LEGAL_8X13IN 38112
#define PRINT_PREVIEW_MEDIA_NA_GOVT_LETTER_8X10IN 38113
#define PRINT_PREVIEW_MEDIA_NA_INDEX_3X5_3X5IN 38114
#define PRINT_PREVIEW_MEDIA_NA_INDEX_4X6_4X6IN 38115
#define PRINT_PREVIEW_MEDIA_NA_INDEX_4X6_EXT_6X8IN 38116
#define PRINT_PREVIEW_MEDIA_NA_INDEX_5X8_5X8IN 38117
#define PRINT_PREVIEW_MEDIA_NA_INVOICE_5_5X8_5IN 38118
#define PRINT_PREVIEW_MEDIA_NA_LEDGER_11X17IN 38119
#define PRINT_PREVIEW_MEDIA_NA_LEGAL_8_5X14IN 38120
#define PRINT_PREVIEW_MEDIA_NA_LEGAL_EXTRA_9_5X15IN 38121
#define PRINT_PREVIEW_MEDIA_NA_LETTER_8_5X11IN 38122
#define PRINT_PREVIEW_MEDIA_NA_LETTER_EXTRA_9_5X12IN 38123
#define PRINT_PREVIEW_MEDIA_NA_LETTER_PLUS_8_5X12_69IN 38124
#define PRINT_PREVIEW_MEDIA_NA_MONARCH_3_875X7_5IN 38125
#define PRINT_PREVIEW_MEDIA_NA_NUMBER_9_3_875X8_875IN 38126
#define PRINT_PREVIEW_MEDIA_NA_NUMBER_10_4_125X9_5IN 38127
#define PRINT_PREVIEW_MEDIA_NA_NUMBER_11_4_5X10_375IN 38128
#define PRINT_PREVIEW_MEDIA_NA_NUMBER_12_4_75X11IN 38129
#define PRINT_PREVIEW_MEDIA_NA_NUMBER_14_5X11_5IN 38130
#define PRINT_PREVIEW_MEDIA_NA_OFICIO_8_5X13_4IN 38131
#define PRINT_PREVIEW_MEDIA_NA_PERSONAL_3_625X6_5IN 38132
#define PRINT_PREVIEW_MEDIA_NA_QUARTO_8_5X10_83IN 38133
#define PRINT_PREVIEW_MEDIA_NA_SUPER_A_8_94X14IN 38134
#define PRINT_PREVIEW_MEDIA_NA_SUPER_B_13X19IN 38135
#define PRINT_PREVIEW_MEDIA_NA_WIDE_FORMAT_30X42IN 38136
#define PRINT_PREVIEW_MEDIA_OE_12X16_12X16IN 38137
#define PRINT_PREVIEW_MEDIA_OE_14X17_14X17IN 38138
#define PRINT_PREVIEW_MEDIA_OE_18X22_18X22IN 38139
#define PRINT_PREVIEW_MEDIA_OE_A2PLUS_17X24IN 38140
#define PRINT_PREVIEW_MEDIA_OE_BUSINESS_CARD_2X3_5IN 38141
#define PRINT_PREVIEW_MEDIA_OE_PHOTO_10R_10X12IN 38142
#define PRINT_PREVIEW_MEDIA_OE_PHOTO_12R_12X15IN 38143
#define PRINT_PREVIEW_MEDIA_OE_PHOTO_14X18_14X18IN 38144
#define PRINT_PREVIEW_MEDIA_OE_PHOTO_16R_16X20IN 38145
#define PRINT_PREVIEW_MEDIA_OE_PHOTO_20R_20X24IN 38146
#define PRINT_PREVIEW_MEDIA_OE_PHOTO_22R_22X29_5IN 38147
#define PRINT_PREVIEW_MEDIA_OE_PHOTO_22X28_22X28IN 38148
#define PRINT_PREVIEW_MEDIA_OE_PHOTO_24R_24X31_5IN 38149
#define PRINT_PREVIEW_MEDIA_OE_PHOTO_24X30_24X30IN 38150
#define PRINT_PREVIEW_MEDIA_OE_PHOTO_30R_30X40IN 38151
#define PRINT_PREVIEW_MEDIA_OE_PHOTO_L_3_5X5IN 38152
#define PRINT_PREVIEW_MEDIA_OE_PHOTO_S8R_8X12IN 38153
#define PRINT_PREVIEW_MEDIA_OE_SQUARE_PHOTO_4X4IN 38154
#define PRINT_PREVIEW_MEDIA_OE_SQUARE_PHOTO_5X5IN 38155
#define PRINT_PREVIEW_MEDIA_OM_16K_184X260MM 38156
#define PRINT_PREVIEW_MEDIA_OM_16K_195X270MM 38157
#define PRINT_PREVIEW_MEDIA_OM_BUSINESS_CARD_55X85MM 38158
#define PRINT_PREVIEW_MEDIA_OM_BUSINESS_CARD_55X91MM 38159
#define PRINT_PREVIEW_MEDIA_OM_CARD_54X86MM 38160
#define PRINT_PREVIEW_MEDIA_OM_DAI_PA_KAI_275X395MM 38161
#define PRINT_PREVIEW_MEDIA_OM_DSC_PHOTO_89X119MM 38162
#define PRINT_PREVIEW_MEDIA_OM_FOLIO_210X330MM 38163
#define PRINT_PREVIEW_MEDIA_OM_FOLIO_SP_215X315MM 38164
#define PRINT_PREVIEW_MEDIA_OM_INVITE_220X220MM 38165
#define PRINT_PREVIEW_MEDIA_OM_ITALIAN_110X230MM 38166
#define PRINT_PREVIEW_MEDIA_OM_JUURO_KU_KAI_198X275MM 38167
#define PRINT_PREVIEW_MEDIA_OM_LARGE_PHOTO_200X300 38168
#define PRINT_PREVIEW_MEDIA_OM_MEDIUM_PHOTO_130X180MM 38169
#define PRINT_PREVIEW_MEDIA_OM_PA_KAI_267X389MM 38170
#define PRINT_PREVIEW_MEDIA_OM_PHOTO_30X40_300X400MM 38171
#define PRINT_PREVIEW_MEDIA_OM_PHOTO_30X45_300X450MM 38172
#define PRINT_PREVIEW_MEDIA_OM_PHOTO_35X46_350X460MM 38173
#define PRINT_PREVIEW_MEDIA_OM_PHOTO_40X60_400X600MM 38174
#define PRINT_PREVIEW_MEDIA_OM_PHOTO_50X75_500X750MM 38175
#define PRINT_PREVIEW_MEDIA_OM_PHOTO_50X76_500X760MM 38176
#define PRINT_PREVIEW_MEDIA_OM_PHOTO_60X90_600X900MM 38177
#define PRINT_PREVIEW_MEDIA_OM_SQUARE_PHOTO_89X89MM 38178
#define PRINT_PREVIEW_MEDIA_OM_WIDE_PHOTO_100X200MM 38179
#define PRINT_PREVIEW_MEDIA_OM_SMALL_PHOTO_100X150MM 38180
#define PRINT_PREVIEW_MEDIA_PRC_16K_146X215MM 38181
#define PRINT_PREVIEW_MEDIA_PRC_1_102X165MM 38182
#define PRINT_PREVIEW_MEDIA_PRC_2_102X176MM 38183
#define PRINT_PREVIEW_MEDIA_PRC_32K_97X151MM 38184
#define PRINT_PREVIEW_MEDIA_PRC_4_110X208MM 38185
#define PRINT_PREVIEW_MEDIA_PRC_6_120X320MM 38186
#define PRINT_PREVIEW_MEDIA_PRC_7_160X230MM 38187
#define PRINT_PREVIEW_MEDIA_PRC_8_120X309MM 38188
#define PRINT_PREVIEW_MEDIA_ROC_16K_7_75X10_75IN 38189
#define PRINT_PREVIEW_MEDIA_ROC_8K_10_75X15_5IN 38190
#define PRINT_PREVIEW_MEDIA_DIMENSIONS_INCHES 38191
#define PRINT_PREVIEW_MEDIA_DIMENSIONS_MM 38192
#define PRINT_PREVIEW_MEDIA_NAME_WITH_DIMENSIONS_INCHES 38193
#define PRINT_PREVIEW_MEDIA_NAME_WITH_DIMENSIONS_MM 38194
#define IDS_PRINT_COMPOSITOR_SERVICE_DISPLAY_NAME 38196
#define IDS_PRIVACY_SANDBOX_M1_CONSENT_TITLE 38600
#define IDS_PRIVACY_SANDBOX_M1_CONSENT_DESCRIPTION_1 38601
#define IDS_PRIVACY_SANDBOX_M1_CONSENT_DESCRIPTION_2 38602
#define IDS_PRIVACY_SANDBOX_M1_CONSENT_DESCRIPTION_3 38603
#define IDS_PRIVACY_SANDBOX_M1_CONSENT_LEARN_MORE_EXPAND_LABEL 38604
#define IDS_PRIVACY_SANDBOX_M1_CONSENT_DESCRIPTION_4 38605
#define IDS_PRIVACY_SANDBOX_M1_CONSENT_LEARN_MORE_BULLET_1 38606
#define IDS_PRIVACY_SANDBOX_M1_CONSENT_LEARN_MORE_BULLET_2 38607
#define IDS_PRIVACY_SANDBOX_M1_CONSENT_LEARN_MORE_BULLET_3 38608
#define IDS_PRIVACY_SANDBOX_M1_CONSENT_LEARN_MORE_LINK 38609
#define IDS_SETTINGS_TOPICS_PAGE_TITLE 38610
#define IDS_SETTINGS_TOPICS_PAGE_TOGGLE_LABEL 38611
#define IDS_SETTINGS_TOPICS_PAGE_TOGGLE_SUB_LABEL_V2 38612
#define IDS_SETTINGS_TOPICS_PAGE_DISCLAIMER_DESKTOP 38613
#define IDS_SETTINGS_TOPICS_PAGE_CURRENT_TOPICS_DESCRIPTION_CANONICAL 38614
#define IDS_SETTINGS_TOPICS_PAGE_CURRENT_TOPICS_DESCRIPTION_DISABLED 38615
#define IDS_SETTINGS_TOPICS_PAGE_CURRENT_TOPICS_DESCRIPTION_EMPTY_TEXT_V2 38616
#define IDS_SETTINGS_TOPICS_PAGE_CURRENT_TOPICS_DESCRIPTION_EMPTY_TEXT_HEADING 38617
#define IDS_SETTINGS_TOPICS_PAGE_BLOCKED_TOPICS_DESCRIPTION_EMPTY_TEXT_V2 38618
#define IDS_SETTINGS_TOPICS_PAGE_BLOCKED_TOPICS_DESCRIPTION_EMPTY_TEXT_HEADING 38619
#define IDS_SETTINGS_TOPICS_PAGE_BLOCK_TOPIC 38620
#define IDS_SETTINGS_TOPICS_PAGE_BLOCK_TOPIC_A11Y_LABEL 38621
#define IDS_SETTINGS_TOPICS_PAGE_ALLOW_TOPIC 38622
#define IDS_SETTINGS_TOPICS_PAGE_ALLOW_TOPIC_A11Y_LABEL 38623
#define IDS_SETTINGS_TOPICS_PAGE_UNBLOCK_TOPIC_A11Y_LABEL 38624
#define IDS_SETTINGS_TOPICS_PAGE_FOOTER_CANONICAL 38625
#define IDS_SETTINGS_TOPICS_PAGE_FIRST_LEVEL_TOPIC_DESCRIPTOR 38626
#define IDS_SETTINGS_MANAGE_TOPICS_DIALOG_TITLE 38627
#define IDS_SETTINGS_MANAGE_TOPICS_DIALOG_BODY 38628
#define IDS_SETTINGS_MANAGE_TOPICS_DIALOG_CLANK_TITLE 38629
#define IDS_SETTINGS_MANAGE_TOPICS_DIALOG_CLANK_BODY 38630
#define IDS_SETTINGS_MANAGE_TOPICS_PAGE_DESCRIPTION 38631
#define IDS_SETTINGS_MANAGE_TOPICS_PAGE_LEARN_MORE_LINK 38632
#define IDS_SETTINGS_MANAGE_TOPICS_PAGE_DESCRIPTION_LEARN_MORE_ARIA_LABEL 38633
#define IDS_SETTINGS_UNBLOCK_TOPIC_TOAST_BODY 38634
#define IDS_SETTINGS_UNBLOCK_TOPIC_TOAST_BUTTON_TEXT 38635
#define IDS_SETTINGS_FLEDGE_PAGE_FOOTER_V2 38636
#define IDS_SETTINGS_UNBLOCK_TOPIC_BUTTON_TEXT_V2 38637
#define IDS_SETTINGS_TOPICS_PAGE_BLOCKED_TOPICS_HEADING_NEW 38638
#define IDS_SETTINGS_TOPICS_PAGE_BLOCKED_TOPICS_DESCRIPTION_NEW 38639
#define IDS_SETTINGS_TOPICS_PAGE_MANAGE_TOPICS_HEADING 38640
#define IDS_SETTINGS_TOPICS_PAGE_MANAGE_TOPICS_DESCRIPTION 38641
#define IDS_SETTINGS_TOPICS_PAGE_MANAGE_TOPICS_EXPLANATION 38642
#define IDS_SETTINGS_TOPICS_PAGE_ACTIVE_TOPICS_HEADING 38643
#define IDS_PRIVACY_SANDBOX_TOPICS_CONSENT_STATUS_LABEL 38644
#define IDS_PRIVACY_SANDBOX_TOPICS_CONSENT_LAST_UPDATE_TIME_LABEL 38645
#define IDS_PRIVACY_SANDBOX_TOPICS_CONSENT_LAST_UPDATE_SOURCE_LABEL 38646
#define IDS_PRIVACY_SANDBOX_TOPICS_CONSENT_LAST_UPDATE_TEXT_LABEL 38647
#define IDS_PRIVACY_SANDBOX_TOPICS_CONSENT_UPDATE_SOURCE_SETTINGS 38648
#define IDS_PRIVACY_SANDBOX_TOPICS_CONSENT_UPDATE_SOURCE_DEFAULT 38649
#define IDS_PRIVACY_SANDBOX_TOPICS_CONSENT_UPDATE_SOURCE_CONFIRMATION 38650
#define IDS_PRIVACY_SANDBOX_TOPICS_CONSENT_ACTIVE 38651
#define IDS_PRIVACY_SANDBOX_TOPICS_CONSENT_INACTIVE 38652
#define IDS_PRIVACY_SANDBOX_TOPICS_CONSENT_NOT_REQUIRED 38653
#define IDS_PRIVACY_SANDBOX_TOPICS_INVALID_TOPIC 38654
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_1 38655
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_2 38656
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_3 38657
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_4 38658
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_5 38659
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_6 38660
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_7 38661
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_8 38662
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_9 38663
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_10 38664
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_11 38665
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_12 38666
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_13 38667
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_14 38668
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_15 38669
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_16 38670
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_17 38671
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_18 38672
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_19 38673
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_20 38674
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_21 38675
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_22 38676
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_23 38677
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_24 38678
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_25 38679
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_26 38680
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_27 38681
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_28 38682
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_29 38683
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_30 38684
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_31 38685
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_32 38686
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_33 38687
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_34 38688
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_35 38689
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_36 38690
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_37 38691
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_38 38692
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_39 38693
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_40 38694
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_41 38695
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_42 38696
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_43 38697
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_44 38698
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_45 38699
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_46 38700
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_47 38701
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_48 38702
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_49 38703
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_50 38704
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_51 38705
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_52 38706
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_53 38707
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_54 38708
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_55 38709
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_56 38710
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_57 38711
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_58 38712
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_59 38713
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_60 38714
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_61 38715
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_62 38716
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_63 38717
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_64 38718
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_65 38719
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_66 38720
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_67 38721
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_68 38722
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_69 38723
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_70 38724
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_71 38725
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_72 38726
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_73 38727
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_74 38728
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_75 38729
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_76 38730
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_77 38731
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_78 38732
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_79 38733
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_80 38734
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_81 38735
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_82 38736
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_83 38737
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_84 38738
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_85 38739
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_86 38740
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_87 38741
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_88 38742
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_89 38743
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_90 38744
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_91 38745
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_92 38746
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_93 38747
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_94 38748
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_95 38749
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_96 38750
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_97 38751
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_98 38752
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_99 38753
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_100 38754
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_101 38755
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_102 38756
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_103 38757
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_104 38758
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_105 38759
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_106 38760
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_107 38761
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_108 38762
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_109 38763
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_110 38764
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_111 38765
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_112 38766
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_113 38767
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_114 38768
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_115 38769
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_116 38770
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_117 38771
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_118 38772
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_119 38773
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_120 38774
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_121 38775
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_122 38776
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_123 38777
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_124 38778
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_125 38779
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_126 38780
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_127 38781
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_128 38782
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_129 38783
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_130 38784
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_131 38785
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_132 38786
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_133 38787
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_134 38788
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_135 38789
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_136 38790
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_137 38791
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_138 38792
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_139 38793
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_140 38794
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_141 38795
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_142 38796
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_143 38797
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_144 38798
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_145 38799
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_146 38800
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_147 38801
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_148 38802
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_149 38803
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_150 38804
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_151 38805
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_152 38806
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_153 38807
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_154 38808
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_155 38809
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_156 38810
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_157 38811
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_158 38812
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_159 38813
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_160 38814
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_161 38815
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_162 38816
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_163 38817
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_164 38818
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_165 38819
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_166 38820
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_167 38821
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_168 38822
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_169 38823
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_170 38824
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_171 38825
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_172 38826
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_173 38827
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_174 38828
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_175 38829
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_176 38830
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_177 38831
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_178 38832
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_179 38833
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_180 38834
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_181 38835
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_182 38836
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_183 38837
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_184 38838
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_185 38839
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_186 38840
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_187 38841
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_188 38842
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_189 38843
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_190 38844
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_191 38845
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_192 38846
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_193 38847
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_194 38848
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_195 38849
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_196 38850
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_197 38851
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_198 38852
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_199 38853
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_200 38854
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_201 38855
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_202 38856
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_203 38857
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_204 38858
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_205 38859
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_206 38860
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_207 38861
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_208 38862
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_209 38863
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_210 38864
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_211 38865
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_212 38866
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_213 38867
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_214 38868
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_215 38869
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_216 38870
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_217 38871
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_218 38872
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_219 38873
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_220 38874
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_221 38875
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_222 38876
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_223 38877
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_224 38878
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_225 38879
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_226 38880
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_227 38881
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_228 38882
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_229 38883
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_230 38884
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_231 38885
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_232 38886
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_233 38887
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_234 38888
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_235 38889
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_236 38890
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_237 38891
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_238 38892
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_239 38893
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_240 38894
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_241 38895
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_242 38896
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_243 38897
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_244 38898
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_245 38899
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_246 38900
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_247 38901
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_248 38902
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_249 38903
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_250 38904
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_251 38905
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_252 38906
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_253 38907
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_254 38908
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_255 38909
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_256 38910
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_257 38911
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_258 38912
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_259 38913
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_260 38914
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_261 38915
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_262 38916
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_263 38917
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_264 38918
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_265 38919
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_266 38920
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_267 38921
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_268 38922
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_269 38923
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_270 38924
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_271 38925
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_272 38926
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_273 38927
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_274 38928
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_275 38929
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_276 38930
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_277 38931
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_278 38932
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_279 38933
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_280 38934
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_281 38935
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_282 38936
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_283 38937
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_284 38938
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_285 38939
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_286 38940
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_287 38941
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_288 38942
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_289 38943
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_290 38944
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_291 38945
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_292 38946
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_293 38947
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_294 38948
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_295 38949
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_296 38950
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_297 38951
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_298 38952
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_299 38953
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_300 38954
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_301 38955
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_302 38956
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_303 38957
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_304 38958
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_305 38959
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_306 38960
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_307 38961
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_308 38962
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_309 38963
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_310 38964
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_311 38965
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_312 38966
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_313 38967
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_314 38968
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_315 38969
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_316 38970
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_317 38971
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_318 38972
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_319 38973
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_320 38974
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_321 38975
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_322 38976
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_323 38977
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_324 38978
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_325 38979
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_326 38980
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_327 38981
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_328 38982
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_329 38983
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_330 38984
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_331 38985
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_332 38986
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_333 38987
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_334 38988
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_335 38989
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_336 38990
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_337 38991
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_338 38992
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_339 38993
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_340 38994
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_341 38995
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_342 38996
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_343 38997
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_344 38998
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_345 38999
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_346 39000
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_347 39001
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_348 39002
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V1_TOPIC_ID_349 39003
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_350 39004
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_351 39005
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_352 39006
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_353 39007
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_354 39008
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_355 39009
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_356 39010
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_357 39011
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_358 39012
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_359 39013
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_360 39014
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_361 39015
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_362 39016
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_363 39017
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_364 39018
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_365 39019
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_366 39020
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_367 39021
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_368 39022
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_369 39023
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_370 39024
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_371 39025
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_372 39026
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_373 39027
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_374 39028
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_375 39029
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_376 39030
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_377 39031
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_378 39032
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_379 39033
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_380 39034
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_381 39035
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_382 39036
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_383 39037
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_384 39038
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_385 39039
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_386 39040
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_387 39041
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_388 39042
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_389 39043
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_390 39044
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_391 39045
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_392 39046
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_393 39047
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_394 39048
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_395 39049
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_396 39050
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_397 39051
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_398 39052
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_399 39053
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_400 39054
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_401 39055
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_402 39056
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_403 39057
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_404 39058
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_405 39059
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_406 39060
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_407 39061
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_408 39062
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_409 39063
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_410 39064
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_411 39065
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_412 39066
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_413 39067
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_414 39068
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_415 39069
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_416 39070
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_417 39071
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_418 39072
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_419 39073
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_420 39074
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_421 39075
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_422 39076
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_423 39077
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_424 39078
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_425 39079
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_426 39080
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_427 39081
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_428 39082
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_429 39083
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_430 39084
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_431 39085
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_432 39086
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_433 39087
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_434 39088
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_435 39089
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_436 39090
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_437 39091
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_438 39092
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_439 39093
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_440 39094
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_441 39095
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_442 39096
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_443 39097
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_444 39098
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_445 39099
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_446 39100
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_447 39101
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_448 39102
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_449 39103
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_450 39104
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_451 39105
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_452 39106
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_453 39107
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_454 39108
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_455 39109
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_456 39110
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_457 39111
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_458 39112
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_459 39113
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_460 39114
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_461 39115
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_462 39116
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_463 39117
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_464 39118
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_465 39119
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_466 39120
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_467 39121
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_468 39122
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_469 39123
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_470 39124
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_471 39125
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_472 39126
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_473 39127
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_474 39128
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_475 39129
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_476 39130
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_477 39131
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_478 39132
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_479 39133
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_480 39134
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_481 39135
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_482 39136
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_483 39137
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_484 39138
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_485 39139
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_486 39140
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_487 39141
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_488 39142
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_489 39143
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_490 39144
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_491 39145
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_492 39146
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_493 39147
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_494 39148
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_495 39149
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_496 39150
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_497 39151
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_498 39152
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_499 39153
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_500 39154
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_501 39155
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_502 39156
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_503 39157
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_504 39158
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_505 39159
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_506 39160
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_507 39161
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_508 39162
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_509 39163
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_510 39164
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_511 39165
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_512 39166
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_513 39167
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_514 39168
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_515 39169
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_516 39170
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_517 39171
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_518 39172
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_519 39173
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_520 39174
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_521 39175
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_522 39176
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_523 39177
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_524 39178
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_525 39179
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_526 39180
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_527 39181
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_528 39182
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_529 39183
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_530 39184
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_531 39185
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_532 39186
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_533 39187
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_534 39188
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_535 39189
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_536 39190
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_537 39191
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_538 39192
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_539 39193
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_540 39194
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_541 39195
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_542 39196
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_543 39197
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_544 39198
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_545 39199
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_546 39200
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_547 39201
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_548 39202
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_549 39203
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_550 39204
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_551 39205
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_552 39206
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_553 39207
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_554 39208
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_555 39209
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_556 39210
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_557 39211
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_558 39212
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_559 39213
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_560 39214
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_561 39215
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_562 39216
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_563 39217
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_564 39218
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_565 39219
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_566 39220
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_567 39221
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_568 39222
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_569 39223
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_570 39224
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_571 39225
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_572 39226
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_573 39227
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_574 39228
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_575 39229
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_576 39230
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_577 39231
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_578 39232
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_579 39233
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_580 39234
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_581 39235
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_582 39236
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_583 39237
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_584 39238
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_585 39239
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_586 39240
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_587 39241
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_588 39242
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_589 39243
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_590 39244
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_591 39245
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_592 39246
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_593 39247
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_594 39248
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_595 39249
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_596 39250
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_597 39251
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_598 39252
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_599 39253
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_600 39254
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_601 39255
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_602 39256
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_603 39257
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_604 39258
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_605 39259
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_606 39260
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_607 39261
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_608 39262
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_609 39263
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_610 39264
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_611 39265
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_612 39266
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_613 39267
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_614 39268
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_615 39269
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_616 39270
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_617 39271
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_618 39272
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_619 39273
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_620 39274
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_621 39275
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_622 39276
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_623 39277
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_624 39278
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_625 39279
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_626 39280
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_627 39281
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_628 39282
#define IDS_PRIVACY_SANDBOX_TOPICS_TAXONOMY_V2_TOPIC_ID_629 39283
#define IDS_REGISTER_PROTOCOL_HANDLER_TOOLTIP 39284
#define IDS_REGISTER_PROTOCOL_HANDLER_MAILTO_NAME 39285
#define IDS_REGISTER_PROTOCOL_HANDLER_WEBCAL_NAME 39286
#define IDS_REGISTER_PROTOCOL_HANDLER_CONFIRM 39287
#define IDS_REGISTER_PROTOCOL_HANDLER_CONFIRM_REPLACE 39288
#define IDS_REGISTER_PROTOCOL_HANDLER_CONFIRM_FRAGMENT 39289
#define IDS_REGISTER_PROTOCOL_HANDLER_CONFIRM_REPLACE_FRAGMENT 39290
#define IDS_REGISTER_PROTOCOL_HANDLER_ACCEPT 39291
#define IDS_REGISTER_PROTOCOL_HANDLER_DENY 39292
#define IDS_REGISTER_PROTOCOL_HANDLER_IGNORE 39293
#define IDS_CONFIRM_PROTOCOL_HANDLER_TITLE 39294
#define IDS_CONFIRM_PROTOCOL_HANDLER_MESSAGE 39295
#define IDS_CONFIRM_PROTOCOL_HANDLER_MESSAGE_WITH_INITIATING_ORIGIN 39296
#define IDS_CONFIRM_PROTOCOL_HANDLER_REMEMBER 39297
#define IDS_RESET_PASSWORD_TITLE 39298
#define IDS_RESET_PASSWORD_WARNING_HEADING 39299
#define IDS_RESET_PASSWORD_WARNING_HEADING_V2 39300
#define IDS_RESET_PASSWORD_WARNING_HEADING_WITH_ORG_NAME 39301
#define IDS_RESET_PASSWORD_HEADING 39302
#define IDS_RESET_PASSWORD_WARNING_EXPLANATION_PARAGRAPH 39303
#define IDS_RESET_PASSWORD_WARNING_EXPLANATION_PARAGRAPH_V2 39304
#define IDS_RESET_PASSWORD_WARNING_EXPLANATION_PARAGRAPH_WITH_ORG_NAME 39305
#define IDS_RESET_PASSWORD_BUTTON 39306
#define IDS_RESET_PASSWORD_LEARN_MORE 39307
#define IDS_RESET_PASSWORD_EXPLANATION_PARAGRAPH 39308
#define IDS_RESET_PASSWORD_EXPLANATION_PARAGRAPH_WITH_ORG_NAME 39309
#define IDS_SAVED_TAB_GROUPS_CREATION_JUST_NOW 39310
#define IDS_SAVED_TAB_GROUPS_CREATION_MINUTES_AGO_FORMAT 39311
#define IDS_SAVED_TAB_GROUPS_CREATION_HOURS_AGO_FORMAT 39312
#define IDS_SAVED_TAB_GROUPS_CREATION_DAYS_AGO_FORMAT 39313
#define IDS_SAVED_TAB_GROUPS_CREATION_MONTHS_AGO_FORMAT 39314
#define IDS_SAVED_TAB_GROUPS_CREATION_YEARS_AGO_FORMAT 39315
#define IDS_SEARCH_ENGINE_CHOICE_SETTINGS_SUBTITLE 39316
#define IDS_SEARCH_ENGINE_CHOICE_SETTINGS_SUBTITLE_NON_EEA 39317
#define IDS_SEARCH_ENGINE_CHOICE_SETTINGS_CHANGE_DEFAULT_ENGINE 39318
#define IDS_SEARCH_ENGINE_CHOICE_SETTINGS_CONFIRMATION_TOAST_LABEL 39319
#define IDS_SEARCH_ENGINE_CHOICE_SETTINGS_ENTRY_POINT_SUBTITLE 39320
#define IDS_SEARCH_ENGINE_CHOICE_BUTTON_TITLE 39321
#define IDS_SEARCH_ENGINE_CHOICE_PAGE_TITLE 39322
#define IDS_SEARCH_ENGINE_CHOICE_PAGE_SUBTITLE 39323
#define IDS_SEARCH_ENGINE_CHOICE_PAGE_SUBTITLE_INFO_LINK 39324
#define IDS_SEARCH_ENGINE_CHOICE_PAGE_SUBTITLE_INFO_LINK_A11Y_LABEL 39325
#define IDS_SEARCH_ENGINE_CHOICE_INFO_DIALOG_TITLE 39332
#define IDS_SEARCH_ENGINE_CHOICE_INFO_DIALOG_BODY_FIRST_PARAGRAPH 39333
#define IDS_SEARCH_ENGINE_CHOICE_INFO_DIALOG_BODY_SECOND_PARAGRAPH 39334
#define IDS_SEARCH_ENGINE_CHOICE_INFO_DIALOG_BODY_THIRD_PARAGRAPH 39335
#define IDS_SEARCH_ENGINE_FALLBACK_MARKETING_SNIPPET 39336
#define IDS_SEARCH_ENGINE_CHOICE_MORE_BUTTON 39337
#define IDS_SEARCH_ENGINE_CHOICE_CONTINUE_BUTTON 39338
#define IDS_SEARCH_ENGINE_CHOICE_CHEVRON_A11Y_LABEL 39339
#define IDS_SEARCH_ENGINE_CHOICE_LIST_A11Y_LABEL 39340
#define IDS_SEARCH_ENGINE_CHOICE_GUEST_SESSION_CHECKBOX 39341
#define IDS_SSL_OPEN_DETAILS_BUTTON 39342
#define IDS_SSL_CLOSE_DETAILS_BUTTON 39343
#define IDS_CAPTIVE_PORTAL_AUTHORIZATION_DIALOG_NAME 39344
#define IDS_CAPTIVE_PORTAL_HEADING_WIRED 39345
#define IDS_CAPTIVE_PORTAL_HEADING_WIFI 39346
#define IDS_CAPTIVE_PORTAL_PRIMARY_PARAGRAPH_WIRED 39347
#define IDS_CAPTIVE_PORTAL_PRIMARY_PARAGRAPH_WIFI 39348
#define IDS_CAPTIVE_PORTAL_PRIMARY_PARAGRAPH_WIFI_SSID 39349
#define IDS_CAPTIVE_PORTAL_PRIMARY_PARAGRAPH_NO_LOGIN_URL_WIRED 39350
#define IDS_CAPTIVE_PORTAL_PRIMARY_PARAGRAPH_NO_LOGIN_URL_WIFI 39351
#define IDS_CAPTIVE_PORTAL_PRIMARY_PARAGRAPH_NO_LOGIN_URL_WIFI_SSID 39352
#define IDS_CAPTIVE_PORTAL_BUTTON_OPEN_LOGIN_PAGE 39353
#define IDS_MITM_SOFTWARE_HEADING 39354
#define IDS_MITM_SOFTWARE_PRIMARY_PARAGRAPH_ENTERPRISE 39355
#define IDS_MITM_SOFTWARE_PRIMARY_PARAGRAPH_NONENTERPRISE 39356
#define IDS_MITM_SOFTWARE_EXPLANATION_ENTERPRISE 39357
#define IDS_MITM_SOFTWARE_EXPLANATION_NONENTERPRISE 39358
#define IDS_MITM_SOFTWARE_EXPLANATION 39359
#define IDS_LOOKALIKE_URL_TITLE 39360
#define IDS_LOOKALIKE_URL_HEADING 39361
#define IDS_LOOKALIKE_URL_IGNORE 39362
#define IDS_LOOKALIKE_URL_CONTINUE 39363
#define IDS_LOOKALIKE_URL_PRIMARY_PARAGRAPH 39364
#define IDS_LOOKALIKE_URL_HEADING_NO_SUGGESTED_URL 39365
#define IDS_LOOKALIKE_URL_PRIMARY_PARAGRAPH_NO_SUGGESTED_URL 39366
#define IDS_LOOKALIKE_URL_BACK_TO_SAFETY 39367
#define IDS_LOOKALIKE_URL_CLOSE_PAGE 39368
#define IDS_ENTERPRISE_BLOCK_TITLE 39369
#define IDS_ENTERPRISE_BLOCK_HEADING 39370
#define IDS_ENTERPRISE_BLOCK_PRIMARY_PARAGRAPH 39371
#define IDS_ENTERPRISE_BLOCK_PRIMARY_PARAGRAPH_CUSTOM_MESSAGE 39372
#define IDS_ENTERPRISE_BLOCK_GO_BACK 39373
#define IDS_ENTERPRISE_WARN_TITLE 39374
#define IDS_ENTERPRISE_WARN_HEADING 39375
#define IDS_ENTERPRISE_WARN_PRIMARY_PARAGRAPH 39376
#define IDS_ENTERPRISE_WARN_PRIMARY_PARAGRAPH_CUSTOM_MESSAGE 39377
#define IDS_ENTERPRISE_WARN_GO_BACK 39378
#define IDS_ENTERPRISE_WARN_CONTINUE_TO_SITE 39379
#define IDS_ENTERPRISE_INTERSTITIALS_LEARN_MORE_ACCCESSIBILITY_TEXT 39380
#define IDS_CLOCK_ERROR_TITLE 39381
#define IDS_CLOCK_ERROR_AHEAD_HEADING 39382
#define IDS_CLOCK_ERROR_BEHIND_HEADING 39383
#define IDS_CLOCK_ERROR_UPDATE_DATE_AND_TIME 39384
#define IDS_CLOCK_ERROR_PRIMARY_PARAGRAPH 39385
#define IDS_CLOCK_ERROR_EXPLANATION 39386
#define IDS_SSL_V2_TITLE 39387
#define IDS_SSL_V2_HEADING 39388
#define IDS_SSL_V2_PRIMARY_PARAGRAPH 39389
#define IDS_SSL_OVERRIDABLE_SAFETY_BUTTON 39390
#define IDS_SSL_OVERRIDABLE_CLOSE_PAGE_BUTTON 39391
#define IDS_SSL_OVERRIDABLE_PROCEED_PARAGRAPH 39392
#define IDS_SSL_RELOAD 39393
#define IDS_SSL_NONOVERRIDABLE_PINNED 39394
#define IDS_SSL_NONOVERRIDABLE_HSTS 39395
#define IDS_SSL_NONOVERRIDABLE_REVOKED 39396
#define IDS_SSL_NONOVERRIDABLE_MORE 39397
#define IDS_SSL_NONOVERRIDABLE_INVALID 39398
#define IDS_SAFEBROWSING_V3_TITLE 39399
#define IDS_SAFEBROWSING_V3_OPEN_DETAILS_BUTTON 39400
#define IDS_SAFEBROWSING_V3_CLOSE_DETAILS_BUTTON 39401
#define IDS_SAFEBROWSING_OVERRIDABLE_SAFETY_BUTTON 39402
#define IDS_SAFEBROWSING_HEADING 39403
#define IDS_MALWARE_V3_PRIMARY_PARAGRAPH 39404
#define IDS_MALWARE_V3_EXPLANATION_PARAGRAPH 39405
#define IDS_MALWARE_V3_PROCEED_PARAGRAPH 39406
#define IDS_SAFE_BROWSING_SCOUT_REPORTING_AGREE 39407
#define IDS_SAFE_BROWSING_ENHANCED_PROTECTION_MESSAGE 39408
#define IDS_HARMFUL_V3_PRIMARY_PARAGRAPH 39409
#define IDS_HARMFUL_V3_EXPLANATION_PARAGRAPH 39410
#define IDS_HARMFUL_V3_PROCEED_PARAGRAPH 39411
#define IDS_PHISHING_V4_PRIMARY_PARAGRAPH 39412
#define IDS_PHISHING_V4_EXPLANATION_PARAGRAPH 39413
#define IDS_PHISHING_V4_PROCEED_PARAGRAPH 39414
#define IDS_MALWARE_WEBVIEW_HEADING 39415
#define IDS_MALWARE_WEBVIEW_EXPLANATION_PARAGRAPH 39416
#define IDS_PHISHING_WEBVIEW_HEADING 39417
#define IDS_PHISHING_WEBVIEW_EXPLANATION_PARAGRAPH 39418
#define IDS_HARMFUL_WEBVIEW_HEADING 39419
#define IDS_HARMFUL_WEBVIEW_EXPLANATION_PARAGRAPH 39420
#define IDS_BILLING_WEBVIEW_HEADING 39421
#define IDS_BILLING_WEBVIEW_EXPLANATION_PARAGRAPH 39422
#define IDS_SUPERVISED_USER_URL_BLOCKED_MESSAGE 39423
#define IDS_SUPERVISED_USER_LEARN_MORE 39424
#define IDS_CONNECTION_HELP_SHOW_MORE 39425
#define IDS_CONNECTION_HELP_SHOW_LESS 39426
#define IDS_CONNECTION_HELP_TITLE 39427
#define IDS_CONNECTION_HELP_HEADING 39428
#define IDS_CONNECTION_HELP_GENERAL_HELP 39429
#define IDS_CONNECTION_HELP_SPECIFIC_ERROR_HEADING 39430
#define IDS_CONNECTION_HELP_CONNECTION_NOT_PRIVATE_TITLE 39431
#define IDS_CONNECTION_HELP_CONNECT_TO_NETWORK_TITLE 39432
#define IDS_CONNECTION_HELP_INCORRECT_CLOCK_TITLE 39433
#define IDS_CONNECTION_HELP_CONNECTION_NOT_PRIVATE_DETAILS 39434
#define IDS_CONNECTION_HELP_CONNECT_TO_NETWORK_DETAILS 39435
#define IDS_CONNECTION_HELP_INCORRECT_CLOCK_DETAILS 39436
#define IDS_BILLING_TITLE 39439
#define IDS_BILLING_HEADING 39440
#define IDS_BILLING_PRIMARY_PARAGRAPH 39441
#define IDS_BILLING_PRIMARY_BUTTON 39442
#define IDS_BILLING_PROCEED_BUTTON 39443
#define IDS_BLOCKED_INTERCEPTION_HEADING 39444
#define IDS_KNOWN_INTERCEPTION_TITLE 39445
#define IDS_KNOWN_INTERCEPTION_HEADER 39446
#define IDS_KNOWN_INTERCEPTION_BODY1 39447
#define IDS_KNOWN_INTERCEPTION_BODY2 39448
#define IDS_KNOWN_INTERCEPTION_INFOBAR_HEADING 39449
#define IDS_KNOWN_INTERCEPTION_INFOBAR_BUTTON_TEXT 39450
#define IDS_LEGACY_TLS_HEADING 39451
#define IDS_LEGACY_TLS_PRIMARY_PARAGRAPH 39452
#define IDS_LEGACY_TLS_EXPLANATION 39453
#define IDS_INSECURE_FORM_TITLE 39454
#define IDS_INSECURE_FORM_HEADING 39455
#define IDS_INSECURE_FORM_PRIMARY_PARAGRAPH 39456
#define IDS_INSECURE_FORM_BACK_BUTTON 39457
#define IDS_INSECURE_FORM_SUBMIT_BUTTON 39458
#define IDS_HTTPS_ONLY_MODE_TITLE 39459
#define IDS_HTTPS_ONLY_MODE_HEADING 39460
#define IDS_HTTPS_ONLY_MODE_PRIMARY_PARAGRAPH 39461
#define IDS_HTTPS_ONLY_BALANCED_MODE_HEADING 39462
#define IDS_HTTPS_ONLY_BALANCED_MODE_PRIMARY_PARAGRAPH 39463
#define IDS_HTTPS_ONLY_MODE_WITH_ADVANCED_PROTECTION_PRIMARY_PARAGRAPH 39464
#define IDS_HTTPS_ONLY_MODE_WITH_SITE_ENGAGEMENT_PRIMARY_PARAGRAPH 39466
#define IDS_HTTPS_ONLY_MODE_FOR_TYPICALLY_SECURE_BROWSING_PRIMARY_PARAGRAPH 39467
#define IDS_HTTPS_ONLY_MODE_FOR_INCOGNITO_PRIMARY_PARAGRAPH 39468
#define IDS_HTTPS_ONLY_MODE_BACK_BUTTON 39469
#define IDS_HTTPS_ONLY_MODE_SUBMIT_BUTTON 39470
#define IDS_ABH_PROMPT_TITLE 39471
#define IDS_ABH_PROMPT_BALANCED_MODE_FIRST_ITEM_TEXT 39472
#define IDS_ABH_PROMPT_BALANCED_MODE_SECOND_ITEM_TEXT 39473
#define IDS_ABH_PROMPT_SECONDARY_TEXT 39474
#define IDS_ABH_PROMPT_LEARN_MORE_LINK 39475
#define IDS_ABH_PROMPT_SITE_ENGAGEMENT_PRIMARY_PARAGRAPH 39476
#define IDS_ABH_PROMPT_TYPICALLY_SECURE_BROWSING_PRIMARY_PARAGRAPH 39477
#define IDS_ABH_PROMPT_INCOGNITO_PRIMARY_PARAGRAPH 39478
#define IDS_ABH_PROMPT_ADVANCED_PROTECTION_PRIMARY_PARAGRAPH 39479
#define IDS_WEBVIEW_CONTENT_RESTRICTION_BLOCKED_MESSAGE 39480
#define IDS_WEBVIEW_CONTENT_RESTRICTION_LEARN_MORE 39481
#define IDS_WEBVIEW_CONTENT_RESTRICTION_GO_BACK 39482
#define IDS_SHARING_DEVICE_TYPE_COMPUTER 39483
#define IDS_SHARING_DEVICE_TYPE_DEVICE 39484
#define IDS_SHARING_DEVICE_TYPE_PHONE 39485
#define IDS_SHARING_DEVICE_TYPE_TABLET 39486
#define IDS_SEND_TAB_PUSH_NOTIFICATION_TITLE_DEAULT_DEVICE_NAME 39487
#define IDS_SEND_TAB_PUSH_NOTIFICATION_TITLE_USER_GIVEN_DEVICE_NAME 39488
#define IDS_SEND_TAB_PUSH_NOTIFICATION_BODY 39489
#define IDS_SEND_TAB_PUSH_NOTIFICATION_PLACEHOLDER_TITLE 39490
#define IDS_SEND_TAB_PUSH_NOTIFICATION_PLACEHOLDER_BODY 39491
#define IDS_SEND_TAB_RECEIVE_TOAST_FOREGROUND 39492
#define IDS_SEND_TAB_RECEIVE_TOAST_BACKGROUND 39493
#define IDS_SEND_TAB_RECEIVE_TOAST_BACKGROUND_ACTION_BUTTON 39494
#define IDS_SEND_TAB_TO_SELF_POST_SEND_SUCCESS_TOAST 39495
#define IDS_SEND_TAB_TO_SELF_POST_SEND_SUCCESS_TOAST_ANDROID 39496
#define IDS_SEND_TAB_TO_SELF_POST_SEND_THROTTLED_TOAST 39497
#define IDS_SEND_TAB_TO_SELF_POST_SEND_THROTTLED_TOAST_ANDROID 39498
#define IDS_SEND_TAB_TO_SELF_POST_SEND_NO_INTERNET_TOAST 39499
#define IDS_SEND_TAB_TO_SELF_POST_SEND_FAILURE_TOAST 39500
#define IDS_SITE_SETTINGS_TYPE_ADS 39501
#define IDS_SITE_SETTINGS_TYPE_ADS_MID_SENTENCE 39502
#define IDS_SITE_SETTINGS_TYPE_ANTI_ABUSE 39503
#define IDS_SITE_SETTINGS_TYPE_AR 39504
#define IDS_SITE_SETTINGS_TYPE_AR_MID_SENTENCE 39505
#define IDS_SITE_SETTINGS_TYPE_AUTOMATIC_DOWNLOADS 39506
#define IDS_SITE_SETTINGS_TYPE_AUTOMATIC_DOWNLOADS_MID_SENTENCE 39507
#define IDS_SITE_SETTINGS_TYPE_AUTOMATIC_FULLSCREEN 39508
#define IDS_SITE_SETTINGS_TYPE_AUTOMATIC_FULLSCREEN_MID_SENTENCE 39509
#define IDS_SITE_SETTINGS_TYPE_AUTO_PICTURE_IN_PICTURE 39510
#define IDS_SITE_SETTINGS_TYPE_AUTO_PICTURE_IN_PICTURE_MID_SENTENCE 39511
#define IDS_SITE_SETTINGS_TYPE_BACKGROUND_SYNC 39512
#define IDS_SITE_SETTINGS_TYPE_BACKGROUND_SYNC_MID_SENTENCE 39513
#define IDS_SITE_SETTINGS_TYPE_BLUETOOTH_DEVICES 39514
#define IDS_SITE_SETTINGS_TYPE_BLUETOOTH_DEVICES_MID_SENTENCE 39515
#define IDS_SITE_SETTINGS_TYPE_BLUETOOTH_SCANNING 39516
#define IDS_SITE_SETTINGS_TYPE_BLUETOOTH_SCANNING_MID_SENTENCE 39517
#define IDS_SITE_SETTINGS_TYPE_CAMERA 39518
#define IDS_SITE_SETTINGS_TYPE_CAMERA_MID_SENTENCE 39519
#define IDS_SITE_SETTINGS_TYPE_CAMERA_WITH_COUNT 39520
#define IDS_SITE_SETTINGS_TYPE_CAMERA_PAN_TILT_ZOOM 39521
#define IDS_SITE_SETTINGS_TYPE_CAMERA_PAN_TILT_ZOOM_MID_SENTENCE 39522
#define IDS_SITE_SETTINGS_TYPE_CAMERA_PAN_TILT_ZOOM_WITH_COUNT 39523
#define IDS_SITE_SETTINGS_TYPE_CAPTURED_SURFACE_CONTROL 39524
#define IDS_SITE_SETTINGS_TYPE_CAPTURED_SURFACE_CONTROL_MID_SENTENCE 39525
#define IDS_SITE_SETTINGS_TYPE_CAPTURED_SURFACE_CONTROL_SHARED_TABS 39526
#define IDS_SITE_SETTINGS_TYPE_CAPTURED_SURFACE_CONTROL_SHARED_TABS_MID_SENTENCE 39527
#define IDS_SITE_SETTINGS_TYPE_CAPTURED_SURFACE_CONTROL_SUB_MENU 39528
#define IDS_SITE_SETTINGS_TYPE_CLIPBOARD 39529
#define IDS_SITE_SETTINGS_TYPE_CLIPBOARD_MID_SENTENCE 39530
#define IDS_SITE_SETTINGS_TYPE_COOKIES 39531
#define IDS_SITE_SETTINGS_TYPE_COOKIES_MID_SENTENCE 39532
#define IDS_SITE_SETTINGS_TYPE_FEDERATED_IDENTITY_API 39533
#define IDS_SITE_SETTINGS_TYPE_FEDERATED_IDENTITY_API_MID_SENTENCE 39534
#define IDS_SITE_SETTINGS_TYPE_HAND_TRACKING 39535
#define IDS_SITE_SETTINGS_TYPE_HAND_TRACKING_MID_SENTENCE 39536
#define IDS_SITE_SETTINGS_TYPE_IDLE_DETECTION 39537
#define IDS_SITE_SETTINGS_TYPE_IDLE_DETECTION_MID_SENTENCE 39538
#define IDS_SITE_SETTINGS_TYPE_JAVASCRIPT 39539
#define IDS_SITE_SETTINGS_TYPE_JAVASCRIPT_MID_SENTENCE 39540
#define IDS_SITE_SETTINGS_TYPE_JAVASCRIPT_OPTIMIZER 39541
#define IDS_SITE_SETTINGS_TYPE_LOCATION 39542
#define IDS_SITE_SETTINGS_TYPE_LOCATION_MID_SENTENCE 39543
#define IDS_SITE_SETTINGS_TYPE_MIC 39544
#define IDS_SITE_SETTINGS_TYPE_MIC_MID_SENTENCE 39545
#define IDS_SITE_SETTINGS_TYPE_MIC_WITH_COUNT 39546
#define IDS_SITE_SETTINGS_TYPE_MIDI_SYSEX 39547
#define IDS_SITE_SETTINGS_TYPE_MIDI_SYSEX_MID_SENTENCE 39548
#define IDS_SITE_SETTINGS_TYPE_MOTION_AND_LIGHT_SENSORS 39549
#define IDS_SITE_SETTINGS_TYPE_MOTION_AND_LIGHT_SENSORS_MID_SENTENCE 39550
#define IDS_SITE_SETTINGS_TYPE_MOTION_SENSORS 39551
#define IDS_SITE_SETTINGS_TYPE_MOTION_SENSORS_MID_SENTENCE 39552
#define IDS_SITE_SETTINGS_TYPE_NFC 39553
#define IDS_SITE_SETTINGS_TYPE_NFC_MID_SENTENCE 39554
#define IDS_SITE_SETTINGS_TYPE_NOTIFICATIONS 39555
#define IDS_SITE_SETTINGS_TYPE_NOTIFICATIONS_MID_SENTENCE 39556
#define IDS_SITE_SETTINGS_TYPE_POPUPS_REDIRECTS 39557
#define IDS_SITE_SETTINGS_TYPE_POPUPS_REDIRECTS_MID_SENTENCE 39558
#define IDS_SITE_SETTINGS_TYPE_PROTECTED_MEDIA_ID 39559
#define IDS_SITE_SETTINGS_TYPE_PROTECTED_MEDIA_ID_MID_SENTENCE 39560
#define IDS_SITE_SETTINGS_TYPE_SOUND 39561
#define IDS_SITE_SETTINGS_TYPE_SOUND_MID_SENTENCE 39562
#define IDS_SITE_SETTINGS_TYPE_USB_DEVICES 39563
#define IDS_SITE_SETTINGS_TYPE_USB_DEVICES_MID_SENTENCE 39564
#define IDS_SITE_SETTINGS_TYPE_VR 39565
#define IDS_SITE_SETTINGS_TYPE_VR_MID_SENTENCE 39566
#define IDS_SITE_SETTINGS_TYPE_STORAGE_ACCESS 39567
#define IDS_SITE_SETTINGS_TYPE_STORAGE_ACCESS_MID_SENTENCE 39568
#define IDS_SITE_SETTINGS_TYPE_FILE_SYSTEM_ACCESS_WRITE 39569
#define IDS_SITE_SETTINGS_TYPE_FILE_SYSTEM_ACCESS_WRITE_MID_SENTENCE 39570
#define IDS_SITE_SETTINGS_TYPE_LOCAL_NETWORK 39571
#define IDS_SITE_SETTINGS_TYPE_LOCAL_NETWORK_MID_SENTENCE 39572
#define IDS_SITE_SETTINGS_TYPE_LOOPBACK_NETWORK 39573
#define IDS_SITE_SETTINGS_TYPE_LOOPBACK_NETWORK_MID_SENTENCE 39574
#define IDS_SITE_SETTINGS_TYPE_WINDOW_MANAGEMENT 39575
#define IDS_SITE_SETTINGS_TYPE_WINDOW_MANAGEMENT_MID_SENTENCE 39576
#define IDS_SITE_SETTINGS_TYPE_FONT_ACCESS 39577
#define IDS_SITE_SETTINGS_TYPE_FONT_ACCESS_MID_SENTENCE 39578
#define IDS_SITE_SETTINGS_TYPE_HANDLERS 39579
#define IDS_SITE_SETTINGS_TYPE_HANDLERS_MID_SENTENCE 39580
#define IDS_SITE_SETTINGS_TYPE_HID_DEVICES 39581
#define IDS_SITE_SETTINGS_TYPE_HID_DEVICES_MID_SENTENCE 39582
#define IDS_SITE_SETTINGS_TYPE_IMAGES 39583
#define IDS_SITE_SETTINGS_TYPE_IMAGES_MID_SENTENCE 39584
#define IDS_SITE_SETTINGS_TYPE_INSECURE_CONTENT 39585
#define IDS_SITE_SETTINGS_TYPE_INSECURE_CONTENT_MID_SENTENCE 39586
#define IDS_SITE_SETTINGS_TYPE_KEYBOARD_LOCK 39587
#define IDS_SITE_SETTINGS_TYPE_KEYBOARD_LOCK_MID_SENTENCE 39588
#define IDS_SITE_SETTINGS_TYPE_PAYMENT_HANDLER 39589
#define IDS_SITE_SETTINGS_TYPE_PAYMENT_HANDLER_MID_SENTENCE 39590
#define IDS_SITE_SETTINGS_TYPE_PDF_DOCUMENTS 39591
#define IDS_SITE_SETTINGS_TYPE_POINTER_LOCK 39592
#define IDS_SITE_SETTINGS_TYPE_POINTER_LOCK_MID_SENTENCE 39593
#define IDS_SITE_SETTINGS_TYPE_SERIAL_PORTS 39594
#define IDS_SITE_SETTINGS_TYPE_SERIAL_PORTS_MID_SENTENCE 39595
#define IDS_SITE_SETTINGS_TYPE_ZOOM_LEVELS 39597
#define IDS_SITE_SETTINGS_TYPE_ZOOM_LEVELS_MID_SENTENCE 39598
#define IDS_SITE_SETTINGS_TYPE_WEB_APP_INSTALLATION 39599
#define IDS_SITE_SETTINGS_TYPE_WEB_APP_INSTALLATION_MID_SENTENCE 39600
#define IDS_SITE_SETTINGS_TYPE_WEB_PRINTING 39601
#define IDS_SITE_SETTINGS_TYPE_WEB_PRINTING_MID_SENTENCE 39602
#define IDS_SITE_SETTINGS_TYPE_PERFORMANCE 39603
#define IDS_SITE_SETTINGS_TYPE_PERFORMANCE_SUBLABEL 39604
#define IDS_SITE_SETTINGS_TYPE_OFFER_WRITING_HELP 39605
#define IDS_STS_IPH_PROMPT_FIRST_TIME_ADDING_CONTEXT_HEADER 39606
#define IDS_STS_IPH_PROMPT_FIRST_TIME_ADDING_CONTEXT_HEADER_V2 39607
#define IDS_STS_IPH_PROMPT_FIRST_TIME_ADDING_CONTEXT_MESSAGE_BODY 39608
#define IDS_STS_IPH_PROMPT_FIRST_TIME_ADDING_CONTEXT_MESSAGE_BODY_V2 39609
#define IDS_STS_IPH_PROMPT_FIRST_TIME_ADDING_CONTEXT_TURN_ON 39610
#define IDS_STS_SETTINGS_OPTION_1_SHARE_OPEN_TABS_FOR_EVERY_THREAD_V3 39611
#define IDS_STS_SETTINGS_OPTION_1_CHOOSE_HOW_RELEVANT_OPEN_TABS_CAN_BE_USED_V3 39612
#define IDS_STS_SETTINGS_OPTION_1_LEARN_MORE 39613
#define IDS_STS_SETTINGS_OPTION_1_NEVER_SHARE_THESE_SITES 39614
#define IDS_STS_SETTINGS_OPTION_1_SITES_ADDED_HERE_WONT_BE_REFERENCED_V2 39615
#define IDS_STS_SETTINGS_OPTION_1_SITES_ADDED_HERE_WONT_BE_REFERENCED_V3 39616
#define IDS_STS_SETTINGS_ENTRYPOINT_GOOGLE_SEARCH_AI_MODE 39617
#define IDS_STS_SETTINGS_ENTRYPOINT_GET_RESPONSES_WITH_CONTEXT 39618
#define IDS_STS_MEGAPLUS_SHARE_RELEVANT_OPEN_TABS 39619
#define IDS_STS_MEGAPLUS_SHARE_RELEVANT_OPEN_TABS_V2 39620
#define IDS_STS_MEGAPLUS_SHARE_RELEVANT_OPEN_TABS_V3 39621
#define IDS_STS_IPH_TRY_IT_HEADER 39622
#define IDS_STS_IPH_TRY_IT_HEADER_V2 39623
#define IDS_STS_IPH_TRY_IT_BODY 39624
#define IDS_STS_IPH_TRY_IT_BODY_V2 39625
#define IDS_STS_IPH_TRY_IT_LINK 39626
#define IDS_STS_IPH_TRY_IT_BODY_END 39627
#define IDS_STS_IPH_TRY_IT_TURN_ON 39628
#define IDS_STS_IPH_TRY_IT_NOT_NOW 39629
#define IDS_STS_IPH_DEFAULT_ON_HEADER 39630
#define IDS_STS_IPH_DEFAULT_ON_HEADER_V2 39631
#define IDS_STS_IPH_DEFAULT_ON_BODY 39632
#define IDS_STS_IPH_DEFAULT_ON_BODY_V2 39633
#define IDS_STS_IPH_DEFAULT_ON_LINK 39634
#define IDS_STS_IPH_DEFAULT_ON_BODY_END 39635
#define IDS_STS_IPH_DEFAULT_ON_TURN_ON 39636
#define IDS_STS_IPH_DEFAULT_ON_NOT_NOW 39637
#define IDS_CERT_ERROR_NO_SUBJECT_ALTERNATIVE_NAMES_DETAILS 39638
#define IDS_CERT_ERROR_COMMON_NAME_INVALID_DETAILS 39639
#define IDS_CERT_ERROR_COMMON_NAME_INVALID_DESCRIPTION 39640
#define IDS_CERT_ERROR_EXPIRED_DETAILS 39641
#define IDS_CERT_ERROR_EXPIRED_DESCRIPTION 39642
#define IDS_CERT_ERROR_NOT_YET_VALID_DETAILS 39643
#define IDS_CERT_ERROR_NOT_YET_VALID_DESCRIPTION 39644
#define IDS_CERT_ERROR_NOT_VALID_AT_THIS_TIME_DETAILS 39645
#define IDS_CERT_ERROR_NOT_VALID_AT_THIS_TIME_DESCRIPTION 39646
#define IDS_CERT_ERROR_AUTHORITY_INVALID_DESCRIPTION 39647
#define IDS_CERT_ERROR_CONTAINS_ERRORS_DETAILS 39648
#define IDS_CERT_ERROR_CONTAINS_ERRORS_DESCRIPTION 39649
#define IDS_CERT_ERROR_UNABLE_TO_CHECK_REVOCATION_DETAILS 39650
#define IDS_CERT_ERROR_UNABLE_TO_CHECK_REVOCATION_DESCRIPTION 39651
#define IDS_CERT_ERROR_NO_REVOCATION_MECHANISM_DETAILS 39652
#define IDS_CERT_ERROR_NO_REVOCATION_MECHANISM_DESCRIPTION 39653
#define IDS_CERT_ERROR_REVOKED_CERT_DETAILS 39654
#define IDS_CERT_ERROR_REVOKED_CERT_DESCRIPTION 39655
#define IDS_CERT_ERROR_INVALID_CERT_DETAILS 39656
#define IDS_CERT_ERROR_INVALID_CERT_DESCRIPTION 39657
#define IDS_CERT_ERROR_WEAK_SIGNATURE_ALGORITHM_DETAILS 39658
#define IDS_CERT_ERROR_WEAK_SIGNATURE_ALGORITHM_DESCRIPTION 39659
#define IDS_CERT_ERROR_WEAK_KEY_DETAILS 39660
#define IDS_CERT_ERROR_WEAK_KEY_DESCRIPTION 39661
#define IDS_CERT_ERROR_NAME_CONSTRAINT_VIOLATION_DETAILS 39662
#define IDS_CERT_ERROR_NAME_CONSTRAINT_VIOLATION_DESCRIPTION 39663
#define IDS_CERT_ERROR_VALIDITY_TOO_LONG_DETAILS 39664
#define IDS_CERT_ERROR_VALIDITY_TOO_LONG_DESCRIPTION 39665
#define IDS_CERT_ERROR_NON_UNIQUE_NAME_DETAILS 39666
#define IDS_CERT_ERROR_NON_UNIQUE_NAME_DESCRIPTION 39667
#define IDS_CERT_ERROR_UNKNOWN_ERROR_DETAILS 39668
#define IDS_CERT_ERROR_UNKNOWN_ERROR_DESCRIPTION 39669
#define IDS_CERT_ERROR_SUMMARY_PINNING_FAILURE_DETAILS 39670
#define IDS_CERT_ERROR_SUMMARY_PINNING_FAILURE_DESCRIPTION 39671
#define IDS_CERT_ERROR_CERTIFICATE_TRANSPARENCY_REQUIRED_DETAILS 39672
#define IDS_CERT_ERROR_CERTIFICATE_TRANSPARENCY_REQUIRED_DESCRIPTION 39673
#define IDS_SSL_ERROR_OBSOLETE_VERSION_DETAILS 39674
#define IDS_SSL_ERROR_OBSOLETE_VERSION_DESCRIPTION 39675
#define IDS_CERT_ERROR_AUTHORITY_INVALID_DETAILS 39676
#define IDS_CERT_ERROR_AUTHORITY_INVALID_DETAILS_V2 39677
#define IDS_ALWAYS_ALLOW_ADS 39678
#define IDS_BLOCKED_ADS_PROMPT_EXPLANATION 39679
#define IDS_BLOCKED_ADS_PROMPT_TITLE 39680
#define IDS_BLOCK_INTERSTITIAL_TITLE 39687
#define IDS_CHILD_BLOCK_INTERSTITIAL_HEADER 39688
#define IDS_BLOCK_INTERSTITIAL_HEADER_ACCESS_REQUESTS_DISABLED 39689
#define IDS_SUPERVISED_USER_INTERSTITIAL_MESSAGE_BLOCK_ALL 39690
#define IDS_SUPERVISED_USER_INTERSTITIAL_MESSAGE_SAFE_SITES 39691
#define IDS_SUPERVISED_USER_INTERSTITIAL_MESSAGE_MANUAL 39692
#define IDS_CHILD_BLOCK_INTERSTITIAL_MESSAGE_NOT_SIGNED_IN 39693
#define IDS_REQUEST_SENT_OK 39694
#define IDS_BLOCK_INTERSTITIAL_ASK_IN_PERSON_BUTTON 39695
#define IDS_BLOCK_INTERSTITIAL_ASK_IN_PERSON_INSTEAD_BUTTON 39696
#define IDS_BLOCK_INTERSTITIAL_ASK_IN_A_MESSAGE_BUTTON 39697
#define IDS_CHILD_BLOCK_INTERSTITIAL_WAITING_APPROVAL_MESSAGE 39698
#define IDS_CHILD_BLOCK_INTERSTITIAL_WAITING_APPROVAL_DESCRIPTION_SINGLE_PARENT 39699
#define IDS_CHILD_BLOCK_INTERSTITIAL_WAITING_APPROVAL_DESCRIPTION_MULTI_PARENT 39700
#define IDS_CHILD_BLOCK_INTERSTITIAL_REQUEST_FAILED_MESSAGE_SINGLE_PARENT 39701
#define IDS_CHILD_BLOCK_INTERSTITIAL_REQUEST_FAILED_MESSAGE_MULTI_PARENT 39702
#define IDS_SUPERVISED_USER_VERIFY_PAGE_TAB_TITLE 39703
#define IDS_SUPERVISED_USER_VERIFY_PAGE_PRIMARY_HEADING 39704
#define IDS_SUPERVISED_USER_VERIFY_PAGE_PRIMARY_PARAGRAPH 39705
#define IDS_SUPERVISED_USER_VERIFY_PAGE_PRIMARY_BUTTON 39706
#define IDS_SUPERVISED_USER_VERIFY_PAGE_SUBFRAME_YOUTUBE_HEADING 39707
#define IDS_SUPERVISED_USER_VERIFY_PAGE_SUBFRAME_BLOCKED_SITE_HEADING 39708
#define IDS_PARENT_WEBSITE_APPROVAL_BLOCK_ALL_URL 39709
#define IDS_PARENT_WEBSITE_APPROVAL_SAFE_SITES_URL 39710
#define IDS_PARENT_WEBSITE_APPROVAL_MANUAL_URL 39711
#define IDS_PARENT_WEBSITE_APPROVAL_DIALOG_A11Y_NAME 39712
#define IDS_PARENTAL_LOCAL_APPROVAL_DIALOG_GENERIC_ERROR_TITLE 39713
#define IDS_PARENTAL_LOCAL_APPROVAL_DIALOG_GENERIC_ERROR_DESCRIPTION 39714
#define IDS_PARENTAL_LOCAL_APPROVAL_DIALOG_GENERIC_ERROR_BACK_BUTTON 39715
#define IDS_PARENTAL_LOCAL_APPROVAL_SNACKBAR_GENERIC_ERROR_TITLE 39716
#define IDS_PARENTAL_LOCAL_APPROVAL_SNACKBAR_GENERIC_ERROR_BACK_BUTTON 39717
#define IDS_SYNC_BASIC_ENCRYPTION_DATA 39721
#define IDS_SYNC_DATATYPE_AUTOFILL 39722
#define IDS_SYNC_DATATYPE_BOOKMARKS 39723
#define IDS_SYNC_DATATYPE_PASSWORDS_AND_PASSKEYS 39724
#define IDS_SYNC_DATATYPE_PREFERENCES 39725
#define IDS_SYNC_DATATYPE_TABS 39726
#define IDS_SYNC_DATATYPE_HISTORY_AND_TABS 39727
#define IDS_SYNC_DATATYPE_READING_LIST 39728
#define IDS_SYNC_EMPTY_PASSPHRASE_ERROR 39729
#define IDS_SYNC_FULL_ENCRYPTION_DATA 39730
#define IDS_SYNC_LOGIN_SETTING_UP 39731
#define IDS_SYNC_PASSPHRASE_LABEL 39732
#define IDS_SYNC_PASSPHRASE_MISMATCH_ERROR 39733
#define IDS_SYNC_ENTER_PASSPHRASE_BODY_WITH_DATE 39734
#define IDS_SYNC_ENTER_PASSPHRASE_BODY 39735
#define IDS_BULK_UPLOAD_SUBTITLE 39736
#define IDS_BULK_UPLOAD_BOOKMARK_FOLDER_SUBTITLE 39737
#define IDS_TAB_GROUP_COLOR_GREY 39738
#define IDS_TAB_GROUP_COLOR_BLUE 39739
#define IDS_TAB_GROUP_COLOR_RED 39740
#define IDS_TAB_GROUP_COLOR_YELLOW 39741
#define IDS_TAB_GROUP_COLOR_MAGENTA 39742
#define IDS_TAB_GROUP_COLOR_GREEN 39743
#define IDS_TAB_GROUP_COLOR_PINK 39744
#define IDS_TAB_GROUP_COLOR_LIME 39745
#define IDS_TAB_GROUP_COLOR_PURPLE 39746
#define IDS_TAB_GROUP_COLOR_CYAN 39747
#define IDS_TAB_GROUP_COLOR_ORANGE 39748
#define IDS_TAB_RESUME_DECORATORS_VISITED_X_AGO 39749
#define IDS_TAB_RESUME_DECORATORS_VISITED_RECENTLY 39750
#define IDS_TAB_RESUME_DECORATORS_FREQUENTLY_VISITED 39751
#define IDS_TAB_RESUME_DECORATORS_CONTINUE_READING 39752
#define IDS_TAB_RESUME_DECORATORS_MOST_RECENT 39753
#define IDS_TAB_RESUME_DECORATORS_TIME_OF_DAY 39754
#define IDS_TAB_RESUME_DECORATORS_VISITED_X_AGO_IOS 39755
#define IDS_TAB_RESUME_DECORATORS_VISITED_RECENTLY_IOS 39756
#define IDS_TAB_RESUME_DECORATORS_FREQUENTLY_VISITED_IOS 39757
#define IDS_TAB_RESUME_DECORATORS_MOST_RECENT_IOS 39758
#define IDS_TAB_RESUME_N_MINUTES_AGO_NARROW 39759
#define IDS_TAB_RESUME_N_HOURS_AGO_NARROW 39760
#define IDS_TRANSLATE_NOTIFICATION_UNDO 39776
#define IDS_TRANSLATE_HEADER 39777
#define IDS_TRANSLATE_BUBBLE_CHOOSE_LANGUAGE_TO_TRANSLATE_TO 39778
#define IDS_TRANSLATE_BUBBLE_SEARCH_LANGUAGES 39779
#define IDS_TRANSLATE_BUBBLE_NO_RESULTS 39780
#define IDS_TRANSLATE_DETECTED_LANGUAGE 39781
#define IDS_BOOKMARK_BAR_UNDO 39782
#define IDS_BOOKMARK_BAR_REDO 39783
#define IDS_BOOKMARK_BAR_UNDO_ADD 39784
#define IDS_BOOKMARK_BAR_REDO_ADD 39785
#define IDS_BOOKMARK_BAR_UNDO_DELETE 39786
#define IDS_BOOKMARK_BAR_REDO_DELETE 39787
#define IDS_BOOKMARK_BAR_UNDO_EDIT 39788
#define IDS_BOOKMARK_BAR_REDO_EDIT 39789
#define IDS_BOOKMARK_BAR_UNDO_MOVE 39790
#define IDS_BOOKMARK_BAR_REDO_MOVE 39791
#define IDS_BOOKMARK_BAR_UNDO_REORDER 39792
#define IDS_BOOKMARK_BAR_REDO_REORDER 39793
#define IDS_IMPORTED_FROM_SAFARI_FOLDER 39794
#define IDS_IMPORTED_FOLDER 39795
#define IDS_CLOSE_TUTORIAL 39796
#define IDS_CLOSE_PROMO 39797
#define IDS_HELP_BUBBLE 39798
#define IDS_PROMO_DISMISS_BUTTON 39799
#define IDS_PROMO_SNOOZE_BUTTON 39800
#define IDS_PROMO_SHOW_TUTORIAL_BUTTON 39801
#define IDS_TUTORIAL_RESTART_TUTORIAL 39802
#define IDS_PROMO_LEARN_MORE_BUTTON 39803
#define IDS_TUTORIAL_CLOSE_TUTORIAL 39804
#define IDS_TUTORIAL_NEXT_BUTTON 39805
#define IDS_PROMO_MANAGE_BUTTON 39806
#define IDS_PROMO_UNDO_BUTTON 39807
#define IDS_VERSION_UI_TITLE 39808
#define IDS_VERSION_UI_OFFICIAL 39809
#define IDS_VERSION_UI_UNOFFICIAL 39810
#define IDS_VERSION_UI_32BIT 39811
#define IDS_VERSION_UI_64BIT 39812
#define IDS_VERSION_UI_REVISION 39817
#define IDS_VERSION_UI_OS 39818
#define IDS_VERSION_UI_USER_AGENT 39820
#define IDS_VERSION_UI_COMMAND_LINE 39821
#define IDS_VERSION_UI_COPY_LABEL 39825
#define IDS_VERSION_UI_COPY_NOTICE 39826
#define IDS_VERSION_UI_EXECUTABLE_PATH 39827
#define IDS_VERSION_UI_PROFILE_PATH 39828
#define IDS_VERSION_UI_MODULE_PATH 39829
#define IDS_VERSION_UI_USER_DATA_PATH 39830
#define IDS_VERSION_UI_PATH_NOTFOUND 39831
#define IDS_VERSION_UI_VARIATIONS 39832
#define IDS_VERSION_UI_VARIATIONS_SOURCE_NAME 39833
#define IDS_VERSION_UI_VARIATIONS_SEED_NAME 39834
#define IDS_VERSION_UI_VARIATIONS_CMD 39835
#define IDS_VERSION_UI_COPY_VARIATIONS_LABEL 39836
#define IDS_VERSION_UI_COPY_VARIATIONS_NOTICE 39837
#define IDS_WALLET_WALLETABLE_PASS_CONSENT_DIALOG_TITLE 39842
#define IDS_WALLET_WALLETABLE_PASS_CONSENT_DIALOG_NO_THANKS_BUTTON 39843
#define IDS_WALLET_WALLETABLE_PASS_CONSENT_DIALOG_TURN_ON_BUTTON 39844
#define IDS_WALLET_WALLETABLE_PASS_CONSENT_DIALOG_LEARN_MORE 39845
#define IDS_WALLET_WALLETABLE_PASS_CONSENT_DIALOG_SUBTITLE_DESCRIPTION 39846
#define IDS_WALLET_WALLETABLE_PASS_CONSENT_DIALOG_SUBTITLE_ACTION 39847
#define IDS_WALLET_WALLETABLE_PASS_SAVE_LOYALTY_CARD_DIALOG_TITLE 39848
#define IDS_WALLET_WALLETABLE_PASS_SAVE_EVENT_TICKET_DIALOG_TITLE 39849
#define IDS_WALLET_WALLETABLE_PASS_SAVE_TRANSPORT_TICKET_DIALOG_TITLE 39850
#define IDS_WALLET_WALLETABLE_PASS_SAVE_BOARDING_PASS_DIALOG_TITLE 39851
#define IDS_WALLET_WALLETABLE_PASS_GOOGLE_WALLET_TITLE 39852
#define IDS_WALLET_WALLETABLE_PASS_SAVE_DIALOG_SUBTITLE 39853
#define IDS_WALLET_WALLETABLE_PASS_SAVE_DIALOG_NO_THANKS_BUTTON 39854
#define IDS_WALLET_WALLETABLE_PASS_SAVE_DIALOG_SAVE_BUTTON 39855
#define IDS_WALLET_WALLETABLE_PASS_LOYALTY_CARD_ISSUER_ATTRIBUTE_NAME 39856
#define IDS_WALLET_WALLETABLE_PASS_LOYALTY_CARD_MEMBER_ID_ATTRIBUTE_NAME 39857
#define IDS_WALLET_WALLETABLE_PASS_EVENT_PASS_EVENT_NAME_ATTRIBUTE_NAME 39858
#define IDS_WALLET_WALLETABLE_PASS_EVENT_PASS_ISSUER_ATTRIBUTE_NAME 39859
#define IDS_WALLET_WALLETABLE_PASS_EVENT_PASS_VENUE_ATTRIBUTE_NAME 39860
#define IDS_WALLET_WALLETABLE_PASS_EVENT_PASS_DATE_ATTRIBUTE_NAME 39861
#define IDS_WALLET_WALLETABLE_PASS_TRANSPORT_TICKET_OPERATOR_ATTRIBUTE_NAME 39862
#define IDS_WALLET_WALLETABLE_PASS_TRANSPORT_TICKET_TRIP_ATTRIBUTE_NAME 39863
#define IDS_WALLET_WALLETABLE_PASS_TRANSPORT_TICKET_TRIP_VALUE 39864
#define IDS_WALLET_WALLETABLE_PASS_TRANSPORT_TICKET_DATE_ATTRIBUTE_NAME 39865
#define IDS_WALLET_WALLETABLE_PASS_BOARDING_PASS_AIRLINE_ATTRIBUTE_NAME 39866
#define IDS_WALLET_WALLETABLE_PASS_BOARDING_PASS_FLIGHT_CODE_ATTRIBUTE_NAME 39867
#define IDS_WALLET_WALLETABLE_PASS_BOARDING_PASS_TRIP_ATTRIBUTE_NAME 39868
#define IDS_WALLET_WALLETABLE_PASS_BOARDING_PASS_TRIP_VALUE 39869
#define IDS_WALLET_WALLETABLE_PASS_BOARDING_PASS_DATE_ATTRIBUTE_NAME 39870
#define IDS_COLLABORATION_SIGNIN_TITLE 39871
#define IDS_COLLABORATION_SIGNIN_DESCRIPTION 39872
#define IDS_COLLABORATION_SIGNIN_BOTTOM_SHEET_TITLE 39873
#define IDS_COLLABORATION_SIGNIN_BOTTOM_SHEET_DESCRIPTION 39874
#define IDS_COLLABORATION_SIGNIN_SYNC_DISMISS 39875
#define IDS_COLLABORATION_SYNC_TITLE 39876
#define IDS_COLLABORATION_SYNC_DESCRIPTION 39877
#define IDS_COLLABORATION_ONE_GROUP_REMOVED_NOTIFICATION 39878
#define IDS_COLLABORATION_ONE_UNNAMED_GROUP_REMOVED_NOTIFICATION 39879
#define IDS_COLLABORATION_TWO_GROUPS_REMOVED_NOTIFICATION 39880
#define IDS_COLLABORATION_SEVERAL_GROUPS_REMOVED_NOTIFICATION 39881
#define IDS_COLLABORATION_RECENT_ACTIVITY_MENU_OPTION 39882
#define IDS_COLLABORATION_SOMETHING_WENT_WRONG_HEADER 39883
#define IDS_COLLABORATION_SOMETHING_WENT_WRONG_BODY 39884
#define IDS_COLLABORATION_LINK_FAILED_HEADER 39885
#define IDS_COLLABORATION_LINK_FAILED_BODY 39886
#define IDS_COLLABORATION_CHROME_OUT_OF_DATE_ERROR_DIALOG_HEADER 39887
#define IDS_COLLABORATION_SHARE_BUTTON_CHROME_OUT_OF_DATE_ERROR_DIALOG_BODY 39888
#define IDS_COLLABORATION_JOIN_BUTTON_CHROME_OUT_OF_DATE_ERROR_DIALOG_BODY 39889
#define IDS_COLLABORATION_CHROME_OUT_OF_DATE_ERROR_DIALOG_UPDATE_BUTTON 39890
#define IDS_COLLABORATION_CHROME_OUT_OF_DATE_ERROR_DIALOG_NOT_NOW_BUTTON 39891
#define IDS_COLLABORATION_CHROME_OUT_OF_DATE_ERROR_DIALOG_CONTINUE_BODY 39892
#define IDS_COLLABORATION_SIGNED_OUT_HEADER 39893
#define IDS_COLLABORATION_SIGNED_OUT_BODY 39894
#define IDS_COLLABORATION_SIGNED_OUT_POSITIVE_BUTTON 39895
#define IDS_COLLABORATION_GROUP_IS_FULL_DESCRIPTION 39896
#define IDS_COLLABORATION_GROUP_IS_FULL_ERROR_DIALOG_HEADER 39897
#define IDS_COLLABORATION_GROUP_IS_FULL_ERROR_DIALOG_BODY 39898
#define IDS_COLLABORATION_ENTREPRISE_SIGNIN_DISABLED_HEADER 39899
#define IDS_COLLABORATION_ENTREPRISE_SIGNIN_DISABLED_BODY 39900
#define IDS_COLLABORATION_ENTREPRISE_SIGNIN_DISABLED_SYNC_HISTORY_BODY 39901
#define IDS_COLLABORATION_ENTREPRISE_SYNC_DISABLED_HEADER 39902
#define IDS_COLLABORATION_ENTREPRISE_SYNC_DISABLED_BODY 39903
#define IDS_COLLABORATION_ENTREPRISE_TABS_SYNC_DISABLED_BODY 39904
#define IDS_COLLABORATION_ENTREPRISE_GROUP_CLOSED_HEADER 39905
#define IDS_COLLABORATION_ENTREPRISE_GROUP_CLOSED_BODY 39906
#define IDS_COLLABORATION_ENTREPRISE_SHARING_OFF_HEADER 39907
#define IDS_COLLABORATION_ENTREPRISE_SHARING_OFF_BODY 39908
#define IDS_COLLABORATION_SHARED_TAB_GROUPS_PANEL_OUT_OF_DATE_MESSAGE_CELL_TEXT 39912
#define IDS_COLLABORATION_SHARED_TAB_GROUPS_AVAILABLE_AGAIN_IPH_MESSAGE 39913
#define IDS_DATA_SHARING_RECENT_ACTIVITY_USER_SELF 39914
#define IDS_DATA_SHARING_RECENT_ACTIVITY_TAB_ADDED 39915
#define IDS_DATA_SHARING_RECENT_ACTIVITY_MEMBER_ADDED_THIS_TAB 39916
#define IDS_DATA_SHARING_RECENT_ACTIVITY_MEMBER_CHANGED_THIS_TAB 39917
#define IDS_DATA_SHARING_RECENT_ACTIVITY_TAB_REMOVED 39918
#define IDS_DATA_SHARING_RECENT_ACTIVITY_TAB_UPDATED 39919
#define IDS_DATA_SHARING_RECENT_ACTIVITY_TAB_GROUP_NAME_UPDATED 39920
#define IDS_DATA_SHARING_RECENT_ACTIVITY_TAB_GROUP_COLOR_UPDATED 39921
#define DATA_SHARING_GROUP_LABEL_NEW_ACTIVITY 39922
#define IDS_DATA_SHARING_RECENT_ACTIVITY_USER_JOINED_GROUP 39923
#define IDS_DATA_SHARING_RECENT_ACTIVITY_USER_LEFT_GROUP 39924
#define IDS_DATA_SHARING_RECENT_ACTIVITY_TIME_DELTA 39925
#define IDS_DATA_SHARING_RECENT_ACTIVITY_SEPARATOR 39926
#define IDS_DATA_SHARING_RECENT_ACTIVITY_DESCRIPTION_FULL 39927
#define IDS_DATA_SHARING_RECENT_ACTIVITY_EMPTY 39928
#define IDS_DATA_SHARING_RECENT_ACTIVITY_JUST_NOW 39929
#define IDS_DATA_SHARING_RECENT_ACTIVITY_UNKNOWN_USER 39930
#define IDS_DATA_SHARING_SHARED_TAB_GROUPS_ACTIVITY 39931
#define IDS_DATA_SHARING_TOAST_TAB_REMOVED 39932
#define IDS_DATA_SHARING_TOAST_TAB_UPDATED 39933
#define IDS_DATA_SHARING_TOAST_TAB_REMOVED_ACTION 39934
#define IDS_DATA_SHARING_TOAST_NEW_MEMBER 39935
#define IDS_DATA_SHARING_TOAST_NEW_MEMBER_MULTIPLE_MEMBERS 39936
#define IDS_DATA_SHARING_TOAST_NEW_MEMBER_ACTION 39937
#define IDS_DATA_SHARING_TOAST_BLOCK_LEAVE 39938
#define IDS_DATA_SHARING_TOAST_BLOCK_LEAVE_MULTIPLE_GROUPS 39939
#define IDS_DATA_SHARING_TAB_GROUP_DEFAULT_TITLE_TABS_COUNT 39940
#define IDS_COLLABORATION_SHARED_TAB_GROUPS_COMMENTS_TITLE 39941
#define IDS_COLLABORATION_SHARED_TAB_GROUPS_COMMENTS_TITLE_WITH_NAME 39942
#define IDS_CONTEXTUAL_CUEING_MENU_DISMISS 39943
#define IDS_CONTEXTUAL_CUEING_MENU_EDIT_PROMPT 39944
#define IDS_CONTEXTUAL_CUEING_MENU_SETTINGS 39945
#define IDS_CONTEXTUAL_CUEING_SETTINGS_PAGE_TITLE 39946
#define IDS_CONTEXTUAL_CUEING_SETTINGS_PAGE_SUBLABEL 39947
#define IDS_CONTEXTUAL_CUEING_SETTINGS_TOGGLE_LABEL 39948
#define IDS_CONTEXTUAL_CUEING_SETTINGS_TOGGLE_SUBLABEL 39949
#define IDS_CONTEXTUAL_CUEING_SETTINGS_WHEN_ON_1 39950
#define IDS_CONTEXTUAL_CUEING_SETTINGS_WHEN_ON_2 39951
#define IDS_CONTEXTUAL_CUEING_SETTINGS_CONSIDER_1_PRIVATE 39952
#define IDS_CONTEXTUAL_CUEING_SETTINGS_CONSIDER_1_NON_PRIVATE 39953
#define IDS_CONTEXTUAL_CUEING_SETTINGS_CONSIDER_2 39954
#define IDS_CONTEXTUAL_CUEING_SETTINGS_CONSIDER_2_LINK 39955
#define IDS_CONTEXTUAL_CUEING_TAB_SHARING_HEADING 39956
#define IDS_CONTEXTUAL_CUEING_TAB_SHARING_CURRENT_TAB 39957
#define IDS_CONTEXTUAL_CUEING_TAB_SHARING_EXPAND_BUTTON_ANNOUNCEMENT 39958
#define IDS_CONTEXTUAL_TASKS_CONTEXTUAL_TASKS_TITLE 39959
#define IDS_CONTEXTUAL_TASKS_ENTRY_POINT_TOOLTIP 39960
#define IDS_CONTEXTUAL_TASKS_AI_MODE_TITLE 39961
#define IDS_CONTEXTUAL_TASKS_MENU_OPEN_IN_NEW_TAB 39962
#define IDS_CONTEXTUAL_TASKS_MENU_MY_ACTIVITY 39963
#define IDS_CONTEXTUAL_TASKS_MENU_HELP 39964
#define IDS_CONTEXTUAL_TASKS_SOURCES_MENU_TITLE 39965
#define IDS_CONTEXTUAL_TASKS_FIRST_RUN_EXPERIENCE_TITLE 39966
#define IDS_CONTEXTUAL_TASKS_FIRST_RUN_EXPERIENCE_DESCRIPTION 39967
#define IDS_CONTEXTUAL_TASKS_FIRST_RUN_EXPERIENCE_LEARN_MORE 39968
#define IDS_CONTEXTUAL_TASKS_FIRST_RUN_EXPERIENCE_ACCEPT_BUTTON 39969
#define IDS_CONTEXTUAL_TASKS_CONTINUE_THREAD_MESSAGE 39970
#define IDS_CONTEXTUAL_TASKS_REOPEN_TABS_BUTTON_TEXT 39971
#define IDS_CONTEXTUAL_TASKS_CONTEXT_MENU_MOST_RECENT_TABS 39972
#define IDS_CONTEXTUAL_TASKS_CONTEXT_MENU_GEMINI_MODELS 39973
#define IDS_CONTEXTUAL_TASKS_SIDE_PANEL_NEW_THREAD_TOOL_TIP 39974
#define IDS_CONTEXTUAL_TASKS_SIDE_PANEL_HISTORY_TOOL_TIP 39975
#define IDS_CONTEXTUAL_TASKS_SIDE_PANEL_CONTEXT_TOOL_TIP 39976
#define IDS_CONTEXTUAL_TASKS_SIDE_PANEL_MORE_OPTIONS_TOOL_TIP 39977
#define IDS_CONTEXTUAL_TASKS_SIDE_PANEL_CLOSE_TOOL_TIP 39978
#define IDS_CONTEXTUAL_TASKS_TOOLBAR_CLOSE_TAB_TOOL_TIP 39979
#define IDS_CONTEXTUAL_TASKS_OAUTH_ERROR_DIALOG_TITLE 39980
#define IDS_CONTEXTUAL_TASKS_OAUTH_ERROR_DIALOG_BODY 39981
#define IDS_CONTEXTUAL_TASKS_OAUTH_ERROR_DIALOG_RELOAD_BUTTON 39982
#define IDS_AI_MODE_FRIENDLY_ZERO_STATE_TITLE 39983
#define IDS_AI_MODE_FRIENDLY_ZERO_STATE_TITLE_WITHOUT_NAME 39984
#define IDS_CONTEXTUAL_TASKS_COMPOSEBOX_PLACEHOLDER_TEXT 39985
#define IDS_CONTEXTUAL_TASKS_THREAD_CLOSED 39986
#define IDS_CANCEL 40054
#define IDS_CONTINUE 40055
#define IDS_CLOSE 40056
#define IDS_CLEAR 40057
#define IDS_DONE 40058
#define IDS_LEARN_MORE 40059
#define IDS_LEARN_MORE_MAYBE_TITLE_CASE 40060
#define IDS_OK 40061
#define IDS_RELOAD 40062
#define IDS_ADD 40063
#define IDS_MOVE 40064
#define IDS_REMOVE 40065
#define IDS_OPEN 40066
#define IDS_RESTORE 40067
#define IDS_SAVE 40068
#define IDS_MENU 40069
#define IDS_INSTALL 40070
#define IDS_UPDATE 40071
#define IDS_NO_THANKS 40074
#define IDS_NOT_NOW 40075
#define IDS_TURN_OFF 40076
#define IDS_PLUGIN_NOT_SUPPORTED 40077
#define IDS_PRINT 40078
#define IDS_RECENT_TABS 40079
#define IDS_RECENTLY_CLOSED 40080
#define IDS_YOUR_DEVICES 40081
#define IDS_CHOOSE 40082
#define IDS_ACCNAME_BACK 40083
#define IDS_ACCNAME_FORWARD 40084
#define IDS_ACCNAME_OK 40085
#define IDS_ACCNAME_CANCEL 40086
#define IDS_ACCNAME_DONE 40087
#define IDS_ACCNAME_SAVE 40088
#define IDS_ACCNAME_CLOSE 40089
#define IDS_ACCNAME_OPEN 40090
#define IDS_ACCNAME_OPEN_IN_NEW_TAB 40091
#define IDS_ACCNAME_PREVIOUS 40092
#define IDS_ACCNAME_NEXT 40093
#define IDS_ACCNAME_LOCATION 40094
#define IDS_ACCNAME_TAB_LIST 40095
#define IDS_UTILITY_PROCESS_JSON_PARSER_NAME 40096
#define IDS_SESSION_CRASHED_VIEW_RESTORE_BUTTON 40097
#define IDS_SESSION_CRASHED_VIEW_STARTUP_PAGES_BUTTON 40098
#define IDS_OPTIONS_ADVANCED_SECTION_TITLE_PRIVACY 40099
#define IDS_OPTIONS_ADVANCED_SECTION_TITLE_SAFETY_CHECK 40100
#define IDS_CONTENT_BOOKMARK_PARSER_SERVICE_DISPLAY_NAME 40101
#define IDS_PATCH_SERVICE_DISPLAY_NAME 40102
#define IDS_PASSWORD_MANAGER_CSV_PASSWORD_PARSER_SERVICE_DISPLAY_NAME 40103
#define IDS_UNZIP_SERVICE_DISPLAY_NAME 40104
#define IDS_INTERNAL_DEBUG_PAGES_DISABLED_HEADING 40105
#define IDS_SEND_TAB_TO_SELF_DEVICE_LAST_UPDATE_DAYS 40106
#define IDS_SEND_TAB_TO_SELF_DEVICE_ACTIVE_NOW 40107
#define IDS_SEND_TAB_TO_SELF_DEVICE_ACTIVE_MINUTES 40108
#define IDS_SEND_TAB_TO_SELF_DEVICE_ACTIVE_HOURS 40109
#define IDS_IOS_DESKTOP_PRICE_TRACKING_PROMO_NOTIFICATION_TITLE 40110
#define IDS_IOS_DESKTOP_TAB_GROUPS_PROMO_NOTIFICATION_TITLE 40111
#define IDS_IOS_DESKTOP_PASSWORD_PROMO_NOTIFICATION_BODY 40112
#define IDS_IOS_DESKTOP_SAFE_BROWSING_PROMO_NOTIFICATION_BODY 40113
#define IDS_IOS_DESKTOP_LENS_PROMO_NOTIFICATION_BODY 40114
#define IDS_IOS_DESKTOP_PRICE_TRACKING_PROMO_NOTIFICATION_BODY 40115
#define IDS_IOS_DESKTOP_TAB_GROUPS_PROMO_NOTIFICATION_BODY 40116

// ---------------------------------------------------------------------------
// From extensions_strings.h:

#define IDS_EXTENSION_BAD_FILE_ENCODING 45630
#define IDS_EXTENSION_CANT_GET_ABSOLUTE_PATH 45631
#define IDS_EXTENSION_CONTAINS_PRIVATE_KEY 45632
#define IDS_EXTENSION_CRX_EXISTS 45633
#define IDS_EXTENSION_CONTENT_SCRIPT_FILE_TOO_LARGE 45634
#define IDS_EXTENSION_CONTENT_SCRIPT_FILE_BAD_JS_MIME_TYPE 45635
#define IDS_EXTENSION_CONTENT_SCRIPT_FILE_BAD_CSS_MIME_TYPE 45636
#define IDS_EXTENSION_DIRECTORY_NO_EXISTS 45637
#define IDS_EXTENSION_ERROR_WHILE_SIGNING 45638
#define IDS_EXTENSION_FAILED_DURING_PACKAGING 45639
#define IDS_EXTENSION_LOAD_ABOUT_PAGE_FAILED 45640
#define IDS_EXTENSION_LOAD_BACKGROUND_SCRIPT_FAILED 45641
#define IDS_EXTENSION_LOAD_BACKGROUND_PAGE_FAILED 45642
#define IDS_EXTENSION_LOAD_CSS_FAILED 45643
#define IDS_EXTENSION_LOAD_JAVASCRIPT_FAILED 45644
#define IDS_EXTENSION_LOAD_OPTIONS_PAGE_FAILED 45645
#define IDS_EXTENSION_LOCALES_NO_DEFAULT_LOCALE_SPECIFIED 45646
#define IDS_EXTENSION_MANIFEST_UNREADABLE 45647
#define IDS_EXTENSION_MANIFEST_INVALID 45648
#define IDS_EXTENSION_PACKAGE_IMAGE_ERROR 45649
#define IDS_EXTENSION_PACKAGE_UNZIP_ERROR 45650
#define IDS_EXTENSION_PRIVATE_KEY_EXISTS 45651
#define IDS_EXTENSION_PRIVATE_KEY_FAILED_TO_READ 45652
#define IDS_EXTENSION_PRIVATE_KEY_FAILED_TO_EXPORT 45653
#define IDS_EXTENSION_PRIVATE_KEY_FAILED_TO_OUTPUT 45654
#define IDS_EXTENSION_PRIVATE_KEY_INVALID 45655
#define IDS_EXTENSION_PRIVATE_KEY_NO_EXISTS 45656
#define IDS_EXTENSION_PRIVATE_KEY_INVALID_PATH 45657
#define IDS_EXTENSION_PRIVATE_KEY_INVALID_FORMAT 45658
#define IDS_EXTENSION_PUBLIC_KEY_FAILED_TO_EXPORT 45659
#define IDS_EXTENSION_SHARING_VIOLATION 45660
#define IDS_EXTENSION_CANT_INSTALL_POLICY_BLOCKED 45661
#define IDS_EXTENSION_CANT_MODIFY_POLICY_REQUIRED 45662
#define IDS_EXTENSION_CANT_UNINSTALL_POLICY_REQUIRED 45663
#define IDS_EXTENSION_MANIFEST_VERSION_NOT_SUPPORTED 45664
#define IDS_EXTENSION_CANT_INSTALL_BLOCKED_BY_RISK_SCORE 45665
#define IDS_EXTENSION_CANT_INSTALL_BLOCKED_BY_CATEGORY 45666
#define IDS_EXTENSION_CANT_INSTALL_BLOCKED_BY_POLICY_FALLBACK 45667
#define IDS_DEVICE_NAME_WITH_PRODUCT_SERIAL 45668
#define IDS_DEVICE_NAME_WITH_PRODUCT_UNKNOWN_VENDOR 45669
#define IDS_DEVICE_NAME_WITH_PRODUCT_UNKNOWN_VENDOR_SERIAL 45670
#define IDS_DEVICE_NAME_WITH_PRODUCT_VENDOR 45671
#define IDS_DEVICE_NAME_WITH_PRODUCT_VENDOR_SERIAL 45672
#define IDS_DEVICE_NAME_WITH_UNKNOWN_PRODUCT_UNKNOWN_VENDOR 45673
#define IDS_DEVICE_NAME_WITH_UNKNOWN_PRODUCT_UNKNOWN_VENDOR_SERIAL 45674
#define IDS_DEVICE_NAME_WITH_UNKNOWN_PRODUCT_VENDOR 45675
#define IDS_DEVICE_NAME_WITH_UNKNOWN_PRODUCT_VENDOR_SERIAL 45676
#define IDS_DEVICE_PERMISSIONS_PROMPT_SINGLE_SELECTION 45677
#define IDS_DEVICE_PERMISSIONS_PROMPT_MULTIPLE_SELECTION 45678
#define IDS_CAMERA_FACING_USER 45679
#define IDS_CAMERA_FACING_ENVIRONMENT 45680
#define IDS_EXTENSION_USB_DEVICE_PRODUCT_NAME_AND_VENDOR 45681
#define IDS_EXTENSION_TASK_MANAGER_APPVIEW_TAG_PREFIX 45682
#define IDS_EXTENSION_TASK_MANAGER_EXTENSIONOPTIONS_TAG_PREFIX 45683
#define IDS_EXTENSION_TASK_MANAGER_MIMEHANDLERVIEW_TAG_PREFIX 45684
#define IDS_EXTENSION_TASK_MANAGER_WEBVIEW_TAG_PREFIX 45685
#define IDS_EXTENSION_WARNINGS_NETWORK_DELAY 45686
#define IDS_EXTENSION_WARNINGS_DOWNLOAD_FILENAME_CONFLICT 45687
#define IDS_EXTENSION_WARNING_RELOAD_TOO_FREQUENT 45688
#define IDS_EXTENSION_WARNING_RULESET_FAILED_TO_LOAD 45689
#define IDS_EXTENSION_WARNING_ENABLED_RULE_COUNT_EXCEEDED 45690
#define IDS_EXTENSION_IS_BLOCKLISTED 45691
#define IDS_APP_CANT_DOWNGRADE_VERSION 45692
#define IDS_EXTENSION_CANT_DOWNGRADE_VERSION 45693
#define IDS_EXTENSION_MOVE_DIRECTORY_TO_PROFILE_FAILED 45694
#define IDS_EXTENSION_INSTALL_DEPENDENCY_OLD_VERSION 45695
#define IDS_EXTENSION_INSTALL_DEPENDENCY_NOT_SHARED_MODULE 45696
#define IDS_EXTENSION_INSTALL_DEPENDENCY_NOT_ALLOWLISTED 45697
#define IDS_EXTENSION_INSTALL_DISALLOWED_ON_SITE 45698
#define IDS_EXTENSION_INSTALL_GALLERY_ONLY 45699
#define IDS_EXTENSION_INSTALL_INCORRECT_APP_CONTENT_TYPE 45700
#define IDS_EXTENSION_INSTALL_INCORRECT_INSTALL_HOST 45701
#define IDS_EXTENSION_INSTALL_KIOSK_MODE_ONLY 45702
#define IDS_EXTENSION_INSTALL_NOT_ENABLED 45703
#define IDS_EXTENSION_OVERLAPPING_WEB_EXTENT 45704
#define IDS_EXTENSION_INSTALL_PROCESS_CRASHED 45705
#define IDS_EXTENSION_INSTALL_UNEXPECTED_ID 45706
#define IDS_EXTENSION_INSTALL_UNEXPECTED_VERSION 45707
#define IDS_EXTENSION_PACKAGE_ERROR_CODE 45708
#define IDS_EXTENSION_PACKAGE_ERROR_MESSAGE 45709
#define IDS_EXTENSION_PACKAGE_INSTALL_ERROR 45710
#define IDS_EXTENSION_WEBGL_NOT_SUPPORTED 45711
#define IDS_EXTENSION_WEB_STORE_TITLE 45712
#define IDS_EXTENSIONS_LOAD_ERROR_ALERT_HEADING 45713
#define IDS_EXTENSIONS_LOAD_ERROR_MESSAGE 45714
#define IDS_WEBSTORE_DOWNLOAD_ACCESS_DENIED 45715
#define IDS_EXTENSIONS_SUPERVISED_USER_PARENTAL_PERMISSION_FAILURE 45716
#define IDS_EXTENSIONS_SUPERVISED_USER_BLOCKED_BY_PARENT 45717
#define IDS_EXTENSION_PROMPT_MESSAGE_FROM_ADMIN 45718

// ---------------------------------------------------------------------------
// From generated_resources.h:

#define IDS_ACCESS_CODE_CAST_ACCESS_CODE_MESSAGE 1800
#define IDS_ACCESS_CODE_CAST_BACK 1801
#define IDS_ACCESS_CODE_CAST_CAST 1802
#define IDS_ACCESS_CODE_CAST_CONNECT 1803
#define IDS_ACCESS_CODE_CAST_DIALOG_TITLE 1804
#define IDS_ACCESS_CODE_CAST_ENTER_CHARACTER 1805
#define IDS_ACCESS_CODE_CAST_ERROR_ACCESS_CODE 1806
#define IDS_ACCESS_CODE_CAST_ERROR_DIFFERENT_NETWORK 1807
#define IDS_ACCESS_CODE_CAST_ERROR_NETWORK 1808
#define IDS_ACCESS_CODE_CAST_ERROR_PERMISSION 1809
#define IDS_ACCESS_CODE_CAST_ERROR_TOO_MANY_REQUESTS 1810
#define IDS_ACCESS_CODE_CAST_ERROR_UNKNOWN 1811
#define IDS_ACCESS_CODE_CAST_ERROR_PROFILE_SYNC 1812
#define IDS_ACCESS_CODE_CAST_INPUT_ARIA_LABEL 1813
#define IDS_ACCESS_CODE_CAST_MANAGED_FOOTNOTE_DAYS 1814
#define IDS_ACCESS_CODE_CAST_MANAGED_FOOTNOTE_HOURS 1815
#define IDS_ACCESS_CODE_CAST_MANAGED_FOOTNOTE_MONTHS 1816
#define IDS_ACCESS_CODE_CAST_MANAGED_FOOTNOTE_YEARS 1817
#define IDS_ACCESS_CODE_CAST_SUBMIT 1818
#define IDS_ACCESS_CODE_CAST_USE_CAMERA 1819
#define IDS_HANDOFF_TAKE_OVER_TASK_LABEL 1820
#define IDS_HANDOFF_GIVE_TASK_BACK_LABEL 1821
#define IDS_HANDOFF_TAKE_OVER_TASK_A11Y_LABEL 1822
#define IDS_HANDOFF_GIVE_TASK_BACK_A11Y_LABEL 1823
#define IDS_TASK_IN_PROGRESS_TOAST_BODY 1824
#define IDS_ACTOR_TASK_NUDGE_CHECK_TASK_LABEL 1825
#define IDS_ACTOR_TASK_NUDGE_TASK_COMPLETE_LABEL 1826
#define IDS_ACTOR_TASK_LIST_BUBBLE_ROW_TAB_CLOSED_SUBTITLE 1827
#define IDS_ACTOR_TASK_LIST_BUBBLE_ROW_CHECK_TASK_SUBTITLE 1828
#define IDS_ACTOR_TASK_LIST_BUBBLE_ROW_ACTING_TASK_SUBTITLE 1829
#define IDS_ACTOR_TASK_LIST_BUBBLE_A11Y_LABEL 1830
#define IDS_ACTOR_TASK_INDICATOR_TOOLTIP 1831
#define IDS_ACTOR_TASK_LIST_BUBBLE_ROW_PAUSED_TASK_SUBTITLE 1832
#define IDS_ACTOR_TASK_LIST_BUBBLE_ROW_COMPLETED_TASK_SUBTITLE 1833
#define IDS_ACTOR_TASK_LIST_BUBBLE_ROW_FAILED_TASK_SUBTITLE 1834
#define IDS_TOOLTIP_TAB_ALERT_STATE_ACTOR_ACCESSING 1835
#define IDS_TAB_AX_LABEL_ACTOR_ACCESSING 1836
#define IDS_ACTOR_PIP_WORKING_STATUS 1837
#define IDS_ACTOR_PIP_PAUSED_STATUS 1838
#define IDS_ACTOR_PIP_NEEDS_ATTENTION_STATUS 1839
#define IDS_ACTOR_PIP_COMPLETE_STATUS 1840
#define IDS_ACTOR_OVERLAY_SNACKBAR_MESSAGE 1841
#define IDS_GLIC_BUTTON_STATUS_REVIEW 1842
#define IDS_GLIC_BUTTON_STATUS_REVIEW_A11Y_LABEL 1843
#define IDS_GLIC_BUTTON_STATUS_DONE 1844
#define IDS_GLIC_BUTTON_STATUS_DONE_A11Y_LABEL 1845
#define IDS_GLIC_BUTTON_STATUS_WORKING_A11Y_LABEL 1846
#define IDS_ACTOR_NOTIFICATION_TITLE_WORKING_ON_TASK 1847
#define IDS_ACTOR_NOTIFICATION_TITLE_CHECK_YOUR_TASK 1848
#define IDS_ACTOR_NOTIFICATION_TITLE_TASK_PAUSED 1849
#define IDS_ACTOR_NOTIFICATION_TITLE_TASK_STOPPED 1850
#define IDS_ACTOR_NOTIFICATION_TITLE_TASK_COMPLETE 1851
#define IDS_ACTOR_NOTIFICATION_BUTTON_GO_TO_CHROME 1852
#define IDS_ACTOR_NOTIFICATION_BODY_WORKING 1853
#define IDS_ACTOR_NOTIFICATION_BODY_USER_INPUT 1854
#define IDS_ACTOR_NOTIFICATION_BODY_PAUSED 1855
#define IDS_ACTOR_NOTIFICATION_BODY_COMPLETE 1856
#define IDS_ACTOR_NOTIFICATION_BODY_STOPPED 1857
#define IDS_ACTOR_NOTIFICATION_TITLE_WILL_STOP_TASK 1858
#define IDS_ACTOR_NOTIFICATION_BODY_WILL_STOP_TASK_LONG_RUNNING 1859
#define IDS_ACTOR_NOTIFICATION_BODY_WILL_STOP_TASK_NO_RESPONSE 1860
#define IDS_PEEK_STATE_ACTING 1861
#define IDS_PEEK_STATE_PAUSED 1862
#define IDS_PEEK_STATE_WAITING 1863
#define IDS_PEEK_STATE_VIEW_BUTTON_LABEL 1864
#define IDS_PEEK_STATE_CLOSE_BUTTON_DESCRIPTION 1865
#define IDS_PEEK_STATE_PLAY_BUTTON_A11Y_LABEL 1866
#define IDS_PEEK_STATE_PAUSE_BUTTON_A11Y_LABEL 1867
#define IDS_ACTOR_LEAVE_SITE_DIALOG_TITLE 1868
#define IDS_ACTOR_LEAVE_SITE_DIALOG_DESCRIPTION 1869
#define IDS_ACTOR_LEAVE_SITE_DIALOG_LEAVE_SITE 1870
#define IDS_APP_MANAGEMENT_CAMERA 1871
#define IDS_APP_MANAGEMENT_LOCATION 1872
#define IDS_APP_MANAGEMENT_MICROPHONE 1873
#define IDS_APP_MANAGEMENT_NOTIFICATIONS 1874
#define IDS_APP_MANAGEMENT_PERMISSIONS 1876
#define IDS_APP_MANAGEMENT_PERMISSIONS_WITH_ORIGIN 1877
#define IDS_APP_MANAGEMENT_MORE_SETTINGS 1878
#define IDS_APP_MANAGEMENT_WINDOW 1879
#define IDS_APP_MANAGEMENT_UNINSTALL_APP 1880
#define IDS_APP_MANAGEMENT_RUN_ON_OS_LOGIN 1881
#define IDS_APP_MANAGEMENT_POLICY_APP_POLICY_STRING 1882
#define IDS_APP_MANAGEMENT_INTENT_SETTINGS_TITLE 1883
#define IDS_APP_MANAGEMENT_INTENT_SHARING_APP_OPEN 1884
#define IDS_APP_MANAGEMENT_INTENT_SHARING_BROWSER_OPEN 1885
#define IDS_APP_MANAGEMENT_INTENT_SHARING_TAB_EXPLANATION 1886
#define IDS_APP_MANAGEMENT_INTENT_SETTINGS_DIALOG_TITLE 1887
#define IDS_APP_MANAGEMENT_INTENT_OVERLAP_CHANGE_BUTTON 1888
#define IDS_APP_MANAGEMENT_INTENT_OVERLAP_DIALOG_TITLE 1889
#define IDS_APP_MANAGEMENT_INTENT_OVERLAP_DIALOG_TEXT_1_APP 1890
#define IDS_APP_MANAGEMENT_INTENT_OVERLAP_DIALOG_TEXT_2_APPS 1891
#define IDS_APP_MANAGEMENT_INTENT_OVERLAP_DIALOG_TEXT_3_APPS 1892
#define IDS_APP_MANAGEMENT_INTENT_OVERLAP_DIALOG_TEXT_4_APPS 1893
#define IDS_APP_MANAGEMENT_INTENT_OVERLAP_DIALOG_TEXT_5_OR_MORE_APPS 1894
#define IDS_APP_MANAGEMENT_INTENT_OVERLAP_WARNING_TEXT_1_APP 1895
#define IDS_APP_MANAGEMENT_INTENT_OVERLAP_WARNING_TEXT_2_APPS 1896
#define IDS_APP_MANAGEMENT_INTENT_OVERLAP_WARNING_TEXT_3_APPS 1897
#define IDS_APP_MANAGEMENT_INTENT_OVERLAP_WARNING_TEXT_4_APPS 1898
#define IDS_APP_MANAGEMENT_INTENT_OVERLAP_WARNING_TEXT_5_OR_MORE_APPS 1899
#define IDS_APP_MANAGEMENT_APP_VERSION_TITLE 1900
#define IDS_APP_MANAGEMENT_APP_SIZE_TITLE 1901
#define IDS_APP_MANAGEMENT_APP_DATA_SIZE_TITLE 1902
#define IDS_APP_MANAGEMENT_FILE_HANDLING_HEADER 1903
#define IDS_APP_MANAGEMENT_FILE_HANDLING_OVERFLOW_DIALOG_TITLE 1904
#define IDS_APP_MANAGEMENT_FILE_HANDLING_SET_DEFAULTS_LINK 1905
#define IDS_APP_MANAGEMENT_FILE_HANDLING_TYPES 1906
#define IDS_APP_MANAGEMENT_APP_CONTENT_TITLE 1907
#define IDS_APP_MANAGEMENT_APP_CONTENT_SUBTITLE 1908
#define IDS_APP_MANAGEMENT_APP_CONTENT_DIALOG_SUBTITLE 1909
#define IDS_APP_MANAGEMENT_IS_SUB_APP_PERMISSION_EXPLANATION 1910
#define IDS_APP_MANAGEMENT_HAS_SUB_APPS_PERMISSION_EXPLANATION 1911
#define IDS_SUB_APPS_INSTALL_DIALOG_PRIVACY_DESCRIPTION 1912
#define IDS_BOOKMARK_GROUP_FROM_FIREFOX 1915
#define IDS_BOOKMARK_GROUP_FROM_SAFARI 1916
#define IDS_BOOKMARK_GROUP 1917
#define IDS_UNNAMED_BOOKMARK_BUTTON_ACCESSIBLE_NAME 1918
#define IDS_BOOKMARK_BAR_SHOW_APPS_SHORTCUT 1919
#define IDS_BOOKMARK_BAR_SHOW_TAB_GROUPS 1920
#define IDS_BOOKMARK_BAR_SHOW_MANAGED_BOOKMARKS_DEFAULT_NAME 1921
#define IDS_BOOKMARK_BAR_SHOW_MANAGED_BOOKMARKS 1922
#define IDS_BOOKMARK_BAR_APPS_SHORTCUT_NAME 1923
#define IDS_BOOKMARK_BAR_APPS_SHORTCUT_TOOLTIP 1924
#define IDS_BOOKMARK_BAR_OVERFLOW_BUTTON_TOOLTIP 1925
#define IDS_BOOKMARK_BAR_OPEN_ALL_COUNT 1926
#define IDS_BOOKMARK_BAR_OPEN_ALL_COUNT_NEW_WINDOW 1927
#define IDS_BOOKMARK_BAR_OPEN_ALL_COUNT_INCOGNITO 1928
#define IDS_BOOKMARK_BAR_OPEN_ALL_COUNT_NEW_TAB_GROUP 1929
#define IDS_BOOKMARK_BAR_OPEN_IN_NEW_TAB 1930
#define IDS_BOOKMARK_BAR_OPEN_IN_NEW_WINDOW 1931
#define IDS_BOOKMARK_BAR_OPEN_INCOGNITO 1932
#define IDS_BOOKMARK_BAR_OPEN_IN_SPLIT_VIEW 1933
#define IDS_BOOKMARK_BAR_EDIT 1934
#define IDS_BOOKMARK_BAR_RENAME_FOLDER 1935
#define IDS_BOOKMARK_BAR_REMOVE 1936
#define IDS_BOOKMARK_BAR_ADD_NEW_BOOKMARK 1937
#define IDS_BOOKMARK_BAR_NEW_FOLDER 1938
#define IDS_SHOW_BOOKMARK_BAR 1939
#define IDS_HIDE_BOOKMARK_BAR 1940
#define IDS_BOOKMARK_BAR_SUBMENU_LABEL 1941
#define IDS_BOOKMARK_BAR_SUBMENU_ALWAYS_HIDE 1942
#define IDS_BOOKMARK_BAR_SUBMENU_ALWAYS_SHOW 1943
#define IDS_BOOKMARK_BAR_SUBMENU_ONLY_ON_NTP 1944
#define IDS_BOOKMARK_BAR_SHOULD_OPEN_ALL 1945
#define IDS_BOOKMARK_BAR_ALREADY_CREATED_GROUP 1946
#define IDS_BOOKMARK_BAR_REPLACE_OR_CREATE_NEW_GROUP_TITLE 1947
#define IDS_BOOKMARK_BAR_REPLACE_OLD_GROUP_BUTTON 1948
#define IDS_BOOKMARK_BUBBLE_PAGE_BOOKMARKED 1949
#define IDS_BOOKMARK_BUBBLE_PAGE_BOOKMARK 1950
#define IDS_BOOKMARK_BUBBLE_NAME_LABEL 1951
#define IDS_BOOKMARK_AX_BUBBLE_NAME_LABEL 1952
#define IDS_BOOKMARK_BUBBLE_FOLDER_LABEL 1953
#define IDS_BOOKMARK_BUBBLE_CHOOSER_ANOTHER_FOLDER 1954
#define IDS_BOOKMARK_DICE_PROMO_SYNC_MESSAGE 1955
#define IDS_BOOKMARK_VERIFY_PROMO_SUBTITLE 1956
#define IDS_BOOKMARK_PROMO_TITLE 1957
#define IDS_BOOKMARK_EDITOR_NAME_LABEL 1958
#define IDS_BOOKMARK_AX_EDITOR_NAME_LABEL 1959
#define IDS_BOOKMARK_AX_EDITOR_URL_LABEL 1960
#define IDS_BOOKMARK_EDITOR_URL_LABEL 1961
#define IDS_BOOKMARK_EDITOR_CONFIRM_DELETE 1962
#define IDS_TAB_GROUP_TO_BOOKMARK_FOLDER_SUBTITLE 1963
#define IDS_BOOKMARK_EDITOR_NEW_FOLDER_BUTTON 1964
#define IDS_BOOKMARK_EDITOR_NEW_FOLDER_MENU_ITEM 1965
#define IDS_BOOKMARK_FOLDER_EDITOR_WINDOW_TITLE 1966
#define IDS_BOOKMARK_FOLDER_EDITOR_WINDOW_TITLE_NEW 1967
#define IDS_UNNAMED_BOOKMARK_FOLDER 1968
#define IDS_BOOKMARK_ALL_TABS_DIALOG_TITLE 1969
#define IDS_TAB_GROUP_TO_BOOKMARK_FOLDER_TITLE 1970
#define IDS_BOOKMARK_MANAGER_TITLE 1971
#define IDS_BOOKMARK_MANAGER_SEARCH_BUTTON 1972
#define IDS_BOOKMARK_MANAGER 1973
#define IDS_BOOKMARK_MANAGER_ORGANIZE_MENU 1974
#define IDS_BOOKMARK_MANAGER_INVALID_URL 1975
#define IDS_EXPORT_BOOKMARKS_DEFAULT_FILENAME 1976
#define IDS_BOOKMARK_MANAGER_ADD_BOOKMARK_TITLE 1977
#define IDS_BOOKMARK_MANAGER_ADD_FOLDER_TITLE 1978
#define IDS_BOOKMARK_MANAGER_CLEAR_SEARCH 1979
#define IDS_BOOKMARK_MANAGER_EMPTY_LIST 1980
#define IDS_BOOKMARK_MANAGER_EMPTY_UNMODIFIABLE_LIST 1981
#define IDS_BOOKMARK_MANAGER_FOLDER_LABEL 1982
#define IDS_BOOKMARK_MANAGER_FOLDER_RENAME_TITLE 1983
#define IDS_BOOKMARK_MANAGER_FOLDER_LIST_CHANGED 1984
#define IDS_BOOKMARK_MANAGER_LIST_AX_LABEL 1985
#define IDS_BOOKMARK_MANAGER_MENU_ADD_BOOKMARK 1986
#define IDS_BOOKMARK_MANAGER_MENU_ADD_FOLDER 1987
#define IDS_BOOKMARK_MANAGER_MENU_CUT 1988
#define IDS_BOOKMARK_MANAGER_MENU_COPY 1989
#define IDS_BOOKMARK_MANAGER_MENU_PASTE 1990
#define IDS_BOOKMARK_MANAGER_MENU_EXPORT 1991
#define IDS_BOOKMARK_MANAGER_MENU_HELP_CENTER 1992
#define IDS_BOOKMARK_MANAGER_MENU_IMPORT 1993
#define IDS_BOOKMARK_MANAGER_MENU_IMPORT_BEGAN 1994
#define IDS_BOOKMARK_MANAGER_MENU_IMPORT_ENDED 1995
#define IDS_BOOKMARK_MANAGER_MENU_OPEN_ALL 1996
#define IDS_BOOKMARK_MANAGER_MENU_OPEN_ALL_WITH_COUNT 1997
#define IDS_BOOKMARK_MANAGER_MENU_OPEN_ALL_NEW_WINDOW 1998
#define IDS_BOOKMARK_MANAGER_MENU_OPEN_ALL_NEW_WINDOW_WITH_COUNT 1999
#define IDS_BOOKMARK_MANAGER_MENU_OPEN_ALL_INCOGNITO 2000
#define IDS_BOOKMARK_MANAGER_MENU_OPEN_ALL_INCOGNITO_WITH_COUNT 2001
#define IDS_BOOKMARK_MANAGER_MENU_OPEN_ALL_NEW_TAB_GROUP 2002
#define IDS_BOOKMARK_MANAGER_MENU_OPEN_ALL_NEW_TAB_GROUP_WITH_COUNT 2003
#define IDS_BOOKMARK_MANAGER_MENU_OPEN_IN_NEW_TAB 2004
#define IDS_BOOKMARK_MANAGER_MENU_OPEN_IN_NEW_WINDOW 2005
#define IDS_BOOKMARK_MANAGER_MENU_OPEN_INCOGNITO 2006
#define IDS_BOOKMARK_MANAGER_MENU_OPEN_IN_NEW_TAB_GROUP 2007
#define IDS_BOOKMARK_MANAGER_MENU_OPEN_IN_SPLIT_VIEW 2008
#define IDS_BOOKMARK_MANAGER_MENU_RENAME 2009
#define IDS_BOOKMARK_MANAGER_MENU_SHOW_IN_FOLDER 2010
#define IDS_BOOKMARK_MANAGER_MENU_SORT 2011
#define IDS_BOOKMARK_MANAGER_MORE_ACTIONS 2012
#define IDS_BOOKMARK_MANAGER_MORE_ACTIONS_AX_LABEL 2013
#define IDS_BOOKMARK_MANAGER_MORE_ACTIONS_MULTI_AX_LABEL 2014
#define IDS_BOOKMARK_MANAGER_OPEN_DIALOG_TITLE 2015
#define IDS_BOOKMARK_MANAGER_OPEN_DIALOG_CONFIRM 2016
#define IDS_BOOKMARK_MANAGER_ITEMS_SELECTED 2017
#define IDS_BOOKMARK_MANAGER_ITEMS_UNSELECTED 2018
#define IDS_BOOKMARK_MANAGER_SIDEBAR_AX_LABEL 2019
#define IDS_BOOKMARK_MANAGER_TOAST_FOLDER_SORTED 2020
#define IDS_BOOKMARK_MANAGER_TOAST_ITEM_DELETED 2021
#define IDS_BOOKMARK_MANAGER_TOAST_ITEMS_DELETED 2022
#define IDS_BOOKMARK_MANAGER_TOAST_ITEM_COPIED 2023
#define IDS_BOOKMARK_MANAGER_TOAST_ITEMS_COPIED 2024
#define IDS_BOOKMARKS_MENU 2025
#define IDS_BOOKMARKS_AND_LISTS_MENU 2026
#define IDS_SHOW_BOOKMARK_SIDE_PANEL 2027
#define IDS_BOOKMARK_THIS_TAB 2028
#define IDS_BOOKMARK_ALL_TABS 2029
#define IDS_BOOKMARKS_LIST_TITLE 2030
#define IDS_TOOLTIP_STARRED 2031
#define IDS_BOOKMARK_SCREEN_READER_CREATED 2032
#define IDS_BOOKMARK_SCREEN_READER_FOLDER_CREATED 2033
#define IDS_BOOKMARK_SCREEN_READER_REORDERED 2034
#define IDS_BOOKMARK_SCREEN_READER_MOVED 2035
#define IDS_BOOKMARK_BAR_HIDDEN_INACTIVITY_PROMO_LABEL 2036
#define IDS_TOOLTIP_TAB_ALERT_STATE_GLIC_ACCESSING 2037
#define IDS_TOOLTIP_TAB_ALERT_STATE_GLIC_SHARING 2038
#define IDS_TOOLTIP_GLIC_CLOSE 2039
#define IDS_TAB_AX_LABEL_GLIC_ACCESSING 2040
#define IDS_TAB_AX_LABEL_GLIC_SHARING 2041
#define IDS_GLIC_PROMO_BODY 2042
#define IDS_GLIC_STATUS_ICON_MENU_CUSTOMIZE_KEYBOARD_SHORTCUT 2043
#define IDS_GLIC_STATUS_ICON_MENU_EXIT 2044
#define IDS_GLIC_STATUS_ICON_MENU_SETTINGS 2045
#define IDS_GLIC_OS_WIDGET_TOGGLE_HELP_BUBBLE 2046
#define IDS_GLIC_OS_WIDGET_KEYBOARD_SHORTCUT_HELP_BUBBLE 2047
#define IDS_GLIC_OFFLINE_NOTICE_HEADER 2048
#define IDS_GLIC_OFFLINE_NOTICE_ACTION 2049
#define IDS_GLIC_OFFLINE_NOTICE_ACTION_BUTTON 2050
#define IDS_GLIC_EXPERIMENTAL_OPT_IN_OFFLINE_NOTICE_MESSAGE 2051
#define IDS_GLIC_UNRESPONSIVE_MESSAGE 2052
#define IDS_GLIC_ERROR_NOTICE_HEADER 2053
#define IDS_GLIC_ERROR_NOTICE 2054
#define IDS_GLIC_ERROR_NOTICE_ACTION_BUTTON 2055
#define IDS_GLIC_SHOW_ERROR_BUTTON 2056
#define IDS_GLIC_NOTICE_CLOSE_BUTTON_LABEL 2057
#define IDS_GLIC_SIGN_IN_NOTICE_HEADER 2058
#define IDS_GLIC_SIGN_IN_NOTICE 2059
#define IDS_GLIC_SIGN_IN_NOTICE_ACTION_BUTTON 2060
#define IDS_GLIC_INELIGIBLE_PROFILE_NOTICE_HEADER 2061
#define IDS_GLIC_INELIGIBLE_PROFILE_NOTICE_ACTION_BUTTON 2062
#define IDS_GLIC_DISABLED_BY_ADMIN_NOTICE 2063
#define IDS_GLIC_DISABLED_BY_ADMIN_NOTICE_WITH_LINK 2064
#define IDS_GLIC_DISABLED_BY_ADMIN_NOTICE_CLOSE_BUTTON 2065
#define IDS_GLIC_DISABLED_BY_ADMIN_NOTICE_HEADER 2066
#define IDS_GLIC_INELIGIBLE_ACCOUNT_NOTICE_HEADER 2067
#define IDS_GLIC_INELIGIBLE_ACCOUNT_NOTICE 2068
#define IDS_GLIC_MANAGED_BY_ORGANIZATION 2069
#define IDS_SETTINGS_GLIC_ROW_SUBLABEL 2070
#define IDS_SETTINGS_GLIC_POLICY_DISABLED_MESSAGE 2071
#define IDS_SETTINGS_GLIC_PREFERENCES_SECTION 2072
#define IDS_SETTINGS_GLIC_PERMISSIONS_SECTION 2073
#define IDS_SETTINGS_GLIC_AUTO_BROWSE_SECTION 2074
#define IDS_SETTINGS_GLIC_BUTTON_TOGGLE 2075
#define IDS_SETTINGS_GLIC_KEYBOARD_SHORTCUT 2076
#define IDS_SETTINGS_GLIC_KEYBOARD_SHORTCUT_EDIT_LABEL 2077
#define IDS_SETTINGS_GLIC_KEYBOARD_SHORTCUT_CLEAR_LABEL 2078
#define IDS_SETTINGS_GLIC_KEYBOARD_SHORTCUT_LEARN_MORE_LABEL 2079
#define IDS_SETTINGS_GLIC_KEYBOARD_SHORTCUT_LEARN_MORE_ARIA_LABEL 2080
#define IDS_SETTINGS_GLIC_NAVIGATION_SHORTCUT_LEARN_MORE_ARIA_LABEL 2081
#define IDS_SETTINGS_GLIC_SELECTION_SHORTCUT_LEARN_MORE_ARIA_LABEL 2082
#define IDS_SETTINGS_GLIC_NAVIGATION_SHORTCUT 2083
#define IDS_SETTINGS_GLIC_NAVIGATION_SHORTCUT_EDIT_LABEL 2084
#define IDS_SETTINGS_GLIC_NAVIGATION_SHORTCUT_CLEAR_LABEL 2085
#define IDS_SETTINGS_GLIC_SELECTION_SHORTCUT 2086
#define IDS_SETTINGS_GLIC_SELECTION_SHORTCUT_EDIT_LABEL 2087
#define IDS_SETTINGS_GLIC_SELECTION_SHORTCUT_CLEAR_LABEL 2088
#define IDS_SETTINGS_GLIC_CLOSED_CAPTIONING 2089
#define IDS_SETTINGS_GLIC_CLOSED_CAPTIONING_SUBLABEL 2090
#define IDS_SETTINGS_GLIC_KEEP_SIDEPANEL_OPEN_ON_NEW_TABS 2091
#define IDS_SETTINGS_GLIC_PERMISSIONS_LOCATION_TOGGLE 2092
#define IDS_SETTINGS_GLIC_PERMISSIONS_LOCATION_TOGGLE_SUBLABEL 2093
#define IDS_SETTINGS_GLIC_PERMISSIONS_LOCATION_TOGGLE_SUBLABEL_DATA_PROTECTED 2094
#define IDS_SETTINGS_GLIC_PERMISSIONS_MICROPHONE_TOGGLE 2095
#define IDS_SETTINGS_GLIC_PERMISSIONS_MICROPHONE_TOGGLE_SUBLABEL 2096
#define IDS_SETTINGS_GLIC_PERMISSIONS_MICROPHONE_TOGGLE_SUBLABEL_DATA_PROTECTED 2097
#define IDS_SETTINGS_GLIC_PERMISSIONS_TAB_ACCESS_TOGGLE 2098
#define IDS_SETTINGS_GLIC_PERMISSIONS_TAB_ACCESS_TOGGLE_SUBLABEL 2099
#define IDS_SETTINGS_GLIC_PERMISSIONS_TAB_ACCESS_TOGGLE_SUBLABEL_DATA_PROTECTED 2100
#define IDS_SETTINGS_GLIC_PERMISSIONS_DEFAULT_TAB_ACCESS_TOGGLE 2101
#define IDS_SETTINGS_GLIC_PERMISSIONS_DEFAULT_TAB_ACCESS_TOGGLE_SUBLABEL 2102
#define IDS_SETTINGS_GLIC_PERMISSIONS_DEFAULT_TAB_ACCESS_TOGGLE_SUBLABEL_DATA_PROTECTED 2103
#define IDS_SETTINGS_GLIC_PERMISSIONS_ACTIVITY_BUTTON 2104
#define IDS_SETTINGS_GLIC_PERMISSIONS_ACTIVITY_BUTTON_SUBLABEL 2105
#define IDS_SETTINGS_GLIC_EXTENSIONS_BUTTON 2106
#define IDS_SETTINGS_GLIC_EXTENSIONS_BUTTON_SUBLABEL 2107
#define IDS_SETTINGS_GLIC_PERMISSIONS_TAB_ACCESS_WHEN_ON_1 2108
#define IDS_SETTINGS_GLIC_PERMISSIONS_TAB_ACCESS_CONSIDER_1 2109
#define IDS_SETTINGS_GLIC_PERMISSIONS_TAB_ACCESS_CONSIDER_1_LEARN_MORE_LABEL 2110
#define IDS_SETTINGS_GLIC_PERMISSIONS_DEFAULT_TAB_ACCESS_WHEN_ON_1 2111
#define IDS_SETTINGS_GLIC_PERMISSIONS_DEFAULT_TAB_ACCESS_WHEN_ON_2 2112
#define IDS_SETTINGS_GLIC_PERMISSIONS_DEFAULT_TAB_ACCESS_WHEN_ON_2_NO_LIVE 2113
#define IDS_SETTINGS_GLIC_PERMISSIONS_DEFAULT_TAB_ACCESS_CONSIDER_1 2114
#define IDS_SETTINGS_GLIC_PERMISSIONS_DEFAULT_TAB_ACCESS_CONSIDER_2 2115
#define IDS_SETTINGS_GLIC_PERSONAL_CONTEXT_LABEL 2116
#define IDS_SETTINGS_GLIC_PERSONAL_CONTEXT 2117
#define IDS_SETTINGS_GLIC_INSTRUCTIONS_BUTTON 2118
#define IDS_SETTINGS_GLIC_INSTRUCTIONS_BUTTON_SUBLABEL 2119
#define IDS_SETTINGS_GLIC_EXPERIMENTAL_TRIGGERING_TOGGLE 2120
#define IDS_SETTINGS_GLIC_EXPERIMENTAL_TRIGGERING_SUB_LABEL 2121
#define IDS_SETTINGS_GLIC_EXPERIMENTAL_TRIGGERING_WHEN_ON_1 2122
#define IDS_SETTINGS_GLIC_EXPERIMENTAL_TRIGGERING_WHEN_ON_2 2123
#define IDS_SETTINGS_GLIC_EXPERIMENTAL_TRIGGERING_CONSIDER_1 2124
#define IDS_SETTINGS_GLIC_EXPERIMENTAL_TRIGGERING_CONSIDER_2 2125
#define IDS_SETTINGS_GLIC_EXPERIMENTAL_TRIGGERING_CONSIDER_3 2126
#define IDS_SETTINGS_GLIC_PERMISSIONS_CHROME_WEB_ACTUATION_TOGGLE 2127
#define IDS_SETTINGS_GLIC_PERMISSIONS_CHROME_WEB_ACTUATION_TOGGLE_SUBLABEL 2128
#define IDS_SETTINGS_GLIC_PERMISSIONS_WEB_ACTUATION_TOGGLE_WHEN_ON_1 2129
#define IDS_SETTINGS_GLIC_PERMISSIONS_WEB_ACTUATION_TOGGLE_WHEN_ON_2 2130
#define IDS_SETTINGS_GLIC_PERMISSIONS_WEB_ACTUATION_TOGGLE_CONSIDER_1 2131
#define IDS_SETTINGS_GLIC_PERMISSIONS_WEB_ACTUATION_TOGGLE_CONSIDER_2 2132
#define IDS_SETTINGS_GLIC_ACTOR_LOGIN_PERMISSIONS_SECTION_TITLE 2133
#define IDS_SETTINGS_GLIC_ACTOR_LOGIN_PERMISSIONS_SECTION_SUBLABEL 2134
#define IDS_SETTINGS_GLIC_LOGIN_PERMISSIONS_PAGE_TITLE 2135
#define IDS_SETTINGS_GLIC_LOGIN_PERMISSIONS_OFFLINE_WARNING 2136
#define IDS_SETTINGS_GLIC_LOGIN_PERMISSIONS_PAGE_DESCRIPTION 2137
#define IDS_SETTINGS_GLIC_LOGIN_PERMISSIONS_NO_SITES 2138
#define IDS_SETTINGS_GLIC_REVOKE_ACTOR_LOGIN_PERMISSION_ARIA_LABEL 2139
#define IDS_SETTINGS_GLIC_REMOVE_ACTOR_LOGIN_DIALOG_TITLE 2140
#define IDS_SETTINGS_GLIC_REMOVE_ACTOR_LOGIN_DIALOG_DESCRIPTION 2141
#define IDS_SETTINGS_GLIC_REVOKE_ACTOR_LOGIN_DIALOG_TITLE 2142
#define IDS_SETTINGS_GLIC_REVOKE_ACTOR_LOGIN_DIALOG_DESCRIPTION 2143
#define IDS_SETTINGS_GLIC_LOGIN_PERMISSIONS_REMOVE_ERROR 2144
#define IDS_GLIC_PIN 2145
#define IDS_GLIC_UNPIN 2146
#define IDS_GLIC_SCREEN_PICKER_REQUESTER 2147
#define IDS_GLIC_SCREEN_PICKER_HEADLINE 2148
#define IDS_GLIC_SCREEN_PICKER_DESCRIPTION 2149
#define IDS_GLIC_SCREEN_PICKER_CTA 2150
#define IDS_GLIC_INACTIVE_VIEW_CARD_TEXT 2151
#define IDS_GLIC_WINDOW_TITLE 2152
#define IDS_TASK_MANAGER_GLIC_RENDERER_PREFIX 2153
#define IDS_TAB_CXMENU_GLIC_START_SHARE 2154
#define IDS_TAB_CXMENU_GLIC_CREATE_NEW_CHAT 2155
#define IDS_TAB_CXMENU_GLIC_UNSHARE 2156
#define IDS_GLIC_BUTTON_CXMENU_UNPIN 2157
#define IDS_GLIC_SETTING_LABEL 2158
#define IDS_GLIC_BUTTON_ENTRYPOINT_LABEL 2159
#define IDS_GLIC_BUTTON_ENTRYPOINT_PINNED_LABEL 2160
#define IDS_GLIC_BUTTON_ENTRYPOINT_ASK_GEMINI_LABEL 2161
#define IDS_GLIC_BUTTON_ENTRYPOINT_ASK_ABOUT_THIS_PAGE_LABEL 2162
#define IDS_CONTENT_CONTEXT_GLICSHAREIMAGE 2163
#define IDS_GLIC_CONTEXT_MENU_SUMMARIZE_PAGE_WITH_GEMINI 2164
#define IDS_GLIC_SUMMARIZE_PAGE_PROMPT 2165
#define IDS_GLIC_PROMO_CONFIRM 2166
#define IDS_GLIC_TRYIT_TITLE 2167
#define IDS_GLIC_TRYIT_BODY 2168
#define IDS_GLIC_SHORTCUT_IPH_TEXT_TEMPLATE 2169
#define IDS_GLIC_SHORTCUT_IPH_TEXT 2170
#define IDS_GLIC_SELECTION_ASK_ABOUT 2171
#define IDS_EDUCATIONAL_TIP_GLIC_TITLE 2172
#define IDS_EDUCATIONAL_TIP_GLIC_DESCRIPTION 2173
#define IDS_EDUCATIONAL_TIP_GLIC_BUTTON 2174
#define IDS_EDUCATIONAL_TIP_GLIC_DISMISS_BUTTON 2175
#define IDS_TAB_BOTTOM_SHEET_RESIZING_VIEW_TEXT 2176
#define IDS_GLIC_LOCATION_MISMATCH_NOTICE_HEADER 2177
#define IDS_GLIC_LOCATION_MISMATCH_NOTICE 2178
#define IDS_GLIC_GET_HELP 2179
#define IDS_GLIC_CHAT_HIDDEN_TITLE 2180
#define IDS_GLIC_CHAT_HIDDEN_DESCRIPTION 2181
#define IDS_GLIC_TASK_PAUSED_TITLE 2182
#define IDS_GLIC_TASK_PAUSED_DESCRIPTION 2183
#define IDS_GLOBAL_MEDIA_CONTROLS_ICON_TOOLTIP_TEXT 2184
#define IDS_GLOBAL_MEDIA_CONTROLS_DEVICES_LABEL_WITH_COLON 2185
#define IDS_GLOBAL_MEDIA_CONTROLS_DEVICES_LABEL 2186
#define IDS_GLOBAL_MEDIA_CONTROLS_STOP_CASTING_BUTTON_LABEL 2187
#define IDS_GLOBAL_MEDIA_CONTROLS_LIVE_CAPTION 2188
#define IDS_GLOBAL_MEDIA_CONTROLS_LIVE_CAPTION_SHOW_LANGUAGE 2189
#define IDS_GLOBAL_MEDIA_CONTROLS_LIVE_CAPTION_DOWNLOAD_PROGRESS 2190
#define IDS_GLOBAL_MEDIA_CONTROLS_LIVE_CAPTION_DOWNLOAD_ERROR 2191
#define IDS_GLOBAL_MEDIA_CONTROLS_LIVE_CAPTION_DOWNLOAD_ERROR_REBOOT_REQUIRED 2192
#define IDS_GLOBAL_MEDIA_CONTROLS_CAPTION_SETTINGS 2193
#define IDS_GLOBAL_MEDIA_CONTROLS_LIVE_TRANSLATE_TARGET_LANGUAGE_ACCNAME 2194
#define IDS_GLOBAL_MEDIA_CONTROLS_DIALOG_NAME 2195
#define IDS_GLOBAL_MEDIA_CONTROLS_CONTROL_CAST_SESSIONS_PROMO 2196
#define IDS_GMC_LOCAL_MEDIA_CAST_SESSIONS_PROMO 2197
#define IDS_GMC_LOCAL_MEDIA_CAST_START_PROMO 2198
#define IDS_MEDIA_TOOLBAR_CONTEXT_REPORT_CAST_ISSUE 2199
#define IDS_MEDIA_TOOLBAR_CONTEXT_SHOW_OTHER_SESSIONS 2200
#define IDS_MEDIA_PREVIEW_CAMERA_ACCESSIBLE_NAME 2201
#define IDS_MEDIA_PREVIEW_VIDEO_STREAM_ACCESSIBLE_NAME 2202
#define IDS_MEDIA_PREVIEW_NO_CAMERAS_FOUND 2203
#define IDS_MEDIA_PREVIEW_NO_CAMERAS_FOUND_COMBOBOX 2204
#define IDS_MEDIA_PREVIEW_VIDEO_STREAM_PREVIEW_BADGE 2205
#define IDS_MEDIA_PREVIEW_MIC_ACCESSIBLE_NAME 2206
#define IDS_MEDIA_PREVIEW_AUDIO_STREAM_ACCESSIBLE_NAME 2207
#define IDS_MEDIA_PREVIEW_NO_MICS_FOUND 2208
#define IDS_MEDIA_PREVIEW_NO_MICS_FOUND_COMBOBOX 2209
#define IDS_MEDIA_PREVIEW_SYSTEM_DEFAULT_MIC 2210
#define IDS_MEDIA_PREVIEW_SYSTEM_DEFAULT_MIC_PARENTHETICAL 2211
#define IDS_MEDIA_PREVIEW_ANNOUNCE_SELECTED_DEVICE_CHANGE 2212
#define IDS_MEDIA_ROUTER_ICON_TOOLTIP_TEXT 2213
#define IDS_MEDIA_ROUTER_MENU_ITEM_TITLE 2214
#define IDS_MEDIA_ROUTER_PRESENTATION_CAST_MODE 2215
#define IDS_MEDIA_ROUTER_DESKTOP_MIRROR_CAST_MODE 2216
#define IDS_MEDIA_ROUTER_TAB_MIRROR_CAST_MODE 2217
#define IDS_MEDIA_ROUTER_ALTERNATIVE_SOURCES_BUTTON 2218
#define IDS_MEDIA_ROUTER_ABOUT 2219
#define IDS_MEDIA_ROUTER_HELP 2220
#define IDS_MEDIA_ROUTER_TOGGLE_MEDIA_REMOTING 2221
#define IDS_MEDIA_ROUTER_ISSUE_CREATE_ROUTE_TIMEOUT_WITH_HOSTNAME 2222
#define IDS_MEDIA_ROUTER_ISSUE_CREATE_ROUTE_TIMEOUT 2223
#define IDS_MEDIA_ROUTER_ISSUE_CREATE_ROUTE_TIMEOUT_FOR_DESKTOP 2224
#define IDS_MEDIA_ROUTER_ISSUE_CREATE_ROUTE_TIMEOUT_FOR_TAB 2225
#define IDS_MEDIA_ROUTER_ISSUE_UNABLE_TO_CAST_DESKTOP 2226
#define IDS_MEDIA_ROUTER_ISSUE_TAB_AUDIO_NOT_SUPPORTED 2227
#define IDS_MEDIA_ROUTER_ISSUE_DESKTOP_AUDIO_NOT_SUPPORTED 2228
#define IDS_MEDIA_ROUTER_ISSUE_CREATE_ROUTE_USER_PENDING_AUTHORIZATION 2230
#define IDS_MEDIA_ROUTER_ISSUE_CREATE_ROUTE_USER_NOT_ALLOWED 2231
#define IDS_MEDIA_ROUTER_ISSUE_CREATE_ROUTE_NOTIFICATION_DISABLED 2232
#define IDS_MEDIA_ROUTER_ISSUE_FAILED_TO_CAST 2233
#define IDS_MEDIA_ROUTER_ISSUE_CANNOT_TERMINATE 2234
#define IDS_MEDIA_ROUTER_STATUS_LOOKING_FOR_DEVICES 2235
#define IDS_MEDIA_ROUTER_STATUS_NO_DEVICES_FOUND 2236
#define IDS_MEDIA_ROUTER_NO_DEVICES_FOUND_BUTTON 2237
#define IDS_MEDIA_ROUTER_LOCAL_DISCOVERY_PERMISSION_REJECTED_BUTTON 2238
#define IDS_MEDIA_ROUTER_LOCAL_DISCOVERY_PERMISSION_REJECTED_LABEL 2239
#define IDS_MEDIA_ROUTER_LOCAL_DISCOVERY_PERMISSION_REJECTED_LINK 2240
#define IDS_MEDIA_ROUTER_SINK_AVAILABLE 2241
#define IDS_MEDIA_ROUTER_SINK_CONNECTING 2242
#define IDS_MEDIA_ROUTER_SINK_DISCONNECTING 2243
#define IDS_MEDIA_ROUTER_SINK_PAUSED 2244
#define IDS_MEDIA_ROUTER_STOP_CASTING 2245
#define IDS_MEDIA_ROUTER_SOURCE_NOT_SUPPORTED 2246
#define IDS_MEDIA_ROUTER_AVAILABLE_SPECIFIC_SITES 2247
#define IDS_MEDIA_ROUTER_CASTING_DESKTOP 2248
#define IDS_MEDIA_ROUTER_CASTING_TAB 2249
#define IDS_MEDIA_ROUTER_PRESENTATION_ROUTE_DESCRIPTION 2250
#define IDS_MEDIA_ROUTER_SINK_VIEW_PAUSE 2251
#define IDS_MEDIA_ROUTER_SINK_VIEW_PAUSE_GENERIC_ACCESSIBLE_NAME 2252
#define IDS_MEDIA_ROUTER_SINK_VIEW_PAUSE_SCREEN_ACCESSIBLE_NAME 2253
#define IDS_MEDIA_ROUTER_SINK_VIEW_PAUSE_TAB_ACCESSIBLE_NAME 2254
#define IDS_MEDIA_ROUTER_SINK_VIEW_RESUME 2255
#define IDS_MEDIA_ROUTER_SINK_VIEW_RESUME_GENERIC_ACCESSIBLE_NAME 2256
#define IDS_MEDIA_ROUTER_SINK_VIEW_RESUME_SCREEN_ACCESSIBLE_NAME 2257
#define IDS_MEDIA_ROUTER_SINK_VIEW_RESUME_TAB_ACCESSIBLE_NAME 2258
#define IDS_MEDIA_ROUTER_SINK_VIEW_STOP 2259
#define IDS_MEDIA_ROUTER_SINK_VIEW_STOP_GENERIC_ACCESSIBLE_NAME 2260
#define IDS_MEDIA_ROUTER_SINK_VIEW_STOP_SCREEN_ACCESSIBLE_NAME 2261
#define IDS_MEDIA_ROUTER_SINK_VIEW_STOP_TAB_ACCESSIBLE_NAME 2262
#define IDS_MEDIA_ROUTER_REMOTING_DIALOG_TITLE 2263
#define IDS_MEDIA_ROUTER_REMOTING_DIALOG_BODY_TEXT 2264
#define IDS_MEDIA_ROUTER_REMOTING_DIALOG_CHECKBOX 2265
#define IDS_MEDIA_ROUTER_REMOTING_DIALOG_OPTIMIZE_BUTTON 2266
#define IDS_MEDIA_ROUTER_REMOTING_DIALOG_CANCEL_BUTTON 2267
#define IDS_MEDIA_ROUTER_WIRED_DISPLAY_SINK_NAME 2268
#define IDS_MEDIA_ROUTER_FEEDBACK_TITLE 2269
#define IDS_MEDIA_ROUTER_FEEDBACK_NA 2270
#define IDS_MEDIA_ROUTER_FEEDBACK_FORM_DESCRIPTION 2271
#define IDS_MEDIA_ROUTER_FEEDBACK_HEADER 2272
#define IDS_MEDIA_ROUTER_FEEDBACK_YOUR_ANSWER 2273
#define IDS_MEDIA_ROUTER_FEEDBACK_REQUIRED 2274
#define IDS_MEDIA_ROUTER_FEEDBACK_TYPE_QUESTION 2275
#define IDS_MEDIA_ROUTER_FEEDBACK_PROMPT 2276
#define IDS_MEDIA_ROUTER_FEEDBACK_MIRRORING_QUALITY_SUBHEADING 2277
#define IDS_MEDIA_ROUTER_FEEDBACK_VIDEO_SMOOTHNESS 2278
#define IDS_MEDIA_ROUTER_FEEDBACK_VIDEO_QUALITY 2279
#define IDS_MEDIA_ROUTER_FEEDBACK_AUDIO_QUALITY 2280
#define IDS_MEDIA_ROUTER_FEEDBACK_CONTENT_QUESTION 2281
#define IDS_MEDIA_ROUTER_FEEDBACK_ADDITIONAL_COMMENTS 2282
#define IDS_MEDIA_ROUTER_FEEDBACK_ALLOW_CONTACT_BY_EMAIL 2283
#define IDS_MEDIA_ROUTER_FEEDBACK_YOUR_EMAIL_ADDRESS 2284
#define IDS_MEDIA_ROUTER_FEEDBACK_EMAIL_FIELD 2285
#define IDS_MEDIA_ROUTER_FEEDBACK_SEND_BUTTON 2286
#define IDS_MEDIA_ROUTER_FEEDBACK_DISCARD_CONFIRMATION 2287
#define IDS_MEDIA_ROUTER_FEEDBACK_TYPE_BUG_OR_ERROR 2288
#define IDS_MEDIA_ROUTER_FEEDBACK_TYPE_FEATURE_REQUEST 2289
#define IDS_MEDIA_ROUTER_FEEDBACK_TYPE_PROJECTION_QUALITY 2290
#define IDS_MEDIA_ROUTER_FEEDBACK_TYPE_DISCOVERY 2291
#define IDS_MEDIA_ROUTER_FEEDBACK_TYPE_OTHER 2292
#define IDS_MEDIA_ROUTER_FEEDBACK_VIDEO_FREEZES 2293
#define IDS_MEDIA_ROUTER_FEEDBACK_VIDEO_JERKY 2294
#define IDS_MEDIA_ROUTER_FEEDBACK_VIDEO_STUTTER 2295
#define IDS_MEDIA_ROUTER_FEEDBACK_VIDEO_SMOOTH 2296
#define IDS_MEDIA_ROUTER_FEEDBACK_VIDEO_PERFECT 2297
#define IDS_MEDIA_ROUTER_FEEDBACK_VIDEO_UNWATCHABLE 2298
#define IDS_MEDIA_ROUTER_FEEDBACK_VIDEO_POOR 2299
#define IDS_MEDIA_ROUTER_FEEDBACK_VIDEO_ACCEPTABLE 2300
#define IDS_MEDIA_ROUTER_FEEDBACK_VIDEO_GOOD 2301
#define IDS_MEDIA_ROUTER_FEEDBACK_VIDEO_GREAT 2302
#define IDS_MEDIA_ROUTER_FEEDBACK_AUDIO_UNINTELLIGIBLE 2303
#define IDS_MEDIA_ROUTER_FEEDBACK_AUDIO_POOR 2304
#define IDS_MEDIA_ROUTER_FEEDBACK_AUDIO_ACCEPTABLE 2305
#define IDS_MEDIA_ROUTER_FEEDBACK_AUDIO_GOOD 2306
#define IDS_MEDIA_ROUTER_FEEDBACK_AUDIO_PERFECT 2307
#define IDS_MEDIA_ROUTER_FEEDBACK_SENDING 2308
#define IDS_MEDIA_ROUTER_FEEDBACK_SEND_FAIL 2309
#define IDS_MEDIA_ROUTER_FEEDBACK_SEND_SUCCESS 2310
#define IDS_MEDIA_ROUTER_FEEDBACK_RESENDING 2311
#define IDS_MEDIA_ROUTER_FEEDBACK_LOGS_HEADER 2312
#define IDS_MEDIA_ROUTER_FEEDBACK_FINE_LOGS_WARNING 2313
#define IDS_MEDIA_ROUTER_FEEDBACK_SEND_LOGS_HTML 2314
#define IDS_MEDIA_ROUTER_FEEDBACK_SEND_LOGS 2315
#define IDS_MEDIA_ROUTER_FEEDBACK_PRIVACY_DATA_USAGE 2316
#define IDS_MEDIA_ROUTER_FEEDBACK_SETUP_VISIBILITY_QUESTION 2317
#define IDS_MEDIA_ROUTER_FEEDBACK_YES 2318
#define IDS_MEDIA_ROUTER_FEEDBACK_NO 2319
#define IDS_MEDIA_ROUTER_FEEDBACK_DID_NOT_TRY 2320
#define IDS_MEDIA_ROUTER_FEEDBACK_SOFTWARE_QUESTION 2321
#define IDS_MEDIA_ROUTER_FEEDBACK_UNKNOWN 2322
#define IDS_MEDIA_ROUTER_FEEDBACK_NETWORK_QUESTION 2323
#define IDS_MEDIA_ROUTER_FEEDBACK_NETWORK_SAME_WIFI 2324
#define IDS_MEDIA_ROUTER_FEEDBACK_NETWORK_DIFFERENT_WIFI 2325
#define IDS_MEDIA_ROUTER_FEEDBACK_NETWORK_WIRED_PC 2326
#define IDS_AVATAR_BUTTON_GUEST 2327
#define IDS_AVATAR_BUTTON_GUEST_TOOLTIP 2328
#define IDS_INCOGNITO_BUBBLE_ACCESSIBLE_TITLE 2329
#define IDS_AVATAR_BUTTON_INCOGNITO 2330
#define IDS_AVATAR_BUTTON_INCOGNITO_TOOLTIP 2331
#define IDS_AVATAR_BUTTON_SYNC_ERROR 2332
#define IDS_AVATAR_BUTTON_SYNC_ERROR_TOOLTIP 2333
#define IDS_AVATAR_BUTTON_SYNC_PAUSED 2334
#define IDS_AVATAR_BUTTON_SYNC_ERROR_BOOKMARKS_LIMIT_EXCEEDED 2335
#define IDS_AVATAR_BUTTON_SIGNIN_PAUSED 2336
#define IDS_AVATAR_BUTTON_WORK 2337
#define IDS_AVATAR_BUTTON_SCHOOL 2338
#define IDS_AVATAR_BUTTON_GREETING 2339
#define IDS_AVATAR_BUTTON_PASSKEYS_ERROR_UNLOCK 2340
#define IDS_AVATAR_BUTTON_PASSKEYS_ERROR_GET 2341
#define IDS_AVATAR_BUTTON_PASSKEYS_ERROR_VERIFY 2342
#define IDS_PROFILE_MENU_PASSKEYS_ERROR_DESCRIPTION_UNLOCK 2343
#define IDS_PROFILE_MENU_PASSKEYS_ERROR_DESCRIPTION_GET 2344
#define IDS_PROFILE_MENU_PASSKEYS_ERROR_DESCRIPTION_VERIFY 2345
#define IDS_PROFILE_MENU_PASSKEYS_ERROR_BUTTON_UNLOCK 2346
#define IDS_PROFILE_MENU_PASSKEYS_ERROR_BUTTON_GET 2347
#define IDS_PROFILE_MENU_PASSKEYS_ERROR_BUTTON_VERIFY 2348
#define IDS_PROFILES_MENU_NAME 2349
#define IDS_PROFILES_ADD_PROFILE_LABEL 2350
#define IDS_PROFILES_PROFILE_BUBBLE_ACCESSIBLE_TITLE 2351
#define IDS_PROFILES_ACCOUNT_REMOVAL_TITLE 2352
#define IDS_PROFILES_DICE_SIGNIN_BUTTON 2353
#define IDS_PROFILES_DICE_WEB_ONLY_SIGNIN_BUTTON 2354
#define IDS_PROFILES_DICE_SYNC_DISABLED_TITLE 2355
#define IDS_PROFILES_DASHER_FEATURE_DISABLED_TITLE 2356
#define IDS_PROFILES_DICE_SYNC_PAUSED_TITLE 2357
#define IDS_AI_SIGNIN_PROMO_TITLE 2358
#define IDS_AI_VERIFY_PROMO_SUBTITLE 2359
#define IDS_PROFILES_VERIFY_ACCOUNT_BUTTON 2360
#define IDS_PROFILE_MENU_SIGNIN_PROMO_DESCRIPTION 2361
#define IDS_PROFILE_MENU_SIGNIN_PROMO_DESCRIPTION_WITH_BOOKMARKS 2362
#define IDS_PROFILE_MENU_PROFILE_IDENTIFIER_WITH_SEPARATOR 2363
#define IDS_PROFILE_MENU_PROFILE_MANAGED_HEADER 2364
#define IDS_GUEST_PROFILE_MENU_CLOSE_X_WINDOWS_BUTTON 2365
#define IDS_INCOGNITO_PROFILE_MENU_CLOSE_X_WINDOWS_BUTTON 2366
#define IDS_PROFILE_MENU_OPEN_GUEST_PROFILE 2367
#define IDS_PROFILE_MENU_CLOSE_PROFILE_X_WINDOWS_BUTTON 2368
#define IDS_PROFILES_OPEN_SYNC_SETTINGS_BUTTON 2369
#define IDS_PROFILE_MENU_CUSTOMIZE_PROFILE_BUTTON 2370
#define IDS_PROFILE_MENU_OPEN_ACCOUNT_SETTINGS 2371
#define IDS_PROFILE_MENU_AUTOFILL_SETTINGS_BUTTON 2372
#define IDS_PROFILE_MENU_BATCH_UPLOAD_BUTTON 2373
#define IDS_PROFILES_GUEST_PROFILE_NAME 2374
#define IDS_DEFAULT_PROFILE_NAME 2375
#define IDS_LEGACY_DEFAULT_PROFILE_NAME 2376
#define IDS_NEW_NUMBERED_PROFILE_NAME 2377
#define IDS_SINGLE_PROFILE_DISPLAY_NAME 2378
#define IDS_GUEST_PROFILE_NAME 2379
#define IDS_DEFAULT_AVATAR_NAME_8 2380
#define IDS_DEFAULT_AVATAR_NAME_9 2381
#define IDS_DEFAULT_AVATAR_NAME_10 2382
#define IDS_DEFAULT_AVATAR_NAME_11 2383
#define IDS_DEFAULT_AVATAR_NAME_12 2384
#define IDS_DEFAULT_AVATAR_NAME_13 2385
#define IDS_DEFAULT_AVATAR_NAME_14 2386
#define IDS_DEFAULT_AVATAR_NAME_15 2387
#define IDS_DEFAULT_AVATAR_NAME_16 2388
#define IDS_DEFAULT_AVATAR_NAME_17 2389
#define IDS_DEFAULT_AVATAR_NAME_18 2390
#define IDS_DEFAULT_AVATAR_NAME_19 2391
#define IDS_DEFAULT_AVATAR_NAME_20 2392
#define IDS_DEFAULT_AVATAR_NAME_21 2393
#define IDS_DEFAULT_AVATAR_NAME_22 2394
#define IDS_DEFAULT_AVATAR_NAME_23 2395
#define IDS_DEFAULT_AVATAR_NAME_24 2396
#define IDS_DEFAULT_AVATAR_NAME_25 2397
#define IDS_DEFAULT_AVATAR_NAME_26 2398
#define IDS_DEFAULT_AVATAR_LABEL_0 2399
#define IDS_DEFAULT_AVATAR_LABEL_1 2400
#define IDS_DEFAULT_AVATAR_LABEL_2 2401
#define IDS_DEFAULT_AVATAR_LABEL_3 2402
#define IDS_DEFAULT_AVATAR_LABEL_4 2403
#define IDS_DEFAULT_AVATAR_LABEL_5 2404
#define IDS_DEFAULT_AVATAR_LABEL_6 2405
#define IDS_DEFAULT_AVATAR_LABEL_7 2406
#define IDS_DEFAULT_AVATAR_LABEL_8 2407
#define IDS_DEFAULT_AVATAR_LABEL_9 2408
#define IDS_DEFAULT_AVATAR_LABEL_10 2409
#define IDS_DEFAULT_AVATAR_LABEL_11 2410
#define IDS_DEFAULT_AVATAR_LABEL_12 2411
#define IDS_DEFAULT_AVATAR_LABEL_13 2412
#define IDS_DEFAULT_AVATAR_LABEL_14 2413
#define IDS_DEFAULT_AVATAR_LABEL_15 2414
#define IDS_DEFAULT_AVATAR_LABEL_16 2415
#define IDS_DEFAULT_AVATAR_LABEL_17 2416
#define IDS_DEFAULT_AVATAR_LABEL_18 2417
#define IDS_DEFAULT_AVATAR_LABEL_19 2418
#define IDS_DEFAULT_AVATAR_LABEL_20 2419
#define IDS_DEFAULT_AVATAR_LABEL_21 2420
#define IDS_DEFAULT_AVATAR_LABEL_22 2421
#define IDS_DEFAULT_AVATAR_LABEL_23 2422
#define IDS_DEFAULT_AVATAR_LABEL_24 2423
#define IDS_DEFAULT_AVATAR_LABEL_25 2424
#define IDS_DEFAULT_AVATAR_LABEL_26 2425
#define IDS_DEFAULT_AVATAR_LABEL_27 2426
#define IDS_DEFAULT_AVATAR_LABEL_28 2427
#define IDS_DEFAULT_AVATAR_LABEL_29 2428
#define IDS_DEFAULT_AVATAR_LABEL_30 2429
#define IDS_DEFAULT_AVATAR_LABEL_31 2430
#define IDS_DEFAULT_AVATAR_LABEL_32 2431
#define IDS_DEFAULT_AVATAR_LABEL_33 2432
#define IDS_DEFAULT_AVATAR_LABEL_34 2433
#define IDS_DEFAULT_AVATAR_LABEL_35 2434
#define IDS_DEFAULT_AVATAR_LABEL_36 2435
#define IDS_DEFAULT_AVATAR_LABEL_37 2436
#define IDS_DEFAULT_AVATAR_LABEL_38 2437
#define IDS_DEFAULT_AVATAR_LABEL_39 2438
#define IDS_DEFAULT_AVATAR_LABEL_40 2439
#define IDS_DEFAULT_AVATAR_LABEL_41 2440
#define IDS_DEFAULT_AVATAR_LABEL_42 2441
#define IDS_DEFAULT_AVATAR_LABEL_43 2442
#define IDS_DEFAULT_AVATAR_LABEL_44 2443
#define IDS_DEFAULT_AVATAR_LABEL_45 2444
#define IDS_DEFAULT_AVATAR_LABEL_46 2445
#define IDS_DEFAULT_AVATAR_LABEL_47 2446
#define IDS_DEFAULT_AVATAR_LABEL_48 2447
#define IDS_DEFAULT_AVATAR_LABEL_49 2448
#define IDS_DEFAULT_AVATAR_LABEL_50 2449
#define IDS_DEFAULT_AVATAR_LABEL_51 2450
#define IDS_DEFAULT_AVATAR_LABEL_52 2451
#define IDS_DEFAULT_AVATAR_LABEL_53 2452
#define IDS_DEFAULT_AVATAR_LABEL_54 2453
#define IDS_DEFAULT_AVATAR_LABEL_55 2454
#define IDS_PROFILES_LOCAL_PROFILE_STATE 2455
#define IDS_PROFILES_MANAGE_BUTTON_LABEL 2456
#define IDS_PROFILES_DEFAULT_NAME 2457
#define IDS_SYNC_LOGIN_NAME_PROHIBITED 2458
#define IDS_SIGN_IN_LOGIN_NAME_PROHIBITED 2459
#define IDS_SYNC_USER_NAME_IN_USE_ERROR 2460
#define IDS_SYNC_USER_NAME_IN_USE_BY_ERROR 2461
#define IDS_SCREEN_LOCK_SIGN_OUT 2462
#define IDS_FEATURE_SHOWCASE_PASSWORD_MANAGER_TITLE 2467
#define IDS_FEATURE_SHOWCASE_PASSWORD_MANAGER_SUBTITLE 2468
#define IDS_FEATURE_SHOWCASE_PASSWORD_MANAGER_NO_THANKS 2469
#define IDS_FEATURE_SHOWCASE_PASSWORD_MANAGER_ADD_TO_TOOLBAR 2470
#define IDS_FEATURE_SHOWCASE_PASSWORD_MANAGER_ILLUSTRATION_A11Y_LABEL 2471
#define IDS_BATCH_UPLOAD_TITLE 2472
#define IDS_BATCH_UPLOAD_SUBTITLE_DESCRIPTION_ITEMS_WITH_DEDUPLICATION 2473
#define IDS_BATCH_UPLOAD_SUBTITLE_DESCRIPTION_PASSWORDS_WITH_DEDUPLICATION 2474
#define IDS_BATCH_UPLOAD_SUBTITLE_DESCRIPTION_PASSWORDS_COMBO_WITH_DEDUPLICATION 2475
#define IDS_BATCH_UPLOAD_SUBTITLE_DESCRIPTION_BOOKMARKS_WITH_DEDUPLICATION 2476
#define IDS_BATCH_UPLOAD_SUBTITLE_DESCRIPTION_BOOKMARKS_COMBO_WITH_DEDUPLICATION 2477
#define IDS_BATCH_UPLOAD_SUBTITLE_DESCRIPTION_ITEMS 2478
#define IDS_BATCH_UPLOAD_SUBTITLE_DESCRIPTION_PASSWORDS 2479
#define IDS_BATCH_UPLOAD_SUBTITLE_DESCRIPTION_PASSWORDS_COMBO 2480
#define IDS_BATCH_UPLOAD_SUBTITLE_DESCRIPTION_BOOKMARKS 2481
#define IDS_BATCH_UPLOAD_SUBTITLE_DESCRIPTION_BOOKMARKS_COMBO 2482
#define IDS_BATCH_UPLOAD_SECTION_TITLE_PASSWORDS 2483
#define IDS_BATCH_UPLOAD_SECTION_TITLE_BOOKMARKS 2484
#define IDS_BATCH_UPLOAD_SECTION_TITLE_READING_LIST 2485
#define IDS_BATCH_UPLOAD_SECTION_TITLE_ADDRESSES 2486
#define IDS_BATCH_UPLOAD_SECTION_TITLE_EXTENSIONS 2487
#define IDS_BATCH_UPLOAD_SECTION_TITLE_THEMES_WITH_DESCRIPTION 2488
#define IDS_BATCH_UPLOAD_SAVE_TO_ACCOUNT_OK_BUTTON_LABEL 2489
#define IDS_BATCH_UPLOAD_AVATAR_BUTTON_SAVING_TO_ACCOUNT 2490
#define IDS_BATCH_UPLOAD_LAST_ITEM_SELECTED_SCREEN_READER 2491
#define IDS_BATCH_UPLOAD_SCREEN_READER_ITEM_COUNT_SELECTED 2492
#define IDS_BATCH_UPLOAD_SCREEN_READER_SELECT_ALL 2493
#define IDS_BATCH_UPLOAD_PROMO_TITLE 2494
#define IDS_BATCH_UPLOAD_PROMO_TITLE_OK_BUTTON_LABEL 2495
#define IDS_BATCH_UPLOAD_PROMO_SUBTITLE_BOOKMARKS 2496
#define IDS_BATCH_UPLOAD_PROMO_SUBTITLE_BOOKMARKS_COMBO 2497
#define IDS_BATCH_UPLOAD_PROMO_SUBTITLE_ITEMS_WITH_LINK 2498
#define IDS_PROFILE_CUSTOMIZATION_DONE_BUTTON_LABEL 2499
#define IDS_PROFILE_CUSTOMIZATION_THEME_PICKER_LABEL 2500
#define IDS_PROFILE_CUSTOMIZATION_WELCOME 2501
#define IDS_PROFILE_CUSTOMIZATION_INPUT_LABEL 2502
#define IDS_PROFILE_CUSTOMIZATION_INPUT_TITLE 2503
#define IDS_PROFILE_CUSTOMIZATION_INPUT_PLACEHOLDER 2504
#define IDS_PROFILE_CUSTOMIZATION_INPUT_ERROR_MESSAGE 2505
#define IDS_PROFILE_CUSTOMIZATION_SKIP_BUTTON_LABEL 2506
#define IDS_PROFILE_CUSTOMIZATION_DELETE_PROFILE_BUTTON_LABEL 2507
#define IDS_PROFILE_CUSTOMIZATION_LOCAL_PROFILE_CREATION_TITLE 2508
#define IDS_PROFILE_CUSTOMIZATION_AVATAR_SELECTION_BACK_BUTTON_LABEL 2509
#define IDS_ENTERPRISE_PROFILE_WELCOME_TITLE 2510
#define IDS_ENTERPRISE_PROFILE_WELCOME_ACCOUNT_MANAGED_BY 2511
#define IDS_ENTERPRISE_PROFILE_WELCOME_PROFILE_MANAGED_STRICT_SEPARATION 2512
#define IDS_ENTERPRISE_PROFILE_WELCOME_PROFILE_MANAGED_SEPARATION 2513
#define IDS_ENTERPRISE_PROFILE_WELCOME_PROFILE_SEPARATION_DEVICE_MANAGED 2514
#define IDS_ENTERPRISE_PROFILE_WELCOME_PROFILE_SEPARATION_ACCOUNT_MANAGED 2515
#define IDS_ENTERPRISE_PROFILE_WELCOME_PROFILE_SEPARATION_DEVICE_MANAGED_BY 2516
#define IDS_ENTERPRISE_PROFILE_WELCOME_ACCOUNT_EMAIL_MANAGED_BY 2517
#define IDS_ENTERPRISE_PROFILE_WELCOME_DEVICE_MANAGED_BY 2518
#define IDS_ENTERPRISE_PROFILE_WELCOME_DEVICE_MANAGED 2519
#define IDS_ENTERPRISE_WELCOME_PROFILE_REQUIRED_TITLE 2520
#define IDS_ENTERPRISE_WELCOME_PROFILE_SETUP_TITLE 2521
#define IDS_ENTERPRISE_WELCOME_PROFILE_WILL_BE_MANAGED_TITLE 2522
#define IDS_ENTERPRISE_PROFILE_WELCOME_CREATE_PROFILE_BUTTON 2523
#define IDS_ENTERPRISE_PROFILE_WELCOME_LINK_DATA_CHECKBOX 2524
#define IDS_ENTERPRISE_WELCOME_PROFILE_DISCLOSURE_SCHOOL_TITLE 2525
#define IDS_ENTERPRISE_WELCOME_PROFILE_DISCLOSURE_WORK_TITLE 2526
#define IDS_ENTERPRISE_WELCOME_PROFILE_OIDC_DISCLOSURE_TITLE 2527
#define IDS_ENTERPRISE_WELCOME_PROFILE_DISCLOSURE_ALREADY_SIGNED_IN_TITLE 2528
#define IDS_ENTERPRISE_WELCOME_PROFILE_DISCLOSURE_ALREADY_SIGNED_IN_ENFORCED_SUBTITLE 2529
#define IDS_ENTERPRISE_WELCOME_PROFILE_DISCLOSURE_ALREADY_SIGNED_IN_SUBTITLE 2530
#define IDS_ENTERPRISE_WELCOME_PROFILE_DISCLOSURE_SUBTITLE 2531
#define IDS_ENTERPRISE_WELCOME_PROFILE_DISCLOSURE_KNOWN_DOMAIN_SUBTITLE 2532
#define IDS_ENTERPRISE_WELCOME_PROFILE_INFORMATION_TITLE 2533
#define IDS_ENTERPRISE_WELCOME_DEVICE_INFORMATION_TITLE 2534
#define IDS_ENTERPRISE_WELCOME_DEVICE_INFORMATION_DETAILS 2535
#define IDS_ENTERPRISE_PROFILE_WELCOME_BACK 2536
#define IDS_ENTERPRISE_OIDC_WELCOME_PROCESSING_SUBTITLE 2537
#define IDS_ENTERPRISE_OIDC_WELCOME_LONG_PROCESSING_SUBTITLE 2538
#define IDS_ENTERPRISE_OIDC_WELCOME_SUCCESS_TITLE 2539
#define IDS_ENTERPRISE_OIDC_WELCOME_SUCCESS_SUBTITLE 2540
#define IDS_ENTERPRISE_OIDC_WELCOME_TIMEOUT_TITLE 2541
#define IDS_ENTERPRISE_OIDC_WELCOME_TIMEOUT_SUBTITLE 2542
#define IDS_ENTERPRISE_OIDC_WELCOME_TIMEOUT_RETRY_LABEL 2543
#define IDS_ENTERPRISE_OIDC_WELCOME_ERROR_TITLE 2544
#define IDS_ENTERPRISE_OIDC_WELCOME_ERROR_SUBTITLE 2545
#define IDS_ENTERPRISE_OIDC_WELCOME_ERROR_NO_SIGNIN_TITLE 2546
#define IDS_ENTERPRISE_OIDC_WELCOME_ERROR_NO_SIGNIN_SUBTITLE 2547
#define IDS_ENTERPRISE_VALUE_PROPOSITION_CONSUMER_SUBTITLE 2548
#define IDS_ENTERPRISE_VALUE_PROPOSITION_CONSUMER_SUBTITLE_WITH_BOOKMARKS 2549
#define IDS_ENTERPRISE_VALUE_PROPOSITION_SCHOOL_SUBTITLE 2550
#define IDS_ENTERPRISE_VALUE_PROPOSITION_WORK_SUBTITLE 2551
#define IDS_ENTERPRISE_WELCOME_SEPARATE_BROWSING_WORK_TITLE 2552
#define IDS_ENTERPRISE_WELCOME_SEPARATE_BROWSING_SCHOOL_TITLE 2553
#define IDS_ENTERPRISE_WELCOME_SEPARATE_BROWSING_ALREADY_SIGNED_IN_TITLE 2554
#define IDS_ENTERPRISE_WELCOME_SEPARATE_BROWSING_CONSUMER_TITLE 2555
#define IDS_ENTERPRISE_WELCOME_SEPARATE_BROWSING_DATA_SCHOOL_CHOICE 2556
#define IDS_ENTERPRISE_WELCOME_SEPARATE_BROWSING_DATA_WORK_CHOICE 2557
#define IDS_ENTERPRISE_WELCOME_SEPARATE_BROWSING_ALREADY_SIGNED_IN_CHOICE 2558
#define IDS_ENTERPRISE_WELCOME_SEPARATE_BROWSING_DATA_CHOICE_SCHOOL_NOT_RECOMMENDED 2559
#define IDS_ENTERPRISE_WELCOME_SEPARATE_BROWSING_DATA_CHOICE_WORK_NOT_RECOMMENDED 2560
#define IDS_ENTERPRISE_WELCOME_SEPARATE_BROWSING_DATA_CHOICE_DETAILS 2561
#define IDS_ENTERPRISE_WELCOME_SEPARATE_BROWSING_DATA_CHOICE_ALREADY_SIGNED_IN_DETAILS 2562
#define IDS_ENTERPRISE_WELCOME_MERGE_BROWSING_DATA_WORK_CHOICE 2563
#define IDS_ENTERPRISE_WELCOME_MERGE_BROWSING_DATA_SCHOOL_CHOICE 2564
#define IDS_ENTERPRISE_WELCOME_MERGE_BROWSING_DATA_ALREADY_SIGNED_IN_CHOICE 2565
#define IDS_ENTERPRISE_WELCOME_MERGE_BROWSING_DATA_CHOICE_DETAILS 2566
#define IDS_BROWSING_HISTORY_COUNT 2567
#define IDS_BOOKMARKS_COUNT 2568
#define IDS_EXTENSIONS_COUNT 2569
#define IDS_ENTERPRISE_WELCOME_MERGE_BROWSING_DATA_WITH_THREE_COUNTS_CHOICE_DETAILS 2570
#define IDS_ENTERPRISE_WELCOME_MERGE_BROWSING_DATA_WITH_TWO_COUNTS_CHOICE_DETAILS 2571
#define IDS_ENTERPRISE_WELCOME_MERGE_BROWSING_DATA_WITH_ONE_COUNT_CHOICE_DETAILS 2572
#define IDS_ACCNAME_YOUR_AVATAR 2573
#define IDS_ACCNAME_ENTERPRISE_ORGANIZATION_ICON 2574
#define IDS_SUPERVISED_USER_PROFILE_SIGNIN_IPH_TITLE 2575
#define IDS_SUPERVISED_USER_PROFILE_SIGNIN_IPH_TEXT 2576
#define IDS_MANAGED_BY_PARENT_A11Y 2577
#define IDS_CHROME_SIGNOUT_CONFIRMATION_PROMPT_KIDS_BODY 2578
#define IDS_PROFILE_PICKER_ADD_SPACE_BUTTON 2579
#define IDS_PROFILE_PICKER_BROWSE_AS_GUEST_BUTTON 2580
#define IDS_PROFILE_PICKER_BACK_BUTTON_ARIA_LABEL 2581
#define IDS_PROFILE_PICKER_BACK_BUTTON_SIGN_IN_LABEL 2582
#define IDS_PROFILE_PICKER_PAUSE_EFFECTS_BUTTON 2583
#define IDS_PROFILE_PICKER_PLAY_EFFECTS_BUTTON 2584
#define IDS_PROFILE_PICKER_PROFILE_CARD_NEEDS_SIGNIN_PROMPT 2585
#define IDS_PROFILE_PICKER_PROFILE_CARD_LABEL_SUPERVISED 2586
#define IDS_PROFILE_PICKER_PROFILE_CARD_LABEL 2587
#define IDS_PROFILE_PICKER_PROFILE_CARD_INPUT_LABEL 2588
#define IDS_PROFILE_PICKER_PROFILE_MORE_ACTIONS_ARIA_LABEL 2589
#define IDS_PROFILE_PICKER_PROFILE_MENU_REMOVE_TEXT 2590
#define IDS_PROFILE_PICKER_PROFILE_MENU_CUSTOMIZE_TEXT 2591
#define IDS_PROFILE_PICKER_ASK_ON_STARTUP 2592
#define IDS_PROFILE_PICKER_OPEN_ALL_PROFILES_BUTTON 2593
#define IDS_PROFILE_PICKER_REMOVE_WARNING_LOCAL_PROFILE_TITLE 2594
#define IDS_PROFILE_PICKER_FORCE_SIGN_IN_ERROR_DIALOG_NOT_ALLOWED_TITLE 2595
#define IDS_PROFILE_PICKER_FORCE_SIGN_IN_ERROR_DIALOG_NOT_ALLOWED_BODY 2596
#define IDS_PROFILE_PICKER_FORCE_SIGN_IN_ERROR_DIALOG_WRONG_ACCOUNT_TITLE 2597
#define IDS_PROFILE_PICKER_FORCE_SIGN_IN_ERROR_DIALOG_WRONG_ACCOUNT_BODY 2598
#define IDS_PROFILE_PICKER_FORCE_SIGN_IN_ERROR_TIMEOUT_TITLE 2599
#define IDS_PROFILE_PICKER_FORCE_SIGN_IN_ERROR_TIMEOUT_BODY 2600
#define IDS_PROFILE_PICKER_FORCE_SIGN_IN_ERROR_DIALOG_NOT_SUPPORTED_BY_GLIC_FLOW_TITLE 2601
#define IDS_PROFILE_PICKER_REMOVE_WARNING_HISTORY 2602
#define IDS_PROFILE_PICKER_REMOVE_WARNING_PASSWORDS 2603
#define IDS_PROFILE_PICKER_REMOVE_WARNING_BOOKMARKS 2604
#define IDS_PROFILE_PICKER_REMOVE_WARNING_AUTOFILL 2605
#define IDS_PROFILE_PICKER_REMOVE_WARNING_CALCULATING 2606
#define IDS_PROFILE_PICKER_PROFILE_CREATION_FLOW_SIGNIN_BUTTON_LABEL 2607
#define IDS_PROFILE_PICKER_PROFILE_CREATION_FLOW_NOT_NOW_BUTTON_LABEL 2608
#define IDS_PROFILE_PICKER_PROFILE_CREATION_FLOW_DONT_SIGN_IN_BUTTON_LABEL 2609
#define IDS_PROFILE_PICKER_PROFILE_CREATION_FLOW_STAY_SIGNED_OUT_BUTTON_LABEL 2610
#define IDS_PROFILE_PICKER_PROFILE_CREATION_FLOW_LOCAL_PROFILE_CREATION_CUSTOMIZE_AVATAR_BUTTON_LABEL 2611
#define IDS_PROFILE_PICKER_PROFILE_CREATION_FLOW_LOCAL_PROFILE_CREATION_AVATAR_TEXT 2612
#define IDS_PROFILE_PICKER_IPH_NEXT_BUTTON_LABEL 2613
#define IDS_PROFILE_PICKER_PROFILE_SWITCH_SWITCH_BUTTON_LABEL 2614
#define IDS_PROFILE_PICKER_PROFILE_CREATION_FLOW_DEVICE_MANAGED_BY_DESCRIPTION 2615
#define IDS_PROFILE_PICKER_PROFILE_CREATION_FLOW_DEVICE_MANAGED_DESCRIPTION 2616
#define IDS_PROFILE_PICKER_REMOVE_WARNING_LOCAL_PROFILE 2617
#define IDS_PROFILE_PICKER_REMOVE_WARNING_SIGNED_IN_PROFILE_TITLE 2618
#define IDS_PROFILE_PICKER_REMOVE_WARNING_SIGNED_IN_PROFILE 2619
#define IDS_PROFILE_PICKER_REMOVE_WARNING_SIGNED_IN_PROFILE_SIGN_IN_AS 2620
#define IDS_PROFILE_PICKER_PROFILE_CREATION_FLOW_PROFILE_TYPE_CHOICE_SUBTITLE_UNO 2621
#define IDS_PROFILE_PICKER_PROFILE_CREATION_FLOW_PROFILE_TYPE_CHOICE_SUBTITLE_UNO_BOOKMARKS 2622
#define IDS_FRE_SIGN_IN_SUBTITLE_0 2623
#define IDS_FRE_DEVICES_CARD_TITLE 2624
#define IDS_FRE_DEVICES_CARD_DESCRIPTION 2625
#define IDS_FRE_SECURITY_CARD_TITLE 2626
#define IDS_FRE_SECURITY_CARD_DESCRIPTION 2627
#define IDS_FRE_BACKUP_CARD_TITLE 2628
#define IDS_UNO_FRE_BACKUP_CARD_DESCRIPTION 2629
#define IDS_UNO_FRE_BACKUP_CARD_DESCRIPTION_WITH_PASSWORDS 2630
#define IDS_UNO_FRE_REFRESH_BACKUP_CARD_DESCRIPTION_WITH_PASSWORDS 2631
#define IDS_FRE_DECLINE_SIGN_IN_BUTTON_TITLE 2632
#define IDS_FRE_STAY_SIGNED_OUT_BUTTON_TITLE 2633
#define IDS_FRE_ACCEPT_SIGN_IN_BUTTON_TITLE 2634
#define IDS_FRE_CREATE_ACCOUNT_DESCRIPTION 2635
#define IDS_FRE_SIGN_IN_CELEBRATION_WELCOME_TITLE 2636
#define IDS_FRE_MANAGED_DESCRIPTION 2637
#define IDS_FRE_MANAGED_BY_DESCRIPTION 2638
#define IDS_FRE_NATIVE_TOOLBAR_DONT_SIGN_IN_BUTTON_LABEL 2639
#define IDS_FRE_NEXT_BUTTON_TITLE 2640
#define IDS_FRE_FINISH_OR_CONTINUE_TITLE 2641
#define IDS_FRE_FINISH_OR_CONTINUE_SEE_MORE_TIPS_BUTTON_LABEL 2642
#define IDS_FRE_FINISH_OR_CONTINUE_START_BROWSING_BUTTON_LABEL 2643
#define IDS_FRE_DEFAULT_BROWSER_TITLE_NEW 2644
#define IDS_FRE_DEFAULT_BROWSER_SET_AS_DEFAULT 2645
#define IDS_FRE_DEFAULT_BROWSER_SKIP 2646
#define IDS_FRE_REFRESH_DEFAULT_BROWSER_SET_AS_DEFAULT 2647
#define IDS_FRE_REFRESH_DEFAULT_BROWSER_NO_THANKS 2648
#define IDS_SIGNOUT_CONFIRMATION_EXTENSIONS_SECTION_TITLE 2649
#define IDS_SIGNOUT_CONFIRMATION_EXTENSIONS_SECTION_TOOLTIP 2650
#define IDS_SIGNOUT_CONFIRMATION_UNSYNCED_DATA_WITH_ACCOUNT_EXTENSIONS 2651
#define IDS_HISTORY_SYNC_OPT_IN_TITLE 2652
#define IDS_HISTORY_SYNC_OPT_IN_SUBTITLE 2653
#define IDS_HISTORY_SYNC_OPT_IN_DESCRIPTION 2654
#define IDS_HISTORY_SYNC_OPT_IN_ACCEPT_BUTTON 2655
#define IDS_HISTORY_SYNC_OPT_IN_REJECT_BUTTON 2656
#define IDS_TABS_DISABLED_ERROR_DESCRIPTION 2657
#define IDS_AVATAR_BUTTON_SYNC_HISTORY 2658
#define IDS_AVATAR_BUTTON_SYNC_PROMO 2659
#define IDS_AVATAR_BUTTON_BATCH_UPLOAD_PROMO 2660
#define IDS_AVATAR_BUTTON_BATCH_UPLOAD_PROMO_WITH_BOOKMARK_CLEANUP_PROMO 2661
#define IDS_PROFILE_MENU_SYNC_PROMO_SYNC_HISTORY_DESCRIPTION 2662
#define IDS_PROFILE_MENU_SYNC_PROMO_BUTTON_LABEL 2663
#define IDS_PROFILE_MENU_DESCRIPTION_WITH_SYNC_PROMO 2664
#define IDS_PROFILE_MENU_BUTTON_LABEL_WITH_SYNC_PROMO 2665
#define IDS_PROFILE_MENU_PROMO_DESCRIPTION_WITH_BATCH_UPLOAD 2666
#define IDS_PROFILE_MENU_PROMO_BUTTON_WITH_BATCH_UPLOAD 2667
#define IDS_PROFILE_MENU_PROMO_DESCRIPTION_WITH_BATCH_UPLOAD_BOOKMARK_CLEANUP 2668
#define IDS_SIGN_IN_BENEFITS_IPH_TEXT 2669
#define IDS_SIGN_IN_BENEFITS_WITHOUT_BOOKMARKS_AND_EXTENSIONS_IPH_TEXT 2670
#define IDS_TYPE_A_SHORTCUT 2671
#define IDS_SHORTCUT_SET 2672
#define IDS_SHORTCUT_NOT_SET 2673
#define IDS_SHORTCUT_INCLUDE_START_MODIFIER 2674
#define IDS_SHORTCUT_TOO_MANY_MODIFIERS 2675
#define IDS_SHORTCUT_NEED_CHARACTER 2676
#define IDS_SHORTCUT_CLEAR 2677
#define IDS_SETTINGS_CONTINUE 6916
#define IDS_SETTINGS_GOT_IT 6917
#define IDS_SETTINGS_MORE_ACTIONS 6918
#define IDS_SETTINGS_MORE_ACTIONS_BUTTON_ARIA_LABEL 6919
#define IDS_SETTINGS_OPENS_IN_NEW_TAB 6920
#define IDS_SETTINGS_SEND_FEEDBACK_ROLE_DESCRIPTION 6921
#define IDS_SETTINGS_COLUMN_HEADING_WHEN_ON 6922
#define IDS_SETTINGS_COLUMN_HEADING_CONSIDER 6923
#define IDS_SETTINGS_COLUMN_HEADING_WHEN_USED 6924
#define IDS_RELAUNCH_CONFIRMATION_DIALOG_BODY 6925
#define IDS_SETTINGS_ABOUT_PAGE_BROWSER_VERSION 6926
#define IDS_SETTINGS_ABOUT_PAGE_LEARN_MORE_UPDATE_ERRORS 6927
#define IDS_SETTINGS_ABOUT_PAGE_LEARN_MORE_SYSTEM_REQUIREMENTS 6928
#define IDS_SETTINGS_ABOUT_PAGE_RELAUNCH 4091
#define IDS_SETTINGS_ACCESSIBILITY 6935
#define IDS_SETTINGS_ACCESSIBILITY_WEB_STORE 6936
#define IDS_SETTINGS_MORE_FEATURES_LINK 6937
#define IDS_SETTINGS_MORE_FEATURES_LINK_DESCRIPTION 6938
#define IDS_SETTINGS_ACCESSIBLE_IMAGE_LABELS_TITLE 6939
#define IDS_SETTINGS_ACCESSIBLE_IMAGE_LABELS_SUBTITLE 6940
#define IDS_SETTINGS_MAIN_NODE_ANNOTATIONS_DOWNLOAD_COMPLETE 6941
#define IDS_SETTINGS_MAIN_NODE_ANNOTATIONS_DOWNLOAD_ERROR 6942
#define IDS_SETTINGS_MAIN_NODE_ANNOTATIONS_DOWNLOAD_PROGRESS 6943
#define IDS_SETTINGS_MAIN_NODE_ANNOTATIONS_DOWNLOADING 6944
#define IDS_SETTINGS_MAIN_NODE_ANNOTATIONS_TITLE 6945
#define IDS_SETTINGS_MAIN_NODE_ANNOTATIONS_SUBTITLE 6946
#define IDS_SETTINGS_AX_TREE_FIXING_TITLE 6947
#define IDS_SETTINGS_AX_TREE_FIXING_SUBTITLE 6948
#define IDS_SETTINGS_CAPTIONS_ENABLE_LIVE_CAPTION_TITLE 6949
#define IDS_SETTINGS_CAPTIONS_ENABLE_LIVE_TRANSLATE_TITLE 6950
#define IDS_SETTINGS_CAPTIONS_ENABLE_LIVE_TRANSLATE_SUBTITLE 6951
#define IDS_SETTINGS_CAPTIONS_ENABLE_LIVE_CAPTION_SUBTITLE 6952
#define IDS_SETTINGS_CAPTIONS_LIVE_CAPTION_DOWNLOAD_PROGRESS 6953
#define IDS_SETTINGS_CAPTIONS_LIVE_CAPTION_DOWNLOAD_COMPLETE 6954
#define IDS_SETTINGS_CAPTIONS_LIVE_CAPTION_DOWNLOAD_ERROR 6955
#define IDS_SETTINGS_CAPTIONS_LIVE_CAPTION_DOWNLOAD_ERROR_REBOOT_REQUIRED 6956
#define IDS_SETTINGS_CAPTIONS_MASK_OFFENSIVE_WORDS_TITLE 6957
#define IDS_SETTINGS_ENABLE_CARET_BROWSING_TITLE 6958
#define IDS_SETTINGS_ENABLE_CARET_BROWSING_SUBTITLE 6959
#define IDS_SETTINGS_ACCESSIBILITY_FOCUS_HIGHLIGHT_DESCRIPTION 4802
#define IDS_SETTINGS_OVERSCROLL_HISTORY_NAVIGATION_TITLE 4735
#define IDS_SETTINGS_OVERSCROLL_HISTORY_NAVIGATION_SUBTITLE 6960
#define IDS_SETTINGS_ACCESSIBILITY_TOAST_FREQUENCY_TITLE 6961
#define IDS_SETTINGS_ACCESSIBILITY_TOAST_FREQUENCY_DESCRIPTION 6962
#define IDS_SETTINGS_APPEARANCE 6963
#define IDS_SETTINGS_CUSTOM_WEB_ADDRESS 6964
#define IDS_SETTINGS_ENTER_CUSTOM_WEB_ADDRESS 6965
#define IDS_SETTINGS_HOME_BUTTON_DISABLED 6966
#define IDS_SETTINGS_THEMES 6967
#define IDS_SETTINGS_GTK_THEME 6968
#define IDS_SETTINGS_USE_GTK_THEME 6969
#define IDS_SETTINGS_QT_THEME 6970
#define IDS_SETTINGS_USE_QT_THEME 6971
#define IDS_SETTINGS_CLASSIC_THEME 6972
#define IDS_SETTINGS_USE_CLASSIC_THEME 6973
#define IDS_SETTINGS_RESET_TO_DEFAULT 6974
#define IDS_SETTINGS_CUSTOMIZE_TOOLBAR 6976
#define IDS_SETTINGS_RESET_TOOLBAR_TO_DEFAULT 6977
#define IDS_SETTINGS_CHROME_COLORS 6978
#define IDS_SETTINGS_COLOR_SCHEME_MODE 6979
#define IDS_SETTINGS_SHOW_HOME_BUTTON 6980
#define IDS_SETTINGS_CTRL_TAB_MRU 6981
#define IDS_SETTINGS_SHOW_BOOKMARKS_BAR 6982
#define IDS_SETTINGS_BOOKMARKS_BAR 6983
#define IDS_SETTINGS_BOOKMARKS_BAR_ALWAYS_SHOW 6984
#define IDS_SETTINGS_BOOKMARKS_BAR_ONLY_SHOW_ON_NTP 6985
#define IDS_SETTINGS_BOOKMARKS_BAR_ALWAYS_HIDE 6986
#define IDS_SETTINGS_SHOW_TAB_SEARCH_BUTTON 6987
#define IDS_SETTINGS_SHOW_PROJECTS_PANEL_BUTTON 6988
#define IDS_SETTINGS_SHOW_EVERYTHING_MENU_BUTTON 6989
#define IDS_SETTINGS_TAB_STRIP_POSITION 6990
#define IDS_SETTINGS_VERTICAL_TABS_EXPAND_ON_HOVER 6991
#define IDS_SETTINGS_ALLOW_SPLIT_VIEW_DRAG_AND_DROP 6992
#define IDS_SETTINGS_SHOW_TAB_GROUPS_IN_BOOKMARKS_BAR 6993
#define IDS_SETTINGS_AUTO_PIN_NEW_TAB_GROUPS 6994
#define IDS_SETTINGS_HOVER_CARD_TITLE 6995
#define IDS_SETTINGS_SHOW_HOVER_CARD_IMAGES 6996
#define IDS_SETTINGS_SHOW_HOVER_CARD_MEMORY_USAGE 6997
#define IDS_SETTINGS_SHOW_HOVER_CARD_MEMORY_USAGE_STANDALONE 6998
#define IDS_SETTINGS_SIDE_PANEL_POSITION 6999
#define IDS_SETTINGS_SIDE_PANEL_ALIGNMENT_GLIC 7000
#define IDS_SETTINGS_SIDE_PANEL_ALIGNMENT_CONTEXTUAL_TASKS 7001
#define IDS_SETTINGS_SIDE_PANEL_ALIGNMENT_CHROME_PANELS 7002
#define IDS_SETTINGS_TAB_SEARCH_POSITION 7003
#define IDS_SETTINGS_UI_FEATURE_ALIGN_LEFT 7004
#define IDS_SETTINGS_UI_FEATURE_ALIGN_RIGHT 7005
#define IDS_SETTINGS_UI_FEATURE_ALIGN_SIDE 7006
#define IDS_SETTINGS_UI_FEATURE_ALIGN_TOP 7007
#define IDS_SETTINGS_HOME_PAGE_NTP 7008
#define IDS_SETTINGS_CHANGE_HOME_PAGE 7009
#define IDS_SETTINGS_ADVANCED 7012
#define IDS_SETTINGS_BASIC 7013
#define IDS_SETTINGS_MENU_BUTTON_LABEL 7014
#define IDS_SETTINGS_MENU_EXTENSIONS_LINK_TOOLTIP 7015
#define IDS_SETTINGS_SEARCH_PROMPT 7016
#define IDS_SETTINGS_SEARCH_NO_RESULTS_HELP 7017
#define IDS_SETTINGS_SETTINGS 7018
#define IDS_SETTINGS_ALT_PAGE_TITLE 7019
#define IDS_SETTINGS_SUBPAGE_BUTTON 7020
#define IDS_SETTINGS_RESTART 7021
#define IDS_SETTINGS_CONTROLLED_BY_EXTENSION 7022
#define IDS_SETTINGS_CONTROLLED_BY_EXTENSION_TITLE 7023
#define IDS_SETTINGS_CONTROLLED_BY_EXTENSION_WITH_DISABLE_AND_MANAGE_OPTION 7024
#define IDS_SETTINGS_CONTROLLED_BY_EXTENSION_WITH_MANAGE_OPTION 7025
#define IDS_SETTINGS_CLEAR 7026
#define IDS_SETTINGS_CUSTOM 7027
#define IDS_SETTINGS_DELETE 7028
#define IDS_SETTINGS_EDIT 7029
#define IDS_SETTINGS_END_TIME 7030
#define IDS_SETTINGS_NOT_VALID 7031
#define IDS_SETTINGS_NOT_VALID_WEB_ADDRESS 7032
#define IDS_SETTINGS_NOT_VALID_WEB_ADDRESS_FOR_CONTENT_TYPE 7033
#define IDS_SETTINGS_INPUT_MAX_LENGTH_DESCRIPTION 7034
#define IDS_SETTINGS_RETRY 7035
#define IDS_SETTINGS_SLIDER_MIN_MAX_ARIA_ROLE_DESCRIPTION 7036
#define IDS_SETTINGS_START_TIME 7037
#define IDS_PASSWORDS_PAGE_AUTHENTICATION_PROMPT_BIOMETRIC_SUFFIX 7038
#define IDS_PASSWORDS_PAGE_COPY_AUTHENTICATION_PROMPT_BIOMETRIC_SUFFIX 7039
#define IDS_PASSWORDS_PAGE_EDIT_AUTHENTICATION_PROMPT_BIOMETRIC_SUFFIX 7040
#define IDS_PASSWORDS_PAGE_EXPORT_AUTHENTICATION_PROMPT_BIOMETRIC_SUFFIX 7041
#define IDS_PASSWORDS_PAGE_DELETE_ALL_DATA_AUTHENTICATION_PROMPT_BIOMETRIC_SUFFIX 7042
#define IDS_SETTINGS_AUTOFILL_AND_PASSWORDS 7043
#define IDS_IDENTITY_DOCS_SUBMENU_OPTION 7044
#define IDS_TRAVEL_SUBMENU_OPTION 7045
#define IDS_SETTINGS_YOUR_SAVED_INFO_DESCRIPTION 7046
#define IDS_SETTINGS_YOUR_SAVED_INFO_AUTOFILL_SETTINGS_LABEL 7047
#define IDS_SETTINGS_YOUR_SAVED_INFO_AUTOFILL_SETTINGS_DESCRIPTION 7048
#define IDS_SETTINGS_RELATED_SERVICES_TITLE 7049
#define IDS_PASSWORDS_PAGE_IMPORT_AUTHENTICATION_PROMPT_BIOMETRIC_SUFFIX 7050
#define IDS_SETTINGS_GOOGLE_PAYMENTS 7051
#define IDS_SETTINGS_AUTOFILL_ADDRESSES_ADD_TITLE 7052
#define IDS_SETTINGS_AUTOFILL_ADDRESSES_EDIT_TITLE 7053
#define IDS_SETTINGS_AUTOFILL_ADDRESSES_COUNTRY 7054
#define IDS_SETTINGS_AUTOFILL_ADDRESSES_PHONE 7055
#define IDS_SETTINGS_AUTOFILL_ADDRESSES_EMAIL 7056
#define IDS_SETTINGS_AUTOFILL_DETAIL 7057
#define IDS_SETTINGS_AUTOFILL_DROPDOWN_NO_OPTION_SELECTED 7058
#define IDS_SETTINGS_AUTOFILL_CARD_DESCRIPTION 7059
#define IDS_SETTINGS_AUTOFILL_CARD_A11Y_LABELED 7060
#define IDS_SETTINGS_AUTOFILL_CARD_EXP_DATE_A11Y_LABELED 7061
#define IDS_SETTINGS_AUTOFILL_MORE_ACTIONS_FOR_ADDRESS 7062
#define IDS_SETTINGS_AUTOFILL_MORE_ACTIONS_FOR_CREDIT_CARD 7063
#define IDS_SETTINGS_AUTOFILL_MORE_ACTIONS_FOR_CREDIT_CARD_WITH_CVC 7064
#define IDS_SETTINGS_AUTOFILL_MORE_ACTIONS_FOR_HOME_ADDRESS 7065
#define IDS_SETTINGS_AUTOFILL_MORE_ACTIONS_FOR_WORK_ADDRESS 7066
#define IDS_AUTOFILL_ADD_VIRTUAL_CARD 7067
#define IDS_AUTOFILL_REMOVE_VIRTUAL_CARD 7068
#define IDS_AUTOFILL_EDIT_SERVER_CREDIT_CARD 7069
#define IDS_AUTOFILL_EDIT_SERVER_CREDIT_CARD_IN_WALLET 7070
#define IDS_AUTOFILL_VIRTUAL_CARD_TURNED_ON_LABEL 7071
#define IDS_AUTOFILL_VIRTUAL_CARD_UNENROLL_DIALOG_TITLE 7072
#define IDS_AUTOFILL_VIRTUAL_CARD_UNENROLL_DIALOG_LABEL 7073
#define IDS_AUTOFILL_VIRTUAL_CARD_UNENROLL_DIALOG_CONFIRM_BUTTON_LABEL 7074
#define IDS_SETTINGS_ADDRESS_REMOVE 7075
#define IDS_SETTINGS_REMOVE_VERIFIED_EMAIL_PERMISSION 7076
#define IDS_SETTINGS_REMOVE_VERIFIED_EMAIL_PERMISSION_TITLE 7077
#define IDS_SETTINGS_REMOVE_VERIFIED_EMAIL_PERMISSION_BODY 7078
#define IDS_SETTINGS_ADDRESS_REMOVE_CONFIRMATION_TITLE 7079
#define IDS_SETTINGS_ADDRESS_REMOVED_MESSAGE 7080
#define IDS_SETTINGS_EDIT_CREDIT_CARD_TITLE 7081
#define IDS_SETTINGS_LOCAL_CARD_REMOVE_CONFIRMATION_TITLE 7082
#define IDS_SETTINGS_LOCAL_PAYMENT_METHOD_REMOVE_CONFIRMATION_DESCRIPTION 7083
#define IDS_SETTINGS_GOOGLE_WALLET 7084
#define IDS_SETTINGS_PAYMENTS_MANAGE_LOYALTY_CARDS_AND_PAYMENT_METHODS 7085
#define IDS_SETTINGS_REMOTE_WALLET_PASSES_LINK_LABEL 7086
#define IDS_SETTINGS_PAYMENTS_SAVED_TO_THIS_DEVICE_ONLY 7087
#define IDS_SETTINGS_CREDIT_AND_DEBIT_CARD_TITLE 7088
#define IDS_SETTINGS_ADD_CREDIT_CARD_TITLE 7089
#define IDS_SETTINGS_REMOTE_PAYMENT_METHODS_LINK_LABEL 7090
#define IDS_SETTINGS_REMOTE_PAYMENT_METHODS_WALLET_LINK_LABEL 7091
#define IDS_SETTINGS_NAME_ON_CREDIT_CARD 7092
#define IDS_SETTINGS_CREDIT_CARD_NUMBER 7093
#define IDS_SETTINGS_CREDIT_CARD_EXPIRATION_DATE 7094
#define IDS_SETTINGS_CREDIT_CARD_EXPIRATION_MONTH 7095
#define IDS_SETTINGS_CREDIT_CARD_EXPIRATION_YEAR 7096
#define IDS_SETTINGS_CREDIT_CARD_EXPIRED 7097
#define IDS_SETTINGS_CREDIT_CARD_NICKNAME 7098
#define IDS_SETTINGS_CREDIT_CARD_NICKNAME_INVALID 7099
#define IDS_SETTINGS_CREDIT_CARD_CVC_TITLE 7100
#define IDS_SETTINGS_CREDIT_CARD_CVC_IMAGE_TITLE 7101
#define IDS_SETTINGS_CREDIT_CARD_CVC_IMAGE_TITLE_AMEX 7102
#define IDS_SETTINGS_CREDIT_CARD_CVC_PLACEHOLDER 7103
#define IDS_SETTINGS_ADD_PAYMENT_METHODS 7104
#define IDS_SETTINGS_ADD_PAYMENT_METHOD_CREDIT_OR_DEBIT_CARD 7105
#define IDS_SETTINGS_ADD_PAYMENT_METHOD_IBAN 7106
#define IDS_SETTINGS_IBAN_SAVED_TO_THIS_DEVICE_ONLY 7107
#define IDS_SETTINGS_IBAN_INVALID_VALIDATION_MESSAGE 7108
#define IDS_IBAN_NICKNAME 7109
#define IDS_SETTINGS_AUTOFILL_MORE_ACTIONS_FOR_IBAN 7110
#define IDS_SETTINGS_AUTOFILL_A11Y_IBAN_DESCRIPTION 7111
#define IDS_SETTINGS_ADD_IBAN_TITLE 7112
#define IDS_SETTINGS_EDIT_IBAN_TITLE 7113
#define IDS_SETTINGS_IBAN_EDIT 7114
#define IDS_SETTINGS_LOCAL_IBAN_REMOVE_CONFIRMATION_TITLE 7115
#define IDS_SETTINGS_PASSWORD_MANAGER 7116
#define IDS_SETTINGS_PASSWORD_MANAGER_DESCRIPTION 7117
#define IDS_SETTINGS_PAYMENT_METHODS_NONE 7118
#define IDS_AUTOFILL_PASSKEYS_TITLE 7119
#define IDS_SETTINGS_PASSKEYS_LENGTH_ERROR 7120
#define IDS_SETTINGS_PASSKEYS_SITE_LABEL 7121
#define IDS_SETTINGS_PASSKEYS_DIALOG_TITLE 7122
#define IDS_SETTINGS_PASSKEYS_EDIT_DIALOG_FOOTNOTE 7123
#define IDS_SETTINGS_ADDRESS_NONE 7124
#define IDS_SETTINGS_PASSWORDS_LEAK_DETECTION_LABEL 7125
#define IDS_SETTINGS_PASSWORDS_LEAK_DETECTION_SIGNED_OUT_ENABLED_DESC 7126
#define IDS_SETTINGS_AUTOFILL_AI_WHEN_ON_SAVED_INFO 7127
#define IDS_SETTINGS_AUTOFILL_AI_ENTITY_INSTANCES_HEADER 7128
#define IDS_SETTINGS_AUTOFILL_AI_ENTITY_INSTANCES_NONE 7129
#define IDS_SETTINGS_AUTOFILL_AI_MORE_ACTIONS_FOR_ENTITY_INSTANCE 7130
#define IDS_SETTINGS_AUTOFILL_AI_DELETE_ENTITY_INSTANCE_DIALOG_TEXT 7131
#define IDS_SETTINGS_AUTOFILL_AI_ACCESSIBILITY_LABEL_MONTH_DROPDOWN 7132
#define IDS_SETTINGS_AUTOFILL_AI_ACCESSIBILITY_LABEL_DAY_DROPDOWN 7133
#define IDS_SETTINGS_AUTOFILL_AI_ACCESSIBILITY_LABEL_YEAR_DROPDOWN 7134
#define IDS_SETTINGS_AUTOFILL_AI_MONTH_DROPDOWN_NO_OPTION_SELECTED 7135
#define IDS_SETTINGS_AUTOFILL_AI_DAY_DROPDOWN_NO_OPTION_SELECTED 7136
#define IDS_SETTINGS_AUTOFILL_AI_YEAR_DROPDOWN_NO_OPTION_SELECTED 7137
#define IDS_SETTINGS_AUTOFILL_AI_ADD_OR_EDIT_DIALOG_DATE_VALIDATION_ERROR 7138
#define IDS_SETTINGS_AUTOFILL_AI_ADD_OR_EDIT_DIALOG_VALIDATION_ERROR 7139
#define IDS_AUTOFILL_AI_ADD_OR_EDIT_DIALOG_REQUIRED_FIELD_ERROR 7140
#define IDS_SETTINGS_WALLETABLE_PASS_DETECTION_TOGGLE_LABEL 7141
#define IDS_SETTINGS_WALLETABLE_PASS_DETECTION_TOGGLE_SUB_LABEL 7142
#define IDS_SETTINGS_WALLETABLE_PASS_DETECTION_WHEN_ON_SAVED_INFO 7143
#define IDS_SETTINGS_WALLETABLE_PASS_DETECTION_WHEN_ON_NOTIFICATIONS 7144
#define IDS_SETTINGS_WALLETABLE_PASS_DETECTION_TO_CONSIDER_DATA_USAGE 7145
#define IDS_SETTINGS_WALLETABLE_PASS_DETECTION_TO_CONSIDER_DATA_STORAGE 7146
#define IDS_SETTINGS_HOME_AND_WORK_ADDRESS_EDIT 7158
#define IDS_SETTINGS_HOME_AND_WORK_ADDRESS_REMOVE 7159
#define IDS_SETTINGS_HOME_ADDRESS_REMOVE_CONFIRMATION_DIALOG_TITLE 7160
#define IDS_SETTINGS_WORK_ADDRESS_REMOVE_CONFIRMATION_DIALOG_TITLE 7161
#define IDS_SETTINGS_NAME_EMAIL_ADDRESS_REMOVE_CONFIRMATION_DIALOG_TITLE 7162
#define IDS_SETTINGS_HOME_ADDRESS_REMOVE_CONFIRMATION_DIALOG_NOTICE 7163
#define IDS_SETTINGS_WORK_ADDRESS_REMOVE_CONFIRMATION_DIALOG_NOTICE 7164
#define IDS_SETTINGS_NAME_EMAIL_ADDRESS_REMOVE_CONFIRMATION_DIALOG_NOTICE 7165
#define IDS_SETTINGS_HOME_AND_WORK_ADDRESS_REMOVE_CONFIRMATION_DIALOG_ACTION_OK 7166
#define IDS_SETTINGS_HOME_AND_WORK_ADDRESS_REMOVED_MESSAGE 7167
#define IDS_SETTINGS_NAME_EMAIL_ADDRESS_REMOVED_MESSAGE 7168
#define IDS_SETTINGS_DEFAULT_BROWSER 7169
#define IDS_SETTINGS_DEFAULT_BROWSER_MAKE_DEFAULT_BUTTON 7170
#define IDS_SETTINGS_CLEAR_PERIOD_TITLE 7171
#define IDS_SETTINGS_CLEAR_BROWSING_DATA_WITH_SYNC 7172
#define IDS_SETTINGS_CLEAR_BROWSING_DATA_WITH_SYNC_ERROR 7173
#define IDS_SETTINGS_CLEAR_BROWSING_DATA_WITH_SYNC_PASSPHRASE_ERROR 7174
#define IDS_SETTINGS_CLEAR_BROWSING_DATA_WITH_SYNC_PAUSED 7175
#define IDS_SETTINGS_CLEAR_BROWSING_HISTORY 7176
#define IDS_SETTINGS_CLEAR_COOKIES_AND_SITE_DATA_SUMMARY_BASIC 7177
#define IDS_SETTINGS_CLEAR_COOKIES_AND_SITE_DATA_SUMMARY_BASIC_WITH_EXCEPTION 7178
#define IDS_SETTINGS_CLEAR_COOKIES_AND_SITE_DATA_SUMMARY_BASIC_SUPERVISED_PROFILE 7179
#define IDS_SETTINGS_CLEAR_COOKIES_AND_SITE_DATA_SUMMARY_BASIC_SIGNED_IN_PROFILE 7180
#define IDS_SETTINGS_CLEAR_BROWSING_HISTORY_SUMMARY 7181
#define IDS_SETTINGS_CLEAR_BROWSING_HISTORY_SUMMARY_SIGNED_IN_NO_LINK 7182
#define IDS_SETTINGS_CLEAR_GOOGLE_SEARCH_HISTORY_GOOGLE_DSE 7183
#define IDS_SETTINGS_CLEAR_GOOGLE_SEARCH_HISTORY_NON_GOOGLE_DSE 7184
#define IDS_SETTINGS_CLEAR_DOWNLOAD_HISTORY 7185
#define IDS_SETTINGS_CLEAR_CACHE 7186
#define IDS_SETTINGS_CLEAR_COOKIES 7187
#define IDS_SETTINGS_CLEAR_PASSWORDS 7188
#define IDS_SETTINGS_CLEAR_FORM_DATA 7189
#define IDS_SETTINGS_CLEAR_HOSTED_APP_DATA 7190
#define IDS_SETTINGS_CLEAR_PERIOD_15_MIN 7191
#define IDS_SETTINGS_CLEAR_PERIOD_15_MINUTES 7192
#define IDS_SETTINGS_CLEAR_PERIOD_HOUR 7193
#define IDS_SETTINGS_CLEAR_PERIOD_24_HOURS 7194
#define IDS_SETTINGS_CLEAR_PERIOD_7_DAYS 7195
#define IDS_SETTINGS_CLEAR_PERIOD_FOUR_WEEKS 7196
#define IDS_SETTINGS_CLEAR_PERIOD_EVERYTHING 7197
#define IDS_SETTINGS_NOTIFICATION_WARNING 7198
#define IDS_SETTINGS_CLEAR_BROWSING_DATA_SHOW_MORE 7199
#define IDS_SETTINGS_CLEAR_BROWSING_DATA_MORE 7200
#define IDS_SETTINGS_MANAGE_IN_YOUR_GEMINI_APPS_ACTIVITY 7201
#define IDS_SETTINGS_GEMINI_APPS_ACTIVITY 7202
#define IDS_SETTINGS_DELETE_DATA_FROM_DEVICE 7203
#define IDS_SETTINGS_DELETION_CONFIRMATION_TOAST_LABEL 7204
#define IDS_SETTINGS_DELETION_CONFIRMATION_ALL_TIME_TOAST_LABEL 7205
#define IDS_SETTINGS_SEARCH_GEMINI_PASSWORDS_OTHER_DATA_SUB_LABEL 7206
#define IDS_SETTINGS_DOWNLOADS 7207
#define IDS_SETTINGS_DOWNLOAD_LOCATION 7208
#define IDS_SETTINGS_CHANGE_DOWNLOAD_LOCATION 7209
#define IDS_SETTINGS_PROMPT_FOR_DOWNLOAD 7210
#define IDS_SETTINGS_OPEN_FILE_TYPES_AUTOMATICALLY 7211
#define IDS_SETTINGS_DOWNLOADS_SHOW_WHEN_FINISHED 7212
#define IDS_SETTINGS_ON_STARTUP 7213
#define IDS_SETTINGS_ON_STARTUP_OPEN_NEW_TAB 7214
#define IDS_SETTINGS_ON_STARTUP_CONTINUE 7215
#define IDS_SETTINGS_ON_STARTUP_OPEN_SPECIFIC 7216
#define IDS_SETTINGS_ON_STARTUP_CONTINUE_AND_OPEN_SPECIFIC 7217
#define IDS_SETTINGS_ON_STARTUP_USE_CURRENT 7218
#define IDS_SETTINGS_ON_STARTUP_ADD_NEW_PAGE 7219
#define IDS_SETTINGS_ON_STARTUP_EDIT_PAGE 7220
#define IDS_SETTINGS_ON_STARTUP_SITE_URL 7221
#define IDS_SETTINGS_ON_STARTUP_REMOVE 7222
#define IDS_SETTINGS_ON_STARTUP_PAGE_TOOLTIP 7223
#define IDS_SETTINGS_INVALID_URL 7224
#define IDS_SETTINGS_URL_TOOL_LONG 7225
#define IDS_SETTINGS_PERFORMANCE_PAGE_TITLE 7226
#define IDS_SETTINGS_PERFORMANCE_GENERAL_PAGE_TITLE 7227
#define IDS_SETTINGS_PERFORMANCE_MEMORY_PAGE_TITLE 7228
#define IDS_SETTINGS_PERFORMANCE_SPEED_PAGE_TITLE 7229
#define IDS_SETTINGS_PERFORMANCE_MEMORY_SAVER_MODE_SETTING 7230
#define IDS_SETTINGS_PERFORMANCE_MEMORY_SAVER_MODE_HEURISTICS_LABEL 7231
#define IDS_SETTINGS_PERFORMANCE_MEMORY_SAVER_MODE_RECOMMENDED_BADGE 7232
#define IDS_SETTINGS_PERFORMANCE_MEMORY_SAVER_MODE_ON_TIMER_LABEL 7233
#define IDS_SETTINGS_PERFORMANCE_MEMORY_SAVER_MODE_RADIO_GROUP_ARIA_LABEL 7234
#define IDS_SETTINGS_PERFORMANCE_MEMORY_SAVER_MODE_CHOOSE_DISCARD_TIME_ARIA_LABEL 7235
#define IDS_SETTINGS_PERFORMANCE_MEMORY_SAVER_MODE_CONSERVATIVE_LABEL 7236
#define IDS_SETTINGS_PERFORMANCE_MEMORY_SAVER_MODE_MEDIUM_LABEL 7237
#define IDS_SETTINGS_PERFORMANCE_MEMORY_SAVER_MODE_AGGRESSIVE_LABEL 7238
#define IDS_SETTINGS_PERFORMANCE_MEMORY_SAVER_MODE_CONSERVATIVE_DESCRIPTION 7239
#define IDS_SETTINGS_PERFORMANCE_MEMORY_SAVER_MODE_MEDIUM_DESCRIPTION 7240
#define IDS_SETTINGS_PERFORMANCE_MEMORY_SAVER_MODE_AGGRESSIVE_DESCRIPTION 7241
#define IDS_SETTINGS_PERFORMANCE_TAB_DISCARDING_EXCEPTIONS_ADD_BUTTON_ARIA_LABEL 7242
#define IDS_SETTINGS_PERFORMANCE_TAB_DISCARDING_EXCEPTIONS_SAVE_BUTTON_ARIA_LABEL 7243
#define IDS_SETTINGS_PERFORMANCE_TAB_DISCARDING_EXCEPTIONS_HEADER 7244
#define IDS_SETTINGS_PERFORMANCE_TAB_DISCARDING_EXCEPTIONS_DESCRIPTION 7245
#define IDS_SETTINGS_PERFORMANCE_TAB_DISCARDING_EXCEPTIONS_ADDITIONAL_SITES 7246
#define IDS_SETTINGS_PERFORMANCE_TAB_DISCARDING_EXCEPTIONS_ADD_DIALOG_CURRENT_TABS 7247
#define IDS_SETTINGS_PERFORMANCE_TAB_DISCARDING_EXCEPTIONS_ADD_DIALOG_CURRENT_TABS_EMPTY 7248
#define IDS_SETTINGS_PERFORMANCE_TAB_DISCARDING_EXCEPTIONS_ADD_DIALOG_MANUAL 7249
#define IDS_SETTINGS_PERFORMANCE_TAB_DISCARDING_EXCEPTIONS_ACTIVE_SITE_ARIA_DESCRIPTION 7250
#define IDS_SETTINGS_PERFORMANCE_TAB_DISCARDING_EXCEPTIONS_ADD_DIALOG_HELP 7251
#define IDS_SETTINGS_PERFORMANCE_BATTERY_PAGE_TITLE 7252
#define IDS_SETTINGS_PERFORMANCE_BATTERY_SAVER_MODE_SETTING 7253
#define IDS_SETTINGS_PERFORMANCE_BATTERY_SAVER_MODE_ON_BATTERY_LABEL 7254
#define IDS_SETTINGS_PERFORMANCE_BATTERY_SAVER_MODE_BELOW_THRESHOLD_LABEL 7255
#define IDS_SETTINGS_PERFORMANCE_BATTERY_SAVER_MODE_RADIO_GROUP_ARIA_LABEL 7256
#define IDS_SETTINGS_PERFORMANCE_BATTERY_SAVER_MODE_LINK_OS_SETTING_DESCRIPTION 7257
#define IDS_SETTINGS_PERFORMANCE_DISCARD_RING_TREATMENT_ENABLED_LABEL 7258
#define IDS_SETTINGS_PERFORMANCE_DISCARD_RING_TREATMENT_ENABLED_DESCRIPTION_WITH_LEARN_LINK 7259
#define IDS_SETTINGS_PERFORMANCE_TAB_HOVER_PREVIEW_CARD_LINK_TITLE 7260
#define IDS_SETTINGS_PERFORMANCE_TAB_HOVER_PREVIEW_CARD_LINK_SUBTITLE 7261
#define IDS_SETTINGS_PERFORMANCE_INTERVENTION_NOTIFICATION_ENABLED_LABEL 7262
#define IDS_SETTINGS_PERFORMANCE_INTERVENTION_NOTIFICATION_ENABLED_DESCRIPTION 7263
#define IDS_SETTINGS_LANGUAGES_PAGE_TITLE 7264
#define IDS_SETTINGS_LANGUAGES_CARD_TITLE 7265
#define IDS_SETTINGS_LANGUAGE_SEARCH 7266
#define IDS_SETTINGS_LANGUAGES_LANGUAGES_LIST_MOVE_TO_TOP 7267
#define IDS_SETTINGS_LANGUAGES_LANGUAGES_LIST_MOVE_UP 7268
#define IDS_SETTINGS_LANGUAGES_LANGUAGES_LIST_MOVE_DOWN 7269
#define IDS_SETTINGS_LANGUAGES_LANGUAGES_LIST_REMOVE 7270
#define IDS_SETTINGS_LANGUAGES_LANGUAGES_ADD 7271
#define IDS_SETTINGS_LANGUAGES_MANAGE_LANGUAGES_TITLE 7272
#define IDS_SETTINGS_LANGUAGES_NO_LANGUAGES_ADDED 7273
#define IDS_SETTINGS_LANGUAGES_EXPAND_ACCESSIBILITY_LABEL 7274
#define IDS_SETTINGS_LANGUAGES_PREFERRED_LANGUAGES_HEADER 7275
#define IDS_SETTINGS_LANGUAGES_PREFERRED_LANGUAGES_DESC 7276
#define IDS_SETTINGS_LANGUAGES_OFFER_TO_ENABLE_TRANSLATE 7277
#define IDS_SETTINGS_LANGUAGES_OFFER_TO_ENABLE_TRANSLATE_SUBLABEL 7278
#define IDS_SETTINGS_LANGUAGES_TRANSLATE_TARGET 7279
#define IDS_SETTINGS_LANGUAGES_MANAGED_DIALOG_TITLE 7280
#define IDS_SETTINGS_LANGUAGES_MANAGED_DIALOG_BODY 7281
#define IDS_SETTINGS_LANGUAGES_ADD_ARIA_LABEL 7282
#define IDS_SETTINGS_LANGUAGES_REMOVE_AUTOMATIC_ARIA_LABEL 7283
#define IDS_SETTINGS_LANGUAGES_REMOVE_NEVER_ARIA_LABEL 7284
#define IDS_SETTINGS_TRANSLATE_PAGE_TITLE 7285
#define IDS_SETTINGS_TARGET_TRANSLATE_LABEL 7286
#define IDS_SETTINGS_LANGUAGES_AUTOMATIC_TRANSLATE 7287
#define IDS_SETTINGS_LANGUAGES_AUTOMATIC_TRANSLATE_ADD_ARIA_LABEL 7288
#define IDS_SETTINGS_LANGUAGES_NEVER_LANGUAGES 7289
#define IDS_SETTINGS_LANGUAGES_NEVER_TRANSLATE_ADD_ARIA_LABEL 7290
#define IDS_SETTINGS_LANGUAGES_SPELL_CHECK_TITLE 7291
#define IDS_SETTINGS_LANGUAGES_SPELL_CHECK_BASIC_LABEL 7292
#define IDS_SETTINGS_LANGUAGES_SPELL_CHECK_ENHANCED_LABEL 7293
#define IDS_SETTINGS_LANGUAGES_SPELL_CHECK_ENHANCED_DESCRIPTION 7294
#define IDS_SETTINGS_LANGUAGES_OFFER_TO_ENABLE_SPELL_CHECK 7295
#define IDS_SETTING_LANGUAGES_SPELL_CHECK_DISABLED_REASON 7296
#define IDS_SETTINGS_LANGUAGES_SPELL_CHECK_LANGUAGES_LIST_TITLE 7297
#define IDS_SETTINGS_LANGUAGES_SPELL_CHECK_MANAGE 7298
#define IDS_SETTINGS_LANGUAGES_EDIT_DICTIONARY_TITLE 7299
#define IDS_SETTINGS_LANGUAGES_ADD_DICTIONARY_WORD 7300
#define IDS_SETTINGS_LANGUAGES_ADD_DICTIONARY_WORD_BUTTON 7301
#define IDS_SETTINGS_LANGUAGES_ADD_DICTIONARY_WORD_DUPLICATE_ERROR 7302
#define IDS_SETTINGS_LANGUAGES_ADD_DICTIONARY_WORD_LENGTH_ERROR 7303
#define IDS_SETTINGS_LANGUAGES_DELETE_DICTIONARY_WORD_BUTTON 7304
#define IDS_SETTINGS_LANGUAGES_DICTIONARY_WORDS 7305
#define IDS_SETTINGS_LANGUAGES_DICTIONARY_WORDS_NONE 7306
#define IDS_SETTINGS_LANGUAGES_DICTIONARY_DOWNLOAD_FAILED 7307
#define IDS_SETTINGS_LANGUAGES_DICTIONARY_DOWNLOAD_FAILED_HELP 7308
#define IDS_SETTINGS_PRELOAD_PAGES_TITLE 7311
#define IDS_SETTINGS_PRELOAD_PAGES_STANDARD_PRELOADING_TITLE 7312
#define IDS_SETTINGS_PRELOAD_PAGES_STANDARD_PRELOADING_EXPAND_A11Y_LABEL 7313
#define IDS_SETTINGS_PRELOAD_PAGES_STANDARD_PRELOADING_SUMMARY 7314
#define IDS_SETTINGS_PRELOAD_PAGES_STANDARD_PRELOADING_WHEN_ON_BULLET_ONE 7315
#define IDS_SETTINGS_PRELOAD_PAGES_EXTENDED_PRELOADING_TITLE 7316
#define IDS_SETTINGS_PRELOAD_PAGES_EXTENDED_PRELOADING_SUMMARY 7317
#define IDS_SETTINGS_PRELOAD_PAGES_EXTENDED_PRELOADING_EXPAND_A11Y_LABEL 7318
#define IDS_SETTINGS_PRELOAD_PAGES_EXTENDED_PRELOADING_WHEN_ON_BULLET_ONE 7319
#define IDS_SETTINGS_PRIVACY 7320
#define IDS_SETTINGS_PRIVACY_MORE 7321
#define IDS_SETTINGS_AI_COMPOSE_SUBLABEL_V2 7323
#define IDS_SETTINGS_OFFER_WRITING_HELP_TOGGLE_LABEL 7324
#define IDS_SETTINGS_OFFER_WRITING_HELP_TOGGLE_SUB_LABEL 7325
#define IDS_SETTINGS_OFFER_WRITING_HELP_ENABLED_SUB_LABEL 7326
#define IDS_SETTINGS_OFFER_WRITING_HELP_DISABLED_SUB_LABEL 7327
#define IDS_SETTINGS_OFFER_WRITING_HELP_DISABLED_SITES_LABEL_V2 7328
#define IDS_SETTINGS_OFFER_WRITING_HELP_NO_DISABLED_SITES 7329
#define IDS_SETTINGS_OFFER_WRITING_HELP_REMOVE_SITE_ARIA_LABEL 7330
#define IDS_SETTINGS_COMPOSE_SETTING_LINK_A11Y 7331
#define IDS_SETTINGS_COMPOSE_WHEN_ON_WRITING_HELP 7332
#define IDS_SETTINGS_COMPOSE_WHEN_ON_WRITING_EXAMPLES 7333
#define IDS_SETTINGS_COMPOSE_WHEN_ON_WRITING_REFERENCES 7334
#define IDS_SETTINGS_COMPOSE_CONSIDER_DATA 7335
#define IDS_SETTINGS_ANTI_ABUSE_ENABLED_SUB_LABEL 7336
#define IDS_SETTINGS_ANTI_ABUSE_DISABLED_SUB_LABEL 7337
#define IDS_SETTINGS_ANTI_ABUSE_DESCRIPTION 7338
#define IDS_SETTINGS_ANTI_ABUSE_WHEN_ON_SECTION_ONE 7339
#define IDS_SETTINGS_ANTI_ABUSE_WHEN_ON_SECTION_TWO 7340
#define IDS_SETTINGS_ANTI_ABUSE_WHEN_ON_SECTION_THREE 7341
#define IDS_SETTINGS_ANTI_ABUSE_THINGS_TO_CONSIDER_SECTION_ONE 7342
#define IDS_SETTINGS_PRIVACY_SANDBOX_COOKIES_DIALOG 7343
#define IDS_SETTINGS_PRIVACY_SANDBOX_COOKIES_DIALOG_MORE 7344
#define IDS_SETTINGS_AD_PRIVACY_LINK_ROW_LABEL 7345
#define IDS_SETTINGS_AD_PRIVACY_LINK_ROW_SUB_LABEL 7346
#define IDS_SETTINGS_AD_PRIVACY_RESTRICTED_LINK_ROW_SUB_LABEL 7347
#define IDS_SETTINGS_AD_PRIVACY_PAGE_TITLE 7348
#define IDS_SETTINGS_AD_PRIVACY_PAGE_TOPICS_LINK_ROW_LABEL 7349
#define IDS_SETTINGS_AD_PRIVACY_PAGE_TOPICS_LINK_ROW_SUB_LABEL_ENABLED 7350
#define IDS_SETTINGS_AD_PRIVACY_PAGE_TOPICS_LINK_ROW_SUB_LABEL_DISABLED 7351
#define IDS_SETTINGS_AD_PRIVACY_PAGE_FLEDGE_LINK_ROW_LABEL 7352
#define IDS_SETTINGS_AD_PRIVACY_PAGE_FLEDGE_LINK_ROW_SUB_LABEL_ENABLED 7353
#define IDS_SETTINGS_AD_PRIVACY_PAGE_FLEDGE_LINK_ROW_SUB_LABEL_DISABLED 7354
#define IDS_SETTINGS_AD_PRIVACY_PAGE_AD_MEASUREMENT_LINK_ROW_LABEL 7355
#define IDS_SETTINGS_AD_PRIVACY_PAGE_AD_MEASUREMENT_LINK_ROW_SUB_LABEL_ENABLED 7356
#define IDS_SETTINGS_AD_PRIVACY_PAGE_AD_MEASUREMENT_LINK_ROW_SUB_LABEL_DISABLED 7357
#define IDS_SETTINGS_TOPICS_PAGE_CURRENT_TOPICS_REGION_A11Y_DESCRIPTION 7358
#define IDS_SETTINGS_TOPICS_PAGE_BLOCKED_TOPICS_REGION_A11Y_DESCRIPTION 7359
#define IDS_SETTINGS_TOPICS_PAGE_FOOTER_V2 7360
#define IDS_SETTINGS_FLEDGE_PAGE_TITLE 7361
#define IDS_SETTINGS_FLEDGE_PAGE_TOGGLE_LABEL 7362
#define IDS_SETTINGS_FLEDGE_PAGE_TOGGLE_SUB_LABEL 7363
#define IDS_SETTINGS_FLEDGE_PAGE_CURRENT_SITES_HEADING 7364
#define IDS_SETTINGS_FLEDGE_PAGE_CURRENT_SITES_DESCRIPTION 7365
#define IDS_SETTINGS_FLEDGE_PAGE_CURRENT_SITES_DESCRIPTION_LEARN_MORE 7366
#define IDS_SETTINGS_FLEDGE_PAGE_CURRENT_SITES_DESCRIPTION_DISABLED 7367
#define IDS_SETTINGS_FLEDGE_PAGE_CURRENT_SITES_DESCRIPTION_EMPTY 7368
#define IDS_SETTINGS_FLEDGE_PAGE_CURRENT_SITES_REGION_A11Y_DESCRIPTION 7369
#define IDS_SETTINGS_FLEDGE_PAGE_SEE_ALL_SITES_LABEL 7370
#define IDS_SETTINGS_FLEDGE_PAGE_BLOCK_SITE 7371
#define IDS_SETTINGS_FLEDGE_PAGE_BLOCK_SITE_A11Y_LABEL 7372
#define IDS_SETTINGS_FLEDGE_PAGE_BLOCKED_SITES_HEADING 7373
#define IDS_SETTINGS_FLEDGE_PAGE_BLOCKED_SITES_DESCRIPTION 7374
#define IDS_SETTINGS_FLEDGE_PAGE_BLOCKED_SITES_DESCRIPTION_EMPTY 7375
#define IDS_SETTINGS_FLEDGE_PAGE_BLOCKED_SITES_REGION_A11Y_DESCRIPTION 7376
#define IDS_SETTINGS_FLEDGE_PAGE_ALLOW_SITE 7377
#define IDS_SETTINGS_FLEDGE_PAGE_ALLOW_SITE_A11Y_LABEL 7378
#define IDS_SETTINGS_FLEDGE_PAGE_LEARN_MORE_HEADING 7379
#define IDS_SETTINGS_FLEDGE_PAGE_LEARN_MORE_BULLET_1 7380
#define IDS_SETTINGS_FLEDGE_PAGE_LEARN_MORE_BULLET_2 7381
#define IDS_SETTINGS_FLEDGE_PAGE_LEARN_MORE_BULLET_3 7382
#define IDS_SETTINGS_FLEDGE_PAGE_CURRENT_SITES_DESCRIPTION_LEARN_MORE_A11Y_LABEL 7383
#define IDS_SETTINGS_AD_MEASUREMENT_PAGE_TITLE 7384
#define IDS_SETTINGS_AD_MEASUREMENT_PAGE_TOGGLE_LABEL 7385
#define IDS_SETTINGS_AD_MEASUREMENT_PAGE_TOGGLE_SUB_LABEL 7386
#define IDS_SETTINGS_AD_MEASUREMENT_PAGE_ENABLED_BULLET_1 7387
#define IDS_SETTINGS_AD_MEASUREMENT_PAGE_ENABLED_BULLET_2 7388
#define IDS_SETTINGS_AD_MEASUREMENT_PAGE_ENABLED_BULLET_3 7389
#define IDS_SETTINGS_AD_MEASUREMENT_PAGE_CONSIDER_BULLET_1 7390
#define IDS_SETTINGS_AD_MEASUREMENT_PAGE_CONSIDER_BULLET_2 7391
#define IDS_SETTINGS_PRIVACY_GUIDE_LABEL 7392
#define IDS_SETTINGS_PRIVACY_GUIDE_SUBLABEL 7393
#define IDS_SETTINGS_PRIVACY_GUIDE_PROMO_HEADER 7394
#define IDS_SETTINGS_PRIVACY_GUIDE_PROMO_START_BUTTON 7395
#define IDS_SETTINGS_PRIVACY_GUIDE_BACK_TO_SETTINGS_ARIA_LABEL 7396
#define IDS_SETTINGS_PRIVACY_GUIDE_BACK_TO_SETTINGS_ARIA_ROLE_DESC 7397
#define IDS_SETTINGS_PRIVACY_GUIDE_BACK_BUTTON 7398
#define IDS_SETTINGS_PRIVACY_GUIDE_STEPS 7399
#define IDS_SETTINGS_PRIVACY_GUIDE_NEXT_BUTTON 7400
#define IDS_SETTINGS_PRIVACY_GUIDE_WELCOME_CARD_HEADER 7401
#define IDS_SETTINGS_PRIVACY_GUIDE_WELCOME_CARD_SUB_HEADER 7402
#define IDS_SETTINGS_PRIVACY_GUIDE_COMPLETION_CARD_HEADER 7403
#define IDS_SETTINGS_PRIVACY_GUIDE_COMPLETION_CARD_SUB_HEADER 7404
#define IDS_SETTINGS_PRIVACY_GUIDE_COMPLETION_CARD_SUB_HEADER_NO_LINKS 7405
#define IDS_SETTINGS_PRIVACY_GUIDE_COMPLETION_CARD_LEAVE_BUTTON 7406
#define IDS_SETTINGS_PRIVACY_GUIDE_COMPLETION_CARD_PRIVACY_SANDBOX_LABEL 7407
#define IDS_SETTINGS_PRIVACY_GUIDE_COMPLETION_CARD_PRIVACY_SANDBOX_SUB_LABEL_AD_TOPICS 7408
#define IDS_SETTINGS_PRIVACY_GUIDE_COMPLETION_CARD_WAA_LABEL 7409
#define IDS_SETTINGS_PRIVACY_GUIDE_COMPLETION_CARD_AI_SETTINGS_LABEL 7410
#define IDS_SETTINGS_PRIVACY_GUIDE_MSBB_CARD_HEADER 7411
#define IDS_SETTINGS_PRIVACY_GUIDE_MSBB_FEATURE_DESCRIPTION1 7412
#define IDS_SETTINGS_PRIVACY_GUIDE_MSBB_FEATURE_DESCRIPTION2 7413
#define IDS_SETTINGS_PRIVACY_GUIDE_MSBB_PRIVACY_DESCRIPTION1 7414
#define IDS_SETTINGS_PRIVACY_GUIDE_HISTORY_SYNC_CARD_HEADER 7415
#define IDS_SETTINGS_PRIVACY_GUIDE_HISTORY_AND_TABS_SYNC_CARD_HEADER 7416
#define IDS_SETTINGS_PRIVACY_GUIDE_HISTORY_SYNC_SETTING_LABEL 7417
#define IDS_SETTINGS_PRIVACY_GUIDE_HISTORY_AND_TABS_SYNC_SETTING_LABEL 7418
#define IDS_SETTINGS_PRIVACY_GUIDE_HISTORY_SYNC_FEATURE_DESCRIPTION1 7419
#define IDS_SETTINGS_PRIVACY_GUIDE_HISTORY_AND_TABS_SYNC_FEATURE_DESCRIPTION1 7420
#define IDS_SETTINGS_PRIVACY_GUIDE_HISTORY_SYNC_FEATURE_DESCRIPTION2 7421
#define IDS_SETTINGS_PRIVACY_GUIDE_HISTORY_SYNC_PRIVACY_DESCRIPTION1 7422
#define IDS_SETTINGS_PRIVACY_GUIDE_COOKIES_CARD_HEADER 7423
#define IDS_SETTINGS_PRIVACY_GUIDE_SAFE_BROWSING_CARD_HEADER 7424
#define IDS_SETTINGS_PRIVACY_GUIDE_SAFE_BROWSING_CARD_ENHANCED_PROTECTION_PRIVACY_DESCRIPTION1 7425
#define IDS_SETTINGS_PRIVACY_GUIDE_SAFE_BROWSING_CARD_ENHANCED_PROTECTION_PRIVACY_DESCRIPTION2 7426
#define IDS_SETTINGS_PRIVACY_GUIDE_SAFE_BROWSING_CARD_ENHANCED_PROTECTION_PRIVACY_DESCRIPTION3 7427
#define IDS_SETTINGS_PRIVACY_GUIDE_SAFE_BROWSING_CARD_STANDARD_PROTECTION_FEATURE_DESCRIPTION1 7428
#define IDS_SETTINGS_SAFETY_CHECK_REVIEW 7431
#define IDS_SETTINGS_SAFETY_CHECK_UNUSED_SITE_PERMISSIONS_ALLOW_AGAIN_ARIA_LABEL 7432
#define IDS_SETTINGS_SAFETY_CHECK_UNUSED_SITE_PERMISSIONS_ALLOW_AGAIN_LABEL 7433
#define IDS_SETTINGS_SAFETY_CHECK_UNUSED_SITE_PERMISSIONS_PRIMARY_LABEL 7434
#define IDS_SETTINGS_SAFETY_CHECK_UNUSED_SITE_PERMISSIONS_SECONDARY_LABEL 7435
#define IDS_SETTINGS_SAFETY_HUB_REVOKED_PERMISSIONS_SECONDARY_LABEL 7436
#define IDS_SETTINGS_SAFETY_CHECK_UNUSED_SITE_PERMISSIONS_TOAST_BULK_LABEL 7437
#define IDS_SETTINGS_SAFETY_CHECK_UNUSED_SITE_PERMISSIONS_TOAST_LABEL 7438
#define IDS_SETTINGS_SAFETY_CHECK_UNUSED_SITE_PERMISSIONS_SETTING_LABEL 7439
#define IDS_SETTINGS_SAFETY_CHECK_PASSWORDS_PRIMARY_LABEL 7440
#define IDS_SETTINGS_SAFETY_CHECK_REVIEW_EXTENSIONS 7441
#define IDS_SETTINGS_SAFETY_CHECK_NOTIFICATION_PERMISSION_REVIEW_IGNORED_TOAST_LABEL 7442
#define IDS_SETTINGS_SAFETY_CHECK_NOTIFICATION_PERMISSION_REVIEW_BLOCKED_TOAST_LABEL 7443
#define IDS_SETTINGS_SAFETY_CHECK_NOTIFICATION_PERMISSION_REVIEW_RESET_TOAST_LABEL 7444
#define IDS_SETTINGS_SAFETY_CHECK_REVIEW_NOTIFICATION_PERMISSIONS_COUNT_LABEL 7445
#define IDS_SETTINGS_SAFETY_CHECK_REVIEW_NOTIFICATION_PERMISSIONS_DONT_ALLOW_LABEL 7446
#define IDS_SETTINGS_SAFETY_CHECK_REVIEW_NOTIFICATION_PERMISSIONS_DONT_ALLOW_ARIA_LABEL 7447
#define IDS_SETTINGS_SAFETY_CHECK_REVIEW_NOTIFICATION_PERMISSIONS_IGNORE_LABEL 7448
#define IDS_SETTINGS_SAFETY_CHECK_REVIEW_NOTIFICATION_PERMISSIONS_IGNORE_ARIA_LABEL 7449
#define IDS_SETTINGS_SAFETY_CHECK_REVIEW_NOTIFICATION_PERMISSIONS_RESET_LABEL 7450
#define IDS_SETTINGS_SAFETY_CHECK_REVIEW_NOTIFICATION_PERMISSIONS_RESET_ARIA_LABEL 7451
#define IDS_SETTINGS_SAFETY_CHECK_REVIEW_NOTIFICATION_PERMISSIONS_MORE_ACTIONS_ARIA_LABEL 7452
#define IDS_SETTINGS_SAFETY_CHECK_NOTIFICATION_PERMISSION_REVIEW_BLOCK_ALL_LABEL 7453
#define IDS_SETTINGS_SAFETY_CHECK_NOTIFICATION_PERMISSION_REVIEW_BLOCK_ALL_TOAST_LABEL 7454
#define IDS_SETTINGS_SAFETY_CHECK_TOAST_UNDO_BUTTON_LABEL 7455
#define IDS_SETTINGS_NETWORK_PREDICTION_ENABLED_DESC 7456
#define IDS_SETTINGS_SAFEBROWSING_ENABLEPROTECTION 7457
#define IDS_SETTINGS_SAFEBROWSING_ENABLEPROTECTION_DESC 7458
#define IDS_SETTINGS_SAFEBROWSING_ENABLE_REPORTING_DESC 7459
#define IDS_SETTINGS_SAFEBROWSING_SECTION_LABEL 7460
#define IDS_SETTINGS_SAFEBROWSING_ENHANCED 7461
#define IDS_SETTINGS_SAFEBROWSING_ENHANCED_DESC_UPDATED 7462
#define IDS_SETTINGS_SAFEBROWSING_ENHANCED_EXPAND_ACCESSIBILITY_LABEL 7463
#define IDS_SETTINGS_SAFEBROWSING_ENHANCED_BULLET_ONE 7464
#define IDS_SETTINGS_SAFEBROWSING_ENHANCED_BULLET_THREE 7465
#define IDS_SETTINGS_SAFEBROWSING_ENHANCED_BULLET_FOUR 7466
#define IDS_SETTINGS_SAFEBROWSING_ENHANCED_BULLET_FIVE 7467
#define IDS_SETTINGS_SAFEBROWSING_ENHANCED_WHEN_ON_BULLET_TWO 7468
#define IDS_SETTINGS_SAFEBROWSING_ENHANCED_WHEN_ON_BULLET_THREE 7469
#define IDS_SETTINGS_SAFEBROWSING_ENHANCED_WHEN_ON_BULLET_FOUR 7470
#define IDS_SETTINGS_SAFEBROWSING_ENHANCED_THINGS_TO_CONSIDER_BULLET_ONE 7471
#define IDS_SETTINGS_SAFEBROWSING_ENHANCED_THINGS_TO_CONSIDER_BULLET_TWO 7472
#define IDS_SETTINGS_SAFEBROWSING_ENHANCED_THINGS_TO_CONSIDER_BULLET_THREE 7473
#define IDS_SETTINGS_SAFEBROWSING_STANDARD 7474
#define IDS_SETTINGS_SAFEBROWSING_STANDARD_DESC 7475
#define IDS_SETTINGS_SAFEBROWSING_STANDARD_EXPAND_ACCESSIBILITY_LABEL 7478
#define IDS_SETTINGS_SAFEBROWSING_STANDARD_BULLET_ONE 7479
#define IDS_SETTINGS_SAFEBROWSING_STANDARD_HELP_IMPROVE 7480
#define IDS_SETTINGS_SAFEBROWSING_NONE 7481
#define IDS_SETTINGS_SAFEBROWSING_NONE_DESC 7482
#define IDS_SETTINGS_SAFEBROWSING_DISABLE_DIALOG_TITLE 7483
#define IDS_SETTINGS_SAFEBROWSING_DISABLE_DIALOG_DESC 7484
#define IDS_SETTINGS_SAFEBROWSING_DISABLE_DIALOG_CONFIRM 7485
#define IDS_SETTINGS_ENABLE_DO_NOT_TRACK 7486
#define IDS_SETTINGS_ENABLE_DO_NOT_TRACK_DIALOG_TITLE 7487
#define IDS_SETTINGS_ENABLE_DO_NOT_TRACK_DIALOG_TEXT 7488
#define IDS_SETTINGS_ENABLE_DO_NOT_TRACK_DIALOG_LEARN_MORE_ACCESSIBILITY_LABEL 7489
#define IDS_SETTINGS_PERMISSIONS 7490
#define IDS_SETTINGS_PERMISSIONS_DESCRIPTION 7491
#define IDS_SETTINGS_SECURITY 7492
#define IDS_SETTINGS_SECURITY_DESCRIPTION 7493
#define IDS_SETTINGS_ADVANCED_PROTECTION_PROGRAM 7494
#define IDS_SETTINGS_ADVANCED_PROTECTION_PROGRAM_DESC 7495
#define IDS_SETTINGS_SECURITY_BUNDLE_TOAST_FOR_USER_OPTED_INTO_ENHANCED_BUNDLE 7496
#define IDS_SETTINGS_SECURITY_BUNDLE_RESET_TO_DEFAULTS_BUTTON_LABEL 7497
#define IDS_SETTINGS_SECURITY_TITLE 7498
#define IDS_SETTINGS_SECURITY_STANDARD_BUNDLE_TITLE 7499
#define IDS_SETTINGS_SECURITY_ENHANCED_BUNDLE_TITLE 7500
#define IDS_SETTINGS_SECURITY_SAFE_BROWSING_TITLE 7501
#define IDS_SETTINGS_SECURITY_SAFE_BROWSING_DESCRIPTION 7502
#define IDS_SETTINGS_SECURITY_SAFE_BROWSING_STANDARD_TITLE 7503
#define IDS_SETTINGS_SECURITY_SAFE_BROWSING_STANDARD_SUBTITLE 7504
#define IDS_SETTINGS_SECURITY_SAFE_BROWSING_STANDARD_WHAT_IT_DOES_HEADING 7505
#define IDS_SETTINGS_SECURITY_SAFE_BROWSING_STANDARD_WHAT_IT_DOES_CONTENT 7506
#define IDS_SETTINGS_SECURITY_SAFE_BROWSING_STANDARD_HOW_IT_WORKS_HEADING 7507
#define IDS_SETTINGS_SECURITY_BUNDLE_STANDARD_BULLET_ONE 7508
#define IDS_SETTINGS_SECURITY_BUNDLE_STANDARD_BULLET_TWO 7509
#define IDS_SETTINGS_SECURITY_BUNDLE_STANDARD_BULLET_THREE 7510
#define IDS_SETTINGS_SECURITY_SAFE_BROWSING_ENHANCED_TITLE 7511
#define IDS_SETTINGS_SECURITY_SAFE_BROWSING_ENHANCED_SUBTITLE 7512
#define IDS_SETTINGS_SECURITY_BUNDLE_ENHANCED_BULLET_TWO 7513
#define IDS_SETTINGS_SECURITY_BUNDLE_ENHANCED_BULLET_THREE 7514
#define IDS_SETTINGS_SECURITY_FEATURE_ROW_STATE_ENHANCED 7515
#define IDS_SETTINGS_SECURITY_FEATURE_ROW_STATE_ENHANCED_CUSTOM 7516
#define IDS_SETTINGS_SECURITY_FEATURE_ROW_STATE_ENHANCED_STRICT 7517
#define IDS_SETTINGS_SECURITY_FEATURE_ROW_STATE_STANDARD 7518
#define IDS_SETTINGS_SECURITY_FEATURE_ROW_STATE_OFF 7519
#define IDS_SETTINGS_SECURITY_ACCOUNT_AND_NETWORK_SECTION_TITLE 7520
#define IDS_SETTINGS_SECURITY_ADVANCED_PROTECTION_PROGRAM_LABEL 7521
#define IDS_SETTINGS_SECURITY_PASSWORDS_LEAK_DETECTION_TITLE 7522
#define IDS_SETTINGS_SECURITY_SECURE_DNS_TITLE 7523
#define IDS_SETTINGS_SECURITY_SECURE_DNS_DESCRIPTION 7524
#define IDS_SETTINGS_SECURITY_SECURE_DNS_DEFAULT 7525
#define IDS_SETTINGS_SECURITY_SECURE_DNS_FALLBACK 7526
#define IDS_SETTINGS_SECURITY_SECURE_DNS_CUSTOM 7527
#define IDS_SETTINGS_SECURITY_JAVASCRIPT_GUARDRAILS_TITLE 7528
#define IDS_SETTINGS_SECURITY_JAVASCRIPT_GUARDRAILS_DESCRIPTION 7529
#define IDS_SETTINGS_SECURITY_JAVASCRIPT_GUARDRAILS_UNRECOGNIZED_SITES 7530
#define IDS_SETTINGS_SECURITY_JAVASCRIPT_GUARDRAILS_STRICT_FOR_ALL_SITES 7531
#define IDS_SETTINGS_SECURITY_JAVASCRIPT_GUARDRAILS_STRICT_FOR_ALL_SITES_DESCRIPTION 7532
#define IDS_SETTINGS_SECURITY_JAVASCRIPT_GUARDRAILS_MANAGE_SITE_EXCEPTIONS 7533
#define IDS_SETTINGS_SECURITY_JAVASCRIPT_GUARDRAILS_MANAGE_SITE_EXCEPTIONS_DESCRIPTION 7534
#define IDS_SETTINGS_SECURE_CONNECTIONS_SECTION_TITLE 7535
#define IDS_SETTINGS_HTTPS_ONLY_MODE 7536
#define IDS_SETTINGS_HTTPS_ONLY_MODE_DESCRIPTION 7537
#define IDS_SETTINGS_HTTPS_ONLY_MODE_DESCRIPTION_ADVANCED_PROTECTION 7538
#define IDS_SETTINGS_HTTPS_FIRST_MODE_TITLE 7539
#define IDS_SETTINGS_HTTPS_FIRST_MODE_DESCRIPTION 7540
#define IDS_SETTINGS_HTTPS_FIRST_MODE_DESCRIPTION_ADVANCED_PROTECTION 7541
#define IDS_SETTINGS_HTTPS_FIRST_MODE_ENABLED_FULL_LABEL 7542
#define IDS_SETTINGS_HTTPS_FIRST_MODE_ENABLED_INCOGNITO_LABEL 7543
#define IDS_SETTINGS_HTTPS_FIRST_MODE_DISABLED_LABEL 7544
#define IDS_SETTINGS_HTTPS_FIRST_MODE_ENABLED_STRICT_LABEL 7545
#define IDS_SETTINGS_HTTPS_FIRST_MODE_ENABLED_STRICT_SUBLABEL 7546
#define IDS_SETTINGS_HTTPS_FIRST_MODE_ENABLED_BALANCED_LABEL 7547
#define IDS_SETTINGS_HTTPS_FIRST_MODE_ENABLED_BALANCED_SUBLABEL 7548
#define IDS_SETTINGS_SECURITY_HTTPS_FIRST_MODE_TOGGLE_LABEL 7549
#define IDS_SETTINGS_SECURITY_HTTPS_FIRST_MODE_TOGGLE_SUBLABEL 7550
#define IDS_SETTINGS_SECURITY_HTTPS_FIRST_MODE_BALANCED_LABEL 7551
#define IDS_SETTINGS_SECURITY_HTTPS_FIRST_MODE_BALANCED_SUBLABEL 7552
#define IDS_SETTINGS_SECURITY_HTTPS_FIRST_MODE_STRICT_LABEL 7553
#define IDS_SETTINGS_SECURITY_HTTPS_FIRST_MODE_STRICT_SUBLABEL 7554
#define IDS_SETTINGS_MANAGE_CERTIFICATES 7555
#define IDS_SETTINGS_MANAGE_CERTIFICATES_DESCRIPTION 7556
#define IDS_SETTINGS_MANAGE_DEVICE_CERTIFICATES 7557
#define IDS_SETTINGS_MANAGE_DEVICE_CERTIFICATES_DESCRIPTION 7558
#define IDS_SETTINGS_MANAGE 7559
#define IDS_SETTINGS_SECURE_DNS 7560
#define IDS_SETTINGS_AUTOMATIC_MODE_DESCRIPTION 7562
#define IDS_SETTINGS_SECURE_DNS_DROPDOWN_ACCESSIBILITY_LABEL 7563
#define IDS_SETTINGS_SECURE_DROPDOWN_MODE_DESCRIPTION 7564
#define IDS_SETTINGS_SECURE_DROPDOWN_MODE_PRIVACY_POLICY 7565
#define IDS_SETTINGS_SECURE_DNS_DISABLED_FOR_MANAGED_ENVIRONMENT 7566
#define IDS_SETTINGS_SECURE_DNS_DISABLED_FOR_PARENTAL_CONTROL 7567
#define IDS_SETTINGS_SECURE_DNS_CUSTOM_DESCRIPTION 7568
#define IDS_SETTINGS_SECURE_DNS_CUSTOM_PLACEHOLDER 7569
#define IDS_SETTINGS_SECURE_DNS_CUSTOM_FORMAT_ERROR 7570
#define IDS_SETTINGS_SECURE_DNS_CUSTOM_CONNECTION_ERROR 7571
#define IDS_SETTINGS_SECURITY_JAVASCRIPT_OPTIMIZATION_LINK_TITLE 7572
#define IDS_SETTINGS_CONTENT_SETTINGS 7573
#define IDS_SETTINGS_SITE_SETTINGS 7574
#define IDS_SETTINGS_SITE_SETTINGS_DESCRIPTION 7575
#define IDS_SETTINGS_CLEAR_DATA 7576
#define IDS_SETTINGS_CLEARING_DATA 7577
#define IDS_SETTINGS_CLEARED_DATA 7578
#define IDS_SETTINGS_CLEAR_BROWSING_DATA 7579
#define IDS_SETTINGS_CLEAR_DATA_DESCRIPTION 7580
#define IDS_SETTINGS_TITLE_AND_COUNT 7581
#define IDS_SETTINGS_SYNC_AND_GOOGLE_SERVICES_PRIVACY_DESC_UNIFIED_CONSENT 7582
#define IDS_SETTINGS_SEARCH_AGGREGATOR_PREF 7583
#define IDS_SETTINGS_RECENT_PERMISSIONS_NO_CHANGES 7584
#define IDS_SETTINGS_RECENT_PERMISSIONS_ALLOWED_ONE_ITEM 7585
#define IDS_SETTINGS_RECENT_PERMISSIONS_ALLOWED_TWO_ITEMS 7586
#define IDS_SETTINGS_RECENT_PERMISSIONS_ALLOWED_MORE_THAN_TWO_ITEMS 7587
#define IDS_SETTINGS_RECENT_PERMISSIONS_AUTOMATICALLY_BLOCKED_ONE_ITEM 7588
#define IDS_SETTINGS_RECENT_PERMISSIONS_AUTOMATICALLY_BLOCKED_TWO_ITEMS 7589
#define IDS_SETTINGS_RECENT_PERMISSIONS_AUTOMATICALLY_BLOCKED_MORE_THAN_TWO_ITEMS 7590
#define IDS_SETTINGS_RECENT_PERMISSIONS_BLOCKED_ONE_ITEM 7591
#define IDS_SETTINGS_RECENT_PERMISSIONS_BLOCKED_TWO_ITEMS 7592
#define IDS_SETTINGS_RECENT_PERMISSIONS_BLOCKED_MORE_THAN_TWO_ITEMS 7593
#define IDS_SETTINGS_RESET_PROMPT_TITLE 7594
#define IDS_SETTINGS_RESET 7595
#define IDS_SETTINGS_RESET_SETTINGS_TRIGGER 7596
#define IDS_SETTINGS_RESET_AUTOMATED_DIALOG_TITLE 7597
#define IDS_SETTINGS_RESET_BANNER_TEXT 7598
#define IDS_SETTINGS_RESET_BANNER_RESET_BUTTON_TEXT 7599
#define IDS_SETTINGS_RESET_LEARN_MORE_ACCESSIBILITY_TEXT 7600
#define IDS_SETTINGS_RESET_AUTOMATED_DIALOG_V2_TITLE 7601
#define IDS_SETTINGS_RESET_AUTOMATED_DIALOG_V2_BODY 7602
#define IDS_SETTINGS_RESET_PINNED_TABS 7603
#define IDS_SETTINGS_RESET_EXTENSIONS 7604
#define IDS_SETTINGS_RESET_HOMEPAGE 7605
#define IDS_SETTINGS_RESET_DSE 7606
#define IDS_SETTINGS_SEARCH 7607
#define IDS_SETTINGS_DEFAULT_SEARCH 7608
#define IDS_SETTINGS_SEARCH_EXPLANATION_ACCESSIBILITY_LABEL 7609
#define IDS_SETTINGS_SEARCH_MANAGE_SEARCH_ENGINES_AND_SITE_SEARCH 7610
#define IDS_SETTINGS_SEARCH_SITE_SEARCH 7611
#define IDS_SETTINGS_SEARCH_ACTIVE_SHORTCUTS 7612
#define IDS_SETTINGS_SEARCH_ACTIVE_SHORTCUTS_EXPLANATION 7613
#define IDS_SETTINGS_SEARCH_INACTIVE_SHORTCUTS 7614
#define IDS_SETTINGS_SEARCH_INACTIVE_SHORTCUTS_EXPLANATION 7615
#define IDS_SETTINGS_SEARCH_EXTENSION_ENGINES 7616
#define IDS_SETTINGS_SEARCH_EXTENSION_ENGINES_ACTIVE_SHORTCUTS 7617
#define IDS_SETTINGS_SEARCH_EXTENSION_ENGINES_INACTIVE_SHORTCUTS 7618
#define IDS_SETTINGS_SEARCH_KEYBOARD_KEY_TITLE 7619
#define IDS_SETTINGS_SEARCH_KEYBOARD_KEY_DESCRIPTION 7620
#define IDS_SETTINGS_SEARCH_ACTIVATE 7621
#define IDS_SETTINGS_SEARCH_DEACTIVATE 7622
#define IDS_SETTINGS_SEARCH_ACTIVATE_SHORTCUT 7623
#define IDS_SETTINGS_SEARCH_DEACTIVATE_SHORTCUT 7624
#define IDS_SETTINGS_SEARCH_DISABLE_EXTENSION 7625
#define IDS_SETTINGS_SEARCH_MANAGE_EXTENSION 7626
#define IDS_SETTINGS_SEARCH_NO_SITE_SHORTCUTS 7627
#define IDS_SETTINGS_SEARCH_NO_FEATURE_SHORTCUTS 7628
#define IDS_SETTINGS_SEARCH_ENGINES 7629
#define IDS_SETTINGS_SEARCH_ENGINES_PAGE_EXPLANATION 7630
#define IDS_SETTINGS_SEARCH_ENGINES_SEARCH 7631
#define IDS_SETTINGS_SEARCH_ENGINES_SUBTITLE_FOR_POLICY 7632
#define IDS_SETTINGS_SEARCH_ENGINES_ADD_SITE_SEARCH 7633
#define IDS_SETTINGS_SEARCH_ENGINES_EDIT_SEARCH_ENGINE 7634
#define IDS_SETTINGS_SEARCH_ENGINES_VIEW_SEARCH_ENGINE 7635
#define IDS_SETTINGS_SEARCH_ENGINES_EDIT_SITE_SEARCH 7636
#define IDS_SETTINGS_SEARCH_ENGINES_VIEW_SITE_SEARCH 7637
#define IDS_SETTINGS_SEARCH_ENGINES_DELETE_CONFIRMATION_TITLE 7638
#define IDS_SETTINGS_SEARCH_ENGINES_DELETE_CONFIRMATION_DESCRIPTION 7639
#define IDS_SETTINGS_SEARCH_ENGINES_DELETE_CONFIRMATION_DESCRIPTION_FOR_POLICY 7640
#define IDS_SETTINGS_SEARCH_ENGINES_SEARCH_ENGINES 7641
#define IDS_SETTINGS_SEARCH_ENGINES_SEARCH_ENGINES_EXPLANATION 7642
#define IDS_SETTINGS_SEARCH_ENGINES_SITE_SEARCH 7643
#define IDS_SETTINGS_SEARCH_ENGINES_SITE_SEARCH_EXPLANATION 7644
#define IDS_SETTINGS_SEARCH_ENGINES_NO_SITES_ADDED 7645
#define IDS_SETTINGS_SEARCH_ENGINES_INACTIVE_SHORTCUTS 7646
#define IDS_SETTINGS_SEARCH_ENGINES_NO_OTHER_ENGINES 7647
#define IDS_SETTINGS_SEARCH_ENGINES_EXTENSION_ENGINES 7648
#define IDS_SETTINGS_SEARCH_ENGINES_EXTENSION_ENGINES_EXPLANATION 7649
#define IDS_SETTINGS_SEARCH_ENGINES_NAME 7650
#define IDS_SETTINGS_SEARCH_ENGINES_SHORTCUT 7651
#define IDS_SETTINGS_SEARCH_ENGINES_QUERY_URL 7652
#define IDS_SETTINGS_SEARCH_ENGINES_QUERY_URL_EXPLANATION 7653
#define IDS_SETTINGS_SEARCH_ENGINES_MAKE_DEFAULT 7654
#define IDS_SETTINGS_SEARCH_ENGINES_ACTIVATE 7655
#define IDS_SETTINGS_SEARCH_ENGINES_DEACTIVATE 7656
#define IDS_SETTINGS_SEARCH_ENGINES_VIEW_DETAILS 7657
#define IDS_SETTINGS_SEARCH_ENGINES_MANAGE_EXTENSION 7658
#define IDS_SETTINGS_SEARCH_ENGINES_KEYBOARD_SHORTCUTS_TITLE 7659
#define IDS_SETTINGS_SEARCH_ENGINES_KEYBOARD_SHORTCUTS_DESCRIPTION 7660
#define IDS_SETTINGS_SEARCH_ENGINES_KEYBOARD_SHORTCUTS_SPACE_OR_TAB 7661
#define IDS_SETTINGS_SEARCH_ENGINES_KEYBOARD_SHORTCUTS_TAB 7662
#define IDS_SETTINGS_SEARCH_ENGINES_ADDITIONAL_SITES 7663
#define IDS_SETTINGS_SEARCH_ENGINES_ADDITIONAL_INACTIVE_SITES 7664
#define IDS_SETTINGS_SEARCH_ENGINES_MORE_ACTIONS_ARIA_LABEL 7665
#define IDS_SETTINGS_SEARCH_ENGINES_ACTIVATE_BUTTON_ARIA_LABEL 7666
#define IDS_SETTINGS_SEARCH_ENGINES_ADD_SITE_SEARCH_BUTTON_ARIA_LABEL 7667
#define IDS_SETTINGS_SEARCH_ENGINES_EDIT_SEARCH_ENGINE_BUTTON_ARIA_LABEL 7668
#define IDS_SETTINGS_EXCEPTIONS_EMBEDDED_ON_HOST 7669
#define IDS_SETTINGS_SITE_SETTINGS_ALL_SITES 7670
#define IDS_SETTINGS_SITE_SETTINGS_ALL_SITES_DESCRIPTION 7671
#define IDS_SETTINGS_SITE_SETTINGS_ALL_SITES_FILTER 7672
#define IDS_SETTINGS_SITE_SETTINGS_ALL_SITES_SEARCH 7673
#define IDS_SETTINGS_SITE_SETTINGS_ALL_SITES_SORT 7674
#define IDS_SETTINGS_SITE_SETTINGS_ALL_SITES_SORT_METHOD_MOST_VISITED 7675
#define IDS_SETTINGS_SITE_SETTINGS_ALL_SITES_SORT_METHOD_STORAGE 7676
#define IDS_SETTINGS_SITE_SETTINGS_ALL_SITES_SORT_METHOD_NAME 7677
#define IDS_SETTINGS_SITE_SETTINGS_FILE_SYSTEM_SITE_LIST_HEADER 7678
#define IDS_SETTINGS_SITE_SETTINGS_FILE_SYSTEM_SITE_LIST_EDIT_HEADER 7679
#define IDS_SETTINGS_SITE_SETTINGS_FILE_SYSTEM_SITE_LIST_REMOVE_GRANT_LABEL 7680
#define IDS_SETTINGS_SITE_SETTINGS_FILE_SYSTEM_SITE_LIST_REMOVE_GRANTS 7681
#define IDS_SETTINGS_SITE_SETTINGS_FILE_SYSTEM_SITE_LIST_VIEW_HEADER 7682
#define IDS_SETTINGS_SITE_SETTINGS_FILE_SYSTEM_SITE_LIST_VIEW_SITE_DETAILS 7683
#define IDS_SETTINGS_SITE_SETTINGS_SITE_ENTRY_PARTITIONED_LABEL 7684
#define IDS_SETTINGS_SITE_SETTINGS_SITE_REPRESENTATION_SEPARATOR 7685
#define IDS_SETTINGS_SITE_SETTINGS_DEFAULT_BEHAVIOR 7686
#define IDS_SETTINGS_SITE_SETTINGS_DEFAULT_BEHAVIOR_DESCRIPTION 7687
#define IDS_SETTINGS_SITE_SETTINGS_NOTIFICATIONS_DEFAULT_BEHAVIOR_DESC 7688
#define IDS_SETTINGS_SITE_SETTINGS_NOTIFICATIONS_DESC 7689
#define IDS_SETTINGS_SITE_SETTINGS_CUSTOMIZED_BEHAVIORS 7690
#define IDS_SETTINGS_SITE_SETTINGS_CUSTOMIZED_BEHAVIORS_DESCRIPTION 7691
#define IDS_SETTINGS_SITE_SETTINGS_CUSTOMIZED_BEHAVIORS_DESCRIPTION_SHORT 7692
#define IDS_SETTINGS_SITE_SETTINGS_ADS_DESCRIPTION 7693
#define IDS_SETTINGS_SITE_SETTINGS_ADS_ALLOWED 7694
#define IDS_SETTINGS_SITE_SETTINGS_ADS_BLOCKED 7695
#define IDS_SETTINGS_SITE_SETTINGS_ADS_ALLOWED_EXCEPTIONS 7696
#define IDS_SETTINGS_SITE_SETTINGS_ADS_BLOCKED_EXCEPTIONS 7697
#define IDS_SETTINGS_SITE_SETTINGS_AR_DESCRIPTION 7698
#define IDS_SETTINGS_SITE_SETTINGS_AR_ALLOWED 7699
#define IDS_SETTINGS_SITE_SETTINGS_AR_BLOCKED 7700
#define IDS_SETTINGS_SITE_SETTINGS_AR_ALLOWED_EXCEPTIONS 7701
#define IDS_SETTINGS_SITE_SETTINGS_AR_BLOCKED_EXCEPTIONS 7702
#define IDS_SETTINGS_SITE_SETTINGS_AUTOMATIC_DOWNLOADS_DESCRIPTION 7703
#define IDS_SETTINGS_SITE_SETTINGS_AUTOMATIC_DOWNLOADS_ASK 7704
#define IDS_SETTINGS_SITE_SETTINGS_AUTOMATIC_DOWNLOADS_BLOCKED 7705
#define IDS_SETTINGS_SITE_SETTINGS_AUTOMATIC_DOWNLOADS_ALLOWED_EXCEPTIONS 7706
#define IDS_SETTINGS_SITE_SETTINGS_AUTOMATIC_DOWNLOADS_BLOCKED_EXCEPTIONS 7707
#define IDS_SETTINGS_SITE_SETTINGS_AUTO_PICTURE_IN_PICTURE_DESCRIPTION 7708
#define IDS_SETTINGS_SITE_SETTINGS_AUTO_PICTURE_IN_PICTURE_ALLOWED 7709
#define IDS_SETTINGS_SITE_SETTINGS_AUTO_PICTURE_IN_PICTURE_BLOCKED 7710
#define IDS_SETTINGS_SITE_SETTINGS_AUTO_PICTURE_IN_PICTURE_ALLOWED_EXCEPTIONS 7711
#define IDS_SETTINGS_SITE_SETTINGS_AUTO_PICTURE_IN_PICTURE_BLOCKED_EXCEPTIONS 7712
#define IDS_SETTINGS_SITE_SETTINGS_BACKGROUND_SYNC_DESCRIPTION 7713
#define IDS_SETTINGS_SITE_SETTINGS_BACKGROUND_SYNC_ALLOWED 7714
#define IDS_SETTINGS_SITE_SETTINGS_BACKGROUND_SYNC_BLOCKED 7715
#define IDS_SETTINGS_SITE_SETTINGS_BACKGROUND_SYNC_BLOCKED_SUB_LABEL 7716
#define IDS_SETTINGS_SITE_SETTINGS_BACKGROUND_SYNC_ALLOWED_EXCEPTIONS 7717
#define IDS_SETTINGS_SITE_SETTINGS_BACKGROUND_SYNC_BLOCKED_EXCEPTIONS 7718
#define IDS_SETTINGS_SITE_SETTINGS_BLUETOOTH_DEVICES_DESCRIPTION 7719
#define IDS_SETTINGS_SITE_SETTINGS_BLUETOOTH_DEVICES_ASK 7720
#define IDS_SETTINGS_SITE_SETTINGS_BLUETOOTH_DEVICES_BLOCKED 7721
#define IDS_SETTINGS_SITE_SETTINGS_CAPTURED_SURFACE_CONTROL_DESCRIPTION 7722
#define IDS_SETTINGS_SITE_SETTINGS_CAPTURED_SURFACE_CONTROL_ASK 7723
#define IDS_SETTINGS_SITE_SETTINGS_CAPTURED_SURFACE_CONTROL_BLOCKED 7724
#define IDS_SETTINGS_SITE_SETTINGS_CAPTURED_SURFACE_CONTROL_ALLOWED_EXCEPTIONS 7725
#define IDS_SETTINGS_SITE_SETTINGS_CAPTURED_SURFACE_CONTROL_BLOCKED_EXCEPTIONS 7726
#define IDS_SETTINGS_SITE_SETTINGS_CAMERA_DESCRIPTION 7727
#define IDS_SETTINGS_SITE_SETTINGS_CAMERA_ASK 7728
#define IDS_SETTINGS_SITE_SETTINGS_CAMERA_BLOCKED 7729
#define IDS_SETTINGS_SITE_SETTINGS_CAMERA_BLOCKED_SUB_LABEL 7730
#define IDS_SETTINGS_SITE_SETTINGS_CAMERA_ALLOWED_EXCEPTIONS 7731
#define IDS_SETTINGS_SITE_SETTINGS_CAMERA_BLOCKED_EXCEPTIONS 7732
#define IDS_SETTINGS_SITE_SETTINGS_CLIPBOARD_DESCRIPTION 7733
#define IDS_SETTINGS_SITE_SETTINGS_CLIPBOARD_ASK 7734
#define IDS_SETTINGS_SITE_SETTINGS_CLIPBOARD_BLOCKED 7735
#define IDS_SETTINGS_SITE_SETTINGS_CLIPBOARD_ALLOWED_EXCEPTIONS 7736
#define IDS_SETTINGS_SITE_SETTINGS_CLIPBOARD_BLOCKED_EXCEPTIONS 7737
#define IDS_SETTINGS_SITE_SETTINGS_DEVICE_USE_DESCRIPTION 7738
#define IDS_SETTINGS_SITE_SETTINGS_DEVICE_USE_ASK 7739
#define IDS_SETTINGS_SITE_SETTINGS_DEVICE_USE_BLOCKED 7740
#define IDS_SETTINGS_SITE_SETTINGS_DEVICE_USE_ALLOWED_EXCEPTIONS 7741
#define IDS_SETTINGS_SITE_SETTINGS_DEVICE_USE_BLOCKED_EXCEPTIONS 7742
#define IDS_SETTINGS_SITE_SETTINGS_FEDERATED_IDENTITY_API_DESCRIPTION 7743
#define IDS_SETTINGS_SITE_SETTINGS_FEDERATED_IDENTITY_API_ALLOWED 7744
#define IDS_SETTINGS_SITE_SETTINGS_FEDERATED_IDENTITY_API_BLOCKED 7745
#define IDS_SETTINGS_SITE_SETTINGS_FEDERATED_IDENTITY_API_ALLOWED_EXCEPTIONS 7746
#define IDS_SETTINGS_SITE_SETTINGS_FEDERATED_IDENTITY_API_BLOCKED_EXCEPTIONS 7747
#define IDS_SETTINGS_SITE_SETTINGS_FILE_SYSTEM_WRITE_DESCRIPTION 7748
#define IDS_SETTINGS_SITE_SETTINGS_FILE_SYSTEM_WRITE_ASK 7749
#define IDS_SETTINGS_SITE_SETTINGS_FILE_SYSTEM_WRITE_BLOCKED 7750
#define IDS_SETTINGS_SITE_SETTINGS_FILE_SYSTEM_WRITE_BLOCKED_EXCEPTIONS 7751
#define IDS_SETTINGS_SITE_SETTINGS_FONTS_DESCRIPTION 7752
#define IDS_SETTINGS_SITE_SETTINGS_FONTS_ASK 7753
#define IDS_SETTINGS_SITE_SETTINGS_FONTS_BLOCKED 7754
#define IDS_SETTINGS_SITE_SETTINGS_FONTS_ALLOWED_EXCEPTIONS 7755
#define IDS_SETTINGS_SITE_SETTINGS_FONTS_BLOCKED_EXCEPTIONS 7756
#define IDS_SETTINGS_SITE_SETTINGS_AUTOMATIC_FULLSCREEN_DESCRIPTION 7757
#define IDS_SETTINGS_SITE_SETTINGS_AUTOMATIC_FULLSCREEN_ALLOWED_EXCEPTIONS 7758
#define IDS_SETTINGS_SITE_SETTINGS_AUTOMATIC_FULLSCREEN_BLOCKED_EXCEPTIONS 7759
#define IDS_SETTINGS_SITE_SETTINGS_HAND_TRACKING_DESCRIPTION 7760
#define IDS_SETTINGS_SITE_SETTINGS_HAND_TRACKING_ALLOWED_EXCEPTIONS 7761
#define IDS_SETTINGS_SITE_SETTINGS_HAND_TRACKING_BLOCKED_EXCEPTIONS 7762
#define IDS_SETTINGS_SITE_SETTINGS_HID_DEVICES_DESCRIPTION 7763
#define IDS_SETTINGS_SITE_SETTINGS_HID_DEVICES_ASK 7764
#define IDS_SETTINGS_SITE_SETTINGS_HID_DEVICES_BLOCKED 7765
#define IDS_SITE_SETTINGS_SMART_CARD_READERS 7766
#define IDS_SITE_SETTINGS_SMART_CARD_READERS_DESCRIPTION 7767
#define IDS_SITE_SETTINGS_SMART_CARDS_DEFAULT_DESCRIPTION 7768
#define IDS_SITE_SETTINGS_SMART_CARDS_ASK 7769
#define IDS_SITE_SETTINGS_SMART_CARDS_BLOCKED 7770
#define IDS_SITE_SETTINGS_NO_SMART_CARD_READERS_FOUND 7771
#define IDS_SITE_SETTINGS_RESET_SMART_CARD_CONFIRMATION 7772
#define IDS_SETTINGS_SITE_SETTINGS_IMAGES_DESCRIPTION 7773
#define IDS_SETTINGS_SITE_SETTINGS_IMAGES_ALLOWED 7774
#define IDS_SETTINGS_SITE_SETTINGS_IMAGES_BLOCKED 7775
#define IDS_SETTINGS_SITE_SETTINGS_IMAGES_BLOCKED_SUB_LABEL 7776
#define IDS_SETTINGS_SITE_SETTINGS_IMAGES_ALLOWED_EXCEPTIONS 7777
#define IDS_SETTINGS_SITE_SETTINGS_IMAGES_BLOCKED_EXCEPTIONS 7778
#define IDS_SETTINGS_SITE_SETTINGS_INSECURE_CONTENT_DESCRIPTION 7779
#define IDS_SETTINGS_SITE_SETTINGS_INSECURE_CONTENT_ALLOWED_EXCEPTIONS 7780
#define IDS_SETTINGS_SITE_SETTINGS_INSECURE_CONTENT_BLOCKED_EXCEPTIONS 7781
#define IDS_SETTINGS_SITE_SETTINGS_JAVASCRIPT_DESCRIPTION 7782
#define IDS_SETTINGS_SITE_SETTINGS_JAVASCRIPT_ALLOWED 7783
#define IDS_SETTINGS_SITE_SETTINGS_JAVASCRIPT_BLOCKED 7784
#define IDS_SETTINGS_SITE_SETTINGS_JAVASCRIPT_ALLOWED_EXCEPTIONS 7785
#define IDS_SETTINGS_SITE_SETTINGS_JAVASCRIPT_BLOCKED_EXCEPTIONS 7786
#define IDS_SETTINGS_SITE_SETTINGS_JAVASCRIPT_OPTIMIZER_ALLOWED 7787
#define IDS_SETTINGS_SITE_SETTINGS_JAVASCRIPT_OPTIMIZER_BLOCKED 7788
#define IDS_SETTINGS_SITE_SETTINGS_JAVASCRIPT_OPTIMIZER_BLOCKED_UNFAMILIAR_SITES 7789
#define IDS_SETTINGS_SITE_SETTINGS_JAVASCRIPT_OPTIMIZER_BLOCKED_UNFAMILIAR_SITES_SUB_LABEL 7790
#define IDS_SETTINGS_SITE_SETTINGS_JAVASCRIPT_OPTIMIZER_BLOCKED_UNFAMILIAR_SITES_SAFE_BROWSING_OFF_SUB_LABEL 7791
#define IDS_SETTINGS_SITE_SETTINGS_JAVASCRIPT_OPTIMIZER_ALLOWED_EXCEPTIONS 7792
#define IDS_SETTINGS_SITE_SETTINGS_JAVASCRIPT_OPTIMIZER_BLOCKED_EXCEPTIONS 7793
#define IDS_SETTINGS_SITE_SETTINGS_KEYBOARD_LOCK_DESCRIPTION 7794
#define IDS_SETTINGS_SITE_SETTINGS_KEYBOARD_LOCK_ASK 7795
#define IDS_SETTINGS_SITE_SETTINGS_KEYBOARD_LOCK_BLOCKED 7796
#define IDS_SETTINGS_SITE_SETTINGS_KEYBOARD_LOCK_ALLOWED_EXCEPTIONS 7797
#define IDS_SETTINGS_SITE_SETTINGS_KEYBOARD_LOCK_BLOCKED_EXCEPTIONS 7798
#define IDS_SETTINGS_SITE_SETTINGS_LOCATION_DESCRIPTION 7799
#define IDS_SETTINGS_SITE_SETTINGS_LOCATION_ASK 7800
#define IDS_SETTINGS_SITE_SETTINGS_LOCATION_BLOCKED 7801
#define IDS_SETTINGS_SITE_SETTINGS_LOCATION_BLOCKED_SUB_LABEL 7802
#define IDS_SETTINGS_SITE_SETTINGS_LOCATION_ALLOWED_EXCEPTIONS 7803
#define IDS_SETTINGS_SITE_SETTINGS_LOCATION_BLOCKED_EXCEPTIONS 7804
#define IDS_SETTINGS_SITE_SETTINGS_MIC_DESCRIPTION 7805
#define IDS_SETTINGS_SITE_SETTINGS_MIC_ASK 7806
#define IDS_SETTINGS_SITE_SETTINGS_MIC_BLOCKED 7807
#define IDS_SETTINGS_SITE_SETTINGS_MIC_BLOCKED_SUB_LABEL 7808
#define IDS_SETTINGS_SITE_SETTINGS_MIC_ALLOWED_EXCEPTIONS 7809
#define IDS_SETTINGS_SITE_SETTINGS_MIC_BLOCKED_EXCEPTIONS 7810
#define IDS_SETTINGS_SITE_SETTINGS_MIDI_DESCRIPTION 7811
#define IDS_SETTINGS_SITE_SETTINGS_MIDI_ASK 7812
#define IDS_SETTINGS_SITE_SETTINGS_MIDI_BLOCKED 7813
#define IDS_SETTINGS_SITE_SETTINGS_MIDI_ALLOWED_EXCEPTIONS 7814
#define IDS_SETTINGS_SITE_SETTINGS_MIDI_BLOCKED_EXCEPTIONS 7815
#define IDS_SETTINGS_SITE_SETTINGS_MOTION_AND_LIGHT_SENSORS_DESCRIPTION 7816
#define IDS_SETTINGS_SITE_SETTINGS_MOTION_AND_LIGHT_SENSORS_ALLOWED 7817
#define IDS_SETTINGS_SITE_SETTINGS_MOTION_AND_LIGHT_SENSORS_ASK 7818
#define IDS_SETTINGS_SITE_SETTINGS_MOTION_AND_LIGHT_SENSORS_BLOCKED 7819
#define IDS_SETTINGS_SITE_SETTINGS_MOTION_AND_LIGHT_SENSORS_BLOCKED_SUB_LABEL 7820
#define IDS_SETTINGS_SITE_SETTINGS_MOTION_AND_LIGHT_SENSORS_ALLOWED_EXCEPTIONS 7821
#define IDS_SETTINGS_SITE_SETTINGS_MOTION_AND_LIGHT_SENSORS_BLOCKED_EXCEPTIONS 7822
#define IDS_SETTINGS_SITE_SETTINGS_MOTION_SENSORS_DESCRIPTION 7823
#define IDS_SETTINGS_SITE_SETTINGS_MOTION_SENSORS_ALLOWED 7824
#define IDS_SETTINGS_SITE_SETTINGS_MOTION_SENSORS_ASK 7825
#define IDS_SETTINGS_SITE_SETTINGS_MOTION_SENSORS_BLOCKED 7826
#define IDS_SETTINGS_SITE_SETTINGS_MOTION_SENSORS_BLOCKED_SUB_LABEL 7827
#define IDS_SETTINGS_SITE_SETTINGS_MOTION_SENSORS_ALLOWED_EXCEPTIONS 7828
#define IDS_SETTINGS_SITE_SETTINGS_MOTION_SENSORS_BLOCKED_EXCEPTIONS 7829
#define IDS_SETTINGS_SITE_SETTINGS_NOTIFICATIONS_ALLOWED 7830
#define IDS_SETTINGS_SITE_SETTINGS_NOTIFICATIONS_PARTIAL 7831
#define IDS_SETTINGS_SITE_SETTINGS_NOTIFICATIONS_PARTIAL_SUB_LABEL 7832
#define IDS_SETTINGS_SITE_SETTINGS_NOTIFICATIONS_ASK_STATE 7833
#define IDS_SETTINGS_SITE_SETTINGS_PERMISSION_QUIET 7834
#define IDS_SETTINGS_SITE_SETTINGS_PERMISSION_CPSS 7835
#define IDS_SETTINGS_SITE_SETTINGS_PERMISSION_LOUD 7836
#define IDS_SETTINGS_SITE_SETTINGS_PERMISSION_TITLE 7837
#define IDS_SETTINGS_SITE_SETTINGS_NOTIFICATIONS_BLOCKED 7838
#define IDS_SETTINGS_SITE_SETTINGS_NOTIFICATIONS_BLOCKED_SUB_LABEL 7839
#define IDS_SETTINGS_SITE_SETTINGS_NOTIFICATIONS_ALLOWED_EXCEPTIONS 7840
#define IDS_SETTINGS_SITE_SETTINGS_NOTIFICATIONS_BLOCKED_EXCEPTIONS 7841
#define IDS_SETTINGS_SITE_SETTINGS_PAYMENT_HANDLERS_DESCRIPTION 7842
#define IDS_SETTINGS_SITE_SETTINGS_PAYMENT_HANDLERS_ALLOWED 7843
#define IDS_SETTINGS_SITE_SETTINGS_PAYMENT_HANDLERS_BLOCKED 7844
#define IDS_SETTINGS_SITE_SETTINGS_PAYMENT_HANDLERS_ALLOWED_EXCEPTIONS 7845
#define IDS_SETTINGS_SITE_SETTINGS_PAYMENT_HANDLERS_BLOCKED_EXCEPTIONS 7846
#define IDS_SETTINGS_SITE_SETTINGS_PDFS_DESCRIPTION 7847
#define IDS_SETTINGS_SITE_SETTINGS_PDFS_ALLOWED 7848
#define IDS_SETTINGS_SITE_SETTINGS_POINTER_LOCK_DESCRIPTION 7849
#define IDS_SETTINGS_SITE_SETTINGS_POINTER_LOCK_ALLOWED 7850
#define IDS_SETTINGS_SITE_SETTINGS_POINTER_LOCK_BLOCKED 7851
#define IDS_SETTINGS_SITE_SETTINGS_POINTER_LOCK_ALLOWED_EXCEPTIONS 7852
#define IDS_SETTINGS_SITE_SETTINGS_POINTER_LOCK_BLOCKED_EXCEPTIONS 7853
#define IDS_SETTINGS_SITE_SETTINGS_POPUPS_DESCRIPTION 7854
#define IDS_SETTINGS_SITE_SETTINGS_POPUPS_ALLOWED 7855
#define IDS_SETTINGS_SITE_SETTINGS_POPUPS_BLOCKED 7856
#define IDS_SETTINGS_SITE_SETTINGS_POPUPS_ALLOWED_EXCEPTIONS 7857
#define IDS_SETTINGS_SITE_SETTINGS_POPUPS_BLOCKED_EXCEPTIONS 7858
#define IDS_SETTINGS_SITE_SETTINGS_PROTECTED_CONTENT_DESCRIPTION 7859
#define IDS_SETTINGS_SITE_SETTINGS_PROTECTED_CONTENT_ALLOWED 7860
#define IDS_SETTINGS_SITE_SETTINGS_PROTECTED_CONTENT_BLOCKED 7861
#define IDS_SETTINGS_SITE_SETTINGS_PROTECTED_CONTENT_BLOCKED_SUB_LABEL 7862
#define IDS_SETTINGS_SITE_SETTINGS_PROTOCOL_HANDLERS_DESCRIPTION 7870
#define IDS_SETTINGS_SITE_SETTINGS_PROTOCOL_HANDLERS_ALLOWED 7871
#define IDS_SETTINGS_SITE_SETTINGS_PROTOCOL_HANDLERS_BLOCKED 7872
#define IDS_SETTINGS_SITE_SETTINGS_PROTOCOL_HANDLERS_BLOCKED_EXCEPTIONS 7873
#define IDS_SETTINGS_SITE_SETTINGS_SERIAL_PORTS_DESCRIPTION 7874
#define IDS_SETTINGS_SITE_SETTINGS_SERIAL_PORTS_ASK 7875
#define IDS_SETTINGS_SITE_SETTINGS_SERIAL_PORTS_BLOCKED 7876
#define IDS_SETTINGS_SITE_SETTINGS_SOUND_DESCRIPTION 7877
#define IDS_SETTINGS_SITE_SETTINGS_SOUND_ALLOWED 7878
#define IDS_SETTINGS_SITE_SETTINGS_SOUND_BLOCKED 7879
#define IDS_SETTINGS_SITE_SETTINGS_SOUND_BLOCKED_SUB_LABEL 7880
#define IDS_SETTINGS_SITE_SETTINGS_SOUND_ALLOWED_EXCEPTIONS 7881
#define IDS_SETTINGS_SITE_SETTINGS_SOUND_BLOCKED_EXCEPTIONS 7882
#define IDS_SETTINGS_SITE_SETTINGS_USB_DESCRIPTION 7883
#define IDS_SETTINGS_SITE_SETTINGS_USB_ASK 7884
#define IDS_SETTINGS_SITE_SETTINGS_USB_BLOCKED 7885
#define IDS_SETTINGS_SITE_SETTINGS_VR_DESCRIPTION 7886
#define IDS_SETTINGS_SITE_SETTINGS_VR_ASK 7887
#define IDS_SETTINGS_SITE_SETTINGS_VR_BLOCKED 7888
#define IDS_SETTINGS_SITE_SETTINGS_VR_ALLOWED_EXCEPTIONS 7889
#define IDS_SETTINGS_SITE_SETTINGS_VR_BLOCKED_EXCEPTIONS 7890
#define IDS_SETTINGS_SITE_SETTINGS_WEB_APP_INSTALLATION_DESCRIPTION 7891
#define IDS_SETTINGS_SITE_SETTINGS_WEB_APP_INSTALLATION_ASK 7892
#define IDS_SETTINGS_SITE_SETTINGS_WEB_APP_INSTALLATION_BLOCK 7893
#define IDS_SETTINGS_SITE_SETTINGS_WEB_APP_INSTALLATION_ALLOWED_EXCEPTIONS 7894
#define IDS_SETTINGS_SITE_SETTINGS_WEB_APP_INSTALLATION_BLOCKED_EXCEPTIONS 7895
#define IDS_SETTINGS_SITE_SETTINGS_WEB_PRINTING_DESCRIPTION 7896
#define IDS_SETTINGS_SITE_SETTINGS_WEB_PRINTING_ASK 7897
#define IDS_SETTINGS_SITE_SETTINGS_WEB_PRINTING_BLOCK 7898
#define IDS_SETTINGS_SITE_SETTINGS_WEB_PRINTING_ALLOWED_EXCEPTIONS 7899
#define IDS_SETTINGS_SITE_SETTINGS_WEB_PRINTING_BLOCKED_EXCEPTIONS 7900
#define IDS_SETTINGS_SITE_SETTINGS_ZOOM_LEVELS_DESCRIPTION 7901
#define IDS_SETTINGS_SITE_SETTINGS_AR_ASK 7902
#define IDS_SETTINGS_SITE_SETTINGS_AR_BLOCK 7903
#define IDS_SETTINGS_SITE_SETTINGS_HAND_TRACKING_ASK 7904
#define IDS_SETTINGS_SITE_SETTINGS_HAND_TRACKING_BLOCK 7905
#define IDS_SETTINGS_SITE_SETTINGS_LOCAL_NETWORK_DESCRIPTION 7906
#define IDS_SETTINGS_SITE_SETTINGS_LOCAL_NETWORK_ASK 7907
#define IDS_SETTINGS_SITE_SETTINGS_LOCAL_NETWORK_BLOCK 7908
#define IDS_SETTINGS_SITE_SETTINGS_LOCAL_NETWORK_ALLOWED_EXCEPTIONS 7909
#define IDS_SETTINGS_SITE_SETTINGS_LOCAL_NETWORK_BLOCKED_EXCEPTIONS 7910
#define IDS_SETTINGS_SITE_SETTINGS_LOOPBACK_NETWORK_DESCRIPTION 7911
#define IDS_SETTINGS_SITE_SETTINGS_LOOPBACK_NETWORK_ASK 7912
#define IDS_SETTINGS_SITE_SETTINGS_LOOPBACK_NETWORK_BLOCK 7913
#define IDS_SETTINGS_SITE_SETTINGS_LOOPBACK_NETWORK_ALLOWED_EXCEPTIONS 7914
#define IDS_SETTINGS_SITE_SETTINGS_LOOPBACK_NETWORK_BLOCKED_EXCEPTIONS 7915
#define IDS_SETTINGS_THIRD_PARTY_COOKIES_PAGE_DESCRIPTION 7916
#define IDS_SETTINGS_THIRD_PARTY_COOKIES_PAGE_TITLE 7917
#define IDS_SETTINGS_THIRD_PARTY_COOKIES_LINK_ROW_LABEL 7918
#define IDS_SETTINGS_THIRD_PARTY_COOKIES_LINK_ROW_SUB_LABEL_ENABLED 7919
#define IDS_SETTINGS_THIRD_PARTY_COOKIES_LINK_ROW_SUB_LABEL_DISABLED 7920
#define IDS_SETTINGS_THIRD_PARTY_COOKIES_PAGE_ALLOW_RADIO_LABEL 7921
#define IDS_SETTINGS_THIRD_PARTY_COOKIES_PAGE_BLOCK_RADIO_LABEL 7922
#define IDS_SETTINGS_COOKIES_RELATED_WEBSITE_SETS_TOGGLE_LABEL 7923
#define IDS_SETTINGS_COOKIES_RELATED_WEBSITE_SETS_TOGGLE_SUB_LABEL 7924
#define IDS_SETTINGS_COOKIES_ALL_SITES_LINK 7925
#define IDS_SETTINGS_TRACKING_PROTECTION_ADVANCED_LABEL 7926
#define IDS_SETTINGS_TRACKING_PROTECTION_DO_NOT_TRACK_TOGGLE_SUB_LABEL 7927
#define IDS_SETTINGS_TRACKING_PROTECTION_SITES_ALLOWED_COOKIES_TITLE 7928
#define IDS_SETTINGS_TRACKING_PROTECTION_SITES_ALLOWED_COOKIES_DESCRIPTION 7929
#define IDS_SETTINGS_SITE_SETTINGS_APP_PROTOCOL_HANDLERS 7930
#define IDS_SETTINGS_SITE_SETTINGS_APP_ALLOWED_PROTOCOL_HANDLERS_DESCRIPTION 7931
#define IDS_SETTINGS_SITE_SETTINGS_APP_DISALLOWED_PROTOCOL_HANDLERS_DESCRIPTION 7932
#define IDS_SETTINGS_SITE_SETTINGS_INSECURE_CONTENT_BLOCK 7933
#define IDS_SETTINGS_SITE_SETTINGS_PDF_DOWNLOAD_PDFS 7934
#define IDS_SETTINGS_SITE_SETTINGS_RECENT_ACTIVITY 7935
#define IDS_SETTINGS_SITE_SETTINGS_REMOVE_ZOOM_LEVEL 7936
#define IDS_SETTINGS_SITE_SETTINGS_ASK_BEFORE_SENDING 7937
#define IDS_SETTINGS_SITE_SETTINGS_HANDLERS_ASK_RECOMMENDED 7938
#define IDS_SETTINGS_SITE_SETTINGS_HANDLERS_BLOCKED 7939
#define IDS_SETTINGS_SITE_SETTINGS_WINDOW_MANAGEMENT_DESCRIPTION 7940
#define IDS_SETTINGS_SITE_SETTINGS_WINDOW_MANAGEMENT_ASK 7941
#define IDS_SETTINGS_SITE_SETTINGS_WINDOW_MANAGEMENT_BLOCKED 7942
#define IDS_SETTINGS_SITE_SETTINGS_WINDOW_MANAGEMENT_ASK_EXCEPTIONS 7943
#define IDS_SETTINGS_SITE_SETTINGS_WINDOW_MANAGEMENT_BLOCKED_EXCEPTIONS 7944
#define IDS_SETTINGS_SITE_SETTINGS_BLOCKED 7945
#define IDS_SETTINGS_SITE_SETTINGS_ALLOW 7946
#define IDS_SETTINGS_SITE_SETTINGS_BLOCK 7947
#define IDS_SETTINGS_SITE_SETTINGS_SESSION_ONLY 7948
#define IDS_SETTINGS_SITE_SETTINGS_ASK_DEFAULT_MENU 7949
#define IDS_SETTINGS_SITE_SETTINGS_ALLOW_DEFAULT_MENU 7950
#define IDS_SETTINGS_SITE_SETTINGS_AUTOMATIC_DEFAULT_MENU 7951
#define IDS_SETTINGS_SITE_SETTINGS_BLOCK_DEFAULT_MENU 7952
#define IDS_SETTINGS_SITE_SETTINGS_ACTION_BLOCK_ON_UNFAMILIAR_SITES_DEFAULT_MENU 7953
#define IDS_SETTINGS_SITE_SETTINGS_MUTE_DEFAULT_MENU 7954
#define IDS_SETTINGS_SITE_SETTINGS_ALLOW_MENU 7955
#define IDS_SETTINGS_SITE_SETTINGS_BLOCK_MENU 7956
#define IDS_SETTINGS_SITE_SETTINGS_ASK_MENU 7957
#define IDS_SETTINGS_SITE_SETTINGS_MUTE_MENU 7958
#define IDS_SETTINGS_SITE_SETTINGS_RESET_MENU 7959
#define IDS_SETTINGS_SITE_SETTINGS_RESET_FROM_LIST_A11Y 7960
#define IDS_SETTINGS_SITE_SETTINGS_VIEW_FROM_LIST_A11Y 7961
#define IDS_SETTINGS_SITE_SETTINGS_SESSION_ONLY_MENU 7962
#define IDS_SETTINGS_SITE_SETTINGS_USAGE 7963
#define IDS_SETTINGS_SITE_SETTINGS_USAGE_NONE 7964
#define IDS_SETTINGS_SITE_SETTINGS_PERMISSIONS 7965
#define IDS_SETTINGS_SITE_SETTINGS_PERMISSIONS_MORE 7966
#define IDS_SETTINGS_SITE_SETTINGS_CONTENT 7967
#define IDS_SETTINGS_SITE_SETTINGS_CONTENT_MORE 7968
#define IDS_SETTINGS_SITE_SETTINGS_ALLOWLISTED 7969
#define IDS_SETTINGS_SITE_SETTINGS_ADS_BLOCK_BLOCKLISTED_SINGULAR 7970
#define IDS_SETTINGS_SITE_SETTINGS_ADS_BLOCK_NOT_BLOCKLISTED_SINGULAR 7971
#define IDS_SETTINGS_SITE_SETTINGS_SOURCE_KILL_SWITCH 7972
#define IDS_SETTINGS_SITE_SETTINGS_SOURCE_INSECURE_ORIGIN 7973
#define IDS_SETTINGS_SITE_SETTINGS_RESET_BUTTON 7974
#define IDS_SETTINGS_SITE_SETTINGS_DELETE 7975
#define IDS_SETTINGS_SITE_SETTINGS_THIRD_PARTY_COOKIES_EXCEPTION_LABEL 7976
#define IDS_SETTINGS_SITE_SETTINGS_DELETE_ALL_STORAGE_DIALOG_TITLE 7977
#define IDS_SETTINGS_SITE_SETTINGS_DELETE_DISPLAYED_STORAGE_DIALOG_TITLE 7978
#define IDS_SETTINGS_SITE_SETTINGS_RELATED_WEBSITE_SETS_LEARN_MORE 7979
#define IDS_SETTINGS_SITE_SETTINGS_RELATED_WEBSITE_SETS_LEARN_MORE_ACCESSIBILITY 7980
#define IDS_SETTINGS_SITE_SETTINGS_CLEAR_ALL_STORAGE_DESCRIPTION 7981
#define IDS_SETTINGS_SITE_SETTINGS_CLEAR_DISPLAYED_STORAGE_DESCRIPTION 7982
#define IDS_SETTINGS_SITE_SETTINGS_DELETE_ALL_STORAGE_LABEL 7983
#define IDS_SETTINGS_SITE_SETTINGS_DELETE_DISPLAYED_STORAGE_LABEL 7984
#define IDS_SETTINGS_SITE_SETTINGS_SITE_DETAILS_SUBPAGE_ACCESSIBILITY_LABEL 7985
#define IDS_SETTINGS_SITE_SETTINGS_RELATED_WEBSITE_SETS_MEMBERSHIP_LABEL 7986
#define IDS_SETTINGS_SITE_SETTINGS_RELATED_WEBSITE_SETS_MORE_ACTIONS_TITLE 7987
#define IDS_SETTINGS_SITE_SETTINGS_RELATED_WEBSITE_SETS_SHOW_RELATED_SITES_BUTTON 7988
#define IDS_SETTINGS_SITE_SETTINGS_RELATED_WEBSITE_SETS_SITE_DELETE_STORAGE_BUTTON 7989
#define IDS_SETTINGS_SITE_SETTINGS_DELETE_ALL_STORAGE_CONFIRMATION 7990
#define IDS_SETTINGS_SITE_SETTINGS_DELETE_DISPLAYED_STORAGE_CONFIRMATION 7991
#define IDS_SETTINGS_SITE_SETTINGS_DELETE_RWS_STORAGE_CONFIRMATION 7992
#define IDS_SETTINGS_SITE_SETTINGS_DELETE_ALL_STORAGE_CONFIRMATION_INSTALLED 7993
#define IDS_SETTINGS_SITE_SETTINGS_DELETE_DISPLAYED_STORAGE_CONFIRMATION_INSTALLED 7994
#define IDS_SETTINGS_SITE_SETTINGS_DELETE_RWS_STORAGE_CONFIRMATION_INSTALLED 7995
#define IDS_SETTINGS_SITE_SETTINGS_CLEAR_ALL_STORAGE_SIGN_OUT 7996
#define IDS_SETTINGS_SITE_SETTINGS_CLEAR_DISPLAYED_STORAGE_SIGN_OUT 7997
#define IDS_SETTINGS_SITE_SETTINGS_CLEAR_RWS_STORAGE_SIGN_OUT 7998
#define IDS_SETTINGS_SITE_SETTINGS_SITE_RESET_DIALOG_TITLE 7999
#define IDS_SETTINGS_SITE_SETTINGS_SITE_RESET_CONFIRMATION 8000
#define IDS_SETTINGS_SITE_SETTINGS_PARENT_APP_PERMISSION_EXPLANATION 8001
#define IDS_SETTINGS_SITE_SETTINGS_SUB_APP_PERMISSION_EXPLANATION 8002
#define IDS_SETTINGS_SITE_SETTINGS_SITE_DELETE_STORAGE_DIALOG_TITLE 8003
#define IDS_SETTINGS_SITE_SETTINGS_SITE_CLEAR_STORAGE_CONFIRMATION 8004
#define IDS_SETTINGS_SITE_SETTINGS_SITE_CLEAR_STORAGE_CONFIRMATION_NEW 8005
#define IDS_SETTINGS_SITE_SETTINGS_SITE_CLEAR_STORAGE_SIGN_OUT 8006
#define IDS_SETTINGS_SITE_SETTINGS_SITE_DELETE_STORAGE_OFFLINE_DATA 8007
#define IDS_SETTINGS_SITE_SETTINGS_REMOVE_SITE_AD_PERSONALIZATION 8008
#define IDS_SETTINGS_SITE_SETTINGS_SITE_GROUP_DELETE_OFFLINE_DATA 8009
#define IDS_SETTINGS_SITE_SETTINGS_REMOVE_SITE_ORIGIN_DIALOG_TITLE 8010
#define IDS_SETTINGS_SITE_SETTINGS_REMOVE_SITE_ORIGIN_APP_DIALOG_TITLE 8011
#define IDS_SETTINGS_SITE_SETTINGS_REMOVE_SITE_ORIGIN_PARTITIONED_DIALOG_TITLE 8012
#define IDS_SETTINGS_SITE_SETTINGS_REMOVE_SITE_GROUP_DIALOG_TITLE 8013
#define IDS_SETTINGS_SITE_SETTINGS_REMOVE_SITE_GROUP_APP_DIALOG_TITLE 8014
#define IDS_SETTINGS_SITE_SETTINGS_REMOVE_SITE_GROUP_APP_PLURAL_DIALOG_TITLE 8015
#define IDS_SETTINGS_SITE_SETTINGS_REMOVE_SITE_ORIGIN_LOGOUT 8016
#define IDS_SETTINGS_SITE_SETTINGS_REMOVE_SITE_GROUP_LOGOUT 8017
#define IDS_SETTINGS_SITE_SETTINGS_REMOVE_SITE_OFFLINE_DATA 8018
#define IDS_SETTINGS_SITE_SETTINGS_REMOVE_SITE_PERMISSIONS 8019
#define IDS_SETTINGS_SITE_SETTINGS_REMOVE_SITE_CONFIRM 8020
#define IDS_SETTINGS_SITE_SETTINGS_COOKIE_REMOVE_SITE 8021
#define IDS_SETTINGS_SITE_SETTINGS_SITE_RESET_ALL 8022
#define IDS_SETTINGS_SITE_SETTINGS_SITE_CLEAR_STORAGE 8023
#define IDS_SETTINGS_SITE_SETTINGS_HANDLER_IS_DEFAULT 8024
#define IDS_SETTINGS_SITE_SETTINGS_HANDLER_SET_DEFAULT 8025
#define IDS_SETTINGS_SITE_SETTINGS_REMOVE 8026
#define IDS_SETTINGS_SITE_SETTINGS_INCOGNITO_ONLY 8027
#define IDS_SETTINGS_SITE_SETTINGS_INCOGNITO_SITE_EXCEPTION_DESC 8028
#define IDS_SETTINGS_SITE_SETTINGS_NO_ZOOMED_SITES 8029
#define IDS_SETTINGS_SITE_NO_SITES_ADDED 8030
#define IDS_SETTINGS_SITE_SETTINGS_BLOCK_AUTOPLAY 8031
#define IDS_SETTINGS_SITE_SETTINGS_EMPTY_ALL_SITES_PAGE 8032
#define IDS_SETTINGS_SITE_SETTINGS_NO_SITES_FOUND 8033
#define IDS_SETTINGS_SITE_SETTINGS_BLUETOOTH_SCANNING_DESCRIPTION 8034
#define IDS_SETTINGS_SITE_SETTINGS_BLUETOOTH_SCANNING_ASK 8035
#define IDS_SETTINGS_SITE_SETTINGS_BLUETOOTH_SCANNING_BLOCK 8036
#define IDS_SETTINGS_SITE_SETTINGS_BLUETOOTH_SCANNING_ALLOWED_EXCEPTIONS 8037
#define IDS_SETTINGS_SITE_SETTINGS_BLUETOOTH_SCANNING_BLOCKED_EXCEPTIONS 8038
#define IDS_SETTINGS_SITE_SETTINGS_EXTENSION_ID_DESCRIPTION 8039
#define IDS_SETTINGS_SITE_SETTINGS_PAGE_SITE_DATA_ALLOWED_SUB_LABEL 8040
#define IDS_SETTINGS_SITE_SETTINGS_PAGE_SITE_DATA_BLOCKED_SUB_LABEL 8041
#define IDS_SETTINGS_SITE_SETTINGS_PAGE_SITE_DATA_DELETE_ON_EXIT_SUB_LABEL 8042
#define IDS_SETTINGS_EXPIRES_AFTER_TIME_LABEL 8043
#define IDS_SETTINGS_SITE_DATA_PAGE_TITLE 8044
#define IDS_SETTINGS_SITE_DATA_PAGE_DESCRIPTION 8045
#define IDS_SETTINGS_SITE_DATA_PAGE_DEFAULT_BEHAVIOR_HEADING 8046
#define IDS_SETTINGS_SITE_DATA_PAGE_DEFAULT_BEHAVIOR_DESCRIPTION 8047
#define IDS_SETTINGS_SITE_DATA_PAGE_ALLOW_RADIO_LABEL 8048
#define IDS_SETTINGS_SITE_DATA_PAGE_ALLOW_RADIO_SUB_LABEL 8049
#define IDS_SETTINGS_SITE_DATA_PAGE_CLEAR_ON_EXIT_RADIO_LABEL 8050
#define IDS_SETTINGS_SITE_DATA_PAGE_BLOCK_RADIO_LABEL 8051
#define IDS_SETTINGS_SITE_DATA_PAGE_BLOCK_RADIO_SUB_LABEL 8052
#define IDS_SETTINGS_SITE_DATA_PAGE_CUSTOMIZED_BEHAVIOR_HEADING 8053
#define IDS_SETTINGS_SITE_DATA_PAGE_CUSTOMIZED_BEHAVIOR_DESCRIPTION 8054
#define IDS_SETTINGS_SITE_DATA_PAGE_ALLOW_EXCEPTIONS_SUB_HEADING 8055
#define IDS_SETTINGS_SITE_DATA_PAGE_BLOCK_EXCEPTIONS_SUB_HEADING 8056
#define IDS_SETTINGS_SITE_DATA_PAGE_BLOCK_CONFIRM_DIALOG_TITLE 8057
#define IDS_SETTINGS_SITE_DATA_PAGE_BLOCK_CONFIRM_DIALOG_DESCRIPTION 8058
#define IDS_SETTINGS_SITE_DATA_PAGE_BLOCK_CONFIRM_DIALOG_CONFIRM_BUTTON 8059
#define IDS_SETTINGS_SITE_DATA_PAGE_BLOCK_CONFIRM_DIALOG_CANCEL_BUTTON 8060
#define IDS_SETTINGS_ADD_SITE_TO_ALLOW_LIST_LABEL 8061
#define IDS_SETTINGS_ADD_SITE_TO_BLOCK_LIST_LABEL 8062
#define IDS_SETTINGS_ADD_SITE_CONTEXT_MENU_LABEL 8063
#define IDS_SETTINGS_STORAGE_ACCESS_DESCRIPTION 8064
#define IDS_SETTINGS_STORAGE_ACCESS_ASK 8065
#define IDS_SETTINGS_STORAGE_ACCESS_BLOCKED 8066
#define IDS_SETTINGS_STORAGE_ACCESS_ALLOWED_EXCEPTIONS 8067
#define IDS_SETTINGS_STORAGE_ACCESS_BLOCKED_EXCEPTIONS 8068
#define IDS_SETTINGS_STORAGE_ACCESS_RESET_ALL 8069
#define IDS_SETTINGS_STORAGE_ACCESS_RESET_SITE 8070
#define IDS_SETTINGS_STORAGE_ACCESS_OPEN_EXPAND 8071
#define IDS_SETTINGS_STORAGE_ACCESS_CLOSE_EXPAND 8072
#define IDS_SETTINGS_STORAGE_ACCESS_ALLOWED_SITE_LABEL 8073
#define IDS_SETTINGS_STORAGE_ACCESS_BLOCKED_SITE_LABEL 8074
#define IDS_SETTINGS_NO_BLUETOOTH_DEVICES_FOUND 8075
#define IDS_SETTINGS_NO_USB_DEVICES_FOUND 8076
#define IDS_SETTINGS_NO_SERIAL_PORTS_FOUND 8077
#define IDS_SETTINGS_NO_HID_DEVICES_FOUND 8078
#define IDS_SETTINGS_RESET_BLUETOOTH_CONFIRMATION 8079
#define IDS_SETTINGS_RESET_HID_CONFIRMATION 8080
#define IDS_SETTINGS_RESET_SERIAL_PORTS_CONFIRMATION 8081
#define IDS_SETTINGS_RESET_USB_CONFIRMATION 8082
#define IDS_SETTINGS_ADD_SITE_TITLE 8083
#define IDS_SETTINGS_ADD_SITES_TITLE 8084
#define IDS_SETTINGS_EDIT_SITE_TITLE 8085
#define IDS_SETTINGS_ADD_SITE 8086
#define IDS_SETTINGS_SITE_SETTINGS_NUM_COOKIES 8087
#define IDS_SETTINGS_PEOPLE 8088
#define IDS_SETTINGS_CHANGE_PICTURE_PROFILE_PHOTO 8089
#define IDS_SETTINGS_PEOPLE_VERIFY_ACCOUNT_BUTTON 8090
#define IDS_SETTINGS_TURN_OFF_SYNC_MANAGED_PROFILE_EXPLANATION 8091
#define IDS_SETTINGS_TURN_OFF_SYNC_AND_SIGN_OUT_DIALOG_TITLE 8092
#define IDS_SETTINGS_TURN_OFF_SYNC_AND_SIGN_OUT_DIALOG_TITLE_SUPERVISED_PROFILE 8093
#define IDS_SETTINGS_TURN_OFF_SYNC_DIALOG_MANAGED_CONFIRM 8094
#define IDS_SETTINGS_TURN_OFF_SYNC_DIALOG_CHECKBOX 8095
#define IDS_SETTINGS_SYNC_SETTINGS_SAVED_TOAST_LABEL 8096
#define IDS_SETTINGS_CHROME_SIGNIN_OPTION_SAVED_TOAST_LABEL 8097
#define IDS_SETTINGS_PROFILE_NAME_INPUT_LABEL 8098
#define IDS_SETTINGS_PROFILE_SHORTCUT_TOGGLE_LABEL 8099
#define IDS_SETTINGS_CUSTOMIZE_PROFILE 8100
#define IDS_SETTINGS_PICK_A_THEME_COLOR 8101
#define IDS_SETTINGS_PICK_AN_AVATAR 8102
#define IDS_SETTINGS_CREATE_SHORTCUT 8103
#define IDS_SETTINGS_CREATE_SHORTCUT_SUBTITLE 8104
#define IDS_SETTINGS_ACCOUNT_PAGE_TITLE 8105
#define IDS_SETTINGS_ACCOUNT_DATATYPES_HEADING 8106
#define IDS_SETTINGS_ACCOUNT_BODY 8107
#define IDS_SETTINGS_ACCOUNT_SYNC_DISABLED 8108
#define IDS_SETTINGS_ACCOUNT_HISTORY_TOGGLE 8109
#define IDS_SETTINGS_ACCOUNT_HISTORY_TOGGLE_SUB_LABEL_OFF 8110
#define IDS_SETTINGS_GOOGLE_SERVICES_PAGE_TITLE 8111
#define IDS_SETTINGS_SYNC_DISCONNECT_EXPLANATION 8112
#define IDS_SETTINGS_SYNC_DISCONNECT_AND_SIGN_OUT_EXPLANATION 8113
#define IDS_SETTINGS_SYNC_DISCONNECT_AND_SIGN_OUT_EXPLANATION_SUPERVISED_PROFILE 8114
#define IDS_SETTINGS_SYNC_DISCONNECT_EXPAND_ACCESSIBILITY_LABEL 8115
#define IDS_SETTINGS_ENCRYPT_WITH_SYNC_PASSPHRASE_LABEL 8116
#define IDS_SETTINGS_PASSPHRASE_EXPLANATION_TEXT 8117
#define IDS_SETTINGS_PASSPHRASE_RESET_HINT_ENCRYPTION 8118
#define IDS_SETTINGS_PASSPHRASE_RECOVER 8119
#define IDS_SETTINGS_PERSONALIZE_GOOGLE_SERVICES_TITLE 8120
#define IDS_SETTINGS_PERSONALIZE_GOOGLE_SERVICES_TITLE_V2 8121
#define IDS_SETTINGS_PERSONALIZE_GOOGLE_SERVICES_WAA_TITLE 8122
#define IDS_SETTINGS_PERSONALIZE_GOOGLE_SERVICES_LINKED_SERVICES_TITLE 8123
#define IDS_SETTINGS_SIGNIN_CHROME_SIGNIN_OPTION_PLACEHOLDER 8124
#define IDS_SETTINGS_SIGNIN_CHROME_SIGNIN_OPTION_SIGNIN 8125
#define IDS_SETTINGS_SIGNIN_CHROME_SIGNIN_OPTION_DO_NOT_SIGNIN 8126
#define IDS_SETTINGS_SIGNIN_CHROME_SIGNIN_OPTION_ALWAYS_ASK 8127
#define IDS_SETTINGS_GOOGLE_ACCOUNT 8128
#define IDS_SETTINGS_MANAGE_GOOGLE_ACCOUNT 8129
#define IDS_SETTINGS_IMPORT_SETTINGS_TITLE 8131
#define IDS_SETTINGS_IMPORT_FROM_LABEL 8132
#define IDS_SETTINGS_IMPORT_ITEMS_LABEL 8133
#define IDS_SETTINGS_IMPORT_LOADING_PROFILES 8134
#define IDS_SETTINGS_IMPORT_HISTORY_CHECKBOX 8135
#define IDS_SETTINGS_IMPORT_FAVORITES_CHECKBOX 8136
#define IDS_SETTINGS_IMPORT_PASSWORDS_CHECKBOX 8137
#define IDS_SETTINGS_IMPORT_SEARCH_ENGINES_CHECKBOX 8138
#define IDS_SETTINGS_IMPORT_AUTOFILL_FORM_DATA_CHECKBOX 8139
#define IDS_SETTINGS_IMPORT_CHOOSE_FILE 8140
#define IDS_SETTINGS_IMPORT_COMMIT 8141
#define IDS_SETTINGS_IMPORT_SUCCESS 8142
#define IDS_SETTINGS_IMPORT_NO_PROFILE_FOUND 8143
#define IDS_SETTINGS_HISTORY_SEARCH_SETTING_LABEL 8144
#define IDS_SETTINGS_HISTORY_SEARCH_SETTING_SUBLABEL_V2 8145
#define IDS_SETTINGS_HISTORY_SEARCH_LEARN_MORE_A11Y_LABEL 8146
#define IDS_SETTINGS_HISTORY_SEARCH_WHEN_ON_PAGE_CONTENT 8147
#define IDS_SETTINGS_HISTORY_SEARCH_WHEN_ON_SEARCH_FROM 8148
#define IDS_SETTINGS_HISTORY_SEARCH_WHEN_ON_LOG_START 8149
#define IDS_SETTINGS_HISTORY_SEARCH_CONSIDER_DATA 8150
#define IDS_SETTINGS_HISTORY_SEARCH_CONSIDER_DATA_ENCRYPTED 8151
#define IDS_SETTINGS_HISTORY_SEARCH_SUBLABEL_ON 8152
#define IDS_SETTINGS_HISTORY_SEARCH_SUBLABEL_OFF 8153
#define IDS_SETTINGS_HISTORY_SEARCH_WITH_ANSWERS_SETTING_SUBLABEL_V2 8154
#define IDS_SETTINGS_HISTORY_SEARCH_WITH_ANSWERS_WHEN_ON_PAGE_CONTENT 8155
#define IDS_SETTINGS_HISTORY_SEARCH_WITH_ANSWERS_WHEN_ON_RECALL_INFO 8156
#define IDS_SETTINGS_HISTORY_SEARCH_WITH_ANSWERS_CONSIDER_DATA_ENCRYPTED 8157
#define IDS_SETTINGS_HISTORY_SEARCH_WITH_ANSWERS_CONSIDER_OUTDATED 8158
#define IDS_SETTINGS_HISTORY_SEARCH_WITH_ANSWERS_SUBLABEL_ON 8159
#define IDS_SETTINGS_HISTORY_SEARCH_WITH_ANSWERS_SUBLABEL_OFF 8160
#define IDS_SETTINGS_PAGE_ZOOM_LABEL 8161
#define IDS_SETTINGS_FONT_SIZE_LABEL 8162
#define IDS_SETTINGS_VERY_SMALL_FONT 8163
#define IDS_SETTINGS_SMALL_FONT 8164
#define IDS_SETTINGS_MEDIUM_FONT 8165
#define IDS_SETTINGS_LARGE_FONT 8166
#define IDS_SETTINGS_VERY_LARGE_FONT 8167
#define IDS_SETTINGS_CUSTOMIZE_FONTS 8168
#define IDS_SETTINGS_FONTS 8169
#define IDS_SETTINGS_STANDARD_FONT_LABEL 8170
#define IDS_SETTINGS_SERIF_FONT_LABEL 8171
#define IDS_SETTINGS_SANS_SERIF_FONT_LABEL 8172
#define IDS_SETTINGS_FIXED_WIDTH_FONT_LABEL 8173
#define IDS_SETTINGS_MATH_FONT_LABEL 8174
#define IDS_SETTINGS_MINIMUM_FONT_SIZE_LABEL 8175
#define IDS_SETTINGS_TINY_FONT_SIZE 8176
#define IDS_SETTINGS_HUGE_FONT_SIZE 8177
#define IDS_SETTINGS_QUICK_BROWN_FOX 8178
#define IDS_SETTINGS_SYSTEM 8179
#define IDS_SETTINGS_SYSTEM_HARDWARE_ACCELERATION_LABEL 8180
#define IDS_SETTINGS_SYSTEM_FEATURE_NOTIFICATIONS_LABEL 8183
#define IDS_SETTINGS_SYSTEM_PROXY_SETTINGS_LABEL 8184
#define IDS_SETTINGS_SYSTEM_PROXY_SETTINGS_EXTENSION_LABEL 8185
#define IDS_SETTINGS_SYSTEM_PROXY_SETTINGS_POLICY_LABEL 8186
#define IDS_SETTINGS_SYSTEM_PROXY_SETTINGS_YOUR_ORGANIZATION_LABEL 8187
#define IDS_SETTINGS_SYSTEM_PROXY_SETTINGS_YOUR_DEVICE_LABEL 8188
#define IDS_SETTINGS_SYSTEM_PROXY_SETTINGS_MULTIPLE_SOURCES_LABEL 8189
#define IDS_PAGE_NOT_AVAILABLE_FOR_GUEST_HEADING 8192
#define IDS_SETTINGS_SECURITY_KEYS_TITLE 8193
#define IDS_SETTINGS_SECURITY_KEYS_DESC 8194
#define IDS_SETTINGS_SECURITY_KEYS_SET_PIN 8195
#define IDS_SETTINGS_SECURITY_KEYS_SET_PIN_BUTTON 8196
#define IDS_SETTINGS_SECURITY_KEYS_SET_PIN_DESC 8197
#define IDS_SETTINGS_SECURITY_KEYS_SET_PIN_INITIAL_TITLE 8198
#define IDS_SETTINGS_SECURITY_KEYS_SET_PIN_CREATE_TITLE 8199
#define IDS_SETTINGS_SECURITY_KEYS_SET_PIN_CHANGE_TITLE 8200
#define IDS_SETTINGS_SECURITY_KEYS_RESET 8201
#define IDS_SETTINGS_SECURITY_KEYS_RESET_DESC 8202
#define IDS_SETTINGS_SECURITY_KEYS_RESET_TITLE 8203
#define IDS_SETTINGS_SECURITY_KEYS_RESET_CONFIRM_TITLE 8204
#define IDS_SETTINGS_SECURITY_KEYS_RESET_STEP1 8205
#define IDS_SETTINGS_SECURITY_KEYS_RESET_STEP2 8206
#define IDS_SETTINGS_SECURITY_KEYS_NO_RESET 8207
#define IDS_SETTINGS_SECURITY_KEYS_RESET_ERROR 8208
#define IDS_SETTINGS_SECURITY_KEYS_RESET_SUCCESS 8209
#define IDS_SETTINGS_SECURITY_KEYS_RESET_NOTALLOWED 8210
#define IDS_SETTINGS_SECURITY_KEYS_NO_PIN 8211
#define IDS_SETTINGS_SECURITY_KEYS_CURRENT_PIN_INTRO 8212
#define IDS_SETTINGS_SECURITY_KEYS_PIN_INCORRECT 8213
#define IDS_SETTINGS_SECURITY_KEYS_PIN_INCORRECT_RETRIES_SIN 8214
#define IDS_SETTINGS_SECURITY_KEYS_PIN_INCORRECT_RETRIES_PL 8215
#define IDS_SETTINGS_SECURITY_KEYS_SAME_PIN_AS_CURRENT 8216
#define IDS_SETTINGS_SECURITY_KEYS_NEW_PIN 8217
#define IDS_SETTINGS_SECURITY_KEYS_SET_PIN_CONFIRM 8218
#define IDS_SETTINGS_SECURITY_KEYS_CURRENT_PIN 8219
#define IDS_SETTINGS_SECURITY_KEYS_PIN 8220
#define IDS_SETTINGS_SECURITY_KEYS_PIN_ERROR_TOO_SHORT_SMALL 8221
#define IDS_SETTINGS_SECURITY_KEYS_PIN_ERROR_TOO_LONG 8222
#define IDS_SETTINGS_SECURITY_KEYS_PIN_ERROR_MISMATCH 8223
#define IDS_SETTINGS_SECURITY_KEYS_CONFIRM_PIN 8224
#define IDS_SETTINGS_SECURITY_KEYS_PIN_SUCCESS 8225
#define IDS_SETTINGS_SECURITY_KEYS_PIN_ERROR 8226
#define IDS_SETTINGS_SECURITY_KEYS_PIN_HARD_LOCK 8227
#define IDS_SETTINGS_SECURITY_KEYS_PIN_SOFT_LOCK 8228
#define IDS_SETTINGS_SECURITY_KEYS_SHOW_PINS 8229
#define IDS_SETTINGS_SECURITY_KEYS_HIDE_PINS 8230
#define IDS_SETTINGS_SECURITY_KEYS_CREDENTIAL_MANAGEMENT_LABEL 8231
#define IDS_SETTINGS_SECURITY_KEYS_CREDENTIAL_USERNAME_LABEL 8232
#define IDS_SETTINGS_SECURITY_KEYS_CREDENTIAL_DISPLAYNAME_LABEL 8233
#define IDS_SETTINGS_SECURITY_KEYS_CREDENTIAL_WEBSITE_LABEL 8234
#define IDS_SETTINGS_SECURITY_KEYS_CREDENTIAL_MANAGEMENT_DESC 8235
#define IDS_SETTINGS_SECURITY_KEYS_CREDENTIAL_MANAGEMENT_DIALOG_TITLE 8236
#define IDS_SETTINGS_SECURITY_KEYS_UPDATE_CREDENTIAL_DIALOG_TITLE 8237
#define IDS_SETTINGS_SECURITY_KEYS_CREDENTIAL_MANAGEMENT_CONFIRM_DELETE_TITLE 8238
#define IDS_SETTINGS_SECURITY_KEYS_CREDENTIAL_MANAGEMENT_CONFIRM_DELETE_CREDENTIAL 8239
#define IDS_SETTINGS_SECURITY_KEYS_INPUT_ERROR_TOO_LONG 8240
#define IDS_SETTINGS_SECURITY_KEYS_CREDENTIAL_MANAGEMENT_NO_CREDENTIALS 8241
#define IDS_SETTINGS_SECURITY_KEYS_NO_CREDENTIAL_MANAGEMENT 8242
#define IDS_SETTINGS_SECURITY_KEYS_CREDENTIAL_MANAGEMENT_REMOVED 8243
#define IDS_SETTINGS_SECURITY_KEYS_CREDENTIAL_MANAGEMENT_NO_PIN 8244
#define IDS_SETTINGS_SECURITY_KEYS_CREDENTIAL_MANAGEMENT_ERROR 8245
#define IDS_SETTINGS_SECURITY_KEYS_CREDENTIAL_MANAGEMENT_DELETE_SUCCESS 8246
#define IDS_SETTINGS_SECURITY_KEYS_CREDENTIAL_MANAGEMENT_DELETE_FAILED 8247
#define IDS_SETTINGS_SECURITY_KEYS_CREDENTIAL_MANAGEMENT_UPDATE_SUCCESS 8248
#define IDS_SETTINGS_SECURITY_KEYS_CREDENTIAL_MANAGEMENT_UPDATE_FAILED 8249
#define IDS_SETTINGS_SECURITY_KEYS_BIO_ENROLLMENT_SUBPAGE_LABEL 8250
#define IDS_SETTINGS_SECURITY_KEYS_BIO_ENROLLMENT_SUBPAGE_DESCRIPTION 8251
#define IDS_SETTINGS_SECURITY_KEYS_BIO_ENROLLMENT_DIALOG_TITLE 8252
#define IDS_SETTINGS_SECURITY_KEYS_BIO_ENROLLMENT_ADD_TITLE 8253
#define IDS_SETTINGS_SECURITY_KEYS_BIO_CHOOSE_NAME 8254
#define IDS_SETTINGS_SECURITY_KEYS_BIO_NAME_LABEL 8255
#define IDS_SETTINGS_SECURITY_KEYS_BIO_NAME_LABEL_TOO_LONG 8256
#define IDS_SETTINGS_SECURITY_KEYS_BIO_ENROLLMENT_NO_ENROLLMENTS_LABEL 8257
#define IDS_SETTINGS_SECURITY_KEYS_BIO_ENROLLMENT_ENROLLMENTS_LABEL 8258
#define IDS_SETTINGS_SECURITY_KEYS_BIO_ENROLLMENT_ENROLLING_LABEL 8259
#define IDS_SETTINGS_SECURITY_KEYS_BIO_ENROLLMENT_TRY_AGAIN_LABEL 8260
#define IDS_SETTINGS_SECURITY_KEYS_BIO_ENROLLMENT_FAILED_LABEL 8261
#define IDS_SETTINGS_SECURITY_KEYS_BIO_ENROLLMENT_ENROLLING_COMPLETE_LABEL 8262
#define IDS_SETTINGS_SECURITY_KEYS_NO_BIOMETRIC_ENROLLMENT 8263
#define IDS_SETTINGS_SECURITY_KEYS_BIO_ENROLLMENT_DELETE 8264
#define IDS_SETTINGS_SECURITY_KEYS_BIO_NO_PIN 8265
#define IDS_SETTINGS_SECURITY_KEYS_BIO_ENROLLMENT_STORAGE_FULL 8266
#define IDS_SETTINGS_SECURITY_KEYS_TOUCH_TO_CONTINUE 8267
#define IDS_SETTINGS_SECURITY_KEYS_PIN_PROMPT 8268
#define IDS_SETTINGS_SECURITY_KEYS_FORCE_PIN_CHANGE 8269
#define IDS_SETTINGS_EXTENSION_OR_APP_DISPLAY_NAME 8270
#define IDS_SETTINGS_SAFETY_HUB 8271
#define IDS_SETTINGS_SAFETY_HUB_ENTRY_POINT_BUTTON_LABEL 8272
#define IDS_SETTINGS_SAFETY_HUB_PAGE_CARD_SECTION_HEADER 8273
#define IDS_SETTINGS_SAFETY_HUB_PAGE_MODULE_SECTION_HEADER 8274
#define IDS_SETTINGS_SAFETY_HUB_EMPTY_STATE_MODULE_HEADER 8275
#define IDS_SETTINGS_SAFETY_HUB_PASSWORD_CHECK_SUBHEADER_RECENTLY 8276
#define IDS_SETTINGS_SAFETY_HUB_PASSWORD_CHECK_SUBHEADER_SOME_TIME_AGO 8277
#define IDS_SETTINGS_SAFETY_HUB_PASSWORD_CHECK_HEADER_NO_WEAK_OR_REUSED 8278
#define IDS_SETTINGS_SAFETY_HUB_PASSWORD_CHECK_SUBHEADER_SIGN_IN 8279
#define IDS_SETTINGS_SAFETY_HUB_PASSWORD_CHECK_SUBHEADER_GO_TO_PASSWORD_MANAGER 8280
#define IDS_SETTINGS_SAFETY_HUB_PASSWORD_CHECK_HEADER_NO_PASSWORDS 8281
#define IDS_SETTINGS_SAFETY_HUB_PASSWORD_CHECK_SUBHEADER_NO_PASSWORDS_POLICY 8282
#define IDS_SETTINGS_SAFETY_HUB_PASSWORD_NAVIGATION_ARIA_LABEL 8283
#define IDS_SETTINGS_SAFETY_HUB_SB_ON_ENHANCED_HEADER 8284
#define IDS_SETTINGS_SAFETY_HUB_SB_ON_ENHANCED_SUBHEADER 8285
#define IDS_SETTINGS_SAFETY_HUB_SB_ON_STANDARD_HEADER 8286
#define IDS_SETTINGS_SAFETY_HUB_SB_ON_STANDARD_SUBHEADER 8287
#define IDS_SETTINGS_SAFETY_HUB_SB_OFF_HEADER 8288
#define IDS_SETTINGS_SAFETY_HUB_SB_OFF_MANAGED_SUBHEADER 8289
#define IDS_SETTINGS_SAFETY_HUB_SB_OFF_EXTENSION_SUBHEADER 8290
#define IDS_SETTINGS_SAFETY_HUB_SB_OFF_USER_SUBHEADER 8291
#define IDS_SETTINGS_SAFETY_HUB_SB_NAVIGATION_ARIA_LABEL 8292
#define IDS_SETTINGS_SAFETY_HUB_GO_SITE_SETTINGS_ITEM 8293
#define IDS_SETTINGS_SAFETY_HUB_GO_NOTIFICATION_SETTINGS_ITEM 8294
#define IDS_SETTINGS_SAFETY_HUB_USER_EDU_INCOGNITO_SUBHEADER 8295
#define IDS_SETTINGS_SAFETY_HUB_USER_EDU_SAFE_BROWSING_HEADER 8296
#define IDS_SETTINGS_SAFETY_HUB_VERSION_CARD_SUBHEADER_RESTART 8297
#define IDS_SETTINGS_SAFETY_HUB_NOTIFICATION_PERMISSIONS_PRIMARY_LABEL 8298
#define IDS_SETTINGS_SAFETY_HUB_NOTIFICATION_PERMISSIONS_SECONDARY_LABEL 8299
#define IDS_SETTINGS_SAFETY_HUB_REVOKED_PERMISSIONS_MENU_NOTIFICATION 8300
#define IDS_SETTINGS_SAFETY_HUB_REVIEW_NOTIFICATION_PERMISSIONS_MENU_NOTIFICATION 8301
#define IDS_SETTINGS_SAFETY_HUB_COMPROMISED_PASSWORDS_MENU_NOTIFICATION 8302
#define IDS_SETTINGS_SAFETY_HUB_EXTENSIONS_MENU_NOTIFICATION 8303
#define IDS_SETTINGS_SAFETY_HUB_SAFE_BROWSING_MENU_NOTIFICATION 8304
#define IDS_SETTINGS_SAFETY_HUB_PASSWORDS_MODULE_NAME 8305
#define IDS_SETTINGS_SAFETY_HUB_SAFE_BROWSING_MODULE_NAME 8306
#define IDS_SETTINGS_SAFETY_HUB_EXTENSIONS_MODULE_UPPERCASE_NAME 8307
#define IDS_SETTINGS_SAFETY_HUB_EXTENSIONS_MODULE_LOWERCASE_NAME 8308
#define IDS_SETTINGS_SAFETY_HUB_NOTIFICATIONS_MODULE_UPPERCASE_NAME 8309
#define IDS_SETTINGS_SAFETY_HUB_NOTIFICATIONS_MODULE_LOWERCASE_NAME 8310
#define IDS_SETTINGS_SAFETY_HUB_PERMISSIONS_MODULE_UPPERCASE_NAME 8311
#define IDS_SETTINGS_SAFETY_HUB_PERMISSIONS_MODULE_LOWERCASE_NAME 8312
#define IDS_SETTINGS_SAFETY_HUB_MODULE_NAME_SEPARATOR 8313
#define IDS_SETTINGS_AI_INNOVATIONS_PAGE_TITLE 8314
#define IDS_SETTINGS_AI_PAGE_TITLE 8315
#define IDS_SETTINGS_AI_PAGE_MAIN_SUBLABEL_3 8316
#define IDS_SETTINGS_AI_PAGE_MAIN_MANAGED_LEARN_MORE_ACCESSIBILITY_LABEL 8317
#define IDS_SETTINGS_AI_SUBPAGE_SUBLABEL_AI 8318
#define IDS_SETTINGS_AI_SUBPAGE_SUBLABEL_REVIEWERS 8319
#define IDS_SETTINGS_AI_SUBPAGE_SUBLABEL_LOGGING_MANAGED_DISABLED 8320
#define IDS_SETTINGS_AI_SUBPAGE_FEATURE_MANAGED_DISABLED_LABEL 8321
#define IDS_SETTINGS_PASSWORD_CHANGE_LABEL 8322
#define IDS_SETTINGS_PASSWORD_CHANGE_SUBLABEL 8323
#define IDS_SETTINGS_PASSWORD_CHANGE_DATA_BREACH 8324
#define IDS_SETTINGS_PASSWORD_CHANGE_WHERE_SAVED 8325
#define IDS_SETTINGS_PASSWORD_CHANGE_EXPERIMENTAL 8326
#define IDS_SETTINGS_PASSWORD_CHANGE_CONTENT 8327
#define IDS_SETTINGS_PASSWORD_CHANGE_ENCRYPTION 8328
#define IDS_SETTINGS_SAFEBROWSING_TURN_ON_ENHANCED_TOAST_BUTTON 8329
#define IDS_SETTINGS_SAFEBROWSING_ENHANCED_ON_TOAST_MESSAGE 8330
#define IDS_SETTINGS_SAFEBROWSING_ENHANCED_OFF_TOAST_MESSAGE 8331
#define IDS_SYNC_ERROR_BOOKMARKS_LIMIT_EXCEEDED_DESCRIPTION 8332
#define IDS_SETTINGS_AI_PAGE_MAIN_TITLE 8333
#define IDS_SETTINGS_AI_PAGE_MAIN_SUBLABEL_1 8334
#define IDS_SETTINGS_AI_PAGE_MAIN_SUBLABEL_2 8335
#define IDS_SETTINGS_AI_PAGE_MAIN_MANAGED_SUBLABEL_3 8336
#define IDS_SETTINGS_CPU_PERFORMANCE_OVERRIDE_TITLE 8337
#define IDS_SETTINGS_CPU_PERFORMANCE_OVERRIDE_DESCRIPTION 8338
#define IDS_SETTINGS_CPU_PERFORMANCE_NOMINAL_TIER 8339
#define IDS_SETTINGS_CPU_PERFORMANCE_MODEL 8340
#define IDS_SETTINGS_CPU_PERFORMANCE_CORES 8341
#define IDS_SETTINGS_CPU_PERFORMANCE_TIER_DEFAULT 8342
#define IDS_SETTINGS_CPU_PERFORMANCE_TIER_UNKNOWN 8343
#define IDS_SETTINGS_CPU_PERFORMANCE_TIER_LOW 8344
#define IDS_SETTINGS_CPU_PERFORMANCE_TIER_MID 8345
#define IDS_SETTINGS_CPU_PERFORMANCE_TIER_HIGH 8346
#define IDS_SETTINGS_CPU_PERFORMANCE_TIER_ULTRA 8347
#define IDS_SETTINGS_SUBPAGE_BACK_BUTTON_ARIA_LABEL 8348
#define IDS_SETTINGS_SUBPAGE_BACK_BUTTON_ARIA_ROLE_DESCRIPTION 8349
#define IDS_SETTINGS_SUBPAGE_LEARN_MORE_ARIA_LABEL 8350
#define IDS_SETTINGS_CAPTIONS 8351
#define IDS_SETTINGS_CAPTIONS_PREFERENCES_TITLE 8352
#define IDS_SETTINGS_CAPTIONS_PREFERENCES_SUBTITLE 8353
#define IDS_SETTINGS_CAPTIONS_TEXT_SIZE 8354
#define IDS_SETTINGS_CAPTIONS_TEXT_FONT 8355
#define IDS_SETTINGS_CAPTIONS_TEXT_COLOR 8356
#define IDS_SETTINGS_CAPTIONS_TEXT_OPACITY 8357
#define IDS_SETTINGS_CAPTIONS_BACKGROUND_OPACITY 8358
#define IDS_SETTINGS_CAPTIONS_OPACITY_OPAQUE 8359
#define IDS_SETTINGS_CAPTIONS_OPACITY_SEMI_TRANSPARENT 8360
#define IDS_SETTINGS_CAPTIONS_OPACITY_TRANSPARENT 8361
#define IDS_SETTINGS_CAPTIONS_TEXT_SHADOW 8362
#define IDS_SETTINGS_CAPTIONS_TEXT_SHADOW_NONE 8363
#define IDS_SETTINGS_CAPTIONS_TEXT_SHADOW_RAISED 8364
#define IDS_SETTINGS_CAPTIONS_TEXT_SHADOW_DEPRESSED 8365
#define IDS_SETTINGS_CAPTIONS_TEXT_SHADOW_UNIFORM 8366
#define IDS_SETTINGS_CAPTIONS_TEXT_SHADOW_DROP_SHADOW 8367
#define IDS_SETTINGS_CAPTIONS_BACKGROUND_COLOR 8368
#define IDS_SETTINGS_CAPTIONS_COLOR_BLACK 8369
#define IDS_SETTINGS_CAPTIONS_COLOR_WHITE 8370
#define IDS_SETTINGS_CAPTIONS_COLOR_RED 8371
#define IDS_SETTINGS_CAPTIONS_COLOR_GREEN 8372
#define IDS_SETTINGS_CAPTIONS_COLOR_BLUE 8373
#define IDS_SETTINGS_CAPTIONS_COLOR_YELLOW 8374
#define IDS_SETTINGS_CAPTIONS_COLOR_CYAN 8375
#define IDS_SETTINGS_CAPTIONS_COLOR_MAGENTA 8376
#define IDS_SETTINGS_CAPTIONS_DEFAULT_SETTING 8377
#define IDS_SETTINGS_CAPTIONS_LANGUAGE 8378
#define IDS_SETTINGS_CAPTIONS_MANAGE_LANGUAGES_TITLE 8379
#define IDS_SETTINGS_CAPTIONS_MANAGE_LANGUAGES_SUBTITLE 8380
#define IDS_SETTINGS_CAPTIONS_MANAGE_LANGUAGES_SUBTITLE_LIVE_CAPTION_ONLY 8381
#define IDS_SETTINGS_CAPTIONS_LIVE_TRANSLATE_TARGET_LANGUAGE 8382
#define IDS_SETTINGS_CAPTIONS_LIVE_TRANSLATE_TARGET_LANGUAGE_SUBTITLE 8383
#define IDS_SETTINGS_CAPTIONS_REMOVE_LANGUAGE_LABEL 8384
#define IDS_SETTINGS_CAPTIONS_MAKE_DEFAULT_LANGUAGE_LABEL 8385
#define IDS_SETTINGS_CAPTIONS_DEFAULT_LANGUAGE_LABEL 8386
#define IDS_SETTINGS_NEARBY_SHARE_TITLE 8387
#define IDS_SETTINGS_NEARBY_SHARE_TITLE_PH 8388
#define IDS_SETTINGS_NEARBY_SHARE_SET_UP_BUTTON_TITLE 8389
#define IDS_SETTINGS_NEARBY_SHARE_DEVICE_NAME_ROW_TITLE 8390
#define IDS_SETTINGS_NEARBY_SHARE_DEVICE_NAME_DIALOG_TITLE 8391
#define IDS_SETTINGS_NEARBY_SHARE_DEVICE_NAME_FIELD_LABEL 8392
#define IDS_SETTINGS_NEARBY_SHARE_EDIT_DEVICE_NAME 8393
#define IDS_SETTINGS_NEARBY_SHARE_FAST_INITIATION_NOTIFICATION_TOGGLE_TITLE 8394
#define IDS_SETTINGS_NEARBY_SHARE_FAST_INITIATION_NOTIFICATION_TOGGLE_DESCRIPTION 8395
#define IDS_SETTINGS_NEARBY_SHARE_FAST_INITIATION_NOTIFICATION_TOGGLE_ARIA_LABEL 8396
#define IDS_SETTINGS_NEARBY_SHARE_DEVICE_NAME_ARIA_DESCRIPTION 8397
#define IDS_SETTINGS_NEARBY_SHARE_CONFIRM_DEVICE_NAME 8398
#define IDS_SETTINGS_NEARBY_SHARE_MANAGE_CONTACTS_LABEL 8399
#define IDS_SETTINGS_NEARBY_SHARE_MANAGE_CONTACTS_ROW_TITLE 8400
#define IDS_SETTINGS_NEARBY_SHARE_EDIT_DATA_USAGE 8401
#define IDS_SETTINGS_NEARBY_SHARE_UPDATE_DATA_USAGE 8402
#define IDS_SETTINGS_NEARBY_SHARE_DATA_USAGE_DIALOG_TITLE 8403
#define IDS_SETTINGS_NEARBY_SHARE_DATA_USAGE_WIFI_ONLY_LABEL 8404
#define IDS_SETTINGS_NEARBY_SHARE_DATA_USAGE_WIFI_ONLY_DESCRIPTION 8405
#define IDS_SETTINGS_NEARBY_SHARE_DATA_USAGE_MOBILE_DATA_LABEL 8406
#define IDS_SETTINGS_NEARBY_SHARE_DATA_USAGE_MOBILE_DATA_DESCRIPTION 8407
#define IDS_SETTINGS_NEARBY_SHARE_DATA_USAGE_MOBILE_DATA_TOOLTIP 8408
#define IDS_SETTINGS_NEARBY_SHARE_DATA_USAGE_OFFLINE_LABEL 8409
#define IDS_SETTINGS_NEARBY_SHARE_DATA_USAGE_OFFLINE_DESCRIPTION 8410
#define IDS_SETTINGS_NEARBY_SHARE_DATA_USAGE_EDIT_BUTTON_DATA_DESCRIPTION 8411
#define IDS_SETTINGS_NEARBY_SHARE_DATA_USAGE_EDIT_BUTTON_WIFI_ONLY_DESCRIPTION 8412
#define IDS_SETTINGS_NEARBY_SHARE_DATA_USAGE_EDIT_BUTTON_OFFLINE_DESCRIPTION 8413
#define IDS_SETTINGS_NEARBY_SHARE_CONTACT_VISIBILITY_ROW_TITLE 8414
#define IDS_SETTINGS_NEARBY_SHARE_EDIT_VISIBILITY 8415
#define IDS_SETTINGS_NEARBY_SHARE_VISIBILITY_DIALOG_TITLE 8416
#define IDS_SETTINGS_NEARBY_SHARE_VISIBILITY_DIALOG_SAVE 8417
#define IDS_SETTINGS_NEARBY_SHARE_DESCRIPTION 8418
#define IDS_SETTINGS_NEARBY_SHARE_HIGH_VISIBILITY_TITLE 8419
#define IDS_SETTINGS_NEARBY_SHARE_HIGH_VISIBILITY_ON 8420
#define IDS_SETTINGS_NEARBY_SHARE_HIGH_VISIBILITY_OFF 8421
#define IDS_SETTINGS_ENABLE_URL_KEYED_ANONYMIZED_DATA_COLLECTION 8422
#define IDS_SETTINGS_ENABLE_URL_KEYED_ANONYMIZED_DATA_COLLECTION_DESC 8423
#define IDS_SETTINGS_SPELLING_PREF 8424
#define IDS_SETTINGS_ENABLE_LOGGING_PREF 8425
#define IDS_SETTINGS_ENABLE_LOGGING_PREF_DESC 8426
#define IDS_SETTINGS_SUGGEST_PREF 8427
#define IDS_SETTINGS_PEOPLE_SIGNED_IN_TO_ACCOUNT 8428
#define IDS_SETTINGS_PEOPLE_SYNCING_TO_ACCOUNT 8429
#define IDS_SETTINGS_PEOPLE_SYNC_PAUSED 8430
#define IDS_SETTINGS_PEOPLE_SIGN_IN_PROMPT 8431
#define IDS_SETTINGS_PEOPLE_SYNC_TURN_OFF 8432
#define IDS_SETTINGS_SETTINGS_CHECKBOX_LABEL 8433
#define IDS_SETTINGS_PEOPLE_SYNC_NOT_WORKING 8434
#define IDS_SETTINGS_PEOPLE_SYNC_PASSWORDS_NOT_WORKING 8435
#define IDS_SETTINGS_SYNC_ADVANCED_PAGE_TITLE 8436
#define IDS_SETTINGS_NEW_SYNC_ADVANCED_PAGE_TITLE 8437
#define IDS_SETTINGS_PEOPLE_SYNC_ANOTHER_ACCOUNT 8438
#define IDS_SETTINGS_TURN_ON_SYNC_BUTTON_UPDATE_SETTINGS_UI_ENABLED 8439
#define IDS_SETTINGS_ACCOUNT_AWARE_SIGNIN_BUTTON_LABEL 8440
#define IDS_SETTINGS_ERROR_PASSWORDS_USER_ERROR_DESCRIPTION 8441
#define IDS_SETTINGS_ERROR_RECOVERABILITY_DEGRADED_FOR_PASSWORDS_USER_ERROR_DESCRIPTION 8442
#define IDS_SETTINGS_PEOPLE_ACCOUNT_AWARE_SIGNIN_ACCOUNT_ROW_SUBTITLE_WITH_EMAIL 8443
#define IDS_SETTINGS_PEOPLE_ACCOUNT_AWARE_SIGNIN_ACCOUNT_ROW_SUBTITLE_WITH_EMAIL_WITH_BOOKMARKS 8444
#define IDS_SETTINGS_PEOPLE_ACCOUNT_AWARE_SIGNIN_ACCOUNT_ROW_SUBTITLE_WITH_EMAIL_SIMPLE 8445
#define IDS_SETTINGS_CHANGE_ACCOUNT 8446
#define IDS_SETTINGS_SYNC_DISCONNECT_CONFIRM 8447
#define IDS_SETTINGS_PEOPLE_SIGN_OUT 8448
#define IDS_SETTINGS_AUTOFILL_CHECKBOX_LABEL 8449
#define IDS_SETTINGS_HISTORY_CHECKBOX_LABEL 8450
#define IDS_SETTINGS_EXTENSIONS_CHECKBOX_LABEL 8451
#define IDS_SETTINGS_OPEN_TABS_CHECKBOX_LABEL 8452
#define IDS_SETTINGS_SAVED_TAB_GROUPS_CHECKBOX_LABEL 8453
#define IDS_SETTINGS_PRODUCT_COMPARISONS_CHECKBOX_LABEL 8454
#define IDS_SETTINGS_WIFI_CONFIGURATIONS_CHECKBOX_LABEL 8455
#define IDS_SETTINGS_SYNC_EVERYTHING_CHECKBOX_LABEL 8456
#define IDS_SETTINGS_APPS_CHECKBOX_LABEL 8457
#define IDS_SETTINGS_APPS_CHECKBOX_SUBLABEL 8458
#define IDS_SETTINGS_NON_PERSONALIZED_SERVICES_SECTION_LABEL 8459
#define IDS_SETTINGS_CUSTOMIZE_SYNC 8460
#define IDS_SETTINGS_SYNC_DATA 8461
#define IDS_SETTINGS_PASSWORDS_AND_PASSKEYS_CHECKBOX_LABEL 8462
#define IDS_SETTINGS_PASSPHRASE_PLACEHOLDER 8463
#define IDS_SETTINGS_EXISTING_PASSPHRASE_TITLE 8464
#define IDS_SETTINGS_SUBMIT_PASSPHRASE 8465
#define IDS_SETTINGS_ENCRYPT_WITH_GOOGLE_CREDENTIALS_LABEL 8466
#define IDS_SETTINGS_ENCRYPT_ACCOUNT_DATA_WITH_GOOGLE_CREDENTIALS_LABEL 8467
#define IDS_SETTINGS_BOOKMARKS_CHECKBOX_LABEL 8468
#define IDS_SETTINGS_READING_LIST_CHECKBOX_LABEL 8469
#define IDS_SETTINGS_ENCRYPTION_OPTIONS 8470
#define IDS_SETTINGS_MISMATCHED_PASSPHRASE_ERROR 8471
#define IDS_SETTINGS_EMPTY_PASSPHRASE_ERROR 8472
#define IDS_SETTINGS_INCORRECT_PASSPHRASE_ERROR 8473
#define IDS_SETTINGS_NEW_MANAGE_SYNCED_DATA_TITLE_UNIFIED_CONSENT 8474
#define IDS_SETTINGS_NEW_SYNC_ADVANCED_BROWSER_PAGE_TITLE 8475
#define IDS_SETTINGS_SYNC_SYNC_AND_NON_PERSONALIZED_SERVICES 8476
#define IDS_SETTINGS_PASSPHRASE_CONFIRMATION_PLACEHOLDER 8477
#define IDS_SETTINGS_SYNC_LOADING 8478
#define IDS_SETTINGS_SYNC_TIMEOUT 8479
#define IDS_SETTINGS_SYNC 8480
#define IDS_SETTINGS_SYNC_SETTINGS_CANCEL_SYNC 8481
#define IDS_SETTINGS_SYNC_SETUP_CANCEL_DIALOG_TITLE 8482
#define IDS_SETTINGS_SYNC_SETUP_CANCEL_DIALOG_BODY 8483
#define IDS_SYNC_UNAVAILABLE_FOR_NON_GOOGLE_ACCOUNT 8484
#define IDS_SETTINGS_ABOUT_UPGRADE_CHECK_STARTED 8487
#define IDS_SETTINGS_UPDATE_TO_ROLLBACK_VERSION_DISALLOWED 8488
#define IDS_SETTINGS_THEME_CHECKBOX_LABEL 8490
#define IDS_SETTINGS_SKILLS_SETTING_LABEL 8494
#define IDS_SETTINGS_SKILLS_SETTING_SUBLABEL 8495
#define IDS_SETTINGS_SKILLS_TOGGLE_LABEL 8496
#define IDS_SETTINGS_SKILLS_GALLERY_HEADER 8497
#define IDS_SETTINGS_SKILLS_GALLERY_LABEL 8498
#define IDS_SETTINGS_CERTIFICATE_MANAGER_V2_TITLE 8499
#define IDS_SETTINGS_CERTIFICATE_MANAGER_V2_CLIENT_CERTIFICATES 8500
#define IDS_SETTINGS_CERTIFICATE_MANAGER_V2_LOCAL_CERTIFICATES 8501
#define IDS_SETTINGS_CERTIFICATE_MANAGER_V2_CRS_CERTIFICATES 8502
#define IDS_SETTINGS_CERTIFICATE_MANAGER_V2_HASH_COPIED_TOAST 8503
#define IDS_SETTINGS_CERTIFICATE_MANAGER_V2_PLATFORM 8504
#define IDS_SETTINGS_CERTIFICATE_MANAGER_V2_PLATFORM_CERTS_TITLE 8505
#define IDS_SETTINGS_CERTIFICATE_MANAGER_V2_PLATFORM_CERTS_VIEW_LINK 8507
#define IDS_SETTINGS_CERTIFICATE_MANAGER_V2_ADMIN_CERTS_TITLE 8509
#define IDS_SETTINGS_CERTIFICATE_MANAGER_V2_USER_CERTS_TITLE 8510
#define IDS_SETTINGS_CERTIFICATE_MANAGER_V2_TRUSTED_CERTS_LIST 8511
#define IDS_SETTINGS_CERTIFICATE_MANAGER_V2_INTERMEDIATE_CERTS_LIST 8512
#define IDS_SETTINGS_CERTIFICATE_MANAGER_V2_DISTRUSTED_CERTS_LIST 8513
#define IDS_SETTINGS_CERTIFICATE_MANAGER_V2_NO_CERTIFICATES_ROW 8514
#define IDS_SETTINGS_CERTIFICATE_MANAGER_V2_EXPORT_BUTTON_LABEL 8515
#define IDS_SETTINGS_CERTIFICATE_MANAGER_V2_EXPORT_BUTTON_ARIA_LABEL 8516
#define IDS_SETTINGS_CERTIFICATE_MANAGER_V2_IMPORT_BUTTON_LABEL 8517
#define IDS_SETTINGS_CERTIFICATE_MANAGER_V2_IMPORT_BUTTON_ARIA_LABEL 8518
#define IDS_SETTINGS_CERTIFICATE_MANAGER_V2_IMPORT_AND_BIND_BUTTON_LABEL 8519
#define IDS_SETTINGS_CERTIFICATE_MANAGER_V2_IMPORT_AND_BIND_BUTTON_ARIA_LABEL 8520
#define IDS_SETTINGS_CERTIFICATE_MANAGER_V2_NUM_CERTS 8521
#define IDS_SETTINGS_CERTIFICATE_MANAGER_V2_CLIENT_CERTIFICATES_DESCRIPTION 8522
#define IDS_SETTINGS_CERTIFICATE_MANAGER_V2_LOCAL_CERTIFICATES_DESCRIPTION 8523
#define IDS_SETTINGS_CERTIFICATE_MANAGER_V2_CRS_CERTIFICATES_DESCRIPTION 8524
#define IDS_SETTINGS_CERTIFICATE_MANAGER_V2_CRS_LEARN_MORE_LINK 8525
#define IDS_SETTINGS_CERTIFICATE_MANAGER_V2_CRS_LEARN_MORE_LINK_ARIA_LABEL 8526
#define IDS_SETTINGS_CERTIFICATE_MANAGER_V2_PLATFORM_CERTS_TOGGLE_LABEL 8527
#define IDS_SETTINGS_CERTIFICATE_MANAGER_V2_CUSTOM_CERTS_TITLE 8528
#define IDS_SETTINGS_CERTIFICATE_MANAGER_V2_CLIENT_CERTIFICATES_FROM_PLATFORM 8529
#define IDS_SETTINGS_CERTIFICATE_MANAGER_V2_CLIENT_CERTIFICATES_FROM_ADMIN 8530
#define IDS_SETTINGS_CERTIFICATE_MANAGER_V2_CLIENT_CERTIFICATES_FROM_EXTENSION 8531
#define IDS_SETTINGS_CERTIFICATE_MANAGER_V2_DELETE_CERT_TITLE 8532
#define IDS_SETTINGS_CERTIFICATE_MANAGER_V2_DELETE_CLIENT_CERT_DESCRIPTION 8533
#define IDS_SETTINGS_CERTIFICATE_MANAGER_V2_DELETE_SERVER_CERT_DESCRIPTION 8534
#define IDS_SETTINGS_CERTIFICATE_MANAGER_V2_READ_FILE_ERROR 8535
#define IDS_SETTINGS_CERTIFICATE_MANAGER_V2_MULTIPLE_CERT_ERROR 8536
#define IDS_SETTINGS_CERTIFICATE_MANAGER_V2_DELETE_ERROR_TITLE 8537
#define IDS_SETTINGS_CERTIFICATE_MANAGER_V2_DELETE_ERROR 8538
#define IDS_SETTINGS_CERTIFICATE_MANAGER_V2_ENTER_PASSWORD_TITLE 8539
#define IDS_SETTINGS_CERTIFICATE_MANAGER_V2_IMPORT_ERROR_TITLE 8540
#define IDS_SETTINGS_CERTIFICATE_MANAGER_V2_IMPORT_INVALID_FILE 8541
#define IDS_SETTINGS_CERTIFICATE_MANAGER_V2_IMPORT_FAILED 8542
#define IDS_SETTINGS_CERTIFICATE_MANAGER_V2_IMPORT_BAD_PASSWORD 8543
#define IDS_SETTINGS_CERTIFICATE_MANAGER_V2_IMPORT_INVALID_MAC 8544
#define IDS_SETTINGS_CERTIFICATE_MANAGER_V2_IMPORT_UNSUPPORTED 8545
#define IDS_CERTIFICATE_MANAGER_V2_SUBPAGE_BACK_BUTTON_ARIA_LABEL 8546
#define IDS_CERTIFICATE_MANAGER_V2_SUBPAGE_BACK_BUTTON_ARIA_ROLE_DESCRIPTION 8547
#define IDS_CERTIFICATE_MANAGER_V2_LIST_EXPAND_ARIA_LABEL 8548
#define IDS_CERTIFICATE_MANAGER_V2_CERT_ENTRY_VIEW_ARIA_LABEL 8549
#define IDS_CERTIFICATE_MANAGER_V2_CERT_ENTRY_DELETE_ARIA_LABEL 8550
#define IDS_CERTIFICATE_MANAGER_V2_CERT_HASH_COPY_ARIA_LABEL 8551
#define IDS_SUPPORT_TOOL_ISSUE_DETAILS_PAGE_TITLE 8575
#define IDS_SUPPORT_TOOL_DATA_SELECTION_PAGE_TITLE 8576
#define IDS_SUPPORT_TOOL_REVIEW_PII_PAGE_TITLE 8577
#define IDS_SUPPORT_TOOL_DATA_EXPORT_DONE_PAGE_TITLE 8578
#define IDS_SUPPORT_TOOL_DATA_EXPORTED_TEXT 8579
#define IDS_SUPPORT_TOOL_SUPPORT_CASE_ID 8580
#define IDS_SUPPORT_TOOL_EMAIL 8581
#define IDS_SUPPORT_TOOL_DESCRIBE_ISSUE_TEXT 8582
#define IDS_SUPPORT_TOOL_ISSUE_DESCRIPTION_PLACEHOLDER 8583
#define IDS_SUPPORT_TOOL_PII_WARNING_TEXT 8584
#define IDS_SUPPORT_TOOL_INCLUDE_ALL_PII_RADIO_BUTTON 8585
#define IDS_SUPPORT_TOOL_REMOVE_PII_RADIO_BUTTON 8586
#define IDS_SUPPORT_TOOL_PII_REMOVAL_DISCLAIMER 8587
#define IDS_SUPPORT_TOOL_MANUALLY_SELECT_PII_RADIO_BUTTON 8588
#define IDS_SUPPORT_TOOL_CANCEL_BUTTON_TEXT 8589
#define IDS_SUPPORT_TOOL_EXPORT_BUTTON_TEXT 8590
#define IDS_SUPPORT_TOOL_BACK_BUTTON_TEXT 8591
#define IDS_SUPPORT_TOOL_DISMISS_BUTTON_TEXT 8592
#define IDS_SUPPORT_TOOL_CONTINUE_BUTTON_TEXT 8593
#define IDS_SUPPORT_TOOL_DATA_COLLECTION_SPINNER 8594
#define IDS_SUPPORT_TOOL_DATA_EXPORT_SPINNER 8595
#define IDS_SUPPORT_TOOL_TAB_TITLE 8596
#define IDS_SUPPORT_TOOL_URL_GENERATOR_PAGE_TITLE 8597
#define IDS_SUPPORT_TOOL_DATA_COLLECTOR_LIST_TITLE 8598
#define IDS_SUPPORT_TOOL_GET_LINK_TEXT 8599
#define IDS_SUPPORT_TOOL_COPY_LINK_DESCRIPTION 8600
#define IDS_SUPPORT_TOOL_COPY_LINK_BUTTON_TEXT 8601
#define IDS_SUPPORT_TOOL_COPY_TOKEN_BUTTON_TEXT 8602
#define IDS_SUPPORT_TOOL_DONT_INCLUDE_EMAIL 8603
#define IDS_SUPPORT_TOOL_LINK_COPIED_TEXT 8604
#define IDS_SUPPORT_TOOL_TOKEN_COPIED_TEXT 8605
#define IDS_SUPPORT_TOOL_SELECT_DATA_COLLECTOR_ERROR 8606
#define IDS_SUPPORT_TOOL_SELECT_ALL 8607
#define IDS_SUPPORT_TOOL_CHROME_SYSTEM_INFO 8608
#define IDS_SUPPORT_TOOL_CRASH_IDS 8609
#define IDS_SUPPORT_TOOL_MEMORY_DETAILS 8610
#define IDS_SUPPORT_TOOL_UI_HIEARCHY 8611
#define IDS_SUPPORT_TOOL_ADDITIONAL_CROS_PLATFROM_LOGS 8612
#define IDS_SUPPORT_TOOL_DEVICE_EVENT 8613
#define IDS_SUPPORT_TOOL_INTEL_WIFI_DEBUG_DUMP 8614
#define IDS_SUPPORT_TOOL_TOUCH_EVENTS 8615
#define IDS_SUPPORT_TOOL_CHROMEOS_FLEX_LOGS 8616
#define IDS_SUPPORT_TOOL_DBUS_DETAILS 8617
#define IDS_SUPPORT_TOOL_CHROMEOS_NETWORK_ROUTES 8618
#define IDS_SUPPORT_TOOL_CHROMEOS_SHILL_LOGS 8619
#define IDS_SUPPORT_TOOL_POLICIES 8620
#define IDS_SUPPORT_TOOL_CHROMEOS_SYSTEM_STATE 8621
#define IDS_SUPPORT_TOOL_CHROMEOS_SYSTEM_LOGS 8622
#define IDS_SUPPORT_TOOL_CHROMEOS_CHROME_USER_LOGS 8623
#define IDS_SUPPORT_TOOL_CHROMEOS_BLUETOOTH_FLOSS 8624
#define IDS_SUPPORT_TOOL_CHROMEOS_CONNECTED_INPUT_DEVICES 8625
#define IDS_SUPPORT_TOOL_CHROMEOS_TRAFFIC_COUNTERS 8626
#define IDS_SUPPORT_TOOL_CHROMEOS_VIRTUAL_KEYBOARD 8627
#define IDS_SUPPORT_TOOL_CHROMEOS_NETWORK_HEALTH 8628
#define IDS_SUPPORT_TOOL_PERFORMANCE 8629
#define IDS_SUPPORT_TOOL_SIGN_IN 8630
#define IDS_SUPPORT_TOOL_CHROMEOS_APP_SERVICE 8631
#define IDS_SUPPORT_TOOL_CHROMEOS_KIOSK_APP_LEVEL_LOGS 8632
#define IDS_SUPPORT_TOOL_CHROME_UPDATER 8633
#define IDS_SUPPORT_TOOL_ANDROID_APP_INFO 8634
#define IDS_SUPPORT_TOOL_WIFI_SSID 8635
#define IDS_SUPPORT_TOOL_CELLULAR_LOCATION_INFO 8636
#define IDS_SUPPORT_TOOL_EMAIL_ADDRESS 8637
#define IDS_SUPPORT_TOOL_GAIA_ID 8638
#define IDS_SUPPORT_TOOL_STABLE_IDENTIDIERS 8639
#define IDS_SUPPORT_TOOL_PRINTING_IPP_ADDRESS 8640
#define IDS_SUPPORT_TOOL_IP_ADDRESS 8641
#define IDS_SUPPORT_TOOL_MAC_ADDRESS 8642
#define IDS_SUPPORT_TOOL_WINDOW_TITLES 8643
#define IDS_SUPPORT_TOOL_URLS 8644
#define IDS_SUPPORT_TOOL_SERIAL_NUMBERS 8645
#define IDS_SUPPORT_TOOL_REMOVABLE_STORAGE_NAMES 8646
#define IDS_SUPPORT_TOOL_EAP 8647
#define IDS_SUPPORT_TOOL_CREDIT_CARD 8648
#define IDS_SUPPORT_TOOL_IBAN 8649
#define IDS_SUPPORT_TOOL_CRASH_ID 8650
#define IDS_SUPPORT_TOOL_MEMORY 8651
#define IDS_SUPPORT_TOOL_BLUETOOTH_HID_DEVICE 8652
#define IDS_EXTENSIONS_ALLOW_FILE_ACCESS 8653
#define IDS_EXTENSIONS_ALLOW_ON_ALL_URLS 8654
#define IDS_EXTENSIONS_ALLOW_ON_FOLLOWING_SITES 8655
#define IDS_EXTENSIONS_VIEW_ACTIVITY_LOG 8656
#define IDS_EXTENSIONS_BACKGROUND_PAGE 8657
#define IDS_EXTENSIONS_SERVICE_WORKER_BACKGROUND 8658
#define IDS_EXTENSIONS_CORRUPTED_EXTENSION 8659
#define IDS_EXTENSIONS_ENABLE_ERROR_COLLECTION 8660
#define IDS_EXTENSIONS_ERROR_NO_ERRORS_CODE_MESSAGE 8661
#define IDS_EXTENSIONS_INSTALL_DROP_TARGET 8662
#define IDS_EXTENSIONS_INSTALL_WARNINGS 8663
#define IDS_EXTENSIONS_LOG_LEVEL_ERROR 8664
#define IDS_EXTENSIONS_LOG_LEVEL_INFO 8665
#define IDS_EXTENSIONS_LOG_LEVEL_WARN 8666
#define IDS_EXTENSIONS_PATH 8667
#define IDS_EXTENSIONS_PERMISSIONS_OFF 8668
#define IDS_EXTENSIONS_RELOAD_TERMINATED 8669
#define IDS_EXTENSIONS_REPAIR_CORRUPTED 8670
#define IDS_EXTENSIONS_MOVE_TO_ACCOUNT_ICON_TOOLTIP 8671
#define IDS_EXTENSIONS_VIEW_IFRAME 8672
#define IDS_EXTENSIONS_VIEW_INACTIVE 8673
#define IDS_EXTENSIONS_VIEW_INCOGNITO 8674
#define IDS_EXTENSIONS_DEVELOPER_MODE 8675
#define IDS_EXTENSIONS_DISABLED_UNSUPPORTED_DEVELOPER_MODE 8676
#define IDS_EXTENSIONS_DISABLED_UNSUPPORTED_DEVELOPER_MODE_DETAILS 8677
#define IDS_EXTENSIONS_DISABLED_UNSUPPORTED_DEVELOPER_MODE_TOAST 8678
#define IDS_EXTENSIONS_DISABLED_UPDATE_REQUIRED_BY_POLICY 8679
#define IDS_EXTENSIONS_DISABLED_PUBLISHED_IN_STORE_REQUIRED_BY_POLICY 8680
#define IDS_EXTENSIONS_MATCHING_RESTRICTED_SITES_ALLOW 8681
#define IDS_EXTENSIONS_MATCHING_RESTRICTED_SITES_TITLE 8682
#define IDS_EXTENSIONS_MATCHING_RESTRICTED_SITES_WARNING 8683
#define IDS_EXTENSIONS_MENU_BUTTON_LABEL 8684
#define IDS_EXTENSIONS_ERROR_PAGE_HEADING 8685
#define IDS_EXTENSIONS_EDIT_SITE_PERMISSIONS_ALLOW_ALL_EXTENSIONS 8686
#define IDS_EXTENSIONS_EDIT_SITE_PERMISSIONS_CUSTOMIZE_PER_EXTENSION 8687
#define IDS_EXTENSIONS_EDIT_SITE_PERMISSIONS_RESTRICT_EXTENSIONS 8688
#define IDS_EXTENSIONS_ERROR_ANONYMOUS_FUNCTION 8689
#define IDS_EXTENSIONS_ERROR_CONTEXT 8690
#define IDS_EXTENSIONS_ERROR_CONTEXT_UNKNOWN 8691
#define IDS_EXTENSIONS_CLEAR_ACTIVITIES 8692
#define IDS_EXTENSIONS_ERROR_CLEAR_ALL 8693
#define IDS_EXTENSIONS_A11Y_CLEAR_ENTRY 8694
#define IDS_EXTENSIONS_ERROR_STACK_TRACE 8695
#define IDS_EXTENSIONS_ERROR_LINES_NOT_SHOWN 8696
#define IDS_EXTENSIONS_HOST_PERMISSIONS_DESCRIPTION 8697
#define IDS_EXTENSIONS_PERMISSIONS_LEARN_MORE_A11Y_LABEL 8698
#define IDS_EXTENSIONS_HOST_PERMISSIONS_EDIT 8699
#define IDS_EXTENSIONS_ITEM_ERRORS 8700
#define IDS_EXTENSIONS_ITEM_WARNINGS 8701
#define IDS_EXTENSIONS_ITEM_HOST_PERMISSIONS_HEADING 8702
#define IDS_EXTENSIONS_NEW_HOST_PERMISSIONS_HEADING 8703
#define IDS_EXTENSIONS_HOST_PERMISSIONS_SUB_HEADING 8704
#define IDS_EXTENSIONS_HOST_ACCESS_ON_CLICK 8705
#define IDS_EXTENSIONS_HOST_ACCESS_ASK_ON_EVERY_VISIT 8706
#define IDS_EXTENSIONS_HOST_ACCESS_ON_SPECIFIC_SITES 8707
#define IDS_EXTENSIONS_HOST_ACCESS_ALLOW_ON_SPECIFIC_SITES 8708
#define IDS_EXTENSIONS_HOST_ACCESS_ON_ALL_SITES 8709
#define IDS_EXTENSIONS_HOST_ACCESS_ALLOW_ON_ALL_SITES 8710
#define IDS_EXTENSIONS_ITEM_ALLOWED_HOSTS 8711
#define IDS_EXTENSIONS_ACCESSIBILITY_ERROR_LINE 8712
#define IDS_EXTENSIONS_ACCESSIBILITY_ERROR_MULTI_LINE 8713
#define IDS_EXTENSIONS_ACTIVITY_LOG_PAGE_HEADING 8714
#define IDS_EXTENSIONS_ACTIVITY_LOG_SEARCH_LABEL 8715
#define IDS_EXTENSIONS_ACTIVITY_LOG_TYPE_COLUMN 8716
#define IDS_EXTENSIONS_ACTIVITY_LOG_NAME_COLUMN 8717
#define IDS_EXTENSIONS_ACTIVITY_LOG_COUNT_COLUMN 8718
#define IDS_EXTENSIONS_ACTIVITY_LOG_TIME_COLUMN 8719
#define IDS_EXTENSIONS_ACTIVITY_LOG_HISTORY_TAB_HEADING 8720
#define IDS_EXTENSIONS_ACTIVITY_LOG_STREAM_TAB_HEADING 8721
#define IDS_EXTENSIONS_START_ACTIVITY_STREAM 8722
#define IDS_EXTENSIONS_STOP_ACTIVITY_STREAM 8723
#define IDS_EXTENSIONS_EMPTY_STREAM_STARTED 8724
#define IDS_EXTENSIONS_EMPTY_STREAM_STOPPED 8725
#define IDS_EXTENSIONS_ACTIVITY_ARGUMENTS_HEADING 8726
#define IDS_EXTENSIONS_WEB_REQUEST_INFO_HEADING 8727
#define IDS_EXTENSIONS_ACTIVITY_LOG_MORE_ACTIONS_LABEL 8728
#define IDS_EXTENSIONS_ACTIVITY_LOG_EXPAND_ALL 8729
#define IDS_EXTENSIONS_ACTIVITY_LOG_COLLAPSE_ALL 8730
#define IDS_EXTENSIONS_ACTIVITY_LOG_EXPORT_HISTORY 8731
#define IDS_EXTENSIONS_ITEM_ID 8732
#define IDS_EXTENSIONS_ITEM_INSPECT_VIEWS 8733
#define IDS_EXTENSIONS_ITEM_INSPECT_VIEWS_EXTRA 8734
#define IDS_EXTENSIONS_ITEM_NO_ACTIVE_VIEWS 8735
#define IDS_EXTENSIONS_ITEM_ALLOW_INCOGNITO 8736
#define IDS_EXTENSIONS_ITEM_ALLOW_USER_SCRIPTS 8737
#define IDS_EXTENSIONS_ALLOW_USER_SCRIPTS_WARNING 8738
#define IDS_EXTENSIONS_ITEM_DEPENDENCIES 8739
#define IDS_EXTENSIONS_DEPENDENT_ENTRY 8740
#define IDS_EXTENSIONS_ITEM_DESCRIPTION 8741
#define IDS_EXTENSIONS_ITEM_DETAILS 8742
#define IDS_EXTENSIONS_DETAILS_BACK_BUTTON_ARIA_LABEL 8743
#define IDS_EXTENSIONS_DETAILS_BACK_BUTTON_ARIA_ROLE_DESCRIPTION 8744
#define IDS_EXTENSIONS_EXTENSION_A11Y_ASSOCIATION 8745
#define IDS_EXTENSIONS_APP_ICON 8746
#define IDS_EXTENSIONS_EXTENSION_ICON 8747
#define IDS_EXTENSIONS_ITEM_ID_HEADING 8748
#define IDS_EXTENSIONS_EXTENSION_ENABLED 8749
#define IDS_EXTENSIONS_APP_ENABLED 8750
#define IDS_EXTENSIONS_ITEM_OFF 8751
#define IDS_EXTENSIONS_ITEM_ON 8752
#define IDS_EXTENSIONS_ITEM_EXTENSION_WEBSITE 8753
#define IDS_EXTENSIONS_ITEM_CHROME_WEB_STORE 8754
#define IDS_EXTENSIONS_VIEW_IN_DEVTOOLS 8755
#define IDS_EXTENSIONS_ITEM_OPTIONS 8756
#define IDS_EXTENSIONS_ITEM_PERMISSIONS 8757
#define IDS_EXTENSIONS_ITEM_PERMISSIONS_EMPTY 8758
#define IDS_EXTENSIONS_ITEM_PERMISSIONS_AND_SITE_ACCESS_EMPTY 8759
#define IDS_EXTENSIONS_ITEM_PIN_TO_TOOLBAR 8760
#define IDS_EXTENSIONS_ITEM_REMOVE_EXTENSION 8761
#define IDS_EXTENSIONS_ITEM_SITE_ACCESS 8762
#define IDS_EXTENSIONS_ITEM_SITE_ACCESS_ADD_HOST 8763
#define IDS_EXTENSIONS_ITEM_SITE_ACCESS_EMPTY 8764
#define IDS_EXTENSIONS_REMOVE_SITES_DIALOG_TITLE 8765
#define IDS_EXTENSIONS_ITEM_SOURCE 8766
#define IDS_EXTENSIONS_ITEM_SOURCE_INSTALLED_BY_DEFAULT 8767
#define IDS_EXTENSIONS_ITEM_SOURCE_POLICY 8768
#define IDS_EXTENSIONS_ITEM_SOURCE_SIDELOADED 8769
#define IDS_EXTENSIONS_ITEM_SOURCE_UNPACKED 8770
#define IDS_EXTENSIONS_ITEM_SOURCE_WEBSTORE 8771
#define IDS_EXTENSIONS_ITEM_VERSION 8772
#define IDS_EXTENSIONS_ITEM_RELOADED 8773
#define IDS_EXTENSIONS_ITEM_RELOADING 8774
#define IDS_EXTENSIONS_LOAD_ERROR_HEADING 8775
#define IDS_EXTENSIONS_LOAD_ERROR_ERROR_LABEL 8776
#define IDS_EXTENSIONS_LOAD_ERROR_FILE_LABEL 8777
#define IDS_EXTENSIONS_LOAD_ERROR_COULD_NOT_LOAD_MANIFEST 8778
#define IDS_EXTENSIONS_LOAD_ERROR_RETRY 8779
#define IDS_EXTENSIONS_LOADING_ACTIVITIES 8780
#define IDS_MISSING_OR_UNINSTALLED_EXTENSION 8781
#define IDS_EXTENSIONS_NO_ACTIVITIES 8782
#define IDS_EXTENSIONS_NO_INSTALLED_ITEMS 8783
#define IDS_EXTENSIONS_NO_DESCRIPTION 8784
#define IDS_EXTENSIONS_OPENS_IN_NEW_TAB 8785
#define IDS_EXTENSIONS_PACK_DIALOG_TITLE 8786
#define IDS_EXTENSIONS_PACK_DIALOG_WARNING_TITLE 8787
#define IDS_EXTENSIONS_PACK_DIALOG_ERROR_TITLE 8788
#define IDS_EXTENSIONS_PACK_DIALOG_PROCEED_ANYWAY 8789
#define IDS_EXTENSIONS_PACK_DIALOG_BROWSE_BUTTON 8790
#define IDS_EXTENSIONS_PACK_DIALOG_EXTENSION_ROOT_LABEL 8791
#define IDS_EXTENSIONS_PACK_DIALOG_KEY_FILE_LABEL 8792
#define IDS_EXTENSIONS_PACK_DIALOG_CONFIRM_BUTTON 8793
#define IDS_EXTENSIONS_TOOLBAR_TITLE 8794
#define IDS_EXTENSIONS_SEARCH 8795
#define IDS_EXTENSIONS_SITE_PERMISSIONS 8796
#define IDS_EXTENSIONS_SITE_PERMISSIONS_PAGE_TITLE 8797
#define IDS_EXTENSIONS_SITE_PERMISSIONS_VIEW_ALL_SITES 8798
#define IDS_EXTENSIONS_SITE_PERMISSIONS_ALL_SITES_PAGE_TITLE 8799
#define IDS_EXTENSIONS_SITE_PERMISSIONS_ALL_SITES_ONE_EXTENSION 8800
#define IDS_EXTENSIONS_SITE_PERMISSIONS_ALL_SITES_EXTENSION_COUNT 8801
#define IDS_EXTENSIONS_SITE_PERMISSIONS_ALWAYS_ON_ALL_SITES 8802
#define IDS_EXTENSIONS_SITE_PERMISSIONS_ALWAYS_ON_THIS_SITE 8803
#define IDS_EXTENSIONS_SITE_SETTINGS 8804
#define IDS_EXTENSIONS_PERMITTED_SITES 8805
#define IDS_EXTENSIONS_RESTRICTED_SITES 8806
#define IDS_EXTENSIONS_NO_SITES_ADDED 8807
#define IDS_EXTENSIONS_SITE_PERMISSIONS_ADD_SITE_DIALOG_TITLE 8808
#define IDS_EXTENSIONS_SITE_PERMISSIONS_EDIT_SITE_DIALOG_TITLE 8809
#define IDS_EXTENSIONS_SITE_PERMISSIONS_EDIT_URL 8810
#define IDS_EXTENSIONS_SITE_PERMISSIONS_EDIT_PERMISSIONS_DIALOG_TITLE 8811
#define IDS_EXTENSIONS_SITE_PERMISSIONS_EDIT_PERMISSIONS 8812
#define IDS_EXTENSIONS_SITE_PERMISSIONS_ASK_ON_EVERY_VISIT 8813
#define IDS_EXTENSIONS_SITE_PERMISSIONS_DIALOG_INPUT_ERROR 8814
#define IDS_EXTENSIONS_SITE_PERMISSIONS_DIALOG_INPUT_LABEL 8815
#define IDS_EXTENSIONS_SITE_PERMISSIONS_INCLUDES_SUBDOMAINS 8816
#define IDS_EXTENSIONS_EDIT_SHORTCUT_INPUT_LABEL 8817
#define IDS_EXTENSIONS_EDIT_SHORTCUT_BUTTON_LABEL 8818
#define IDS_EXTENSIONS_SET_SHORTCUT_IN_SYSTEM_SETTINGS 8819
#define IDS_EXTENSIONS_SHORTCUT_SCOPE_LABEL 8820
#define IDS_EXTENSIONS_SHORTCUT_SCOPE_GLOBAL 8821
#define IDS_EXTENSIONS_APPS_TITLE 8822
#define IDS_EXTENSIONS_REMOVE 8823
#define IDS_EXTENSIONS_MORE_OPTIONS 8824
#define IDS_EXTENSIONS_RUNTIME_HOSTS_DIALOG_TITLE 8825
#define IDS_EXTENSIONS_RUNTIME_HOSTS_DIALOG_INPUT_ERROR 8826
#define IDS_EXTENSIONS_RUNTIME_HOSTS_DIALOG_INPUT_LABEL 8827
#define IDS_EXTENSIONS_SIDEBAR_EXTENSIONS 8828
#define IDS_EXTENSIONS_SIDEBAR_DISCOVER_MORE 8829
#define IDS_EXTENSIONS_WHATS_NEW_SIDEBAR_PROMO 8830
#define IDS_EXTENSIONS_SIDEBAR_KEYBOARD_SHORTCUTS 8831
#define IDS_EXTENSIONS_TOOLBAR_LOAD_UNPACKED 8832
#define IDS_EXTENSIONS_TOOLBAR_LOAD_UNPACKED_DONE 8833
#define IDS_EXTENSIONS_TOOLBAR_PACK 8834
#define IDS_EXTENSIONS_TOOLBAR_UPDATE_NOW 8835
#define IDS_EXTENSIONS_TOOLBAR_UPDATE_NOW_TOOLTIP 8836
#define IDS_EXTENSIONS_TOOLBAR_UPDATE_DONE 8837
#define IDS_EXTENSIONS_TOOLBAR_UPDATING_TOAST 8838
#define IDS_EXTENSIONS_SUBPAGE_BUTTON 8839
#define IDS_EXTENSIONS_SECTION_HEADER 8840
#define IDS_EXTENSIONS_SAFETY_HUB_HEADER 8841
#define IDS_EXTENSIONS_SC_MALWARE 8842
#define IDS_EXTENSIONS_SC_POLICY_VIOLATION_OFF 8843
#define IDS_EXTENSIONS_SC_POLICY_VIOLATION_ON 8844
#define IDS_EXTENSIONS_SC_UNPUBLISHED_OFF 8845
#define IDS_EXTENSIONS_SC_UNPUBLISHED_ON 8846
#define IDS_EXTENSIONS_SAFETY_CHECK_NO_PRIVACY_PRACTICES_ON 8847
#define IDS_EXTENSIONS_SAFETY_CHECK_NO_PRIVACY_PRACTICES_OFF 8848
#define IDS_EXTENSIONS_SC_TITLE 8849
#define IDS_EXTENSIONS_SC_KEEP_EXT 8850
#define IDS_EXTENSIONS_SC_THREEDOT_DETAILS 8851
#define IDS_EXTENSIONS_SC_REMOVE_ALL 8852
#define IDS_EXTENSIONS_SC_ALL_DONE_FOR_NOW 8853
#define IDS_EXTENSIONS_SC_REMOVE_BUTTON_A11Y_LABEL 8854
#define IDS_EXTENSIONS_SC_OPTION_MENU_A11Y_LABEL 8855
#define IDS_EXTENSIONS_MV2_DEPRECATION_PANEL_TITLE 8856
#define IDS_EXTENSIONS_MV2_DEPRECATION_PANEL_WARNING_HEADER 8857
#define IDS_EXTENSIONS_MV2_DEPRECATION_PANEL_WARNING_SUBTITLE 8858
#define IDS_EXTENSIONS_MV2_DEPRECATION_PANEL_DISABLED_HEADER 8859
#define IDS_EXTENSIONS_MV2_DEPRECATION_MESSAGE_WARNING_HEADER 8860
#define IDS_EXTENSIONS_MV2_DEPRECATION_MESSAGE_WARNING_SUBTITLE 8861
#define IDS_EXTENSIONS_MV2_DEPRECATION_MESSAGE_DISABLED_HEADER 8862
#define IDS_EXTENSIONS_MV2_DEPRECATION_MESSAGE_REMOVE_BUTTON 8863
#define IDS_EXTENSIONS_MV2_DEPRECATION_PANEL_DISMISS_BUTTON 8864
#define IDS_EXTENSIONS_MV2_DEPRECATION_PANEL_FIND_ALTERNATIVE_BUTTON 8865
#define IDS_EXTENSIONS_MV2_DEPRECATION_PANEL_ACTION_MENU_BUTTON_LABEL 8866
#define IDS_EXTENSIONS_MV2_DEPRECATION_PANEL_FIND_ALTERNATIVE_BUTTON_ACC_LABEL 8867
#define IDS_EXTENSIONS_MV2_DEPRECATION_UNSUPPORTED_EXTENSION_OFF_TEXT 8868
#define IDS_EXTENSIONS_CANT_INSTALL_MV2_EXTENSION 8869
#define IDS_EXTENSIONS_ZERO_STATE_IPH_HEADER 8881
#define IDS_EXTENSIONS_ZERO_STATE_IPH_HEADER_V2 8882
#define IDS_EXTENSIONS_ZERO_STATE_CHIPS_IPH_DESCRIPTION 8883
#define IDS_EXTENSIONS_ZERO_STATE_CHIPS_WITH_LINK_IPH_LINK_LABEL 8884
#define IDS_EXTENSIONS_ZERO_STATE_CHIPS_WITH_LINK_IPH_DESCRIPTION 8885
#define IDS_EXTENSIONS_ZERO_STATE_PLAIN_LINK_IPH_DESCRIPTION 8886
#define IDS_EXTENSIONS_ZERO_STATE_IPH_SHOPPING_CATEGORY_LABEL 8887
#define IDS_EXTENSIONS_ZERO_STATE_IPH_WRITING_HELP_COLLECTION_LABEL 8888
#define IDS_EXTENSIONS_ZERO_STATE_IPH_PRODUCTIVITY_CATEGORY_LABEL 8889
#define IDS_EXTENSIONS_ZERO_STATE_IPH_AI_PRODUCTIVITY_COLLECTION_LABEL 8890
#define IDS_EXTENSIONS_ZERO_STATE_IPH_DISMISS_BUTTON_TITLE 8891
#define IDS_EXTENSIONS_ZERO_STATE_IPH_SHOPPING_CATEGORY_LINK 8892
#define IDS_EXTENSIONS_ZERO_STATE_IPH_WRITING_HELP_COLLECTION_LINK 8893
#define IDS_EXTENSIONS_ZERO_STATE_IPH_PRODUCTIVITY_CATEGORY_LINK 8894
#define IDS_EXTENSIONS_ZERO_STATE_IPH_AI_PRODUCTIVITY_COLLECTION_LINK 8895
#define IDS_EXTENSIONS_ZERO_STATE_IPH_WEB_STORE_LINK 8896
#define IDS_EXTENSIONS_ZERO_STATE_IPH_DISMISS_BUTTON_TEXT 8897
#define IDS_NEARBY_CONFIRMATION_PAGE_ADD_CONTACT_SUBTITLE 8898
#define IDS_NEARBY_CONFIRMATION_PAGE_ADD_CONTACT_TITLE 8899
#define IDS_NEARBY_CONFIRMATION_PAGE_TITLE 8900
#define IDS_QUICK_SHARE_V2_VISIBILITY_SECTION_TITLE 8901
#define IDS_QUICK_SHARE_V2_VISIBILITY_SECTION_SUBTITLE_ON_DISABLED 8902
#define IDS_QUICK_SHARE_V2_VISIBILITY_YOUR_DEVICES_SUBLABEL 8903
#define IDS_QUICK_SHARE_V2_VISIBILITY_CONTACTS_SUBLABEL 8904
#define IDS_QUICK_SHARE_V2_VISIBILITY_EVERYONE_LABEL 8905
#define IDS_QUICK_SHARE_V2_VISIBILITY_EVERYONE_SUBLABEL 8906
#define IDS_QUICK_SHARE_V2_VISIBILITY_ONLY_FOR_TEN_MINUTES_LABEL 8907
#define IDS_NEARBY_CONTACT_VISIBILITY_DOWNLOAD_FAILED 8908
#define IDS_NEARBY_CONTACT_VISIBILITY_DOWNLOADING 8909
#define IDS_NEARBY_CONTACT_VISIBILITY_NO_CONTACTS_SUBTITLE 8910
#define IDS_NEARBY_CONTACT_VISIBILITY_NO_CONTACTS_SUBTITLE_PH 8911
#define IDS_NEARBY_CONTACT_VISIBILITY_NO_CONTACTS_TITLE 8912
#define IDS_NEARBY_CONTACT_VISIBILITY_NUM_UNREACHABLE 8913
#define IDS_NEARBY_CONTACT_VISIBILITY_NUM_UNREACHABLE_PH 8914
#define IDS_NEARBY_CONTACT_VISIBILITY_OWN_ALL 8915
#define IDS_NEARBY_CONTACT_VISIBILITY_OWN_ALL_SELF_SHARE 8916
#define IDS_NEARBY_CONTACT_VISIBILITY_OWN_NONE 8917
#define IDS_NEARBY_CONTACT_VISIBILITY_OWN_SOME 8918
#define IDS_NEARBY_CONTACT_VISIBILITY_OWN_SOME_SELF_SHARE 8919
#define IDS_NEARBY_CONTACT_VISIBILITY_ZERO_STATE_TEXT 8920
#define IDS_NEARBY_CONTACT_VISIBILITY_OWN_YOUR_DEVICES 8921
#define IDS_NEARBY_DEVICE_NAME_EMPTY_ERROR 8922
#define IDS_NEARBY_DEVICE_NAME_TOO_LONG_ERROR 8923
#define IDS_NEARBY_DEVICE_NAME_INVALID_CHARACTERS_ERROR 8924
#define IDS_NEARBY_DISCOVERY_PAGE_INFO 8925
#define IDS_NEARBY_DISCOVERY_PAGE_SUBTITLE 8926
#define IDS_NEARBY_DISCOVERY_PAGE_TITLE 8927
#define IDS_NEARBY_DISCOVERY_PAGE_TITLE_PH 8928
#define IDS_NEARBY_DISCOVERY_PAGE_PLACEHOLDER 8929
#define IDS_NEARBY_ONBOARDING_PAGE_DEVICE_NAME 8930
#define IDS_NEARBY_ONBOARDING_PAGE_DEVICE_NAME_HELP 8931
#define IDS_NEARBY_ONBOARDING_PAGE_SUBTITLE 8932
#define IDS_NEARBY_ONBOARDING_PAGE_TITLE 8933
#define IDS_NEARBY_ONBOARDING_PAGE_TITLE_PH 8934
#define IDS_NEARBY_ONBOARDING_PAGE_DEVICE_VISIBILITY 8935
#define IDS_NEARBY_ONBOARDING_PAGE_DEVICE_VISIBILITY_HELP_ALL_CONTACTS 8936
#define IDS_NEARBY_VISIBILITY_PAGE_MANAGE_CONTACTS 8937
#define IDS_NEARBY_VISIBILITY_PAGE_SUBTITLE 8938
#define IDS_NEARBY_VISIBILITY_PAGE_TITLE 8939
#define IDS_NEARBY_SHARE_FEATURE_NAME 8940
#define IDS_NEARBY_SHARE_FEATURE_NAME_PH 8941
#define IDS_NEARBY_ACTIONS_ACCEPT 8942
#define IDS_NEARBY_ACTIONS_CANCEL 8943
#define IDS_NEARBY_ACTIONS_CLOSE 8944
#define IDS_NEARBY_ACTIONS_CONFIRM 8945
#define IDS_NEARBY_ACTIONS_DECLINE 8946
#define IDS_NEARBY_ACTIONS_NEXT 8947
#define IDS_NEARBY_ACTIONS_REJECT 8948
#define IDS_NEARBY_DEFAULT_DEVICE_NAME 8949
#define IDS_NEARBY_ERROR_CANCELLED 8950
#define IDS_NEARBY_ERROR_CANT_RECEIVE 8951
#define IDS_NEARBY_ERROR_CANT_SHARE 8952
#define IDS_NEARBY_ERROR_NO_RESPONSE 8953
#define IDS_NEARBY_ERROR_TRANSFER_IN_PROGRESS 8954
#define IDS_NEARBY_ERROR_NOT_ENOUGH_SPACE 8955
#define IDS_NEARBY_ERROR_REJECTED 8956
#define IDS_NEARBY_ERROR_SOMETHING_WRONG 8957
#define IDS_NEARBY_ERROR_TIME_OUT 8958
#define IDS_NEARBY_ERROR_TRY_AGAIN 8959
#define IDS_NEARBY_ERROR_UNSUPPORTED_FILE_TYPE 8960
#define IDS_NEARBY_FILE_ATTACHMENTS_CAPITALIZED_APPS 8961
#define IDS_NEARBY_FILE_ATTACHMENTS_NOT_CAPITALIZED_APPS 8962
#define IDS_NEARBY_FILE_ATTACHMENTS_CAPITALIZED_IMAGES 8963
#define IDS_NEARBY_FILE_ATTACHMENTS_NOT_CAPITALIZED_IMAGES 8964
#define IDS_NEARBY_FILE_ATTACHMENTS_CAPITALIZED_UNKNOWN 8965
#define IDS_NEARBY_FILE_ATTACHMENTS_NOT_CAPITALIZED_UNKNOWN 8966
#define IDS_NEARBY_FILE_ATTACHMENTS_CAPITALIZED_VIDEOS 8967
#define IDS_NEARBY_FILE_ATTACHMENTS_NOT_CAPITALIZED_VIDEOS 8968
#define IDS_NEARBY_SECURE_CONNECTION_ID 8969
#define IDS_NEARBY_TEXT_ATTACHMENTS_CAPITALIZED_ADDRESSES 8970
#define IDS_NEARBY_TEXT_ATTACHMENTS_NOT_CAPITALIZED_ADDRESSES 8971
#define IDS_NEARBY_TEXT_ATTACHMENTS_CAPITALIZED_LINKS 8972
#define IDS_NEARBY_TEXT_ATTACHMENTS_NOT_CAPITALIZED_LINKS 8973
#define IDS_NEARBY_TEXT_ATTACHMENTS_CAPITALIZED_PHONE_NUMBERS 8974
#define IDS_NEARBY_TEXT_ATTACHMENTS_NOT_CAPITALIZED_PHONE_NUMBERS 8975
#define IDS_NEARBY_TEXT_ATTACHMENTS_CAPITALIZED_UNKNOWN 8976
#define IDS_NEARBY_TEXT_ATTACHMENTS_NOT_CAPITALIZED_UNKNOWN 8977
#define IDS_NEARBY_CAPITALIZED_UNKNOWN_ATTACHMENTS 8978
#define IDS_NEARBY_NOT_CAPITALIZED_UNKNOWN_ATTACHMENTS 8979
#define IDS_NEARBY_HIGH_VISIBILITY_SUB_TITLE 8980
#define IDS_NEARBY_HIGH_VISIBILITY_SUB_TITLE_MINUTES 8981
#define IDS_NEARBY_HIGH_VISIBILITY_SUB_TITLE_SECONDS 8982
#define IDS_NEARBY_HIGH_VISIBILITY_HELP_TEXT 8983
#define IDS_NEARBY_HIGH_VISIBILITY_TIMEOUT_TEXT 8984
#define IDS_NEARBY_HIGH_VISIBILITY_TRANSFER_IN_PROGRESS_ERROR 8985
#define IDS_NEARBY_HIGH_VISIBILITY_TRANSFER_IN_PROGRESS_DESCRIPTION 8986
#define IDS_NEARBY_HIGH_VISIBILITY_CONNECTION_MEDIUM_ERROR 8987
#define IDS_NEARBY_HIGH_VISIBILITY_CONNECTION_MEDIUM_DESCRIPTION 8988
#define IDS_NEARBY_HIGH_VISIBILITY_CONNECTION_MEDIUM_DESCRIPTION_PH 8989
#define IDS_NEARBY_RECEIVE_CONFIRM_PAGE_TITLE 8990
#define IDS_NEARBY_RECEIVE_CONFIRM_PAGE_CONNECTION_ID 8991
#define IDS_NEARBY_PREVIEW_TITLE_MULTIPLE_FILE 8992
#define IDS_NEARBY_ACCOUNT_ROW_LABEL 8993
#define IDS_NEARBY_ACCOUNT_ROW_LABEL_PH 8994
#define IDS_NEARBY_SETTINGS_HELP_CAPTION_TOP 8995
#define IDS_NEARBY_SETTINGS_HELP_CAPTION_TOP_PH 8996
#define IDS_NEARBY_SETTINGS_HELP_CAPTION_BOTTOM 8997
#define IDS_NEARBY_VISIBLITY_ALL_CONTACTS 8998
#define IDS_NEARBY_VISIBLITY_ALL_CONTACTS_DESCRIPTION 8999
#define IDS_NEARBY_VISIBILITY_CONTACTS_BUTTON 9000
#define IDS_NEARBY_VISIBLITY_SOME_CONTACTS 9001
#define IDS_NEARBY_VISIBLITY_SOME_CONTACTS_DESCRIPTION 9002
#define IDS_NEARBY_VISIBILITY_YOUR_DEVICES 9003
#define IDS_NEARBY_VISIBILITY_YOUR_DEVICES_DESCRIPTION 9004
#define IDS_NEARBY_VISIBILITY_ALL_CONTACTS_TOGGLE 9005
#define IDS_NEARBY_VISIBLITY_HIDDEN 9006
#define IDS_NEARBY_VISIBLITY_HIDDEN_DESCRIPTION 9007
#define IDS_NEARBY_VISIBLITY_UNKNOWN 9008
#define IDS_NEARBY_VISIBLITY_UNKNOWN_DESCRIPTION 9009
#define IDS_NEARBY_NOTIFICATION_ACTION_COPY_TO_CLIPBOARD 9010
#define IDS_NEARBY_NOTIFICATION_ACTION_OPEN_FOLDER 9011
#define IDS_NEARBY_NOTIFICATION_ACTION_OPEN_NETWORK_LIST 9012
#define IDS_NEARBY_NOTIFICATION_ACTION_OPEN_URL 9013
#define IDS_NEARBY_NOTIFICATION_ACCEPT_ACTION 9014
#define IDS_NEARBY_NOTIFICATION_CONNECTION_REQUEST_MESSAGE 9015
#define IDS_NEARBY_NOTIFICATION_CONNECTION_REQUEST_MESSAGE_WIFI_CREDENTIALS 9016
#define IDS_NEARBY_NOTIFICATION_CONNECTION_REQUEST_TITLE 9017
#define IDS_NEARBY_NOTIFICATION_CONNECTION_REQUEST_TITLE_PH 9018
#define IDS_NEARBY_NOTIFICATION_DECLINE_ACTION 9019
#define IDS_NEARBY_NOTIFICATION_ONBOARDING_MESSAGE 9020
#define IDS_NEARBY_NOTIFICATION_ONBOARDING_MESSAGE_PH 9021
#define IDS_NEARBY_NOTIFICATION_GO_VISIBLE_MESSAGE 9022
#define IDS_NEARBY_NOTIFICATION_ONBOARDING_TITLE 9023
#define IDS_NEARBY_NOTIFICATION_SET_UP_ACTION 9024
#define IDS_NEARBY_NOTIFICATION_GO_VISIBLE_ACTION 9025
#define IDS_NEARBY_NOTIFICATION_VISIBILITY_REMINDER_TITLE 9026
#define IDS_NEARBY_NOTIFICATION_VISIBILITY_REMINDER_MESSAGE 9027
#define IDS_NEARBY_NOTIFICATION_GO_TO_SETTINGS_ACTION 9028
#define IDS_NEARBY_NOTIFICATION_DISMISS_ACTION 9029
#define IDS_NEARBY_NOTIFICATION_RECEIVE_FAILURE_TITLE 9030
#define IDS_NEARBY_NOTIFICATION_RECEIVE_FAILURE_TITLE_WIFI_CREDENTIALS 9031
#define IDS_NEARBY_NOTIFICATION_RECEIVE_PROGRESS_TITLE 9032
#define IDS_NEARBY_NOTIFICATION_RECEIVE_PROGRESS_TITLE_WIFI_CREDENTIALS 9033
#define IDS_NEARBY_NOTIFICATION_RECEIVE_SUCCESS_TITLE 9034
#define IDS_NEARBY_NOTIFICATION_RECEIVE_SUCCESS_TITLE_WIFI_CREDENTIALS 9035
#define IDS_NEARBY_NOTIFICATION_SEND_FAILURE_TITLE 9036
#define IDS_NEARBY_NOTIFICATION_SEND_PROGRESS_TITLE 9037
#define IDS_NEARBY_NOTIFICATION_SEND_SUCCESS_TITLE 9038
#define IDS_NEARBY_NOTIFICATION_SOURCE 9039
#define IDS_NEARBY_NOTIFICATION_SOURCE_PH 9040
#define IDS_NEARBY_NOTIFICATION_SENDER_CANCELLED 9041
#define IDS_WHATS_NEW_TITLE 9042
#define IDS_PASSWORD_MANAGER_UI_TITLE 9043
#define IDS_PASSWORD_MANAGER_UI_DESCRIPTION 9044
#define IDS_PASSWORD_MANAGER_UI_SEARCH_PROMPT 9045
#define IDS_PASSWORD_MANAGER_UI_CHECKUP 9046
#define IDS_PASSWORD_MANAGER_UI_PASSWORDS 9047
#define IDS_PASSWORD_MANAGER_UI_SETTINGS 9048
#define IDS_PASSWORD_MANAGER_UI_SAVE_PASSWORDS_TOGGLE_LABEL 9049
#define IDS_PASSWORD_MANAGER_UI_AUTOSIGNIN_TOGGLE_LABEL 9050
#define IDS_PASSWORD_MANAGER_UI_AUTOSIGNIN_TOGGLE_DESC 9051
#define IDS_PASSWORD_MANAGER_UI_PASSWORDS_DESCRIPTION 9052
#define IDS_PASSWORD_MANAGER_UI_ADD_PASSWORD_BUTTON 9053
#define IDS_PASSWORD_MANAGER_UI_ADD_SHORTCUT_TITLE 9054
#define IDS_PASSWORD_MANAGER_UI_ADD_SHORTCUT_DESCRIPTION 9055
#define IDS_PASSWORD_MANAGER_UI_TRUSTED_VAULT_OPT_IN_TITLE 9056
#define IDS_PASSWORD_MANAGER_UI_TRUSTED_VAULT_OPT_IN_DESCRIPTION 9057
#define IDS_PASSWORD_MANAGER_UI_TRUSTED_VAULT_OPTED_IN_TITLE 9058
#define IDS_PASSWORD_MANAGER_UI_TRUSTED_VAULT_OPTED_IN_DESCRIPTION 9059
#define IDS_PASSWORD_MANAGER_UI_SHARE 9060
#define IDS_PASSWORD_MANAGER_UI_SHARE_PASSWORD_ERROR_DESCRIPTION 9061
#define IDS_PASSWORD_MANAGER_UI_SHARE_PASSWORD_ERROR_TITLE 9062
#define IDS_PASSWORD_MANAGER_UI_SHARE_PASSWORD_LOADING_TITLE 9063
#define IDS_PASSWORD_MANAGER_UI_SHARE_PASSWORD_SUCCESS_TITLE 9064
#define IDS_PASSWORD_MANAGER_UI_SHARE_PASSWORD_CANCELED_TITLE 9065
#define IDS_PASSWORD_MANAGER_UI_SHARE_PASSWORD_NOT_FAMILY_MEMBER 9066
#define IDS_PASSWORD_MANAGER_UI_SHARE_PASSWORD_NO_OTHER_FAMILY_MEMBERS 9067
#define IDS_PASSWORD_MANAGER_UI_SHARE_PASSWORD_FAMILY_PICKER_DESCRIPTION 9068
#define IDS_PASSWORD_MANAGER_UI_SHARE_PASSWORD_CONFIRMATION_DESCRIPTION_SINGLE 9069
#define IDS_PASSWORD_MANAGER_UI_SHARE_PASSWORD_CONFIRMATION_DESCRIPTION_MULTIPLE 9070
#define IDS_PASSWORD_MANAGER_UI_SHARE_PASSWORD_CONFIRMATION_FOOTER_WEBSITE 9071
#define IDS_PASSWORD_MANAGER_UI_SHARE_PASSWORD_CONFIRMATION_FOOTER_ANDROID_APP 9072
#define IDS_PASSWORD_MANAGER_UI_SHARE_PASSWORD_VIEW_FAMILY 9073
#define IDS_PASSWORD_MANAGER_UI_SHARE_PASSWORD_MEMBER_UNAVAILABLE 9074
#define IDS_PASSWORD_MANAGER_UI_SHARING_IS_MANAGED_BY_ADMIN 9075
#define IDS_PASSWORD_MANAGER_UI_SHARE_PASSWORD_NOT_AVAILABLE 9076
#define IDS_PASSWORD_MANAGER_UI_SHARE_PASSWORD_TRY_AGAIN 9077
#define IDS_PASSWORD_MANAGER_UI_SHARE_PASSWORD_GOT_IT 9078
#define IDS_PASSWORD_MANAGER_UI_SHARE_DIALOG_TITLE 9079
#define IDS_PASSWORD_MANAGER_UI_IMPORT_BANNER_TITLE 9080
#define IDS_PASSWORD_MANAGER_UI_IMPORT_DESCRIPTION_ACCOUNT_STORE_USERS 9081
#define IDS_PASSWORD_MANAGER_UI_IMPORT_DESCRIPTION_SYNCING_USERS 9082
#define IDS_PASSWORD_MANAGER_UI_IMPORT_DESCRIPTION_SIGNEDOUT_USERS 9083
#define IDS_PASSWORD_MANAGER_UI_STORE_PICKER_OPTION_DEVICE 9084
#define IDS_PASSWORD_MANAGER_UI_STORE_PICKER_OPTION_ACCOUNT 9085
#define IDS_PASSWORD_MANAGER_UI_IMPORT_STORE_PICKER_ACCESSIBLE_NAME 9086
#define IDS_PASSWORD_MANAGER_UI_IMPORT_SELECT_FILE_DESCRIPTION 9087
#define IDS_PASSWORD_MANAGER_UI_SELECT_FILE 9088
#define IDS_PASSWORD_MANAGER_UI_IMPORT_MISSING_PASSWORD 9089
#define IDS_PASSWORD_MANAGER_UI_IMPORT_MISSING_URL 9090
#define IDS_PASSWORD_MANAGER_UI_IMPORT_INVALID_URL 9091
#define IDS_PASSWORD_MANAGER_UI_IMPORT_LONG_URL 9092
#define IDS_PASSWORD_MANAGER_UI_IMPORT_LONG_USERNAME 9093
#define IDS_PASSWORD_MANAGER_UI_IMPORT_LONG_PASSWORD 9094
#define IDS_PASSWORD_MANAGER_UI_IMPORT_LONG_NOTE 9095
#define IDS_PASSWORD_MANAGER_UI_IMPORT_CONFLICT_ACCOUNT 9096
#define IDS_PASSWORD_MANAGER_UI_IMPORT_CONFLICT_DEVICE 9097
#define IDS_PASSWORD_MANAGER_UI_IMPORT_ERROR_UNKNOWN 9098
#define IDS_PASSWORD_MANAGER_UI_IMPORT_ERROR_BAD_FORMAT 9099
#define IDS_PASSWORD_MANAGER_UI_IMPORT_FILE_SIZE_EXCEEDED 9100
#define IDS_PASSWORD_MANAGER_UI_IMPORT_ERROR_LIMIT_EXCEEDED 9101
#define IDS_PASSWORD_MANAGER_UI_IMPORT_ERROR_TITLE 9102
#define IDS_PASSWORD_MANAGER_UI_IMPORT_SUCCESS_TITLE 9103
#define IDS_PASSWORD_MANAGER_UI_IMPORT_COMPLETE_TITLE 9104
#define IDS_PASSWORD_MANAGER_UI_IMPORT_DELETE_FILE_OPTION 9105
#define IDS_PASSWORD_MANAGER_UI_IMPORT_VIEW_PASSWORDS 9106
#define IDS_PASSWORD_MANAGER_UI_IMPORT_ALREADY_ACTIVE 9107
#define IDS_PASSWORD_MANAGER_UI_IMPORT_SUCCESS_TIP 9108
#define IDS_PASSWORD_MANAGER_UI_IMPORT_FAILURES_SUMMARY 9109
#define IDS_PASSWORD_MANAGER_UI_IMPORT_BAD_ROWS_FORMAT 9110
#define IDS_PASSWORD_MANAGER_UI_IMPORT_SUCCESS_SUMMARY_ACCOUNT 9111
#define IDS_PASSWORD_MANAGER_UI_IMPORT_SUCCESS_SUMMARY_DEVICE 9112
#define IDS_PASSWORD_MANAGER_UI_IMPORT_CONFLICTS_TITLE 9113
#define IDS_PASSWORD_MANAGER_UI_IMPORT_CONFLICTS_DESCRIPTION 9114
#define IDS_PASSWORD_MANAGER_UI_IMPORT_CANCEL 9115
#define IDS_PASSWORD_MANAGER_UI_IMPORT_SKIP 9116
#define IDS_PASSWORD_MANAGER_UI_IMPORT_REPLACE 9117
#define IDS_PASSWORD_MANAGER_UI_EXPORT_TITLE 9118
#define IDS_PASSWORD_MANAGER_UI_DOWNLOAD_FILE 9119
#define IDS_PASSWORD_MANAGER_UI_EXPORT_BANNER_DESCRIPTION 9120
#define IDS_PASSWORD_MANAGER_UI_EXPORT_SUCCESSFUL 9121
#define IDS_PASSWORD_MANAGER_UI_EXPORT_TRY_AGAIN 9122
#define IDS_PASSWORD_MANAGER_UI_EXPORTING_FAILURE_TITLE 9123
#define IDS_PASSWORD_MANAGER_UI_EXPORTING_FAILURE_TIPS 9124
#define IDS_PASSWORD_MANAGER_UI_EXPORTING_FAILURE_TIP_ENOUGH_SPACE 9125
#define IDS_PASSWORD_MANAGER_UI_EXPORTING_FAILURE_TIP_ANOTHER_FOLDER 9126
#define IDS_PASSWORD_MANAGER_UI_CHECKUP_TITLE 9127
#define IDS_PASSWORD_MANAGER_UI_CHECKUP_RESULT 9128
#define IDS_PASSWORD_MANAGER_UI_JUST_NOW 9129
#define IDS_PASSWORD_MANAGER_UI_COMPROMISED_PASSWORDS_COUNT 9130
#define IDS_PASSWORD_MANAGER_UI_NO_COMPROMISED_PASSWORDS 9131
#define IDS_PASSWORD_MANAGER_UI_HAS_COMPROMISED_PASSWORDS 9132
#define IDS_PASSWORD_MANAGER_UI_REUSED_PASSWORDS_COUNT 9133
#define IDS_PASSWORD_MANAGER_UI_NO_REUSED_PASSWORDS 9134
#define IDS_PASSWORD_MANAGER_UI_HAS_REUSED_PASSWORDS 9135
#define IDS_PASSWORD_MANAGER_UI_WEAK_PASSWORDS_COUNT 9136
#define IDS_PASSWORD_MANAGER_UI_CHECKUP_GREEN_STATE_A11Y 9137
#define IDS_PASSWORD_MANAGER_UI_CHECKUP_YELLOW_STATE_A11Y 9138
#define IDS_PASSWORD_MANAGER_UI_CHECKUP_RED_STATE_A11Y 9139
#define IDS_PASSWORD_MANAGER_UI_NO_WEAK_PASSWORDS 9140
#define IDS_PASSWORD_MANAGER_UI_HAS_WEAK_PASSWORDS 9141
#define IDS_PASSWORD_MANAGER_UI_CHECK_PASSWORDS_AFTER_ERROR 9142
#define IDS_PASSWORD_MANAGER_UI_BLOCKED_SITES_TITLE 9143
#define IDS_PASSWORD_MANAGER_UI_BLOCKED_SITES_DESCRIPTION 9144
#define IDS_PASSWORD_MANAGER_UI_ALLOWED_ACTOR_LOGIN_SITES_TITLE 9145
#define IDS_PASSWORD_MANAGER_UI_ALLOWED_ACTOR_LOGIN_SITES_DESCRIPTION 9146
#define IDS_PASSWORD_MANAGER_UI_REMOVE_ACTOR_LOGIN_PERMISSION_DIALOG_TITLE 9147
#define IDS_PASSWORD_MANAGER_UI_REMOVE_ACTOR_LOGIN_PERMISSION_DIALOG_DESCRIPTION 9148
#define IDS_PASSWORD_MANAGER_UI_REMOVE_ACTOR_LOGIN_PERMISSION_DIALOG_CONFIRMATION 9149
#define IDS_PASSWORD_MANAGER_UI_NUMBER_OF_ACCOUNTS 9150
#define IDS_PASSWORD_MANAGER_UI_USERNAME_LABEL 9151
#define IDS_PASSWORD_MANAGER_UI_SITES_LABEL 9152
#define IDS_PASSWORD_MANAGER_UI_PASSWORD_LABEL 9153
#define IDS_PASSWORD_MANAGER_UI_NOTE_LABEL 9154
#define IDS_PASSWORD_MANAGER_UI_NO_NOTE_ADDED 9155
#define IDS_PASSWORD_MANAGER_UI_COPY_PASSWORD 9156
#define IDS_PASSWORD_MANAGER_UI_COPY_USERNAME 9157
#define IDS_PASSWORD_MANAGER_UI_SHOW_PASSWORD 9158
#define IDS_PASSWORD_MANAGER_UI_HIDE_PASSWORD 9159
#define IDS_PASSWORD_MANAGER_UI_SHOW_PASSWORD_A11Y 9160
#define IDS_PASSWORD_MANAGER_UI_HIDE_PASSWORD_A11Y 9161
#define IDS_PASSWORD_MANAGER_UI_FEDERATION_LABEL 9162
#define IDS_PASSWORD_MANAGER_UI_USERNAME_COPIED_TO_CLIPBOARD 9163
#define IDS_PASSWORD_MANAGER_UI_PASSWORD_COPIED_TO_CLIPBOARD 9164
#define IDS_PASSWORD_MANAGER_UI_ADD_PASSWORD 9165
#define IDS_PASSWORD_MANAGER_UI_WEBSITE_LABEL 9166
#define IDS_PASSWORD_MANAGER_UI_ADD_PASSWORD_FOOTNOTE 9167
#define IDS_PASSWORD_MANAGER_UI_PASSWORD_LEAKED 9168
#define IDS_PASSWORD_MANAGER_UI_PASSWORD_PHISHED 9169
#define IDS_PASSWORD_MANAGER_UI_PASSWORD_PHISHED_AND_LEAKED 9170
#define IDS_PASSWORD_MANAGER_UI_CHANGE_PASSWORD_BUTTON 9171
#define IDS_PASSWORD_MANAGER_UI_CHANGE_PASSWORD_BUTTON_ARIA_DESCRIPTION 9172
#define IDS_PASSWORD_MANAGER_UI_MORE_ACTIONS 9173
#define IDS_PASSWORD_MANAGER_UI_MORE_ACTIONS_ARIA_DESCRIPTION 9174
#define IDS_PASSWORD_MANAGER_UI_COMPROMISED_PASSWORDS_DESCRIPTION 9175
#define IDS_PASSWORD_MANAGER_UI_REUSED_PASSWORDS_DESCRIPTION 9176
#define IDS_PASSWORD_MANAGER_UI_WEAK_PASSWORDS_DESCRIPTION 9177
#define IDS_PASSWORD_MANAGER_UI_CHECKUP_SIGNED_OUT 9178
#define IDS_PASSWORD_MANAGER_UI_CHECKUP_OFFLINE 9179
#define IDS_PASSWORD_MANAGER_UI_CHECKUP_COMPROMISED_SECTION 9180
#define IDS_PASSWORD_MANAGER_UI_CHECKUP_NO_PASSWORDS 9181
#define IDS_PASSWORD_MANAGER_UI_CHECKUP_QUOTA_LIMIT 9182
#define IDS_PASSWORD_MANAGER_UI_CHECKUP_OTHER_ERROR 9183
#define IDS_PASSWORD_MANAGER_UI_CHECKUP_RUNNING_LABEL 9184
#define IDS_PASSWORD_MANAGER_UI_CHECKUP_PROGRESS 9185
#define IDS_PASSWORD_MANAGER_UI_CHECKUP_CANCELED 9186
#define IDS_PASSWORD_MANAGER_UI_MUTED_COMPROMISED_PASSWORDS 9187
#define IDS_PASSWORD_MANAGER_UI_MUTE_ISSUE 9188
#define IDS_PASSWORD_MANAGER_UI_UNMUTE_ISSUE 9189
#define IDS_PASSWORD_MANAGER_UI_NUMBER_OF_CREDENTIALS_WITH_REUSED_PASSWORD 9190
#define IDS_PASSWORD_MANAGER_UI_HELP 9191
#define IDS_PASSWORD_MANAGER_UI_EDIT_PASSWORD 9198
#define IDS_PASSWORD_MANAGER_UI_PASSWORD_EDIT_FOOTNOTE 9199
#define IDS_PASSWORD_MANAGER_UI_PASSWORD_DELETED 9200
#define IDS_PASSWORD_MANAGER_UI_UNDO 9201
#define IDS_PASSWORD_MANAGER_UI_SEARCH_RESULT 9202
#define IDS_PASSWORD_MANAGER_UI_CHANGE_PASSWORD_IN_APP 9203
#define IDS_PASSWORD_MANAGER_UI_CHANGE_PASSWORD_MANAGER_PIN 9204
#define IDS_PASSWORD_MANAGER_UI_ALREADY_CHANGED_PASSWORD 9205
#define IDS_PASSWORD_MANAGER_UI_EDIT_DISCLAIMER_DESCRIPTION 9206
#define IDS_PASSWORD_MANAGER_UI_EDIT_DISCLAIMER_TITLE 9207
#define IDS_PASSWORD_MANAGER_UI_DELETE_PASSWORD_CONFIRMATION_TITLE 9208
#define IDS_PASSWORD_MANAGER_UI_DELETE_PASSWORD_CONFIRMATION_DESCRIPTION 9209
#define IDS_PASSWORD_MANAGER_UI_SHOW_MORE 9210
#define IDS_PASSWORD_MANAGER_UI_NOT_VALID_WEB_ADDRESS 9211
#define IDS_PASSWORD_MANAGER_UI_MISSING_TLD 9212
#define IDS_PASSWORD_MANAGER_UI_USERNAME_ALREADY_USED 9213
#define IDS_PASSWORD_MANAGER_UI_NOTE_CHARACTER_COUNT 9214
#define IDS_PASSWORD_MANAGER_UI_NOTE_CHARACTER_COUNT_WARNING 9215
#define IDS_PASSWORD_MANAGER_UI_AUTH_TIMED_OUT 9216
#define IDS_PASSWORD_MANAGER_UI_AUTH_TIMED_OUT_DESCRIPTION 9217
#define IDS_PASSWORD_MANAGER_UI_VIEW_EXISTING_PASSWORD 9218
#define IDS_PASSWORD_MANAGER_UI_VIEW_EXISTING_PASSWORD_ARIA_DESCRIPTION 9219
#define IDS_PASSWORD_MANAGER_UI_MOVE_PASSWORDS_TO_ACCOUNT 9220
#define IDS_PASSWORD_MANAGER_UI_MOVE_PASSWORDS_TO_ACCOUNT_DIALOG_DESCRIPTION 9221
#define IDS_PASSWORD_MANAGER_UI_MOVE_SINGLE_PASSWORD_TO_ACCOUNT_DIALOG_DESCRIPTION 9222
#define IDS_PASSWORD_MANAGER_UI_MOVE_PASSWORDS_TO_ACCOUNT_DIALOG_PASSWORDS_TITLE 9223
#define IDS_PASSWORD_MANAGER_UI_OPENS_IN_NEW_TAB 9224
#define IDS_PASSWORD_MANAGER_UI_VIEW_PASSWORD_ARIA_DESCRIPTION 9225
#define IDS_PASSWORD_MANAGER_UI_RUN_CHECKUP_ARIA_DESCRIPTION 9226
#define IDS_PASSWORD_MANAGER_UI_REMOVE_BLOCKED_SITE_ARIA_DESCRIPTION 9227
#define IDS_PASSWORD_MANAGER_UI_EMPTY_STATE_ACCOUNT_STORE_USERS 9228
#define IDS_PASSWORD_MANAGER_UI_EMPTY_STATE_SYNCING_USERS 9229
#define IDS_PASSWORD_MANAGER_UI_EMPTY_STATE_SIGNEDOUT_USERS 9230
#define IDS_PASSWORD_MANAGER_UI_CHECKUP_PROMO_CARD_TITLE 9231
#define IDS_PASSWORD_MANAGER_UI_CHECKUP_PROMO_CARD_DESCRIPTION 9232
#define IDS_PASSWORD_MANAGER_UI_CHECKUP_PROMO_CARD_ACTION 9233
#define IDS_PASSWORD_MANAGER_UI_WEB_PROMO_CARD_TITLE 9234
#define IDS_PASSWORD_MANAGER_UI_WEB_PROMO_CARD_DESCRIPTION 9235
#define IDS_PASSWORD_MANAGER_UI_SHORTCUT_PROMO_CARD_TITLE 9236
#define IDS_PASSWORD_MANAGER_UI_SHORTCUT_PROMO_CARD_DESCRIPTION 9237
#define IDS_PASSWORD_MANAGER_UI_ANY_DEVICE_PROMO_CARD_TITLE 9238
#define IDS_PASSWORD_MANAGER_UI_ANY_DEVICE_PROMO_CARD_DESCRIPTION 9239
#define IDS_PASSWORD_MANAGER_UI_ACCOUNT_STORAGE_TOGGLE_SUB_LABEL 9240
#define IDS_PASSWORD_MANAGER_UI_ACCOUNT_STORAGE_WITH_PASSKEYS_TOGGLE_LABEL 9241
#define IDS_PASSWORD_MANAGER_UI_DELETE_DIALOG_TITLE 9242
#define IDS_PASSWORD_MANAGER_UI_DELETE_DIALOG_BODY 9243
#define IDS_PASSWORD_MANAGER_UI_DELETE_DIALOG_FROM_ACCOUNT_CHECKBOX_LABEL 9244
#define IDS_PASSWORD_MANAGER_UI_DELETE_DIALOG_FROM_DEVICE_CHECKBOX_LABEL 9245
#define IDS_PASSWORD_MANAGER_UI_APPS_LABEL 9246
#define IDS_PASSWORD_MANAGER_UI_SITES_AND_APPS_LABEL 9247
#define IDS_PASSWORD_MANAGER_UI_BACK_TO_CHECKUP_ARIA_DESCRIPTION 9248
#define IDS_PASSWORD_MANAGER_UI_BACK_TO_PASSWORDS_ARIA_DESCRIPTION 9249
#define IDS_PASSWORD_MANAGER_UI_BACK_TO_PASSWORD_SETTINGS_ARIA_DESCRIPTION 9250
#define IDS_PASSWORD_MANAGER_UI_MOVE_PASSWORDS_TITLE 9251
#define IDS_PASSWORD_MANAGER_UI_MOVE_PASSWORDS_DESCRIPTION 9252
#define IDS_PASSWORD_MANAGER_UI_MOVE_PASSWORDS_BUTTON 9253
#define IDS_PASSWORD_MANAGER_UI_NO_PASSWORDS_FOUND 9256
#define IDS_PASSWORD_MANAGER_UI_EDIT_PASSKEY 9257
#define IDS_PASSWORD_MANAGER_UI_DISPLAY_NAME_LABEL 9258
#define IDS_PASSWORD_MANAGER_UI_DISPLAY_NAME_PLACEHOLDER 9259
#define IDS_PASSWORD_MANAGER_UI_USERNAME_PLACEHOLDER 9260
#define IDS_PASSWORD_MANAGER_UI_DELETE_PASSKEY_CONFIRMATION_TITLE 9261
#define IDS_PASSWORD_MANAGER_UI_DELETE_PASSKEY_CONFIRMATION_DESCRIPTION 9262
#define IDS_PASSWORD_MANAGER_UI_PASSKEY_HIDDEN_INFO_LABEL 9263
#define IDS_PASSWORD_MANAGER_UI_PASSKEY_MANAGEMENT_INFO_LABEL 9264
#define IDS_PASSWORD_MANAGER_UI_PASSKEY_UPGRADE_TOGGLE_LABEL 9265
#define IDS_PASSWORD_MANAGER_UI_PASSKEY_UPGRADE_TOGGLE_SUBLABEL 9266
#define IDS_PASSKEYS_MANAGER_UI_UNENROLL_TITLE 9267
#define IDS_PASSKEYS_MANAGER_UI_UNENROLL_DESCRIPTION 9268
#define IDS_PASSKEYS_MANAGER_UI_UNENROLL_BUTTON 9269
#define IDS_PASSKEYS_MANAGER_UI_UNENROLL_TOAST_MESSAGE 9270
#define IDS_PASSWORD_MANAGER_UI_FULL_RESET_TITLE 9271
#define IDS_PASSWORD_MANAGER_UI_FULL_RESET_DESCRIPTION 9272
#define IDS_PASSWORD_MANAGER_UI_FULL_RESET_DELETE_ALL 9273
#define IDS_PASSWORD_MANAGER_UI_FULL_RESET_CONFIRM 9274
#define IDS_PASSWORD_MANAGER_UI_FULL_RESET_SUCCESS_TOAST 9275
#define IDS_PASSWORD_MANAGER_UI_FULL_RESET_CONFIRMATION_TITLE 9276
#define IDS_PASSWORD_MANAGER_UI_FULL_RESET_CONFIRMATION_TITLE_LOCAL 9277
#define IDS_PASSWORD_MANAGER_UI_FULL_RESET_CONFIRMATION_DESCIPTION 9278
#define IDS_PASSWORD_MANAGER_UI_FULL_RESET_DOMAINS_DISPLAY_ONE 9279
#define IDS_PASSWORD_MANAGER_UI_FULL_RESET_DOMAINS_DISPLAY_TWO 9280
#define IDS_PASSWORD_MANAGER_UI_FULL_RESET_DOMAINS_DISPLAY_TWO_AND_X_MORE 9281
#define IDS_PASSWORD_MANAGER_PASSWORDS_COUNTER 9282
#define IDS_PASSWORD_MANAGER_PASSKEYS_COUNTER 9283
#define IDS_PASSWORD_MANAGER_UI_COPY_DISPLAY_NAME_LABEL 9284
#define IDS_PASSWORD_MANAGER_UI_DISPLAY_NAME_COPIED_TO_CLIPBOARD 9285
#define IDS_PASSWORD_MANAGER_UI_PASSKEY_DELETED 9286
#define IDS_PASSWORD_MANAGER_UI_PASSWORD_LIST_ARIA_LABEL 9287
#define IDS_PASSWORD_MANAGER_UI_DEVICE_ONLY_PASSWORDS_ICON_TOOLTIP 9288
#define IDS_PASSWORD_MANAGER_UI_MOVE_TO_ACCOUNT_ICON_TOOLTIP 9289
#define IDS_PASSWORD_MANAGER_UI_PASSWORD_DETAILS_CARD_ARIA_LABEL 9290
#define IDS_PASSWORD_MANAGER_UI_BACKUP_PASSWORD_DETAILS_CARD_ARIA_LABEL 9291
#define IDS_PASSWORD_MANAGER_UI_PASSWORD_DETAILS_CARD_EDIT_BUTTON_ARIA_LABEL 9292
#define IDS_PASSWORD_MANAGER_UI_PASSWORD_DETAILS_CARD_EDIT_BUTTON_NO_USERNAME_ARIA_LABEL 9293
#define IDS_PASSWORD_MANAGER_UI_PASSWORD_DETAILS_CARD_DELETE_BUTTON_ARIA_LABEL 9294
#define IDS_PASSWORD_MANAGER_UI_PASSWORD_DETAILS_CARD_DELETE_BUTTON_NO_USERNAME_ARIA_LABEL 9295
#define IDS_PASSWORD_MANAGER_UI_PASSWORD_DETAILS_CARD_SHARE_BUTTON_ARIA_LABEL 9296
#define IDS_PASSWORD_MANAGER_UI_PASSWORD_DETAILS_CARD_SHARE_BUTTON_NO_USERNAME_ARIA_LABEL 9297
#define IDS_PASSWORD_MANAGER_UI_PASSKEY_DETAILS_CARD_NO_USERNAME_ARIA_LABEL 9298
#define IDS_PASSWORD_MANAGER_UI_PASSKEY_DETAILS_CARD_ARIA_LABEL 9299
#define IDS_PASSWORD_MANAGER_UI_PASSKEY_DETAILS_CARD_EDIT_BUTTON_ARIA_LABEL 9300
#define IDS_PASSWORD_MANAGER_UI_PASSKEY_DETAILS_CARD_EDIT_BUTTON_NO_USERNAME_ARIA_LABEL 9301
#define IDS_PASSWORD_MANAGER_UI_PASSKEY_DETAILS_CARD_DELETE_BUTTON_ARIA_LABEL 9302
#define IDS_PASSWORD_MANAGER_UI_PASSKEY_DETAILS_CARD_DELETE_BUTTON_NO_USERNAME_ARIA_LABEL 9303
#define IDS_PASSWORD_MANAGER_UI_FEDERATED_CREDENTIAL_ARIA_LABEL 9304
#define IDS_PASSWORD_MANAGER_UI_MOVE_SINGLE_PASSWORD_TO_ACCOUNT 9305
#define IDS_PASSWORD_MANAGER_UI_MOVE_SINGLE_PASSWORD_TITLE 9306
#define IDS_PASSWORD_MANAGER_UI_MOVE_SINGLE_PASSWORD_DESCRIPTION 9307
#define IDS_PASSWORD_MANAGER_UI_MOVE_SINGLE_PASSWORD_ACTION_BUTTON 9308
#define IDS_PASSWORD_MANAGER_UI_MOVE_PASSWORDS_IN_SETTINGS_SUB_LABEL 9309
#define IDS_PASSWORD_MANAGER_UI_RELAUNCH_CHROME_PROMO_CARD_TITLE_LINUX 9312
#define IDS_PASSWORD_MANAGER_UI_RELAUNCH_CHROME_PROMO_CARD_DESCRIPTION_LINUX 9313
#define IDS_PASSWORD_MANAGER_UI_RELAUNCH_CHROME_PROMO_CARD_ACTION 9314
#define IDS_PASSWORD_MANAGER_UI_NO_USERNAME 9315
#define IDS_PASSWORD_MANAGER_UI_PASSWORD_MOVED_TOAST_MESSAGE 9316
#define IDS_PASSWORD_MANAGER_UI_MOVE_PASSWORDS_PROMO_CARD_TITLE 9317
#define IDS_PASSWORD_MANAGER_UI_MOVE_PASSWORDS_PROMO_CARD_DESCRIPTION 9318
#define IDS_PASSWORD_MANAGER_UI_MOVE_PASSWORDS_PROMO_CARD_ACTION_BUTTON 9319
#define IDS_PASSWORD_MANAGER_UI_BATCH_UPLOAD_PROMO_CARD_TITLE 9320
#define IDS_PASSWORD_MANAGER_UI_BATCH_UPLOAD_PROMO_CARD_ACTION_BUTTON 9321
#define IDS_PASSWORD_MANAGER_SAVE_IN_ACCOUNT_BUBBLE_DESCRIPTION 9322
#define IDS_PASSWORD_MANAGER_SAVE_IN_ACCOUNT_BUBBLE_TITLE 9323
#define IDS_PASSWORD_MANAGER_SAVE_IN_ACCOUNT_BUBBLE_SAVE_BUTTON 9324
#define IDS_PASSWORD_MANAGER_MANAGEMENT_BUBBLE_LIST_ITEM_DEVICE_ONLY_ACCESSIBLE_TEXT 9325
#define IDS_PASSWORD_MANAGER_UI_PASSWORD_LIST_ITEM_ARIA_LABEL 9326
#define IDS_PASSWORD_MANAGER_UI_PROMO_CARD_ARIA_LABEL 9327
#define IDS_PASSWORD_MANAGER_UI_CLOSE_PROMO_CARD_BUTTON_ARIA_LABEL 9328
#define IDS_PASSWORD_MANAGER_PIN_CHANGED 9329
#define IDS_PASSWORD_MANAGER_UI_CONFIRM 9330
#define IDS_PASSWORD_MANAGER_UI_DISCONNECT_CLOUD_AUTHENTICATOR_DIALOG_TITLE 9331
#define IDS_PASSWORD_MANAGER_UI_DISCONNECT_CLOUD_AUTHENTICATOR_DIALOG_DESCRIPTION 9332
#define IDS_PASSWORD_MANAGER_UI_PASSWORD_CHANGED_TITLE 9333
#define IDS_PASSWORD_MANAGER_UI_PASSWORD_CHANGE_FAILED_TITLE 9334
#define IDS_PASSWORD_MANAGER_UI_PASSWORD_CHANGE_FAILED_BODY 9335
#define IDS_PASSWORD_MANAGER_UI_PASSWORD_CHANGE_FAILED_ACCEPT_BUTTON 9336
#define IDS_PASSWORD_MANAGER_UI_PASSWORD_CHANGE_OMNIBOX_SIGN_IN_CHECK 9337
#define IDS_PASSWORD_MANAGER_UI_PASSWORD_CHANGE_OMNIBOX_CHANGING_PASSWORD 9338
#define IDS_PASSWORD_MANAGER_UI_PASSWORD_CHANGE_TOAST_SIGN_IN_TO_CONTINUE 9339
#define IDS_PASSWORD_MANAGER_UI_PASSWORD_CHANGE_CHANGE_PASSWORD 9340
#define IDS_PASSWORD_MANAGER_UI_PASSWORD_CHANGE_LEAK_DIALOG_TITLE 9341
#define IDS_PASSWORD_MANAGER_UI_PASSWORD_CHANGE_LEAK_DIALOG_DETAILS_WITHOUT_PRIVACY_NOTICE 9342
#define IDS_PASSWORD_MANAGER_UI_PASSWORD_CHANGE_LEAK_DIALOG_DETAILS_WITH_PRIVACY_NOTICE 9343
#define IDS_PASSWORD_MANAGER_UI_PASSWORD_CHANGE_LEAK_DIALOG_LINK_WITHOUT_PRIVACY_NOTICE 9344
#define IDS_PASSWORD_MANAGER_UI_PASSWORD_CHANGE_LEAK_DIALOG_LINK_WITH_PRIVACY_NOTICE 9345
#define IDS_PASSWORD_MANAGER_UI_PASSWORD_CHANGE_PRIVACY_NOTICE 9346
#define IDS_PASSWORD_MANAGER_UI_PASSWORD_CHANGE_SETTINGS_DESCRIPTION 9347
#define IDS_PASSWORD_MANAGER_UI_PASSWORD_CHANGE_SETTINGS_TITLE 9348
#define IDS_SETTINGS_PASSWORD_CHANGE_SUBLABEL_WITH_LEARN_MORE 9349
#define IDS_PASSWORD_MANAGER_UI_BACKUP_PASSWORD_SETTINGS_DESCRIPTION 9350
#define IDS_PASSWORD_MANAGER_UI_BACKUP_PASSWORD_SETTINGS_DESCRIPTION_DETAILS 9351
#define IDS_PASSWORD_MANAGER_UI_VIEW_DETAILS_BUTTON 9352
#define IDS_PASSWORD_MANAGER_UI_USER_INTERVENTION_NEEDED_TITLE 9353
#define IDS_PASSWORD_MANAGER_UI_USER_INTERVENTION_NEEDED_DIALOG_DETAILS 9354
#define IDS_PASSWORD_MANAGER_UI_PASSWORD_CHANGE_TAKE_OVER_TASK_BUTTON 9355
#define IDS_PASSWORD_MANAGER_UI_PASSWORD_UNCHANGED 9356
#define IDS_PASSWORD_MANAGER_UI_PASSWORD_CHANGE_TOAST_RETRY_BUTTON 9357
#define IDS_UTILITY_PROCESS_PRINTING_SERVICE_NAME 9358
#define IDS_UTILITY_PROCESS_PRINT_BACKEND_SERVICE_NAME 9359
#define IDS_PRINT_INVALID_PRINTER_SETTINGS 9360
#define IDS_PRINT_PREVIEW_TITLE 9361
#define IDS_PRINT_PREVIEW_DESCRIPTION 9362
#define IDS_PRINT_PREVIEW_LOADING 9363
#define IDS_PRINT_PREVIEW_FAILED 9364
#define IDS_PRINT_PREVIEW_INVALID_PRINTER_SETTINGS 9365
#define IDS_PRINT_PREVIEW_PRINT_BUTTON 9366
#define IDS_PRINT_PREVIEW_SAVE_BUTTON 9367
#define IDS_PRINT_PREVIEW_PRINTING 9368
#define IDS_PRINT_PREVIEW_SAVING 9369
#define IDS_PRINT_PREVIEW_OPTION_ALL_PAGES 9370
#define IDS_PRINT_PREVIEW_OPTION_ODD_PAGES 9371
#define IDS_PRINT_PREVIEW_OPTION_EVEN_PAGES 9372
#define IDS_PRINT_PREVIEW_OPTION_CUSTOM_PAGES 9373
#define IDS_PRINT_PREVIEW_DESTINATION_LABEL 9374
#define IDS_PRINT_PREVIEW_OPTION_BW 9375
#define IDS_PRINT_PREVIEW_OPTION_COLLATE 9376
#define IDS_PRINT_PREVIEW_OPTION_COLOR 9377
#define IDS_PRINT_PREVIEW_OPTION_LANDSCAPE 9378
#define IDS_PRINT_PREVIEW_OPTION_PORTRAIT 9379
#define IDS_PRINT_PREVIEW_OPTION_TWO_SIDED 9380
#define IDS_PRINT_PREVIEW_PRINT_ON_BOTH_SIDES_LABEL 9381
#define IDS_PRINT_PREVIEW_OPTION_LONG_EDGE 9382
#define IDS_PRINT_PREVIEW_OPTION_SHORT_EDGE 9383
#define IDS_PRINT_PREVIEW_PAGES_LABEL 9384
#define IDS_PRINT_PREVIEW_LAYOUT_LABEL 9385
#define IDS_PRINT_PREVIEW_COPIES_LABEL 9386
#define IDS_PRINT_PREVIEW_SCALING_LABEL 9387
#define IDS_PRINT_PREVIEW_OPTION_DEFAULT_SCALING 9388
#define IDS_PRINT_PREVIEW_OPTION_CUSTOM_SCALING 9389
#define IDS_PRINT_PREVIEW_PAGES_PER_SHEET_LABEL 9390
#define IDS_PRINT_PREVIEW_EXAMPLE_PAGE_RANGE_TEXT 9391
#define IDS_PRINT_PREVIEW_PRINT_TO_PDF 9392
#define IDS_PRINT_PREVIEW_SHEET_SUMMARY_LABEL 9393
#define IDS_PRINT_PREVIEW_PAGE_SUMMARY_LABEL 9394
#define IDS_PRINT_PREVIEW_PAGE_RANGE_SYNTAX_INSTRUCTION 9395
#define IDS_PRINT_PREVIEW_PAGE_RANGE_LIMIT_INSTRUCTION_WITH_VALUE 9396
#define IDS_PRINT_PREVIEW_COPIES_INSTRUCTION 9397
#define IDS_PRINT_PREVIEW_SCALING_INSTRUCTION 9398
#define IDS_PRINT_PREVIEW_PRINT_PAGES_LABEL 9399
#define IDS_PRINT_PREVIEW_OPTIONS_LABEL 9400
#define IDS_PRINT_PREVIEW_OPTION_HEADER_FOOTER 9401
#define IDS_PRINT_PREVIEW_OPTION_FIT_TO_PAGE 9402
#define IDS_PRINT_PREVIEW_OPTION_FIT_TO_PAPER 9403
#define IDS_PRINT_PREVIEW_OPTION_ACTUAL_SIZE 9404
#define IDS_PRINT_PREVIEW_OPTION_BACKGROUND_COLORS_AND_IMAGES 9405
#define IDS_PRINT_PREVIEW_OPTION_SELECTION_ONLY 9406
#define IDS_PRINT_PREVIEW_OPTION_RASTERIZE 9407
#define IDS_PRINT_PREVIEW_MARGINS_LABEL 9408
#define IDS_PRINT_PREVIEW_DEFAULT_MARGINS 9409
#define IDS_PRINT_PREVIEW_NO_MARGINS 9410
#define IDS_PRINT_PREVIEW_CUSTOM_MARGINS 9411
#define IDS_PRINT_PREVIEW_MINIMUM_MARGINS 9412
#define IDS_PRINT_PREVIEW_TOP_MARGIN_LABEL 9413
#define IDS_PRINT_PREVIEW_BOTTOM_MARGIN_LABEL 9414
#define IDS_PRINT_PREVIEW_LEFT_MARGIN_LABEL 9415
#define IDS_PRINT_PREVIEW_RIGHT_MARGIN_LABEL 9416
#define IDS_PRINT_PREVIEW_MEDIA_SIZE_LABEL 9417
#define IDS_PRINT_PREVIEW_DPI_LABEL 9418
#define IDS_PRINT_PREVIEW_NON_ISOTROPIC_DPI_ITEM_LABEL 9419
#define IDS_PRINT_PREVIEW_DPI_ITEM_LABEL 9420
#define IDS_PRINT_PREVIEW_DESTINATION_SEARCH_TITLE 9421
#define IDS_PRINT_PREVIEW_SEARCH_BOX_PLACEHOLDER 9422
#define IDS_PRINT_PREVIEW_NO_DESTINATIONS_MESSAGE 9423
#define IDS_PRINT_PREVIEW_PRINT_DESTINATIONS_TITLE 9424
#define IDS_PRINT_PREVIEW_MANAGE 9425
#define IDS_PRINT_PREVIEW_SEE_MORE 9426
#define IDS_PRINT_PREVIEW_SEE_MORE_DESTINATIONS_LABEL 9427
#define IDS_PRINT_PREVIEW_EXTENSION_DESTINATION_ICON_TOOLTIP 9428
#define IDS_MORE_OPTIONS_LABEL 9429
#define IDS_PRINT_PREVIEW_COULD_NOT_PRINT 9430
#define IDS_PRINT_PREVIEW_ADVANCED_SETTINGS_SEARCH_BOX_PLACEHOLDER 9431
#define IDS_PRINT_PREVIEW_ADVANCED_SETTINGS_DIALOG_TITLE 9432
#define IDS_PRINT_PREVIEW_NO_ADVANCED_SETTINGS_MATCH_SEARCH_HINT 9433
#define IDS_PRINT_PREVIEW_ADVANCED_SETTINGS_DIALOG_CONFIRM 9434
#define IDS_PRINT_PREVIEW_NEW_SHOW_ADVANCED_OPTIONS 9435
#define IDS_PRINT_PREVIEW_BUTTON_SELECT 9436
#define IDS_PRINT_PREVIEW_BUTTON_GO_BACK 9437
#define IDS_PRINT_PREVIEW_RESOLVE_EXTENSION_USB_DIALOG_TITLE 9438
#define IDS_PRINT_PREVIEW_RESOLVE_EXTENSION_USB_PERMISSION_MESSAGE 9439
#define IDS_PRINT_PREVIEW_RESOLVE_EXTENSION_USB_ERROR_MESSAGE 9440
#define IDS_PRINT_PREVIEW_MANAGED_SETTINGS_TEXT 9441
#define IDS_PRINT_PREVIEW_SYSTEM_DIALOG_OPTION 9472
#define IDS_DEFAULT_PRINT_DOCUMENT_TITLE 9475
#define IDS_PRINT_SPOOL_FAILED_TITLE_TEXT 9476
#define IDS_PRINT_SPOOL_FAILED_ERROR_TEXT 9477
#define IDS_SKILL_SAVED_TOAST_BODY 9478
#define IDS_SKILL_DELETED_TOAST_BODY 9479
#define IDS_SKILL_UNDO_TOAST_BUTTON 9480
#define IDS_SKILL_CARD_ACTION_MENU_LABEL 9481
#define IDS_ADD_NEW_SKILL_LABEL 9482
#define IDS_SKILL_PAGE_TITLE 9483
#define IDS_SKILL_PAGE_SEARCH_BAR_PLACEHOLDER_TEXT 9484
#define IDS_SKILL_PAGE_MAIN_MENU 9485
#define IDS_SKILLS_SIDEBAR_FOOTER_TEXT 9486
#define IDS_SKILLS_SIDEBAR_FOOTER_BRANDING 9487
#define IDS_SKILLS_ERROR_PAGE_TITLE 9488
#define IDS_SKILLS_ERROR_PAGE_DESCRIPTION 9489
#define IDS_SKILLS_DISABLED_ERROR_PAGE_DESCRIPTION 9490
#define IDS_SKILLS_GO_TO_SETTINGS 9491
#define IDS_SKILLS_NO_SEARCH_RESULT_TITLE 9492
#define IDS_SKILLS_NO_SEARCH_RESULT_DESCRIPTION 9493
#define IDS_SKILL_PAGE_BROWSE_SKILLS_TITLE 9494
#define IDS_SKILL_PAGE_FIRST_PARTY_ADD_SKILL_ERROR_TOAST 9495
#define IDS_SKILLS_PAGE_ALL_CATEGORY 9496
#define IDS_SKILLS_CURATED_PREFIX 9497
#define IDS_SKILL_PAGE_EMPTY_STATE_TITLE 9498
#define IDS_SKILL_PAGE_EMPTY_STATE_DESCRIPTION 9499
#define IDS_SKILL_PAGE_USER_SKILLS_TITLE 9500
#define IDS_SKILL_PAGE_USER_SKILLS_DESCRIPTION 9501
#define IDS_SKILL_PAGE_USER_SKILLS_DELETE 9502
#define IDS_SKILL_PAGE_USER_SKILLS_COPY_INSTRUCTIONS 9503
#define IDS_SKILL_PAGE_INVALID_SKILL_TOAST 9504
#define IDS_SKILLS_DIALOG_ADD_SKILL_HEADER 9505
#define IDS_SKILLS_DIALOG_EDIT_SKILL_HEADER 9506
#define IDS_SKILLS_DIALOG_DESCRIPTION 9507
#define IDS_SKILLS_DIALOG_NAME_LABEL 9508
#define IDS_SKILLS_DIALOG_NAME_PLACEHOLDER 9509
#define IDS_SKILLS_DIALOG_CHOOSE_ICON_TOOLTIP 9510
#define IDS_SKILLS_DIALOG_INSTRUCTIONS_LABEL 9511
#define IDS_SKILLS_DIALOG_INSTRUCTIONS_PLACEHOLDER 9512
#define IDS_SKILLS_DIALOG_UNDO_TOOLTIP 9513
#define IDS_SKILLS_DIALOG_REDO_TOOLTIP 9514
#define IDS_SKILLS_DIALOG_REFINE_TOOLTIP 9515
#define IDS_SKILLS_DIALOG_REFINE_ERROR 9516
#define IDS_SKILLS_DIALOG_CHAR_LIMIT_ERROR 9517
#define IDS_SKILLS_DIALOG_ACCOUNT_INFO 9518
#define IDS_SKILLS_DIALOG_SAVE_ERROR 9519
#define IDS_SKILLS_EMOJI_PICKER_SEARCH_PLACEHOLDER 9520
#define IDS_SKILLS_EMOJI_PICKER_ARIA_LABEL 9521
#define IDS_VR_SHELL_SITE_IS_TRACKING_LOCATION 9522
#define IDS_VR_SHELL_SITE_IS_USING_MICROPHONE 9523
#define IDS_VR_SHELL_SITE_IS_USING_CAMERA 9524
#define IDS_VR_SHELL_SITE_IS_SHARING_SCREEN 9525
#define IDS_VR_SHELL_BG_IS_USING_MICROPHONE 9526
#define IDS_VR_SHELL_BG_IS_USING_CAMERA 9527
#define IDS_VR_SHELL_BG_IS_SHARING_SCREEN 9528
#define IDS_VR_SHELL_SITE_CAN_TRACK_LOCATION 9529
#define IDS_VR_SHELL_SITE_CAN_USE_MICROPHONE 9530
#define IDS_VR_SHELL_SITE_CAN_USE_CAMERA 9531
#define IDS_VR_SHELL_SITE_CAN_SHARE_SCREEN 9532
#define IDS_VR_SHELL_SITE_IS_USING_BLUETOOTH 9533
#define IDS_VR_SHELL_SITE_CAN_USE_BLUETOOTH 9535
#define IDS_DESKTOP_PROMPT_DOFF_HEADSET 9536
#define IDS_VR_DESKTOP_GENERIC_PERMISSION_PROMPT 9537
#define IDS_VR_SHELL_SITE_IS_USING_USB 9538
#define IDS_VR_SHELL_SITE_IS_USING_MIDI 9539
#define IDS_VR_SHELL_SITE_CAN_USE_MIDI 9540
#define IDS_VR_WEB_VR_TIMEOUT_MESSAGE 9541
#define IDS_WILDCARD 9542
#define IDS_BACKGROUND_CRASHED_APP_BALLOON_MESSAGE 9553
#define IDS_BACKGROUND_CRASHED_EXTENSION_BALLOON_MESSAGE 9554
#define IDS_BACKGROUND_APP_NOT_INSTALLED 9555
#define IDS_PERMISSIONS_REQUESTED_SCREENREADER_ANNOUNCEMENT 9556
#define IDS_PERMISSIONS_EXPIRED_SCREENREADER_ANNOUNCEMENT 9557
#define IDS_PERMISSIONS_BUBBLE_PROMPT 9558
#define IDS_PERMISSIONS_BUBBLE_PROMPT_ACCESSIBLE_TITLE_ONE_PERM 9559
#define IDS_PERMISSIONS_BUBBLE_PROMPT_ACCESSIBLE_TITLE_TWO_PERMS 9560
#define IDS_PERMISSIONS_BUBBLE_PROMPT_ACCESSIBLE_TITLE_TWO_PERMS_MORE 9561
#define IDS_PERMISSIONS_BUBBLE_PROMPT_THIS_FILE 9562
#define IDS_PERMISSION_CUSTOMIZE 9563
#define IDS_ALTERNATE_NAV_URL_VIEW_LABEL 9564
#define IDS_DOWNLOAD_HISTORY_TITLE 9565
#define IDS_TAB_LOADING_TITLE 9566
#define IDS_HOVER_CARD_FILE_URL_SOURCE 9567
#define IDS_HOVER_CARD_BLOB_URL_SOURCE 9568
#define IDS_HOVER_CARD_VIEW_SOURCE_URL_SOURCE 9569
#define IDS_HOVER_CARD_CRASHED_TITLE 9570
#define IDS_HISTORY_SEARCH_PROMPT 9571
#define IDS_HISTORY_DELETE 9572
#define IDS_HISTORY_OPEN 9573
#define IDS_HISTORY_ITEMS_SELECTED 9574
#define IDS_HISTORY_ITEMS_UNSELECTED 9575
#define IDS_HISTORY_HISTORY_MENU_DESCRIPTION 9576
#define IDS_HISTORY_NO_SYNCED_RESULTS 9577
#define IDS_HISTORY_OPEN_TABS_MENU_ITEM 9578
#define IDS_HISTORY_TURN_ON_SYNC_BUTTON 9579
#define IDS_HISTORY_TURN_ON_SYNC_PROMO 9580
#define IDS_HISTORY_TURN_ON_SYNC_PROMO_DESC 9581
#define IDS_HISTORY_SYNC_HISTORY_PROMO 9582
#define IDS_RECENT_TABS_SYNC_HISTORY_PROMO_BODY_SIGNED_OUT 9583
#define IDS_RECENT_TABS_SYNC_HISTORY_PROMO_BODY_PENDING_SIGN_IN 9584
#define IDS_RECENT_TABS_SYNC_HISTORY_PROMO_BODY_PENDING_SIGN_IN_SYNC_HISTORY_ON 9585
#define IDS_RECENT_TABS_SYNC_HISTORY_PROMO_BODY_SIGNED_IN_SYNC_OFF 9586
#define IDS_HISTORY_SYNC_HISTORY_BUTTON 9587
#define IDS_EDIT 9588
#define IDS_CLEAR_SEARCH 9589
#define IDS_CONFIRM 9590
#define IDS_DISABLE 9591
#define IDS_SEARCH_CLEARED 9592
#define IDS_SEARCH_RESULTS 9593
#define IDS_SEARCH_RESULTS_SINGULAR 9594
#define IDS_SEARCH_RESULTS_PLURAL 9595
#define IDS_SEARCH_RESULT_BUBBLE_TEXT 9596
#define IDS_SEARCH_RESULTS_BUBBLE_TEXT 9597
#define IDS_SEARCH_NO_RESULTS 9598
#define IDS_SHOW_BUBBLE_INACTIVE_DESCRIPTION 9599
#define IDS_FOCUS_HELP_BUBBLE_DESCRIPTION 9600
#define IDS_FOCUS_HELP_BUBBLE_TOGGLE_DESCRIPTION 9601
#define IDS_FOCUS_HELP_BUBBLE_TUTORIAL_DESCRIPTION 9602
#define IDS_CONTENT_CONTEXT_INSPECTELEMENT 9603
#define IDS_CONTENT_CONTEXT_INSPECTELEMENT_WITH_GEMINI 9604
#define IDS_CONTENT_CONTEXT_INSPECTELEMENT_WITH_DEVTOOLS 9605
#define IDS_CONTENT_CONTEXT_ACCESSIBILITY_LABELS_DIALOG_TITLE 9606
#define IDS_CONTENT_CONTEXT_ACCESSIBILITY_LABELS_BUBBLE_ENABLE 9607
#define IDS_CONTENT_CONTEXT_ACCESSIBILITY_LABELS_BUBBLE_DISABLE 9608
#define IDS_CONTENT_CONTEXT_LIVE_CAPTION_ENABLE 9609
#define IDS_CONTENT_CONTEXT_LIVE_CAPTION_DISABLE 9610
#define IDS_CONTENT_CONTEXT_BACK 9611
#define IDS_CONTENT_CONTEXT_FORWARD 9612
#define IDS_CONTENT_CONTEXT_SAVEPAGEAS 9613
#define IDS_CONTENT_CONTEXT_PRINT 9614
#define IDS_CONTENT_CONTEXT_PRINT_SELECTION 9615
#define IDS_CONTENT_CONTEXT_VIEWPAGESOURCE 9616
#define IDS_CONTENT_CONTEXT_OPENLINKWITH 9617
#define IDS_CONTENT_CONTEXT_OPENLINKWITH_CONFIGURE 9618
#define IDS_CONTENT_CONTEXT_INSPECTBACKGROUNDPAGE 9619
#define IDS_CONTENT_CONTEXT_READING_MODE 9620
#define IDS_CONTENT_CONTEXT_READING_MODE_SELECTION 9621
#define IDS_CONTENT_CONTEXT_LISTEN_TO_THIS_PAGE 9622
#define IDS_CONTENT_CONTEXT_CLOSE_GLIC 9623
#define IDS_CONTENT_CONTEXT_ARCHIVE_GLIC 9624
#define IDS_CONTENT_CONTEXT_DICTATION 9625
#define IDS_CONTENT_CONTEXT_RELOAD 9626
#define IDS_CONTENT_CONTEXT_RESTART_APP 9627
#define IDS_CONTENT_CONTEXT_RELOAD_PACKAGED_APP 9628
#define IDS_CONTENT_CONTEXT_TRANSLATE 9629
#define IDS_CONTENT_CONTEXT_EXIT_FULLSCREEN 9630
#define IDS_CONTENT_CONTEXT_RELOADFRAME 9631
#define IDS_CONTENT_CONTEXT_VIEWFRAMESOURCE 9632
#define IDS_CONTENT_CONTEXT_OPENLINKNEWTAB 9633
#define IDS_CONTENT_CONTEXT_OPENLINKNEWWINDOW 9634
#define IDS_CONTENT_CONTEXT_OPENLINKOFFTHERECORD 9635
#define IDS_CONTENT_CONTEXT_OPENLINKOFFTHERECORD_V2 9636
#define IDS_CONTENT_CONTEXT_OPENLINKSPLITVIEW 9637
#define IDS_CONTENT_CONTEXT_OPENLINKLEFTVIEW 9638
#define IDS_CONTENT_CONTEXT_OPENLINKRIGHTVIEW 9639
#define IDS_CONTENT_CONTEXT_OPENLINKTOPVIEW 9640
#define IDS_CONTENT_CONTEXT_OPENLINKBOTTOMVIEW 9641
#define IDS_CONTENT_CONTEXT_OPENLINKINPROFILES 9642
#define IDS_CONTENT_CONTEXT_OPENLINKINPROFILES_V2 9643
#define IDS_CONTENT_CONTEXT_OPENLINKINPROFILE 9644
#define IDS_CONTENT_CONTEXT_OPENLINKBOOKMARKAPP 9645
#define IDS_CONTENT_CONTEXT_OPENLINKBOOKMARKAPP_SAMEAPP 9646
#define IDS_CONTENT_CONTEXT_OPENLINKWEBAPP_NEWTAB 9647
#define IDS_CONTENT_CONTEXT_SAVELINKAS 9648
#define IDS_CONTENT_CONTEXT_COPYLINKLOCATION 9649
#define IDS_CONTENT_CONTEXT_COPYEMAILADDRESS 9650
#define IDS_CONTENT_CONTEXT_COPYLINKTEXT 9651
#define IDS_CONTENT_CONTEXT_COPYLINKTOTEXT 9652
#define IDS_CONTENT_CONTEXT_ADD_LINK_TO_READING_LIST 9653
#define IDS_CONTENT_CONTEXT_ADD_A_NOTE 9654
#define IDS_CONTENT_CONTEXT_REMOVELINKTOTEXT 9655
#define IDS_CONTENT_CONTEXT_RESHARELINKTOTEXT 9656
#define IDS_CONTENT_CONTEXT_SAVEIMAGEAS 9657
#define IDS_CONTENT_CONTEXT_COPYIMAGELOCATION 9658
#define IDS_CONTENT_CONTEXT_COPYIMAGE 9659
#define IDS_CONTENT_CONTEXT_OPENIMAGENEWTAB 9660
#define IDS_CONTENT_CONTEXT_OPEN_ORIGINAL_IMAGE_NEW_TAB 9661
#define IDS_CONTENT_CONTEXT_LOAD_IMAGE 9662
#define IDS_CONTENT_CONTEXT_LOOP 9663
#define IDS_CONTENT_CONTEXT_CONTROLS 9664
#define IDS_CONTENT_CONTEXT_ROTATECW 9665
#define IDS_CONTENT_CONTEXT_ROTATECCW 9666
#define IDS_CONTENT_CONTEXT_VIDEO_FRAME 9667
#define IDS_CONTENT_CONTEXT_SAVEVIDEOFRAMEAS 9668
#define IDS_CONTENT_CONTEXT_SAVEVIDEOAS 9669
#define IDS_CONTENT_CONTEXT_COPYVIDEOLOCATION 9670
#define IDS_CONTENT_CONTEXT_COPYVIDEOFRAME 9671
#define IDS_CONTENT_CONTEXT_SEARCHFORVIDEOFRAME 9672
#define IDS_CONTENT_CONTEXT_OPENVIDEONEWTAB 9673
#define IDS_CONTENT_CONTEXT_SAVEAUDIOAS 9674
#define IDS_CONTENT_CONTEXT_COPYAUDIOLOCATION 9675
#define IDS_CONTENT_CONTEXT_OPENAUDIONEWTAB 9676
#define IDS_CONTENT_CONTEXT_PICTUREINPICTURE 9677
#define IDS_CONTENT_CONTEXT_UNDO 9678
#define IDS_CONTENT_CONTEXT_REDO 9679
#define IDS_CONTENT_CONTEXT_CUT 9680
#define IDS_CONTENT_CONTEXT_COPY 9681
#define IDS_CONTENT_CONTEXT_COPY_SELECTION 9682
#define IDS_CONTENT_CONTEXT_PASTE 9683
#define IDS_CONTENT_CONTEXT_PASTE_AND_MATCH_STYLE 9684
#define IDS_CONTENT_CONTEXT_ADD_TO_DICTIONARY 9685
#define IDS_CONTENT_CONTEXT_ACCESSIBILITY_LABELS_MENU_OPTION 9686
#define IDS_CONTENT_CONTEXT_ACCESSIBILITY_LABELS_SEND 9687
#define IDS_CONTENT_CONTEXT_ACCESSIBILITY_LABELS_SEND_ONCE 9688
#define IDS_CONTENT_CONTEXT_RUN_LAYOUT_EXTRACTION 9689
#define IDS_CONTENT_CONTEXT_SPELLING_ASK_GOOGLE 9690
#define IDS_CONTENT_CONTEXT_SPELLING_BUBBLE_TITLE 9691
#define IDS_CONTENT_CONTEXT_SPELLING_BUBBLE_ENABLE 9692
#define IDS_CONTENT_CONTEXT_SPELLING_BUBBLE_DISABLE 9693
#define IDS_CONTENT_CONTEXT_SPELLING_CHECKING 9694
#define IDS_CONTENT_CONTEXT_SPELLING_NO_SUGGESTIONS_FROM_GOOGLE 9695
#define IDS_CONTENT_CONTEXT_SELECTALL 9696
#define IDS_CONTENT_CONTEXT_PARTIAL_TRANSLATE 9697
#define IDS_CONTENT_CONTEXT_PARTIAL_TRANSLATE_SELECTION 9698
#define IDS_CONTENT_CONTEXT_SEARCHWEBFOR 9699
#define IDS_CONTENT_CONTEXT_SEARCHWEBFORNEWTAB 9700
#define IDS_CONTENT_CONTEXT_SEARCHWEBFORIMAGE 9701
#define IDS_CONTENT_CONTEXT_SEARCHLENSFORIMAGE 9702
#define IDS_CONTENT_CONTEXT_LENS_REGION_SEARCH_ALT1 9703
#define IDS_CONTENT_CONTEXT_LENS_REGION_SEARCH 9704
#define IDS_CONTENT_CONTEXT_GOTOURL 9705
#define IDS_CONTENT_CONTEXT_GENERATEPASSWORD 9706
#define IDS_CONTENT_CONTEXT_AUTOFILL_FALLBACK_PASSWORDS_USE_PASSKEY_FROM_ANOTHER_DEVICE 9707
#define IDS_CONTENT_CONTEXT_MORE_APPS 9708
#define IDS_CONTENT_CONTEXT_OPEN_WITH_APP 9709
#define IDS_CONTENT_CONTEXT_AUTOFILL_FEEDBACK 9710
#define IDS_CONTENT_CONTEXT_AUTOFILL_FALLBACK_AT_MEMORY 9711
#define IDS_CONTENT_CONTEXT_AUTOFILL_FALLBACK_PASSWORDS_SELECT_PASSWORD 9712
#define IDS_CONTENT_CONTEXT_AUTOFILL_FALLBACK_PASSWORDS_IMPORT_PASSWORDS 9713
#define IDS_CONTENT_CONTEXT_AUTOFILL_FALLBACK_PASSWORDS_SUGGEST_PASSWORD 9714
#define IDS_SHARE_MENU_TITLE 9717
#define IDS_CONTENT_CONTEXT_PLUGIN_RUN 9718
#define IDS_CONTENT_CONTEXT_PLUGIN_HIDE 9719
#define IDS_CONTENT_CONTEXT_ENABLE_FLASH 9720
#define IDS_CONTENT_CONTEXT_SPELLCHECK_MENU 9722
#define IDS_CONTENT_CONTEXT_LANGUAGE_SETTINGS 9721
#define IDS_CONTENT_CONTEXT_SPELLCHECK_MULTI_LINGUAL 9723
#define IDS_CONTENT_CONTEXT_CHECK_SPELLING_WHILE_TYPING 9724
#define IDS_PROFILE_ROW_VERIFY_MESSAGE 9725
#define IDS_ADDRESSES_AND_MORE_SUBMENU_OPTION 9726
#define IDS_YOUR_SAVED_INFO_CONTACT_INFO_SUBMENU_OPTION 9727
#define IDS_PASSWORDS_AND_AUTOFILL_MENU 9728
#define IDS_RELAUNCH_TO_UPDATE_ALT 9729
#define IDS_RELAUNCH_TO_UPDATE_ALT_MINOR_TEXT 9730
#define IDS_NEW_TAB 9731
#define IDS_NEW_INCOGNITO_TAB 9732
#define IDS_SHOW_AS_TAB 9733
#define IDS_NEW_WINDOW 9734
#define IDS_NEW_INCOGNITO_WINDOW 9735
#define IDS_PROFILE_ROW_SIGNED_IN_MESSAGE_WITH_EMAIL 9736
#define IDS_PROFILE_ROW_SIGNED_IN_MESSAGE 9737
#define IDS_PROFILE_ROW_SYNC_IS_ON 9738
#define IDS_PROFILE_ROW_TURN_ON_SYNC 9739
#define IDS_PROFILE_ROW_SYNC_ERROR_MESSAGE 9740
#define IDS_EDIT2 9741
#define IDS_CUT 9742
#define IDS_COPY 9743
#define IDS_PASTE 9744
#define IDS_DELETE 9745
#define IDS_FIND 9746
#define IDS_FIND_AND_EDIT_MENU 9747
#define IDS_PAYMENT_METHOD_SUBMENU_OPTION 9748
#define IDS_YOUR_SAVED_INFO_PAYMENTS_SUBMENU_OPTION 9749
#define IDS_SAVE_AND_SHARE_MENU 9750
#define IDS_CAST_SAVE_AND_SHARE_MENU 9751
#define IDS_SAVE_AND_SHARE_MENU_CAST 9752
#define IDS_SAVE_AND_SHARE_MENU_SAVE 9753
#define IDS_SAVE_AND_SHARE_MENU_SHARE 9754
#define IDS_SAVE_PAGE 9755
#define IDS_MORE_TOOLS_MENU 9756
#define IDS_ZOOM_MENU 9757
#define IDS_ZOOM_MENU2 9758
#define IDS_ZOOM_PLUS 9759
#define IDS_ZOOM_PLUS2 9760
#define IDS_ZOOM_NORMAL 9761
#define IDS_ZOOM_MINUS 9762
#define IDS_ZOOM_MINUS2 9763
#define IDS_COPY_URL 9764
#define IDS_OPEN_IN_APP_WINDOW 9765
#define IDS_MOVE_TAB_TO_NEW_WINDOW 9766
#define IDS_SEARCH_TABS 9767
#define IDS_CLOSE_PROFILE 9768
#define IDS_MANAGE_GOOGLE_ACCOUNT 9769
#define IDS_OPEN_GUEST_PROFILE 9770
#define IDS_ADD_NEW_PROFILE 9771
#define IDS_ACCNAME_ZOOM_PLUS2 9775
#define IDS_ACCNAME_ZOOM_MINUS2 9776
#define IDS_VIEW_SOURCE 9777
#define IDS_FEEDBACK 9778
#define IDS_DEV_TOOLS 9779
#define IDS_DEV_TOOLS_ELEMENTS 9780
#define IDS_DEV_TOOLS_CONSOLE 9781
#define IDS_DEV_TOOLS_DEVICES 9782
#define IDS_TASK_MANAGER 9783
#define IDS_TAKE_SCREENSHOT 9784
#define IDS_REPORT_UNSAFE_SITE 9785
#define IDS_RESTORE_ALL_TABS 9786
#define IDS_RESTORE_TAB 9787
#define IDS_REOPEN_WINDOW 9788
#define IDS_REOPEN_GROUP 9789
#define IDS_REOPEN_SPLIT 9790
#define IDS_RESTORE_WINDOW 9791
#define IDS_RESTORE_GROUP 9792
#define IDS_NAME_WINDOW 9793
#define IDS_CHROMELABS 9794
#define IDS_PREFERENCES 9795
#define IDS_TOS_NOTIFICATION_TITLE 9797
#define IDS_TOS_NOTIFICATION_BODY_TEXT 9798
#define IDS_TOS_NOTIFICATION_ACK_BUTTON_TEXT 9799
#define IDS_TOS_NOTIFICATION_REVIEW_BUTTON_TEXT 9800
#define IDS_TOS_NOTIFICATION_LINK 9801
#define IDS_HELP_MENU 9802
#define IDS_MANAGED 9803
#define IDS_MANAGED_BY 9804
#define IDS_MANAGED_BY_PARENT 9805
#define IDS_PROFILE_MANAGED_BY 9806
#define IDS_BROWSER_PROFILE_MANAGED 9807
#define IDS_CHROME_WHATS_NEW 9808
#define IDS_IMPORT_SETTINGS_MENU_LABEL 9809
#define IDS_PROFILING_ENABLED 9810
#define IDS_FULLSCREEN 9811
#define IDS_CLEAR_BROWSING_DATA 9812
#define IDS_SHOW_DOWNLOADS 9813
#define IDS_FIND_EXTENSIONS 9814
#define IDS_SHOW_EXTENSIONS 9815
#define IDS_EXTENSIONS_SUBMENU 9816
#define IDS_EXTENSIONS_SUBMENU_MANAGE_EXTENSIONS_ITEM 9817
#define IDS_EXTENSIONS_SUBMENU_CHROME_WEBSTORE_ITEM 9818
#define IDS_SHOW_PERFORMANCE 9819
#define IDS_SHOW_SEARCH_COMPANION 9820
#define IDS_SHOW_TRANSLATE 9821
#define IDS_SETTINGS 9822
#define IDS_OPTIONS 9823
#define IDS_HELP_PAGE 9824
#define IDS_BETA_FORUM 9825
#define IDS_GET_HELP 9826
#define IDS_EXIT 9827
#define IDS_AUTOCOMPLETE_MATCH_DESCRIPTION_SEPARATOR 9828
#define IDS_MANAGE_SEARCH_ENGINES 9829
#define IDS_MANAGE_SEARCH_ENGINES_AND_SITE_SEARCH 9830
#define IDS_MANAGE_SEARCH_ENGINES_AND_SHORTCUTS 9831
#define IDS_SEARCH_ENGINES_EDITOR_KEYWORD_COLUMN 9832
#define IDS_SEARCH_ENGINES_EDITOR_DESCRIPTION_COLUMN 9833
#define IDS_SEARCH_ENGINES_EDITOR_DEFAULT_ENGINE 9834
#define IDS_DOWNLOAD_INTERRUPTED_STATUS 9835
#define IDS_DOWNLOAD_INTERRUPTED_STATUS_ACCESS_DENIED 9836
#define IDS_DOWNLOAD_INTERRUPTED_DESCRIPTION_ACCESS_DENIED 9837
#define IDS_DOWNLOAD_INTERRUPTED_STATUS_PATH_TOO_LONG 9838
#define IDS_DOWNLOAD_INTERRUPTED_DESCRIPTION_PATH_TOO_LONG 9839
#define IDS_DOWNLOAD_INTERRUPTED_STATUS_DISK_FULL 9840
#define IDS_DOWNLOAD_INTERRUPTED_DESCRIPTION_DISK_FULL 9841
#define IDS_DOWNLOAD_INTERRUPTED_STATUS_FILE_TOO_LARGE 9842
#define IDS_DOWNLOAD_INTERRUPTED_DESCRIPTION_FILE_TOO_LARGE 9843
#define IDS_DOWNLOAD_INTERRUPTED_STATUS_TEMPORARY_PROBLEM 9844
#define IDS_DOWNLOAD_INTERRUPTED_DESCRIPTION_TEMPORARY_PROBLEM 9845
#define IDS_DOWNLOAD_INTERRUPTED_STATUS_VIRUS 9846
#define IDS_DOWNLOAD_INTERRUPTED_DESCRIPTION_VIRUS 9847
#define IDS_DOWNLOAD_INTERRUPTED_STATUS_BLOCKED 9848
#define IDS_DOWNLOAD_INTERRUPTED_STATUS_LOCAL_DOWNLOAD_BLOCKED 9849
#define IDS_DOWNLOAD_INTERRUPTED_STATUS_LOCAL_DOWNLOAD_BLOCKED_DOWNLOAD_UI 9850
#define IDS_DOWNLOAD_INTERRUPTED_DESCRIPTION_BLOCKED 9851
#define IDS_DOWNLOAD_INTERRUPTED_STATUS_CONTENT_LENGTH_MISMATCH 9852
#define IDS_DOWNLOAD_INTERRUPTED_DESCRIPTION_CONTENT_LENGTH_MISMATCH 9853
#define IDS_DOWNLOAD_INTERRUPTED_STATUS_SECURITY_CHECK_FAILED 9854
#define IDS_DOWNLOAD_INTERRUPTED_DESCRIPTION_SECURITY_CHECK_FAILED 9855
#define IDS_DOWNLOAD_INTERRUPTED_STATUS_FILE_TOO_SHORT 9856
#define IDS_DOWNLOAD_INTERRUPTED_DESCRIPTION_FILE_TOO_SHORT 9857
#define IDS_DOWNLOAD_INTERRUPTED_STATUS_FILE_SAME_AS_SOURCE 9858
#define IDS_DOWNLOAD_INTERRUPTED_DESCRIPTION_FILE_SAME_AS_SOURCE 9859
#define IDS_DOWNLOAD_INTERRUPTED_STATUS_NETWORK_TIMEOUT 9860
#define IDS_DOWNLOAD_INTERRUPTED_DESCRIPTION_NETWORK_TIMEOUT 9861
#define IDS_DOWNLOAD_INTERRUPTED_STATUS_NETWORK_DISCONNECTED 9862
#define IDS_DOWNLOAD_INTERRUPTED_DESCRIPTION_NETWORK_DISCONNECTED 9863
#define IDS_DOWNLOAD_INTERRUPTED_STATUS_SERVER_DOWN 9864
#define IDS_DOWNLOAD_INTERRUPTED_DESCRIPTION_SERVER_DOWN 9865
#define IDS_DOWNLOAD_INTERRUPTED_STATUS_NETWORK_ERROR 9866
#define IDS_DOWNLOAD_INTERRUPTED_DESCRIPTION_NETWORK_ERROR 9867
#define IDS_DOWNLOAD_INTERRUPTED_STATUS_NO_FILE 9868
#define IDS_DOWNLOAD_INTERRUPTED_DESCRIPTION_NO_FILE 9869
#define IDS_DOWNLOAD_INTERRUPTED_STATUS_SERVER_PROBLEM 9870
#define IDS_DOWNLOAD_INTERRUPTED_DESCRIPTION_SERVER_PROBLEM 9871
#define IDS_DOWNLOAD_INTERRUPTED_STATUS_SHUTDOWN 9872
#define IDS_DOWNLOAD_INTERRUPTED_DESCRIPTION_SHUTDOWN 9873
#define IDS_DOWNLOAD_INTERRUPTED_STATUS_CRASH 9874
#define IDS_DOWNLOAD_INTERRUPTED_DESCRIPTION_CRASH 9875
#define IDS_DOWNLOAD_INTERRUPTED_STATUS_UNAUTHORIZED 9876
#define IDS_DOWNLOAD_INTERRUPTED_DESCRIPTION_UNAUTHORIZED 9877
#define IDS_DOWNLOAD_INTERRUPTED_STATUS_SERVER_CERT_PROBLEM 9878
#define IDS_DOWNLOAD_INTERRUPTED_DESCRIPTION_SERVER_CERT_PROBLEM 9879
#define IDS_DOWNLOAD_INTERRUPTED_STATUS_FORBIDDEN 9880
#define IDS_DOWNLOAD_INTERRUPTED_DESCRIPTION_FORBIDDEN 9881
#define IDS_DOWNLOAD_INTERRUPTED_STATUS_UNREACHABLE 9882
#define IDS_DOWNLOAD_INTERRUPTED_DESCRIPTION_UNREACHABLE 9883
#define IDS_DOWNLOAD_NOTIFICATION_LABEL_OPEN_WHEN_COMPLETE 9884
#define IDS_DOWNLOAD_NOTIFICATION_LABEL_OPEN 9885
#define IDS_DOWNLOAD_STATUS_STARTING 9886
#define IDS_DOWNLOAD_STATUS_IN_PROGRESS 9896
#define IDS_DOWNLOAD_STATUS_SIZES 9897
#define IDS_DOWNLOAD_STATUS_OPEN_IN 9898
#define IDS_DOWNLOAD_STATUS_OPEN_WHEN_COMPLETE 9899
#define IDS_DOWNLOAD_STATUS_IN_PROGRESS_SHORT 9900
#define IDS_DOWNLOAD_STATUS_CANCELLED 9901
#define IDS_DOWNLOAD_STATUS_REMOVED 9902
#define IDS_DOWNLOAD_STATUS_INTERRUPTED 9903
#define IDS_DOWNLOAD_UNCONFIRMED_PREFIX 9904
#define IDS_PROMPT_DANGEROUS_DOWNLOAD 9905
#define IDS_PROMPT_DANGEROUS_DOWNLOAD_EXTENSION 9906
#define IDS_PROMPT_UNCOMMON_DOWNLOAD_CONTENT 9907
#define IDS_PROMPT_UNCOMMON_DOWNLOAD_CONTENT_IN_ADVANCED_PROTECTION 9908
#define IDS_PROMPT_DEEP_SCANNING_APP_DOWNLOAD 9909
#define IDS_PROMPT_CONFIRM_INSECURE_DOWNLOAD 9910
#define IDS_PROMPT_DOWNLOAD_BLOCKED_TOO_LARGE 9911
#define IDS_PROMPT_DOWNLOAD_BLOCKED_PASSWORD_PROTECTED 9912
#define IDS_PROMPT_DOWNLOAD_SENSITIVE_CONTENT_WARNING 9913
#define IDS_PROMPT_DOWNLOAD_SENSITIVE_CONTENT_BLOCKED 9914
#define IDS_PROMPT_DOWNLOAD_FORCED_SAVE_TO_CLOUD 9915
#define IDS_PROMPT_DOWNLOAD_BLOCKED_SCAN_FAILED 9916
#define IDS_PROMPT_DOWNLOAD_DEEP_SCANNED_OPENED_DANGEROUS 9917
#define IDS_PROMPT_DOWNLOAD_INSECURE_WARNING 9918
#define IDS_PROMPT_DOWNLOAD_INSECURE_BLOCKED 9919
#define IDS_PROMPT_APP_DEEP_SCANNING 9920
#define IDS_PROMPT_DEEP_SCANNING 9921
#define IDS_BLOCK_REASON_UNCOMMON_DOWNLOAD_IN_ADVANCED_PROTECTION 9922
#define IDS_BLOCK_REASON_DEEP_SCANNING_UPDATED 9923
#define IDS_BLOCK_REASON_DEEP_SCANNING_SECOND_UPDATED 9924
#define IDS_BLOCK_REASON_PROMPT_FOR_SCANNING_UPDATED 9925
#define IDS_BLOCK_REASON_PROMPT_FOR_LOCAL_PASSWORD_SCANNING 9926
#define IDS_DEEP_SCANNED_OPENED_DANGEROUS_DESCRIPTION 9927
#define IDS_BLOCK_REASON_SENSITIVE_CONTENT_WARNING 9928
#define IDS_SENSITIVE_CONTENT_BLOCKED_DESCRIPTION 9929
#define IDS_FORCED_SAVE_TO_CLOUD_DESCRIPTION 9930
#define IDS_BLOCKED_TOO_LARGE_DESCRIPTION 9931
#define IDS_BLOCKED_PASSWORD_PROTECTED_DESCRIPTION 9932
#define IDS_DEEP_SCANNED_FAILED_DESCRIPTION 9933
#define IDS_DOWNLOADS_TOAST_DELETED_FROM_HISTORY_STILL_ON_DEVICE 9934
#define IDS_DOWNLOADS_TOAST_DELETED_FROM_HISTORY 9935
#define IDS_DOWNLOADS_TOAST_COPIED_DOWNLOAD_LINK 9936
#define IDS_DOWNLOADS_TOAST_COPY_DOWNLOAD_LINK_FAILED 9937
#define IDS_DOWNLOADS_TOAST_COPIED_LINK 9938
#define IDS_EMAIL_VERIFIED 9939
#define IDS_CONFIRM_KEEP_DANGEROUS_DOWNLOAD_TITLE 9943
#define IDS_CONFIRM_DOWNLOAD 9944
#define IDS_CONTINUE_EXTENSION_DOWNLOAD 9945
#define IDS_DISCARD_DOWNLOAD 9946
#define IDS_OPEN_DOWNLOAD_NOW 9947
#define IDS_SCAN_DOWNLOAD 9948
#define IDS_REVIEW_DOWNLOAD 9949
#define IDS_OPEN_DOWNLOAD_ANYWAY 9950
#define IDS_DOWNLOAD_LINK_PAUSE 9951
#define IDS_DOWNLOAD_HISTORY_SEARCH 9952
#define IDS_DOWNLOAD_NO_DOWNLOADS 9953
#define IDS_DOWNLOAD_LINK_RESUME 9954
#define IDS_DOWNLOAD_LINK_CANCEL 9955
#define IDS_DOWNLOAD_LINK_RETRY 9956
#define IDS_DOWNLOAD_LINK_SHOW 9957
#define IDS_DOWNLOAD_TAB_CANCELLED 9958
#define IDS_DOWNLOAD_FILE_REMOVED 9959
#define IDS_DOWNLOAD_TAB_PROGRESS_STATUS_TIME_UNKNOWN 9960
#define IDS_DOWNLOAD_TAB_PROGRESS_STATUS 9961
#define IDS_DOWNLOAD_TAB_PROGRESS_SIZE 9962
#define IDS_DOWNLOAD_PROGRESS_PAUSED 9963
#define IDS_DOWNLOAD_LINK_CLEAR_ALL 9964
#define IDS_DOWNLOAD_LINK_OPEN_DOWNLOADS_FOLDER 9965
#define IDS_DOWNLOAD_MORE_ACTIONS 9966
#define IDS_DOWNLOAD_ACTION_MENU_DESCRIPTION 9967
#define IDS_DOWNLOAD_BY_EXTENSION_URL 9968
#define IDS_DOWNLOADS_PAGE_INITIATOR_LINE 9969
#define IDS_DOWNLOAD_IN_INCOGNITO 9970
#define IDS_UNDO_DESCRIPTION 9971
#define IDS_DOWNLOAD_UNDO 9972
#define IDS_DOWNLOAD_TOAST_CLEARED_ALL 9973
#define IDS_DOWNLOAD_DEEP_SCAN_UPDATED 9974
#define IDS_DOWNLOAD_LOCAL_PASSWORD_SCAN 9975
#define IDS_DOWNLOAD_KEEP_DANGEROUS_FILE 9976
#define IDS_DOWNLOAD_KEEP_SUSPICIOUS_FILE 9977
#define IDS_DOWNLOAD_KEEP_UNVERIFIED_FILE 9978
#define IDS_DOWNLOAD_KEEP_INSECURE_FILE 9979
#define IDS_DOWNLOAD_DELETE_FROM_HISTORY 9980
#define IDS_DOWNLOAD_COPY_DOWNLOAD_LINK 9981
#define IDS_DOWNLOAD_DANGEROUS_ICON_ACCESSIBLE_LABEL 9982
#define IDS_DOWNLOAD_SUSPICIOUS_ICON_ACCESSIBLE_LABEL 9983
#define IDS_DOWNLOAD_INSECURE_ICON_ACCESSIBLE_LABEL 9984
#define IDS_DOWNLOAD_UNVERIFIED_ICON_ACCESSIBLE_LABEL 9985
#define IDS_DOWNLOAD_SCREENREADER_SAVED_DANGEROUS 9986
#define IDS_DOWNLOAD_SCREENREADER_SAVED_SUSPICIOUS 9987
#define IDS_DOWNLOAD_SCREENREADER_SAVED_INSECURE 9988
#define IDS_DOWNLOAD_SCREENREADER_SAVED_UNVERIFIED 9989
#define IDS_DOWNLOAD_SCREENREADER_PAUSED 9990
#define IDS_DOWNLOAD_SCREENREADER_RESUMED 9991
#define IDS_DOWNLOAD_SCREENREADER_CANCELED 9992
#define IDS_DOWNLOAD_WARNING_BYPASS_DIALOG_TITLE 9993
#define IDS_DOWNLOAD_WARNING_BYPASS_PROMPT_DESCRIPTION 9994
#define IDS_DOWNLOAD_STATUS_IN_PROGRESS_ACCESSIBLE_ALERT 9995
#define IDS_DOWNLOAD_STATUS_PERCENT_COMPLETE_ACCESSIBLE_ALERT 9996
#define IDS_DOWNLOAD_STATUS_TIME_REMAINING_ACCESSIBLE_ALERT 9997
#define IDS_DOWNLOAD_FAILED_ACCESSIBLE_ALERT 9998
#define IDS_DOWNLOAD_CANCELLED_ACCESSIBLE_ALERT 9999
#define IDS_DOWNLOAD_COMPLETE_ACCESSIBLE_ALERT 10000
#define IDS_DOWNLOAD_PAUSED_ACCESSIBLE_ALERT 10001
#define IDS_PROMPT_DEEP_SCANNING_ACCESSIBLE_ALERT 10002
#define IDS_DEEP_SCANNING_ACCESSIBLE_ALERT 10003
#define IDS_PROMPT_DOWNLOAD_INSECURE_BLOCKED_ACCESSIBLE_ALERT 10004
#define IDS_DOWNLOAD_NOTIFICATION_COPY_TO_CLIPBOARD 10005
#define IDS_DOWNLOAD_MENU_SHOW 10006
#define IDS_DOWNLOAD_MENU_OPEN_WHEN_COMPLETE 10007
#define IDS_DOWNLOAD_MENU_OPEN 10008
#define IDS_DOWNLOAD_MENU_ALWAYS_OPEN_TYPE 10009
#define IDS_DOWNLOAD_MENU_PLATFORM_OPEN 10010
#define IDS_DOWNLOAD_MENU_PLATFORM_OPEN_ALWAYS 10011
#define IDS_DOWNLOAD_MENU_CANCEL 10012
#define IDS_DOWNLOAD_MENU_PAUSE_ITEM 10013
#define IDS_DOWNLOAD_MENU_RESUME_ITEM 10014
#define IDS_DOWNLOAD_MENU_DISCARD 10015
#define IDS_DOWNLOAD_MENU_KEEP 10016
#define IDS_DOWNLOAD_MENU_LEARN_MORE_SCANNING 10017
#define IDS_DOWNLOAD_MENU_LEARN_MORE_INTERRUPTED 10018
#define IDS_DOWNLOAD_MENU_LEARN_MORE_INSECURE 10019
#define IDS_DOWNLOAD_MENU_DEEP_SCAN 10020
#define IDS_ABANDON_DOWNLOAD_DIALOG_TITLE 10031
#define IDS_ABANDON_DOWNLOAD_DIALOG_CONTINUE_BUTTON 10032
#define IDS_ABANDON_DOWNLOAD_DIALOG_INCOGNITO_MESSAGE 10033
#define IDS_ABANDON_DOWNLOAD_DIALOG_GUEST_MESSAGE 10034
#define IDS_ABANDON_DOWNLOAD_DIALOG_EXIT_BUTTON 10035
#define IDS_DOWNLOAD_BUBBLE_HEADER_LABEL 10036
#define IDS_DOWNLOAD_BUBBLE_FOOTER_LABEL 10037
#define IDS_DOWNLOAD_BUBBLE_FOOTER_TOOLTIP_LABEL 10038
#define IDS_DOWNLOAD_BUBBLE_ESB_PROMO 10039
#define IDS_DOWNLOAD_BUBBLE_ESB_PROMO_CUSTOM_ACTION 10040
#define IDS_DOWNLOAD_BUBBLE_ESB_PROMO_DISMISS 10041
#define IDS_DOWNLOAD_BUBBLE_ESB_PROMO_TITLE 10042
#define IDS_DOWNLOAD_BUBBLE_DOWNLOAD_STATUS_MESSAGE_WITH_SEPARATOR 10043
#define IDS_DOWNLOAD_BUBBLE_DOWNLOAD_STATUS_WITH_SYMBOL 10044
#define IDS_DOWNLOAD_BUBBLE_STATUS_RESUMING 10045
#define IDS_DOWNLOAD_BUBBLE_STATUS_DONE 10046
#define IDS_DOWNLOAD_BUBBLE_STATUS_DEEP_SCANNING_DONE_UPDATED 10047
#define IDS_DOWNLOAD_BUBBLE_STATUS_WARNING_INSECURE 10048
#define IDS_DOWNLOAD_BUBBLE_STATUS_WARNING_SUSPICIOUS 10049
#define IDS_DOWNLOAD_BUBBLE_STATUS_WARNING_DANGEROUS 10050
#define IDS_DOWNLOAD_BUBBLE_STATUS_WARNING_UNVERIFIED 10051
#define IDS_DOWNLOAD_BUBBLE_STATUS_BLOCKED 10052
#define IDS_DOWNLOAD_BUBBLE_STATUS_SUSPICIOUS 10053
#define IDS_DOWNLOAD_BUBBLE_STATUS_DEEP_SCANNED_FAILED_UPDATED 10054
#define IDS_DOWNLOAD_BUBBLE_CONTINUE 10055
#define IDS_DOWNLOAD_BUBBLE_CONTINUE_INSECURE_FILE 10056
#define IDS_DOWNLOAD_BUBBLE_CONTINUE_SUSPICIOUS_FILE 10057
#define IDS_DOWNLOAD_BUBBLE_CONTINUE_UNVERIFIED_FILE 10058
#define IDS_DOWNLOAD_BUBBLE_DELETE 10059
#define IDS_DOWNLOAD_BUBBLE_DELETE_FROM_HISTORY 10060
#define IDS_DOWNLOAD_BUBBLE_MAIN_BUTTON_SUBPAGE 10061
#define IDS_DOWNLOAD_BUBBLE_RESUME 10062
#define IDS_DOWNLOAD_BUBBLE_SCAN 10063
#define IDS_DOWNLOAD_BUBBLE_SCAN_UPDATED 10064
#define IDS_DOWNLOAD_BUBBLE_OPEN_UPDATED 10065
#define IDS_DOWNLOAD_BUBBLE_OPEN_NOW 10066
#define IDS_DOWNLOAD_BUBBLE_REVIEW 10067
#define IDS_DOWNLOAD_BUBBLE_RETRY 10068
#define IDS_DOWNLOAD_BUBBLE_OPEN_ANYWAY 10069
#define IDS_DOWNLOAD_BUBBLE_INTERRUPTED_STATUS_DISK_FULL 10070
#define IDS_DOWNLOAD_BUBBLE_INTERRUPTED_SUBPAGE_SUMMARY_DISK_FULL 10071
#define IDS_DOWNLOAD_BUBBLE_INTERRUPTED_STATUS_PATH_TOO_LONG 10072
#define IDS_DOWNLOAD_BUBBLE_INTERRUPTED_SUBPAGE_SUMMARY_PATH_TOO_LONG 10073
#define IDS_DOWNLOAD_BUBBLE_INTERRUPTED_STATUS_NEEDS_PERMISSION 10074
#define IDS_DOWNLOAD_BUBBLE_INTERRUPTED_STATUS_FILE_TOO_LARGE 10075
#define IDS_DOWNLOAD_BUBBLE_INTERRUPTED_SUBPAGE_SUMMARY_BLOCKED_ORGANIZATION 10076
#define IDS_DOWNLOAD_BUBBLE_INTERRUPTED_STATUS_WRONG 10077
#define IDS_DOWNLOAD_BUBBLE_INTERRUPTED_STATUS_NETWORK_ERROR 10078
#define IDS_DOWNLOAD_BUBBLE_INTERRUPTED_STATUS_SITE_UNAVAILABLE 10079
#define IDS_DOWNLOAD_BUBBLE_INTERRUPTED_STATUS_FILE_UNAVAILABLE 10080
#define IDS_DOWNLOAD_BUBBLE_INTERRUPTED_SUBPAGE_SUMMARY_FILE_UNAVAILABLE 10081
#define IDS_DOWNLOAD_BUBBLE_INTERRUPTED_STATUS_FILE_UNFINISHED 10082
#define IDS_DOWNLOAD_BUBBLE_SUBPAGE_SUMMARY_WARNING_INSECURE 10083
#define IDS_DOWNLOAD_BUBBLE_SUBPAGE_SUMMARY_WARNING_DANGEROUS 10084
#define IDS_DOWNLOAD_BUBBLE_SUBPAGE_SUMMARY_WARNING_DECEPTIVE 10085
#define IDS_DOWNLOAD_BUBBLE_SUBPAGE_SUMMARY_WARNING_DANGEROUS_FILE_TYPE 10086
#define IDS_DOWNLOAD_BUBBLE_SUBPAGE_SUMMARY_WARNING_UNCOMMON_FILE 10087
#define IDS_DOWNLOAD_BUBBLE_SUBPAGE_SUMMARY_WARNING_NO_SAFE_BROWSING 10088
#define IDS_DOWNLOAD_BUBBLE_SUBPAGE_SUMMARY_WARNING_SAFE_BROWSING_SETTING_LABEL 10089
#define IDS_DOWNLOAD_BUBBLE_SUBPAGE_SUMMARY_WARNING_SAFE_BROWSING_SETTING_LINK 10090
#define IDS_DOWNLOAD_BUBBLE_STATUS_ENCRYPTED 10091
#define IDS_DOWNLOAD_BUBBLE_STATUS_TOO_BIG 10092
#define IDS_DOWNLOAD_BUBBLE_STATUS_ADVANCED_PROTECTION 10093
#define IDS_DOWNLOAD_BUBBLE_SUBPAGE_SUMMARY_ADVANCED_PROTECTION 10094
#define IDS_DOWNLOAD_BUBBLE_STATUS_UNKNOWN_SOURCE 10095
#define IDS_DOWNLOAD_BUBBLE_STATUS_SENSITIVE_CONTENT 10096
#define IDS_DOWNLOAD_BUBBLE_STATUS_SCAN_FAILED 10097
#define IDS_DOWNLOAD_BUBBLE_SUBPAGE_SUMMARY_SENSITIVE_CONTENT_WARNING 10098
#define IDS_DOWNLOAD_BUBBLE_SUBPAGE_SUMMARY_SENSITIVE_CONTENT_BLOCK 10099
#define IDS_DOWNLOAD_BUBBLE_SUBPAGE_SUMMARY_ENCRYPTED 10100
#define IDS_DOWNLOAD_BUBBLE_SUBPAGE_SUMMARY_TOO_BIG 10101
#define IDS_DOWNLOAD_BUBBLE_SUBPAGE_SUMMARY_ASYNC_SCANNING 10102
#define IDS_DOWNLOAD_BUBBLE_SUBPAGE_SUMMARY_ASYNC_SCANNING_SECONDARY 10103
#define IDS_DOWNLOAD_BUBBLE_SUBPAGE_SUMMARY_ASYNC_SCANNING_ENTERPRISE_SECONDARY 10104
#define IDS_DOWNLOAD_BUBBLE_SUBPAGE_SUMMARY_ASYNC_SCANNING_DISCARD 10105
#define IDS_DOWNLOAD_BUBBLE_SUBPAGE_SUMMARY_ASYNC_SCANNING_CANCEL 10106
#define IDS_DOWNLOAD_BUBBLE_SUBPAGE_SUMMARY_SCAN_FAILED 10107
#define IDS_DOWNLOAD_BUBBLE_STATUS_ASYNC_SCANNING_ENTERPRISE 10108
#define IDS_DOWNLOAD_BUBBLE_INCOGNITO_INFORMATION_ROW 10109
#define IDS_DOWNLOAD_BUBBLE_RESUME_QUICK_ACTION 10110
#define IDS_DOWNLOAD_BUBBLE_RESUME_QUICK_ACTION_ACCESSIBILITY 10111
#define IDS_DOWNLOAD_BUBBLE_PAUSE_QUICK_ACTION 10112
#define IDS_DOWNLOAD_BUBBLE_PAUSE_QUICK_ACTION_ACCESSIBILITY 10113
#define IDS_DOWNLOAD_BUBBLE_CANCEL_QUICK_ACTION 10114
#define IDS_DOWNLOAD_BUBBLE_CANCEL_QUICK_ACTION_ACCESSIBILITY 10115
#define IDS_DOWNLOAD_BUBBLE_OPEN_QUICK_ACTION 10116
#define IDS_DOWNLOAD_BUBBLE_OPEN_QUICK_ACTION_ACCESSIBILITY 10117
#define IDS_DOWNLOAD_BUBBLE_SHOW_IN_FOLDER_QUICK_ACTION 10118
#define IDS_DOWNLOAD_BUBBLE_SHOW_IN_FOLDER_QUICK_ACTION_ACCESSIBILITY 10119
#define IDS_DOWNLOAD_BUBBLE_BACK_RECENT_DOWNLOADS 10120
#define IDS_DOWNLOAD_BUBBLE_CANCEL_MAIN_BUTTON_ACCESSIBILITY 10121
#define IDS_DOWNLOAD_BUBBLE_DELETE_MAIN_BUTTON_ACCESSIBILITY 10122
#define IDS_DOWNLOAD_BUBBLE_KEEP_MAIN_BUTTON_ACCESSIBILITY 10123
#define IDS_DOWNLOAD_BUBBLE_SCAN_MAIN_BUTTON_ACCESSIBILITY 10124
#define IDS_DOWNLOAD_BUBBLE_OPEN_NOW_MAIN_BUTTON_ACCESSIBILITY 10125
#define IDS_DOWNLOAD_BUBBLE_RESUME_MAIN_BUTTON_ACCESSIBILITY 10126
#define IDS_DOWNLOAD_BUBBLE_REVIEW_MAIN_BUTTON_ACCESSIBILITY 10127
#define IDS_DOWNLOAD_BUBBLE_RETRY_MAIN_BUTTON_ACCESSIBILITY 10128
#define IDS_DOWNLOAD_BUBBLE_OPEN_MAIN_BUTTON_ACCESSIBILITY 10129
#define IDS_DOWNLOAD_BUBBLE_SUPPRESS_PARTIAL_VIEW 10130
#define IDS_DOWNLOAD_BUBBLE_SUPPRESS_PARTIAL_VIEW_SETTINGS_REMINDER 10131
#define IDS_DOWNLOAD_BUBBLE_SUPPRESS_PARTIAL_VIEW_SETTINGS_LINK 10132
#define IDS_DOWNLOAD_BUBBLE_TOOLTIP_IN_PROGRESS_COUNT 10133
#define IDS_DOWNLOAD_BUBBLE_SUBPAGE_SUMMARY_WARNING_COOKIE_THEFT 10136
#define IDS_DOWNLOAD_BUBBLE_SUBPAGE_SUMMARY_WARNING_ARCHIVE_MALWARE 10137
#define IDS_DOWNLOAD_BUBBLE_STATUS_DEEP_SCANNING_PROMPT_UPDATED 10138
#define IDS_DOWNLOAD_BUBBLE_STATUS_ASYNC_SCANNING 10139
#define IDS_DOWNLOAD_BUBBLE_STATUS_ASYNC_CHECKING_SAFETY 10140
#define IDS_DOWNLOAD_BUBBLE_SUBPAGE_SUMMARY_DEEP_SCANNING_PROMPT_UPDATED 10141
#define IDS_DOWNLOAD_BUBBLE_SUBPAGE_SUMMARY_DEEP_SCANNING_PROMPT_ENCRYPTED_ARCHIVE 10142
#define IDS_DOWNLOAD_BUBBLE_SUBPAGE_DEEP_SCANNING_LINK 10143
#define IDS_DOWNLOAD_BUBBLE_PASSWORD_PROMPT 10144
#define IDS_DOWNLOAD_BUBBLE_PASSWORD_PROMPT_EMPTY 10145
#define IDS_DOWNLOAD_BUBBLE_PASSWORD_PROMPT_INVALID 10146
#define IDS_DOWNLOAD_BUBBLE_PASSWORD_PROMPT_ACCESSIBLE_ALERT 10147
#define IDS_DOWNLOAD_BUBBLE_PASSWORD_PROMPT_ACCESSIBLE_ALERT_EMPTY 10148
#define IDS_DOWNLOAD_BUBBLE_PASSWORD_PROMPT_ACCESSIBLE_ALERT_INVALID 10149
#define IDS_DOWNLOAD_BUBBLE_SUBPAGE_SUMMARY_IMMEDIATE_DEEP_SCAN_IN_PROGRESS 10150
#define IDS_DOWNLOAD_BUBBLE_SUBPAGE_IMMEDIATE_DEEP_SCAN_CANCEL 10151
#define IDS_DOWNLOAD_BUBBLE_SUBPAGE_IMMEDIATE_DEEP_SCAN_BYPASS 10152
#define IDS_DOWNLOAD_BUBBLE_STATUS_LOCAL_DECRYPTION_STATUS 10153
#define IDS_DOWNLOAD_BUBBLE_STATUS_PASSWORD_NEEDED 10154
#define IDS_DOWNLOAD_BUBBLE_ACCEPT_LOCAL_DECRYPTION 10155
#define IDS_DOWNLOAD_BUBBLE_BYPASS_LOCAL_DECRYPTION 10156
#define IDS_DOWNLOAD_BUBBLE_LOCAL_DECRYPTION_CANCEL 10157
#define IDS_DOWNLOAD_BUBBLE_SUBPAGE_SUMMARY_LOCAL_DECRYPTION_IN_PROGRESS 10158
#define IDS_DOWNLOAD_BUBBLE_STATUS_LOCAL_DECRYPTING 10159
#define IDS_DOWNLOAD_LOCAL_DECRYPTION_PROMPT_ALERT 10160
#define IDS_WEB_APP_WITH_APP_TITLE 10161
#define IDS_OMNIBOX_PWA_INSTALL_ICON_LABEL 10162
#define IDS_OMNIBOX_PWA_INSTALL_ICON_TOOLTIP 10163
#define IDS_WEBAPP_UPDATE_REVIEW_ACCEPT_BUTTON 10164
#define IDS_WEBAPP_UPDATE_REVIEW_UNINSTALL_BUTTON 10165
#define IDS_WEBAPP_UPDATE_REVIEW_IGNORE_BUTTON 10166
#define IDS_WEBAPP_UPDATE_REVIEW_ACCEPT_BUTTON_MIGRATION 10167
#define IDS_ADD_TO_OS_LAUNCH_SURFACE_BUBBLE_TITLE 10168
#define IDS_CREATE_SHORTCUTS_BUTTON_LABEL 10169
#define IDS_CREATE_SHORTCUT_NOT_APPS_DIALOG_TITLE 10170
#define IDS_CREATE_SHORTCUT_NOT_APPS_AX_BUBBLE_LABEL 10171
#define IDS_INSTALL_TO_OS_LAUNCH_SURFACE_BUBBLE_TITLE 10172
#define IDS_INSTALL_PWA_BUTTON_LABEL 10173
#define IDS_INSTALL_PWA_DIALOG_TITLE 10174
#define IDS_WEB_APP_INSTALL_FLOW_SUBTITLE 10175
#define IDS_WEB_APP_INSTALL_FLOW_NEXT 10176
#define IDS_WEB_APP_INSTALL_SUCCESS_TITLE 10177
#define IDS_WEB_APP_INSTALL_SUCCESS_OPEN_APP 10178
#define IDS_INSTALL_PWA_DIALOG_INSTALLING 10179
#define IDS_WEB_APP_DETAILED_INSTALL_DIALOG_DESCRIPTION_TITLE 10180
#define IDS_DIY_APP_INSTALL_DIALOG_TITLE 10181
#define IDS_DIY_APP_INSTALL_DIALOG_SUBTITLE 10182
#define IDS_WEB_APP_INSTALL_PIN_TO_SHELF 10183
#define IDS_DIY_APP_AX_BUBBLE_NAME_LABEL 10184
#define IDS_INSTALL_PWA_DIALOG_ORIGIN_LABEL 10185
#define IDS_WEB_APP_INSTALL_ADD_TO_START_MENU 10186
#define IDS_WEB_APP_INSTALL_CHROME_APPS_LOCATION 10187
#define IDS_WEB_APP_INSTALL_CREATE_DESKTOP_SHORTCUT 10188
#define IDS_WEB_APP_INSTALL_PIN_TO_TASKBAR 10189
#define IDS_WEB_APP_INSTALL_LAUNCHER 10190
#define IDS_BOOKMARK_APP_AX_BUBBLE_NAME_LABEL 10191
#define IDS_BOOKMARK_APP_BUBBLE_OPEN_AS_TAB 10192
#define IDS_BOOKMARK_APP_BUBBLE_OPEN_AS_WINDOW 10193
#define IDS_BOOKMARK_APP_BUBBLE_OPEN_AS_TABBED_WINDOW 10194
#define IDS_WEB_APP_INSTALL_NOT_SUPPORTED_DIALOG_TITLE_INCOGNITO_MODE 10195
#define IDS_WEB_APP_INSTALL_NOT_SUPPORTED_DIALOG_TITLE_GUEST_MODE 10196
#define IDS_WEB_APP_INSTALL_NOT_SUPPORTED_DIALOG_TITLE_POLICY_MODE 10197
#define IDS_WEBAPP_UPDATE_DIALOG_TITLE_NAME 10198
#define IDS_WEBAPP_UPDATE_DIALOG_TITLE_ICON 10199
#define IDS_WEBAPP_UPDATE_DIALOG_TITLE_NAME_AND_ICON 10200
#define IDS_WEBAPP_UPDATE_DIALOG_TITLE_LOGO 10201
#define IDS_WEBAPP_UPDATE_DIALOG_TITLE_NAME_AND_LOGO 10202
#define IDS_WEBAPP_UPDATE_DIALOG_TITLE_URL 10203
#define IDS_WEBAPP_UPDATE_DIALOG_TITLE_URL_FORCED_MIGRATION 10204
#define IDS_WEBAPP_UPDATE_DIALOG_TITLE_NAME_AND_URL 10205
#define IDS_WEBAPP_UPDATE_DIALOG_TITLE_LOGO_AND_URL 10206
#define IDS_WEBAPP_UPDATE_DIALOG_TITLE_NAME_AND_LOGO_AND_URL 10207
#define IDS_WEBAPP_UPDATE_NEW_EXPLANATION 10208
#define IDS_WEBAPP_UPDATE_URL_ONLY_NEW_EXPLANATION 10209
#define IDS_WEBAPP_UPDATE_EXPLANATION 10210
#define IDS_WEBAPP_UPDATE_CURRENT_ICON 10211
#define IDS_WEBAPP_UPDATE_NEW_ICON 10212
#define IDS_WEBAPP_UPDATE_NEGATIVE_BUTTON 10213
#define IDS_FINISH_POLICY_WEB_APP_INSTALLATION 10214
#define IDS_FINISH_POLICY_WEB_APP_INSTALATION_RESTART 10215
#define IDS_FINISH_POLICY_WEB_APP_INSTALLATION_NOT_NOW 10216
#define IDS_ADD_TO_OS_LAUNCH_SURFACE 10221
#define IDS_INSTALL_TO_OS_LAUNCH_SURFACE 10222
#define IDS_INSTALL_DIY_TO_OS_LAUNCH_SURFACE 10223
#define IDS_UNINSTALL_FROM_OS_LAUNCH_SURFACE 10224
#define IDS_REVIEW_APP_UPDATE 10225
#define IDS_APPLICATION_INFO_WEB_STORE_LINK 10226
#define IDS_APPLICATION_INFO_HOMEPAGE_LINK 10227
#define IDS_ARC_APPLICATION_INFO_MANAGE_LINK 10228
#define IDS_APPLICATION_INFO_APP_OVERVIEW_TITLE 10229
#define IDS_APPLICATION_INFO_APP_PERMISSIONS_TITLE 10230
#define IDS_APPLICATION_INFO_UNINSTALL_BUTTON_TEXT 10231
#define IDS_APPLICATION_INFO_LICENSES_BUTTON_TEXT 10232
#define IDS_APPLICATION_INFO_SIZE_LABEL 10233
#define IDS_APPLICATION_INFO_VERSION_LABEL 10234
#define IDS_APPLICATION_INFO_CREATE_SHORTCUTS_BUTTON_TEXT 10235
#define IDS_APPLICATION_INFO_LAUNCH_OPTIONS_ACCNAME 10236
#define IDS_APPLICATION_INFO_SIZE_LOADING_LABEL 10237
#define IDS_APPLICATION_INFO_SIZE_SMALL_LABEL 10238
#define IDS_APPLICATION_INFO_REVOKE_PERMISSION_ALT_TEXT 10239
#define IDS_APPLICATION_INFO_APP_NO_PERMISSIONS_TEXT 10240
#define IDS_APPLICATION_INFO_EXTENSION_NO_PERMISSIONS_TEXT 10241
#define IDS_APPLICATION_INFO_RETAINED_FILES 10242
#define IDS_APPLICATION_INFO_RETAINED_DEVICES 10243
#define IDS_CREATE_SHORTCUTS_TITLE 10244
#define IDS_CREATE_SHORTCUTS_LABEL 10245
#define IDS_CREATE_SHORTCUTS_DESKTOP_CHKBOX 10246
#define IDS_CREATE_SHORTCUTS_MENU_CHKBOX 10247
#define IDS_CREATE_SHORTCUTS_COMMIT 10248
#define IDS_BLUETOOTH_DEVICE_CREDENTIALS_TITLE 10251
#define IDS_BLUETOOTH_DEVICE_CREDENTIALS_LABEL 10252
#define IDS_BLUETOOTH_DEVICE_PAIR_CONFIRM_TITLE 10253
#define IDS_BLUETOOTH_DEVICE_PAIR_CONFIRM_LABEL 10254
#define IDS_BLUETOOTH_DEVICE_PASSKEY_CONFIRM_TITLE 10255
#define IDS_BLUETOOTH_DEVICE_PASSKEY_CONFIRM_LABEL 10256
#define IDS_MANAGE 10257
#define IDS_LIST_BULLET 10258
#define IDS_BLOCKED_DOWNLOAD_NO_ACTION 10259
#define IDS_BLOCKED_DOWNLOAD_UNBLOCK 10260
#define IDS_ALLOWED_DOWNLOAD_TITLE 10261
#define IDS_BLOCKED_DOWNLOAD_TITLE 10262
#define IDS_BLOCKED_DOWNLOADS_EXPLANATION 10263
#define IDS_ALLOWED_DOWNLOAD_NO_ACTION 10264
#define IDS_ALLOWED_DOWNLOAD_BLOCK 10265
#define IDS_BLOCKED_ON_DEVICE_SITE_DATA_TITLE 10266
#define IDS_ACCESSED_ON_DEVICE_SITE_DATA_TITLE 10267
#define IDS_BLOCKED_ON_DEVICE_SITE_DATA_MESSAGE 10268
#define IDS_ACCESSED_ON_DEVICE_SITE_DATA_MESSAGE 10269
#define IDS_BLOCKED_IMAGES_TITLE 10270
#define IDS_BLOCKED_IMAGES_MESSAGE 10271
#define IDS_BLOCKED_ON_DEVICE_SITE_DATA_UNBLOCK 10272
#define IDS_BLOCKED_IMAGES_UNBLOCK 10273
#define IDS_BLOCKED_ON_DEVICE_SITE_DATA_NO_ACTION 10274
#define IDS_ALLOWED_ON_DEVICE_SITE_DATA_NO_ACTION 10275
#define IDS_ALLOWED_ON_DEVICE_SITE_DATA_BLOCK 10276
#define IDS_BLOCKED_IMAGES_NO_ACTION 10277
#define IDS_BLOCKED_POPUPS_TOOLTIP 10278
#define IDS_BLOCKED_POPUPS_TITLE 10279
#define IDS_BLOCKED_POPUPS_REDIRECTS_UNBLOCK 10280
#define IDS_BLOCKED_POPUPS_REDIRECTS_NO_ACTION 10281
#define IDS_BLOCKED_MEDIASTREAM_MIC_AND_CAMERA_ALLOW 10282
#define IDS_BLOCKED_MEDIASTREAM_MIC_ALLOW 10283
#define IDS_BLOCKED_MEDIASTREAM_CAMERA_ALLOW 10284
#define IDS_BLOCKED_MEDIASTREAM_MIC_AND_CAMERA_ASK 10285
#define IDS_BLOCKED_MEDIASTREAM_MIC_ASK 10286
#define IDS_BLOCKED_MEDIASTREAM_CAMERA_ASK 10287
#define IDS_ALLOWED_MEDIASTREAM_MIC_AND_CAMERA_BLOCK 10288
#define IDS_ALLOWED_MEDIASTREAM_MIC_BLOCK 10289
#define IDS_ALLOWED_MEDIASTREAM_CAMERA_BLOCK 10290
#define IDS_BLOCKED_MEDIASTREAM_MIC_AND_CAMERA_NO_ACTION 10291
#define IDS_BLOCKED_MEDIASTREAM_MIC_NO_ACTION 10292
#define IDS_BLOCKED_MEDIASTREAM_CAMERA_NO_ACTION 10293
#define IDS_ALLOWED_MEDIASTREAM_MIC_AND_CAMERA_NO_ACTION 10294
#define IDS_ALLOWED_MEDIASTREAM_MIC_AND_CAMERA_PAN_TILT_ZOOM_NO_ACTION 10295
#define IDS_ALLOWED_MEDIASTREAM_MIC_NO_ACTION 10296
#define IDS_ALLOWED_MEDIASTREAM_CAMERA_NO_ACTION 10297
#define IDS_ALLOWED_CAMERA_PAN_TILT_ZOOM_NO_ACTION 10298
#define IDS_BLOCKED_POPUPS_EXPLANATORY_TEXT 10299
#define IDS_BLOCKED_JAVASCRIPT_TITLE 10300
#define IDS_BLOCKED_JAVASCRIPT_MESSAGE 10301
#define IDS_BLOCKED_JAVASCRIPT_UNBLOCK 10302
#define IDS_BLOCKED_JAVASCRIPT_NO_ACTION 10303
#define IDS_BLOCKED_SOUND_TITLE 10304
#define IDS_BLOCKED_SOUND_UNBLOCK 10305
#define IDS_BLOCKED_SOUND_NO_ACTION 10306
#define IDS_NOTIFICATIONS_OFF_EXPLANATORY_TEXT 10307
#define IDS_NOTIFICATIONS_QUIET_PERMISSION_BUBBLE_TITLE 10308
#define IDS_NOTIFICATIONS_QUIET_PERMISSION_BUBBLE_DESCRIPTION 10309
#define IDS_GEOLOCATION_QUIET_PERMISSION_BUBBLE_DESCRIPTION 10310
#define IDS_NOTIFICATIONS_QUIET_PERMISSION_BUBBLE_ALLOW_BUTTON 10311
#define IDS_NOTIFICATIONS_QUIET_PERMISSION_BUBBLE_COMPACT_ALLOW_BUTTON 10312
#define IDS_NOTIFICATIONS_QUIET_PERMISSION_BUBBLE_CONTINUE_BLOCKING_BUTTON 10313
#define IDS_NOTIFICATIONS_QUIET_PERMISSION_EARLY_PROMO 10314
#define IDS_NOTIFICATIONS_QUIET_PERMISSION_BUBBLE_CROWD_DENY_DESCRIPTION 10315
#define IDS_GEOLOCATION_QUIET_PERMISSION_BUBBLE_CROWD_DENY_DESCRIPTION 10316
#define IDS_NOTIFICATIONS_QUIET_PERMISSION_BUBBLE_ABUSIVE_DESCRIPTION 10317
#define IDS_NOTIFICATIONS_QUIET_PERMISSION_BUBBLE_DISRUPTIVE_DESCRIPTION 10318
#define IDS_NOTIFICATIONS_QUIET_PERMISSION_BUBBLE_PREDICTION_SERVICE_DESCRIPTION 10319
#define IDS_GEOLOCATION_OFF_EXPLANATORY_TEXT 10320
#define IDS_GEOLOCATION_QUIET_PERMISSION_BUBBLE_TITLE 10321
#define IDS_GEOLOCATION_QUIET_PERMISSION_BUBBLE_PREDICTION_SERVICE_DESCRIPTION 10322
#define IDS_GEOLOCATION_QUIET_PERMISSION_BUBBLE_ALLOW_BUTTON 10323
#define IDS_STORAGE_ACCESS_PERMISSION_BUBBLE_MESSAGE 10324
#define IDS_STORAGE_ACCESS_PERMISSION_ALLOWED_TOOLTIP 10325
#define IDS_STORAGE_ACCESS_PERMISSION_BLOCKED_TOOLTIP 10326
#define IDS_COOKIE_CONTROLS_BUBBLE_COOKIES_BLOCKED_TITLE 10327
#define IDS_COOKIE_CONTROLS_BUBBLE_COOKIES_ALLOWED_TITLE 10328
#define IDS_COOKIE_CONTROLS_BUBBLE_SITE_NOT_WORKING_TITLE 10329
#define IDS_TRACKING_PROTECTION_BUBBLE_SITE_NOT_WORKING_DESCRIPTION 10330
#define IDS_COOKIE_CONTROLS_BUBBLE_SITE_NOT_WORKING_DESCRIPTION 10331
#define IDS_TRACKING_PROTECTION_BUBBLE_BLOCKING_RESTART_TITLE 10332
#define IDS_TRACKING_PROTECTION_BUBBLE_BLOCKING_RESTART_DESCRIPTION 10333
#define IDS_TRACKING_PROTECTION_BUBBLE_PERMANENT_ALLOWED_TITLE 10334
#define IDS_TRACKING_PROTECTION_BUBBLE_PERMANENT_ALLOWED_DESCRIPTION 10335
#define IDS_COOKIE_CONTROLS_BUBBLE_SEND_FEEDBACK_BUTTON_TITLE 10336
#define IDS_COOKIE_CONTROLS_BUBBLE_SEND_FEEDBACK_BUTTON_DESCRIPTION 10337
#define IDS_COOKIE_CONTROLS_BUBBLE_RELOADING_LABEL 10338
#define IDS_COOKIE_CONTROLS_BUBBLE_THIRD_PARTY_COOKIES_LABEL 10339
#define IDS_COOKIE_CONTROLS_BUBBLE_SEND_FEEDBACK_FORM_PLACEHOLDER 10340
#define IDS_COOKIE_CONTROLS_PAGE_ACTION_COOKIES_ALLOWED_LABEL 10341
#define IDS_COOKIE_CONTROLS_PAGE_ACTION_COOKIES_BLOCKED_LABEL 10342
#define IDS_COOKIE_CONTROLS_PROMO_TITLE 10343
#define IDS_COOKIE_CONTROLS_PROMO_TEXT 10344
#define IDS_COOKIE_CONTROLS_PROMO_CLOSE_BUTTON_TEXT 10345
#define IDS_COOKIE_CONTROLS_PROMO_SEE_HOW_BUTTON_TEXT 10346
#define IDS_JS_OPTIMIZATIONS_DISABLED_ICON_TOOLTIP 10347
#define IDS_JS_OPTIMIZATION_BUBBLE_TITLE 10348
#define IDS_JS_OPTIMIZATION_BUBBLE_POLICY_BODY 10349
#define IDS_JS_OPTIMIZATION_BUBBLE_ENABLE_BUTTON 10350
#define IDS_JS_OPTIMIZATION_SITE_RELOAD_TEXT 10351
#define IDS_JS_OPTIMIZATION_SITE_RELOAD_BUTTON 10352
#define IDS_CERT_SELECTOR_SUBJECT_COLUMN 10353
#define IDS_CERT_SELECTOR_ISSUER_COLUMN 10354
#define IDS_CERT_SELECTOR_PROVIDER_COLUMN 10355
#define IDS_CERT_SELECTOR_SERIAL_COLUMN 10356
#define IDS_CERT_EXPORT_TYPE_BASE64_ALL 10357
#define IDS_CERT_EXPORT_TYPE_BASE64 10358
#define IDS_CERT_EXPORT_TYPE_BASE64_CHAIN 10359
#define IDS_CERT_EXPORT_TYPE_DER 10360
#define IDS_CERT_EXPORT_TYPE_PKCS7 10361
#define IDS_CERT_EXPORT_TYPE_PKCS7_CHAIN 10362
#define IDS_CERT_INFO_DIALOG_TITLE 10363
#define IDS_CERT_INFO_GENERAL_TAB_LABEL 10364
#define IDS_CERT_INFO_DETAILS_TAB_LABEL 10365
#define IDS_CERT_INFO_MODIFICATIONS_TAB_LABEL 10366
#define IDS_CERT_INFO_TRUST_LABEL 10367
#define IDS_CERT_INFO_TRUST_STATE_LABEL 10368
#define IDS_CERT_INFO_TRUST_STATE_DISTRUSTED 10369
#define IDS_CERT_INFO_TRUST_STATE_HINT 10370
#define IDS_CERT_INFO_TRUST_STATE_TRUSTED 10371
#define IDS_CERT_INFO_CONSTRAINTS_LABEL 10372
#define IDS_CERT_INFO_ADD_CONSTRAINTS_LABEL 10373
#define IDS_CERT_INFO_ADD_CONSTRAINTS_BUTTON_LABEL 10374
#define IDS_CERT_INFO_ADD_CONSTRAINTS_PLACEHOLDER 10375
#define IDS_CERT_INFO_ADD_CONSTRAINT_ERROR_MESSAGE 10376
#define IDS_CERT_INFO_DELETE_CONSTRAINT_ERROR_MESSAGE 10377
#define IDS_CERT_INFO_TRUST_STATE_ERROR_MESSAGE 10378
#define IDS_CERT_USAGE_SSL_CLIENT 10379
#define IDS_CERT_USAGE_SSL_SERVER 10380
#define IDS_CERT_USAGE_OBJECT_SIGNER 10381
#define IDS_CERT_USAGE_SSL_CA 10382
#define IDS_CERT_INFO_IDN_VALUE_FORMAT 10383
#define IDS_CERT_INFO_FIELD_NOT_PRESENT 10384
#define IDS_CERT_DETAILS_CERTIFICATE_HIERARCHY_LABEL 10385
#define IDS_CERT_DETAILS_CERTIFICATE_FIELDS_LABEL 10386
#define IDS_CERT_DETAILS_CERTIFICATE_FIELD_VALUE_LABEL 10387
#define IDS_CERT_DETAILS_CERTIFICATE 10388
#define IDS_CERT_DETAILS_VERSION_FORMAT 10389
#define IDS_CERT_DETAILS_CERTIFICATE_SIG_ALG 10390
#define IDS_CERT_DETAILS_SUBJECT_KEY_ALG 10391
#define IDS_CERT_DETAILS_SUBJECT_KEY 10392
#define IDS_CERT_RSA_PUBLIC_KEY_DUMP_FORMAT 10393
#define IDS_CERT_DETAILS_CERTIFICATE_SIG_VALUE 10394
#define IDS_CERT_DETAILS_EXPORT_CERTIFICATE 10395
#define IDS_CERT_DETAILS_EXPORT_CERTIFICATE_A11Y_LABEL 10396
#define IDS_CERT_OID_AVA_COMMON_NAME 10397
#define IDS_CERT_OID_AVA_STATE_OR_PROVINCE 10398
#define IDS_CERT_OID_AVA_ORGANIZATION_NAME 10399
#define IDS_CERT_OID_AVA_ORGANIZATIONAL_UNIT_NAME 10400
#define IDS_CERT_OID_AVA_DN_QUALIFIER 10401
#define IDS_CERT_OID_AVA_COUNTRY_NAME 10402
#define IDS_CERT_OID_AVA_SERIAL_NUMBER 10403
#define IDS_CERT_OID_AVA_LOCALITY 10404
#define IDS_CERT_OID_AVA_DC 10405
#define IDS_CERT_OID_RFC1274_MAIL 10406
#define IDS_CERT_OID_RFC1274_UID 10407
#define IDS_CERT_OID_PKCS9_EMAIL_ADDRESS 10408
#define IDS_CERT_OID_BUSINESS_CATEGORY 10409
#define IDS_CERT_OID_EV_INCORPORATION_LOCALITY 10410
#define IDS_CERT_OID_EV_INCORPORATION_STATE 10411
#define IDS_CERT_OID_EV_INCORPORATION_COUNTRY 10412
#define IDS_CERT_OID_AVA_STREET_ADDRESS 10413
#define IDS_CERT_OID_AVA_POSTAL_CODE 10414
#define IDS_CERT_EXT_NS_CERT_TYPE 10415
#define IDS_CERT_EXT_NS_CERT_TYPE_EMAIL 10416
#define IDS_CERT_EXT_NS_CERT_TYPE_EMAIL_CA 10417
#define IDS_CERT_EXT_NS_CERT_BASE_URL 10418
#define IDS_CERT_EXT_NS_CERT_REVOCATION_URL 10419
#define IDS_CERT_EXT_NS_CA_REVOCATION_URL 10420
#define IDS_CERT_EXT_NS_CERT_RENEWAL_URL 10421
#define IDS_CERT_EXT_NS_CA_POLICY_URL 10422
#define IDS_CERT_EXT_NS_SSL_SERVER_NAME 10423
#define IDS_CERT_EXT_NS_COMMENT 10424
#define IDS_CERT_EXT_NS_LOST_PASSWORD_URL 10425
#define IDS_CERT_EXT_NS_CERT_RENEWAL_TIME 10426
#define IDS_CERT_X509_SUBJECT_DIRECTORY_ATTR 10427
#define IDS_CERT_KEYID_FORMAT 10428
#define IDS_CERT_ISSUER_FORMAT 10429
#define IDS_CERT_SERIAL_NUMBER_FORMAT 10430
#define IDS_CERT_X509_NAME_CONSTRAINTS 10431
#define IDS_CERT_X509_POLICY_MAPPINGS 10432
#define IDS_CERT_X509_POLICY_CONSTRAINTS 10433
#define IDS_CERT_X509_BASIC_CONSTRAINT_IS_CA 10434
#define IDS_CERT_X509_BASIC_CONSTRAINT_IS_NOT_CA 10435
#define IDS_CERT_X509_BASIC_CONSTRAINT_PATH_LEN 10436
#define IDS_CERT_X509_BASIC_CONSTRAINT_PATH_LEN_UNLIMITED 10437
#define IDS_CERT_REVOCATION_REASON_UNUSED 10438
#define IDS_CERT_REVOCATION_REASON_KEY_COMPROMISE 10439
#define IDS_CERT_REVOCATION_REASON_CA_COMPROMISE 10440
#define IDS_CERT_REVOCATION_REASON_AFFILIATION_CHANGED 10441
#define IDS_CERT_REVOCATION_REASON_SUPERSEDED 10442
#define IDS_CERT_REVOCATION_REASON_CESSATION_OF_OPERATION 10443
#define IDS_CERT_REVOCATION_REASON_CERTIFICATE_HOLD 10444
#define IDS_CERT_OCSP_RESPONDER_FORMAT 10445
#define IDS_CERT_CA_ISSUERS_FORMAT 10446
#define IDS_CERT_UNKNOWN_OID_INFO_FORMAT 10447
#define IDS_CERT_EXT_KEY_USAGE_FORMAT 10448
#define IDS_CERT_MULTILINE_INFO_START_FORMAT 10449
#define IDS_CERT_EXT_MS_APP_POLICIES 10450
#define IDS_CERT_EXT_MS_CERT_TEMPLATE 10451
#define IDS_CERT_EXT_MS_CERT_TYPE 10452
#define IDS_CERT_EXT_MS_CA_VERSION 10453
#define IDS_CERT_EXT_MS_NT_PRINCIPAL_NAME 10454
#define IDS_CERT_EXT_MS_NTDS_REPLICATION 10455
#define IDS_CERT_POLICY_ETSI_QEVCP_W 10456
#define IDS_CERT_POLICY_ETSI_QNCP_W 10457
#define IDS_CERT_EKU_MS_INDIVIDUAL_CODE_SIGNING 10458
#define IDS_CERT_EKU_MS_COMMERCIAL_CODE_SIGNING 10459
#define IDS_CERT_EKU_MS_TRUST_LIST_SIGNING 10460
#define IDS_CERT_EKU_MS_TIME_STAMPING 10461
#define IDS_CERT_EKU_MS_SERVER_GATED_CRYPTO 10462
#define IDS_CERT_EKU_MS_ENCRYPTING_FILE_SYSTEM 10463
#define IDS_CERT_EKU_MS_FILE_RECOVERY 10464
#define IDS_CERT_EKU_MS_WINDOWS_HARDWARE_DRIVER_VERIFICATION 10465
#define IDS_CERT_EKU_MS_QUALIFIED_SUBORDINATION 10466
#define IDS_CERT_EKU_MS_KEY_RECOVERY 10467
#define IDS_CERT_EKU_MS_DOCUMENT_SIGNING 10468
#define IDS_CERT_EKU_MS_LIFETIME_SIGNING 10469
#define IDS_CERT_EKU_MS_SMART_CARD_LOGON 10470
#define IDS_CERT_EKU_MS_KEY_RECOVERY_AGENT 10471
#define IDS_CERT_EKU_NETSCAPE_INTERNATIONAL_STEP_UP 10472
#define IDS_CERT_DUMP_ERROR 10473
#define IDS_CERT_EXTENSION_DUMP_ERROR 10474
#define IDS_CERTIFICATE_MANAGER_TITLE 10475
#define IDS_CERT_MANAGER_HARDWARE_BACKED_KEY_FORMAT 10476
#define IDS_CERT_MANAGER_HARDWARE_BACKED 10477
#define IDS_CERT_MANAGER_EXTENSION_PROVIDED_FORMAT 10478
#define IDS_DEV_TOOLS_SHARED_PROCESS_INFOBAR 10479
#define IDS_DEV_TOOLS_SHARED_PROCESS_INFOBAR_LEARN_MORE 10480
#define IDS_DEV_TOOLS_SHARED_PROCESS_INFOBAR_OPT_OUT 10481
#define IDS_DEV_TOOLS_SHARED_PROCESS_INFOBAR_RESTART_NEEDED 10482
#define IDS_DEV_TOOLS_INFOBAR_LABEL 10483
#define IDS_DEV_TOOLS_CONFIRM_ADD_FILE_SYSTEM_MESSAGE 10484
#define IDS_DEV_TOOLS_EDIT_DIRECTORY_PERMISSION_TITLE 10485
#define IDS_DEV_TOOLS_CONNECTION_DIALOG_TITLE 10486
#define IDS_DEV_TOOLS_CONNECTION_DIALOG_MESSAGE_PART_1 10487
#define IDS_DEV_TOOLS_CONNECTION_DIALOG_MESSAGE_PART_2 10488
#define IDS_DEV_TOOLS_CONNECTION_DIALOG_ALLOW_TEXT 10489
#define IDS_DEV_TOOLS_CONNECTION_DIALOG_DISABLE_TEXT 10490
#define IDS_DEV_TOOLS_CONFIRM_ALLOW_BUTTON 10491
#define IDS_DEV_TOOLS_CONFIRM_DENY_BUTTON 10492
#define IDS_RELOAD_MENU_NORMAL_RELOAD_ITEM 10493
#define IDS_RELOAD_MENU_HARD_RELOAD_ITEM 10494
#define IDS_RELOAD_MENU_EMPTY_AND_HARD_RELOAD_ITEM 10495
#define IDS_TAB_SHARING_INFOBAR_SHARING_CURRENT_TAB_LABEL 10496
#define IDS_TAB_SHARING_INFOBAR_SHARING_ANOTHER_UNTITLED_TAB_LABEL 10497
#define IDS_TAB_SHARING_INFOBAR_SHARING_ANOTHER_TAB_LABEL 10498
#define IDS_TAB_SHARING_INFOBAR_SHARING_ANOTHER_TAB_TO_THIS_TAB_LABEL 10499
#define IDS_TAB_SHARING_INFOBAR_SHARE_BUTTON 10500
#define IDS_TAB_SHARING_INFOBAR_STOP_BUTTON 10501
#define IDS_TAB_SHARING_INFOBAR_SWITCH_TO_BUTTON 10502
#define IDS_TAB_SHARING_INFOBAR_SWITCH_TO_CAPTURER_BUTTON 10503
#define IDS_TAB_SHARING_INFOBAR_SWITCH_TO_CAPTURED_BUTTON 10504
#define IDS_TAB_SHARING_INFOBAR_CAPTURED_SURFACE_CONTROL_PERMISSION_BUTTON 10505
#define IDS_TAB_CASTING_INFOBAR_CASTING_CURRENT_TAB_LABEL 10506
#define IDS_TAB_CASTING_INFOBAR_CASTING_CURRENT_TAB_NO_DEVICE_NAME_LABEL 10507
#define IDS_TAB_CASTING_INFOBAR_CASTING_ANOTHER_UNTITLED_TAB_LABEL 10508
#define IDS_TAB_CASTING_INFOBAR_CASTING_ANOTHER_UNTITLED_TAB_NO_DEVICE_NAME_LABEL 10509
#define IDS_TAB_CASTING_INFOBAR_CASTING_ANOTHER_TAB_LABEL 10510
#define IDS_TAB_CASTING_INFOBAR_CASTING_ANOTHER_TAB_NO_DEVICE_NAME_LABEL 10511
#define IDS_TAB_CASTING_INFOBAR_CAST_BUTTON 10512
#define IDS_TAB_CASTING_INFOBAR_STOP_BUTTON 10513
#define IDS_TASK_MANAGER_KILL 10514
#define IDS_TASK_MANAGER_KILL_V2 10515
#define IDS_TASK_MANAGER_PROCESS_ID_COLUMN 10516
#define IDS_TASK_MANAGER_GDI_HANDLES_COLUMN 10517
#define IDS_TASK_MANAGER_USER_HANDLES_COLUMN 10518
#define IDS_TASK_MANAGER_TASK_COLUMN 10519
#define IDS_TASK_MANAGER_NET_COLUMN 10520
#define IDS_TASK_MANAGER_CPU_COLUMN 10521
#define IDS_TASK_MANAGER_START_TIME_COLUMN 10522
#define IDS_TASK_MANAGER_CPU_TIME_COLUMN 10523
#define IDS_TASK_MANAGER_MEM_FOOTPRINT_COLUMN 10524
#define IDS_TASK_MANAGER_SWAPPED_MEM_COLUMN 10525
#define IDS_TASK_MANAGER_PROFILE_NAME_COLUMN 10526
#define IDS_TASK_MANAGER_IDLE_WAKEUPS_COLUMN 10527
#define IDS_TASK_MANAGER_HARD_FAULTS_COLUMN 10528
#define IDS_TASK_MANAGER_OPEN_FD_COUNT_COLUMN 10529
#define IDS_TASK_MANAGER_PROCESS_PRIORITY_COLUMN 10530
#define IDS_TASK_MANAGER_WEBCORE_IMAGE_CACHE_COLUMN 10531
#define IDS_TASK_MANAGER_WEBCORE_SCRIPTS_CACHE_COLUMN 10532
#define IDS_TASK_MANAGER_WEBCORE_CSS_CACHE_COLUMN 10533
#define IDS_TASK_MANAGER_VIDEO_MEMORY_COLUMN 10534
#define IDS_TASK_MANAGER_SQLITE_MEMORY_USED_COLUMN 10535
#define IDS_TASK_MANAGER_JAVASCRIPT_MEMORY_ALLOCATED_COLUMN 10536
#define IDS_TASK_MANAGER_KEEPALIVE_COUNT_COLUMN 10537
#define IDS_TASK_MANAGER_MEM_CELL_TEXT 10538
#define IDS_TASK_MANAGER_CACHE_SIZE_CELL_TEXT 10539
#define IDS_TASK_MANAGER_NA_CELL_TEXT 10540
#define IDS_TASK_MANAGER_BACKGROUNDED_TEXT 10541
#define IDS_TASK_MANAGER_FOREGROUNDED_TEXT 10542
#define IDS_TASK_MANAGER_HANDLES_CELL_TEXT 10543
#define IDS_TASK_MANAGER_TASK_GROUP_CONNECT_TEXT 10544
#define IDS_TASK_MANAGER_WEB_BROWSER_CELL_TEXT 10545
#define IDS_TASK_MANAGER_EXTENSION_PREFIX 10546
#define IDS_TASK_MANAGER_EXTENSION_INCOGNITO_PREFIX 10547
#define IDS_TASK_MANAGER_APP_PREFIX 10548
#define IDS_TASK_MANAGER_APP_INCOGNITO_PREFIX 10549
#define IDS_TASK_MANAGER_TAB_PREFIX 10550
#define IDS_TASK_MANAGER_TAB_INCOGNITO_PREFIX 10551
#define IDS_TASK_MANAGER_BACKGROUND_APP_PREFIX 10552
#define IDS_TASK_MANAGER_BACKGROUND_PREFIX 10553
#define IDS_TASK_MANAGER_BACK_FORWARD_CACHE_PREFIX 10554
#define IDS_TASK_MANAGER_BACK_FORWARD_CACHE_INCOGNITO_PREFIX 10555
#define IDS_TASK_MANAGER_PRELOADED_RENDERER_FOR_UI 10556
#define IDS_TASK_MANAGER_NO_STATE_PREFETCH_PREFIX 10557
#define IDS_TASK_MANAGER_PRERENDER_PREFIX 10558
#define IDS_TASK_MANAGER_SPARE_RENDERER_PREFIX 10559
#define IDS_TASK_MANAGER_UNKNOWN_RENDERER_PREFIX 10560
#define IDS_TASK_MANAGER_DEDICATED_WORKER_PREFIX 10561
#define IDS_TASK_MANAGER_SHARED_WORKER_PREFIX 10562
#define IDS_TASK_MANAGER_SERVICE_WORKER_PREFIX 10563
#define IDS_TASK_MANAGER_UTILITY_PREFIX 10564
#define IDS_TASK_MANAGER_GPU_PREFIX 10565
#define IDS_TASK_MANAGER_PRINT_PREFIX 10566
#define IDS_TASK_MANAGER_SUBFRAME_PREFIX 10567
#define IDS_TASK_MANAGER_SUBFRAME_INCOGNITO_PREFIX 10568
#define IDS_TASK_MANAGER_BACK_FORWARD_CACHE_SUBFRAME_PREFIX 10569
#define IDS_TASK_MANAGER_BACK_FORWARD_CACHE_INCOGNITO_SUBFRAME_PREFIX 10570
#define IDS_TASK_MANAGER_PDF_SUBFRAME_PREFIX 10571
#define IDS_TASK_MANAGER_PDF_SUBFRAME_INCOGNITO_PREFIX 10572
#define IDS_TASK_MANAGER_PORTAL_PREFIX 10573
#define IDS_TASK_MANAGER_PORTAL_INCOGNITO_PREFIX 10574
#define IDS_TASK_MANAGER_FENCED_FRAME_PREFIX 10575
#define IDS_TASK_MANAGER_FENCED_FRAME_INCOGNITO_PREFIX 10576
#define IDS_TASK_MANAGER_ARC_PREFIX 10577
#define IDS_TASK_MANAGER_TOOL_PREFIX 10578
#define IDS_TASK_MANAGER_ARC_PREFIX_BACKGROUND_SERVICE 10579
#define IDS_TASK_MANAGER_ARC_PREFIX_RECEIVER 10580
#define IDS_TASK_MANAGER_ARC_SYSTEM 10581
#define IDS_TASK_MANAGER_LINUX_VM_PREFIX 10582
#define IDS_TASK_MANAGER_PLUGIN_VM_PREFIX 10583
#define IDS_TASK_MANAGER_OMNIBOX 10584
#define IDS_UTILITY_PROCESS_FILE_UTILITY_NAME 10585
#define IDS_UTILITY_PROCESS_PROFILE_IMPORTER_NAME 10586
#define IDS_UTILITY_PROCESS_QRCODE_GENERATOR_SERVICE_NAME 10587
#define IDS_UTILITY_PROCESS_IMAGE_WRITER_NAME 10589
#define IDS_UTILITY_PROCESS_MEDIA_GALLERY_UTILITY_NAME 10590
#define IDS_UTILITY_PROCESS_NOOP_SERVICE_NAME 10591
#define IDS_SERVICE_PROCESS_DOCUMENT_ANALYSIS_NAME 10592
#define IDS_THEME_INSTALL_INFOBAR_LABEL 10593
#define IDS_THEME_INSTALL_INFOBAR_UNDO_BUTTON 10594
#define IDS_CRITICAL_NOTIFICATION_RESTART 10595
#define IDS_EXTENSION_DISABLED_ERROR_LABEL 10596
#define IDS_EXTENSION_DISABLED_REMOTE_INSTALL_ERROR_TITLE 10597
#define IDS_EXTENSION_DISABLED_ERROR_TITLE 10598
#define IDS_EXTENSION_SITE_RELOAD_PAGE_BUBBLE_HEADING 10599
#define IDS_EXTENSION_RELOAD_PAGE_BUBBLE_OK_BUTTON 10600
#define IDS_EXTENSION_RELOAD_PAGE_BUBBLE_ALLOW_SINGLE_EXTENSION_TITLE 10601
#define IDS_EXTENSION_RELOAD_PAGE_BUBBLE_ALLOW_MULTIPLE_EXTENSIONS_TITLE 10602
#define IDS_APP_UNINSTALL_PROMPT_TITLE 10603
#define IDS_ARC_APP_UNINSTALL_PROMPT_DATA_REMOVAL_WARNING 10604
#define IDS_EXTENSION_UNINSTALL_PROMPT_TITLE 10605
#define IDS_EXTENSION_CONFIRM_PERMISSIONS 10606
#define IDS_EXTENSION_INSTALL_PROMPT_TITLE 10607
#define IDS_EXTENSION_REQUEST_PROMPT_TITLE 10608
#define IDS_EXTENSION_PENDING_REQUEST_PROMPT_TITLE 10609
#define IDS_EXTENSION_BLOCKED_BY_POLICY_PROMPT_TITLE 10610
#define IDS_EXTENSION_RE_ENABLE_PROMPT_TITLE 10611
#define IDS_EXTENSION_PERMISSIONS_PROMPT_TITLE 10612
#define IDS_EXTENSION_REMOTE_INSTALL_PROMPT_TITLE 10613
#define IDS_EXTENSION_REPAIR_PROMPT_TITLE 10614
#define IDS_EXTENSION_ASK_PARENT_PROMPT_TITLE 10615
#define IDS_EXTENSION_ASK_PARENT_PROMPT_BODY 10616
#define IDS_EXTENSION_ASK_PARENT_PROMPT_ASK_IN_PERSON_BUTTON 10617
#define IDS_EXTENSION_PARENT_APPROVAL_PROMPT_TITLE 10618
#define IDS_EXTENSION_EXTERNAL_INSTALL_PROMPT_ACCEPT_BUTTON_EXTENSION 10619
#define IDS_EXTENSION_EXTERNAL_INSTALL_PROMPT_ACCEPT_BUTTON_APP 10620
#define IDS_EXTENSION_EXTERNAL_INSTALL_PROMPT_ACCEPT_BUTTON_THEME 10621
#define IDS_EXTENSION_ALERT_TITLE 10622
#define IDS_APP_ALERT_TITLE 10623
#define IDS_EXTENSION_AND_APP_ALERT_TITLE 10624
#define IDS_POLICY_BLOCKED_EXTENSION_ALERT_TITLE 10625
#define IDS_POLICY_BLOCKED_EXTENSIONS_ALERT_ITEM_TITLE 10626
#define IDS_BLOCKLISTED_EXTENSIONS_ALERT_ITEM 10627
#define IDS_POLICY_BLOCKED_EXTENSION_ALERT_ITEM_DETAIL 10628
#define IDS_EXTENSION_ALERT_ITEM_OK 10629
#define IDS_EXTENSION_ALERT_ITEM_DETAILS 10630
#define IDS_EXTENSION_PROMPT_APP_CONNECT_FROM_INCOGNITO 10631
#define IDS_EXTENSION_PROMPT_EXTENSION_CONNECT_FROM_INCOGNITO 10632
#define IDS_EXTENSION_PROMPT_WILL_HAVE_ACCESS_TO 10633
#define IDS_EXTENSION_PROMPT_WILL_NOW_HAVE_ACCESS_TO 10634
#define IDS_EXTENSION_PROMPT_WANTS_ACCESS_TO 10635
#define IDS_EXTENSION_PROMPT_CAN_ACCESS 10636
#define IDS_EXTENSION_PROMPT_REQUESTS_PERMISSIONS 10637
#define IDS_EXTENSION_PROMPT_GRANT_PERMISSIONS_CHECKBOX 10638
#define IDS_EXTENSION_PERMISSION_LINE 10639
#define IDS_EXTENSION_RATING_COUNT 10640
#define IDS_EXTENSION_PROMPT_RATING_ACCESSIBLE_TEXT 10641
#define IDS_EXTENSION_PROMPT_NO_RATINGS_ACCESSIBLE_TEXT 10642
#define IDS_EXTENSION_USER_COUNT 10643
#define IDS_EXTENSION_PROMPT_STORE_LINK 10644
#define IDS_EXTENSION_PROMPT_WARNING_FULL_ACCESS 10645
#define IDS_EXTENSION_PROMPT_WARNING_ALL_HOSTS 10646
#define IDS_EXTENSION_PROMPT_WARNING_ALL_HOSTS_READ_ONLY 10647
#define IDS_EXTENSION_PROMPT_WARNING_AUDIO_CAPTURE 10648
#define IDS_EXTENSION_PROMPT_WARNING_VIDEO_CAPTURE 10649
#define IDS_EXTENSION_PROMPT_WARNING_AUDIO_AND_VIDEO_CAPTURE 10650
#define IDS_EXTENSION_PROMPT_WARNING_BLUETOOTH 10651
#define IDS_EXTENSION_PROMPT_WARNING_BLUETOOTH_DEVICES 10652
#define IDS_EXTENSION_PROMPT_WARNING_BLUETOOTH_PRIVATE 10653
#define IDS_EXTENSION_PROMPT_WARNING_BLUETOOTH_SERIAL 10654
#define IDS_EXTENSION_PROMPT_WARNING_BOOKMARKS 10655
#define IDS_EXTENSION_PROMPT_WARNING_READING_LIST 10656
#define IDS_EXTENSION_PROMPT_WARNING_CLIPBOARD 10657
#define IDS_EXTENSION_PROMPT_WARNING_CLIPBOARD_READWRITE 10658
#define IDS_EXTENSION_PROMPT_WARNING_CLIPBOARD_WRITE 10659
#define IDS_EXTENSION_PROMPT_WARNING_DEBUGGER 10660
#define IDS_EXTENSION_PROMPT_WARNING_DECLARATIVE_WEB_REQUEST 10661
#define IDS_EXTENSION_PROMPT_WARNING_DECLARATIVE_NET_REQUEST 10662
#define IDS_EXTENSION_PROMPT_WARNING_DOCUMENT_SCAN 10663
#define IDS_EXTENSION_PROMPT_WARNING_ENTERPRISE_HARDWARE_PLATFORM 10664
#define IDS_EXTENSION_PROMPT_WARNING_FAVICON 10665
#define IDS_EXTENSION_PROMPT_WARNING_GEOLOCATION 10666
#define IDS_EXTENSION_PROMPT_WARNING_HISTORY_READ 10667
#define IDS_EXTENSION_PROMPT_WARNING_HISTORY_READ_ON_ALL_DEVICES 10668
#define IDS_EXTENSION_PROMPT_WARNING_HISTORY_WRITE_ON_ALL_DEVICES 10669
#define IDS_EXTENSION_PROMPT_WARNING_HOME_PAGE_SETTING_OVERRIDE 10670
#define IDS_EXTENSION_PROMPT_WARNING_1_HOST 10671
#define IDS_EXTENSION_PROMPT_WARNING_1_HOST_READ_ONLY 10672
#define IDS_EXTENSION_PROMPT_WARNING_2_HOSTS 10673
#define IDS_EXTENSION_PROMPT_WARNING_2_HOSTS_READ_ONLY 10674
#define IDS_EXTENSION_PROMPT_WARNING_3_HOSTS 10675
#define IDS_EXTENSION_PROMPT_WARNING_3_HOSTS_READ_ONLY 10676
#define IDS_EXTENSION_PROMPT_WARNING_HOSTS_LIST 10677
#define IDS_EXTENSION_PROMPT_WARNING_HOSTS_LIST_READ_ONLY 10678
#define IDS_EXTENSION_PROMPT_WARNING_HOST_AND_SUBDOMAIN 10679
#define IDS_EXTENSION_PROMPT_WARNING_HOST_AND_SUBDOMAIN_LIST 10680
#define IDS_EXTENSION_PROMPT_WARNING_INPUT 10681
#define IDS_EXTENSION_PROMPT_WARNING_LOGIN 10682
#define IDS_EXTENSION_PROMPT_WARNING_ENTERPRISE_LOGIN 10683
#define IDS_EXTENSION_PROMPT_WARNING_LOGIN_SCREEN_UI 10684
#define IDS_EXTENSION_PROMPT_WARNING_LOGIN_SCREEN_STORAGE 10685
#define IDS_EXTENSION_PROMPT_WARNING_MANAGEMENT 10686
#define IDS_EXTENSION_PROMPT_WARNING_MDNS 10687
#define IDS_EXTENSION_PROMPT_WARNING_NETWORK_STATE 10688
#define IDS_EXTENSION_PROMPT_WARNING_NETWORKING_PRIVATE 10689
#define IDS_EXTENSION_PROMPT_WARNING_PRINTING 10690
#define IDS_EXTENSION_PROMPT_WARNING_PRINTING_METRICS 10691
#define IDS_EXTENSION_PROMPT_WARNING_SEARCH_SETTINGS_OVERRIDE 10692
#define IDS_EXTENSION_PROMPT_WARNING_SERIAL 10693
#define IDS_EXTENSION_PROMPT_WARNING_SOCKET_ANY_HOST 10694
#define IDS_EXTENSION_PROMPT_WARNING_SOCKET_HOSTS_IN_DOMAIN 10695
#define IDS_EXTENSION_PROMPT_WARNING_SOCKET_HOSTS_IN_DOMAINS 10696
#define IDS_EXTENSION_PROMPT_WARNING_SOCKET_SPECIFIC_HOST 10697
#define IDS_EXTENSION_PROMPT_WARNING_SOCKET_SPECIFIC_HOSTS 10698
#define IDS_EXTENSION_PROMPT_WARNING_SPEECH_RECOGNITION 10699
#define IDS_EXTENSION_PROMPT_WARNING_START_PAGE_SETTING_OVERRIDE 10700
#define IDS_EXTENSION_PROMPT_WARNING_SYSTEM_STORAGE 10701
#define IDS_EXTENSION_PROMPT_WARNING_TAB_GROUPS 10702
#define IDS_EXTENSION_PROMPT_WARNING_TOPSITES 10703
#define IDS_EXTENSION_PROMPT_WARNING_TTS_ENGINE 10704
#define IDS_EXTENSION_PROMPT_WARNING_U2F_DEVICES 10705
#define IDS_EXTENSION_PROMPT_WARNING_NOTIFICATIONS 10706
#define IDS_EXTENSION_PROMPT_WARNING_USB_DEVICE 10707
#define IDS_EXTENSION_PROMPT_WARNING_USB_DEVICE_LIST 10708
#define IDS_EXTENSION_PROMPT_WARNING_USB_DEVICE_LIST_ITEM_UNKNOWN_PRODUCT 10709
#define IDS_EXTENSION_PROMPT_WARNING_USB_DEVICE_LIST_ITEM_UNKNOWN_VENDOR 10710
#define IDS_EXTENSION_PROMPT_WARNING_USB_DEVICE_UNKNOWN_PRODUCT 10711
#define IDS_EXTENSION_PROMPT_WARNING_USB_DEVICE_UNKNOWN_VENDOR 10712
#define IDS_EXTENSION_PROMPT_WARNING_VPN 10713
#define IDS_EXTENSION_PROMPT_WARNING_CONTENT_SETTINGS 10714
#define IDS_EXTENSION_PROMPT_WARNING_PRIVACY 10715
#define IDS_EXTENSION_PROMPT_WARNING_DOWNLOADS 10716
#define IDS_EXTENSION_PROMPT_WARNING_DOWNLOADS_OPEN 10717
#define IDS_EXTENSION_PROMPT_WARNING_IDENTITY_EMAIL 10718
#define IDS_EXTENSION_PROMPT_WARNING_WALLPAPER 10719
#define IDS_EXTENSION_PROMPT_WARNING_FILE_SYSTEM_DIRECTORY 10720
#define IDS_EXTENSION_PROMPT_WARNING_FILE_SYSTEM_WRITE_DIRECTORY 10721
#define IDS_EXTENSION_PROMPT_WARNING_MEDIA_GALLERIES_READ 10722
#define IDS_EXTENSION_PROMPT_WARNING_MEDIA_GALLERIES_READ_WRITE 10723
#define IDS_EXTENSION_PROMPT_WARNING_MEDIA_GALLERIES_READ_DELETE 10724
#define IDS_EXTENSION_PROMPT_WARNING_MEDIA_GALLERIES_READ_WRITE_DELETE 10725
#define IDS_EXTENSION_PROMPT_WARNING_SYNCFILESYSTEM 10726
#define IDS_EXTENSION_PROMPT_WARNING_NATIVE_MESSAGING 10727
#define IDS_EXTENSION_PROMPT_WARNING_SCREENLOCK_PRIVATE 10728
#define IDS_EXTENSION_PROMPT_WARNING_ACTIVITY_LOG_PRIVATE 10729
#define IDS_EXTENSION_PROMPT_WARNING_DESKTOP_CAPTURE 10730
#define IDS_EXTENSION_PROMPT_WARNING_ACCESSIBILITY_FEATURES_MODIFY 10731
#define IDS_EXTENSION_PROMPT_WARNING_ACCESSIBILITY_FEATURES_READ 10732
#define IDS_EXTENSION_PROMPT_WARNING_ACCESSIBILITY_FEATURES_READ_MODIFY 10733
#define IDS_EXTENSION_PROMPT_WARNING_PLATFORMKEYS 10734
#define IDS_EXTENSION_PROMPT_WARNING_CERTIFICATEPROVIDER 10735
#define IDS_EXTENSION_PROMPT_WARNING_SETTINGS_PRIVATE 10736
#define IDS_EXTENSION_PROMPT_WARNING_AUTOFILL_PRIVATE 10737
#define IDS_EXTENSION_PROMPT_WARNING_PASSWORDS_PRIVATE 10738
#define IDS_EXTENSION_PROMPT_WARNING_USERS_PRIVATE 10739
#define IDS_EXTENSION_PROMPT_WARNING_NEW_TAB_PAGE_OVERRIDE 10740
#define IDS_EXTENSION_PROMPT_WARNING_TRANSIENT_BACKGROUND 10741
#define IDS_EXTENSION_PROMPT_WARNING_ENTERPRISE_DEVICE_ATTRIBUTES 10742
#define IDS_EXTENSION_PROMPT_WARNING_ENTERPRISE_KIOSK_INPUT 10743
#define IDS_EXTENSION_PROMPT_WARNING_ENTERPRISE_NETWORKING_ATTRIBUTES 10744
#define IDS_EXTENSION_PROMPT_WARNING_ENTERPRISE_PLATFORMKEYS 10745
#define IDS_EXTENSION_PROMPT_WARNING_ENTERPRISE_REPORTING_PRIVATE_ENABLED_LINUX_AND_MACOS 10747
#define IDS_EXTENSION_PROMPT_WARNING_ENTERPRISE_REPORTING_PRIVATE 10748
#define IDS_EXTENSION_PROMPT_WARNING_ENTERPRISE_REMOTE_APPS 10749
#define IDS_EXTENSION_PROMPT_WARNING_CHROMEOS_ATTACHED_DEVICE_INFO 10750
#define IDS_EXTENSION_PROMPT_WARNING_CHROMEOS_BLUETOOTH_PERIPHERALS_INFO 10751
#define IDS_EXTENSION_PROMPT_WARNING_CHROMEOS_DIAGNOSTICS 10752
#define IDS_EXTENSION_PROMPT_WARNING_CHROMEOS_DIAGNOSTICS_NETWORK_INFO_FOR_MLAB 10753
#define IDS_EXTENSION_PROMPT_WARNING_CHROMEOS_EVENTS 10754
#define IDS_EXTENSION_PROMPT_WARNING_CHROMEOS_MANAGEMENT_AUDIO 10755
#define IDS_EXTENSION_PROMPT_WARNING_CHROMEOS_TELEMETRY 10756
#define IDS_EXTENSION_PROMPT_WARNING_CHROMEOS_TELEMETRY_SERIAL_NUMBER 10757
#define IDS_EXTENSION_PROMPT_WARNING_CHROMEOS_TELEMETRY_NETWORK_INFORMATION 10758
#define IDS_EXTENSION_PROMPT_WARNING_OMNIBOX_DIRECT_INPUT 10759
#define IDS_EXTENSION_INVALID_IMAGE_PATH 10762
#define IDS_EXTENSION_ASKS_IDENTITY_WHILE_SIGNED_IN_WEB_ONLY_BODY_PART_2 10764
#define IDS_EXTENSION_INSTALLED_PAGE_ACTION_INFO 10766
#define IDS_EXTENSION_INSTALLED_PAGE_ACTION_INFO_WITH_SHORTCUT 10767
#define IDS_EXTENSION_INSTALLED_BROWSER_ACTION_INFO 10768
#define IDS_EXTENSION_INSTALLED_BROWSER_ACTION_INFO_WITH_SHORTCUT 10769
#define IDS_EXTENSION_INSTALLED_OMNIBOX_KEYWORD_INFO 10770
#define IDS_EXTENSION_INSTALLED_MANAGE_INFO 10771
#define IDS_EXTENSION_INSTALLED_MANAGE_SHORTCUTS 10773
#define IDS_EXTENSION_INSTALLED_MANAGE_SHORTCUTS_LINK_TEXT 10774
#define IDS_EXTENSION_LAUNCH_WEB_AUTH_FLOW_TAB_INFO_BAR_TEXT 10775
#define IDS_EXTENSION_INSTALLED_DICE_PROMO_SYNC_MESSAGE 10776
#define IDS_EXTENSION_VERIFY_PROMO_SUBTITLE 10777
#define IDS_EXTENSIONS_DIRECTORY_CONFIRMATION_DIALOG_TITLE 10778
#define IDS_EXTENSIONS_DIRECTORY_CONFIRMATION_DIALOG_MESSAGE_READ_ONLY 10779
#define IDS_EXTENSIONS_DIRECTORY_CONFIRMATION_DIALOG_MESSAGE_WRITABLE 10780
#define IDS_DIRECT_SOCKETS_CONNECTION_BUBBLE_TITLE_LABEL 10781
#define IDS_DIRECT_SOCKETS_CONNECTION_BUBBLE_ADDRESS_LABEL 10782
#define IDS_DIRECT_SOCKETS_CONNECTION_BUBBLE_PORT_LABEL 10783
#define IDS_EXTENSIONS_WANTS_ACCESS_TO_SITE 10784
#define IDS_EXTENSIONS_HAS_ACCESS_TO_SITE 10785
#define IDS_EXTENSIONS_REQUEST_ACCESS_BUTTON_TOOLTIP_SINGLE_EXTENSION 10786
#define IDS_EXTENSIONS_TOOLBAR_ACTION_HOVER_CARD_TITLE_HAS_ACCESS 10787
#define IDS_EXTENSIONS_TOOLBAR_ACTION_HOVER_CARD_TITLE_BLOCKED_ACCESS 10788
#define IDS_EXTENSIONS_TOOLBAR_ACTION_HOVER_CARD_TITLE_REQUESTS_ACCESS 10789
#define IDS_EXTENSIONS_TOOLBAR_ACTION_HOVER_CARD_DESCRIPTION_ALL_EXTENSIONS_ALLOWED_ACCESS 10790
#define IDS_EXTENSIONS_TOOLBAR_ACTION_HOVER_CARD_DESCRIPTION_ALL_EXTENSIONS_BLOCKED_ACCESS 10791
#define IDS_EXTENSIONS_TOOLBAR_ACTION_HOVER_CARD_DESCRIPTION_EXTENSION_HAS_ACCESS 10792
#define IDS_EXTENSIONS_TOOLBAR_ACTION_HOVER_CARD_DESCRIPTION_EXTENSION_REQUESTS_ACCESS 10793
#define IDS_EXTENSIONS_TOOLBAR_ACTION_HOVER_CARD_POLICY_LABEL_PINNED_TEXT 10794
#define IDS_EXTENSIONS_TOOLBAR_ACTION_HOVER_CARD_POLICY_LABEL_INSTALLED_TEXT 10795
#define IDS_EXTENSIONS_CONTEXT_MENU_SITE_PERMISSIONS 10796
#define IDS_EXTENSIONS_CONTEXT_MENU_CANT_ACCESS_PAGE 10797
#define IDS_EXTENSIONS_CONTEXT_MENU_PAGE_ACCESS 10798
#define IDS_EXTENSIONS_CONTEXT_MENU_PAGE_ACCESS_ALL_EXTENSIONS_GRANTED 10799
#define IDS_EXTENSIONS_CONTEXT_MENU_PAGE_ACCESS_ALL_EXTENSIONS_BLOCKED 10800
#define IDS_EXTENSIONS_CONTEXT_MENU_PAGE_ACCESS_RUN_ON_CLICK_V2 10801
#define IDS_EXTENSIONS_CONTEXT_MENU_PAGE_ACCESS_RUN_ON_SITE_V2 10802
#define IDS_EXTENSIONS_CONTEXT_MENU_PAGE_ACCESS_RUN_ON_ALL_SITES_V2 10803
#define IDS_EXTENSIONS_CONTEXT_MENU_PAGE_ACCESS_PERMISSIONS_PAGE 10804
#define IDS_EXTENSIONS_CONTEXT_MENU_PAGE_ACCESS_RUN_ON_CLICK 10805
#define IDS_EXTENSIONS_CONTEXT_MENU_PAGE_ACCESS_RUN_ON_SITE 10806
#define IDS_EXTENSIONS_CONTEXT_MENU_PAGE_ACCESS_RUN_ON_ALL_SITES 10807
#define IDS_EXTENSIONS_CONTEXT_MENU_PAGE_ACCESS_LEARN_MORE 10808
#define IDS_EXTENSIONS_CONTEXT_MENU_PIN_TO_TOOLBAR 10809
#define IDS_EXTENSIONS_CONTEXT_MENU_UNPIN_FROM_TOOLBAR 10810
#define IDS_EXTENSIONS_OPTIONS_MENU_ITEM 10811
#define IDS_EXTENSIONS_INSTALLED_BY_ADMIN 10812
#define IDS_EXTENSIONS_PIN_TO_TOOLBAR 10813
#define IDS_EXTENSIONS_UNPIN_FROM_TOOLBAR 10814
#define IDS_EXTENSIONS_PINNED_BY_ADMIN 10815
#define IDS_MANAGE_EXTENSION 10816
#define IDS_MANAGE_EXTENSIONS 10817
#define IDS_VIEW_WEB_PERMISSIONS 10818
#define IDS_EXTENSION_ACTION_INSPECT_POPUP 10819
#define IDS_EXTENSIONS_DISABLE 10820
#define IDS_EXTENSIONS_PIN_TO_TOOLBAR_ACCESSIBLE_NAME 10821
#define IDS_EXTENSIONS_UNPIN_FROM_TOOLBAR_ACCESSIBLE_NAME 10822
#define IDS_EXTENSIONS_PINNED_BY_ADMIN_ACCESSIBLE_NAME 10823
#define IDS_EXTENSIONS_SUBMENU_OPEN_SIDE_PANEL_ITEM 10824
#define IDS_EXTENSIONS_SUBMENU_CLOSE_SIDE_PANEL_ITEM 10825
#define IDS_EXTENSIONS_LOCKED_SUPERVISED_USER 10829
#define IDS_EXTENSION_LOAD_FROM_DIRECTORY 10830
#define IDS_EXTENSION_COMMANDS_GENERIC_ACTIVATE 10831
#define IDS_EXTENSION_PACK_DIALOG_HEADING 10832
#define IDS_EXTENSION_PACK_DIALOG_SELECT_KEY 10833
#define IDS_EXTENSION_PACK_DIALOG_KEY_FILE_TYPE_DESCRIPTION 10834
#define IDS_EXTENSION_PACK_DIALOG_ERROR_ROOT_REQUIRED 10835
#define IDS_EXTENSION_PACK_DIALOG_ERROR_ROOT_INVALID 10836
#define IDS_EXTENSION_PACK_DIALOG_ERROR_KEY_INVALID 10837
#define IDS_EXTENSION_PACK_DIALOG_SUCCESS_BODY_NEW 10838
#define IDS_EXTENSION_PACK_DIALOG_SUCCESS_BODY_UPDATE 10839
#define IDS_EXTENSION_PROMPT_INSTALL_FRICTION_CONTINUE_BUTTON 10840
#define IDS_EXTENSION_PROMPT_INSTALL_FRICTION_TITLE 10841
#define IDS_EXTENSION_PROMPT_INSTALL_FRICTION_WARNING_TEXT 10842
#define IDS_EXTENSION_INSTALL_PROMPT_ACCEPT_BUTTON_EXTENSION 10843
#define IDS_EXTENSION_INSTALL_PROMPT_ACCEPT_BUTTON_APP 10844
#define IDS_EXTENSION_INSTALL_PROMPT_ACCEPT_BUTTON_THEME 10845
#define IDS_EXTENSION_INSTALL_PROMPT_ASK_A_PARENT_BUTTON 10846
#define IDS_EXTENSION_INSTALL_BLOCKED_BY_PARENT_PROMPT_TITLE 10847
#define IDS_EXTENSION_ENABLE_BLOCKED_BY_PARENT_PROMPT_TITLE 10848
#define IDS_PARENT_PERMISSION_PROMPT_GO_GET_A_PARENT_FOR_EXTENSION_LABEL 10849
#define IDS_PARENT_PERMISSION_PROMPT_EXTENSION_TYPE_EXTENSION 10850
#define IDS_PARENT_PERMISSION_PROMPT_EXTENSION_TYPE_APP 10851
#define IDS_PARENT_PERMISSION_PROMPT_CHILD_WANTS_TO_INSTALL_LABEL 10852
#define IDS_PARENT_PERMISSION_PROMPT_APPROVE_BUTTON 10853
#define IDS_PARENT_PERMISSION_PROMPT_CANCEL_BUTTON 10854
#define IDS_PARENT_PERMISSION_PROMPT_SELECT_PARENT_LABEL 10855
#define IDS_PARENT_PERMISSION_PROMPT_PARENT_ACCOUNT_LABEL 10856
#define IDS_PARENT_PERMISSION_PROMPT_PARENT_ACCOUNT_VERIFY_LABEL 10857
#define IDS_PARENT_PROMPT_ENTER_PASSWORD_TO_ALLOW_EXTENSION_LABEL 10858
#define IDS_SKIP_PARENT_PERMISSION_EXTENSION_INFORMATION 10859
#define IDS_PARENT_PERMISSION_PROMPT_ENTER_PASSWORD_LABEL 10860
#define IDS_PARENT_PERMISSION_PROMPT_PASSWORD_INCORRECT_LABEL 10861
#define IDS_EXTENSION_INSTALL_PROMPT_REQUEST_BUTTON 10862
#define IDS_EXTENSION_PROMPT_UNINSTALL_BUTTON 10863
#define IDS_EXTENSION_PROMPT_UNINSTALL_REPORT_ABUSE 10864
#define IDS_EXTENSION_PROMPT_UNINSTALL_REPORT_ABUSE_FROM_EXTENSION 10865
#define IDS_EXTENSION_PROMPT_UNINSTALL_TITLE 10866
#define IDS_EXTENSION_PROMPT_UNINSTALL_APP_BUTTON 10867
#define IDS_EXTENSION_PROMPT_MULTIPLE_UNINSTALL_NAME_BULLET_POINT 10868
#define IDS_EXTENSION_PROMPT_MULTIPLE_UNINSTALL_TITLE 10869
#define IDS_EXTENSION_PROMPT_UNINSTALL_TRIGGERED_BY_EXTENSION 10870
#define IDS_EXTENSION_PROMPT_RE_ENABLE_BUTTON 10871
#define IDS_EXTENSION_PROMPT_PERMISSIONS_BUTTON 10872
#define IDS_EXTENSION_PROMPT_PERMISSIONS_ACCEPT_BUTTON 10873
#define IDS_EXTENSION_PROMPT_PERMISSIONS_ABORT_BUTTON 10874
#define IDS_EXTENSION_PROMPT_REMOTE_INSTALL_BUTTON_EXTENSION 10875
#define IDS_EXTENSION_PROMPT_REMOTE_INSTALL_BUTTON_APP 10876
#define IDS_EXTENSION_PROMPT_REPAIR_BUTTON_EXTENSION 10877
#define IDS_EXTENSION_PROMPT_REPAIR_BUTTON_APP 10878
#define IDS_EXTENSION_WEB_STORE_TITLE_SHORT 10879
#define IDS_EXTENSIONS_SHOW_DETAILS 10880
#define IDS_EXTENSIONS_HIDE_DETAILS 10881
#define IDS_EXTENSION_WARNINGS_WRENCH_MENU_ITEM 10882
#define IDS_EXTENSION_EXTERNAL_INSTALL_ALERT_EXTENSION 10883
#define IDS_EXTENSION_EXTERNAL_INSTALL_ALERT_APP 10884
#define IDS_EXTENSION_EXTERNAL_INSTALL_ALERT_THEME 10885
#define IDS_EXTENSION_EXTERNAL_INSTALL_ALERT_BUBBLE_TITLE 10886
#define IDS_EXTENSION_EXTERNAL_INSTALL_ALERT_BUBBLE_HEADING_APP 10887
#define IDS_EXTENSION_EXTERNAL_INSTALL_ALERT_BUBBLE_HEADING_EXTENSION 10888
#define IDS_EXTENSION_EXTERNAL_INSTALL_ALERT_BUBBLE_HEADING_THEME 10889
#define IDS_EXTENSIONS_ADDED_WITHOUT_KNOWLEDGE 10890
#define IDS_EXTENSIONS_ADDED_WITHOUT_KNOWLEDGE_LEARN_MORE 10891
#define IDS_EXTENSIONS_MANIFEST_V2_DEPRECATION_KEEP_DIALOG_TITLE 10892
#define IDS_EXTENSIONS_MANIFEST_V2_DEPRECATION_KEEP_DIALOG_OK_BUTTON 10893
#define IDS_EXTENSIONS_MANIFEST_V2_DEPRECATION_DISABLED_DIALOG_TITLE 10894
#define IDS_EXTENSIONS_MANIFEST_V2_DEPRECATION_DISABLED_DIALOG_PLURAL_TITLE 10895
#define IDS_EXTENSIONS_MANIFEST_V2_DEPRECATION_DISABLED_DIALOG_OK_BUTTON 10896
#define IDS_EXTENSIONS_MANIFEST_V2_DEPRECATION_DISABLED_DIALOG_CANCEL_BUTTON 10897
#define IDS_EXTENSIONS_MANIFEST_V2_DEPRECATION_REENABLE_DIALOG_TITLE 10898
#define IDS_EXTENSIONS_MANIFEST_V2_DEPRECATION_REENABLE_DIALOG_OK_BUTTON 10899
#define IDS_EXTENSION_SETTINGS_OVERRIDDEN_DIALOG_CHANGE_IT_BACK 10900
#define IDS_EXTENSION_SETTINGS_OVERRIDDEN_DIALOG_KEEP_IT 10901
#define IDS_EXTENSION_NTP_OVERRIDDEN_DIALOG_TITLE_GENERIC 10902
#define IDS_EXTENSION_NTP_OVERRIDDEN_DIALOG_TITLE_BACK_TO_GOOGLE 10903
#define IDS_EXTENSION_NTP_OVERRIDDEN_DIALOG_BODY_GENERIC 10904
#define IDS_EXTENSION_SEARCH_OVERRIDDEN_DIALOG_TITLE_GENERIC 10905
#define IDS_EXTENSION_SEARCH_OVERRIDDEN_DIALOG_TITLE_BACK_TO_GOOGLE 10906
#define IDS_EXTENSION_SEARCH_OVERRIDDEN_DIALOG_TITLE_BACK_TO_OTHER 10907
#define IDS_EXTENSION_SEARCH_OVERRIDDEN_DIALOG_TITLE_EXPLICIT_CHOICE 10908
#define IDS_EXTENSION_SEARCH_OVERRIDDEN_DIALOG_BODY_GENERIC 10909
#define IDS_EXTENSION_SEARCH_OVERRIDDEN_DIALOG_BODY_EXPLICIT_CHOICE 10910
#define IDS_EXTENSION_SEARCH_OVERRIDDEN_DIALOG_RECENT_CHANGE 10911
#define IDS_EXTENSION_SEARCH_OVERRIDDEN_DIALOG_PREVIOUS_CHOICE 10912
#define IDS_FORCE_INSTALLED_DEPRECATED_APPS_CONTENT 10913
#define IDS_FORCE_INSTALLED_PREINSTALLED_DEPRECATED_APPS_TITLE 10914
#define IDS_FORCE_INSTALLED_PREINSTALLED_DEPRECATED_APPS_CONTENT 10915
#define IDS_FORCE_INSTALLED_PREINSTALLED_DEPRECATED_APPS_GO_TO_SITE_BUTTON 10916
#define IDS_DEPRECATED_APPS_RENDERER_TITLE_PLURAL 10917
#define IDS_DEPRECATED_APPS_RENDERER_TITLE_WITH_APP_NAME 10918
#define IDS_DEPRECATED_APPS_MONITOR_RENDERER 10919
#define IDS_DEPRECATED_APPS_LEARN_MORE 10920
#define IDS_DEPRECATED_APPS_LEARN_MORE_AX_LABEL 10921
#define IDS_DEPRECATED_APPS_DELETION_LINK 10922
#define IDS_DEPRECATED_APPS_OK_LABEL 10923
#define IDS_DEPRECATED_APPS_CANCEL_LABEL 10924
#define IDS_EXTENSIONS_REQUEST_ACCESS_BUTTON 10925
#define IDS_EXTENSIONS_REQUEST_ACCESS_BUTTON_ACCESSIBLE_NAME 10926
#define IDS_EXTENSIONS_REQUEST_ACCESS_BUTTON_DISMISSED_TEXT 10927
#define IDS_EXTENSIONS_REQUEST_ACCESS_BUTTON_TOOLTIP_MULTIPLE_EXTENSIONS 10928
#define IDS_EXTENSIONS_MENU_TITLE 10929
#define IDS_EXTENSIONS_MENU_CONTEXT_MENU_TOOLTIP 10930
#define IDS_EXTENSIONS_MENU_ZERO_STATE_HEADER 10931
#define IDS_EXTENSIONS_MENU_CONTEXT_MENU_TOOLTIP_ACCESSIBLE_NAME 10932
#define IDS_EXTENSIONS_MENU_PIN_BUTTON_TOOLTIP 10933
#define IDS_EXTENSIONS_MENU_UNPIN_BUTTON_TOOLTIP 10934
#define IDS_EXTENSIONS_MENU_ACCESSING_SITE_DATA_SHORT 10935
#define IDS_EXTENSIONS_MENU_ACCESSING_SITE_DATA 10936
#define IDS_EXTENSIONS_MENU_WANTS_TO_ACCESS_SITE_DATA_SHORT 10937
#define IDS_EXTENSIONS_MENU_WANTS_TO_ACCESS_SITE_DATA 10938
#define IDS_EXTENSIONS_MENU_CANT_ACCESS_SITE_DATA_SHORT 10939
#define IDS_EXTENSIONS_MENU_CANT_ACCESS_SITE_DATA 10940
#define IDS_EXTENSIONS_MENU_SITE_SETTINGS_LABEL 10941
#define IDS_EXTENSIONS_MENU_SITE_SETTINGS_NOT_ALLOWED_LABEL 10942
#define IDS_EXTENSIONS_MENU_SITE_SETTINGS_TOGGLE_ON_TOOLTIP 10943
#define IDS_EXTENSIONS_MENU_SITE_SETTINGS_TOGGLE_OFF_TOOLTIP 10944
#define IDS_EXTENSIONS_MENU_SITE_SETTINGS_TOGGLE_ACCESSIBLE_NAME 10945
#define IDS_EXTENSIONS_MENU_MESSAGE_SECTION_ENTERPRISE_TOOLTIP_ICON_TEXT 10946
#define IDS_EXTENSIONS_MENU_MESSAGE_SECTION_RELOAD_CONTAINER_DESCRIPTION_TEXT 10947
#define IDS_EXTENSIONS_MENU_MESSAGE_SECTION_RELOAD_CONTAINER_BUTTON_TEXT 10948
#define IDS_EXTENSIONS_MENU_MESSAGE_SECTION_RELOAD_CONTAINER_BUTTON_TOOLTIP 10949
#define IDS_EXTENSIONS_MENU_REQUESTS_ACCESS_SECTION_TITLE 10950
#define IDS_EXTENSIONS_MENU_REQUESTS_ACCESS_SECTION_ALLOW_BUTTON_TEXT 10951
#define IDS_EXTENSIONS_MENU_REQUESTS_ACCESS_SECTION_ALLOW_BUTTON_TOOLTIP 10952
#define IDS_EXTENSIONS_MENU_REQUESTS_ACCESS_SECTION_ALLOW_BUTTON_ACCESSIBLE_NAME 10953
#define IDS_EXTENSIONS_MENU_REQUESTS_ACCESS_SECTION_DISMISS_BUTTON_TEXT 10954
#define IDS_EXTENSIONS_MENU_REQUESTS_ACCESS_SECTION_DISMISS_BUTTON_TOOLTIP 10955
#define IDS_EXTENSIONS_MENU_REQUESTS_ACCESS_SECTION_DISMISS_BUTTON_ACCESSIBLE_NAME 10956
#define IDS_EXTENSIONS_MENU_EXTENSION_SITE_ACCESS_TOGGLE_ON_TOOLTIP 10957
#define IDS_EXTENSIONS_MENU_EXTENSION_SITE_ACCESS_TOGGLE_OFF_TOOLTIP 10958
#define IDS_EXTENSIONS_MENU_EXTENSION_SITE_ACCESS_TOGGLE_ACCESSIBLE_NAME 10959
#define IDS_EXTENSIONS_MENU_EXTENSION_CONTEXT_MENU_BUTTON_TOOLTIP 10960
#define IDS_EXTENSIONS_MENU_EXTENSION_CONTEXT_MENU_BUTTON_ACCESSIBLE_NAME 10961
#define IDS_EXTENSIONS_MENU_EXTENSION_CONTEXT_MENU_BUTTON_PINNED_ACCESSIBLE_NAME 10962
#define IDS_EXTENSIONS_MENU_MAIN_PAGE_EXTENSION_SITE_ACCESS_NONE 10963
#define IDS_EXTENSIONS_MENU_MAIN_PAGE_DISCOVER_EXTENSIONS 10964
#define IDS_EXTENSIONS_MENU_MAIN_PAGE_EXTENSION_SITE_ACCESS_TOOLTIP 10965
#define IDS_EXTENSIONS_MENU_MAIN_PAGE_ENTERPRISE_EXTENSION_SITE_ACCESS_TOOLTIP 10966
#define IDS_EXTENSIONS_MENU_MAIN_PAGE_EXTENSION_SITE_ACCESS_ACCESSIBLE_NAME 10967
#define IDS_EXTENSIONS_MENU_MAIN_PAGE_ENTERPRISE_EXTENSION_SITE_ACCESS_ACCESSIBLE_NAME 10968
#define IDS_EXTENSIONS_MENU_MAIN_PAGE_EXTENSION_BUTTON_HAS_ACCESS_TOOLTIP 10969
#define IDS_EXTENSIONS_MENU_MAIN_PAGE_EXTENSION_BUTTON_BLOCKED_ACCESS_TOOLTIP 10970
#define IDS_EXTENSIONS_MENU_MAIN_PAGE_EXTENSION_BUTTON_REQUESTS_TOOLTIP 10971
#define IDS_EXTENSIONS_MENU_MAIN_PAGE_EXTENSION_SITE_ACCESS_ON_CLICK 10972
#define IDS_EXTENSIONS_MENU_MAIN_PAGE_EXTENSION_SITE_ACCESS_ON_SITE 10973
#define IDS_EXTENSIONS_MENU_MAIN_PAGE_EXTENSION_SITE_ACCESS_ON_ALL_SITES 10974
#define IDS_EXTENSIONS_MENU_SITE_PERMISSIONS_PAGE_SETTINGS_BUTTON 10975
#define IDS_EXTENSIONS_MENU_SITE_PERMISSIONS_PAGE_SETTINGS_BUTTON_TOOLTIP 10976
#define IDS_EXTENSIONS_MENU_SITE_PERMISSIONS_PAGE_SHOW_REQUESTS_LABEL 10977
#define IDS_EXTENSIONS_MENU_SITE_PERMISSIONS_PAGE_SHOW_REQUESTS_TOGGLE_ON 10978
#define IDS_EXTENSIONS_MENU_SITE_PERMISSIONS_PAGE_SHOW_REQUESTS_TOGGLE_OFF 10979
#define IDS_EXTENSIONS_MENU_SITE_PERMISSIONS_PAGE_SITE_ACCESS_LABEL 10980
#define IDS_EXTENSIONS_MENU_SITE_PERMISSIONS_PAGE_SITE_ACCESS_ON_CLICK_TEXT 10981
#define IDS_EXTENSIONS_MENU_SITE_PERMISSIONS_PAGE_SITE_ACCESS_ON_SITE_TEXT 10982
#define IDS_EXTENSIONS_MENU_SITE_PERMISSIONS_PAGE_SITE_ACCESS_ON_ALL_SITES_TEXT 10983
#define IDS_EXTENSIONS_MENU_SITE_PERMISSIONS_PAGE_SITE_ACCESS_ON_CLICK_DESCRIPTION 10984
#define IDS_EXTENSIONS_MENU_SITE_PERMISSIONS_PAGE_SITE_ACCESS_ON_SITE_DESCRIPTION 10985
#define IDS_EXTENSIONS_MENU_SITE_PERMISSIONS_PAGE_SITE_ACCESS_ON_ALL_SITES_DESCRIPTION 10986
#define IDS_EXTENSIONS_CONTROLLED_HOME_DIALOG_TITLE 10987
#define IDS_EXTENSIONS_CONTROLLED_HOME_DIALOG_DESCRIPTION 10988
#define IDS_EXTENSIONS_CONTROLLED_HOME_DIALOG_LEARN_MORE_LINK 10989
#define IDS_EXTENSIONS_SETTINGS_API_SECOND_LINE_SEARCH_ENGINE 10990
#define IDS_EXTENSIONS_PRINTING_API_PRINT_REQUEST_BUBBLE_TITLE 10991
#define IDS_EXTENSIONS_PRINTING_API_PRINT_REQUEST_BUBBLE_HEADING 10992
#define IDS_EXTENSIONS_PRINTING_API_PRINT_REQUEST_ALLOW 10993
#define IDS_EXTENSIONS_PRINTING_API_PRINT_REQUEST_DENY 10994
#define IDS_EXTENSIONS_DOCUMENT_SCAN_API_DISCOVERY_REQUEST_BUBBLE_TITLE 10995
#define IDS_EXTENSIONS_DOCUMENT_SCAN_API_DISCOVERY_REQUEST_BUBBLE_HEADING 10996
#define IDS_EXTENSIONS_DOCUMENT_SCAN_API_DISCOVERY_REQUEST_ALLOW 10997
#define IDS_EXTENSIONS_DOCUMENT_SCAN_API_DISCOVERY_REQUEST_DENY 10998
#define IDS_EXTENSIONS_DOCUMENT_SCAN_API_START_SCAN_REQUEST_BUBBLE_TITLE 10999
#define IDS_EXTENSIONS_DOCUMENT_SCAN_API_START_SCAN_REQUEST_BUBBLE_HEADING 11000
#define IDS_EXTENSIONS_DOCUMENT_SCAN_API_START_SCAN_REQUEST_ALLOW 11001
#define IDS_EXTENSIONS_DOCUMENT_SCAN_API_START_SCAN_REQUEST_DENY 11002
#define IDS_EXTENSIONS_NTP_CONTROLLED_FIRST_LINE 11003
#define IDS_EXTENSIONS_PROXY_CONTROLLED_FIRST_LINE 11004
#define IDS_EXTENSIONS_PROXY_CONTROLLED_FIRST_LINE_EXTENSION_SPECIFIC 11005
#define IDS_EXTENSIONS_SETTINGS_API_THIRD_LINE_CONFIRMATION 11006
#define IDS_EXTENSION_CONTROLLED_RESTORE_SETTINGS 11007
#define IDS_EXTENSION_CONTROLLED_KEEP_CHANGES 11008
#define IDS_SUB_APPS_INSTALL_DIALOG_TITLE 11009
#define IDS_SUB_APPS_INSTALL_DIALOG_DESCRIPTION 11010
#define IDS_SUB_APPS_INSTALL_DIALOG_PERMISSIONS_DESCRIPTION 11011
#define IDS_SUB_APPS_INSTALL_DIALOG_MANAGE_PERMISSIONS_LINK 11012
#define IDS_SUB_APPS_INSTALL_DIALOG_PERMISSIONS_BUTTON 11013
#define IDS_SUB_APPS_INSTALL_DIALOG_CANCEL_BUTTON 11014
#define IDS_WEB_APP_UNINSTALL_BUTTON_FRAME_TOOLTIP 11015
#define IDS_WEB_APP_UNINSTALL_BUTTON_FRAME 11016
#define IDS_WEB_APP_MENU_BUTTON_TOOLTIP 11017
#define IDS_WEB_APP_MENU_BUTTON_TOOLTIP_UPDATE_AVAILABLE 11018
#define IDS_WEB_APP_MENU_BUTTON_UPDATE 11019
#define IDS_WEB_APP_BLOCKED_MIGRATION_MESSAGE 11020
#define IDS_WEB_APP_ENABLE_WINDOW_CONTROLS_OVERLAY_TOOLTIP 11021
#define IDS_WEB_APP_DISABLE_WINDOW_CONTROLS_OVERLAY_TOOLTIP 11022
#define IDS_WEB_APP_WINDOW_CONTROLS_OVERLAY_ENABLED_ALERT 11023
#define IDS_WEB_APP_WINDOW_CONTROLS_OVERLAY_DISABLED_ALERT 11024
#define IDS_APP_HOME_TITLE 11025
#define IDS_APP_HOME_OPEN_IN_WINDOW 11026
#define IDS_APP_HOME_LAUNCH_AT_STARTUP 11027
#define IDS_APP_HOME_CREATE_SHORTCUT 11028
#define IDS_APP_HOME_INSTALL_LOCALLY 11029
#define IDS_APP_HOME_UNINSTALL_APP 11030
#define IDS_APP_HOME_REMOVE_APP 11031
#define IDS_APP_HOME_APP_SETTINGS 11032
#define IDS_ACCNAME_APP_HOME_NOT_INSTALLED 11033
#define IDS_APP_HOME_APP_NO_APPS 11034
#define IDS_APP_HOME_APP_LEARN_INSTALL 11035
#define IDS_WEB_APP_SETTINGS_TITLE 11036
#define IDS_WEB_APP_SETTINGS_LINK 11037
#define IDS_WEB_APP_SETTINGS_LINK_TOOLTIP 11038
#define IDS_COMPONENTS_TITLE 11039
#define IDS_COMPONENTS_VERSION 11040
#define IDS_COMPONENTS_NONE_INSTALLED 11041
#define IDS_COMPONENTS_NO_COMPONENTS 11042
#define IDS_COMPONENTS_CHECK_FOR_UPDATE 11043
#define IDS_COMPONENTS_STATUS_LABEL 11044
#define IDS_COMPONENTS_CHECKING_LABEL 11045
#define IDS_COMPONENTS_SVC_STATUS_NEW 11046
#define IDS_COMPONENTS_SVC_STATUS_CHECKING 11047
#define IDS_COMPONENTS_SVC_STATUS_UPDATE 11048
#define IDS_COMPONENTS_SVC_STATUS_DNL 11049
#define IDS_COMPONENTS_SVC_STATUS_DECOMPRESSING 11050
#define IDS_COMPONENTS_SVC_STATUS_PATCHING 11051
#define IDS_COMPONENTS_SVC_STATUS_UPDATING 11052
#define IDS_COMPONENTS_SVC_STATUS_UPDATED 11053
#define IDS_COMPONENTS_SVC_STATUS_UPTODATE 11054
#define IDS_COMPONENTS_SVC_STATUS_UPDATE_ERROR 11055
#define IDS_COMPONENTS_UNKNOWN 11056
#define IDS_PASSWORD_MANAGER_ACCOUNT_CHOOSER_TITLE 11057
#define IDS_PASSWORD_GENERATION_SUGGESTION_GPM 11058
#define IDS_PASSWORD_GENERATION_SUGGESTION_GPM_WITHOUT_STRONG 11059
#define IDS_PASSWORD_GENERATION_NUDGE_CANCEL_BUTTON 11060
#define IDS_PASSWORD_MANAGER_CONFIRM_SAVED_TITLE 11061
#define IDS_PASSWORD_MANAGER_CONFIRM_UPDATE_TITLE 11062
#define IDS_PASSWORD_GENERATION_EDITING_SUGGESTION 11063
#define IDS_SAVE_PASSWORD 11064
#define IDS_SAVE_ACCOUNT 11065
#define IDS_ADD_USERNAME_TITLE 11066
#define IDS_ADD_USERNAME 11067
#define IDS_ADD_USERNAME_BODY 11068
#define IDS_UPDATE_PASSWORD 11069
#define IDS_VERIFY_IT_IS_YOU 11070
#define IDS_PASSWORD_ERROR_DESCRIPTION_SIGN_IN 11071
#define IDS_PASSWORD_ERROR_DESCRIPTION_SIGN_UP 11072
#define IDS_PASSWORD_ERROR_VERIFY_BUTTON_TITLE 11073
#define IDS_SAVE_PASSWORD_DIFFERENT_DOMAINS_TITLE 11075
#define IDS_UPDATE_PASSWORD_DIFFERENT_DOMAINS_TITLE 11076
#define IDS_SAVE_PASSWORD_FOOTER 11077
#define IDS_PASSWORD_BUBBLES_PASSWORD_MANAGER_LINK_TEXT_SYNCED_TO_ACCOUNT 11078
#define IDS_PASSWORD_BUBBLES_FOOTER_SYNCED_TO_ACCOUNT 11079
#define IDS_PASSWORD_BUBBLES_FOOTER_SAVING_ON_DEVICE 11080
#define IDS_PASSWORD_GENERATION_PROMPT_GOOGLE_PASSWORD_MANAGER 11081
#define IDS_PASSWORD_GENERATION_NUDGE_TITLE 11082
#define IDS_PASSWORD_MANAGEMENT_BUBBLE_FOOTER_ACCOUNT_STORE_USERS 11083
#define IDS_PASSWORD_MANAGER_GENERATED_PASSWORD_SAVED_MESSAGE_DESCRIPTION 11084
#define IDS_PASSWORD_MANAGER_SAVE_PASSWORD_SIGNED_IN_MESSAGE_DESCRIPTION 11085
#define IDS_PASSWORD_MANAGER_UPDATE_PASSWORD_SIGNED_IN_MESSAGE_DESCRIPTION 11086
#define IDS_PASSWORD_CROSS_DOMAIN_FILLING_CONFIRMATION_TITLE 11087
#define IDS_PASSWORD_CROSS_DOMAIN_FILLING_CONFIRMATION_DESCRIPTION 11088
#define IDS_PASSWORD_CROSS_DOMAIN_FILLING_CONFIRMATION_CONFIRM_BUTTON_LABEL 11089
#define IDS_PASSWORD_CROSS_DOMAIN_FILLING_WARNING_DESCRIPTION 11090
#define IDS_PASSWORD_MANAGER_ACCOUNT_CHOOSER_SIGN_IN 11091
#define IDS_PASSWORD_MANAGER_MANAGE_PASSWORDS_BUTTON 11092
#define IDS_PASSWORD_MANAGER_MANAGE_PASSWORD_BUTTON 11093
#define IDS_SETTINGS_PASSWORDS_ANDROID_APP 11094
#define IDS_SHARED_PASSWORDS_NOTIFICATION_GOT_IT_BUTTON 11095
#define IDS_SHARED_PASSWORDS_NOTIFICATION_TEXT_SINGLE_PASSWORD 11096
#define IDS_SHARED_PASSWORDS_NOTIFICATION_TEXT_MULTIPLE_PASSWORD 11097
#define IDS_SHARED_PASSWORDS_NOTIFICATION_TITLE 11098
#define IDS_PASSWORD_MANAGER_ONBOARDING_TITLE_A 11099
#define IDS_PASSWORD_MANAGER_ONBOARDING_TITLE_B 11100
#define IDS_PASSWORD_MANAGER_ONBOARDING_TITLE_C 11101
#define IDS_PASSWORD_MANAGER_ONBOARDING_DETAILS_A 11102
#define IDS_PASSWORD_MANAGER_ONBOARDING_DETAILS_B 11103
#define IDS_PASSWORD_MANAGER_DICE_PROMO_SIGNIN_MESSAGE 11104
#define IDS_PASSWORD_MANAGER_DICE_PROMO_SYNC_MESSAGE 11105
#define IDS_WEBRTC_LOGS_TITLE 11106
#define IDS_WEBRTC_TEXT_LOGS_LOG_COUNT_BANNER_FORMAT 11107
#define IDS_WEBRTC_EVENT_LOGS_LOG_COUNT_BANNER_FORMAT 11108
#define IDS_WEBRTC_LOGS_LOG_HEADER_FORMAT 11109
#define IDS_WEBRTC_LOGS_LOG_LOCAL_FILE_LABEL_FORMAT 11110
#define IDS_WEBRTC_LOGS_NO_LOCAL_LOG_FILE_MESSAGE 11111
#define IDS_WEBRTC_LOGS_LOG_UPLOAD_TIME_FORMAT 11112
#define IDS_WEBRTC_LOGS_LOG_FAILED_UPLOAD_TIME_FORMAT 11113
#define IDS_WEBRTC_LOGS_LOG_REPORT_ID_FORMAT 11114
#define IDS_WEBRTC_LOGS_BUG_LINK_LABEL 11115
#define IDS_WEBRTC_LOGS_LOG_PENDING_MESSAGE 11116
#define IDS_WEBRTC_LOGS_LOG_ACTIVELY_UPLOADED_MESSAGE 11117
#define IDS_WEBRTC_LOGS_LOG_NOT_UPLOADED_MESSAGE 11118
#define IDS_WEBRTC_LOGS_EVENT_LOG_LOCAL_LOG_ID 11119
#define IDS_WEBRTC_LOGS_NO_TEXT_LOGS_MESSAGE 11120
#define IDS_WEBRTC_LOGS_NO_EVENT_LOGS_MESSAGE 11121
#define IDS_PLUGIN_HIDE 11122
#define IDS_PLUGIN_BLOCKED 11123
#define IDS_PLUGIN_BLOCKED_BY_POLICY 11124
#define IDS_PLUGIN_NOT_AUTHORIZED 11125
#define IDS_PLUGIN_DISABLED 11126
#define IDS_PLUGIN_DEPRECATED 11127
#define IDS_SESSION_CRASHED_BUBBLE_TITLE 11128
#define IDS_SESSION_CRASHED_BUBBLE_UMA_LINK_TEXT 11129
#define IDS_IDLE_BUBBLE_TITLE_CLEAR 11130
#define IDS_SETTINGS_STORAGE_PRESSURE_BUBBLE_VIEW_TITLE 11131
#define IDS_SETTINGS_STORAGE_PRESSURE_BUBBLE_VIEW_MESSAGE 11132
#define IDS_SETTINGS_STORAGE_PRESSURE_BUBBLE_VIEW_BUTTON_LABEL 11133
#define IDS_BAD_FLAGS_WARNING_MESSAGE 11134
#define IDS_BAD_FEATURES_WARNING_MESSAGE 11135
#define IDS_BAD_FLAGS_FROM_FILE_WARNING_MESSAGE 11136
#define IDS_BAD_ENVIRONMENT_VARIABLES_WARNING_MESSAGE 11137
#define IDS_BLOCKED_DISPLAYING_INSECURE_CONTENT_TITLE 11138
#define IDS_BLOCKED_DISPLAYING_INSECURE_CONTENT 11139
#define IDS_ALLOW_INSECURE_CONTENT_BUTTON 11140
#define IDS_ADD_TO_SHELF_INFOBAR_TITLE 11141
#define IDS_ADD_TO_SHELF_INFOBAR_ADD_BUTTON 11142
#define IDS_ABOUT_SYS_TITLE 11143
#define IDS_ABOUT_SYS_DESC 11144
#define IDS_ABOUT_SYS_TABLE_TITLE 11145
#define IDS_ABOUT_SYS_LOG_FILE_TABLE_TITLE 11146
#define IDS_ABOUT_SYS_EXPAND_ALL 11147
#define IDS_ABOUT_SYS_COLLAPSE_ALL 11148
#define IDS_ABOUT_SYS_EXPAND 11149
#define IDS_ABOUT_SYS_COLLAPSE 11150
#define IDS_ABOUT_SYS_PARSE_ERROR 11151
#define IDS_ABOUT_BROWSER_SWITCH_TITLE 11152
#define IDS_ABOUT_BROWSER_SWITCH_OPENING_TITLE_UNKNOWN_BROWSER 11153
#define IDS_ABOUT_BROWSER_SWITCH_OPENING_TITLE_KNOWN_BROWSER 11154
#define IDS_ABOUT_BROWSER_SWITCH_ERROR_TITLE_UNKNOWN_BROWSER 11155
#define IDS_ABOUT_BROWSER_SWITCH_ERROR_TITLE_KNOWN_BROWSER 11156
#define IDS_ABOUT_BROWSER_SWITCH_COUNTDOWN_TITLE_UNKNOWN_BROWSER 11157
#define IDS_ABOUT_BROWSER_SWITCH_COUNTDOWN_TITLE_KNOWN_BROWSER 11158
#define IDS_ABOUT_BROWSER_SWITCH_GENERIC_ERROR_UNKNOWN_BROWSER 11159
#define IDS_ABOUT_BROWSER_SWITCH_GENERIC_ERROR_KNOWN_BROWSER 11160
#define IDS_ABOUT_BROWSER_SWITCH_PROTOCOL_ERROR 11161
#define IDS_ABOUT_BROWSER_SWITCH_INTERNALS_DESC 11162
#define IDS_ABOUT_BROWSER_SWITCH_INTERNALS_TITLE 11163
#define IDS_ABOUT_BROWSER_SWITCH_INTERNALS_NOTHING_SHOWN 11164
#define IDS_ABOUT_BROWSER_SWITCH_INTERNALS_LBS_DISABLED 11165
#define IDS_ABOUT_BROWSER_SWITCH_INTERNALS_URL_CHECKER_TITLE 11166
#define IDS_ABOUT_BROWSER_SWITCH_INTERNALS_URL_CHECKER_DESC 11167
#define IDS_ABOUT_BROWSER_SWITCH_INTERNALS_OPEN_BROWSER 11168
#define IDS_ABOUT_BROWSER_SWITCH_INTERNALS_OPEN_BROWSER_PROTOCOL_REASON 11169
#define IDS_ABOUT_BROWSER_SWITCH_INTERNALS_OPEN_BROWSER_RULE_REASON 11170
#define IDS_ABOUT_BROWSER_SWITCH_INTERNALS_OPEN_BROWSER_INVERT_RULE_REASON 11171
#define IDS_ABOUT_BROWSER_SWITCH_INTERNALS_OPEN_BROWSER_DEFAULT_REASON 11172
#define IDS_ABOUT_BROWSER_SWITCH_INTERNALS_INVALID_URL 11173
#define IDS_ABOUT_BROWSER_SWITCH_INTERNALS_XML_TITLE 11174
#define IDS_ABOUT_BROWSER_SWITCH_INTERNALS_XML_DESC 11175
#define IDS_ABOUT_BROWSER_SWITCH_INTERNALS_XML_SOURCE 11176
#define IDS_ABOUT_BROWSER_SWITCH_INTERNALS_XML_NOT_CONFIGURED 11177
#define IDS_ABOUT_BROWSER_SWITCH_INTERNALS_XML_SITELIST_NOT_FETCHED 11178
#define IDS_ABOUT_BROWSER_SWITCH_INTERNALS_XML_SITELIST_DOWNLOAD_BUTTON 11179
#define IDS_ABOUT_BROWSER_SWITCH_INTERNALS_XML_SITELIST_LAST_DOWNLOAD_DATE 11180
#define IDS_ABOUT_BROWSER_SWITCH_INTERNALS_XML_SITELIST_NEXT_DOWNLOAD_DATE 11181
#define IDS_ABOUT_BROWSER_SWITCH_INTERNALS_FORCE_OPEN_IN_TITLE 11182
#define IDS_ABOUT_BROWSER_SWITCH_INTERNALS_FORCE_OPEN_IN_DESCRIPTION 11183
#define IDS_ABOUT_BROWSER_SWITCH_INTERNALS_FORCE_OPEN_IN_FIRST_PARAGRAPH 11184
#define IDS_ABOUT_BROWSER_SWITCH_INTERNALS_FORCE_OPEN_IN_SECOND_PARAGRAPH 11185
#define IDS_ABOUT_BROWSER_SWITCH_INTERNALS_FORCE_OPEN_TABLE_COLUMN_RULE 11186
#define IDS_ABOUT_BROWSER_SWITCH_INTERNALS_FORCE_OPEN_TABLE_COLUMN_OPENS_IN 11187
#define IDS_ABOUT_BROWSER_SWITCH_INTERNALS_FORCE_OPEN_TABLE_COLUMN_SOURCE 11188
#define IDS_ABOUT_BROWSER_SWITCH_INTERNALS_IGNORE_TITLE 11189
#define IDS_ABOUT_BROWSER_SWITCH_INTERNALS_IGNORE_DESCRIPTION 11190
#define IDS_ABOUT_BROWSER_SWITCH_INTERNALS_IGNORE_FIRST_PARAGRAPH 11191
#define IDS_ABOUT_BROWSER_SWITCH_INTERNALS_IGNORE_SECOND_PARAGRAPH 11192
#define IDS_ABOUT_BROWSER_SWITCH_INTERNALS_IGNORE_TABLE_COLUMN_RULE 11193
#define IDS_ABOUT_BROWSER_SWITCH_INTERNALS_IGNORE_TABLE_COLUMN_SOURCE 11194
#define IDS_ABOUT_BROWSER_SWITCH_INTERNALS_POLICIES_DOWNLOAD_BUTTON 11195
#define IDS_OMNIBOX_PLACEHOLDER_TEXT 11200
#define IDS_PASTE_AND_GO 11201
#define IDS_PASTE_AND_SEARCH 11202
#define IDS_PASTE_AND_GO_EMPTY 11203
#define IDS_OMNIBOX_KEYWORD_TEXT 11204
#define IDS_OMNIBOX_CLEAR_ALL 11205
#define IDS_OMNIBOX_WHY_THIS_SUGGESTION 11206
#define IDS_OMNIBOX_CLOSE_IPH_SUGGESTION 11207
#define IDS_OMNIBOX_THUMBS_UP_SUGGESTION 11208
#define IDS_OMNIBOX_THUMBS_DOWN_SUGGESTION 11209
#define IDS_OMNIBOX_REMOVE_SUGGESTION 11210
#define IDS_OMNIBOX_REMOVE_SUGGESTION_BUBBLE_TITLE 11211
#define IDS_OMNIBOX_REMOVE_SUGGESTION_BUBBLE_DESCRIPTION 11212
#define IDS_CONTEXT_MENU_SHOW_FULL_URLS 11213
#define IDS_CONTEXT_MENU_SHOW_SEARCH_TOOLS 11214
#define IDS_WEBUI_OMNIBOX_PLACEHOLDER_TEXT 11217
#define IDS_WEBUI_OMNIBOX_COMPOSE_ASK_ABOUT_THIS_PAGE 11218
#define IDS_WEBUI_OMNIBOX_COMPOSE_ASK_ABOUT_THIS_PAGE_ARIA_LABEL 11219
#define IDS_WEBUI_OMNIBOX_COMPOSE_ASK_ABOUT_YOUR_TABS 11220
#define IDS_NTP_DRIVE_CONNECT_TITLE 11221
#define IDS_NTP_DRIVE_DISCLAIMER_BODY 11222
#define IDS_NTP_DRIVE_DISCLAIMER_CANCEL 11223
#define IDS_NTP_DRIVE_DISCLAIMER_CONNECT 11224
#define IDS_GOOGLE_SEARCH_BOX_EMPTY_HINT_MD 11225
#define IDS_GOOGLE_SEARCH_BOX_EMPTY_HINT_SHORT 11226
#define IDS_GOOGLE_SEARCH_BOX_EMPTY_HINT_MULTIMODAL 11227
#define IDS_GOOGLE_SEARCH_BOX_EMPTY_HINT_CONTEXTUAL 11228
#define IDS_GOOGLE_SEARCH_BOX_EMPTY_HINT_CONTEXTUAL_PDF 11229
#define IDS_GOOGLE_SEARCH_BOX_CONTEXTUAL_LOADING_HINT_PRIMARY 11230
#define IDS_GOOGLE_SEARCH_BOX_CONTEXTUAL_LOADING_HINT_PRIMARY_PDF 11231
#define IDS_GOOGLE_SEARCH_BOX_CONTEXTUAL_LOADING_HINT_SECONDARY 11232
#define IDS_GOOGLE_SEARCH_BOX_CONTEXTUAL_ERROR_TEXT 11233
#define IDS_GOOGLE_SEARCH_BOX_CONTEXTUAL_LOADING_HINT_PRIMARY_ALT 11234
#define IDS_GOOGLE_SEARCH_BOX_CONTEXTUAL_NO_SUGGEST_TEXT 11235
#define IDS_GOOGLE_SEARCH_BOX_MULTIMODAL_IMAGE_THUMBNAIL 11236
#define IDS_GOOGLE_LENS_SEARCH_BOX_EMPTY_HINT 11237
#define IDS_NTP_SEARCH_BOX_PLACEHOLDER_ASK_OR_TYPE 11238
#define IDS_NTP_SEARCH_BOX_DYNAMIC_PLACEHOLDER_ASK_GOOGLE 11239
#define IDS_NTP_SEARCH_BOX_DYNAMIC_PLACEHOLDER_PLAN 11240
#define IDS_NTP_SEARCH_BOX_DYNAMIC_PLACEHOLDER_COMPARE 11241
#define IDS_NTP_SEARCH_BOX_DYNAMIC_PLACEHOLDER_RESEARCH 11242
#define IDS_NTP_SEARCH_BOX_DYNAMIC_PLACEHOLDER_TEACH 11243
#define IDS_NTP_SEARCH_BOX_DYNAMIC_PLACEHOLDER_WRITE 11244
#define IDS_NTP_SEARCH_BOX_DYNAMIC_PLACEHOLDER_TAB 11245
#define IDS_NTP_SEARCH_BOX_DYNAMIC_PLACEHOLDER_IMAGE 11246
#define IDS_NTP_SEARCH_BOX_DYNAMIC_PLACEHOLDER_CANVAS 11247
#define IDS_NTP_CUSTOM_LINKS_ADD_SHORTCUT_TOOLTIP 11248
#define IDS_NTP_CUSTOM_LINKS_ADD_SHORTCUT_TITLE 11249
#define IDS_NTP_CUSTOM_LINKS_EDIT_SHORTCUT_TOOLTIP 11250
#define IDS_NTP_CUSTOM_LINKS_EDIT_SHORTCUT 11251
#define IDS_NTP_CUSTOM_LINKS_SHORTCUT_DETAILS_TITLE 11252
#define IDS_NTP_CUSTOM_LINKS_MORE_ACTIONS 11253
#define IDS_NTP_ENTERPRISE_SHORTCUT_SUBTITLE 11254
#define IDS_UPLOAD_IMAGE_FORMAT 11255
#define IDS_NTP_CUSTOM_LINKS_NAME 11256
#define IDS_NTP_CUSTOM_LINKS_URL 11257
#define IDS_NTP_CUSTOM_LINKS_REMOVE 11258
#define IDS_NTP_MOST_VISITED_SITES_REMOVE 11259
#define IDS_NTP_CUSTOM_LINKS_CANCEL 11260
#define IDS_NTP_CUSTOM_LINKS_DONE 11261
#define IDS_NTP_CUSTOM_LINKS_DETAILS 11262
#define IDS_NTP_CUSTOM_LINKS_INVALID_URL 11263
#define IDS_NTP_CUSTOM_LINKS_CANT_CREATE 11264
#define IDS_NTP_CUSTOM_LINKS_CANT_EDIT 11265
#define IDS_NTP_CUSTOM_LINKS_CANT_REMOVE 11266
#define IDS_NTP_CUSTOM_LINKS_HELP_PIN 11267
#define IDS_NTP_CONFIRM_MSG_SHORTCUT_REMOVED 11268
#define IDS_NTP_CONFIRM_MSG_SHORTCUT_EDITED 11269
#define IDS_NTP_CONFIRM_MSG_SHORTCUT_ADDED 11270
#define IDS_NTP_CUSTOM_LINKS_ALREADY_EXISTS 11271
#define IDS_NTP_CONFIRM_MSG_RESTORE_DEFAULTS 11272
#define IDS_NTP_CONFIRM_MSG_RESTORE_ENTERPRISE_DEFAULTS 11273
#define IDS_NTP_CUSTOM_BG_UPLOAD_AN_IMAGE 11274
#define IDS_NTP_CUSTOM_BG_SELECT_A_COLLECTION 11275
#define IDS_NTP_CUSTOM_BG_DAILY_REFRESH 11276
#define IDS_NTP_CUSTOM_BG_RESTORE_DEFAULT 11277
#define IDS_NTP_CUSTOM_BG_CANCEL 11278
#define IDS_NTP_CONNECTION_ERROR_NO_PERIOD 11279
#define IDS_NTP_CONNECTION_ERROR 11280
#define IDS_NTP_ERROR_MORE_INFO 11281
#define IDS_NTP_CUSTOM_BG_BACKGROUNDS_UNAVAILABLE 11282
#define IDS_NTP_CUSTOM_BG_BACK_LABEL 11283
#define IDS_NTP_CUSTOM_BG_IMAGE_SELECTED 11284
#define IDS_NTP_CUSTOM_BG_CUSTOMIZE_NTP_LABEL 11285
#define IDS_NTP_CUSTOM_BG_CUSTOMIZE_NTP_WALLPAPER_SEARCH_LABEL 11286
#define IDS_NTP_CLOSE 11287
#define IDS_NTP_DISMISS 11288
#define IDS_NTP_DISMISS_PROMO 11289
#define IDS_NTP_DOODLE_SHARE_LABEL 11290
#define IDS_NTP_DOODLE_SHARE_DIALOG_CLOSE_LABEL 11291
#define IDS_NTP_DOODLE_SHARE_DIALOG_FACEBOOK_LABEL 11292
#define IDS_NTP_DOODLE_SHARE_DIALOG_TWITTER_LABEL 11293
#define IDS_NTP_DOODLE_SHARE_DIALOG_MAIL_LABEL 11294
#define IDS_NTP_DOODLE_SHARE_DIALOG_COPY_LABEL 11295
#define IDS_NTP_DOODLE_SHARE_DIALOG_LINK_LABEL 11296
#define IDS_NTP_CUSTOMIZE_APPEARANCE_LABEL 11297
#define IDS_NTP_CUSTOMIZE_THEMES_HEADER 11298
#define IDS_NTP_CUSTOMIZE_MENU_SHORTCUTS_LABEL 11299
#define IDS_NTP_CUSTOMIZE_MENU_TOOLBAR_LABEL 11300
#define IDS_NTP_CUSTOMIZE_MENU_EXTENSIONS_LABEL 11301
#define IDS_NTP_CUSTOMIZE_MENU_MODULES_LABEL 11302
#define IDS_NTP_CUSTOMIZE_NO_BACKGROUND_LABEL 11303
#define IDS_NTP_CUSTOMIZE_CHROME_CHANGE_THEME_LABEL 11304
#define IDS_NTP_CUSTOMIZE_FOOTER_HEADER 11305
#define IDS_NTP_CUSTOMIZE_TOOLS_HEADER 11306
#define IDS_NTP_MANAGED_BY_SEARCH_ENGINE 11307
#define IDS_NTP_MANAGED_BY_EXTENSION 11308
#define IDS_NTP_CUSTOMIZE_CHROME_MANAGED_NEW_TAB_PAGE_ACCESSIBILITY 11309
#define IDS_NTP_CUSTOMIZE_CHROME_RESET_TO_CLASSIC_CHROME_COMPLETE 11310
#define IDS_NTP_CUSTOMIZE_CHROME_RESET_TO_UPLOADED_IMAGE_COMPLETE 11311
#define IDS_NTP_CUSTOMIZE_CHROME_FOLLOW_THEME_LABEL 11312
#define IDS_NTP_CUSTOMIZE_UPLOADED_IMAGE_LABEL 11313
#define IDS_NTP_CUSTOMIZE_YOUR_UPLOADED_IMAGE_LABEL 11314
#define IDS_NTP_CUSTOMIZE_YOUR_SEARCHED_IMAGE_LABEL 11315
#define IDS_NTP_CUSTOMIZE_CHROME_CURRENT_THEME_LABEL 11316
#define IDS_NTP_CUSTOMIZE_SHOW_CARDS_LABEL 11317
#define IDS_NTP_CUSTOMIZE_SHOW_FOOTER_LABEL 11318
#define IDS_NTP_CUSTOMIZE_SHOW_CHIPS_LABEL 11319
#define IDS_NTP_CUSTOMIZE_SHOW_SUGGESTIONS_CHIPS_LABEL 11320
#define IDS_NTP_CUSTOMIZE_SHOW_SHORTCUTS_LABEL 11321
#define IDS_NTP_CUSTOMIZE_SHOW_PERSONAL_SHORTCUTS_LABEL 11322
#define IDS_NTP_CUSTOMIZE_SHOW_PERSONAL_SHORTCUTS_DESC 11323
#define IDS_NTP_CUSTOMIZE_MY_SHORTCUTS_LABEL 11324
#define IDS_NTP_CUSTOMIZE_MOST_VISITED_LABEL 11325
#define IDS_NTP_CUSTOMIZE_ENTERPRISE_SHORTCUTS_LABEL 11326
#define IDS_NTP_CUSTOMIZE_MOST_VISITED_DESC 11327
#define IDS_NTP_CUSTOMIZE_MY_SHORTCUTS_DESC 11328
#define IDS_NTP_CUSTOMIZE_ENTERPRISE_SHORTCUTS_DESC 11329
#define IDS_NTP_CUSTOMIZE_COLOR_PICKER_LABEL 11330
#define IDS_NTP_CUSTOMIZE_COLOR_HUE_SLIDER_TITLE 11331
#define IDS_NTP_CUSTOMIZE_COLOR_HUE_SLIDER_ARIA_LABEL 11332
#define IDS_NTP_CUSTOMIZE_COLOR_HUE_SLIDER_DELETE_TITLE 11333
#define IDS_NTP_CUSTOMIZE_COLOR_HUE_SLIDER_DELETE_A11Y_LABEL 11334
#define IDS_NTP_THEME_MANAGED_DIALOG_TITLE 11335
#define IDS_NTP_THEME_MANAGED_DIALOG_BODY 11336
#define IDS_NTP_THEMES_CONTAINER_LABEL 11337
#define IDS_NTP_CUSTOMIZE_DEFAULT_LABEL 11338
#define IDS_NTP_CUSTOMIZE_GREY_DEFAULT_LABEL 11339
#define IDS_NTP_COLORS_WARM_GREY 11340
#define IDS_NTP_COLORS_COOL_GREY 11341
#define IDS_NTP_COLORS_MIDNIGHT_BLUE 11342
#define IDS_NTP_COLORS_BLACK 11343
#define IDS_NTP_COLORS_BEIGE_AND_WHITE 11344
#define IDS_NTP_COLORS_YELLOW_AND_WHITE 11345
#define IDS_NTP_COLORS_GREEN_AND_WHITE 11346
#define IDS_NTP_COLORS_LIGHT_TEAL_AND_WHITE 11347
#define IDS_NTP_COLORS_LIGHT_PURPLE_AND_WHITE 11348
#define IDS_NTP_COLORS_PINK_AND_WHITE 11349
#define IDS_NTP_COLORS_BEIGE 11350
#define IDS_NTP_COLORS_ORANGE 11351
#define IDS_NTP_COLORS_LIGHT_GREEN 11352
#define IDS_NTP_COLORS_LIGHT_TEAL 11353
#define IDS_NTP_COLORS_LIGHT_BLUE 11354
#define IDS_NTP_COLORS_PINK 11355
#define IDS_NTP_COLORS_DARK_PINK_AND_RED 11356
#define IDS_NTP_COLORS_DARK_RED_AND_ORANGE 11357
#define IDS_NTP_COLORS_DARK_GREEN 11358
#define IDS_NTP_COLORS_DARK_TEAL 11359
#define IDS_NTP_COLORS_DARK_BLUE 11360
#define IDS_NTP_COLORS_DARK_PURPLE 11361
#define IDS_NTP_COLORS_BLUE 11362
#define IDS_NTP_COLORS_GREY 11363
#define IDS_NTP_COLORS_AQUA 11364
#define IDS_NTP_COLORS_GREEN 11365
#define IDS_NTP_COLORS_VIRIDIAN 11366
#define IDS_NTP_COLORS_CITRON 11367
#define IDS_NTP_COLORS_APRICOT 11368
#define IDS_NTP_COLORS_ROSE 11369
#define IDS_NTP_COLORS_FUCHSIA 11370
#define IDS_NTP_COLORS_VIOLET 11371
#define IDS_NTP_COLORS_BATCH_UPLOAD_DESCRIPTION 11372
#define IDS_NTP_UNDO_DISMISS_PROMO_BUTTON_TOAST 11373
#define IDS_NTP_CUSTOMIZE_CHROME_COLOR_SCHEME_MODE_GROUP_LABEL 11374
#define IDS_NTP_CUSTOMIZE_CHROME_COLOR_SCHEME_MODE_LIGHT_LABEL 11375
#define IDS_NTP_CUSTOMIZE_CHROME_COLOR_SCHEME_MODE_DARK_LABEL 11376
#define IDS_NTP_CUSTOMIZE_CHROME_COLOR_SCHEME_MODE_SYSTEM_LABEL 11377
#define IDS_NTP_CUSTOMIZE_CHROME_WEBSTORE_LABEL 11378
#define IDS_NTP_SHOW_MORE_BUTTON_LABEL 11379
#define IDS_NTP_SHOW_LESS_BUTTON_LABEL 11380
#define IDS_NTP_THREADS_HISTORY_LABEL 11381
#define IDS_NTP_THREADS_NEW_SEARCH_LABEL 11382
#define IDS_NTP_WEBSTORE_SHOPPING_CATEOGRY_LABEL 11383
#define IDS_NTP_WEBSTORE_WRITTING_HELP_COLLECTION_LABEL 11384
#define IDS_NTP_WEBSTORE_PRODUCTIVITY_CATEOGRY_LABEL 11385
#define IDS_NTP_WEBSTORE_TOAST_MESSAGE 11386
#define IDS_NTP_WEBSTORE_TOAST_BUTTON_TEXT 11387
#define IDS_NTP_WALLPAPER_SEARCH_FEEDBACK_PLACEHOLDER 11388
#define IDS_NTP_WALLPAPER_SEARCH_GENERIC_ERROR_DESCRIPTION 11389
#define IDS_NTP_WALLPAPER_SEARCH_GENERIC_ERROR_DESCRIPTION_WITH_HISTORY 11390
#define IDS_NTP_WALLPAPER_SEARCH_GENERIC_ERROR_DESCRIPTION_WITH_INSPIRATION 11391
#define IDS_NTP_WALLPAPER_SEARCH_GENERIC_ERROR_DESCRIPTION_WITH_HISTORY_AND_INSPIRATION 11392
#define IDS_NTP_WALLPAPER_SEARCH_GENERIC_ERROR_TITLE 11393
#define IDS_NTP_WALLPAPER_SEARCH_TILE_LABEL 11394
#define IDS_NTP_WALLPAPER_SEARCH_PAGE_HEADER 11395
#define IDS_NTP_WALLPAPER_SEARCH_HISTORY_HEADER 11396
#define IDS_NTP_WALLPAPER_SEARCH_OFFLINE_DESCRIPTION 11397
#define IDS_NTP_WALLPAPER_SEARCH_OFFLINE_DESCRIPTION_WITH_HISTORY 11398
#define IDS_NTP_WALLPAPER_SEARCH_OFFLINE_TITLE 11399
#define IDS_NTP_WALLPAPER_SEARCH_OPTIONAL_DETAILS_LABEL 11400
#define IDS_NTP_WALLPAPER_SEARCH_COLOR_RED_LABEL 11401
#define IDS_NTP_WALLPAPER_SEARCH_COLOR_BLUE_LABEL 11402
#define IDS_NTP_WALLPAPER_SEARCH_COLOR_YELLOW_LABEL 11403
#define IDS_NTP_WALLPAPER_SEARCH_COLOR_GREEN_LABEL 11404
#define IDS_NTP_WALLPAPER_SEARCH_COLOR_BLACK_LABEL 11405
#define IDS_NTP_WALLPAPER_SEARCH_REQUEST_THROTTLED_DESCRIPTION 11406
#define IDS_NTP_WALLPAPER_SEARCH_REQUEST_THROTTLED_TITLE 11407
#define IDS_NTP_WALLPAPER_SEARCH_SUBMIT_BTN_TEXT 11408
#define IDS_NTP_WALLPAPER_SEARCH_TRY_AGAIN_CTA 11409
#define IDS_NTP_WALLPAPER_SEARCH_HISTORY_RESULT_LABEL_NO_DESCRIPTOR 11410
#define IDS_NTP_WALLPAPER_SEARCH_HISTORY_RESULT_LABEL 11411
#define IDS_NTP_WALLPAPER_SEARCH_HISTORY_RESULT_LABEL_WITH_DESCRIPTOR_B 11412
#define IDS_NTP_WALLPAPER_SEARCH_HISTORY_RESULT_LABEL_WITH_DESCRIPTOR_C 11413
#define IDS_NTP_WALLPAPER_SEARCH_HISTORY_RESULT_LABEL_WITH_DESCRIPTORS_B_AND_C 11414
#define IDS_NTP_WALLPAPER_SEARCH_RESULT_LABEL 11415
#define IDS_NTP_WALLPAPER_SEARCH_RESULT_LABEL_WITH_DESCRIPTOR_B 11416
#define IDS_NTP_WALLPAPER_SEARCH_RESULT_LABEL_WITH_DESCRIPTOR_C 11417
#define IDS_NTP_WALLPAPER_SEARCH_RESULT_LABEL_WITH_DESCRIPTORS_B_AND_C 11418
#define IDS_NTP_WALLPAPER_SEARCH_SUBJECT_LABEL 11419
#define IDS_NTP_WALLPAPER_SEARCH_SUBJECT_DEFAULT_OPTION_LABEL 11420
#define IDS_NTP_WALLPAPER_SEARCH_MOOD_LABEL 11421
#define IDS_NTP_WALLPAPER_SEARCH_MOOD_DEFAULT_OPTION_LABEL 11422
#define IDS_NTP_WALLPAPER_SEARCH_STYLE_LABEL 11423
#define IDS_NTP_WALLPAPER_SEARCH_STYLE_DEFAULT_OPTION_LABEL 11424
#define IDS_LEARN_MORE_ABOUT_FEATURE_A11Y_LABEL 11425
#define IDS_THUMBS_DOWN_OPENS_FEEDBACK_FORM_A11Y_LABEL 11426
#define IDS_THUMBS_UP_RESULTS_A11Y_LABEL 11427
#define IDS_NTP_WALLPAPER_SEARCH_INSPIRATION_HEADER 11428
#define IDS_NTP_WALLPAPER_SEARCH_LOADING_A11Y_MESSAGE 11429
#define IDS_NTP_WALLPAPER_SEARCH_SUCCESS_A11Y_MESSAGE 11430
#define IDS_NTP_WALLPAPER_SEARCH_DISCLAIMER 11431
#define IDS_NTP_WALLPAPER_SEARCH_INSPIRATION_CARD_TOGGLE_TITLE 11432
#define IDS_NTP_WALLPAPER_SEARCH_DESCRIPTORS_CHANGED_A11Y_MESSAGE 11433
#define IDS_NTP_WALLPAPER_SEARCH_SIGNED_OUT_DESCRIPTION 11434
#define IDS_NTP_WALLPAPER_SEARCH_SIGNED_OUT_TITLE 11435
#define IDS_NTP_WALLPAPER_SEARCH_CATEGORY_DESSERTS 11436
#define IDS_NTP_WALLPAPER_SEARCH_CATEGORY_INTERIORS 11437
#define IDS_NTP_WALLPAPER_SEARCH_CATEGORY_LANDSCAPE 11438
#define IDS_NTP_WALLPAPER_SEARCH_CATEGORY_NATURE 11439
#define IDS_NTP_WALLPAPER_SEARCH_CATEGORY_OUTER_SPACE 11440
#define IDS_NTP_WALLPAPER_SEARCH_CATEGORY_PLACES 11441
#define IDS_NTP_WALLPAPER_SEARCH_CATEGORY_STRUCTURES 11442
#define IDS_NTP_WALLPAPER_SEARCH_SUBJECT_CAKE_POPS 11443
#define IDS_NTP_WALLPAPER_SEARCH_SUBJECT_CHEESECAKE 11444
#define IDS_NTP_WALLPAPER_SEARCH_SUBJECT_FRUIT_TART 11445
#define IDS_NTP_WALLPAPER_SEARCH_SUBJECT_ICE_CREAM 11446
#define IDS_NTP_WALLPAPER_SEARCH_SUBJECT_MACARON 11447
#define IDS_NTP_WALLPAPER_SEARCH_SUBJECT_PANNA_COTTA 11448
#define IDS_NTP_WALLPAPER_SEARCH_SUBJECT_CHEMISTRY_LAB 11449
#define IDS_NTP_WALLPAPER_SEARCH_SUBJECT_HALLWAY 11450
#define IDS_NTP_WALLPAPER_SEARCH_SUBJECT_KITCHEN 11451
#define IDS_NTP_WALLPAPER_SEARCH_SUBJECT_LIVING_ROOM 11452
#define IDS_NTP_WALLPAPER_SEARCH_SUBJECT_READING_NOOK 11453
#define IDS_NTP_WALLPAPER_SEARCH_SUBJECT_DESERT 11454
#define IDS_NTP_WALLPAPER_SEARCH_SUBJECT_FOREST 11455
#define IDS_NTP_WALLPAPER_SEARCH_SUBJECT_HIGHLANDS 11456
#define IDS_NTP_WALLPAPER_SEARCH_SUBJECT_MEADOW 11457
#define IDS_NTP_WALLPAPER_SEARCH_SUBJECT_MOUNTAIN 11458
#define IDS_NTP_WALLPAPER_SEARCH_SUBJECT_OCEAN 11459
#define IDS_NTP_WALLPAPER_SEARCH_SUBJECT_RAINFOREST 11460
#define IDS_NTP_WALLPAPER_SEARCH_SUBJECT_RICE_PADDY 11461
#define IDS_NTP_WALLPAPER_SEARCH_SUBJECT_SALT_FLAT 11462
#define IDS_NTP_WALLPAPER_SEARCH_SUBJECT_SHORELINE 11463
#define IDS_NTP_WALLPAPER_SEARCH_SUBJECT_TUNDRA 11464
#define IDS_NTP_WALLPAPER_SEARCH_SUBJECT_WATERFALL 11465
#define IDS_NTP_WALLPAPER_SEARCH_SUBJECT_AURORA_BOREALIS 11466
#define IDS_NTP_WALLPAPER_SEARCH_SUBJECT_CLIFF 11467
#define IDS_NTP_WALLPAPER_SEARCH_SUBJECT_CRASHING_WAVES 11468
#define IDS_NTP_WALLPAPER_SEARCH_SUBJECT_CREEK 11469
#define IDS_NTP_WALLPAPER_SEARCH_SUBJECT_ECLIPSES 11470
#define IDS_NTP_WALLPAPER_SEARCH_SUBJECT_FLOWERS 11471
#define IDS_NTP_WALLPAPER_SEARCH_SUBJECT_LIGHTNING 11472
#define IDS_NTP_WALLPAPER_SEARCH_SUBJECT_NIGHT_SKY 11473
#define IDS_NTP_WALLPAPER_SEARCH_SUBJECT_PALM_TREE 11474
#define IDS_NTP_WALLPAPER_SEARCH_SUBJECT_RAINBOW 11475
#define IDS_NTP_WALLPAPER_SEARCH_SUBJECT_RIVER 11476
#define IDS_NTP_WALLPAPER_SEARCH_SUBJECT_SLOT_CANYON 11477
#define IDS_NTP_WALLPAPER_SEARCH_SUBJECT_SUNSET 11478
#define IDS_NTP_WALLPAPER_SEARCH_SUBJECT_ASTEROID 11479
#define IDS_NTP_WALLPAPER_SEARCH_SUBJECT_COMET 11480
#define IDS_NTP_WALLPAPER_SEARCH_SUBJECT_MOON 11481
#define IDS_NTP_WALLPAPER_SEARCH_SUBJECT_NEBULA 11482
#define IDS_NTP_WALLPAPER_SEARCH_SUBJECT_PLANET 11483
#define IDS_NTP_WALLPAPER_SEARCH_SUBJECT_SPACE_STATION 11484
#define IDS_NTP_WALLPAPER_SEARCH_SUBJECT_STARS 11485
#define IDS_NTP_WALLPAPER_SEARCH_SUBJECT_CHICHEN_ITZA 11486
#define IDS_NTP_WALLPAPER_SEARCH_SUBJECT_GRAND_CANYON 11487
#define IDS_NTP_WALLPAPER_SEARCH_SUBJECT_GRAND_TETON_NATIONAL_PARK 11488
#define IDS_NTP_WALLPAPER_SEARCH_SUBJECT_GREAT_PYRAMID_OF_GIZA 11489
#define IDS_NTP_WALLPAPER_SEARCH_SUBJECT_ROCKY_MOUNTAINS_NATIONAL_PARK 11490
#define IDS_NTP_WALLPAPER_SEARCH_SUBJECT_THE_COLOSSEUM 11491
#define IDS_NTP_WALLPAPER_SEARCH_SUBJECT_ZION_NATIONAL_PARK 11492
#define IDS_NTP_WALLPAPER_SEARCH_SUBJECT_BRICK_BUILDING 11493
#define IDS_NTP_WALLPAPER_SEARCH_SUBJECT_CASTLE 11494
#define IDS_NTP_WALLPAPER_SEARCH_SUBJECT_HIGHWAY 11495
#define IDS_NTP_WALLPAPER_SEARCH_SUBJECT_HOUSE 11496
#define IDS_NTP_WALLPAPER_SEARCH_SUBJECT_LIGHTHOUSE 11497
#define IDS_NTP_WALLPAPER_SEARCH_SUBJECT_PIER 11498
#define IDS_NTP_WALLPAPER_SEARCH_STYLE_ANIMATED 11499
#define IDS_NTP_WALLPAPER_SEARCH_STYLE_COLORED_PENCIL 11500
#define IDS_NTP_WALLPAPER_SEARCH_STYLE_CYBERPUNK 11501
#define IDS_NTP_WALLPAPER_SEARCH_STYLE_DREAM 11502
#define IDS_NTP_WALLPAPER_SEARCH_STYLE_EXPRESSIONISM 11503
#define IDS_NTP_WALLPAPER_SEARCH_STYLE_FANTASY 11504
#define IDS_NTP_WALLPAPER_SEARCH_STYLE_IMPRESSIONISM 11505
#define IDS_NTP_WALLPAPER_SEARCH_STYLE_OIL_PAINTING 11506
#define IDS_NTP_WALLPAPER_SEARCH_STYLE_ORGANIC 11507
#define IDS_NTP_WALLPAPER_SEARCH_STYLE_PHOTOGRAPHY 11508
#define IDS_NTP_WALLPAPER_SEARCH_STYLE_SPLATTER 11509
#define IDS_NTP_WALLPAPER_SEARCH_STYLE_STEAMPUNK 11510
#define IDS_NTP_WALLPAPER_SEARCH_STYLE_WATERCOLOR 11511
#define IDS_NTP_WALLPAPER_SEARCH_MOOD_CHAOTIC 11512
#define IDS_NTP_WALLPAPER_SEARCH_MOOD_CREATIVE 11513
#define IDS_NTP_WALLPAPER_SEARCH_MOOD_DARK 11514
#define IDS_NTP_WALLPAPER_SEARCH_MOOD_DREAMY 11515
#define IDS_NTP_WALLPAPER_SEARCH_MOOD_EXCITED 11516
#define IDS_NTP_WALLPAPER_SEARCH_MOOD_HAPPY 11517
#define IDS_NTP_WALLPAPER_SEARCH_MOOD_INTELLECTUAL 11518
#define IDS_NTP_WALLPAPER_SEARCH_MOOD_ORDERLY 11519
#define IDS_NTP_WALLPAPER_SEARCH_MOOD_ROMANTIC 11520
#define IDS_NTP_WALLPAPER_SEARCH_MOOD_SERENE 11521
#define IDS_NTP_WALLPAPER_SEARCH_MOOD_SNOWY 11522
#define IDS_NTP_WALLPAPER_SEARCH_MOOD_SUBDUED 11523
#define IDS_NTP_WALLPAPER_SEARCH_MOOD_SUNNY 11524
#define IDS_NTP_WALLPAPER_SEARCH_MOOD_THOUGHTFUL 11525
#define IDS_NTP_WALLPAPER_SEARCH_MOOD_WHIMSICAL 11526
#define IDS_NTP_WALLPAPER_SEARCH_MOOD_WILD 11527
#define IDS_NTP_WALLPAPER_SEARCH_INSPIRATION_DESCRIPTION_1 11528
#define IDS_NTP_WALLPAPER_SEARCH_INSPIRATION_DESCRIPTION_2 11529
#define IDS_NTP_WALLPAPER_SEARCH_INSPIRATION_DESCRIPTION_3 11530
#define IDS_NTP_WALLPAPER_SEARCH_INSPIRATION_DESCRIPTION_4 11531
#define IDS_NTP_WALLPAPER_SEARCH_INSPIRATION_DESCRIPTION_5 11532
#define IDS_NTP_WALLPAPER_SEARCH_INSPIRATION_DESCRIPTION_6 11533
#define IDS_NTP_WALLPAPER_SEARCH_INSPIRATION_DESCRIPTION_7 11534
#define IDS_NTP_WALLPAPER_SEARCH_INSPIRATION_DESCRIPTION_8 11535
#define IDS_NTP_WALLPAPER_SEARCH_INSPIRATION_DESCRIPTION_9 11536
#define IDS_NTP_WALLPAPER_SEARCH_SEPARATOR 11537
#define IDS_NTP_CUSTOMIZE_TOOLBAR_BUTTON_A11Y_LABEL 11538
#define IDS_NTP_CUSTOMIZE_TOOLBAR_CHOOSE_ICONS_LABEL 11539
#define IDS_NTP_CUSTOMIZE_TOOLBAR_ENTERPRISE_ENABLED_LABEL 11540
#define IDS_NTP_CUSTOMIZE_TOOLBAR_REORDER_TIP_LABEL 11541
#define IDS_NTP_CUSTOMIZE_TOOLBAR_RESET_TO_DEFAULT_ANNOUNCEMENT 11542
#define IDS_NTP_CUSTOMIZE_TOOLBAR_CATEGORY_NAVIGATION 11543
#define IDS_NTP_CUSTOMIZE_TOOLBAR_CATEGORY_SIDE_PANELS 11544
#define IDS_NTP_CUSTOMIZE_TOOLBAR_CATEGORY_TOOLS_AND_ACTIONS 11545
#define IDS_NTP_CUSTOMIZE_TOOLBAR_RESET_TO_DEFAULT_BUTTON_LABEL 11546
#define IDS_NTP_COMPOSE_ENTRYPOINT 11547
#define IDS_NTP_COMPOSE_ENTRYPOINT_A11Y_LABEL 11548
#define IDS_NTP_COMPOSE_PLACEHOLDER_TEXT 11549
#define IDS_NTP_COMPOSE_CANCEL_BUTTON_A11Y_LABEL 11550
#define IDS_NTP_COMPOSE_CANCEL_BUTTON_A11Y_LABEL_INPUT 11551
#define IDS_NTP_COMPOSE_IMAGE_UPLOAD_BUTTON_A11Y_LABEL 11552
#define IDS_NTP_COMPOSE_PDF_UPLOAD_BUTTON_A11Y_LABEL 11553
#define IDS_NTP_COMPOSE_SUBMIT_BUTTON_A11Y_LABEL 11554
#define IDS_NTP_COMPOSE_DELETE_FILE_A11Y_LABEL 11555
#define IDS_NTP_COMPOSE_FILE_UPLOAD_STARTED_A11Y_TEXT 11556
#define IDS_NTP_COMPOSE_FILE_UPLOAD_COMPLETE_A11Y_TEXT 11557
#define IDS_NTP_COMPOSE_FILE_UPLOAD_INVALID_EMPTY_SIZE 11558
#define IDS_NTP_COMPOSE_FILE_UPLOAD_INVALID_TOO_LARGE 11559
#define IDS_NTP_COMPOSE_FILE_UPLOAD_IMAGE_PROCESSING_ERROR 11560
#define IDS_NTP_COMPOSE_FILE_UPLOAD_VALIDATION_FAILED 11561
#define IDS_NTP_COMPOSE_DRAG_AND_DROP_HINT 11562
#define IDS_NTP_COMPOSE_MAX_FILES_REACHED_ERROR 11563
#define IDS_NTP_COMPOSE_MAX_IMAGES_REACHED_ERROR 11564
#define IDS_NTP_COMPOSE_MAX_PDFS_REACHED_ERROR 11565
#define IDS_NTP_COMPOSE_FILE_TYPE_NOT_ALLOWED_ERROR 11566
#define IDS_NTP_COMPOSE_FILE_UPLOAD_NOT_ALLOWED 11567
#define IDS_NTP_COMPOSE_FILE_UPLOAD_FAILED 11568
#define IDS_NTP_COMPOSE_FILE_UPLOAD_EXPIRED 11569
#define IDS_NTP_COMPOSE_MOST_RECENT_TABS 11570
#define IDS_NTP_COMPOSEBOX_TAB_PICKER_ADD_TABS_TITLE 11571
#define IDS_NTP_COMPOSEBOX_SHARE_TABS 11572
#define IDS_NTP_COMPOSEBOX_RECENT_TAB_SUFFIX 11573
#define IDS_NTP_COMPOSE_ADD_IMAGE 11574
#define IDS_NTP_COMPOSE_ADD_FILE 11575
#define IDS_NTP_COMPOSE_ADD_DRIVE 11576
#define IDS_NTP_COMPOSE_ADD_CONTEXT 11577
#define IDS_NTP_COMPOSE_ADD_CONTEXTS 11578
#define IDS_NTP_COMPOSE_ADD_CONTEXT_TITLE 11579
#define IDS_NTP_COMPOSE_SUGGESTIONS_INFO 11580
#define IDS_NTP_COMPOSE_SMART_COMPOSE_A11Y_LABEL 11581
#define IDS_NTP_COMPOSE_SMART_COMPOSE_TAB 11582
#define IDS_NTP_COMPOSE_DEEP_SEARCH 11583
#define IDS_NTP_COMPOSE_CREATE_IMAGES 11584
#define IDS_NTP_COMPOSE_CANVAS 11585
#define IDS_NTP_COMPOSE_MODELS_LABEL 11586
#define IDS_NTP_COMPOSE_AUTO_MODEL 11587
#define IDS_NTP_COMPOSE_THINKING_3_PRO 11588
#define IDS_COMPOSE_DEEP_SEARCH_PLACEHOLDER 11589
#define IDS_COMPOSE_CREATE_IMAGE_IPH 11590
#define IDS_COMPOSE_DEEP_SEARCH_IPH 11591
#define IDS_COMPOSE_CREATE_IMAGE_PLACEHOLDER 11592
#define IDS_COMPOSE_HINT_TEXT_ASK_ABOUT_THESE 11593
#define IDS_COMPOSE_HINT_TEXT_ASK_ABOUT_THIS_IMAGE 11594
#define IDS_COMPOSE_HINT_TEXT_ASK_ABOUT_THIS_TAB 11595
#define IDS_COMPOSE_HINT_TEXT_ASK_ABOUT_THIS_DOC 11596
#define IDS_COMPOSE_SHARING_TABS 11597
#define IDS_COMPOSE_SHARING_TABS_WITH_GOOGLE 11598
#define IDS_COMPOSE_ADD_TABS 11599
#define IDS_COMPOSE_CURRENT_TAB 11600
#define IDS_NTP_ACTION_CHIP_TAB_HEADING_1 11601
#define IDS_NTP_ACTION_CHIP_TAB_HEADING_2 11602
#define IDS_NTP_ACTION_CHIP_TAB_BODY_1 11603
#define IDS_NTP_ACTION_CHIP_TAB_BODY_2 11604
#define IDS_NTP_ACTION_CHIP_CREATE_IMAGE_HEADING 11605
#define IDS_NTP_ACTION_CHIP_CREATE_IMAGE_BODY_1 11606
#define IDS_NTP_ACTION_CHIP_CREATE_IMAGE_BODY_2 11607
#define IDS_NTP_ACTION_CHIP_DEEP_SEARCH_HEADING 11608
#define IDS_NTP_ACTION_CHIP_DEEP_SEARCH_BODY 11609
#define IDS_NTP_ACTION_CHIP_CANVAS_HEADING 11610
#define IDS_NTP_ACTION_CHIP_CANVAS_BODY 11611
#define IDS_NTP_ACTION_CHIP_STARTER_HEADING 11612
#define IDS_NTP_ACTION_CHIP_STARTER_BODY 11613
#define IDS_NTP_ACTION_CHIP_DISABLE_TEXT 11614
#define IDS_NTP_ACTION_CHIPS_UNDO_DISABLEMENT_TOAST_MESSAGE 11615
#define IDS_COMPOSE_REMOVE_TOOL_CHIP_A11Y_LABEL 11616
#define IDS_COMPOSE_ASK_ABOUT_THIS_TAB 11617
#define IDS_COMPOSE_ASK_ABOUT_THIS_TAB_ARIA_LABEL 11618
#define IDS_COMPOSE_ASK_ABOUT_THIS_PAGE 11619
#define IDS_COMPOSE_ASK_ABOUT_THIS_PAGE_ARIA_LABEL 11620
#define IDS_NTP_MODULES_SETUP_LIST_DISABLE_TOAST_MESSAGE 11621
#define IDS_NTP_MODULES_SETUP_LIST_DISMISS_TOAST_MESSAGE 11622
#define IDS_NTP_MODULES_SETUP_LIST_INFO 11623
#define IDS_NTP_MODULES_INFO_BUTTON_TITLE 11624
#define IDS_NTP_MODULES_DISMISS_TOAST_MESSAGE 11625
#define IDS_NTP_MODULES_DISABLE_TOAST_MESSAGE 11626
#define IDS_NTP_MODULE_INACTIVITY_REMOVAL 11627
#define IDS_NTP_MODULES_INACTIVITY_REMOVAL 11628
#define IDS_NTP_MODULES_HISTORY_CLUSTERS_DISABLE_TOAST_MESSAGE 11629
#define IDS_NTP_MODULES_DISMISS_BUTTON_TEXT 11630
#define IDS_NTP_MODULES_DRIVE_MORE_ACTIONS_BUTTON_TEXT 11631
#define IDS_NTP_MODULES_DISABLE_BUTTON_TEXT 11632
#define IDS_NTP_MODULES_DISABLE_BUTTON_TEXT_V2 11633
#define IDS_NTP_MODULES_MORE_ACTIONS 11634
#define IDS_NTP_MODULE_HEADER_MORE_ACTIONS_MENU 11635
#define IDS_NTP_MODULES_CUSTOMIZE_BUTTON_TEXT 11636
#define IDS_NTP_MODULES_STATEFUL_TASKS_VIEWED_TODAY 11637
#define IDS_NTP_MODULES_STATEFUL_TASKS_VIEWED_YESTERDAY 11638
#define IDS_NTP_MODULES_STATEFUL_TASKS_VIEWED_PAST_WEEK 11639
#define IDS_NTP_MODULES_STATEFUL_TASKS_VIEWED_PAST_MONTH 11640
#define IDS_NTP_MODULES_STATEFUL_TASKS_VIEWED_PREVIOUSLY 11641
#define IDS_NTP_MODULES_SHOPPING_TASKS_SENTENCE 11642
#define IDS_NTP_MODULES_SHOPPING_TASKS_LOWER 11643
#define IDS_NTP_MODULES_DRIVE_DISABLE_BUTTON_TEXT 11644
#define IDS_NTP_MODULES_DRIVE_DISABLE_BUTTON_TEXT_V2 11645
#define IDS_NTP_MODULES_DRIVE_NAME 11646
#define IDS_NTP_MODULES_FILES_SENTENCE 11647
#define IDS_NTP_MODULES_DUMMY_LOWER 11648
#define IDS_NTP_MODULES_DUMMY_TITLE 11649
#define IDS_NTP_MODULES_DRIVE_INFO 11650
#define IDS_NTP_MODULES_MICROSOFT_FILES_NAME 11651
#define IDS_NTP_MODULES_MICROSOFT_FILES_INFO 11652
#define IDS_NTP_MODULES_MICROSOFT_FILES_DISABLE_BUTTON_TEXT 11653
#define IDS_NTP_MODULES_MICROSOFT_FILES_TRENDING_JUSTIFICATION_TEXT 11654
#define IDS_NTP_MODULES_MICROSOFT_FILES_OPENED_TODAY_JUSTIFICATION_TEXT 11655
#define IDS_NTP_MODULES_MICROSOFT_FILES_OPENED_YESTERDAY_JUSTIFICATION_TEXT 11656
#define IDS_NTP_MODULES_MICROSOFT_FILES_OPENED_PAST_WEEK_JUSTIFICATION_TEXT 11657
#define IDS_NTP_MODULES_MICROSOFT_FILES_SHARED_BY_JUSTIFICATION_TEXT 11658
#define IDS_NTP_MODULES_DISMISS_FOR_HOURS_BUTTON_TEXT 11659
#define IDS_NTP_MODULES_DISMISS_FOR_DAYS_BUTTON_TEXT 11660
#define IDS_NTP_MODULES_GOOGLE_CALENDAR_DISMISS_TOAST_MESSAGE 11661
#define IDS_NTP_MODULES_GOOGLE_CALENDAR_DISABLE_TOAST_MESSAGE 11662
#define IDS_NTP_MODULES_GOOGLE_CALENDAR_INFO 11663
#define IDS_NTP_MODULES_GOOGLE_CALENDAR_MORE_ACTIONS 11664
#define IDS_NTP_MODULES_GOOGLE_CALENDAR_TITLE 11665
#define IDS_NTP_MODULES_GOOGLE_CALENDAR_DISABLE_BUTTON_TEXT 11666
#define IDS_NTP_MODULES_MICROSOFT_SIGN_OUT_BUTTON_TEXT 11667
#define IDS_NTP_MODULES_OUTLOOK_CALENDAR_TITLE 11668
#define IDS_NTP_MODULES_OUTLOOK_CALENDAR_DISABLE_BUTTON_TEXT 11669
#define IDS_NTP_MODULES_OUTLOOK_CALENDAR_DISMISS_TOAST_MESSAGE 11670
#define IDS_NTP_MODULES_OUTLOOK_CALENDAR_INFO 11671
#define IDS_NTP_MODULES_CALENDAR_DOUBLE_BOOKED 11672
#define IDS_NTP_MODULES_CALENDAR_JOIN_MEETING_BUTTON_TEXT 11673
#define IDS_NTP_MODULES_CALENDAR_JOIN_MEETING_BUTTON_ACCNAME 11674
#define IDS_NTP_MODULES_CALENDAR_IN_PROGRESS 11675
#define IDS_NTP_MODULES_CALENDAR_IN_X_MIN 11676
#define IDS_NTP_MODULES_CALENDAR_IN_X_HR 11677
#define IDS_NTP_MODULES_CALENDAR_SEE_MORE 11678
#define IDS_NTP_MODULES_CALENDAR_SEE_MORE_ACCNAME 11679
#define IDS_NTP_MODULES_KALEIDOSCOPE_TITLE 11680
#define IDS_NTP_MODULES_SHOPPING_TASKS_INFO_TITLE 11681
#define IDS_NTP_MODULES_TASKS_INFO 11682
#define IDS_NTP_MODULES_SHOPPING_TASKS_INFO_CLOSE 11683
#define IDS_NTP_MODULES_SHOPPING_TASKS_RELATED 11684
#define IDS_NTP_MODULES_CART_DISCOUNT_CHIP_AMOUNT 11685
#define IDS_NTP_MODULES_CART_DISCOUNT_CHIP_UP_TO_AMOUNT 11686
#define IDS_NTP_MODULES_CUSTOMIZE_IPH 11687
#define IDS_NTP_MODULES_HISTORY_CLUSTERS_RESUME_BROWSING 11688
#define IDS_NTP_MODULES_HISTORY_CLUSTERS_RESUME_BROWSING_FOR 11689
#define IDS_NTP_MODULES_SHOW_ALL 11690
#define IDS_NTP_MODULES_HISTORY_CLUSTERS_BOOKMARKED 11691
#define IDS_NTP_MODULES_HISTORY_CLUSTERS_INFO 11692
#define IDS_NTP_MODULES_HISTORY_CLUSTERS_WITH_DISCOUNT_INFO 11693
#define IDS_NTP_MODULES_HISTORY_CLUSTERS_SENTENCE2 11694
#define IDS_NTP_MODULES_HISTORY_CLUSTERS_DISABLE_TOAST_NAME 11695
#define IDS_NTP_MODULES_HISTORY_CLUSTERS_DISMISS_BUTTON 11696
#define IDS_NTP_MODULES_HISTORY_CLUSTERS_DISABLE_DROPDOWN_TEXT 11697
#define IDS_NTP_MODULES_HISTORY_CLUSTERS_DONE_BUTTON 11698
#define IDS_NTP_MODULES_HISTORY_CLUSTERS_OPEN_ALL_IN_NEW_TAB_GROUP_BUTTON_TEXT 11699
#define IDS_NTP_MODULES_HISTORY_CLUSTERS_SHOW_ALL_BUTTON 11700
#define IDS_ACCNAME_SHOW_ALL 11701
#define IDS_ACCNAME_SEARCH_SUGG 11702
#define IDS_NTP_MODULES_TAB_RESUMPTION_DISMISS_BUTTON 11703
#define IDS_NTP_MODULES_TAB_RESUMPTION_INFO 11704
#define IDS_NTP_MODULES_TAB_RESUMPTION_MULTI_DISMISS 11705
#define IDS_NTP_MODULES_TAB_RESUMPTION_SINGLE_DISMISS 11706
#define IDS_NTP_MODULES_TAB_RESUMPTION_RECENTLY_OPENED 11707
#define IDS_NTP_MODULES_TAB_RESUMPTION_DEVICE_PREFIX 11708
#define IDS_NTP_MODULES_MOST_RELEVANT_TAB_RESUMPTION_DISMISS_BUTTON 11709
#define IDS_NTP_MODULES_MOST_RELEVANT_TAB_RESUMPTION_TITLE 11710
#define IDS_NTP_MODULES_MOST_RELEVANT_TAB_RESUMPTION_SEE_MORE 11711
#define IDS_NTP_MODULES_MICROSOFT_AUTHENTICATION_NAME 11712
#define IDS_NTP_MODULES_TAB_GROUPS_DISMISS_TOAST_MESSAGE 11713
#define IDS_NTP_MODULES_TAB_GROUPS_DISABLE_TOAST_MESSAGE 11714
#define IDS_NTP_MODULES_TAB_GROUPS_INFO 11715
#define IDS_NTP_MODULES_TAB_GROUPS_TITLE 11716
#define IDS_NTP_MODULES_TAB_GROUPS_TIME_DELTA 11717
#define IDS_NTP_MODULES_TAB_GROUPS_RECENTLY_USED 11718
#define IDS_NTP_MODULES_TAB_GROUPS_ZERO_STATE_TITLE 11719
#define IDS_NTP_MODULES_TAB_GROUPS_ZERO_STATE_TEXT 11720
#define IDS_NTP_MICROSOFT_AUTHENTICATION_DISABLE_BUTTON_TEXT 11721
#define IDS_NTP_MICROSOFT_AUTHENTICATION_DISMISS_BUTTON_TEXT 11722
#define IDS_NTP_MICROSOFT_AUTHENTICATION_HEADING 11723
#define IDS_NTP_MICROSOFT_AUTHENTICATION_SUBHEADING 11724
#define IDS_NTP_MICROSOFT_AUTHENTICATION_SIGN_IN_BUTTON_TEXT 11725
#define IDS_NTP_MICROSOFT_AUTHENTICATION_SIDE_PANEL_DESCRIPTION 11726
#define IDS_NTP_MOBILE_PROMO_HEADER 11727
#define IDS_NTP_MOBILE_PROMO_DESCRIPTION 11728
#define IDS_NTP_MOBILE_PROMO_QR_CODE_LABEL 11729
#define IDS_NTP_MOST_VISITED_SHORTCUTS_INACTIVITY_REMOVAL 11730
#define IDS_EXTENSIONS_PROMO_PERFORMANCE 11731
#define IDS_EXTENSIONS_PROMO_PRIVACY 11732
#define IDS_EXTENSIONS_PROMO_NEUTRAL 11733
#define IDS_NEW_TAB_FOOTER_NAME 11734
#define IDS_OPENS_MANAGEMENT_PAGE 11735
#define IDS_HIDE_NEW_TAB_FOOTER 11736
#define IDS_NEW_TAB_FOOTER_BACKGROUND_ATTRIBUTION_ARIA_DESC 11737
#define IDS_NEW_TAB_FOOTER_BACKGROUND_ATTRIBUTION_TEXT 11738
#define IDS_STAR_VIEW_MENU_ADD_BOOKMARK 11739
#define IDS_STAR_VIEW_MENU_EDIT_BOOKMARK 11740
#define IDS_STAR_VIEW_MENU_MOVE_TO_READ_LATER 11741
#define IDS_STAR_VIEW_MENU_MARK_AS_READ 11742
#define IDS_READ_LATER_TITLE 11743
#define IDS_READ_LATER_EXPAND_BUTTON_ARIA_LABEL 11744
#define IDS_READ_LATER_COLLAPSE_BUTTON_ARIA_LABEL 11745
#define IDS_READ_LATER_MENU_UNREAD_HEADER 11746
#define IDS_READ_LATER_MENU_READ_HEADER 11747
#define IDS_READ_LATER_MENU_TOOLTIP_MARK_AS_READ 11748
#define IDS_READ_LATER_MENU_TOOLTIP_MARK_AS_UNREAD 11749
#define IDS_READ_LATER_MENU_EMPTY_STATE_ADD_FROM_DIALOG_SUBHEADER 11750
#define IDS_READ_LATER_MENU_EMPTY_STATE_HEADER 11751
#define IDS_READ_LATER_MENU_EMPTY_STATE_SUBHEADER 11752
#define IDS_READ_LATER_ADD_CURRENT_TAB 11753
#define IDS_READ_LATER_CANT_ADD_CURRENT_TAB 11754
#define IDS_READ_LATER_MARK_CURRENT_TAB_READ 11755
#define IDS_READ_LATER_REMOVE_CURRENT_TAB 11756
#define IDS_READ_LATER_CONTEXT_MENU_MARK_AS_UNREAD 11757
#define IDS_READ_LATER_CONTEXT_MENU_MARK_AS_READ 11758
#define IDS_READ_LATER_CONTEXT_MENU_DELETE 11759
#define IDS_READING_LIST_MENU 11760
#define IDS_READING_LIST_MENU_ADD_TAB 11761
#define IDS_READING_LIST_MENU_SHOW_UI 11762
#define IDS_ACCNAME_SIDE_PANEL_RESIZE 11763
#define IDS_ACCNAME_SIDE_PANEL_CLOSE 11764
#define IDS_SIDE_PANEL_RESIZE_ACCESSIBLE_ALERT 11765
#define IDS_SIDE_PANEL_RESIZE_LEFT_SIDE_ACCESSIBLE_ALERT 11766
#define IDS_SIDE_PANEL_RESIZE_RIGHT_SIDE_ACCESSIBLE_ALERT 11767
#define IDS_ACCNAME_SIDE_PANEL_SELECTOR 11768
#define IDS_ACCNAME_SIDE_PANEL_COMPANION_SHOW 11769
#define IDS_ACCNAME_SIDE_PANEL_COMPANION_HIDE 11770
#define IDS_SIDE_PANEL_COMPANION_TITLE 11771
#define IDS_SIDE_PANEL_COMPANION_TOOLBAR_TOOLTIP 11772
#define IDS_SIDE_PANEL_COMPANION_PROMO 11773
#define IDS_SIDE_PANEL_COMPANION_PROMO_PINNING 11774
#define IDS_SIDE_PANEL_COMPANION_PROMO_SCREEN_READER 11775
#define IDS_SIDE_PANEL_COMPANION_ERROR_PAGE_FIRST_LINE 11776
#define IDS_SIDE_PANEL_COMPANION_ERROR_PAGE_SECOND_LINE 11777
#define IDS_SHOW_CUSTOMIZE_CHROME_SIDE_PANEL 11778
#define IDS_SHOW_CUSTOMIZE_CHROME_TOOLBAR 11779
#define IDS_SIDE_PANEL_SHOW_TABS_FROM_OTHER_DEVICES 11780
#define IDS_SIDE_PANEL_TABS_FROM_OTHER_DEVICES_TITLE 11781
#define IDS_SIDE_PANEL_TOOLBAR_BUTTON_CXMENU_UNPIN 11782
#define IDS_SIDE_PANEL_TOOLBAR_BUTTON_CXMENU_PIN 11783
#define IDS_SIDE_PANEL_HEADER_UNPIN_BUTTON_TOOLTIP 11784
#define IDS_SIDE_PANEL_HEADER_PIN_BUTTON_TOOLTIP 11785
#define IDS_SIDE_PANEL_UNPINNED 11786
#define IDS_SIDE_PANEL_PINNED 11787
#define IDS_PINNED_ACTION_BUTTON_ACCESSIBLE_TITLE 11788
#define IDS_SIDE_PANEL_GENERIC_PINNABLE_IPH 11789
#define IDS_SIDE_PANEL_GENERIC_PINNABLE_IPH_SCREENREADER 11790
#define IDS_SIDE_PANEL_HEADER_MORE_INFO_BUTTON_TOOLTIP 11791
#define IDS_BOOKMARKS_MOVE_TO_ACCOUNT_DIALOG_TITLE 11792
#define IDS_BOOKMARKS_MOVE_TO_ACCOUNT_DIALOG_SUBTITLE 11793
#define IDS_BOOKMARKS_MOVE_TO_ACCOUNT_DIALOG_FOLDER_SUBTITLE 11794
#define IDS_BOOKMARKS_MOVE_TO_ACCOUNT_DIALOG_OK_BUTTON_LABEL 11795
#define IDS_BOOKMARKS_MOVE_TO_DEVICE_DIALOG_TITLE 11796
#define IDS_BOOKMARKS_MOVE_TO_DEVICE_DIALOG_SUBTITLE 11797
#define IDS_BOOKMARKS_MOVE_TO_DEVICE_DIALOG_FOLDER_SUBTITLE 11798
#define IDS_BOOKMARKS_MOVE_TO_DEVICE_DIALOG_OK_BUTTON_LABEL 11799
#define IDS_BOOKMARKS_SORT_BY_TYPE 11800
#define IDS_BOOKMARKS_ALL_BOOKMARKS 11801
#define IDS_BOOKMARKS_ALL_BOOKMARKS_OPEN_SIDE_PANEL 11802
#define IDS_BOOKMARKS_ACCOUNT_BOOKMARKS 11803
#define IDS_BOOKMARKS_DEVICE_BOOKMARKS 11804
#define IDS_BOOKMARKS_LABEL_TRACKED_PRODUCTS 11805
#define IDS_BOOKMARKS_SORT_NEWEST 11806
#define IDS_BOOKMARKS_SORT_NEWEST_LOWER 11807
#define IDS_BOOKMARKS_SORT_OLDEST 11808
#define IDS_BOOKMARKS_SORT_OLDEST_LOWER 11809
#define IDS_BOOKMARKS_SORT_ALPHABETICALLY 11810
#define IDS_BOOKMARKS_SORT_REVERSE_ALPHABETICALLY 11811
#define IDS_BOOKMARKS_SORT_LAST_OPENED 11812
#define IDS_BOOKMARKS_SORT_LAST_OPENED_LOWER 11813
#define IDS_BOOKMARKS_VISUAL_VIEW 11814
#define IDS_BOOKMARKS_COMPACT_VIEW 11815
#define IDS_BOOKMARKS_SORT_MENU_A11Y_LABEL 11816
#define IDS_BOOKMARKS_CREATE_NEW_FOLDER_A11Y_LABEL 11817
#define IDS_BOOKMARKS_NEW_FOLDER_TOOLTIP 11818
#define IDS_BOOKMARKS_EDIT_BOOKMARK_LIST_A11Y_LABEL 11819
#define IDS_BOOKMARKS_EMPTY_STATE_TITLE 11820
#define IDS_BOOKMARKS_EMPTY_STATE_BODY 11821
#define IDS_BOOKMARKS_EMPTY_STATE_TITLE_FOLDER 11822
#define IDS_BOOKMARKS_EMPTY_STATE_BODY_FOLDER 11823
#define IDS_BOOKMARKS_EMPTY_STATE_TITLE_GUEST 11824
#define IDS_BOOKMARKS_EMPTY_STATE_BODY_GUEST 11825
#define IDS_BOOKMARKS_EMPTY_STATE_TITLE_SEARCH 11826
#define IDS_BOOKMARKS_EMPTY_STATE_BODY_SEARCH 11827
#define IDS_BOOKMARK_FOLDER_CHILD_COUNT 11828
#define IDS_BOOKMARKS_EDIT_BOOKMARK 11829
#define IDS_BOOKMARKS_EDIT_MOVE_TO_ANOTHER_FOLDER 11830
#define IDS_BOOKMARKS_EDIT_MORE 11831
#define IDS_BOOKMARKS_EDIT_MOVE_TO 11832
#define IDS_BOOKMARKS_EDIT_NEW_FOLDER 11833
#define IDS_BOOKMARKS_EDIT_CANCEL 11834
#define IDS_BOOKMARKS_EDIT_SAVE 11835
#define IDS_BOOKMARKS_EDIT_NAME 11836
#define IDS_BOOKMARKS_EDIT_URL 11837
#define IDS_BOOKMARK_DELETION_COUNT 11838
#define IDS_UNDO_BOOKMARK_DELETION 11839
#define IDS_BOOKMARKS_MOVE_TO_BOOKMARKS_BAR 11840
#define IDS_BOOKMARKS_MOVE_TO_ALL_BOOKMARKS 11841
#define IDS_BOOKMARKS_EDIT 11842
#define IDS_BOOKMARKS_RENAME 11843
#define IDS_BOOKMARKS_URL_FOLDER_DESCRIPTION 11844
#define IDS_BOOKMARKS_DISABLED_FEATURE 11845
#define IDS_BOOKMARKS_BACK_BUTTON_LABEL 11846
#define IDS_BOOKMARKS_BACK_BUTTON_TOOLTIP 11847
#define IDS_BOOKMARKS_FORWARD_BUTTON_LABEL 11848
#define IDS_BOOKMARK_OPTIONS_LABEL 11849
#define IDS_FOLDER_OPTIONS_LABEL 11850
#define IDS_BOOKMARKS_OPEN_FOLDER_LABEL 11851
#define IDS_BOOKMARKS_OPEN_BOOKMARK_LABEL 11852
#define IDS_BOOKMARKS_SELECT_FOLDER_LABEL 11853
#define IDS_BOOKMARKS_SELECT_BOOKMARK_LABEL 11854
#define IDS_BOOKMARKS_DESELECT_FOLDER_LABEL 11855
#define IDS_BOOKMARKS_DESELECT_BOOKMARK_LABEL 11856
#define IDS_BOOKMARK_ACCESSIBLE_DESCRIPTION_PRICE_TRACKING 11857
#define IDS_BOOKMARK_ACCESSIBLE_DESCRIPTION_PRICE_CHANGE 11858
#define IDS_BOOKMARKS_CHECKBOX_LABEL 11859
#define IDS_BOOKMARKS_PRIMARY_FILTER_HEADING 11860
#define IDS_BOOKMARKS_SECONDARY_FILTER_HEADING 11861
#define IDS_SHOW_READING_MODE_SIDE_PANEL 11862
#define IDS_READING_MODE_TITLE 11863
#define IDS_IMMERSIVE_READING_MODE_OPENED_ANNOUNCEMENT 11864
#define IDS_READING_MODE_TOOLTIP 11865
#define IDS_READING_MODE_NOT_SELECTABLE_HEADER 11866
#define IDS_READING_MODE_EMPTY_STATE_HEADER 11867
#define IDS_READING_MODE_EMPTY_STATE_SUBHEADER 11868
#define IDS_READING_MODE_TOOLBAR_LABEL 11869
#define IDS_READING_MODE_READ_ALOUD_TOOLBAR_LABEL 11870
#define IDS_READING_MODE_FONT_SIZE 11871
#define IDS_READING_MODE_FONT_NAME_COMBOBOX_LABEL 11872
#define IDS_READING_MODE_DECREASE_FONT_SIZE_BUTTON_LABEL 11873
#define IDS_READING_MODE_INCREASE_FONT_SIZE_BUTTON_LABEL 11874
#define IDS_READING_MODE_INCREASE_FONT_SIZE_ANNOUNCEMENT 11875
#define IDS_READING_MODE_DECREASE_FONT_SIZE_ANNOUNCEMENT 11876
#define IDS_READING_MODE_DISABLE_LINKS_BUTTON_LABEL 11877
#define IDS_READING_MODE_ENABLE_LINKS_BUTTON_LABEL 11878
#define IDS_READING_MODE_IMAGES_LABEL 11879
#define IDS_READING_MODE_DISABLE_IMAGES_BUTTON_LABEL 11880
#define IDS_READING_MODE_ENABLE_IMAGES_BUTTON_LABEL 11881
#define IDS_READING_MODE_PIN_LABEL 11882
#define IDS_READING_MODE_DISABLE_PIN_BUTTON_LABEL 11883
#define IDS_READING_MODE_ENABLE_PIN_BUTTON_LABEL 11884
#define IDS_READING_MODE_COLORS_COMBOBOX_LABEL 11885
#define IDS_READING_MODE_LINE_SPACING_COMBOBOX_LABEL 11886
#define IDS_READING_MODE_LETTER_SPACING_COMBOBOX_LABEL 11887
#define IDS_READING_MODE_SPACING_COMBOBOX_STANDARD 11888
#define IDS_READING_MODE_SPACING_COMBOBOX_LOOSE 11889
#define IDS_READING_MODE_SPACING_COMBOBOX_VERY_LOOSE 11890
#define IDS_READING_MODE_SPACING_COMBOBOX_WIDE 11891
#define IDS_READING_MODE_SPACING_COMBOBOX_VERY_WIDE 11892
#define IDS_READING_MODE_DEFAULT_COLOR_LABEL 11893
#define IDS_READING_MODE_LIGHT_COLOR_LABEL 11894
#define IDS_READING_MODE_DARK_COLOR_LABEL 11895
#define IDS_READING_MODE_YELLOW_COLOR_LABEL 11896
#define IDS_READING_MODE_HIGH_CONTRAST_COLOR_LABEL 11897
#define IDS_READING_MODE_LOW_CONTRAST_LIGHT_COLOR_LABEL 11898
#define IDS_READING_MODE_LOW_CONTRAST_DARK_COLOR_LABEL 11899
#define IDS_READING_MODE_BLUE_COLOR_LABEL 11900
#define IDS_READING_MODE_VOICE_HIGHLIGHT 11901
#define IDS_READING_MODE_AUTO_HIGHLIGHT_LABEL 11902
#define IDS_READING_MODE_WORD_HIGHLIGHT_LABEL 11903
#define IDS_READING_MODE_PHRASE_HIGHLIGHT_LABEL 11904
#define IDS_READING_MODE_SENTENCE_HIGHLIGHT_LABEL 11905
#define IDS_READING_MODE_OFF_HIGHLIGHT_LABEL 11906
#define IDS_READING_MODE_LINE_FOCUS_WINDOW_HEADING 11907
#define IDS_READING_MODE_LINE_FOCUS_STYLE_HEADING 11908
#define IDS_READING_MODE_LINE_FOCUS_SMALL_FOCUS_LABEL 11909
#define IDS_READING_MODE_LINE_FOCUS_MEDIUM_FOCUS_LABEL 11910
#define IDS_READING_MODE_LINE_FOCUS_LARGE_FOCUS_LABEL 11911
#define IDS_READING_MODE_LINE_FOCUS_UNDERLINE_LABEL 11912
#define IDS_READING_MODE_LINE_FOCUS_LINE_HEADING 11913
#define IDS_READING_MODE_LINE_FOCUS_MOVEMENT_HEADING 11914
#define IDS_READING_MODE_LINE_FOCUS_STATIC_LABEL 11915
#define IDS_READING_MODE_LINE_FOCUS_STATIC_LINE_LABEL 11916
#define IDS_READING_MODE_LINE_FOCUS_CURSOR_LINE_LABEL 11917
#define IDS_READING_MODE_LINE_FOCUS_OFF_LABEL 11918
#define IDS_READING_MODE_LINE_FOCUS_SMALL_FOCUS_LABEL_SCREENREADER 11919
#define IDS_READING_MODE_LINE_FOCUS_MEDIUM_FOCUS_LABEL_SCREENREADER 11920
#define IDS_READING_MODE_LINE_FOCUS_LARGE_FOCUS_LABEL_SCREENREADER 11921
#define IDS_READING_MODE_LINE_FOCUS_UNDERLINE_LABEL_SCREENREADER 11922
#define IDS_READING_MODE_LINE_FOCUS_STATIC_LABEL_SCREENREADER 11923
#define IDS_READING_MODE_LINE_FOCUS_CURSOR_LINE_LABEL_SCREENREADER 11924
#define IDS_READING_MODE_LINE_FOCUS_OFF_LABEL_SCREENREADER 11925
#define IDS_READING_MODE_KEYBOARD_SHORTCUT_IPH_BODY 11926
#define IDS_READING_MODE_KEYBOARD_SHORTCUT_IPH_SCREENREADER 11927
#define IDS_READ_ANYTHING_LOADING 11928
#define IDS_READING_MODE_FONT_RESET 11929
#define IDS_READING_MODE_FONT_RESET_TOOLTIP 11930
#define IDS_READING_MODE_TURN_HIGHLIGHT_OFF 11931
#define IDS_READING_MODE_TURN_HIGHLIGHT_ON 11932
#define IDS_READING_MODE_PLAY_DESCRIPTION 11933
#define IDS_READING_MODE_PLAY_SPEECH 11934
#define IDS_READING_MODE_PLAY_TOOLTIP 11935
#define IDS_READING_MODE_PREVIEW_TOOLTIP 11936
#define IDS_READING_MODE_PAUSE_TOOLTIP 11937
#define IDS_READING_MODE_STOP_SPEECH 11938
#define IDS_READING_MODE_NAVIGATE_PREVIOUS_SENTENCE 11939
#define IDS_READING_MODE_NAVIGATE_NEXT_SENTENCE 11940
#define IDS_READING_MODE_MORE_OPTIONS 11941
#define IDS_READING_MODE_SETTINGS 11942
#define IDS_READING_MODE_VOICE_SPEED 11943
#define IDS_READING_MODE_VOICE_SPEED_WITH_RATE 11944
#define IDS_READING_MODE_VOICE_SPEED_OPTION_TITLE 11945
#define IDS_READING_MODE_VOICE_SELECTION 11946
#define IDS_READING_MODE_SYSTEM_VOICE 11947
#define IDS_READING_MODE_ITEM_SELECTED 11948
#define IDS_READING_MODE_LANGUAGE_MENU 11949
#define IDS_READING_MODE_LANGUAGE_MENU_TITLE 11950
#define IDS_READING_MODE_LANGUAGE_MENU_CLOSE 11951
#define IDS_READING_MODE_CLOSE 11952
#define IDS_READING_MODE_LANGUAGE_MENU_NO_RESULTS 11953
#define IDS_READING_MODE_LANGUAGE_MENU_SEARCH_LABEL 11954
#define IDS_READING_MODE_LANGUAGE_MENU_SEARCH_CLEAR 11955
#define IDS_READING_MODE_LANGUAGE_MENU_DOWNLOADING 11956
#define IDS_READING_MODE_VOICE_MENU_DOWNLOADING 11957
#define IDS_READING_MODE_LANGUAGE_MENU_VOICES_UNAVAILABLE 11958
#define IDS_READING_MODE_LANGUAGE_MENU_NO_INTERNET 11959
#define IDS_READING_MODE_CANT_USE_READ_ALOUD 11960
#define IDS_READING_MODE_VOICE_MENU_NO_INTERNET 11961
#define IDS_READING_MODE_LANGUAGE_MENU_DOWNLOAD_FAILED 11962
#define IDS_READING_MODE_LANGUAGE_MENU_NO_SPACE 11963
#define IDS_READING_MODE_TOAST_NO_SPACE 11964
#define IDS_READING_MODE_VOICE_MENU_NO_SPACE 11965
#define IDS_READING_MODE_VOICE_MENU_PREVIEW_LANGUAGE 11966
#define IDS_READING_MODE_LANGUAGE_MENU_NO_SPACE_BUT_VOICES_EXIST 11967
#define IDS_READING_MODE_LANGUAGE_MENU_ITEM_LABEL 11968
#define IDS_READING_MODE_VOICE_DOWNLOADED_TITLE 11969
#define IDS_READING_MODE_VOICE_DOWNLOADED_MESSAGE 11970
#define IDS_READING_MODE_SIDE_PANEL_PROMO 11971
#define IDS_READING_MODE_SIDE_PANEL_PROMO_PINNING 11972
#define IDS_READING_MODE_OMNIBOX_ENTRY_POINT_PROMO 11973
#define IDS_READING_MODE_OMNIBOX_ENTRY_POINT_PROMO_SCREEN_READER 11974
#define IDS_READING_MODE_VOICE_PREVIEW_STRING 11975
#define IDS_READING_MODE_FONT_LOADING_STRING 11976
#define IDS_READING_MODE_LINE_FOCUS_LABEL 11977
#define IDS_READING_MODE_SIDE_PANEL_LABEL 11978
#define IDS_READING_MODE_VIEW_FULL_PAGE_LABEL 11979
#define IDS_READING_MODE_FULL_SCREEN_LABEL 11980
#define IDS_READING_MODE_VIEW_LABEL 11981
#define IDS_READING_MODE_LINKS_LABEL 11982
#define IDS_READING_MODE_PRESENTATION_MODE_IPH_BODY 11983
#define IDS_READING_MODE_PRESENTATION_MODE_IPH_SCREENREADER 11984
#define IDS_GOOGLE_LENS_TITLE 11985
#define IDS_LENS_DEFAULT_TITLE 11986
#define IDS_TOOLTIP_BACK 11987
#define IDS_ACCDESCRIPTION_BACK 11988
#define IDS_TOOLTIP_CHROMELABS_BUTTON 11989
#define IDS_TOOLTIP_LEFT_ALIGNED_SIDE_PANEL_BUTTON 11990
#define IDS_TOOLTIP_CHROMELABS_COMBOBOX 11991
#define IDS_TOOLTIP_CHROMELABS_FEEDBACK_BUTTON 11992
#define IDS_TOOLTIP_FORWARD 11993
#define IDS_ACCDESCRIPTION_FORWARD 11994
#define IDS_TOOLTIP_DOWNLOAD_ICON 11995
#define IDS_TOOLTIP_DOWNLOAD_ICON_NEW_DOWNLOAD 11996
#define IDS_TOOLTIP_HOME 11997
#define IDS_TOOLTIP_RELOAD 11998
#define IDS_TOOLTIP_RELOAD_WITH_MENU 11999
#define IDS_TOOLTIP_STOP 12000
#define IDS_TOOLTIP_EXTENSIONS_BUTTON 12001
#define IDS_TOOLTIP_OVERFLOW_BUTTON 12002
#define IDS_TOOLTIP_EXTENSIONS_BUTTON_ALL_EXTENSIONS_BLOCKED 12003
#define IDS_TOOLTIP_EXTENSIONS_BUTTON_ANY_EXTENSION_HAS_ACCESS 12004
#define IDS_ACC_NAME_EXTENSIONS_BUTTON 12005
#define IDS_ACC_NAME_EXTENSIONS_BUTTON_ALL_EXTENSIONS_BLOCKED 12006
#define IDS_ACC_NAME_EXTENSIONS_BUTTON_ANY_EXTENSION_HAS_ACCESS 12007
#define IDS_TOOLTIP_EXTENSIONS_SITE_ACCESS_BUTTON 12008
#define IDS_TOOLTIP_SIDE_PANEL_SHOW 12009
#define IDS_TOOLTIP_SIDE_PANEL_HIDE 12010
#define IDS_TOOLTIP_SIDE_PANEL 12011
#define IDS_TOOLTIP_LOCATION_ICON 12013
#define IDS_TOOLTIP_NEW_TAB 12014
#define IDS_TOOLTIP_TAB_SCROLL_LEADING 12015
#define IDS_TOOLTIP_TAB_SCROLL_TRAILING 12016
#define IDS_TOOLTIP_MIC_SEARCH 12017
#define IDS_TOOLTIP_LENS_SEARCH 12018
#define IDS_TOOLTIP_LENS_REINVOKE_VISUAL_SELECTION_A11Y_LABEL 12019
#define IDS_LENS_SEARCH_UPLOAD_DIALOG_CLOSE_BUTTON_LABEL 12020
#define IDS_LENS_SEARCH_UPLOAD_DIALOG_TITLE 12021
#define IDS_LENS_SEARCH_UPLOAD_DIALOG_TITLE_SHORT 12022
#define IDS_LENS_SEARCH_UPLOAD_DIALOG_DRAG_TITLE 12023
#define IDS_LENS_SEARCH_UPLOAD_DIALOG_UPLOAD_FILE_TITLE 12024
#define IDS_LENS_SEARCH_UPLOAD_DIALOG_OR_TEXT 12025
#define IDS_LENS_SEARCH_UPLOAD_DIALOG_TEXT_PLACEHOLDER 12026
#define IDS_LENS_SEARCH_UPLOAD_DIALOG_SEARCH_BUTTON_LABEL 12027
#define IDS_LENS_SEARCH_UPLOAD_DIALOG_DRAG_DROP_TITLE 12028
#define IDS_LENS_SEARCH_UPLOAD_DIALOG_LOADING_TEXT 12029
#define IDS_LENS_SEARCH_UPLOAD_DIALOG_OFFLINE_TEXT 12030
#define IDS_LENS_SEARCH_UPLOAD_DIALOG_OFFLINE_SUBTITLE_TEXT 12031
#define IDS_LENS_SEARCH_UPLOAD_DIALOG_OFFLINE_BUTTON_LABEL 12032
#define IDS_LENS_SEARCH_UPLOAD_DIALOG_ERROR_FILE_TYPE 12033
#define IDS_LENS_SEARCH_UPLOAD_DIALOG_ERROR_FILE_SIZE 12034
#define IDS_LENS_SEARCH_UPLOAD_DIALOG_ERROR_MULTIPLE_FILES 12035
#define IDS_LENS_SEARCH_UPLOAD_DIALOG_VALIDATION_ERROR_SCHEME 12036
#define IDS_LENS_SEARCH_UPLOAD_DIALOG_VALIDATION_ERROR_CONFORMANCE 12037
#define IDS_LENS_SEARCH_UPLOAD_DIALOG_ERROR_MULTIPLE_URLS 12038
#define IDS_TOOLTIP_SAVE_CREDIT_CARD 12039
#define IDS_TOOLTIP_SAVE_CREDIT_CARD_PENDING 12040
#define IDS_TOOLTIP_SAVE_CVC 12041
#define IDS_TOOLTIP_SAVE_IBAN 12042
#define IDS_TOOLTIP_MIGRATE_LOCAL_CARD 12043
#define IDS_TOOLTIP_TRANSLATE 12044
#define IDS_TOOLTIP_ZOOM 12045
#define IDS_TOOLTIP_ZOOM_EXTENSION_ICON 12046
#define IDS_ZOOM_SET_DEFAULT 12047
#define IDS_TOOLTIP_FIND 12048
#define IDS_TOOLTIP_TAB_SEARCH 12049
#define IDS_SPLIT_VIEW_NTP_EMPTY_TITLE 12050
#define IDS_SPLIT_VIEW_NTP_EMPTY_BODY 12051
#define IDS_SPLIT_VIEW_NTP_TITLE 12052
#define IDS_SPLIT_VIEW_NTP_TAB_TITLE 12053
#define IDS_ACCNAME_SPLIT_VIEW_MINI_TOOLBAR_MENU_BUTTON 12054
#define IDS_SPLIT_VIEW_NTP_CLOSE_BUTTON_ARIA_LABEL 12055
#define IDS_SPLIT_VIEW_TITLE_IPH 12056
#define IDS_SPLIT_VIEW_BODY_IPH 12057
#define IDS_SPLIT_VIEW_PIN_ACTION_2_IPH 12058
#define IDS_SPLIT_VIEW_TAB_SWITCH_ENTRY_IPH_TITLE 12059
#define IDS_SPLIT_VIEW_TAB_SWITCH_ENTRY_IPH_BODY 12060
#define IDS_SPLIT_VIEW_TAB_SWITCH_STEP_IPH_BODY 12061
#define IDS_SPLIT_VIEW_TAB_SWITCH_COMPLETION_IPH_BODY 12062
#define IDS_SPLIT_VIEW_TOOLBAR_BUTTON_STEP_IPH_BODY 12063
#define IDS_SPLIT_VIEW_DRAG_ENTRYPOINT_LABEL 12064
#define IDS_SPLIT_VIEW_RESIZE_ACCESSIBLE_ALERT 12065
#define IDS_SPLIT_VIEW_RESIZE_LEFT_SIDE_ACCESSIBLE_ALERT 12066
#define IDS_SPLIT_VIEW_RESIZE_RIGHT_SIDE_ACCESSIBLE_ALERT 12067
#define IDS_SPLIT_VIEW_RESIZE_TOP_SIDE_ACCESSIBLE_ALERT 12068
#define IDS_SPLIT_VIEW_RESIZE_BOTTOM_SIDE_ACCESSIBLE_ALERT 12069
#define IDS_SWITCH_TO_VERTICAL_TAB 12070
#define IDS_SWITCH_TO_HORIZONTAL_TAB 12071
#define IDS_COLLAPSE_VERTICAL_TABS 12072
#define IDS_EXPAND_VERTICAL_TABS 12073
#define IDS_VERTICAL_TABS_SEND_FEEDBACK 12074
#define IDS_VERTICAL_TABS_TUTORIAL_STEP_SHOW_VERTICAL 12075
#define IDS_VERTICAL_TABS_TUTORIAL_STEP_COLLAPSE 12076
#define IDS_VERTICAL_TABS_TUTORIAL_STEP_END 12077
#define IDS_VERTICAL_TABS_ENABLE_EXPAND_ON_HOVER 12078
#define IDS_VERTICAL_TABS_DISABLE_EXPAND_ON_HOVER 12079
#define IDS_VERTICAL_TABS_COLLAPSE_BUTTON_TOGGLE_EXPAND_ON_HOVER 12080
#define IDS_VERTICAL_TABS_IPH_TITLE 12081
#define IDS_VERTICAL_TABS_IPH_BODY 12082
#define IDS_VERTICAL_TABS_EXPAND_ON_HOVER_DEFAULT_ENABLED_IPH_BODY 12083
#define IDS_VERTICAL_TABS_EXPAND_ON_HOVER_DEFAULT_DISABLED_IPH_BODY 12084
#define IDS_VERTICAL_RESIZE_AREA 12085
#define IDS_VERTICAL_TAB_RESIZE_ACCESSIBLE_ALERT 12086
#define IDS_VERTICAL_TAB_GROUP_MORE_OPTIONS 12087
#define IDS_VIEW_PROJECTS_PANEL 12088
#define IDS_HIDE_PROJECTS_PANEL 12089
#define IDS_TAB_GROUPS_TITLE 12090
#define IDS_RECENT_CHATS_TITLE 12091
#define IDS_TAB_GROUP_MORE_OPTIONS 12092
#define IDS_THREADS_SHOW_MORE 12093
#define IDS_THREADS_SHOW_LESS 12094
#define IDS_PROJECTS_PANEL 12095
#define IDS_PROJECTS_PANEL_GEMINI_ACTIVITY 12096
#define IDS_PROJECTS_PANEL_AI_MODE_ACTIVITY 12097
#define IDS_TAB_GROUPS_NO_TAB_GROUPS 12098
#define IDS_OPEN_GEMINI_THREAD_TOOLTIP 12099
#define IDS_OPEN_AI_MODE_THREAD_TOOLTIP 12100
#define IDS_OPEN_GEMINI_THREAD_ACCESSIBILITY_LABEL 12101
#define IDS_OPEN_AI_MODE_THREAD_ACCESSIBILITY_LABEL 12102
#define IDS_TAB_STRIP_UNPIN_TAB_SEARCH 12103
#define IDS_TAB_STRIP_PIN_TAB_SEARCH 12104
#define IDS_TAB_SEARCH_BUTTON_CXMENU_UNPIN 12105
#define IDS_TAB_SEARCH_BUTTON_CXMENU_PIN 12106
#define IDS_PROJECTS_PANEL_BUTTON_CXMENU_UNPIN 12107
#define IDS_PROJECTS_PANEL_BUTTON_CXMENU_PIN 12108
#define IDS_EVERYTHING_MENU_BUTTON_CXMENU_UNPIN 12109
#define IDS_EVERYTHING_MENU_BUTTON_CXMENU_PIN 12110
#define IDS_TOOLTIP_INTENT_PICKER_ICON 12111
#define IDS_INTENT_PICKER_BUBBLE_VIEW_OPEN_WITH 12112
#define IDS_INTENT_PICKER_BUBBLE_VIEW_OPEN_IN_APP 12113
#define IDS_INTENT_PICKER_BUBBLE_VIEW_REMEMBER_SELECTION 12114
#define IDS_INTENT_PICKER_BUBBLE_VIEW_OPEN 12115
#define IDS_INTENT_PICKER_BUBBLE_VIEW_STAY_IN_CHROME 12116
#define IDS_INTENT_PICKER_BUBBLE_VIEW_INITIATING_ORIGIN 12117
#define IDS_INTENT_CHIP_OPEN_IN_APP 12118
#define IDR_INTENT_PICKER_SUPPORTED_LINKS_INFOBAR_MESSAGE 12119
#define IDR_INTENT_PICKER_SUPPORTED_LINKS_INFOBAR_OK_LABEL 12120
#define IDS_IOS_DESKTOP_PROMO_BUBBLE_BUTTON_ACCEPT_REMINDER 12127
#define IDS_IOS_DESKTOP_PROMO_BUBBLE_BUTTON_DECLINE 12128
#define IDS_IOS_DESKTOP_PROMO_BUBBLE_REMINDER_CONFIRMATION_BUTTON 12129
#define IDS_IOS_DESKTOP_PROMO_BUBBLE_REMINDER_CONFIRMATION_TITLE 12130
#define IDS_IOS_DESKTOP_PROMO_BUBBLE_LENS_FEATURE_NAME 12131
#define IDS_IOS_DESKTOP_PROMO_BUBBLE_ENHANCED_BROWSING_FEATURE_NAME 12132
#define IDS_IOS_DEVICE_TYPE_IPHONE 12133
#define IDS_IOS_DEVICE_TYPE_IPAD 12134
#define IDS_NTP_MOBILE_PROMO_BODY 12135
#define IDS_NTP_MOBILE_PROMO_PASSWORDS_BODY 12136
#define IDS_IOS_DESKTOP_PASSWORD_PROMO_BUBBLE_TITLE 12137
#define IDS_IOS_DESKTOP_PASSWORD_PROMO_BUBBLE_FOOTER_TITLE_REMINDER 12138
#define IDS_IOS_DESKTOP_PASSWORD_PROMO_BUBBLE_BUTTON_DECLINE 12139
#define IDS_IOS_DESKTOP_PASSWORD_PROMO_BUBBLE_SUBTITLE 12140
#define IDS_IOS_DESKTOP_ADDRESS_PROMO_BUBBLE_TITLE 12141
#define IDS_IOS_DESKTOP_ADDRESS_PROMO_BUBBLE_FOOTER_TITLE 12142
#define IDS_IOS_DESKTOP_ADDRESS_PROMO_BUBBLE_BUTTON_DECLINE 12143
#define IDS_IOS_DESKTOP_ADDRESS_PROMO_BUBBLE_SUBTITLE 12144
#define IDS_IOS_DESKTOP_ADDRESS_PROMO_BUBBLE_FOOTER_DESCRIPTION_QR 12145
#define IDS_IOS_DESKTOP_PAYMENT_PROMO_BUBBLE_TITLE 12146
#define IDS_IOS_DESKTOP_PAYMENT_PROMO_BUBBLE_FOOTER_TITLE 12147
#define IDS_IOS_DESKTOP_PAYMENT_PROMO_BUBBLE_BUTTON_DECLINE 12148
#define IDS_IOS_DESKTOP_PAYMENT_PROMO_BUBBLE_SUBTITLE 12149
#define IDS_IOS_DESKTOP_PAYMENT_PROMO_BUBBLE_FOOTER_DESCRIPTION_QR 12150
#define IDS_IOS_DESKTOP_ESB_PROMO_BUBBLE_DESCRIPTION_QR 12151
#define IDS_IOS_DESKTOP_ESB_PROMO_BUBBLE_TITLE_REMINDER 12152
#define IDS_IOS_DESKTOP_LENS_PROMO_BUBBLE_DESCRIPTION_QR 12153
#define IDS_IOS_DESKTOP_LENS_PROMO_BUBBLE_TITLE_REMINDER 12154
#define IDS_IOS_DESKTOP_PRICE_TRACKING_PROMO_BUBBLE_DESCRIPTION_REMINDER 12155
#define IDS_IOS_DESKTOP_PRICE_TRACKING_PROMO_BUBBLE_TITLE_QR 12156
#define IDS_IOS_DESKTOP_PRICE_TRACKING_PROMO_BUBBLE_TITLE_REMINDER 12157
#define IDS_IOS_DESKTOP_TAB_GROUPS_PROMO_BUBBLE_DESCRIPTION_REMINDER 12158
#define IDS_IOS_DESKTOP_TAB_GROUPS_PROMO_BUBBLE_TITLE_QR 12159
#define IDS_IOS_DESKTOP_TAB_GROUPS_PROMO_BUBBLE_TITLE_REMINDER 12160
#define IDS_ACCESSIBLE_INCOGNITO_WINDOW_TITLE_FORMAT 12161
#define IDS_ACCESSIBLE_GUEST_WINDOW_TITLE_FORMAT 12162
#define IDS_ACCESSIBLE_WINDOW_TITLE_WITH_PROFILE_FORMAT 12163
#define IDS_ACCNAME_APP_UPGRADE_RECOMMENDED 12164
#define IDS_ACCNAME_FULLSCREEN 12165
#define IDS_ACCNAME_EXIT_FULLSCREEN 12166
#define IDS_ACCNAME_FULLSCREEN_DISABLED 12167
#define IDS_ACCNAME_HOME 12168
#define IDS_ACCNAME_RELOAD 12169
#define IDS_ACCNAME_SPLIT_TABS_TOOLBAR_BUTTON_PINNED 12170
#define IDS_ACCNAME_SPLIT_TABS_TOOLBAR_BUTTON_ENABLED 12171
#define IDS_ACCNAME_FIND 12172
#define IDS_ACCNAME_BOOKMARKS 12173
#define IDS_ACCNAME_SAVED_TAB_GROUPS 12174
#define IDS_ACCNAME_SAVED_TAB_GROUPS_CHEVRON 12175
#define IDS_SAVED_TAB_GROUPS_OVERFLOW_BUTTON_TOOLTIP 12176
#define IDS_TAB_GROUP_EDITOR_BUBBLE_FOOTER_SYNC_DISABLED 12177
#define IDS_TAB_GROUP_EDITOR_BUBBLE_FOOTER_SYNC_ENABLED 12178
#define IDS_TAB_GROUP_EDITOR_BUBBLE_FOOTER_LEARN_MORE 12179
#define IDS_ACCNAME_TAB_GROUPS_EVERYTHING 12180
#define IDS_TAB_GROUPS_EVERYTHING_BUTTON_TOOLTIP 12181
#define IDS_ACCNAME_BOOKMARKS_CHEVRON 12182
#define IDS_ACCNAME_BOOKMARK_BUTTON_ROLE_DESCRIPTION 12183
#define IDS_ACCNAME_BOOKMARK_FOLDER_BUTTON_ROLE_DESCRIPTION 12184
#define IDS_ACCNAME_SAVED_TAB_GROUP_BUTTON_ROLE_DESCRIPTION 12185
#define IDS_ACCNAME_CHROMELABS_BUTTON 12186
#define IDS_ACCNAME_LEFT_ALIGNED_SIDE_PANEL_BUTTON 12187
#define IDS_ACCESSIBLE_TEXT_CHROMELABS_BUTTON_ADDED_BY_ENTERPRISE_POLICY 12188
#define IDS_ACCESSIBLE_TEXT_CHROMELABS_BUTTON_REMOVED_BY_ENTERPRISE_POLICY 12189
#define IDS_ACCNAME_CHROMELABS_COMBOBOX 12190
#define IDS_ACCNAME_CHROMELABS_COMBOBOX_MAC 12191
#define IDS_ACCNAME_SEPARATOR 12192
#define IDS_ACCNAME_EXTENSIONS 12193
#define IDS_ACCNAME_NEWTAB 12194
#define IDS_ACCNAME_MINIMIZE 12195
#define IDS_ACCNAME_MAXIMIZE 12196
#define IDS_ACCNAME_RESTORE 12197
#define IDS_ACCNAME_CLOSE_TAB 12198
#define IDS_ACCNAME_ZOOM_SET_DEFAULT 12199
#define IDS_ACCNAME_TAB_SEARCH 12200
#define IDS_ACCNAME_TAB_SCROLL_LEADING 12201
#define IDS_ACCNAME_TAB_SCROLL_TRAILING 12202
#define IDS_ACCNAME_MUTE_TAB 12203
#define IDS_ACCNAME_WEB_APP_DETAILED_INSTALL_DIALOG_LEADING_SCROLL_BUTTON 12204
#define IDS_ACCNAME_WEB_APP_DETAILED_INSTALL_DIALOG_TRAILING_SCROLL_BUTTON 12205
#define IDS_TOOLBAR_BUTTON_UNPINNED 12206
#define IDS_TOOLBAR_BUTTON_PINNED 12207
#define IDS_ALLOWED_CLIPBOARD_TITLE 12208
#define IDS_BLOCKED_CLIPBOARD_TITLE 12209
#define IDS_ALLOWED_CLIPBOARD_MESSAGE 12210
#define IDS_ALLOWED_CLIPBOARD_BLOCK 12211
#define IDS_ALLOWED_CLIPBOARD_NO_ACTION 12212
#define IDS_BLOCKED_CLIPBOARD_MESSAGE 12213
#define IDS_BLOCKED_CLIPBOARD_UNBLOCK 12214
#define IDS_BLOCKED_CLIPBOARD_NO_ACTION 12215
#define IDS_BACK_NAVIGATION_MENU_PROMO 12216
#define IDS_BACK_NAVIGATION_MENU_PROMO_ACCESSIBLE_TEXT 12217
#define IDS_BOOKMARK_PROMO_0 12218
#define IDS_BOOKMARK_PROMO_1 12219
#define IDS_BOOKMARK_PROMO_2 12220
#define IDS_CHROME_TIP 12221
#define IDS_EXTENSIONS_MENU_IPH_ENTRY_POINT_TITLE 12222
#define IDS_EXTENSIONS_MENU_IPH_ENTRY_POINT_BODY 12223
#define IDS_EXTENSIONS_REQUEST_ACCESS_BUTTON_IPH_ENTRY_POINT_TITLE 12224
#define IDS_EXTENSIONS_REQUEST_ACCESS_BUTTON_IPH_ENTRY_POINT_BODY 12225
#define IDS_EXTENSIONS_ZERO_STATE_PROMO_IPH_TITLE 12226
#define IDS_EXTENSIONS_ZERO_STATE_PROMO_CUSTOM_ACTION_IPH_DESCRIPTION 12227
#define IDS_EXTENSIONS_ZERO_STATE_PROMO_CUSTOM_ACTION_IPH_ACCEPT 12228
#define IDS_GLOBAL_MEDIA_CONTROLS_PROMO 12229
#define IDS_INCOGNITOWINDOW_PROMO_0 12230
#define IDS_INCOGNITOWINDOW_PROMO_1 12231
#define IDS_INCOGNITOWINDOW_PROMO_2 12232
#define IDS_INCOGNITOWINDOW_PROMO_3 12233
#define IDS_NEWTAB_PROMO_0 12234
#define IDS_NEWTAB_PROMO_1 12235
#define IDS_NEWTAB_PROMO_2 12236
#define IDS_POWER_BOOKMARKS_SIDE_PANEL_PROMO 12237
#define IDS_POWER_BOOKMARKS_SIDE_PANEL_PROMO_PINNING 12238
#define IDS_READING_LIST_DISCOVERY_PROMO 12239
#define IDS_READING_LIST_DISCOVERY_PROMO_PINNING 12240
#define IDS_READING_LIST_ENTRY_POINT_PROMO 12241
#define IDS_DESKTOP_PWA_INSTALL_PROMO 12242
#define IDS_DESKTOP_PWA_LINK_CAPTURING_TEXT 12243
#define IDS_DESKTOP_PWA_LINK_CAPTURING_SETTINGS 12244
#define IDS_UPDATED_CONNECTION_SECURITY_INDICATORS_PROMO 12245
#define IDS_READING_LIST_IN_SIDE_PANEL_PROMO 12246
#define IDS_READING_LIST_IN_SIDE_PANEL_PROMO_PINNING 12247
#define IDS_TAB_GROUPS_UNNAMED_GROUP_TOOLTIP 12248
#define IDS_TAB_GROUPS_NAMED_GROUP_TOOLTIP 12249
#define IDS_SHARED_HIGHLIGHTING_PROMO 12250
#define IDS_TAB_AUDIO_MUTING_PROMO 12251
#define IDS_BATTERY_SAVER_MODE_PROMO_TITLE 12252
#define IDS_BATTERY_SAVER_MODE_PROMO_TEXT 12253
#define IDS_BATTERY_SAVER_MODE_PROMO_ACTION_TEXT 12254
#define IDS_MEMORY_SAVER_MODE_PROMO_TEXT 12255
#define IDS_MEMORY_SAVER_MODE_PROMO_ACTION_TEXT 12256
#define IDS_TAB_GROUPS_HOVER_CARD_HEADER 12257
#define IDS_TAB_GROUPS_UNNAMED_GROUP_HOVER_CARD_HEADER 12258
#define IDS_TAB_GROUPS_HOVER_CARD_FOOTER 12259
#define IDS_DISCARD_RING_PROMO_TITLE 12260
#define IDS_DISCARD_RING_PROMO_TEXT 12261
#define IDS_DISCARD_RING_PROMO_ACTION_TEXT 12262
#define IDS_DISCARD_RING_SETTINGS_TOAST 12263
#define IDS_TUTORIAL_GENERIC_SUCCESS_TITLE 12264
#define IDS_TUTORIAL_TAB_GROUP_ADD_TAB_TO_GROUP 12265
#define IDS_TUTORIAL_ADD_TAB_TO_GROUP_WITH_EXISTING_GROUP_IN_TAB_STRIP 12266
#define IDS_TUTORIAL_TAB_GROUP_EDIT_BUBBLE 12267
#define IDS_TUTORIAL_TAB_GROUP_DRAG_TAB 12268
#define IDS_TUTORIAL_TAB_GROUP_COLLAPSE 12269
#define IDS_TUTORIAL_TAB_GROUP_SUCCESS_DESCRIPTION 12270
#define IDS_TUTORIAL_CUSTOMIZE_CHROME_START_TUTORIAL_IPH 12271
#define IDS_TUTORIAL_CUSTOMIZE_CHROME_OPEN_SIDE_PANEL 12272
#define IDS_TUTORIAL_CUSTOMIZE_CHROME_CHANGE_THEME 12273
#define IDS_TUTORIAL_CUSTOMIZE_CHROME_SELECT_COLLECTION 12274
#define IDS_TUTORIAL_CUSTOMIZE_CHROME_APPLY_THEME 12275
#define IDS_TUTORIAL_CUSTOMIZE_CHROME_CLICK_BACK_ARROW 12276
#define IDS_TUTORIAL_CUSTOMIZE_CHROME_SUCCESS_BODY 12277
#define IDS_TUTORIAL_PASSWORD_MANAGER_CLICK_PASSWORDS_MENU 12278
#define IDS_TUTORIAL_PASSWORD_MANAGER_ADD_SHORTCUT 12279
#define IDS_TUTORIAL_PASSWORD_MANAGER_CLICK_INSTALL 12280
#define IDS_TUTORIAL_LENS_OVERLAY_CLICK_ADDRESS_BAR 12281
#define IDS_TUTORIAL_LENS_OVERLAY_HOMEWORK_INTRO_HEADER 12282
#define IDS_TUTORIAL_LENS_OVERLAY_HOMEWORK_INTRO_BODY 12283
#define IDS_TUTORIAL_LENS_OVERLAY_HOMEWORK_CLICK_LENS 12284
#define IDS_TUTORIAL_LENS_OVERLAY_CLICK_SEARCH_BOX 12285
#define IDS_BROWSER_HANGMONITOR_RENDERER_TITLE 12287
#define IDS_BROWSER_HANGMONITOR_RENDERER 12288
#define IDS_BROWSER_HANGMONITOR_IFRAME_TITLE 12289
#define IDS_BROWSER_HANGMONITOR_RENDERER_WAIT 12290
#define IDS_BROWSER_HANGMONITOR_RENDERER_END 12291
#define IDS_BROWSER_HANGMONITOR_PLUGIN_INFOBAR 12292
#define IDS_BROWSER_HANGMONITOR_PLUGIN_INFOBAR_KILLBUTTON 12293
#define IDS_PASSWORDS_AUTO_SIGNIN_TITLE 12294
#define IDS_PASSWORDS_AUTO_SIGNIN_DESCRIPTION 12295
#define IDS_PASSWORDS_VIA_FEDERATION 12296
#define IDS_CONFIRM_MESSAGEBOX_YES_BUTTON_LABEL 12297
#define IDS_CONFIRM_MESSAGEBOX_NO_BUTTON_LABEL 12298
#define IDS_NEAR_OOM_REDUCTION_MESSAGE_TITLE 12299
#define IDS_NEAR_OOM_REDUCTION_MESSAGE_DESCRIPTION 12300
#define IDS_TAILORED_SECURITY_UNCONSENTED_MESSAGE_TITLE 12301
#define IDS_TAILORED_SECURITY_UNCONSENTED_MESSAGE_ACCEPT 12302
#define IDS_TAILORED_SECURITY_DISPLAY_SOURCE 12303
#define IDS_PASSWORD_MANAGER_CANCEL_BUTTON 12304
#define IDS_PASSWORD_MANAGER_USERNAME_LABEL 12305
#define IDS_PASSWORD_MANAGER_PASSWORD_LABEL 12306
#define IDS_PASSWORD_MANAGER_UPDATED_BUBBLE_TITLE 12307
#define IDS_PASSWORD_MANAGER_MORE_TO_FIX_BODY_MESSAGE_GOOGLE_PASSWORD_MANAGER 12308
#define IDS_PASSWORD_MANAGER_SAFE_STATE_BODY_MESSAGE_GOOGLE_PASSWORD_MANAGER 12309
#define IDS_PASSWORD_MANAGER_CHECK_REMAINING_BUTTON 12310
#define IDS_PASSWORD_MANAGER_SAVE_BUTTON 12311
#define IDS_PASSWORD_MANAGER_MOVE_BUBBLE_CANCEL_BUTTON 12312
#define IDS_PASSWORD_MANAGER_UPDATE_BUTTON 12313
#define IDS_PASSWORD_MANAGER_SHORT_UPDATE_BUTTON 12314
#define IDS_PASSWORD_MANAGER_BUBBLE_BLOCKLIST_BUTTON 12315
#define IDS_PASSWORD_MANAGER_DESTINATION_DROPDOWN_ACCESSIBLE_NAME 12316
#define IDS_PASSWORD_MANAGER_DESTINATION_DROPDOWN_SAVE_TO_ACCOUNT 12317
#define IDS_PASSWORD_MANAGER_DESTINATION_DROPDOWN_SAVE_TO_DEVICE 12318
#define IDS_PASSWORD_MANAGER_IPH_TITLE_SAVE_TO_ACCOUNT 12319
#define IDS_PASSWORD_MANAGER_IPH_BODY_SAVE_TO_ACCOUNT 12320
#define IDS_PASSWORDS_SAVE_PRIMING_PROMO_BODY_TEMPLATE 12321
#define IDS_PASSWORDS_SAVE_PRIMING_PROMO_BODY_SIGNED_IN 12322
#define IDS_PASSWORDS_SAVE_PRIMING_PROMO_BODY_NOT_SIGNED_IN 12323
#define IDS_PASSWORDS_SAVE_PRIMING_PROMO_SCREENREADER 12324
#define IDS_PASSWORDS_SAVE_RECOVERY_PROMO_BODY 12325
#define IDS_PASSWORDS_SAVE_RECOVERY_PROMO_SCREENREADER 12326
#define IDS_PASSWORD_MANAGER_TOOLTIP_BLOCKED 12327
#define IDS_PASSWORD_MANAGER_IPH_MANAGEMENT_BUBBLE_AFTER_SAVE 12328
#define IDS_PASSWORD_MANAGER_IPH_MANAGEMENT_BUBBLE_AFTER_SAVE_SCREENREADER 12329
#define IDS_PASSWORD_MANAGER_IPH_SHARE_PASSWORD_BUTTON 12330
#define IDS_PASSWORD_MANAGER_IPH_SHARE_PASSWORD_BUTTON_SCREENREADER 12331
#define IDS_PASSWORD_MANAGER_IPH_ACCOUNT_STORAGE_TOGGLE 12332
#define IDS_PASSWORD_MANAGER_IPH_CREATE_SHORTCUT_TITLE 12333
#define IDS_PASSWORD_MANAGER_IPH_MANAGEMENT_BUBBLE_DURING_SIGNIN 12334
#define IDS_PASSWORD_MANAGER_IPH_MANAGEMENT_BUBBLE_DURING_SIGNIN_SCREENREADER 12335
#define IDS_PASSWORD_MANAGER_TOOLTIP_SAVE 12336
#define IDS_PASSWORD_MANAGER_TOOLTIP_MANAGE 12337
#define IDS_PASSWORD_MANAGER_TOOLTIP_MOVE 12338
#define IDS_PASSWORD_MANAGER_TOOLTIP_PROTECT 12339
#define IDS_PASSWORD_MANAGER_TOOLTIP_SHARED_NOTIFICATION 12340
#define IDS_PASSWORD_MANAGER_TOOLTIP_KEYCHAIN_ERROR 12341
#define IDS_IMPORT_FROM_FIREFOX 12363
#define IDS_IMPORT_FROM_ICEWEASEL 12364
#define IDS_IMPORT_FROM_SAFARI 12365
#define IDS_IMPORT_FROM_BOOKMARKS_HTML_FILE 12366
#define IDS_IMPORTER_LOCK_TITLE 12370
#define IDS_IMPORTER_LOCK_TEXT 12371
#define IDS_IMPORTER_LOCK_OK 12372
#define IDS_FEEDBACK_REPORT_APP_TITLE 12373
#define IDS_FEEDBACK_REPORT_PAGE_TITLE 12374
#define IDS_FEEDBACK_REPORT_PAGE_TITLE_SAD_TAB_FLOW 12375
#define IDS_FEEDBACK_MINIMIZE_BUTTON_LABEL 12376
#define IDS_FEEDBACK_CLOSE_BUTTON_LABEL 12377
#define IDS_FEEDBACK_FREE_TEXT_LABEL 12378
#define IDS_FEEDBACK_FREE_TEXT_AI_LABEL 12379
#define IDS_FEEDBACK_REPORT_URL_LABEL 12380
#define IDS_FEEDBACK_USER_EMAIL_LABEL 12381
#define IDS_FEEDBACK_ANONYMOUS_EMAIL_OPTION 12382
#define IDS_FEEDBACK_CONSENT_CHECKBOX_LABEL 12383
#define IDS_FEEDBACK_SCREENSHOT_LABEL 12384
#define IDS_FEEDBACK_SCREENSHOT_A11Y_TEXT 12385
#define IDS_FEEDBACK_OFFLINE_DIALOG_TITLE 12386
#define IDS_FEEDBACK_OFFLINE_DIALOG_TEXT 12387
#define IDS_FEEDBACK_INCLUDE_SYSTEM_INFORMATION_CHKBOX 12388
#define IDS_FEEDBACK_ATTACH_FILE_NOTE 12391
#define IDS_FEEDBACK_ATTACH_FILE_LABEL 12392
#define IDS_FEEDBACK_ATTACH_FILE_TO_BIG 12393
#define IDS_FEEDBACK_INCLUDE_AUTOFILL_METADATA_CHECKBOX 12394
#define IDS_FEEDBACK_IWLWIFI_DEBUG_DUMP_EXPLAINER 12395
#define IDS_FEEDBACK_LOG_ID_CHECKBOX_LABEL 12396
#define IDS_FEEDBACK_PRIVACY_NOTE 12397
#define IDS_FEEDBACK_NO_DESCRIPTION 12398
#define IDS_FEEDBACK_OFFENSIVE_CHECKBOX_LABEL 12399
#define IDS_FEEDBACK_SEND_REPORT 12400
#define IDS_FEEDBACK_AUTOFILL_METADATA_PAGE_TITLE 12401
#define IDS_FEEDBACK_SYSINFO_PAGE_TITLE 12402
#define IDS_FEEDBACK_SYSINFO_PAGE_LOADING 12403
#define IDS_FEEDBACK_ADDITIONAL_INFO_LABEL 12404
#define IDS_CLEAR_BROWSING_DATA_TITLE 12405
#define IDS_CLEAR_BROWSING_DATA_HISTORY_NOTICE 12406
#define IDS_CLEAR_BROWSING_DATA_HISTORY_NOTICE_TITLE 12407
#define IDS_CLEAR_BROWSING_DATA_HISTORY_NOTICE_OK 12408
#define IDS_CLEAR_BROWSING_DATA_PASSWORDS_NOTICE 12409
#define IDS_CLEAR_BROWSING_DATA_PASSWORDS_NOTICE_TITLE 12410
#define IDS_CLEAR_BROWSING_DATA_PASSWORDS_NOTICE_OK 12411
#define IDS_MEDIA_SELECTED_MIC_LABEL 12412
#define IDS_MEDIA_SELECTED_CAMERA_LABEL 12413
#define IDS_MEDIA_MENU_NO_DEVICE_TITLE 12414
#define IDS_MEDIA_CONTROLS_TITLE_PLACEHOLDER_INCOGNITO 12415
#define IDS_ZOOMLEVELS_CHROME_ERROR_PAGES_LABEL 12416
#define IDS_STORAGE_ACCESS_MANAGE_TEXT 12417
#define IDS_STORAGE_ACCESS_MANAGE_TOOLTIP 12418
#define IDS_REPORT_AN_ISSUE 12429
#define IDS_REENABLE_UPDATES 12430
#define IDS_PICTURE_IN_PICTURE_TITLE_TEXT 12431
#define IDS_PICTURE_IN_PICTURE_PAUSE_CONTROL_TEXT 12432
#define IDS_PICTURE_IN_PICTURE_PLAY_CONTROL_TEXT 12433
#define IDS_PICTURE_IN_PICTURE_REPLAY_CONTROL_TEXT 12434
#define IDS_PICTURE_IN_PICTURE_BACK_TO_TAB_CONTROL_TEXT 12435
#define IDS_PICTURE_IN_PICTURE_SKIP_AD_CONTROL_TEXT 12436
#define IDS_PICTURE_IN_PICTURE_LIVE_STATUS_TEXT 12437
#define IDS_PICTURE_IN_PICTURE_MUTE_MICROPHONE_TEXT 12438
#define IDS_PICTURE_IN_PICTURE_UNMUTE_MICROPHONE_TEXT 12439
#define IDS_PICTURE_IN_PICTURE_MUTE_TEXT 12440
#define IDS_PICTURE_IN_PICTURE_UNMUTE_TEXT 12441
#define IDS_PICTURE_IN_PICTURE_TURN_ON_CAMERA_TEXT 12442
#define IDS_PICTURE_IN_PICTURE_TURN_OFF_CAMERA_TEXT 12443
#define IDS_PICTURE_IN_PICTURE_HANG_UP_TEXT 12444
#define IDS_PICTURE_IN_PICTURE_CLOSE_CONTROL_TEXT 12445
#define IDS_PICTURE_IN_PICTURE_MINIMIZE_CONTROL_TEXT 12446
#define IDS_PICTURE_IN_PICTURE_LIVE_CAPTION_CONTROL_TEXT 12447
#define IDS_PICTURE_IN_PICTURE_REPLAY_10_TEXT 12448
#define IDS_PICTURE_IN_PICTURE_FORWARD_10_TEXT 12449
#define IDS_PICTURE_IN_PICTURE_RESIZE_HANDLE_TEXT 12450
#define IDS_PICTURE_IN_PICTURE_PLAY_PAUSE_CONTROL_ACCESSIBLE_TEXT 12451
#define IDS_PICTURE_IN_PICTURE_NEXT_TRACK_CONTROL_ACCESSIBLE_TEXT 12452
#define IDS_PICTURE_IN_PICTURE_PREVIOUS_TRACK_CONTROL_ACCESSIBLE_TEXT 12453
#define IDS_PICTURE_IN_PICTURE_PREVIOUS_SLIDE_CONTROL_ACCESSIBLE_TEXT 12454
#define IDS_PICTURE_IN_PICTURE_NEXT_SLIDE_CONTROL_ACCESSIBLE_TEXT 12455
#define IDS_AUTO_PICTURE_IN_PICTURE_DESCRIPTION 12456
#define IDS_LOAD_STATE_WAITING_FOR_SOCKET_SLOT 12457
#define IDS_LOAD_STATE_WAITING_FOR_DELEGATE 12458
#define IDS_LOAD_STATE_WAITING_FOR_DELEGATE_GENERIC 12459
#define IDS_LOAD_STATE_WAITING_FOR_CACHE 12460
#define IDS_LOAD_STATE_ESTABLISHING_PROXY_TUNNEL 12461
#define IDS_LOAD_STATE_RESOLVING_PROXY_FOR_URL 12462
#define IDS_LOAD_STATE_RESOLVING_HOST_IN_PAC_FILE 12463
#define IDS_LOAD_STATE_DOWNLOADING_PAC_FILE 12464
#define IDS_LOAD_STATE_RESOLVING_HOST 12465
#define IDS_LOAD_STATE_CONNECTING 12466
#define IDS_LOAD_STATE_SSL_HANDSHAKE 12467
#define IDS_LOAD_STATE_SENDING_REQUEST 12468
#define IDS_LOAD_STATE_SENDING_REQUEST_WITH_PROGRESS 12469
#define IDS_LOAD_STATE_WAITING_FOR_RESPONSE 12470
#define IDS_TAB_CXMENU_NEWTABTORIGHT 12471
#define IDS_TAB_CXMENU_NEWTABTOLEFT 12472
#define IDS_TAB_CXMENU_NEWTABBELOW 12473
#define IDS_TAB_CXMENU_RELOAD 12474
#define IDS_TAB_CXMENU_DUPLICATE 12475
#define IDS_TAB_CXMENU_CLOSETAB 12476
#define IDS_TAB_CXMENU_CLOSEOTHERTABS 12477
#define IDS_TAB_CXMENU_CLOSETABSTORIGHT 12478
#define IDS_TAB_CXMENU_CLOSETABSTOLEFT 12479
#define IDS_TAB_CXMENU_CLOSETABSBELOW 12480
#define IDS_TAB_CXMENU_CLOSEALLTABS 12481
#define IDS_TAB_CXMENU_FOCUS_THIS_TAB 12482
#define IDS_TAB_CXMENU_PIN_TAB 12483
#define IDS_TAB_CXMENU_UNPIN_TAB 12484
#define IDS_TAB_CXMENU_SOUND_MUTE_SITE 12485
#define IDS_TAB_CXMENU_SOUND_UNMUTE_SITE 12486
#define IDS_TAB_CXMENU_READ_LATER 12487
#define IDS_TAB_CXMENU_ADD_TAB_TO_GROUP 12488
#define IDS_TAB_CXMENU_ADD_TAB_TO_NEW_GROUP 12489
#define IDS_TAB_CXMENU_ADD_TAB_TO_NEW_SPLIT 12490
#define IDS_TAB_CXMENU_NEW_SPLIT_WITH_CURRENT 12491
#define IDS_TAB_CXMENU_ARRANGE_SPLIT 12492
#define IDS_TAB_CXMENU_SWAP_WITH_ACTIVE_SPLIT 12493
#define IDS_TAB_CXMENU_SUBMENU_NEW_GROUP 12494
#define IDS_TAB_CXMENU_REMOVE_TAB_FROM_GROUP 12495
#define IDS_TAB_CXMENU_PLACEHOLDER_GROUP_TITLE 12496
#define IDS_TAB_CXMENU_MOVE_TABS_TO_NEW_WINDOW 12497
#define IDS_TAB_CXMENU_MOVETOANOTHERWINDOW 12498
#define IDS_TAB_CXMENU_MOVETOANOTHERNEWWINDOW 12499
#define IDS_TAB_CXMENU_COMMERCE_PRODUCT_SPEC 12500
#define IDS_TAB_CXMENU_ADD_TAB_TO_NEW_COMPARISON_TABLE 12501
#define IDS_ACCNAME_SPLIT_TABS_RESIZE 12505
#define IDS_PIN_SPLIT_TAB_TOGGLE 12506
#define IDS_SPLIT_TAB_REVERSE_VIEWS 12507
#define IDS_SPLIT_TAB_CLOSE 12508
#define IDS_SPLIT_TAB_CLOSE_LEFT_VIEW 12509
#define IDS_SPLIT_TAB_CLOSE_RIGHT_VIEW 12510
#define IDS_SPLIT_TAB_CLOSE_TOP_VIEW 12511
#define IDS_SPLIT_TAB_CLOSE_BOTTOM_VIEW 12512
#define IDS_SPLIT_TAB_SEPARATE_VIEWS 12513
#define IDS_SPLIT_TAB_SWAP_LEFT_VIEW 12514
#define IDS_SPLIT_TAB_SWAP_RIGHT_VIEW 12515
#define IDS_SPLIT_TAB_SWAP_TOP_VIEW 12516
#define IDS_SPLIT_TAB_SWAP_BOTTOM_VIEW 12517
#define IDS_SPLIT_TAB_NEW_VERTICAL 12518
#define IDS_SPLIT_TAB_NEW_HORIZONTAL 12519
#define IDS_SPLIT_TAB_SEND_FEEDBACK 12520
#define IDS_TAB_SEARCH_TOOLBAR_BUTTON_PROMO_TITLE 12521
#define IDS_TAB_SEARCH_TOOLBAR_BUTTON_PROMO_BODY 12522
#define IDS_TAB_SEARCH_TOOLBAR_BUTTON_PROMO_ACTION 12523
#define IDS_TAB_SEARCH_COMBO_BUTTON_PROMO_TITLE 12524
#define IDS_TAB_SEARCH_COMBO_BUTTON_PROMO_BODY 12525
#define IDS_TAB_SEARCH_COMBO_BUTTON_PROMO_SCREENREADER 12526
#define IDS_TAB_SEARCH_SEARCH_TABS 12527
#define IDS_TAB_SEARCH_NO_RESULTS_FOUND 12528
#define IDS_TAB_SEARCH_CLOSE_TAB 12529
#define IDS_TAB_SEARCH_SUBMIT_FEEDBACK 12530
#define IDS_TAB_SEARCH_A11Y_TAB_CLOSED 12531
#define IDS_TAB_SEARCH_A11Y_SPLIT_VIEW_CLOSED 12532
#define IDS_TAB_SEARCH_A11Y_FOUND_TAB 12533
#define IDS_TAB_SEARCH_A11Y_FOUND_TABS 12534
#define IDS_TAB_SEARCH_A11Y_FOUND_TAB_FOR 12535
#define IDS_TAB_SEARCH_A11Y_FOUND_TABS_FOR 12536
#define IDS_TAB_SEARCH_A11Y_OPEN_TAB 12537
#define IDS_TAB_SEARCH_A11Y_RECENTLY_CLOSED_TAB 12538
#define IDS_TAB_SEARCH_A11Y_RECENTLY_CLOSED_TAB_GROUP 12539
#define IDS_TAB_SEARCH_MEDIA_TABS 12540
#define IDS_TAB_SEARCH_OPEN_TABS 12541
#define IDS_TAB_SEARCH_RECENTLY_CLOSED 12542
#define IDS_TAB_SEARCH_RECENTLY_CLOSED_TABS 12543
#define IDS_TAB_SEARCH_ONE_TAB 12544
#define IDS_TAB_SEARCH_TAB_COUNT 12545
#define IDS_TAB_SEARCH_EXPAND_RECENTLY_CLOSED_ITEMS 12546
#define IDS_TAB_SEARCH_EXPAND_RECENTLY_CLOSED 12547
#define IDS_TAB_SEARCH_COLLAPSE_RECENTLY_CLOSED 12548
#define IDS_TAB_SEARCH_TAB_NAME 12549
#define IDS_TAB_SEARCH_MENU 12550
#define IDS_BROWSER_WINDOW_TITLE_MENU_ENTRY 12551
#define IDS_TABS_TITLE_CXMENU 12552
#define IDS_SAVED_TAB_GROUP_TABS_COUNT 12553
#define IDS_CREATE_NEW_TAB_GROUP 12554
#define IDS_GROUP_UNGROUPED_TABS 12555
#define IDS_SAVED_TAB_GROUPS_MENU 12556
#define IDS_OPEN_GROUP_IN_BROWSER_MENU 12557
#define IDS_NEW_TAB_IN_GROUP 12558
#define IDS_NEW_TAB_IN_GROUP_NO_GROUPS 12559
#define IDS_NEW_TAB_IN_GROUP_NO_NAME 12560
#define IDS_NEW_TAB_GROUP 12561
#define IDS_SAVED_TAB_GROUPS_V2_INTRO_IPH_APP_MENU_SYNCED_BODY 12562
#define IDS_SAVED_TAB_GROUPS_V2_INTRO_DEFAULT_BODY_A11Y 12563
#define IDS_SAVED_TAB_GROUPS_V2_INTRO_IPH_APP_MENU_NOT_SYNCED_BODY 12564
#define IDS_SAVED_TAB_GROUPS_V2_INTRO_IPH_BOOKMARKS_BAR_SYNCED_BODY 12565
#define IDS_SAVED_TAB_GROUPS_V2_INTRO_IPH_BOOKMARKS_BAR_NOT_SYNCED_BODY 12566
#define IDS_RESUMPTION_RAIL_IPH_TITLE 12567
#define IDS_RESUMPTION_RAIL_IPH_BODY 12568
#define IDS_RESUMPTION_RAIL_IPH_BODY_ONLY_GEMINI 12569
#define IDS_RESUMPTION_RAIL_IPH_BODY_ONLY_AI_MODE 12570
#define IDS_RESUMPTION_RAIL_IPH_BODY_NO_THREADS 12571
#define IDS_RESUMPTION_RAIL_THREADS_IPH_TITLE 12572
#define IDS_RESUMPTION_RAIL_THREADS_IPH_TITLE_ONLY_GEMINI 12573
#define IDS_RESUMPTION_RAIL_THREADS_IPH_TITLE_ONLY_AI_MODE 12574
#define IDS_RESUMPTION_RAIL_THREADS_IPH_BODY 12575
#define IDS_RESUMPTION_RAIL_THREADS_IPH_BODY_ONLY_GEMINI 12576
#define IDS_RESUMPTION_RAIL_THREADS_IPH_BODY_ONLY_AI_MODE 12577
#define IDS_RESUMPTION_RAIL_IPH_OK_BUTTON 12578
#define IDS_SAVED_TAB_GROUPS_UNSUPPORTED_ERROR 12579
#define IDS_TAB_GROUP_DELETION_DIALOG_MISSING_EMAIL 12580
#define IDS_TAB_GROUP_DELETION_DIALOG_BODY_SYNCED_DELETE 12581
#define IDS_TAB_GROUP_DELETION_DIALOG_BODY_SYNCED_UNGROUP 12582
#define IDS_TAB_GROUP_DELETION_DIALOG_BODY_NOT_SYNCED_DELETE 12583
#define IDS_TAB_GROUP_DELETION_DIALOG_BODY_NOT_SYNCED_UNGROUP 12584
#define IDS_TAB_GROUP_DELETION_DIALOG_DONT_ASK 12585
#define IDS_TAB_GROUP_DELETION_DIALOG_TITLE_DELETE 12586
#define IDS_TAB_GROUP_DELETION_DIALOG_TITLE_UNGROUP 12587
#define IDS_TAB_GROUP_DELETION_DIALOG_TITLE_CLOSE_TAB_AND_DELETE 12588
#define IDS_TAB_GROUP_DELETION_DIALOG_TITLE_REMOVE_TAB_AND_DELETE 12589
#define IDS_TAB_GROUP_DELETION_DIALOG_OK_TEXT_DELETE 12590
#define IDS_TAB_GROUP_DELETION_DIALOG_OK_TEXT_UNGROUP 12591
#define IDS_TAB_GROUP_HEADER_CXMENU_TAB_GROUP_TITLE_ACCESSIBLE_NAME 12592
#define IDS_TAB_GROUP_HEADER_CXMENU_TAB_GROUP_FACE_PILE_ACCESSIBLE_NAME 12593
#define IDS_TAB_GROUP_HEADER_CXMENU_NEW_TAB_IN_GROUP 12594
#define IDS_TAB_GROUP_HEADER_CXMENU_UNGROUP 12595
#define IDS_TAB_GROUP_HEADER_CXMENU_FOCUS_GROUP 12596
#define IDS_TAB_GROUP_HEADER_CXMENU_UNFOCUS_GROUP 12597
#define IDS_TAB_GROUP_HEADER_CXMENU_SHARE_GROUP 12598
#define IDS_TAB_GROUP_HEADER_CXMENU_RECENT_ACTIVITY 12599
#define IDS_TAB_GROUP_HEADER_CXMENU_MANAGE_GROUP 12600
#define IDS_TAB_GROUP_HEADER_CXMENU_CLOSE_GROUP 12601
#define IDS_TAB_GROUP_HEADER_CXMENU_HIDE_GROUP 12602
#define IDS_TAB_GROUP_HEADER_CXMENU_DELETE_GROUP 12603
#define IDS_TAB_GROUP_HEADER_CXMENU_MOVE_GROUP_TO_NEW_WINDOW 12604
#define IDS_TAB_GROUP_HEADER_CXMENU_OPEN_GROUP_IN_NEW_WINDOW 12605
#define IDS_TAB_GROUP_HEADER_CXMENU_PIN_GROUP 12606
#define IDS_TAB_GROUP_HEADER_CXMENU_UNPIN_GROUP 12607
#define IDS_TAB_GROUP_HEADER_CXMENU_CONVERT_GROUP_TO_BOOKMARK_FOLDER 12608
#define IDS_TAB_GROUP_HEADER_CXMENU_SEND_FEEDBACK 12609
#define IDS_TAB_GROUP_HEADER_BUBBLE_TITLE_PLACEHOLDER 12610
#define IDS_DATA_SHARING_SHARE_GROUP_TITLE 12611
#define IDS_DATA_SHARING_COPY_LINK 12612
#define IDS_DATA_SHARING_COPY_LINK_SUCCESS 12613
#define IDS_DATA_SHARING_MANAGE_GROUP_TITLE 12614
#define IDS_DATA_SHARING_MANAGE_STOP_SHARING_OPTION 12615
#define IDS_DATA_SHARING_MANAGE_ACTIVITY_LOG_OPTION 12616
#define IDS_DATA_SHARING_OWNER_STOP_SHARING_DIALOG_TITLE 12617
#define IDS_DATA_SHARING_OWNER_REMOVE_MEMBER_DIALOG_TITLE 12618
#define IDS_DATA_SHARING_OWNER_REMOVE_MEMBER_DIALOG_CONFIRM 12619
#define IDS_DATA_SHARING_OWNER_DELETE_DIALOG_TITLE 12620
#define IDS_DATA_SHARING_OWNER_DELETE_DIALOG_CONFIRM 12621
#define IDS_DATA_SHARING_PREVIEW_DIALOG_TITLE 12622
#define IDS_DATA_SHARING_PREVIEW_DIALOG_TITLE_ZERO 12623
#define IDS_DATA_SHARING_PREVIEW_DIALOG_TITLE_SINGULAR 12624
#define IDS_DATA_SHARING_PREVIEW_DIALOG_TITLE_PLURAL 12625
#define IDS_DATA_SHARING_PREVIEW_DIALOG_CONFIRM 12626
#define IDS_DATA_SHARING_PREVIEW_DIALOG_DETAILS_TITLE 12627
#define IDS_DATA_SHARING_LINK_FAILED_TITLE 12628
#define IDS_DATA_SHARING_LEAVE_DIALOG_TITLE 12629
#define IDS_DATA_SHARING_BLOCK_LEAVE_DIALOG_TITLE 12630
#define IDS_DATA_SHARING_BLOCK_LEAVE_DIALOG_CONFIRM 12631
#define IDS_DATA_SHARING_NEED_SIGN_IN 12632
#define IDS_DATA_SHARING_NEED_SYNC 12633
#define IDS_DATA_SHARING_NEED_SYNC_TAB_GROUPS 12634
#define IDS_DATA_SHARING_NEED_VERIFY_ACCOUNT 12635
#define IDS_DATA_SHARING_NEED_VERIFY_ACCOUNT_BUTTON 12636
#define IDS_DATA_SHARING_GOT_IT 12637
#define IDS_DATA_SHARING_SOMETHING_WRONG 12638
#define IDS_DATA_SHARING_ORGANIZATION_DISALLOW_SIGN_IN 12639
#define IDS_DATA_SHARING_ORGANIZATION_LIMIT_SAVE 12640
#define IDS_DATA_SHARING_SIGNED_OUT 12641
#define IDS_DATA_SHARING_GROUP_FULL 12642
#define IDS_DATA_SHARING_DELETE_LAST_TAB_TITLE 12643
#define IDS_DATA_SHARING_DELETE_LAST_TAB_CONFIRM 12644
#define IDS_DATA_SHARING_OWNER_DELETE_LAST_TAB_CANCEL 12645
#define IDS_DATA_SHARING_MEMBER_DELETE_LAST_TAB_CANCEL 12646
#define IDS_DATA_SHARING_RECENT_ACTIVITY_TITLE 12647
#define IDS_DATA_SHARING_RECENT_ACTIVITY_LATEST_UPDATE_TAB 12648
#define IDS_DATA_SHARING_RECENT_ACTIVITY_LATEST_UPDATE_GROUP 12649
#define IDS_DATA_SHARING_SHARE_GROUP_BODY 12650
#define IDS_DATA_SHARING_SHARE_GROUP_SHARE_AS 12651
#define IDS_DATA_SHARING_MANAGE_SHARE_WISELY 12652
#define IDS_DATA_SHARING_OWNER_STOP_SHARING_DIALOG_BODY 12653
#define IDS_DATA_SHARING_OWNER_REMOVE_MEMBER_DIALOG_BODY 12654
#define IDS_DATA_SHARING_OWNER_DELETE_DIALOG_BODY 12655
#define IDS_DATA_SHARING_OWNER_DELETE_DIALOG_BODY_NO_GROUP_TITLE 12656
#define IDS_DATA_SHARING_PREVIEW_DIALOG_BODY 12657
#define IDS_DATA_SHARING_PREVIEW_DIALOG_DETAILS_TABS_IN_GROUP 12658
#define IDS_DATA_SHARING_LINK_FAILED_BODY 12659
#define IDS_DATA_SHARING_JOIN_GROUP_JOIN_AS 12660
#define IDS_DATA_SHARING_GET_GROUP_PREVIEW_ARIA_LABEL 12661
#define IDS_DATA_SHARING_PREVIEW_A11Y_NAME 12662
#define IDS_DATA_SHARING_LEAVE_DIALOG_BODY 12663
#define IDS_DATA_SHARING_LEAVE_DIALOG_BODY_NO_GROUP_TITLE 12664
#define IDS_DATA_SHARING_LEAVE_DIALOG_CONFIRM 12665
#define IDS_DATA_SHARING_BLOCK_DIALOG_TITLE 12666
#define IDS_DATA_SHARING_BLOCK_DIALOG_BODY 12667
#define IDS_DATA_SHARING_BLOCK_LEAVE_DIALOG_BODY 12668
#define IDS_DATA_SHARING_BLOCK_LEAVE_LEARN_MORE 12669
#define IDS_DATA_SHARING_NEED_SIGN_IN_BODY 12670
#define IDS_DATA_SHARING_NEED_SIGN_IN_CONTINUE_BUTTON 12671
#define IDS_DATA_SHARING_NEED_SYNC_BODY 12672
#define IDS_DATA_SHARING_NEED_SYNC_TAB_GROUPS_BODY 12673
#define IDS_DATA_SHARING_NEED_VERIFY_ACCOUNT_BODY 12674
#define IDS_SYNC_HISTORY_TITLE 12675
#define IDS_DATA_SHARING_NEED_SIGN_IN_AND_SYNC_HISTORY_BODY 12676
#define IDS_DATA_SHARING_NEED_SYNC_HISTORY_BODY 12677
#define IDS_DATA_SHARING_NEED_VERIFY_ACCOUNT_SYNC_HISTORY_BODY 12678
#define IDS_SYNC_HISTORY_BUTTON 12679
#define IDS_SYNC_HISTORY_FOOTER 12680
#define IDS_DATA_SHARING_SOMETHING_WRONG_BODY 12681
#define IDS_DATA_SHARING_SHARE_ERROR_BODY 12682
#define IDS_DATA_SHARING_JOIN_WARNING 12683
#define IDS_DATA_SHARING_ORGANIZATION_DISALLOW_SIGN_IN_BODY 12684
#define IDS_DATA_SHARING_ORGANIZATION_LIMIT_SAVE_BODY 12685
#define IDS_DATA_SHARING_SIGNED_OUT_BODY 12686
#define IDS_DATA_SHARING_SETTINGS 12687
#define IDS_DATA_SHARING_GROUP_FULL_BODY 12688
#define IDS_DATA_SHARING_OWNER_CANNOT_SHARE 12689
#define IDS_DATA_SHARING_DELETE_LAST_DIALOG_HEADER 12690
#define IDS_DATA_SHARING_OWNER_DELETE_LAST_TAB_BODY 12691
#define IDS_DATA_SHARING_OWNER_DELETE_LAST_TAB_BODY_SINGULAR 12692
#define IDS_DATA_SHARING_OWNER_DELETE_LAST_TAB_BODY_2 12693
#define IDS_DATA_SHARING_MEMBER_DELETE_LAST_TAB_BODY 12694
#define IDS_DATA_SHARING_MEMBER_DELETE_LAST_TAB_BODY_SINGULAR 12695
#define IDS_DATA_SHARING_RECENT_ACTIVITY_NO_UPDATES 12696
#define IDS_DATA_SHARING_PAGE_ACTION_ADDED_NEW_TAB 12697
#define IDS_DATA_SHARING_PAGE_ACTION_CHANGED_TAB 12698
#define IDS_ANCHORED_MESSAGE_EXPAND_BUTTON_TOOLTIP 12699
#define IDS_ANCHORED_MESSAGE_COLLAPSE_BUTTON_TOOLTIP 12700
#define IDS_DATA_SHARING_USER_ED_FIRST_TAB_CHANGE 12701
#define IDS_DATA_SHARING_SHARED_GROUPS_FEEDBACK 12702
#define IDS_DATA_SHARING_SHARED_GROUPS_FEEDBACK_IPH 12703
#define IDS_DATA_SHARING_SHARED_GROUPS_FEEDBACK_IPH_SCREENREADER 12704
#define IDS_DATA_SHARING_LEARN_ABOUT_SHARED_GROUPS 12705
#define IDS_DATA_SHARING_PEOPLE_WITH_ACCESS 12706
#define IDS_DATA_SHARING_LINK_EDIT_ACCESS 12707
#define IDS_DATA_SHARING_LINK_JOIN_TOGGLE 12708
#define IDS_DATA_SHARING_CANCEL 12709
#define IDS_DATA_SHARING_CLOSE 12710
#define IDS_DATA_SHARING_BACK 12711
#define IDS_DATA_SHARING_BLOCK 12712
#define IDS_DATA_SHARING_REMOVE 12713
#define IDS_DATA_SHARING_LEAVE_GROUP 12714
#define IDS_DATA_SHARING_KEEP_GROUP 12715
#define IDS_DATA_SHARING_DELETE_GROUP 12716
#define IDS_DATA_SHARING_LOADING 12717
#define IDS_DATA_SHARING_COPY_LINK_FAILED 12718
#define IDS_DATA_SHARING_YOU 12719
#define IDS_DATA_SHARING_OWNER 12720
#define IDS_DATA_SHARING_MEMBER_COUNT_SINGULAR 12721
#define IDS_DATA_SHARING_MEMBER_COUNT_PLURAL 12722
#define IDS_DATA_SHARING_TAB_COUNT_SINGULAR 12723
#define IDS_DATA_SHARING_TAB_COUNT_PLURAL 12724
#define IDS_DATA_SHARING_MORE_OPTIONS 12725
#define IDS_DATA_SHARING_MORE_OPTIONS_DESCRIPTION 12726
#define IDS_APP_MENU_RELOAD 12727
#define IDS_APP_MENU_NEW_WEB_PAGE 12728
#define IDS_APP_MENU_COPY_LINK 12729
#define IDS_APP_MENU_CREATE_QR_CODE 12730
#define IDS_APP_MENU_BUTTON_UPDATE 12731
#define IDS_APP_MENU_BUTTON_UPDATE_ALT1 12732
#define IDS_APP_MENU_BUTTON_UPDATE_ALT2 12733
#define IDS_APP_MENU_BUTTON_UPDATE_ALT3 12734
#define IDS_APP_MENU_BUTTON_ERROR 12735
#define IDS_APP_MENU_BUTTON_ACTION_REQUIRED 12736
#define IDS_MEDIA_SCREEN_CAPTURE_CONFIRMATION_TITLE 12737
#define IDS_MEDIA_SCREEN_CAPTURE_CONFIRMATION_TEXT 12738
#define IDS_MEDIA_SCREEN_AND_AUDIO_CAPTURE_CONFIRMATION_TEXT 12739
#define IDS_MEDIA_SCREEN_CAPTURE_NOTIFICATION_TEXT 12740
#define IDS_MEDIA_SCREEN_CAPTURE_WITH_AUDIO_NOTIFICATION_TEXT 12741
#define IDS_MEDIA_WINDOW_CAPTURE_NOTIFICATION_TEXT 12742
#define IDS_MEDIA_WINDOW_CAPTURE_WITH_AUDIO_NOTIFICATION_TEXT 12743
#define IDS_MEDIA_TAB_CAPTURE_NOTIFICATION_TEXT 12744
#define IDS_MEDIA_TAB_CAPTURE_WITH_AUDIO_NOTIFICATION_TEXT 12745
#define IDS_MEDIA_SCREEN_CAPTURE_NOTIFICATION_HIDE 12746
#define IDS_MEDIA_SCREEN_CAPTURE_NOTIFICATION_SOURCE 12747
#define IDS_MEDIA_SCREEN_CAPTURE_NOTIFICATION_STOP 12748
#define IDS_PLATFORM_KEYS_SELECT_CERT_DIALOG_TEXT 12749
#define IDS_UNSAFE_FRAME_MESSAGE 12750
#define IDS_CLIENT_CERT_DIALOG_TITLE 12751
#define IDS_CLIENT_CERT_DIALOG_TEXT 12752
#define IDS_CRYPTO_MODULE_AUTH_DIALOG_TITLE 12753
#define IDS_CRYPTO_MODULE_AUTH_DIALOG_TEXT_CERT_ENROLLMENT 12754
#define IDS_CRYPTO_MODULE_AUTH_DIALOG_TEXT_CLIENT_AUTH 12755
#define IDS_CRYPTO_MODULE_AUTH_DIALOG_TEXT_LIST_CERTS 12756
#define IDS_CRYPTO_MODULE_AUTH_DIALOG_TEXT_CERT_IMPORT 12757
#define IDS_CRYPTO_MODULE_AUTH_DIALOG_TEXT_CERT_EXPORT 12758
#define IDS_CRYPTO_MODULE_AUTH_DIALOG_PASSWORD_FIELD 12759
#define IDS_CRYPTO_MODULE_AUTH_DIALOG_OK_BUTTON_LABEL 12760
#define IDS_FR_ENABLE_LOGGING 12762
#define IDS_CRASHED_TAB_FEEDBACK_MESSAGE 12766
#define IDS_CRASHED_TAB_FEEDBACK_LINK 12767
#define IDS_KILLED_TAB_FEEDBACK_MESSAGE 12769
#define IDS_DEFAULT_BROWSER_CHANGED_TITLE 12771
#define IDS_DEFAULT_BROWSER_CHANGED_YES_BUTTON 12772
#define IDS_DEFAULT_BROWSER_CHANGED_NO_THANKS_BUTTON 12773
#define IDS_DEFAULT_BROWSER_MODAL_CONFIRM_BUTTON 12774
#define IDS_DEFAULT_BROWSER_MODAL_CANCEL_BUTTON 12775
#define IDS_DEFAULT_BROWSER_BUBBLE_DIALOG_STEPS_COMBINED 12776
#define IDS_DEFAULT_BROWSER_BUBBLE_DIALOG_OPEN_SETTINGS_LABEL 12777
#define IDS_DEFAULT_BROWSER_BUBBLE_DIALOG_SET_DEFAULT_LABEL 12778
#define IDS_DEFAULT_BROWSER_BUBBLE_DIALOG_YES_LABEL 12779
#define IDS_DEFAULT_BROWSER_BUBBLE_DIALOG_SET_LATER_LABEL 12780
#define IDS_RELAUNCH_BUTTON 12781
#define IDS_HELP_MANAGED_BY_YOUR_PARENT 12782
#define IDS_CONTROLLED_SETTING_POLICY 12785
#define IDS_TOOLBAR_INFORM_SET_HOME_PAGE 12786
#define IDS_MANAGE_EXTENSIONS_SETTING_WINDOWS_TITLE 12787
#define IDS_CONTROLLED_SETTING_EXTENSION 12788
#define IDS_CONTROLLED_SETTING_EXTENSION_WITHOUT_NAME 12789
#define IDS_CONTROLLED_SETTING_RECOMMENDED 12790
#define IDS_CONTROLLED_SETTING_HAS_RECOMMENDATION 12791
#define IDS_CONTROLLED_SETTING_CHILD_RESTRICTION 12792
#define IDS_CONTROLLED_SETTING_PARENT 12793
#define IDS_EXTENSIONS_INSTALL_LOCATION_UNKNOWN 12794
#define IDS_EXTENSIONS_INSTALL_LOCATION_3RD_PARTY 12795
#define IDS_EXTENSIONS_INSTALL_LOCATION_ENTERPRISE 12796
#define IDS_EXTENSIONS_INSTALL_LOCATION_SHARED_MODULE 12797
#define IDS_EXTENSIONS_BLOCKLISTED_MALWARE 12798
#define IDS_EXTENSIONS_BLOCKLISTED_SECURITY_VULNERABILITY 12799
#define IDS_EXTENSIONS_BLOCKLISTED_CWS_POLICY_VIOLATION 12800
#define IDS_EXTENSIONS_BLOCKLISTED_POTENTIALLY_UNWANTED 12801
#define IDS_EXTENSIONS_SAFE_BROWSING_CRX_ALLOWLIST_WARNING 12802
#define IDS_EXTENSIONS_SAFE_BROWSING_CRX_ALLOWLIST_WARNING_LEARN_MORE 12803
#define IDS_EXTENSIONS_ACCESS_REQUESTS_LEARN_MORE 12804
#define IDS_UPLOAD_MOVE_TO_ACCOUNT_DIALOG_TITLE 12805
#define IDS_EXTENSIONS_MOVE_TO_ACCOUNT_DIALOG_SUBTITLE 12806
#define IDS_BOOKMARK_UPLOAD_MOVE_TO_ACCOUNT_DIALOG_SUBTITLE 12807
#define IDS_EXTENSIONS_MOVE_TO_ACCOUNT_DIALOG_OK_BUTTON_LABEL 12808
#define IDS_BOOKMARK_MOVE_TO_ACCOUNT_ICON_TOOLTIP 12809
#define IDS_RESET_PROFILE_SETTINGS_EXPLANATION 12810
#define IDS_TRIGGERED_RESET_PROFILE_SETTINGS_TITLE 12811
#define IDS_TRIGGERED_RESET_PROFILE_SETTINGS_EXPLANATION 12812
#define IDS_TRIGGERED_RESET_PROFILE_SETTINGS_DEFAULT_TOOL_NAME 12813
#define IDS_RESET_PROFILE_SETTINGS_LOCALE 12814
#define IDS_RESET_PROFILE_SETTINGS_STARTUP_URLS 12815
#define IDS_RESET_PROFILE_SETTINGS_STARTUP_TYPE 12816
#define IDS_RESET_PROFILE_SETTINGS_HOMEPAGE 12817
#define IDS_RESET_PROFILE_SETTINGS_HOMEPAGE_IS_NTP 12818
#define IDS_RESET_PROFILE_SETTINGS_YES 12819
#define IDS_RESET_PROFILE_SETTINGS_NO 12820
#define IDS_RESET_PROFILE_SETTINGS_SHOW_HOME_BUTTON 12821
#define IDS_RESET_PROFILE_SETTINGS_DSE 12822
#define IDS_RESET_PROFILE_SETTINGS_EXTENSIONS 12823
#define IDS_RESET_PROFILE_SETTINGS_SHORTCUTS 12824
#define IDS_RESET_PROFILE_SETTINGS_PROCESSING_SHORTCUTS 12825
#define IDS_AUTOFILL_DIALOG_PLACEHOLDER_EXPIRY_MONTH 12827
#define IDS_AUTOFILL_DIALOG_PLACEHOLDER_EXPIRY_YEAR 12828
#define IDS_AUTOFILL_FROM_GOOGLE_ACCOUNT 12829
#define IDS_SHOW_WINDOW_DECORATIONS 12830
#define IDS_SEND_TAB_TO_SELF_SEND_BUTTON_LABEL 12831
#define IDS_OMNIBOX_TOOLTIP_SEND_TAB_TO_SELF 12832
#define IDS_TOOLBAR_BUTTON_SEND_TAB_TO_SELF_FROM_DEVICE 12833
#define IDS_TOOLBAR_BUTTON_SEND_TAB_TO_SELF_TITLE 12834
#define IDS_SEND_TAB_TO_SELF_DEVICE_PICKER_TITLE 12835
#define IDS_SEND_TAB_TO_SELF_DEVICE_PICKER_SUBTITLE 12836
#define IDS_TOOLBAR_BUTTON_SEND_TAB_TO_SELF_BUTTON_LABEL 12837
#define IDS_TOOLBAR_BUTTON_SEND_TAB_TO_SELF_BUTTON_A11Y_NAME 12838
#define IDS_TOOLBAR_BUTTON_SEND_TAB_TO_SELF_BUTTON_HINT_TEXT 12839
#define IDS_MENU_SEND_TAB_TO_SELF 12840
#define IDS_SEND_TAB_TO_SELF 12841
#define IDS_SEND_TAB_TO_SELF_DEVICE_LABEL 12842
#define IDS_SEND_TAB_TO_SELF_MANAGE_DEVICES 12843
#define IDS_SEND_TAB_TO_SELF_SENDING_ANNOUNCE 12844
#define IDS_SEND_TAB_TO_SELF_MANAGE_DEVICES_LINK 12845
#define IDS_SEND_TAB_TO_SELF_SIGN_IN_PROMO_LABEL 12846
#define IDS_SEND_TAB_TO_SELF_NO_TARGET_DEVICE_LABEL 12847
#define IDS_SEND_TAB_TO_SELF_SIGN_IN_PROMO_BODY 12848
#define IDS_CONTEXT_MENU_GENERATE_QR_CODE_PAGE 12849
#define IDS_CONTEXT_MENU_GENERATE_QR_CODE_IMAGE 12850
#define IDS_CONTEXT_MENU_GENERATE_QR_CODE_LINK 12851
#define IDS_OMNIBOX_QRCODE_GENERATOR_ICON_LABEL 12852
#define IDS_OMNIBOX_QRCODE_GENERATOR_ICON_TOOLTIP 12853
#define IDS_BROWSER_SHARING_QR_CODE_DIALOG_TITLE 12854
#define IDS_SHARING_HUB_GENERATE_QR_CODE_LABEL 12855
#define IDS_BROWSER_SHARING_QR_CODE_DIALOG_URL_TEXTFIELD_ACCESSIBLE_NAME 12856
#define IDS_BROWSER_SHARING_QR_CODE_DIALOG_TOOLTIP 12857
#define IDS_BROWSER_SHARING_QR_CODE_DIALOG_DOWNLOAD_BUTTON_LABEL 12858
#define IDS_BROWSER_SHARING_QR_CODE_DIALOG_COPY_BUTTON_LABEL 12859
#define IDS_BROWSER_SHARING_QR_CODE_DIALOG_ERROR_TOO_LONG 12860
#define IDS_BROWSER_SHARING_QR_CODE_DIALOG_ERROR_UNKNOWN 12861
#define IDS_SHARING_HUB_TITLE 12862
#define IDS_SHARING_HUB_TOOLTIP 12863
#define IDS_SHARING_HUB_SCREENSHOT_LABEL 12864
#define IDS_SHARING_HUB_COPY_LINK_LABEL 12865
#define IDS_SHARING_HUB_SAVE_PAGE_LABEL 12866
#define IDS_SHARING_HUB_MEDIA_ROUTER_LABEL 12867
#define IDS_SHARING_HUB_SHARE_LABEL 12868
#define IDS_BROWSER_SHARING_SCREENSHOT_POST_CAPTURE_TITLE 12869
#define IDS_BROWSER_SHARING_SCREENSHOT_DIALOG_DOWNLOAD_BUTTON_LABEL 12870
#define IDS_CONTEXT_MENU_PASTE_FROM_CLIPBOARD 12871
#define IDS_SHARING_REMOTE_COPY_NOTIFICATION_TITLE_TEXT_CONTENT_UNKNOWN_DEVICE 12872
#define IDS_SHARING_REMOTE_COPY_NOTIFICATION_TITLE_TEXT_CONTENT 12873
#define IDS_SHARING_REMOTE_COPY_NOTIFICATION_TITLE_IMAGE_CONTENT_UNKNOWN_DEVICE 12874
#define IDS_SHARING_REMOTE_COPY_NOTIFICATION_TITLE_IMAGE_CONTENT 12875
#define IDS_SHARING_REMOTE_COPY_NOTIFICATION_DESCRIPTION 12876
#define IDS_CONTENT_CONTEXT_SHARING_CLICK_TO_CALL_MULTIPLE_DEVICES 12877
#define IDS_CONTENT_CONTEXT_SHARING_CLICK_TO_CALL_SINGLE_DEVICE 12878
#define IDS_CONTENT_CONTEXT_SHARING_SHARED_CLIPBOARD_MULTIPLE_DEVICES 12879
#define IDS_CONTENT_CONTEXT_SHARING_SHARED_CLIPBOARD_SINGLE_DEVICE 12880
#define IDS_CONTENT_CONTEXT_SHARING_SHARED_CLIPBOARD_NOTIFICATION_TITLE_UNKNOWN_DEVICE 12881
#define IDS_CONTENT_CONTEXT_SHARING_SHARED_CLIPBOARD_NOTIFICATION_TITLE 12882
#define IDS_CONTENT_CONTEXT_SHARING_SHARED_CLIPBOARD_NOTIFICATION_DESCRIPTION 12883
#define IDS_OMNIBOX_TOOLTIP_SHARED_CLIPBOARD 12884
#define IDS_COLLECTED_COOKIES_DIALOG_TITLE 12885
#define IDS_COLLECTED_COOKIES_ALLOWED_COOKIES_LABEL 12886
#define IDS_COLLECTED_COOKIES_BLOCKED_COOKIES_LABEL 12887
#define IDS_COLLECTED_COOKIES_BLOCKED_THIRD_PARTY_BLOCKING_ENABLED 12888
#define IDS_COLLECTED_COOKIES_ALLOW_BUTTON 12889
#define IDS_COLLECTED_COOKIES_SESSION_ONLY_BUTTON 12890
#define IDS_COLLECTED_COOKIES_BLOCK_BUTTON 12891
#define IDS_COLLECTED_COOKIES_ALLOW_RULE_CREATED 12892
#define IDS_COLLECTED_COOKIES_BLOCK_RULE_CREATED 12893
#define IDS_COLLECTED_COOKIES_SESSION_RULE_CREATED 12894
#define IDS_COLLECTED_COOKIES_ALLOWED_COOKIES_TAB_LABEL 12895
#define IDS_COLLECTED_COOKIES_BLOCKED_COOKIES_TAB_LABEL 12896
#define IDS_COLLECTED_COOKIES_ALLOWED_AUX_TEXT 12897
#define IDS_COLLECTED_COOKIES_BLOCKED_AUX_TEXT 12898
#define IDS_COLLECTED_COOKIES_CLEAR_ON_EXIT_AUX_TEXT 12899
#define IDS_COLLECTED_COOKIES_INFOBAR_MESSAGE 12900
#define IDS_COLLECTED_COOKIES_INFOBAR_BUTTON 12901
#define IDS_COLLECTED_COOKIES_PARTITIONED_COOKIE 12902
#define IDS_PAGE_SPECIFIC_SITE_DATA_DIALOG_TITLE 12903
#define IDS_PAGE_SPECIFIC_SITE_DATA_DIALOG_SUBTITLE 12904
#define IDS_PAGE_SPECIFIC_SITE_DATA_DIALOG_SETTINGS_LINK 12905
#define IDS_PAGE_SPECIFIC_SITE_DATA_DIALOG_FIRST_PARTY_TITLE 12906
#define IDS_PAGE_SPECIFIC_SITE_DATA_DIALOG_FIRST_PARTY_SUBTITLE 12907
#define IDS_PAGE_SPECIFIC_SITE_DATA_DIALOG_THIRD_PARTY_TITLE 12908
#define IDS_PAGE_SPECIFIC_SITE_DATA_DIALOG_THIRD_PARTY_SUBTITLE 12909
#define IDS_PAGE_SPECIFIC_SITE_DATA_DIALOG_EMPTY_STATE_LABEL 12910
#define IDS_PAGE_SPECIFIC_SITE_DATA_DIALOG_RELATED_APPS_TITLE 12911
#define IDS_PAGE_SPECIFIC_SITE_DATA_DIALOG_RELATED_APPS_SUBTITLE 12912
#define IDS_PAGE_SPECIFIC_SITE_DATA_DIALOG_RELATED_APPS_SETTINGS_LINK_TOOLTIP 12913
#define IDS_PAGE_SPECIFIC_SITE_DATA_DIALOG_ALLOW_MENU_ITEM 12914
#define IDS_PAGE_SPECIFIC_SITE_DATA_DIALOG_ALLOW_THIRD_PARTY_MENU_ITEM 12915
#define IDS_PAGE_SPECIFIC_SITE_DATA_DIALOG_BLOCK_MENU_ITEM 12916
#define IDS_PAGE_SPECIFIC_SITE_DATA_DIALOG_SESSION_ONLY_MENU_ITEM 12917
#define IDS_PAGE_SPECIFIC_SITE_DATA_DIALOG_ALLOWED_STATE_SUBTITLE 12918
#define IDS_PAGE_SPECIFIC_SITE_DATA_DIALOG_BLOCKED_STATE_SUBTITLE 12919
#define IDS_PAGE_SPECIFIC_SITE_DATA_DIALOG_SESSION_ONLY_STATE_SUBTITLE 12920
#define IDS_PAGE_SPECIFIC_SITE_DATA_DIALOG_PARTITIONED_STATE_SUBTITLE 12921
#define IDS_PAGE_SPECIFIC_SITE_DATA_DIALOG_PARTITIONED_SESSION_ONLY_STATE_SUBTITLE 12922
#define IDS_PAGE_SPECIFIC_SITE_DATA_DIALOG_CONTEXT_MENU_TOOLTIP 12923
#define IDS_PAGE_SPECIFIC_SITE_DATA_DIALOG_DELETE_BUTTON_TOOLTIP 12924
#define IDS_ACCNAME_INFOBAR_CONTAINER 12925
#define IDS_ACCNAME_INFOBAR 12926
#define IDS_ONE_CLICK_BUBBLE_UNDO 12927
#define IDS_ENTERPRISE_SIGNIN_CANCEL 12928
#define IDS_ENTERPRISE_SIGNIN_CREATE_NEW_PROFILE 12929
#define IDS_ENTERPRISE_SIGNIN_CREATE_NEW_WORK_PROFILE 12930
#define IDS_ENTERPRISE_SIGNIN_CONTINUE 12931
#define IDS_ENTERPRISE_SIGNIN_ALERT 12932
#define IDS_PROFILE_WILL_BE_DELETED_DIALOG_TITLE 12933
#define IDS_PROFILE_WILL_BE_DELETED_DIALOG_DESCRIPTION 12934
#define IDS_MANAGED_WITH_HYPERLINK 12937
#define IDS_MANAGED_BY_WITH_HYPERLINK 12938
#define IDS_PROFILE_MANAGED_BY_WITH_HYPERLINK 12939
#define IDS_BROWSER_MANAGED_AND_PROFILE_MANAGED_BY_WITH_HYPERLINK 12940
#define IDS_BROWSER_AND_PROFILE_DIFFERENT_MANAGED_BY_WITH_HYPERLINK 12941
#define IDS_BROWSER_AND_PROFILE_SAME_MANAGED_BY_WITH_HYPERLINK 12942
#define IDS_MANAGED_BY_PARENT_WITH_HYPERLINK 12943
#define IDS_BROWSER_MANAGED_AND_PROFILE_MANAGED_BY_TOOLTIP 12944
#define IDS_BROWSER_AND_PROFILE_DIFFERENT_MANAGED_BY_TOOLTIP 12945
#define IDS_COOKIES_COOKIE_EXPIRES_SESSION 12946
#define IDS_COOKIES_COOKIE_SENDFOR_ANY 12947
#define IDS_COOKIES_COOKIE_SENDFOR_SECURE 12948
#define IDS_COOKIES_COOKIE_SENDFOR_SAME_SITE 12949
#define IDS_COOKIES_COOKIE_SENDFOR_SECURE_SAME_SITE 12950
#define IDS_COOKIES_COOKIE_ACCESSIBLE_TO_SCRIPT_YES 12951
#define IDS_COOKIES_COOKIE_ACCESSIBLE_TO_SCRIPT_NO 12952
#define IDS_COOKIES_COOKIES 12953
#define IDS_COOKIES_WEB_DATABASES 12954
#define IDS_COOKIES_LOCAL_STORAGE 12955
#define IDS_COOKIES_SESSION_STORAGE 12956
#define IDS_COOKIES_INDEXED_DBS 12957
#define IDS_COOKIES_FILE_SYSTEMS 12958
#define IDS_COOKIES_FILE_SYSTEM_USAGE_NONE 12959
#define IDS_COOKIES_SERVICE_WORKER 12960
#define IDS_COOKIES_SERVICE_WORKERS 12961
#define IDS_COOKIES_SHARED_WORKERS 12962
#define IDS_COOKIES_CACHE_STORAGE 12963
#define IDS_APP_DEFAULT_PAGE_NAME 12964
#define IDS_APP_LAUNCHER_TAB_TITLE 12965
#define IDS_NEW_TAB_GUEST_SESSION_HEADING 12966
#define IDS_NEW_TAB_GUEST_SESSION_DESCRIPTION 12967
#define IDS_NEW_TAB_GUEST_SESSION_LEARN_MORE_ACCESSIBILITY_TEXT 12968
#define IDS_NEW_TAB_ENTERPRISE_GUEST_SESSION_LEARN_MORE_ACCESSIBILITY_TEXT 12969
#define IDS_NEW_TAB_TILE_GRID_ACCESSIBLE_DESCRIPTION 12970
#define IDS_NEW_TAB_APP_INSTALL_HINT_LABEL 12971
#define IDS_NEW_TAB_MOST_VISITED 12972
#define IDS_NEW_TAB_RESTORE_THUMBNAILS_SHORT_LINK 12973
#define IDS_NEW_TAB_ATTRIBUTION_INTRO 12974
#define IDS_NEW_TAB_THUMBNAIL_REMOVED_NOTIFICATION 12975
#define IDS_NEW_TAB_REMOVE_THUMBNAIL_TOOLTIP 12976
#define IDS_NEW_TAB_PAGE_SWITCHER_CHANGE_TITLE 12977
#define IDS_NEW_TAB_PAGE_SWITCHER_SAME_TITLE 12978
#define IDS_NEW_TAB_VOICE_AUDIO_ERROR 12979
#define IDS_NEW_TAB_VOICE_CLOSE_TOOLTIP 12980
#define IDS_NEW_TAB_VOICE_DETAILS 12981
#define IDS_NEW_TAB_VOICE_LANGUAGE_ERROR 12982
#define IDS_NEW_TAB_VOICE_LISTENING 12983
#define IDS_NEW_TAB_VOICE_NETWORK_ERROR 12984
#define IDS_NEW_TAB_VOICE_NO_TRANSLATION 12985
#define IDS_NEW_TAB_VOICE_NO_VOICE 12986
#define IDS_NEW_TAB_VOICE_OTHER_ERROR 12987
#define IDS_NEW_TAB_VOICE_PERMISSION_ERROR 12988
#define IDS_NEW_TAB_VOICE_LEARN_MORE_ACCESSIBILITY_LABEL 12989
#define IDS_NEW_TAB_VOICE_READY 12990
#define IDS_NEW_TAB_VOICE_TRY_AGAIN 12991
#define IDS_NEW_TAB_VOICE_WAITING 12992
#define IDS_NEW_TAB_VOICE_SEARCH_CLOSED 12993
#define IDS_FUSEBOX_VOICE_SEARCH_TRANSCRIBING 12994
#define IDS_FUSEBOX_VOICE_SEARCH_STOP_TITLE 12995
#define IDS_NEW_TAB_APP_OPTIONS 12996
#define IDS_NEW_TAB_APP_DETAILS 12997
#define IDS_NEW_TAB_APP_CREATE_SHORTCUT 12998
#define IDS_NEW_TAB_APP_INSTALL_LOCALLY 12999
#define IDS_APP_CONTEXT_MENU_SHOW_INFO 13000
#define IDS_APP_CONTEXT_MENU_OPEN_PINNED 13001
#define IDS_APP_CONTEXT_MENU_OPEN_REGULAR 13002
#define IDS_APP_CONTEXT_MENU_OPEN_WINDOW 13003
#define IDS_APP_CONTEXT_MENU_OPEN_FULLSCREEN 13004
#define IDS_APP_CONTEXT_MENU_OPEN_MAXIMIZED 13005
#define IDS_APP_CONTEXT_MENU_OPEN_TAB 13006
#define IDS_APP_CONTEXT_MENU_RUN_ON_OS_LOGIN 13011
#define IDS_APPS_PAGE_DEPRECATED_APP_TITLE 13014
#define IDS_DEVICE_SIGNALS_CONSENT_DIALOG_TITLE 13015
#define IDS_DEVICE_SIGNALS_CONSENT_DIALOG_PROCEED_BUTTON 13016
#define IDS_DEVICE_SIGNALS_CONSENT_DIALOG_CANCEL_BUTTON 13017
#define IDS_DEVICE_SIGNALS_CONSENT_DIALOG_BODY_TEXT 13018
#define IDS_DEVICE_SIGNALS_CONSENT_DIALOG_DEFAULT_BODY_TEXT 13019
#define IDS_SYNC_CONFIRMATION_SETTINGS_INFO 13020
#define IDS_SYNC_CONFIRMATION_CONFIRM_BUTTON_LABEL 13021
#define IDS_SYNC_CONFIRMATION_SETTINGS_BUTTON_LABEL 13022
#define IDS_SYNC_CONFIRMATION_REFRESHED_SETTINGS_BUTTON_LABEL 13023
#define IDS_SYNC_CONFIRMATION_TANGIBLE_SYNC_TITLE 13024
#define IDS_SYNC_CONFIRMATION_TANGIBLE_SYNC_INFO_TITLE 13025
#define IDS_SYNC_CONFIRMATION_TANGIBLE_SYNC_INFO_TITLE_SIGNIN_INTERCEPT_V2 13026
#define IDS_SYNC_CONFIRMATION_TANGIBLE_SYNC_BOOKMARKS 13027
#define IDS_SYNC_CONFIRMATION_TANGIBLE_SYNC_READING_LIST 13028
#define IDS_SYNC_CONFIRMATION_TANGIBLE_SYNC_AUTOFILL 13029
#define IDS_SYNC_CONFIRMATION_TANGIBLE_SYNC_EXTENSIONS 13030
#define IDS_SYNC_CONFIRMATION_TANGIBLE_SYNC_HISTORY_AND_MORE 13031
#define IDS_SYNC_CONFIRMATION_TANGIBLE_SYNC_INFO_DESC 13032
#define IDS_SYNC_CONFIRMATION_WELCOME_TITLE_SIGNIN_INTERCEPT 13033
#define IDS_SYNC_CONFIRMATION_TURN_ON_SYNC_BUTTON_LABEL 13034
#define IDS_SYNC_DISABLED_CONFIRMATION_CHROME_SYNC_TITLE 13035
#define IDS_SYNC_DISABLED_CONFIRMATION_DETAILS 13036
#define IDS_SYNC_DISABLED_CONFIRMATION_CONFIRM_BUTTON_LABEL 13037
#define IDS_SYNC_DISABLED_CONFIRMATION_CONFIRM_BUTTON_MANAGED_ACCOUNT_SIGNOUT_DISALLOWED_LABEL 13038
#define IDS_SYNC_DISABLED_CONFIRMATION_UNDO_BUTTON_LABEL 13039
#define IDS_SYNC_LOADING_CONFIRMATION_TITLE 13040
#define IDS_SIGNIN_DICE_WEB_INTERCEPT_BUBBLE_NEW_PROFILE_BUTTON_LABEL 13041
#define IDS_SIGNIN_DICE_WEB_INTERCEPT_AVATAR_BUTTON_SEPARATE_BROWSING_TEXT 13042
#define IDS_SIGNIN_DICE_WEB_INTERCEPT_AVATAR_BUTTON_SWITCH_PROFILE_TEXT 13043
#define IDS_SIGNIN_DICE_WEB_INTERCEPT_BUBBLE_CREATE_PROFILE_BUTTON_LABEL 13044
#define IDS_SIGNIN_DICE_WEB_INTERCEPT_BUBBLE_STAY_HERE_BUTTON_LABEL 13045
#define IDS_SIGNIN_DICE_WEB_INTERCEPT_BUBBLE_CHROME_SIGNIN_TITLE_SUPERVISED 13046
#define IDS_SIGNIN_DICE_WEB_INTERCEPT_BUBBLE_CANCEL_BUTTON_LABEL 13047
#define IDS_SIGNIN_DICE_WEB_INTERCEPT_BUBBLE_CANCEL_SWITCH_BUTTON_LABEL 13048
#define IDS_SIGNIN_DICE_WEB_INTERCEPT_CREATE_BUBBLE_DESC_SUPERVISED 13049
#define IDS_SIGNIN_DICE_WEB_INTERCEPT_SWITCH_BUBBLE_CONTINUE_BUTTON_LABEL 13050
#define IDS_SIGNIN_DICE_WEB_INTERCEPT_BUBBLE_CHROME_SIGNIN_ACCEPT_TEXT 13051
#define IDS_SIGNIN_DICE_WEB_INTERCEPT_ENTERPRISE_BUBBLE_DESC_MANAGED_BY_TOKEN 13052
#define IDS_SIGNIN_DICE_WEB_INTERCEPT_MANAGED_DISCLAIMER 13053
#define IDS_SIGNIN_DICE_WEB_INTERCEPT_MANAGED_BY_DISCLAIMER 13054
#define IDS_SIGNIN_DICE_WEB_INTERCEPT_BUBBLE_CHROME_SIGNIN_IPH_TITLE_SIGNIN_REMINDER 13055
#define IDS_SIGNIN_DICE_WEB_INTERCEPT_BUBBLE_CHROME_SIGNIN_IPH_OK_BUTTON 13056
#define IDS_CHROME_SIGNOUT_CONFIRMATION_PROMPT_TITLE 13057
#define IDS_CHROME_SIGNOUT_CONFIRMATION_PROMPT_DELETE_AND_SIGNOUT_BUTTON 13058
#define IDS_CLOSE_WARNING_FOR_CLEAR_COOKIES_ON_EXIT_TEXT_LINK 13059
#define IDS_CLOSE_WARNING_FOR_CLEAR_COOKIES_ON_EXIT_CLOSE 13060
#define IDS_SYNC_PASSPHRASE_DIALOG_FOOTER_LINK 13061
#define IDS_SYNC_PASSPHRASE_DIALOG_TITLE 13062
#define IDS_SYNC_PASSPHRASE_DIALOG_OK_BUTTON 13063
#define IDS_SIGNIN_DICE_WEB_INTERCEPT_ENTERPRISE_PROFILE_NAME 13064
#define IDS_SIGNIN_ERROR_TITLE 13065
#define IDS_SIGNIN_ERROR_EMAIL_TITLE 13066
#define IDS_SIGNIN_ERROR_CLOSE_BUTTON_LABEL 13067
#define IDS_SIGNIN_ERROR_OK_BUTTON_LABEL 13068
#define IDS_SIGNIN_ERROR_SWITCH_BUTTON_LABEL 13069
#define IDS_SIGNIN_ACCESSIBLE_CLOSE_BUTTON 13070
#define IDS_SIGNIN_ACCESSIBLE_BACK_BUTTON 13071
#define IDS_SIGNIN_EMAIL_CONFIRMATION_CREATE_PROFILE_RADIO_BUTTON_TITLE 13072
#define IDS_SIGNIN_EMAIL_CONFIRMATION_CREATE_PROFILE_RADIO_BUTTON_SUBTITLE 13073
#define IDS_SIGNIN_EMAIL_CONFIRMATION_START_SYNC_RADIO_BUTTON_TITLE 13074
#define IDS_SIGNIN_EMAIL_CONFIRMATION_START_SYNC_RADIO_BUTTON_SUBTITLE 13075
#define IDS_SIGNIN_EMAIL_CONFIRMATION_CLOSE_BUTTON_LABEL 13076
#define IDS_SIGNIN_EMAIL_CONFIRMATION_CONFIRM_BUTTON_LABEL 13077
#define IDS_PLUGIN_CRASHED_PROMPT 13078
#define IDS_RELOAD_PAGE_WITH_PLUGIN 13079
#define IDS_EXTERNAL_PROTOCOL_TITLE 13080
#define IDS_EXTERNAL_PROTOCOL_MESSAGE_WITH_INITIATING_ORIGIN 13081
#define IDS_EXTERNAL_PROTOCOL_MESSAGE 13082
#define IDS_EXTERNAL_PROTOCOL_OK_BUTTON_TEXT 13083
#define IDS_EXTERNAL_PROTOCOL_CANCEL_BUTTON_TEXT 13084
#define IDS_EXTERNAL_PROTOCOL_CHECKBOX_PER_ORIGIN_TEXT 13085
#define IDS_DIRECTORY_LISTING_HEADER 13087
#define IDS_DIRECTORY_LISTING_PARENT 13088
#define IDS_DIRECTORY_LISTING_NAME 13089
#define IDS_DIRECTORY_LISTING_SIZE 13090
#define IDS_DIRECTORY_LISTING_DATE_MODIFIED 13091
#define IDS_SAVE_PAGE_DESC_HTML_ONLY 13092
#define IDS_SAVE_PAGE_DESC_SINGLE_FILE 13093
#define IDS_SAVE_PAGE_DESC_COMPLETE 13094
#define IDS_PROFILE_ERROR_DIALOG_TITLE 13095
#define IDS_COULDNT_OPEN_PROFILE_ERROR 13096
#define IDS_OPEN_PROFILE_DATA_LOSS 13097
#define IDS_PROFILE_ERROR_DIALOG_CHECKBOX 13098
#define IDS_PROFILE_ERROR_FEEDBACK_DESCRIPTION 13099
#define IDS_REFUSE_TO_RUN_AS_ROOT 13100
#define IDS_REFUSE_TO_RUN_AS_ROOT_2 13101
#define IDS_CANT_WRITE_USER_DIRECTORY_TITLE 13103
#define IDS_RECENT_TABS_MENU 13104
#define IDS_RECENTLY_CLOSED_WINDOW 13105
#define IDS_RECENTLY_CLOSED_GROUP 13106
#define IDS_RECENTLY_CLOSED_GROUP_UNNAMED 13107
#define IDS_RECENTLY_CLOSED_SPLIT 13108
#define IDS_RECENT_TABS_NO_DEVICE_TABS 13109
#define IDS_RECENT_TABS_LOGIN_FOR_DEVICE_TABS 13110
#define IDS_RECENT_TABS_SEE_DEVICE_TABS 13111
#define IDS_HISTORY_MENU 13112
#define IDS_DEFAULT_DOWNLOAD_FILENAME 13113
#define IDS_DEFAULT_BROWSER_INFOBAR_OK_BUTTON_LABEL 13115
#define IDS_MINIMIZE_WINDOW_MENU 13123
#define IDS_MAXIMIZE_WINDOW_MENU 13124
#define IDS_RESTORE_WINDOW_MENU 13125
#define IDS_CLOSE_WINDOW_MENU 13126
#define IDS_SHOW_WINDOW_DECORATIONS_MENU 13127
#define IDS_USED_EXISTING_BROWSER 13128
#define IDS_DECLINE_RECOVERY 13129
#define IDS_SYNC_ACCOUNT_SYNCING 13130
#define IDS_SYNC_ACCOUNT_SYNCING_CUSTOM_DATA_TYPES 13131
#define IDS_SIGNIN_ERROR_DISPLAY_SOURCE 13132
#define IDS_SIGNIN_ERROR_BUBBLE_VIEW_TITLE 13133
#define IDS_SYNC_ERROR_BUBBLE_VIEW_TITLE 13134
#define IDS_SYNC_ERROR_BUBBLE_VIEW_TITLE_2 13135
#define IDS_SYNC_ERROR_PASSWORDS_BUBBLE_VIEW_TITLE 13136
#define IDS_SYNC_ERROR_USER_MENU_TITLE 13137
#define IDS_SYNC_ERROR_PASSWORDS_USER_MENU_ERROR_DESCRIPTION 13138
#define IDS_SYNC_ERROR_RECOVERABILITY_DEGRADED_FOR_PASSWORDS_USER_MENU_ERROR_DESCRIPTION 13139
#define IDS_SYNC_NEEDS_VERIFICATION_BUBBLE_VIEW_TITLE 13140
#define IDS_SYNC_ERROR_USER_MENU_CONFIRM_SYNC_SETTINGS_BUTTON 13141
#define IDS_SYNC_ERROR_USER_MENU_SIGNOUT_BUTTON 13142
#define IDS_SYNC_ERROR_USER_MENU_CONFIRM_SYNC_SETTINGS_BUTTON_MAYBE_TITLE_CASE 13143
#define IDS_SYNC_ERROR_USER_MENU_SIGNOUT_BUTTON_MAYBE_TITLE_CASE 13144
#define IDS_SYNC_EMPTY_STRING 13154
#define IDS_SYNC_OVERVIEW 13155
#define IDS_SYNC_START_SYNC_BUTTON_LABEL 13156
#define IDS_SIGNED_IN_WITH_SYNC_DISABLED_BY_POLICY 13157
#define IDS_SYNC_SETTINGS_NOT_CONFIRMED 13159
#define IDS_SYNC_SETUP_IN_PROGRESS 13160
#define IDS_SYNC_STATUS_UNRECOVERABLE_ERROR 13161
#define IDS_SYNC_STATUS_UNRECOVERABLE_ERROR_NEEDS_SIGNOUT 13162
#define IDS_SYNC_STATUS_NEEDS_PASSWORD 13163
#define IDS_SYNC_STATUS_NEEDS_PASSWORD_BUTTON 13164
#define IDS_SYNC_STATUS_NEEDS_KEYS_BUTTON 13165
#define IDS_SYNC_STATUS_NEEDS_PASSWORD_BUTTON_MAYBE_TITLE_CASE 13166
#define IDS_SYNC_STATUS_NEEDS_KEYS_BUTTON_MAYBE_TITLE_CASE 13167
#define IDS_SYNC_RELOGIN_ERROR 13168
#define IDS_SYNC_RELOGIN_BUTTON 13169
#define IDS_SYNC_RELOGIN_BUTTON_MAYBE_TITLE_CASE 13170
#define IDS_SYNC_ADVANCED_OPTIONS 13172
#define IDS_SYNC_FULL_ENCRYPTION_BODY_CUSTOM 13174
#define IDS_SYNC_FULL_ENCRYPTION_BODY_CUSTOM_WITH_DATE 13175
#define IDS_SYNC_FULL_ACCOUNT_DATA_ENCRYPTION_BODY_CUSTOM 13176
#define IDS_SYNC_FULL_ACCOUNT_DATA_ENCRYPTION_BODY_CUSTOM_WITH_DATE 13177
#define IDS_PARTIAL_TRANSLATE_BUBBLE_WAITING_TITLE 13178
#define IDS_TRANSLATE_BUBBLE_BEFORE_TRANSLATE_TITLE 13179
#define IDS_TRANSLATE_BUBBLE_TRANSLATED_TITLE 13180
#define IDS_PARTIAL_TRANSLATE_BUBBLE_COULD_NOT_TRANSLATE_TITLE 13181
#define IDS_TRANSLATE_BUBBLE_COULD_NOT_TRANSLATE_TITLE 13182
#define IDS_TRANSLATE_BUBBLE_SOURCE_LANG_COMBOBOX_ACCNAME 13183
#define IDS_TRANSLATE_BUBBLE_TARGET_LANG_COMBOBOX_ACCNAME 13184
#define IDS_TRANSLATE_BUBBLE_TRANSLATION_COMPLETE_ANNOUNCEMENT 13185
#define IDS_TRANSLATE_BUBBLE_ADVANCED_BUTTON 13186
#define IDS_TRANSLATE_BUBBLE_ADVANCED_MENU_BUTTON 13187
#define IDS_TRANSLATE_BUBBLE_CHANGE_TARGET_LANGUAGE 13188
#define IDS_TRANSLATE_BUBBLE_CHANGE_SOURCE_LANGUAGE 13189
#define IDS_TRANSLATE_BUBBLE_OPEN_LANGUAGE_SETTINGS 13190
#define IDS_PARTIAL_TRANSLATE_BUBBLE_CHANGE_SOURCE_LANGUAGE 13191
#define IDS_TRANSLATE_BUBBLE_ACCEPT 13192
#define IDS_TRANSLATE_BUBBLE_DENY 13193
#define IDS_TRANSLATE_BUBBLE_ALWAYS_TRANSLATE_LANG 13194
#define IDS_TRANSLATE_BUBBLE_NEVER_TRANSLATE_LANG 13195
#define IDS_TRANSLATE_BUBBLE_NEVER_TRANSLATE_SITE 13196
#define IDS_TRANSLATE_BUBBLE_TRANSLATING 13197
#define IDS_TRANSLATE_BUBBLE_REVERT 13198
#define IDS_TRANSLATE_BUBBLE_TRY_AGAIN 13199
#define IDS_TRANSLATE_BUBBLE_ALWAYS 13200
#define IDS_TRANSLATE_BUBBLE_OPTIONS_MENU_BUTTON 13201
#define IDS_TRANSLATE_BUBBLE_PAGE_LANGUAGE 13202
#define IDS_TRANSLATE_BUBBLE_TRANSLATION_LANGUAGE 13203
#define IDS_TRANSLATE_BUBBLE_ADVANCED_TARGET 13204
#define IDS_TRANSLATE_BUBBLE_ADVANCED_SOURCE 13205
#define IDS_PARTIAL_TRANSLATE_BUBBLE_ADVANCED_SOURCE 13206
#define IDS_PARTIAL_TRANSLATE_BUBBLE_TRANSLATE_FULL_PAGE 13207
#define IDS_TRANSLATE_BUBBLE_RESET 13208
#define IDS_NOTIFICATION_BUTTON_SETTINGS 13209
#define IDS_NOTIFICATION_BUTTON_CLOSE 13210
#define IDS_NOTIFICATION_BUTTON_MORE 13211
#define IDS_NOTIFICATION_REPLY_PLACEHOLDER 13212
#define IDS_NOTIFICATION_MUTED_MESSAGE 13213
#define IDS_NOTIFICATION_MUTED_TITLE 13214
#define IDS_NOTIFICATION_MUTED_ACTION_SHOW 13215
#define IDS_NOTIFICATION_MUTED_ACTION_SNOOZE 13216
#define IDS_FILE_MENU_LINUX 13337
#define IDS_EDIT_MENU_LINUX 13338
#define IDS_VIEW_MENU_LINUX 13339
#define IDS_HISTORY_MENU_LINUX 13340
#define IDS_TOOLS_MENU_LINUX 13341
#define IDS_HELP_MENU_LINUX 13342
#define IDS_REOPEN_CLOSED_TABS_LINUX 13343
#define IDS_OPEN_FILE_LINUX 13344
#define IDS_OPEN_LOCATION_LINUX 13345
#define IDS_CLOSE_WINDOW_LINUX 13346
#define IDS_CLOSE_TAB_LINUX 13347
#define IDS_STOP_MENU_LINUX 13348
#define IDS_RELOAD_MENU_LINUX 13349
#define IDS_TEXT_DEFAULT_LINUX 13350
#define IDS_TEXT_BIGGER_LINUX 13351
#define IDS_TEXT_SMALLER_LINUX 13352
#define IDS_HISTORY_HOME_LINUX 13353
#define IDS_HISTORY_BACK_LINUX 13354
#define IDS_HISTORY_FORWARD_LINUX 13355
#define IDS_HISTORY_VISITED_LINUX 13356
#define IDS_HISTORY_CLOSED_LINUX 13357
#define IDS_HISTORY_CLOSED_RESTORE_WINDOW_LINUX 13358
#define IDS_ALLOWED_GEOLOCATION_TITLE 13359
#define IDS_BLOCKED_GEOLOCATION_TITLE 13360
#define IDS_ALLOWED_GEOLOCATION_MESSAGE 13361
#define IDS_ALLOWED_GEOLOCATION_BLOCK 13362
#define IDS_ALLOWED_GEOLOCATION_NO_ACTION 13363
#define IDS_BLOCKED_GEOLOCATION_MESSAGE 13364
#define IDS_BLOCKED_GEOLOCATION_UNBLOCK 13365
#define IDS_BLOCKED_GEOLOCATION_NO_ACTION 13366
#define IDS_GEOLOCATION 13367
#define IDS_GEOLOCATION_WILL_ASK_AGAIN 13368
#define IDS_GEOLOCATION_TURNED_OFF 13370
#define IDS_ALLOWED_MIDI_SYSEX_TITLE 13371
#define IDS_BLOCKED_MIDI_SYSEX_TITLE 13372
#define IDS_ALLOWED_MIDI_SYSEX_MESSAGE 13373
#define IDS_ALLOWED_MIDI_SYSEX_BLOCK 13374
#define IDS_ALLOWED_MIDI_SYSEX_NO_ACTION 13375
#define IDS_BLOCKED_MIDI_SYSEX_MESSAGE 13376
#define IDS_BLOCKED_MIDI_SYSEX_UNBLOCK 13377
#define IDS_BLOCKED_MIDI_SYSEX_NO_ACTION 13378
#define IDS_MICROPHONE_CAMERA_ALLOWED 13379
#define IDS_MICROPHONE_CAMERA_BLOCKED 13380
#define IDS_MICROPHONE_ACCESSED 13381
#define IDS_CAMERA_ACCESSED 13382
#define IDS_MICROPHONE_BLOCKED 13383
#define IDS_CAMERA_BLOCKED 13384
#define IDS_MICROPHONE_CAMERA_ALLOWED_TITLE 13385
#define IDS_MICROPHONE_CAMERA_BLOCKED_TITLE 13386
#define IDS_MICROPHONE_ACCESSED_TITLE 13387
#define IDS_CAMERA_ACCESSED_TITLE 13388
#define IDS_MICROPHONE_BLOCKED_TITLE 13389
#define IDS_CAMERA_BLOCKED_TITLE 13390
#define IDS_MEDIASTREAM_SETTING_CHANGED_MESSAGE 13391
#define IDS_A11Y_OMNIBOX_CHIP_HINT 13401
#define IDS_A11Y_INDICATORS_ANNOUNCEMENT 13402
#define IDS_MICROPHONE_CAMERA_IN_USE 13403
#define IDS_MICROPHONE_CAMERA_NOT_ALLOWED 13404
#define IDS_MICROPHONE_IN_USE 13405
#define IDS_CAMERA_IN_USE 13406
#define IDS_MICROPHONE_NOT_ALLOWED 13407
#define IDS_CAMERA_NOT_ALLOWED 13408
#define IDS_CAMERA_CANNOT_ACCESS 13409
#define IDS_MICROPHONE_CANNOT_ACCESS 13410
#define IDS_CAMERA_MICROPHONE_CANNOT_ACCESS 13411
#define IDS_MANAGE_PASSWORDS_CONFIRM_GENERATED_TEXT 13412
#define IDS_PASSWORDS_WEB_LINK 13413
#define IDS_MANAGE_PASSWORDS_LINK 13415
#define IDS_MANAGE_PASSWORDS_TITLE 13416
#define IDS_MANAGE_PASSWORDS_NO_PASSWORDS_TITLE 13417
#define IDS_MANAGE_PASSWORDS_DIFFERENT_DOMAIN_TITLE 13418
#define IDS_MANAGE_PASSWORDS_DIFFERENT_DOMAIN_NO_PASSWORDS_TITLE 13419
#define IDS_MANAGE_PASSWORDS_SHOW_PASSWORD 13420
#define IDS_MANAGE_PASSWORDS_HIDE_PASSWORD 13421
#define IDS_MANAGE_PASSWORDS_EMPTY_NOTE 13422
#define IDS_MANAGE_PASSWORDS_EDIT_USERNAME_TOOLTIP 13423
#define IDS_MANAGE_PASSWORDS_EDIT_NOTE_TOOLTIP 13424
#define IDS_MANAGE_PASSWORDS_NOTE_ACCESSIBLE_NAME 13425
#define IDS_MANAGE_PASSWORDS_NOTE_TEXTFIELD 13426
#define IDS_MANAGE_PASSWORDS_UPDATE 13427
#define IDS_MANAGE_PASSWORDS_USERNAME_ACCESSIBLE_NAME 13428
#define IDS_MANAGE_PASSWORDS_USERNAME_TEXTFIELD 13429
#define IDS_MANAGE_PASSWORDS_AUTO_SIGNIN_TITLE_MD 13430
#define IDS_MANAGE_PASSWORDS_AUTO_SIGNIN_TOAST_BODY 13431
#define IDS_AUTO_SIGNIN_FIRST_RUN_TITLE_MANY_DEVICES 13432
#define IDS_AUTO_SIGNIN_FIRST_RUN_TITLE_LOCAL_DEVICE 13433
#define IDS_AUTO_SIGNIN_FIRST_RUN_TEXT 13434
#define IDS_AUTO_SIGNIN_FIRST_RUN_OK 13435
#define IDS_FILE_SELECTION_DIALOG_INFOBAR 13444
#define IDS_ABOUT_LINUX_PROXY_CONFIG_TITLE 13445
#define IDS_ABOUT_LINUX_PROXY_CONFIG_BODY 13446
#define IDS_IMAGE_FILES 13447
#define IDS_AUDIO_FILES 13448
#define IDS_VIDEO_FILES 13449
#define IDS_CUSTOM_FILES 13450
#define IDS_SENSORS_ALLOWED_TOOLTIP 13451
#define IDS_MOTION_SENSORS_ALLOWED_TOOLTIP 13452
#define IDS_SENSORS_BLOCKED_TOOLTIP 13453
#define IDS_MOTION_SENSORS_BLOCKED_TOOLTIP 13454
#define IDS_BLOCKED_SENSORS_UNBLOCK 13455
#define IDS_BLOCKED_SENSORS_NO_ACTION 13456
#define IDS_ALLOWED_SENSORS_TITLE 13457
#define IDS_BLOCKED_SENSORS_TITLE 13458
#define IDS_ALLOWED_SENSORS_MESSAGE 13459
#define IDS_ALLOWED_MOTION_SENSORS_MESSAGE 13460
#define IDS_BLOCKED_SENSORS_MESSAGE 13461
#define IDS_BLOCKED_MOTION_SENSORS_MESSAGE 13462
#define IDS_ALLOWED_SENSORS_NO_ACTION 13463
#define IDS_ALLOWED_SENSORS_BLOCK 13464
#define IDS_LIVE_CAPTION_PROMO 13471
#define IDS_LIVE_CAPTION_PROMO_SCREENREADER 13472
#define IDS_ENABLE_CARET_BROWSING_INFO 13473
#define IDS_CARET_BROWSING_DO_NOT_ASK 13480
#define IDS_ENABLE_CARET_BROWSING_TITLE 13481
#define IDS_ENABLE_CARET_BROWSING_TURN_ON 13482
#define IDS_MEDIA_GALLERIES_DIALOG_HEADER 13483
#define IDS_MEDIA_GALLERIES_DIALOG_SUBTEXT_READ_WRITE 13484
#define IDS_MEDIA_GALLERIES_DIALOG_SUBTEXT_READ_DELETE 13485
#define IDS_MEDIA_GALLERIES_DIALOG_SUBTEXT_READ_ONLY 13486
#define IDS_MEDIA_GALLERIES_PERMISSION_SUGGESTIONS 13487
#define IDS_MEDIA_GALLERIES_LAST_ATTACHED 13488
#define IDS_MEDIA_GALLERIES_DIALOG_ADD_GALLERY 13489
#define IDS_MEDIA_GALLERIES_DIALOG_CONFIRM 13490
#define IDS_MEDIA_GALLERIES_DIALOG_ADD_GALLERY_TITLE 13491
#define IDS_MEDIA_GALLERIES_DIALOG_DEVICE_ATTACHED 13492
#define IDS_MEDIA_GALLERIES_DIALOG_DEVICE_NOT_ATTACHED 13493
#define IDS_MEDIA_GALLERIES_DIALOG_DELETE 13494
#define IDS_CHROME_SHORTCUT_DESCRIPTION 13496
#define IDS_WEBSTORE_NAME_STORE 13497
#define IDS_DESKTOP_MEDIA_PICKER_TITLE 13498
#define IDS_DISPLAY_MEDIA_PICKER_TITLE 13499
#define IDS_DESKTOP_MEDIA_PICKER_TITLE_SCREEN_ONLY 13500
#define IDS_DESKTOP_MEDIA_PICKER_TITLE_WINDOW_ONLY 13501
#define IDS_DESKTOP_MEDIA_PICKER_TEXT 13502
#define IDS_DISPLAY_MEDIA_PICKER_TEXT 13503
#define IDS_DESKTOP_MEDIA_PICKER_TEXT_DELEGATED 13504
#define IDS_DESKTOP_MEDIA_PICKER_AUDIO_SHARE_SCREEN_WITH_MUTE_WARNING 13505
#define IDS_DESKTOP_MEDIA_PICKER_ALSO_SHARE_SYSTEM_AUDIO 13506
#define IDS_DESKTOP_MEDIA_PICKER_ALSO_SHARE_APPLICATION_AUDIO 13507
#define IDS_DESKTOP_MEDIA_PICKER_ALSO_SHARE_ALL_AUDIO_OUTPUT 13508
#define IDS_DESKTOP_MEDIA_PICKER_ALSO_SHARE_TAB_AUDIO 13509
#define IDS_DESKTOP_MEDIA_PICKER_AUDIO_SHARE_HINT_TAB 13510
#define IDS_DESKTOP_MEDIA_PICKER_AUDIO_SHARE_HINT_TAB_OR_SCREEN 13511
#define IDS_DESKTOP_MEDIA_PICKER_AUDIO_SHARE_HINT_TAB_OR_WINDOW 13512
#define IDS_DESKTOP_MEDIA_PICKER_SOURCE_TYPE_SCREEN 13513
#define IDS_DESKTOP_MEDIA_PICKER_SOURCE_TYPE_WINDOW 13514
#define IDS_DESKTOP_MEDIA_PICKER_SINGLE_SCREEN_NAME 13515
#define IDS_DESKTOP_MEDIA_PICKER_MULTIPLE_SCREEN_NAME 13516
#define IDS_DESKTOP_MEDIA_PICKER_MANAGED 13517
#define IDS_DESKTOP_MEDIA_PICKER_PERMISSION_BUTTON_MAC 13518
#define IDS_DESKTOP_MEDIA_PICKER_RESELECT_SCREEN 13519
#define IDS_DESKTOP_MEDIA_PICKER_RESELECT_WINDOW 13520
#define IDS_DESKTOP_MEDIA_PICKER_SOURCE_TYPE_THIS_TAB 13521
#define IDS_DESKTOP_MEDIA_PICKER_EMPTY_PREVIEW 13522
#define IDS_DESKTOP_MEDIA_PICKER_PREVIEW_ACCESSIBLE_NAME 13523
#define IDS_DESKTOP_MEDIA_SOURCE_EMPTY_ACCESSIBLE_NAME 13524
#define IDS_DESKTOP_MEDIA_PICKER_DELEGATED_SOURCE_LIST_WINDOW_TEXT 13525
#define IDS_DESKTOP_MEDIA_PICKER_DELEGATED_SOURCE_LIST_WINDOW_BUTTON 13526
#define IDS_DESKTOP_MEDIA_PICKER_DELEGATED_SOURCE_LIST_SCREEN_TEXT 13527
#define IDS_DESKTOP_MEDIA_PICKER_DELEGATED_SOURCE_LIST_SCREEN_BUTTON 13528
#define IDS_SHARE_THIS_TAB_DIALOG_TITLE 13529
#define IDS_SHARE_THIS_TAB_DIALOG_TEXT 13530
#define IDS_SHARE_THIS_TAB_DIALOG_ALLOW 13531
#define IDS_SHARE_THIS_TAB_AUDIO_SHARE 13532
#define IDS_TAB_CAPTURE_TERMINATED_BY_POLICY_TITLE 13533
#define IDS_TAB_CAPTURE_TERMINATED_BY_POLICY_TEXT 13534
#define IDS_TOOLTIP_TAB_ALERT_STATE_MEDIA_RECORDING 13535
#define IDS_TOOLTIP_TAB_ALERT_STATE_AUDIO_RECORDING 13536
#define IDS_TOOLTIP_TAB_ALERT_STATE_VIDEO_RECORDING 13537
#define IDS_TOOLTIP_TAB_ALERT_STATE_TAB_CAPTURING 13538
#define IDS_TOOLTIP_TAB_ALERT_STATE_AUDIO_PLAYING 13539
#define IDS_TOOLTIP_TAB_ALERT_STATE_AUDIO_MUTING 13540
#define IDS_TOOLTIP_TAB_ALERT_STATE_BLUETOOTH_CONNECTED 13541
#define IDS_TOOLTIP_TAB_ALERT_STATE_BLUETOOTH_SCAN_ACTIVE 13542
#define IDS_TOOLTIP_TAB_ALERT_STATE_USB_CONNECTED 13543
#define IDS_TOOLTIP_TAB_ALERT_STATE_HID_CONNECTED 13544
#define IDS_TOOLTIP_TAB_ALERT_STATE_SERIAL_CONNECTED 13545
#define IDS_TOOLTIP_TAB_ALERT_STATE_PIP_PLAYING 13546
#define IDS_TOOLTIP_TAB_ALERT_STATE_DESKTOP_CAPTURING 13547
#define IDS_TOOLTIP_TAB_ALERT_STATE_VR_PRESENTING 13548
#define IDS_TAB_AX_LABEL_MEDIA_RECORDING_FORMAT 13549
#define IDS_TAB_AX_LABEL_AUDIO_RECORDING_FORMAT 13550
#define IDS_TAB_AX_LABEL_VIDEO_RECORDING_FORMAT 13551
#define IDS_TAB_AX_LABEL_TAB_CAPTURING_FORMAT 13552
#define IDS_TAB_AX_LABEL_PIP_PLAYING_FORMAT 13553
#define IDS_TAB_AX_LABEL_AUDIO_PLAYING_FORMAT 13554
#define IDS_TAB_AX_LABEL_AUDIO_MUTING_FORMAT 13555
#define IDS_TAB_AX_LABEL_BLUETOOTH_CONNECTED_FORMAT 13556
#define IDS_TAB_AX_LABEL_BLUETOOTH_SCAN_ACTIVE_FORMAT 13557
#define IDS_TAB_AX_LABEL_USB_CONNECTED_FORMAT 13558
#define IDS_TAB_AX_LABEL_HID_CONNECTED_FORMAT 13559
#define IDS_TAB_AX_LABEL_SERIAL_CONNECTED_FORMAT 13560
#define IDS_TAB_AX_LABEL_NETWORK_ERROR_FORMAT 13561
#define IDS_TAB_AX_LABEL_CRASHED_FORMAT 13562
#define IDS_TAB_AX_LABEL_DESKTOP_CAPTURING_FORMAT 13563
#define IDS_TAB_AX_LABEL_VR_PRESENTING 13564
#define IDS_TAB_AX_LABEL_SPLIT_TAB_LEFT_VIEW_FORMAT 13565
#define IDS_TAB_AX_LABEL_SPLIT_TAB_RIGHT_VIEW_FORMAT 13566
#define IDS_TAB_AX_LABEL_UNNAMED_GROUP_FORMAT 13567
#define IDS_TAB_AX_LABEL_NAMED_GROUP_FORMAT 13568
#define IDS_TAB_AX_LABEL_PINNED_FORMAT 13569
#define IDS_TAB_AX_LABEL_PERMISSION_REQUESTED_FORMAT 13570
#define IDS_GROUP_AX_LABEL_UNNAMED_GROUP_FORMAT 13571
#define IDS_GROUP_AX_LABEL_NAMED_GROUP_FORMAT 13572
#define IDS_GROUP_AX_LABEL_COLLAPSED 13573
#define IDS_GROUP_AX_LABEL_EXPANDED 13574
#define IDS_GROUP_AX_LABEL_UNNAMED_SAVED_GROUP_FORMAT 13575
#define IDS_GROUP_AX_LABEL_NAMED_SAVED_GROUP_FORMAT 13576
#define IDS_GROUP_AX_LABEL_GROUP_MENU_ITEM_FORMAT 13577
#define IDS_SAVED_GROUP_AX_LABEL_OPENED 13578
#define IDS_SAVED_GROUP_AX_LABEL_CLOSED 13579
#define IDS_SAVED_GROUP_AX_LABEL_SHARED 13580
#define IDS_TAB_AX_ANNOUNCE_MOVED_RIGHT 13581
#define IDS_TAB_AX_ANNOUNCE_MOVED_LEFT 13582
#define IDS_TAB_AX_ANNOUNCE_MOVED_UP 13583
#define IDS_TAB_AX_ANNOUNCE_MOVED_DOWN 13584
#define IDS_TAB_AX_ANNOUNCE_MOVED_FIRST 13585
#define IDS_TAB_AX_ANNOUNCE_MOVED_LAST 13586
#define IDS_TAB_AX_ANNOUNCE_TAB_ADDED_TO_UNNAMED_GROUP 13587
#define IDS_TAB_AX_ANNOUNCE_TAB_ADDED_TO_NAMED_GROUP 13588
#define IDS_TAB_AX_ANNOUNCE_TAB_REMOVED_FROM_UNNAMED_GROUP 13589
#define IDS_TAB_AX_ANNOUNCE_TAB_REMOVED_FROM_NAMED_GROUP 13590
#define IDS_PAGE_LOADING_AX_TITLE_FORMAT 13591
#define IDS_TAB_AX_INACTIVE_TAB 13592
#define IDS_TAB_AX_MEMORY_SAVINGS 13593
#define IDS_TAB_AX_MEMORY_USAGE 13594
#define IDS_TAB_AX_HIGH_MEMORY_USAGE 13595
#define IDS_PROFILE_IN_USE_LINUX_QUIT 13596
#define IDS_PROFILE_IN_USE_LINUX_RELAUNCH 13597
#define IDS_DESKTOP_MEDIA_PICKER_SHARE 13615
#define IDS_PUSH_MESSAGING_GENERIC_NOTIFICATION_BODY 13616
#define IDS_DEVICE_PERMISSIONS_DIALOG_SELECT 13617
#define IDS_DEVICE_PERMISSIONS_DIALOG_LOADING_LABEL 13618
#define IDS_DEVICE_PERMISSIONS_DIALOG_LOADING_LABEL_TOOLTIP 13619
#define IDS_DEVICE_LOG_TITLE 13620
#define IDS_DEVICE_AUTO_REFRESH 13621
#define IDS_DEVICE_SELECT_TYPES 13622
#define IDS_DEVICE_LOG_REFRESH 13623
#define IDS_DEVICE_LOG_CLEAR 13624
#define IDS_DEVICE_LOG_CLEAR_TYPES 13625
#define IDS_DEVICE_LOG_NO_ENTRIES 13626
#define IDS_DEVICE_LOG_LEVEL_LABEL 13627
#define IDS_DEVICE_LOG_LEVEL_ERROR 13628
#define IDS_DEVICE_LOG_LEVEL_USER 13629
#define IDS_DEVICE_LOG_LEVEL_EVENT 13630
#define IDS_DEVICE_LOG_LEVEL_DEBUG 13631
#define IDS_DEVICE_LOG_TYPE_LOGIN 13632
#define IDS_DEVICE_LOG_TYPE_MEMORY 13633
#define IDS_DEVICE_LOG_TYPE_NETWORK 13634
#define IDS_DEVICE_LOG_TYPE_POWER 13635
#define IDS_DEVICE_LOG_TYPE_BLUETOOTH 13636
#define IDS_DEVICE_LOG_TYPE_USB 13637
#define IDS_DEVICE_LOG_TYPE_HID 13638
#define IDS_DEVICE_LOG_TYPE_PRINTER 13639
#define IDS_DEVICE_LOG_TYPE_FIDO 13640
#define IDS_DEVICE_LOG_TYPE_SERIAL 13641
#define IDS_DEVICE_LOG_TYPE_CAMERA 13642
#define IDS_DEVICE_LOG_TYPE_GEOLOCATION 13643
#define IDS_DEVICE_LOG_TYPE_EXTENSIONS 13644
#define IDS_DEVICE_LOG_TYPE_DISPLAY 13645
#define IDS_DEVICE_LOG_TYPE_FIRMWARE 13646
#define IDS_DEVICE_LOG_FILEINFO 13647
#define IDS_DEVICE_LOG_TIME_DETAIL 13648
#define IDS_DEVICE_LOG_ENTRY 13649
#define IDS_WEBUSB_DEVICE_DETECTED_NOTIFICATION 13650
#define IDS_WEBUSB_DEVICE_DETECTED_NOTIFICATION_TITLE 13651
#define IDS_WEBHID_SYSTEM_TRAY_ICON_HID_SETTINGS 13652
#define IDS_WEBHID_SYSTEM_TRAY_ICON_ABOUT_HID_DEVICE 13653
#define IDS_WEBUSB_SYSTEM_TRAY_ICON_USB_SETTINGS 13654
#define IDS_WEBUSB_SYSTEM_TRAY_ICON_ABOUT_USB_DEVICE 13655
#define IDS_DEVICE_CONNECTED_BY_EXTENSION 13658
#define IDS_DEFAULT_AUDIO_DEVICE_NAME 13667
#define IDS_SERIAL_DEVICE_CHOOSER_ADAPTER_OFF 13676
#define IDS_SERIAL_DEVICE_CHOOSER_TURN_ON_BLUETOOTH_LINK_TEXT 13677
#define IDS_SERIAL_DEVICE_CHOOSER_AUTHORIZE_BLUETOOTH_LINK_TEXT 13678
#define IDS_BLUETOOTH_DEVICE_CHOOSER_RE_SCAN 13679
#define IDS_BLUETOOTH_DEVICE_CHOOSER_RE_SCAN_TOOLTIP 13680
#define IDS_DEVICE_CHOOSER_GET_HELP_LINK_WITH_SCANNING_STATUS 13681
#define IDS_DEVICE_CHOOSER_GET_HELP_LINK_WITH_RE_SCAN_LINK 13682
#define IDS_DEVICE_CHOOSER_PAIRED_STATUS_TEXT 13683
#define IDS_DEVICE_CHOOSER_DEVICE_NAME_AND_PAIRED_STATUS_TEXT 13684
#define IDS_DEVICE_CHOOSER_ACCNAME_COMPATIBLE_DEVICES_LIST 13685
#define IDS_DEVICE_CHOOSER_DEVICE_NAME_UNKNOWN_DEVICE_WITH_VENDOR_NAME 13686
#define IDS_DEVICE_CHOOSER_DEVICE_NAME_UNKNOWN_DEVICE_WITH_VENDOR_ID_AND_PRODUCT_ID 13687
#define IDS_USB_POLICY_DEVICE_DESCRIPTION_FOR_PRODUCT_ID_AND_VENDOR_NAME 13688
#define IDS_USB_POLICY_DEVICE_DESCRIPTION_FOR_PRODUCT_ID_AND_VENDOR_ID 13689
#define IDS_USB_POLICY_DEVICE_DESCRIPTION_FOR_VENDOR_ID 13690
#define IDS_USB_POLICY_DEVICE_DESCRIPTION_FOR_VENDOR_NAME 13691
#define IDS_USB_POLICY_DEVICE_DESCRIPTION_FOR_ANY_VENDOR 13692
#define IDS_SERIAL_PORT_CHOOSER_PROMPT 13693
#define IDS_SERIAL_PORT_CHOOSER_CONNECT_BUTTON_TEXT 13694
#define IDS_SERIAL_PORT_CHOOSER_LOADING_LABEL 13695
#define IDS_SERIAL_PORT_CHOOSER_LOADING_LABEL_TOOLTIP 13696
#define IDS_SERIAL_POLICY_DESCRIPTION_FOR_USB_PRODUCT_ID_AND_VENDOR_NAME 13697
#define IDS_SERIAL_POLICY_DESCRIPTION_FOR_USB_VENDOR_NAME 13698
#define IDS_SERIAL_PORT_CHOOSER_NAME_WITH_PATH 13699
#define IDS_SERIAL_POLICY_DESCRIPTION_FOR_USB_PRODUCT_ID_AND_VENDOR_ID 13700
#define IDS_SERIAL_POLICY_DESCRIPTION_FOR_USB_VENDOR_ID 13701
#define IDS_SERIAL_POLICY_DESCRIPTION_FOR_ANY_PORT 13702
#define IDS_HID_CHOOSER_PROMPT 13703
#define IDS_HID_CHOOSER_ITEM_WITHOUT_NAME 13704
#define IDS_HID_CHOOSER_LOADING_LABEL 13705
#define IDS_HID_CHOOSER_LOADING_LABEL_TOOLTIP 13706
#define IDS_HID_POLICY_DESCRIPTION_FOR_VENDOR_ID_AND_PRODUCT_ID 13707
#define IDS_HID_POLICY_DESCRIPTION_FOR_VENDOR_ID 13708
#define IDS_HID_POLICY_DESCRIPTION_FOR_USAGE_AND_USAGE_PAGE 13709
#define IDS_HID_POLICY_DESCRIPTION_FOR_USAGE_PAGE 13710
#define IDS_HID_POLICY_DESCRIPTION_FOR_ANY_DEVICE 13711
#define IDS_WEB_APP_FILE_HANDLING_LIST_SEPARATOR 13712
#define IDS_WEB_APP_FILE_HANDLING_DIALOG_QUESTION 13713
#define IDS_WEB_APP_FILE_HANDLING_DIALOG_QUESTION_MULTIPLE 13714
#define IDS_WEB_APP_FILE_HANDLING_DIALOG_STICKY_CHOICE 13715
#define IDS_WEB_APP_FILE_HANDLING_POSITIVE_BUTTON 13716
#define IDS_WEB_APP_FILE_HANDLING_NEGATIVE_BUTTON 13717
#define IDS_WEB_APP_PERMISSION_NEGATIVE_BUTTON 13718
#define IDS_PROTOCOL_HANDLER_INTENT_PICKER_QUESTION 13719
#define IDS_FONT_ACCESS_CHOOSER_PROMPT_ORIGIN 13720
#define IDS_FONT_ACCESS_CHOOSER_NO_FONTS_FOUND_PROMPT 13721
#define IDS_FONT_ACCESS_CHOOSER_IMPORT_BUTTON_TEXT 13722
#define IDS_FONT_ACCESS_CHOOSER_CANCEL_BUTTON_TEXT 13723
#define IDS_FONT_ACCESS_CHOOSER_SELECT_ALL_CHECKBOX_TEXT 13724
#define IDS_FONT_ACCESS_CHOOSER_LOADING_LABEL 13725
#define IDS_FONT_ACCESS_CHOOSER_LOADING_LABEL_TOOLTIP 13726
#define IDS_IME_API_ACTIVATED_WARNING 13727
#define IDS_IME_API_NEVER_SHOW 13728
#define IDS_BLOCKED_ADS_PROMPT_TOOLTIP 13729
#define IDS_REDIRECT_BLOCKED_MESSAGE 13732
#define IDS_REDIRECT_BLOCKED_TITLE 13733
#define IDS_REDIRECT_BLOCKED_TOOLTIP 13734
#define IDS_ALWAYS_ALLOW_REDIRECTS 13735
#define IDS_OPEN_REDIRECT 13736
#define IDS_UTILITY_PROCESS_SYSTEM_SIGNALS_NAME 13737
#define IDS_CONTROLLED_BY_AUTOMATION 13740
#define IDS_CHROME_FOR_TESTING_DISCLAIMER 13741
#define IDS_DOWNLOAD_CHROME 13742
#define IDS_DOWNLOAD_OPEN_CONFIRMATION_DIALOG_TITLE 13744
#define IDS_DOWNLOAD_OPEN_CONFIRMATION_DIALOG_MESSAGE 13745
#define IDS_CONFIRM_FILE_UPLOAD_TITLE 13746
#define IDS_CONFIRM_FILE_UPLOAD_TEXT 13747
#define IDS_CONFIRM_FILE_UPLOAD_OK_BUTTON 13748
#define IDS_FILE_SYSTEM_ACCESS_WRITE_PERMISSION_TITLE 13749
#define IDS_FILE_SYSTEM_ACCESS_WRITE_PERMISSION_ALLOW_TEXT 13750
#define IDS_FILE_SYSTEM_ACCESS_ORIGIN_SCOPED_WRITE_PERMISSION_FILE_TEXT 13751
#define IDS_FILE_SYSTEM_ACCESS_ORIGIN_SCOPED_WRITE_PERMISSION_DIRECTORY_TEXT 13752
#define IDS_FILE_SYSTEM_ACCESS_ORIGIN_SCOPED_READ_PERMISSION_FILE_TEXT 13753
#define IDS_FILE_SYSTEM_ACCESS_ORIGIN_SCOPED_READ_PERMISSION_DIRECTORY_TEXT 13754
#define IDS_FILE_SYSTEM_ACCESS_WRITE_PERMISSION_FILE_TEXT 13755
#define IDS_FILE_SYSTEM_ACCESS_WRITE_PERMISSION_DIRECTORY_TEXT 13756
#define IDS_FILE_SYSTEM_ACCESS_READ_PERMISSION_FILE_TEXT 13757
#define IDS_FILE_SYSTEM_ACCESS_READ_PERMISSION_DIRECTORY_TEXT 13758
#define IDS_FILE_SYSTEM_ACCESS_EDIT_FILE_PERMISSION_TITLE 13759
#define IDS_FILE_SYSTEM_ACCESS_EDIT_DIRECTORY_PERMISSION_TITLE 13760
#define IDS_FILE_SYSTEM_ACCESS_READ_FILE_PERMISSION_TITLE 13761
#define IDS_FILE_SYSTEM_ACCESS_READ_DIRECTORY_PERMISSION_TITLE 13762
#define IDS_FILE_SYSTEM_ACCESS_EDIT_FILE_PERMISSION_ALLOW_TEXT 13763
#define IDS_FILE_SYSTEM_ACCESS_EDIT_DIRECTORY_PERMISSION_ALLOW_TEXT 13764
#define IDS_FILE_SYSTEM_ACCESS_VIEW_FILE_PERMISSION_ALLOW_TEXT 13765
#define IDS_FILE_SYSTEM_ACCESS_VIEW_DIRECTORY_PERMISSION_ALLOW_TEXT 13766
#define IDS_FILE_SYSTEM_ACCESS_WRITE_PERMISSION_CANCEL_TEXT 13767
#define IDS_FILE_SYSTEM_ACCESS_EDIT_FILE_PERMISSION_CANCEL_TEXT 13768
#define IDS_FILE_SYSTEM_ACCESS_EDIT_DIRECTORY_PERMISSION_CANCEL_TEXT 13769
#define IDS_FILE_SYSTEM_ACCESS_VIEW_FILE_PERMISSION_CANCEL_TEXT 13770
#define IDS_FILE_SYSTEM_ACCESS_VIEW_DIRECTORY_PERMISSION_CANCEL_TEXT 13771
#define IDS_FILE_SYSTEM_ACCESS_RESTORE_PERMISSION_DESCRIPTION 13772
#define IDS_FILE_SYSTEM_ACCESS_WRITE_USAGE_TOOLTIP 13773
#define IDS_FILE_SYSTEM_ACCESS_DIRECTORY_USAGE_TOOLTIP 13774
#define IDS_FILE_SYSTEM_ACCESS_USAGE_BUBBLE_SINGLE_WRITABLE_FILE_TEXT 13775
#define IDS_FILE_SYSTEM_ACCESS_USAGE_BUBBLE_WRITABLE_FILES_TEXT 13776
#define IDS_FILE_SYSTEM_ACCESS_USAGE_BUBBLE_SINGLE_WRITABLE_DIRECTORY_TEXT 13777
#define IDS_FILE_SYSTEM_ACCESS_USAGE_BUBBLE_WRITABLE_DIRECTORIES_TEXT 13778
#define IDS_FILE_SYSTEM_ACCESS_USAGE_BUBBLE_WRITABLE_FILES_AND_DIRECTORIES_TEXT 13779
#define IDS_FILE_SYSTEM_ACCESS_USAGE_BUBBLE_SINGLE_READABLE_FILE_TEXT 13780
#define IDS_FILE_SYSTEM_ACCESS_USAGE_BUBBLE_READABLE_FILES_TEXT 13781
#define IDS_FILE_SYSTEM_ACCESS_USAGE_BUBBLE_SINGLE_READABLE_DIRECTORY_TEXT 13782
#define IDS_FILE_SYSTEM_ACCESS_USAGE_BUBBLE_READABLE_DIRECTORIES_TEXT 13783
#define IDS_FILE_SYSTEM_ACCESS_USAGE_BUBBLE_READABLE_FILES_AND_DIRECTORIES_TEXT 13784
#define IDS_FILE_SYSTEM_ACCESS_USAGE_BUBBLE_READ_AND_WRITE 13785
#define IDS_FILE_SYSTEM_ACCESS_USAGE_BUBBLE_SAVE_CHANGES 13786
#define IDS_FILE_SYSTEM_ACCESS_USAGE_BUBBLE_VIEW_CHANGES 13787
#define IDS_FILE_SYSTEM_ACCESS_USAGE_BUBBLE_FILES_TEXT 13788
#define IDS_FILE_SYSTEM_ACCESS_USAGE_EXPAND 13789
#define IDS_FILE_SYSTEM_ACCESS_USAGE_COLLAPSE 13790
#define IDS_FILE_SYSTEM_ACCESS_USAGE_REMOVE_ACCESS 13791
#define IDS_FILE_SYSTEM_ACCESS_CHOOSER_OPEN_READABLE_DIRECTORY_TITLE 13792
#define IDS_FILE_SYSTEM_ACCESS_CHOOSER_OPEN_WRITABLE_DIRECTORY_TITLE 13793
#define IDS_FILE_SYSTEM_ACCESS_CHOOSER_OPEN_SAVE_FILE_TITLE 13794
#define IDS_FILE_SYSTEM_ACCESS_CHOOSER_OPEN_READABLE_FILE_TITLE 13795
#define IDS_FILE_SYSTEM_ACCESS_CHOOSER_OPEN_READABLE_FILES_TITLE 13796
#define IDS_FILE_SYSTEM_ACCESS_RESTRICTED_DIRECTORY_TITLE 13797
#define IDS_FILE_SYSTEM_ACCESS_RESTRICTED_DIRECTORY_TEXT 13798
#define IDS_FILE_SYSTEM_ACCESS_RESTRICTED_DIRECTORY_BUTTON 13799
#define IDS_FILE_SYSTEM_ACCESS_RESTRICTED_FILE_TITLE 13800
#define IDS_FILE_SYSTEM_ACCESS_RESTRICTED_FILE_TEXT 13801
#define IDS_FILE_SYSTEM_ACCESS_RESTRICTED_FILE_BUTTON 13802
#define IDS_FILE_SYSTEM_ACCESS_DANGEROUS_FILE_TITLE 13803
#define IDS_FILE_SYSTEM_ACCESS_DANGEROUS_FILE_TEXT 13804
#define IDS_FILE_SYSTEM_ACCESS_DANGEROUS_FILE_DONT_SAVE 13805
#define IDS_FILE_SYSTEM_ACCESS_DANGEROUS_FILE_SAVE 13806
#define IDS_RELAUNCH_ACCEPT_BUTTON 13807
#define IDS_RELAUNCH_REQUIRED_CANCEL_BUTTON 13808
#define IDS_IDLE_TIMEOUT_INCOGNITO 13809
#define IDS_IDLE_TIMEOUT_CLOSE_COUNTDOWN 13810
#define IDS_IDLE_TIMEOUT_CLEAR_COUNTDOWN 13811
#define IDS_WEBAUTHN_GENERIC_TITLE 13812
#define IDS_WEBAUTHN_CONTINUE 13813
#define IDS_WEBAUTHN_TRANSPORT_USB 13814
#define IDS_WEBAUTHN_TRANSPORT_INTERNAL 13816
#define IDS_WEBAUTHN_TRANSPORT_CABLE 13817
#define IDS_WEBAUTHN_TRANSPORT_WINDOWS_HELLO_OR_SECURITY_KEY 13818
#define IDS_WEBAUTHN_TRANSPORT_WINDOWS_HELLO 13819
#define IDS_WEBAUTHN_TRANSPORT_ICLOUD_KEYCHAIN 13820
#define IDS_WEBAUTHN_TRANSPORT_EXTERNAL_SECURITY_KEY 13821
#define IDS_WEBAUTHN_USB_ACTIVATE_DESCRIPTION 13822
#define IDS_WEBAUTHN_ERROR_GENERIC_TITLE 13823
#define IDS_WEBAUTHN_ERROR_WRONG_KEY_TITLE 13824
#define IDS_WEBAUTHN_ERROR_WRONG_DEVICE_TITLE 13825
#define IDS_WEBAUTHN_ERROR_WRONG_DEVICE_REGISTER_DESCRIPTION 13826
#define IDS_WEBAUTHN_ERROR_WRONG_KEY_SIGN_DESCRIPTION 13827
#define IDS_WEBAUTHN_ERROR_TIMEOUT_DESCRIPTION 13828
#define IDS_WEBAUTHN_ERROR_INTERNAL_UNRECOGNIZED_TITLE 13829
#define IDS_WEBAUTHN_ERROR_INTERNAL_UNRECOGNIZED_DESCRIPTION 13830
#define IDS_WEBAUTHN_WINDOWS_HELLO_NOT_ENABLED_TITLE 13831
#define IDS_WEBAUTHN_WINDOWS_HELLO_NOT_ENABLED_DESCRIPTION 13832
#define IDS_WEBAUTHN_ERROR_NO_PASSKEYS_TITLE 13833
#define IDS_WEBAUTHN_ERROR_NO_PASSKEYS_DESCRIPTION 13834
#define IDS_WEBAUTHN_ERROR_GPM_DISABLED_TITLE 13835
#define IDS_WEBAUTHN_ERROR_GPM_DISABLED_DESCRIPTION 13836
#define IDS_WEBAUTHN_BLUETOOTH_POWER_ON_AUTO_TITLE 13837
#define IDS_WEBAUTHN_BLUETOOTH_POWER_ON_AUTO_DESCRIPTION 13838
#define IDS_WEBAUTHN_BLUETOOTH_POWER_ON_AUTO_NEXT 13839
#define IDS_WEBAUTHN_BLUETOOTH_POWER_ON_MANUAL_TITLE 13840
#define IDS_WEBAUTHN_BLUETOOTH_POWER_ON_MANUAL_DESCRIPTION 13841
#define IDS_WEBAUTHN_BLUETOOTH_POWER_ON_MANUAL_NEXT 13842
#define IDS_WEBAUTHN_LOADING 13843
#define IDS_WEBAUTHN_CABLEV2_CONNECTING_TITLE 13845
#define IDS_WEBAUTHN_CABLEV2_CONNECTED_DESCRIPTION 13846
#define IDS_WEBAUTHN_CABLEV2_ERROR_DESCRIPTION 13847
#define IDS_WEBAUTHN_CABLEV2_ERROR_CLOSE 13848
#define IDS_WEBAUTHN_PASSKEY_PHONE_OR_TABLET_LABEL 13849
#define IDS_WEBAUTHN_PASSKEY_PHONE_TABLET_OR_SECURITY_KEY_LABEL 13850
#define IDS_WEBAUTHN_PIN_ENTRY_TITLE 13851
#define IDS_WEBAUTHN_PIN_ENTRY_DESCRIPTION 13852
#define IDS_WEBAUTHN_PIN_ENTRY_PIN_LABEL 13853
#define IDS_WEBAUTHN_PIN_ENTRY_NEXT 13854
#define IDS_WEBAUTHN_PIN_SETUP_DESCRIPTION 13855
#define IDS_WEBAUTHN_PIN_SETUP_CONFIRMATION_LABEL 13856
#define IDS_WEBAUTHN_PIN_ENTRY_ERROR_INVALID_CHARACTERS 13857
#define IDS_WEBAUTHN_PIN_ENTRY_ERROR_TOO_SHORT 13858
#define IDS_WEBAUTHN_PIN_ENTRY_ERROR_FAILED_RETRIES 13859
#define IDS_WEBAUTHN_PIN_ENTRY_ERROR_FAILED 13860
#define IDS_WEBAUTHN_PIN_TAP_AGAIN_DESCRIPTION 13861
#define IDS_WEBAUTHN_PIN_ENTRY_ERROR_MISMATCH 13862
#define IDS_WEBAUTHN_CLIENT_PIN_SOFT_BLOCK_DESCRIPTION 13863
#define IDS_WEBAUTHN_CLIENT_PIN_HARD_BLOCK_DESCRIPTION 13864
#define IDS_WEBAUTHN_CLIENT_PIN_AUTHENTICATOR_REMOVED_DESCRIPTION 13865
#define IDS_WEBAUTHN_PIN_ENTRY_ERROR_SAME_AS_CURRENT 13866
#define IDS_WEBAUTHN_INLINE_ENROLLMENT_CANCEL_LABEL 13867
#define IDS_WEBAUTHN_UV_RETRY_TITLE 13868
#define IDS_WEBAUTHN_UV_RETRY_DESCRIPTION 13869
#define IDS_WEBAUTHN_UV_RETRY_ERROR_FAILED_RETRIES 13870
#define IDS_WEBAUTHN_UV_ERROR_LOCKED 13871
#define IDS_WEBAUTHN_FORCE_PIN_CHANGE 13872
#define IDS_WEBAUTHN_UNKNOWN_ACCOUNT 13873
#define IDS_WEBAUTHN_RESIDENT_KEY_PRIVACY 13874
#define IDS_WEBAUTHN_RESIDENT_KEY_PREFERRED_PRIVACY 13875
#define IDS_WEBAUTHN_ERROR_MISSING_CAPABILITY_TITLE 13876
#define IDS_WEBAUTHN_ERROR_MISSING_CAPABILITY_DESC 13877
#define IDS_WEBAUTHN_STORAGE_FULL_DESC 13878
#define IDS_WEBAUTHN_ATTESTATION_WARNING 13879
#define IDS_WEBAUTHN_ENTERPRISE_ATTESTATION_WARNING 13880
#define IDS_WEBAUTHN_RETRY 13881
#define IDS_WEBAUTHN_PLATFORM_AUTHENTICATOR_OFF_THE_RECORD_INTERSTITIAL_TITLE 13882
#define IDS_WEBAUTHN_PLATFORM_AUTHENTICATOR_OFF_THE_RECORD_INTERSTITIAL_DESCRIPTION 13883
#define IDS_WEBAUTHN_PLATFORM_AUTHENTICATOR_OFF_THE_RECORD_INTERSTITIAL_DENY 13884
#define IDS_WEBAUTHN_CREATE_PASSKEY_TITLE 13885
#define IDS_WEBAUTHN_CREATE_PASSKEY_EXTRA 13886
#define IDS_WEBAUTHN_CREATE_PASSKEY_EXTRA_INCOGNITO 13887
#define IDS_WEBAUTHN_CREATE_PASSKEY_CHOOSE_DEVICE_TITLE 13888
#define IDS_WEBAUTHN_USE_PASSKEY_TITLE 13889
#define IDS_WEBAUTHN_CHOOSE_PASSKEY_TITLE 13890
#define IDS_WEBAUTHN_CHOOSE_PASSKEY_BODY 13891
#define IDS_WEBAUTHN_QR_CODE_ALT_TEXT 13892
#define IDS_WEBAUTHN_USE_A_DIFFERENT_PASSKEY 13893
#define IDS_WEBAUTHN_CHOOSE_PASSKEY_FOR_RP_TITLE 13894
#define IDS_WEBAUTHN_THIS_DEVICE_LABEL 13895
#define IDS_WEBAUTHN_OTHER_DEVICES_LABEL 13896
#define IDS_WEBAUTHN_SOURCE_CHROME_PROFILE 13897
#define IDS_WEBAUTHN_SOURCE_ICLOUD_KEYCHAIN 13898
#define IDS_WEBAUTHN_SOURCE_WINDOWS_HELLO 13899
#define IDS_WEBAUTHN_SOURCE_GOOGLE_PASSWORD_MANAGER 13900
#define IDS_WEBAUTHN_SOURCE_USB_SECURITY_KEY 13901
#define IDS_WEBAUTHN_PASSKEYS_AND_SECURITY_KEYS 13902
#define IDS_WEBAUTHN_PASSKEYS 13903
#define IDS_WEBAUTHN_USE_YOUR_PHONE_OR_TABLET 13904
#define IDS_WEBAUTHN_USE_YOUR_PHONE_OR_TABLET_CREATE_DESCRIPTION 13905
#define IDS_WEBAUTHN_USE_YOUR_PHONE_OR_TABLET_SIGN_IN_DESCRIPTION 13906
#define IDS_WEBAUTHN_USE_YOUR_SECURITY_KEY 13907
#define IDS_WEBAUTHN_USE_YOUR_SECURITY_KEY_CREATE_DESCRIPTION 13908
#define IDS_WEBAUTHN_USE_YOUR_SECURITY_KEY_SIGN_IN_DESCRIPTION 13909
#define IDS_WEBAUTHN_CREATE 13910
#define IDS_WEBAUTHN_GPM_CREATE_PASSKEY_TITLE 13911
#define IDS_WEBAUTHN_GPM_CREATE_PASSKEY_DESC 13912
#define IDS_WEBAUTHN_SAVE_ANOTHER_WAY 13913
#define IDS_WEBAUTHN_GPM_TRUST_THIS_COMPUTER_CREATION_TITLE 13914
#define IDS_WEBAUTHN_GPM_TRUST_THIS_COMPUTER_CREATION_DESC 13915
#define IDS_WEBAUTHN_GPM_TRUST_THIS_COMPUTER_ASSERTION_TITLE 13916
#define IDS_WEBAUTHN_GPM_TRUST_THIS_COMPUTER_ASSERTION_DESC 13917
#define IDS_WEBAUTHN_USE_A_DIFFERENT_DEVICE 13918
#define IDS_WEBAUTHN_TOUCH_ID_ASSERTION_DESC 13919
#define IDS_WEBAUTHN_TOUCH_ID_CONTINUE 13920
#define IDS_WEBAUTHN_TOUCH_ID_LOCKED 13921
#define IDS_WEBAUTHN_TOUCH_ID_ENTER_PASSWORD 13922
#define IDS_WEBAUTHN_GPM_ERROR_TITLE 13923
#define IDS_WEBAUTHN_GPM_ERROR_DESC 13924
#define IDS_WEBAUTHN_SIGN_IN_AGAIN_TITLE 13925
#define IDS_WEBAUTHN_SIGN_IN_AGAIN_DESCRIPTION 13926
#define IDS_WEBAUTHN_GPM_PIN_OPTION_NUMBERS 13927
#define IDS_WEBAUTHN_GPM_PIN_OPTION_ALPHANUMERIC 13928
#define IDS_WEBAUTHN_GPM_CREATE_PIN_TITLE 13929
#define IDS_WEBAUTHN_GPM_CREATE_ALPHANUMERIC_PIN_ACCESSIBILITY 13930
#define IDS_WEBAUTHN_GPM_CREATE_PIN_DESC 13931
#define IDS_WEBAUTHN_GPM_ENTER_PIN_TITLE 13932
#define IDS_WEBAUTHN_GPM_ENTER_ALPHANUMERIC_PIN_ACCESSIBILITY_WITH_WEBSITE 13933
#define IDS_WEBAUTHN_GPM_ENTER_PIN_DESC 13934
#define IDS_WEBAUTHN_GPM_SIX_DIGIT_PIN_ACCESSIBILITY_LABEL 13935
#define IDS_WEBAUTHN_FORGOT_GPM_PIN_BUTTON 13936
#define IDS_WEBAUTHN_GPM_PIN_OPTIONS_BUTTON 13937
#define IDS_WEBAUTHN_GPM_WRONG_PIN_ERROR 13938
#define IDS_WEBAUTHN_GPM_PASSKEY_SAVED_TITLE 13939
#define IDS_WEBAUTHN_GPM_PASSKEY_SAVED_PIN_CREATED_TITLE 13940
#define IDS_WEBAUTHN_SHOW_PIN 13941
#define IDS_WEBAUTHN_HIDE_PIN 13942
#define IDS_WEBAUTHN_CHANGE_PIN 13943
#define IDS_WEBAUTHN_LOCKED_GPM_PIN_TITLE 13944
#define IDS_WEBAUTHN_LOCKED_GPM_PIN_DESCRIPTION 13945
#define IDS_WEBAUTHN_GPM_PIN_DIGIT_HINT 13946
#define IDS_WEBAUTHN_GPM_PIN_LENGTH_HINT 13947
#define IDS_WEBAUTHN_GPM_PASSKEY_SAVED_LABEL 13948
#define IDS_WEBAUTHN_GPM_PASSKEY_SAVED_LINK 13949
#define IDS_WEBAUTHN_GPM_PASSKEY_UPDATE_NEEDED_TITLE 13950
#define IDS_WEBAUTHN_GPM_PASSKEY_UPDATE_NEEDED_LABEL 13951
#define IDS_WEBAUTHN_GPM_PASSKEY_UPDATED_TITLE 13952
#define IDS_WEBAUTHN_GPM_PASSKEY_UPDATED_LABEL 13953
#define IDS_WEBAUTHN_GPM_PASSKEY_UPDATED_LINK 13954
#define IDS_WEBAUTHN_GPM_PASSKEY_DELETED_TITLE 13955
#define IDS_WEBAUTHN_GPM_PASSKEY_DOESNT_WORK_LABEL 13956
#define IDS_WEBAUTHN_GPM_PASSKEY_DELETED_LINK 13957
#define IDS_WEBAUTHN_GPM_SETTINGS 13958
#define IDS_WEBAUTHN_SIGN_IN_TO_WEBSITE_DIALOG_TITLE 13959
#define IDS_WEBAUTHN_SIGN_IN_AS_PROMPT 13960
#define IDS_PASSKEY_UPGRADE_BUBBLE_TITLE 13961
#define IDS_PASSKEY_UPGRADE_BUBBLE_DESCRIPTION 13962
#define IDS_PASSKEY_UPGRADE_BUBBLE_LEARN_MORE 13963
#define IDS_PASSKEY_UPGRADE_BUBBLE_MANAGE_BUTTON 13964
#define IDS_WEB_DIGITAL_CREDENTIALS_BLUETOOTH_POWER_ON_MANUAL_TITLE 13965
#define IDS_WEB_DIGITAL_CREDENTIALS_BLUETOOTH_POWER_ON_MANUAL_DESCRIPTION 13966
#define IDS_WEB_DIGITAL_CREDENTIALS_BLUETOOTH_POWER_ON_MANUAL_NEXT 13967
#define IDS_WEB_DIGITAL_CREDENTIALS_QR_TITLE 13968
#define IDS_WEB_DIGITAL_CREDENTIALS_PRESENTATION_QR_BODY 13969
#define IDS_WEB_DIGITAL_CREDENTIALS_ISSUANCE_QR_BODY 13970
#define IDS_WEB_DIGITAL_CREDENTIALS_QR_CODE_ALT_TEXT 13971
#define IDS_WEB_DIGITAL_CREDENTIALS_CABLEV2_CONNECTING_TITLE 13972
#define IDS_WEB_DIGITAL_CREDENTIALS_CABLEV2_CONNECTED_TITLE 13973
#define IDS_WEB_DIGITAL_CREDENTIALS_FLOW_CANCEL_BUTTON_TEXT 13974
#define IDS_WEB_DIGITAL_CREDENTIALS_INTERSTITIAL_DIALOG_TITLE 13975
#define IDS_WEB_DIGITAL_CREDENTIALS_INTERSTITIAL_LOW_RISK_DIALOG_TEXT 13976
#define IDS_WEB_DIGITAL_CREDENTIALS_INTERSTITIAL_LOW_RISK_NEGATIVE_BUTTON_TEXT 13977
#define IDS_WEB_DIGITAL_CREDENTIALS_INTERSTITIAL_HIGH_RISK_DIALOG_TEXT 13978
#define IDS_WEB_DIGITAL_CREDENTIALS_INTERSTITIAL_HIGH_RISK_NEGATIVE_BUTTON_TEXT 13979
#define IDS_WEB_DIGITAL_CREDENTIALS_INTERSTITIAL_POSITIVE_BUTTON_TEXT 13980
#define IDS_WEB_DIGITAL_CREDENTIALS_INTERSTITIAL_REQUEST_ABORTED_DIALOG_TEXT 13981
#define IDS_INCOGNITO_PROFILE_MENU_TITLE 13982
#define IDS_INCOGNITO_PROFILE_MENU_CLOSE_BUTTON 13983
#define IDS_INCOGNITO_CLEAR_BROWSING_DATA_DIALOG_PRIMARY_TEXT 13984
#define IDS_INCOGNITO_CLEAR_BROWSING_DATA_DIALOG_SECONDARY_TEXT 13985
#define IDS_INCOGNITO_CLEAR_BROWSING_DATA_DIALOG_CLOSE_WINDOWS_BUTTON 13986
#define IDS_INCOGNITO_HISTORY_BUBBLE_PRIMARY_TEXT 13987
#define IDS_INCOGNITO_HISTORY_BUBBLE_SECONDARY_TEXT 13988
#define IDS_INCOGNITO_HISTORY_BUBBLE_CLOSE_INCOGNITO_BUTTON_TEXT 13989
#define IDS_INCOGNITO_HISTORY_BUBBLE_CANCEL_BUTTON_TEXT 13990
#define IDS_NOTIFICATION_DEFAULT_HELPFUL_BUTTON_TEXT 13991
#define IDS_NOTIFICATION_DEFAULT_UNHELPFUL_BUTTON_TEXT 13992
#define IDS_DEEP_SCANNING_DIALOG_TITLE 13993
#define IDS_DEEP_SCANNING_DIALOG_UPLOAD_PENDING_MESSAGE 13994
#define IDS_DEEP_SCANNING_DIALOG_PRINT_PENDING_MESSAGE 13995
#define IDS_DEEP_SCANNING_DIALOG_SUCCESS_MESSAGE 13996
#define IDS_DEEP_SCANNING_DIALOG_PRINT_SUCCESS_MESSAGE 13997
#define IDS_DEEP_SCANNING_DIALOG_UPLOAD_FAILURE_MESSAGE 13998
#define IDS_DEEP_SCANNING_DIALOG_UPLOAD_FAIL_CLOSED_MESSAGE 13999
#define IDS_DEEP_SCANNING_DIALOG_UPLOAD_WARNING_MESSAGE 14000
#define IDS_DEEP_SCANNING_DIALOG_PRINT_WARNING_MESSAGE 14001
#define IDS_DEEP_SCANNING_DIALOG_TIMEOUT_MESSAGE 14002
#define IDS_DEEP_SCANNING_DIALOG_PROCEED_BUTTON 14003
#define IDS_DEEP_SCANNING_DIALOG_CANCEL_WARNING_BUTTON 14004
#define IDS_DEEP_SCANNING_DIALOG_CANCEL_UPLOAD_BUTTON 14005
#define IDS_DEEP_SCANNING_DIALOG_LARGE_FILE_FAILURE_MESSAGE 14006
#define IDS_DEEP_SCANNING_DIALOG_LARGE_PRINT_FAILURE_MESSAGE 14007
#define IDS_DEEP_SCANNING_DIALOG_ENCRYPTED_FILE_FAILURE_MESSAGE 14008
#define IDS_DEEP_SCANNING_DIALOG_CUSTOM_MESSAGE 14009
#define IDS_DEEP_SCANNING_DIALOG_COMBINED_CUSTOM_MESSAGE 14010
#define IDS_DEEP_SCANNING_DIALOG_DOWNLOADS_SENSITIVE_DATA 14011
#define IDS_DEEP_SCANNING_DIALOG_CUSTOM_MESSAGE_LEARN_MORE_LINK 14012
#define IDS_DEEP_SCANNING_DIALOG_DOWNLOADS_DISCARD_FILE_BUTTON 14013
#define IDS_DEEP_SCANNING_DIALOG_UPLOAD_BYPASS_JUSTIFICATION_LABEL 14014
#define IDS_DEEP_SCANNING_DIALOG_PASTE_BYPASS_JUSTIFICATION_LABEL 14015
#define IDS_DEEP_SCANNING_DIALOG_PRINT_BYPASS_JUSTIFICATION_LABEL 14016
#define IDS_DEEP_SCANNING_DIALOG_DOWNLOAD_BYPASS_JUSTIFICATION_LABEL 14017
#define IDS_DEEP_SCANNING_DIALOG_BYPASS_JUSTIFICATION_TEXT_LIMIT_LABEL 14018
#define IDS_DEEP_SCANNING_DIALOG_SAVE_TO_CLOUD_STORAGE_LABEL 14019
#define IDS_DEEP_SCANNING_DIALOG_SAVE_TO_CLOUD_FILENAME_PLACEHOLDER 14020
#define IDS_DEEP_SCANNING_DIALOG_SAVE_TO_CLOUD_STORAGE_MESSAGE 14021
#define IDS_DATA_SHARING_BUBBLE_DIALOG_TITLE 14022
#define IDS_SETTINGS_SAFEBROWSING_ENHANCED_IPH_BUBBLE_TEXT 14023
#define IDS_SETTINGS_SAFEBROWSING_ENHANCED_IPH_BUBBLE_CLOSE_BUTTON_ARIA_LABEL_TEXT 14024
#define IDS_TAILORED_SECURITY_ENABLED_DIALOG_TITLE 14025
#define IDS_TAILORED_SECURITY_DISABLED_DIALOG_TITLE 14026
#define IDS_TAILORED_SECURITY_ENABLED_DIALOG_MAIN_TEXT 14027
#define IDS_TAILORED_SECURITY_DISABLED_DIALOG_MAIN_TEXT 14028
#define IDS_TAILORED_SECURITY_DISABLED_DIALOG_ACCEPT_BUTTON 14029
#define IDS_TAILORED_SECURITY_DIALOG_SETTINGS_BUTTON 14030
#define IDS_TAILORED_SECURITY_UNCONSENTED_MODAL_TITLE 14031
#define IDS_TAILORED_SECURITY_UNCONSENTED_MODAL_BODY 14032
#define IDS_TAILORED_SECURITY_UNCONSENTED_ACCEPT_BUTTON 14033
#define IDS_TAILORED_SECURITY_UNCONSENTED_CANCEL_BUTTON 14034
#define IDS_TAILORED_SECURITY_CONSENTED_ENABLE_MESSAGE_TITLE 14035
#define IDS_TAILORED_SECURITY_CONSENTED_ENABLE_MESSAGE_DESCRIPTION 14036
#define IDS_TAILORED_SECURITY_CONSENTED_DISABLE_MESSAGE_TITLE 14037
#define IDS_TAILORED_SECURITY_CONSENTED_DISABLE_MESSAGE_DESCRIPTION 14038
#define IDS_TAILORED_SECURITY_CONSENTED_MESSAGE_OK_BUTTON 14039
#define IDS_TAILORED_SECURITY_CONSENTED_ENHANCED_PROTECTION_ENABLE_MESSAGE_TITLE 14040
#define IDS_PROMPT_APP_UNINSTALL_TITLE 14041
#define IDS_APP_UNINSTALL_PROMPT_REMOVE_DATA_CHECKBOX_FOR_NON_GOOGLE 14042
#define IDS_APP_UNINSTALL_PROMPT_REMOVE_DATA_CHECKBOX_FOR_GOOGLE 14043
#define IDS_APP_UNINSTALL_PROMPT_LEARN_MORE 14044
#define IDS_APP_UNINSTALL_PROMPT_ADDITIONAL_UNINSTALLS_MESSAGE 14045
#define IDS_SUB_APPS_UNINSTALL_NOTIFICATION_TITLE 14046
#define IDS_SUB_APPS_UNINSTALL_NOTIFICATION_DESCRIPTION 14047
#define IDS_UTILITY_PROCESS_SPEECH_RECOGNITION_SERVICE_NAME 14048
#define IDS_APP_PAUSE_PROMPT_TITLE 14049
#define IDS_APP_PAUSE_HEADING 14050
#define IDS_APP_PAUSE_HEADING_FOR_WEB_APPS 14051
#define IDS_APP_BLOCK_PROMPT_TITLE 14052
#define IDS_APP_BLOCK_HEADING_FOR_CHILD 14053
#define IDS_APP_BLOCK_HEADING 14054
#define IDS_APP_LOCAL_BLOCK_PROMPT_TITLE 14055
#define IDS_APP_LOCAL_BLOCK_PROMPT_MULTIPLE_TITLE 14056
#define IDS_APP_LOCAL_BLOCK_HEADING 14057
#define IDS_ENTERPRISE_EXTENSION_REQUEST_APPROVED_TITLE 14058
#define IDS_ENTERPRISE_EXTENSION_REQUEST_REJECTED_TITLE 14059
#define IDS_ENTERPRISE_EXTENSION_REQUEST_FORCE_INSTALLED_TITLE 14060
#define IDS_ENTERPRISE_EXTENSION_REQUEST_CLICK_TO_INSTALL 14061
#define IDS_ENTERPRISE_EXTENSION_REQUEST_CLICK_TO_VIEW 14062
#define IDS_ENTERPRISE_EXTENSION_REQUEST_JUSTIFICATION 14063
#define IDS_ENTERPRISE_EXTENSION_REQUEST_JUSTIFICATION_PLACEHOLDER 14064
#define IDS_ENTERPRISE_EXTENSION_REQUEST_JUSTIFICATION_LENGTH_LIMIT 14065
#define IDS_NAME_WINDOW_PROMPT_TITLE 14066
#define IDS_NAME_WINDOW_PROMPT_FIELD_LABEL 14067
#define IDS_CHROMELABS_RELAUNCH_BUTTON_LABEL 14068
#define IDS_WINDOW_TITLE_EXPERIMENTS 14069
#define IDS_CHROMELABS_SEND_FEEDBACK 14070
#define IDS_CHROMELABS_SEND_FEEDBACK_DESCRIPTION_PLACEHOLDER 14071
#define IDS_CHROMELABS_DEFAULT 14072
#define IDS_CHROMELABS_ENABLED 14073
#define IDS_CHROMELABS_DISABLED 14074
#define IDS_CHROMELABS_ENABLED_WITH_VARIATION_NAME 14075
#define IDS_LENS_REGION_SEARCH_BUBBLE_TEXT 14076
#define IDS_CAPTURE_REGION_BUBBLE_TEXT 14077
#define IDS_THUMBNAIL_TAB_STRIP_EXPERIMENT_NAME 14078
#define IDS_THUMBNAIL_TAB_STRIP_EXPERIMENT_DESCRIPTION 14079
#define IDS_TAILORED_SECURITY_UNCONSENTED_PROMOTION_NOTIFICATION_DESCRIPTION 14082
#define IDS_TAILORED_SECURITY_UNCONSENTED_PROMOTION_NOTIFICATION_ACCEPT 14083
#define IDS_TAILORED_SECURITY_CONSENTED_PROMOTION_MESSAGE_TURN_ON 14084
#define IDS_TAILORED_SECURITY_CONSENTED_ESB_OFF_MESSAGE_TITLE 14085
#define IDS_ACCOUNT_SELECTION_SHEET_TITLE_EXPLICIT_SIGN_IN 14086
#define IDS_ACCOUNT_SELECTION_SHEET_TITLE_EXPLICIT_SIGN_UP 14087
#define IDS_ACCOUNT_SELECTION_SHEET_TITLE_EXPLICIT_USE 14088
#define IDS_ACCOUNT_SELECTION_SHEET_TITLE_EXPLICIT_CONTINUE 14089
#define IDS_ACCOUNT_SELECTION_SHEET_SUBTITLE_FOR_EMBED 14090
#define IDS_ACCOUNT_SELECTION_CONTINUE 14091
#define IDS_ACCOUNT_SELECTION_CHOOSE_AN_ACCOUNT 14092
#define IDS_ACCOUNT_SELECTION_CHOOSE_AN_ACCOUNT_BUTTON 14093
#define IDS_ACCOUNT_SELECTION_DATA_SHARING_NAME 14094
#define IDS_ACCOUNT_SELECTION_DATA_SHARING_EMAIL 14095
#define IDS_ACCOUNT_SELECTION_DATA_SHARING_PICTURE 14096
#define IDS_ACCOUNT_SELECTION_DATA_SHARING_PHONE 14097
#define IDS_ACCOUNT_SELECTION_DATA_SHARING_USERNAME 14098
#define IDS_ACCOUNT_SELECTION_DATA_SHARING_CONSENT_NO_PP_OR_TOS 14099
#define IDS_ACCOUNT_SELECTION_DATA_SHARING_CONSENT_NO_PP 14100
#define IDS_ACCOUNT_SELECTION_DATA_SHARING_CONSENT_NO_TOS 14101
#define IDS_ACCOUNT_SELECTION_DATA_SHARING_CONSENT 14102
#define IDS_ACCOUNT_SELECTION_USE_OTHER_ACCOUNT 14103
#define IDS_ACCOUNT_SELECTION_USE_OTHER_ACCOUNT_MULTI_IDP 14104
#define IDS_ACCOUNT_SELECTION_OPENS_IN_NEW_TAB 14105
#define IDS_ACCOUNT_SELECTION_BACK 14106
#define IDS_SIGNIN_CONTINUE 14107
#define IDS_IDP_SIGNIN_STATUS_MISMATCH_DIALOG_BODY 14108
#define IDS_SIGNIN_GENERIC_ERROR_DIALOG_SUMMARY 14109
#define IDS_SIGNIN_GENERIC_ERROR_DIALOG_DESCRIPTION 14110
#define IDS_SIGNIN_UNAUTHORIZED_CLIENT_ERROR_DIALOG_SUMMARY 14111
#define IDS_SIGNIN_UNAUTHORIZED_CLIENT_ERROR_DIALOG_DESCRIPTION 14112
#define IDS_SIGNIN_INVALID_REQUEST_ERROR_DIALOG_SUMMARY 14113
#define IDS_SIGNIN_INVALID_REQUEST_ERROR_DIALOG_DESCRIPTION 14114
#define IDS_SIGNIN_ACCESS_DENIED_ERROR_DIALOG_SUMMARY 14115
#define IDS_SIGNIN_ACCESS_DENIED_ERROR_DIALOG_DESCRIPTION 14116
#define IDS_SIGNIN_SERVER_ERROR_DIALOG_SUMMARY 14117
#define IDS_SIGNIN_SERVER_ERROR_DIALOG_DESCRIPTION 14118
#define IDS_SIGNIN_TEMPORARILY_UNAVAILABLE_ERROR_DIALOG_SUMMARY 14119
#define IDS_SIGNIN_TEMPORARILY_UNAVAILABLE_ERROR_DIALOG_DESCRIPTION 14120
#define IDS_SIGNIN_ERROR_DIALOG_GOT_IT_BUTTON 14121
#define IDS_SIGNIN_ERROR_DIALOG_MORE_DETAILS_BUTTON 14122
#define IDS_SIGNIN_ERROR_DIALOG_MORE_DETAILS_PROMPT 14123
#define IDS_SIGNIN_ERROR_DIALOG_TRY_OTHER_WAYS_PROMPT 14124
#define IDS_SIGNIN_ERROR_DIALOG_MORE_DETAILS_RETRY_PROMPT 14125
#define IDS_SIGNIN_ERROR_DIALOG_TRY_OTHER_WAYS_RETRY_PROMPT 14126
#define IDS_MULTI_IDP_ACCOUNT_SELECTION_SHEET_TITLE_EXPLICIT 14127
#define IDS_MULTI_IDP_ACCOUNT_ORIGIN_AND_LAST_USED 14128
#define IDS_USED_ON_THIS_SITE 14129
#define IDS_VERIFY_SHEET_TITLE 14130
#define IDS_VERIFY_SHEET_TITLE_AUTO_REAUTHN 14131
#define IDS_AUTO_REAUTHN_OPTOUT_CHECKBOX 14132
#define IDS_FILTERED_ACCOUNT_MESSAGE 14133
#define IDS_FEDERATION_SIGN_IN_TITLE 14134
#define IDS_FEDERATION_SIGNING_IN_TITLE 14135
#define IDS_FEDERATION_SIGN_UP_TITLE 14136
#define IDS_BATTERY_SAVER_BUTTON_TOOLTIP 14139
#define IDS_BATTERY_SAVER_BUTTON_ACCNAME 14140
#define IDS_BATTERY_SAVER_BUBBLE_TITLE 14141
#define IDS_BATTERY_SAVER_BUBBLE_DESCRIPTION 14142
#define IDS_BATTERY_SAVER_SESSION_TURN_OFF 14143
#define IDS_MEMORY_SAVER_CHIP_LABEL 14144
#define IDS_MEMORY_SAVER_CHIP_ACCNAME 14145
#define IDS_MEMORY_SAVER_CHIP_WITH_SAVINGS_ACCNAME 14146
#define IDS_MEMORY_SAVER_CHIP_SAVINGS_LABEL 14147
#define IDS_MEMORY_SAVER_DIALOG_TITLE 14148
#define IDS_MEMORY_SAVER_DIALOG_SETTINGS_BUTTON 14149
#define IDS_MEMORY_SAVER_DIALOG_SAVINGS_ACCNAME 14150
#define IDS_MEMORY_SAVER_DIALOG_BUTTON_ADD_TO_EXCLUSION_LIST 14151
#define IDS_MEMORY_SAVER_DIALOG_SAVINGS_LABEL 14152
#define IDS_MEMORY_SAVER_DIALOG_SMALL_SAVINGS_LABEL 14153
#define IDS_MEMORY_SAVER_DIALOG_MEDIUM_SAVINGS_LABEL 14154
#define IDS_MEMORY_SAVER_DIALOG_LARGE_SAVINGS_LABEL 14155
#define IDS_MEMORY_SAVER_DIALOG_VERY_LARGE_SAVINGS_LABEL 14156
#define IDS_HOVERCARD_TAB_MEMORY_USAGE 14157
#define IDS_HOVERCARD_TAB_HIGH_MEMORY_USAGE 14158
#define IDS_HOVERCARD_INACTIVE_TAB_MEMORY_SAVINGS 14159
#define IDS_HOVERCARD_INACTIVE_TAB 14160
#define IDS_PERFORMANCE_INTERVENTION_CLOSE_BUTTON_ACCNAME 14161
#define IDS_PERFORMANCE_INTERVENTION_SINGLE_SUGGESTED_ROW_ACCNAME 14162
#define IDS_PERFORMANCE_INTERVENTION_TAB_LIST_ACCNAME 14163
#define IDS_PERFORMANCE_INTERVENTION_DIALOG_TITLE_V1 14164
#define IDS_PERFORMANCE_INTERVENTION_DIALOG_TITLE_UPDATED_V1 14165
#define IDS_PERFORMANCE_INTERVENTION_DIALOG_TITLE_UPDATED_V2 14166
#define IDS_PERFORMANCE_INTERVENTION_DEACTIVATE_TABS_BUTTON_V1 14167
#define IDS_PERFORMANCE_INTERVENTION_DEACTIVATE_TABS_BUTTON_UPDATED_V1 14168
#define IDS_PERFORMANCE_INTERVENTION_DEACTIVATE_TABS_BUTTON_UPDATED_V2 14169
#define IDS_PERFORMANCE_INTERVENTION_DISMISS_BUTTON 14170
#define IDS_PERFORMANCE_INTERVENTION_BUTTON_TOOLTIP 14171
#define IDS_PERFORMANCE_INTERVENTION_BUTTON_ACCNAME 14172
#define IDS_TOAST_MENU_BUTTON_NAME 14173
#define IDS_TRANSLATE_TOAST_BODY 14174
#define IDS_TRANSLATE_TOAST_UNDO_BUTTON 14175
#define IDS_MULTISTEP_FILTER_SUGGESTION_TITLE 14176
#define IDS_MULTISTEP_FILTER_SUGGESTION_APPLY_BUTTON 14177
#define IDS_MULTISTEP_FILTER_SUGGESTION_RECENT_TITLE 14178
#define IDS_MULTISTEP_FILTER_SUGGESTION_RECENT_APPLY_BUTTON 14179
#define IDS_MULTISTEP_FILTER_CUE_ACTION_TEXT 14180
#define IDS_LINK_COPIED_TOAST_BODY 14181
#define IDS_IMAGE_COPIED_TOAST_BODY 14182
#define IDS_VIDEO_FRAME_COPIED_TOAST_BODY 14183
#define IDS_LINK_COPIED_TO_HIGHLIGHT_TOAST_BODY 14184
#define IDS_READING_LIST_TOAST_BODY 14185
#define IDS_READING_LIST_TOAST_BUTTON 14186
#define IDS_CLOSE_PINNED_TAB_TOAST_BODY 14187
#define IDS_TOAST_CLOSE_TOOLTIP 14188
#define IDS_NON_MILESTONE_UPDATE_TOAST_BODY 14189
#define IDS_GLIC_SHARE_IMAGE_FAILED_TOAST_BODY 14190
#define IDS_GLIC_SELECTION_OVERLAY_RENDERER_LABEL 14191
#define IDS_GLIC_SELECTION_OVERLAY_PRESELECTION_BUBBLE_TEXT 14192
#define IDS_COPIED_TO_CLIPBOARD_TOAST_BODY 14193
#define IDS_SPLIT_VIEW_CREATED_TOAST_BODY 14194
#define IDS_SPLIT_VIEW_SEPARATE_VIEWS_ACTION_BUTTON 14195
#define IDS_SPLIT_VIEW_UNDO_SPLIT_ACTION_BUTTON 14196
#define IDS_PROTECTED_AUDIENCE_DEBUGGING_DISCLAIMER 14200
#define IDS_OVERFLOW_MENU_ITEM_TEXT_PROFILE 14201
#define IDS_OVERFLOW_MENU_ITEM_TEXT_DOWNLOADS 14202
#define IDS_OVERFLOW_MENU_ITEM_TEXT_EXTENSIONS 14203
#define IDS_OVERFLOW_MENU_ITEM_TEXT_FORWARD 14204
#define IDS_OVERFLOW_MENU_ITEM_TEXT_HOME 14205
#define IDS_OVERFLOW_MENU_ITEM_TEXT_LABS 14206
#define IDS_OVERFLOW_MENU_ITEM_TEXT_MEDIA_CONTROLS 14207
#define IDS_OVERFLOW_MENU_ITEM_TEXT_SIDE_PANEL 14208
#define IDS_OVERFLOW_MENU_ITEM_TEXT_SPLIT_VIEW 14209
#define IDS_OVERFLOW_MENU_ITEM_TEXT_CONTEXTUAL_TASKS 14210
#define IDS_OVERFLOW_MENU_ITEM_TEXT_ENERGY_SAVER 14211
#define IDS_PROMISE_STATUS_WAITING 14212
#define IDS_PROMISE_STATUS_INSTALLING 14213
#define IDS_PROMISE_APP_ACCESSIBLE_LABEL_WAITING 14214
#define IDS_PROMISE_APP_ACCESSIBLE_LABEL_INSTALLING 14215
#define IDS_PROMISE_APP_PLACEHOLDER_ACCESSIBLE_LABEL_WAITING 14216
#define IDS_PROMISE_APP_PLACEHOLDER_ACCESSIBLE_LABEL_INSTALLING 14217
#define IDS_IWA_INSTALLER_DISABLED_TITLE 14218
#define IDS_IWA_INSTALLER_DISABLED_CHANGE_PREFERENCE 14219
#define IDS_IWA_INSTALLER_DISABLED_SUBTITLE 14220
#define IDS_IWA_INSTALLER_ALREADY_INSTALLED_TITLE 14221
#define IDS_IWA_INSTALLER_ALREADY_INSTALLED_SUBTITLE 14222
#define IDS_IWA_INSTALLER_BODY_SCREENREADER_NAME 14223
#define IDS_IWA_INSTALLER_DETAILS_SCREENREADER_NAME 14224
#define IDS_IWA_INSTALLER_PROGRESS_SCREENREADER_NAME 14225
#define IDS_IWA_INSTALLER_INSTALL_SUBTITLE 14226
#define IDS_IWA_INSTALLER_INSTALL_PROGRESS 14227
#define IDS_IWA_INSTALLER_SHOW_METADATA_SUBTITLE 14228
#define IDS_IWA_INSTALLER_SHOW_METADATA_APP_NAME_LABEL 14229
#define IDS_IWA_INSTALLER_SHOW_METADATA_APP_VERSION_LABEL 14230
#define IDS_IWA_INSTALLER_SHOW_METADATA_APP_ENTERPRISE_NAME_LABEL 14231
#define IDS_IWA_INSTALLER_UPDATE_SETTINGS 14232
#define IDS_IWA_INSTALLER_UPDATE_CHANNEL 14233
#define IDS_IWA_INSTALLER_CONFIRM_TITLE 14234
#define IDS_IWA_INSTALLER_CONFIRM_SUBTITLE 14235
#define IDS_IWA_INSTALLER_CONFIRM_CONTINUE 14236
#define IDS_IWA_INSTALLER_INSTALL_FAILED_TITLE 14237
#define IDS_IWA_INSTALLER_INSTALL_FAILED_SUBTITLE 14238
#define IDS_IWA_INSTALLER_INSTALL_FAILED_BUNDLE_BLOCKED_USER_INSTALL_ALLOWLIST_BLOCKLIST_TITLE 14239
#define IDS_IWA_INSTALLER_INSTALL_FAILED_BUNDLE_NOT_ON_USER_INSTALL_ALLOWLIST_MESSAGE 14240
#define IDS_IWA_INSTALLER_INSTALL_FAILED_BUNDLE_BLOCKLISTED_MESSAGE 14241
#define IDS_IWA_INSTALLER_INSTALL_FAILED_RETRY 14242
#define IDS_IWA_INSTALLER_SUCCESS_SUBTITLE 14243
#define IDS_IWA_INSTALLER_SUCCESS_FINISH 14244
#define IDS_IWA_INSTALLER_SUCCESS_LAUNCH_APPLICATION 14245
#define IDS_IWA_INSTALLER_VERIFICATION_TITLE 14246
#define IDS_IWA_INSTALLER_VERIFICATION_SUBTITLE 14247
#define IDS_IWA_INSTALLER_VERIFICATION_STATUS 14248
#define IDS_IWA_INSTALLER_VERIFICATION_ERROR_TITLE 14249
#define IDS_IWA_INSTALLER_VERIFICATION_ERROR_SUBTITLE 14250
#define IDS_ISOLATED_WEB_APPS_WINDOW_OPEN_PERMISSION_NOTIFICATION_TITLE 14252
#define IDS_ISOLATED_WEB_APPS_WINDOW_OPEN_PERMISSION_NOTIFICATION_MESSAGE 14253
#define IDS_ISOLATED_WEB_APPS_WINDOW_OPEN_PERMISSION_NOTIFICATION_BUTTON_SETTINGS 14254
#define IDS_THUMBS_DOWN 14261
#define IDS_THUMBS_UP 14262
#define IDS_EXPERIMENTAL_FEATURE_DISCLAIMER 14263
#define IDS_PDF_INK_SIGNATURES_IPH_BODY 14264
#define IDS_PDF_SEARCHIFY_IPH_TITLE 14265
#define IDS_PDF_SEARCHIFY_IPH_BODY 14266
#define IDS_PDF_SEARCHIFY_IPH_BODY_SCREEN_READER 14267
#define IDS_SELECT_AUDIO_OUTPUT_DIALOG_TITLE 14268
#define IDS_SELECT_AUDIO_OUTPUT_DIALOG_CHOOSE_DEVICE 14269
#define IDS_CHROME_NO_LONGER_SHOW_WARNINGS_NOTIFICATION_TITLE 14285
#define IDS_CHROME_NO_LONGER_SHOW_WARNINGS_NOTIFICATION_BODY 14286
#define IDS_CHROME_NO_LONGER_SHOW_WARNINGS_NOTIFICATION_TITLE_NEW 14287
#define IDS_CHROME_NO_LONGER_SHOW_WARNINGS_NOTIFICATION_BODY_NEW 14288
#define IDS_PDF_GLIC_SUMMARIZE 14293
#define IDS_PDF_GLIC_SUMMARIZE_WITH_GEMINI 14294
#define IDS_PDF_GLIC_SUMMARIZE_TOOLTIP 14295
#define IDS_PDF_GLIC_SUMMARIZE_PROMPT 14296
#define IDS_PDF_GLIC_SUMMARIZE_IPH_TITLE 14297
#define IDS_PDF_GLIC_SUMMARIZE_IPH_GOT_IT 14298
#define IDS_NTP_PROMO_DISMISS 14307
#define IDS_NTP_PROMO_MENU_TOOLTIP 14308
#define IDS_NTP_PROMO_MENU_A11Y_LABEL 14309
#define IDS_NTP_EXTENSIONS_PROMO 14310
#define IDS_ACCOUNT_CHOOSER_SINGLE_ACCOUNT_TITLE 14311
#define IDS_ACCOUNT_CHOOSER_MULTI_ACCOUNT_TITLE 14312
#define IDS_ACCOUNT_CHOOSER_SUBTITLE 14313
#define IDS_SAVE_TO_DRIVE_FOLDER_NAME 14314
#define IDS_ACCOUNT_CHOOSER_DRIVE 14315
#define IDS_ACCOUNT_CHOOSER_ADD_ACCOUNT 14316
#define IDS_ACCOUNT_CHOOSER_RADIO_GROUP_ACCESSIBILITY_LABEL 14317
#define IDS_ACCOUNT_CHOOSER_HEADER_ACCESSIBILITY_LABEL 14318
#define IDS_TIPS_NOTIFICATIONS_HELPFUL_BUTTON_TEXT 14319
#define IDS_TIPS_NOTIFICATIONS_ENHANCED_SAFE_BROWSING_TITLE 14320
#define IDS_TIPS_NOTIFICATIONS_ENHANCED_SAFE_BROWSING_SUBTITLE 14321
#define IDS_TIPS_NOTIFICATIONS_QUICK_DELETE_TITLE 14322
#define IDS_TIPS_NOTIFICATIONS_QUICK_DELETE_SUBTITLE 14323
#define IDS_TIPS_NOTIFICATIONS_GOOGLE_LENS_TITLE 14324
#define IDS_TIPS_NOTIFICATIONS_GOOGLE_LENS_SUBTITLE 14325
#define IDS_TIPS_NOTIFICATIONS_BOTTOM_OMNIBOX_TITLE 14326
#define IDS_TIPS_NOTIFICATIONS_BOTTOM_OMNIBOX_SUBTITLE 14327
#define IDS_TIPS_NOTIFICATIONS_PASSWORD_AUTOFILL_TITLE 14328
#define IDS_TIPS_NOTIFICATIONS_PASSWORD_AUTOFILL_SUBTITLE 14329
#define IDS_TIPS_NOTIFICATIONS_SIGNIN_TITLE 14330
#define IDS_TIPS_NOTIFICATIONS_SIGNIN_SUBTITLE 14331
#define IDS_TIPS_NOTIFICATIONS_CREATE_TAB_GROUPS_TITLE 14332
#define IDS_TIPS_NOTIFICATIONS_CREATE_TAB_GROUPS_SUBTITLE 14333
#define IDS_TIPS_NOTIFICATIONS_CUSTOMIZE_MVT_TITLE 14334
#define IDS_TIPS_NOTIFICATIONS_CUSTOMIZE_MVT_SUBTITLE 14335
#define IDS_TIPS_NOTIFICATIONS_RECENT_TABS_TITLE 14336
#define IDS_TIPS_NOTIFICATIONS_RECENT_TABS_SUBTITLE 14337
#define IDS_UPDATER_ACTIVATION_FAILED 14338
#define IDS_UPDATER_ACTIVATION_SUCCEEDED 14339
#define IDS_UPDATER_ADD_FILTER 14340
#define IDS_UPDATER_APP 14341
#define IDS_UPDATER_APP_COLUMN 14342
#define IDS_UPDATER_APPLY 14343
#define IDS_UPDATER_APP_NAME_OR_ID 14344
#define IDS_UPDATER_APP_POLICIES 14345
#define IDS_UPDATER_APP_STATES_QUERY_FAILED 14346
#define IDS_UPDATER_CANCEL 14347
#define IDS_UPDATER_CLEAR_ALL_FILTERS 14348
#define IDS_UPDATER_COLLAPSE_ALL 14349
#define IDS_UPDATER_COMMAND_LINE 14350
#define IDS_UPDATER_COMMAND_OUTCOME 14351
#define IDS_UPDATER_COMMON 14352
#define IDS_UPDATER_DATE 14353
#define IDS_UPDATER_DATE_FILTER_AFTER 14354
#define IDS_UPDATER_DATE_FILTER_BEFORE 14355
#define IDS_UPDATER_DATE_FILTER_RANGE 14356
#define IDS_UPDATER_DISPLAYED_EVENTS_COUNT 14357
#define IDS_UPDATER_DURATION 14358
#define IDS_UPDATER_EFFECTIVE_POLICY_SET 14359
#define IDS_UPDATER_END_DATE 14360
#define IDS_UPDATER_ERROR_ACCESS_DENIED 14361
#define IDS_UPDATER_ERROR_CORRUPT 14362
#define IDS_UPDATER_ERROR_DISABLED 14363
#define IDS_UPDATER_ERROR_DISK_FULL 14364
#define IDS_UPDATER_ERROR_DOWNLOAD 14365
#define IDS_UPDATER_ERROR_INSTALL 14366
#define IDS_UPDATER_ERROR_INSTALLER_FAILED 14367
#define IDS_UPDATER_ERROR_NETWORK 14368
#define IDS_UPDATER_ERROR_RESTRICTED 14369
#define IDS_UPDATER_ERROR_UNPACK 14370
#define IDS_UPDATER_ERROR_UPDATE_CHECK 14371
#define IDS_UPDATER_ERROR_UNKNOWN 14372
#define IDS_UPDATER_EVENT_LIST_TITLE 14373
#define IDS_UPDATER_EVENT_TYPE 14374
#define IDS_UPDATER_EVENT_TYPE_ACTIVATE 14375
#define IDS_UPDATER_EVENT_TYPE_APP_COMMAND 14376
#define IDS_UPDATER_EVENT_TYPE_INSTALL 14377
#define IDS_UPDATER_EVENT_TYPE_LOAD_POLICY 14378
#define IDS_UPDATER_EVENT_TYPE_PERSISTED_DATA 14379
#define IDS_UPDATER_EVENT_TYPE_POST_REQUEST 14380
#define IDS_UPDATER_EVENT_TYPE_QUALIFY 14381
#define IDS_UPDATER_EVENT_TYPE_UNINSTALL 14382
#define IDS_UPDATER_EVENT_TYPE_UPDATE 14383
#define IDS_UPDATER_EVENT_TYPE_UPDATER_PROCESS 14384
#define IDS_UPDATER_EXPAND_ALL 14385
#define IDS_UPDATER_EXPORT_HISTORY_FILE 14386
#define IDS_UPDATER_FILTER_CHIP_APP 14387
#define IDS_UPDATER_FILTER_CHIP_DATE 14388
#define IDS_UPDATER_FILTER_CHIP_EVENT_TYPE 14389
#define IDS_UPDATER_FILTER_CHIP_UPDATE_OUTCOME 14390
#define IDS_UPDATER_FILTER_CHIP_UPDATER_SCOPE 14391
#define IDS_UPDATER_HELP_CENTER_TOOLTIP 14392
#define IDS_UPDATER_INACTIVE_VERSIONS_LABEL 14393
#define IDS_UPDATER_INSTALL_PATH_LABEL 14394
#define IDS_UPDATER_INSTALL_SUMMARY 14395
#define IDS_UPDATER_INSTALLED_APPS_TITLE 14396
#define IDS_UPDATER_INTERNAL 14397
#define IDS_UPDATER_LAST_CHECKED_LABEL 14398
#define IDS_UPDATER_LAST_STARTED_LABEL 14399
#define IDS_UPDATER_LEARN_MORE 14400
#define IDS_UPDATER_LOAD_HISTORY_FILE 14401
#define IDS_UPDATER_LOAD_HISTORY_FILE_ERROR 14402
#define IDS_UPDATER_NO_APPS_FOUND 14403
#define IDS_UPDATER_NO_POLICIES 14404
#define IDS_UPDATER_NO_UPDATER_FOUND 14405
#define IDS_UPDATER_NEVER 14406
#define IDS_UPDATER_NEXT_VERSION 14407
#define IDS_UPDATER_NO_UPDATE 14408
#define IDS_UPDATER_OMAHA_REQUEST 14409
#define IDS_UPDATER_OMAHA_RESPONSE 14410
#define IDS_UPDATER_OTHER 14411
#define IDS_UPDATER_OUTCOME 14412
#define IDS_UPDATER_OUTCOME_UNKNOWN 14413
#define IDS_UPDATER_PAGE_TITLE 14414
#define IDS_UPDATER_PARSE_ERROR_EVENTS 14415
#define IDS_UPDATER_PERSISTED_DATA_SUMMARY 14416
#define IDS_UPDATER_POLICY_CONFLICT_WARNING 14417
#define IDS_UPDATER_POLICY_IGNORED 14418
#define IDS_UPDATER_POLICY_NAME 14419
#define IDS_UPDATER_POLICY_OK 14420
#define IDS_UPDATER_POLICY_SOURCE 14421
#define IDS_UPDATER_POLICY_STATUS 14422
#define IDS_UPDATER_POLICY_VALUE 14423
#define IDS_UPDATER_PROCESS_SUMMARY 14424
#define IDS_UPDATER_QUALIFICATION_FAILED 14425
#define IDS_UPDATER_QUALIFICATION_SUCCEEDED 14426
#define IDS_UPDATER_QUERY_FAILED 14427
#define IDS_UPDATER_REMOVE_FILTER 14428
#define IDS_UPDATER_RETURN_TO_LOCAL 14429
#define IDS_UPDATER_SCOPE 14430
#define IDS_UPDATER_SCOPE_SYSTEM 14431
#define IDS_UPDATER_SCOPE_USER 14432
#define IDS_UPDATER_START_DATE 14433
#define IDS_UPDATER_STATE_TITLE 14434
#define IDS_UPDATER_UNDATED_EVENTS 14435
#define IDS_UPDATER_UNINSTALL_SUMMARY 14436
#define IDS_UPDATER_UPDATED_TO 14437
#define IDS_UPDATER_UPDATER_POLICIES 14438
#define IDS_UPDATER_UPDATER_VERSION 14439
#define IDS_UPDATER_VERSION_COLUMN 14440
#define IDS_UPDATER_VERSION_LABEL 14441
#define IDS_UPDATER_VIEWING_HISTORY_FILES 14442
#define IDS_UPDATER_UPDATE_ERROR 14443
#define IDS_UPDATER_UPDATE_OUTCOME 14444
#define IDS_UPDATER_UPDATE_OUTCOME_NO_UPDATE 14445
#define IDS_UPDATER_UPDATE_OUTCOME_UPDATED 14446
#define IDS_UPDATER_UPDATE_OUTCOME_UPDATE_ERROR 14447
#define IDS_UPDATER_VIEW_ALL_CHROME_POLICIES 14448
#define IDS_UPDATER_VIEW_RAW_DETAILS 14449
#define IDS_REPORT_UNSAFE_SITE_DIALOG_INCLUDE_SCREENSHOT_CHECKBOX_LABEL 14450
#define IDS_REPORT_UNSAFE_SITE_DIALOG_SEND_BUTTON_LABEL 14451
#define IDS_REPORT_UNSAFE_SITE_DIALOG_URL_LABEL 14452
#define IDS_REPORT_UNSAFE_SITE_DIALOG_SENDING_BUTTON_LABEL 14453
#define IDS_REPORT_UNSAFE_SITE_CONFIRMATION_TOAST 14454
#define IDS_REPORT_UNSAFE_SITE_A11Y_IMAGE_TEXT 14455
#define IDS_CHROME_FINDS_NOTIFICATIONS_HELPFUL_BUTTON_TEXT 14456
#define IDS_CHROME_FINDS_NOTIFICATIONS_UNHELPFUL_BUTTON_TEXT 14457
#define IDS_OSCRYPTASYNC_AVAILABILITY_INFOBAR_MESSAGE 14458
#define IDS_OSCRYPTASYNC_AVAILABILITY_INFOBAR_BUTTON 14459

// ---------------------------------------------------------------------------
// From locale_settings.h:

#define IDS_SPELLCHECK_DICTIONARY 15000
#define IDS_EDITBOOKMARK_DIALOG_WIDTH_CHARS 15001
#define IDS_EDITBOOKMARK_DIALOG_HEIGHT_LINES 15002
#define IDS_WEBSTORE_URL 15003
#define IDS_MEDIA_GALLERIES_DIALOG_CONTENT_WIDTH_CHARS 15004

// ---------------------------------------------------------------------------
// From omnibox_pedal_synonyms.h:

#define IDS_OMNIBOX_PEDALS_IGNORE_GROUP 40170
#define IDS_OMNIBOX_PEDAL_SYNONYMS_CLEAR_BROWSING_DATA_ONE_REQUIRED_CLEAR_BROWSER_CACHE 40171
#define IDS_OMNIBOX_PEDAL_SYNONYMS_CLEAR_BROWSING_DATA_ONE_OPTIONAL_GOOGLE_CHROME 40172
#define IDS_OMNIBOX_PEDAL_SYNONYMS_CLEAR_BROWSING_DATA_ONE_REQUIRED_DELETE 40173
#define IDS_OMNIBOX_PEDAL_SYNONYMS_CLEAR_BROWSING_DATA_ONE_REQUIRED_INFORMATION 40174
#define IDS_OMNIBOX_PEDAL_SYNONYMS_MANAGE_PASSWORDS_ONE_REQUIRED_MANAGE_CHROME_PASSWORDS 40175
#define IDS_OMNIBOX_PEDAL_SYNONYMS_MANAGE_PASSWORDS_ONE_OPTIONAL_GOOGLE_CHROME 40176
#define IDS_OMNIBOX_PEDAL_SYNONYMS_MANAGE_PASSWORDS_ONE_OPTIONAL_MANAGER 40177
#define IDS_OMNIBOX_PEDAL_SYNONYMS_MANAGE_PASSWORDS_ONE_REQUIRED_PASSWORDS 40178
#define IDS_OMNIBOX_PEDAL_SYNONYMS_UPDATE_CREDIT_CARD_ONE_REQUIRED_MANAGE_PAYMENT_METHODS 40179
#define IDS_OMNIBOX_PEDAL_SYNONYMS_UPDATE_CREDIT_CARD_ONE_OPTIONAL_GOOGLE_CHROME 40180
#define IDS_OMNIBOX_PEDAL_SYNONYMS_UPDATE_CREDIT_CARD_ONE_REQUIRED_CHANGE 40181
#define IDS_OMNIBOX_PEDAL_SYNONYMS_UPDATE_CREDIT_CARD_ONE_REQUIRED_CREDIT_CARD_INFORMATION 40182
#define IDS_OMNIBOX_PEDAL_SYNONYMS_LAUNCH_INCOGNITO_ONE_REQUIRED_ENTER_INCOGNITO_MODE 40183
#define IDS_OMNIBOX_PEDAL_SYNONYMS_LAUNCH_INCOGNITO_ONE_OPTIONAL_GOOGLE_CHROME 40184
#define IDS_OMNIBOX_PEDAL_SYNONYMS_LAUNCH_INCOGNITO_ONE_OPTIONAL_CREATE 40185
#define IDS_OMNIBOX_PEDAL_SYNONYMS_LAUNCH_INCOGNITO_ONE_REQUIRED_INCOGNITO_WINDOW 40186
#define IDS_OMNIBOX_PEDAL_SYNONYMS_TRANSLATE_ONE_REQUIRED_TRANSLATE_THIS_PAGE 40187
#define IDS_OMNIBOX_PEDAL_SYNONYMS_TRANSLATE_ONE_OPTIONAL_GOOGLE_CHROME 40188
#define IDS_OMNIBOX_PEDAL_SYNONYMS_TRANSLATE_ONE_REQUIRED_CHANGE_LANGUAGE 40189
#define IDS_OMNIBOX_PEDAL_SYNONYMS_TRANSLATE_ONE_REQUIRED_THIS_PAGE 40190
#define IDS_OMNIBOX_PEDAL_SYNONYMS_UPDATE_CHROME_ONE_REQUIRED_UPDATE_CHROME 40191
#define IDS_OMNIBOX_PEDAL_SYNONYMS_UPDATE_CHROME_ONE_REQUIRED_GOOGLE_CHROME 40192
#define IDS_OMNIBOX_PEDAL_SYNONYMS_UPDATE_CHROME_ONE_REQUIRED_INSTALL 40193
#define IDS_OMNIBOX_PEDAL_SYNONYMS_RUN_CHROME_SAFETY_CHECK_ONE_REQUIRED_RUN_CHROME_SAFETY_CHECK 40194
#define IDS_OMNIBOX_PEDAL_SYNONYMS_RUN_CHROME_SAFETY_CHECK_ONE_OPTIONAL_ACTIVATE 40195
#define IDS_OMNIBOX_PEDAL_SYNONYMS_RUN_CHROME_SAFETY_CHECK_ONE_OPTIONAL_GOOGLE_CHROME 40196
#define IDS_OMNIBOX_PEDAL_SYNONYMS_RUN_CHROME_SAFETY_CHECK_ONE_REQUIRED_CHECKUP 40197
#define IDS_OMNIBOX_PEDAL_SYNONYMS_RUN_CHROME_SAFETY_CHECK_ONE_REQUIRED_PASSWORDS 40198
#define IDS_OMNIBOX_PEDAL_SYNONYMS_MANAGE_SECURITY_SETTINGS_ONE_REQUIRED_MANAGE_SECURITY_SETTINGS 40199
#define IDS_OMNIBOX_PEDAL_SYNONYMS_MANAGE_SECURITY_SETTINGS_ANY_OPTIONAL_GOOGLE_CHROME 40200
#define IDS_OMNIBOX_PEDAL_SYNONYMS_MANAGE_SECURITY_SETTINGS_ONE_REQUIRED_ENHANCED_PROTECTION 40201
#define IDS_OMNIBOX_PEDAL_SYNONYMS_MANAGE_SECURITY_SETTINGS_ONE_OPTIONAL_ALTER 40202
#define IDS_OMNIBOX_PEDAL_SYNONYMS_MANAGE_COOKIES_ONE_REQUIRED_CHROME_COOKIE_SETTINGS 40203
#define IDS_OMNIBOX_PEDAL_SYNONYMS_MANAGE_COOKIES_ONE_REQUIRED_COOKIE_SETTINGS 40204
#define IDS_OMNIBOX_PEDAL_SYNONYMS_MANAGE_COOKIES_ANY_REQUIRED_GOOGLE_CHROME 40205
#define IDS_OMNIBOX_PEDAL_SYNONYMS_MANAGE_COOKIES_ANY_OPTIONAL_THIRD_PARTY 40206
#define IDS_OMNIBOX_PEDAL_SYNONYMS_MANAGE_ADDRESSES_ONE_REQUIRED_ADD_ADDRESS 40207
#define IDS_OMNIBOX_PEDAL_SYNONYMS_MANAGE_ADDRESSES_ONE_REQUIRED_CONTROL 40208
#define IDS_OMNIBOX_PEDAL_SYNONYMS_MANAGE_ADDRESSES_ONE_REQUIRED_SHIPPING_ADDRESSES 40209
#define IDS_OMNIBOX_PEDAL_SYNONYMS_MANAGE_ADDRESSES_ONE_OPTIONAL_GOOGLE_CHROME 40210
#define IDS_OMNIBOX_PEDAL_SYNONYMS_MANAGE_SYNC_ONE_REQUIRED_CHANGE_CHROME_BROWSER_SYNC_SETTINGS 40211
#define IDS_OMNIBOX_PEDAL_SYNONYMS_MANAGE_SYNC_ONE_REQUIRED_SYNC_SETTINGS 40212
#define IDS_OMNIBOX_PEDAL_SYNONYMS_MANAGE_SYNC_ONE_REQUIRED_SYNC_SETTINGS_UPDATED 40213
#define IDS_OMNIBOX_PEDAL_SYNONYMS_MANAGE_SYNC_ANY_REQUIRED_GOOGLE_CHROME 40214
#define IDS_OMNIBOX_PEDAL_SYNONYMS_MANAGE_SYNC_UPDATED 40215
#define IDS_OMNIBOX_PEDAL_SYNONYMS_MANAGE_SITE_SETTINGS_ONE_REQUIRED_ADJUST_SITE_PERMISSIONS 40216
#define IDS_OMNIBOX_PEDAL_SYNONYMS_MANAGE_SITE_SETTINGS_ONE_REQUIRED_SITE_PERMISSIONS 40217
#define IDS_OMNIBOX_PEDAL_SYNONYMS_MANAGE_SITE_SETTINGS_ANY_REQUIRED_GOOGLE_CHROME 40218
#define IDS_OMNIBOX_PEDAL_SYNONYMS_CREATE_GOOGLE_DOC_ONE_REQUIRED_CREATE_GOOGLE_DOC 40219
#define IDS_OMNIBOX_PEDAL_SYNONYMS_CREATE_GOOGLE_DOC_ONE_REQUIRED_GOOGLE_WORKSPACE 40220
#define IDS_OMNIBOX_PEDAL_SYNONYMS_CREATE_GOOGLE_DOC_ANY_REQUIRED_CREATE 40221
#define IDS_OMNIBOX_PEDAL_SYNONYMS_CREATE_GOOGLE_DOC_ONE_REQUIRED_DOCUMENT 40222
#define IDS_OMNIBOX_PEDAL_SYNONYMS_CREATE_GOOGLE_SHEET_ONE_REQUIRED_CREATE_GOOGLE_SHEET 40223
#define IDS_OMNIBOX_PEDAL_SYNONYMS_CREATE_GOOGLE_SHEET_ANY_REQUIRED_CREATE 40224
#define IDS_OMNIBOX_PEDAL_SYNONYMS_CREATE_GOOGLE_SHEET_ONE_REQUIRED_GOOGLE_WORKSPACE 40225
#define IDS_OMNIBOX_PEDAL_SYNONYMS_CREATE_GOOGLE_SHEET_ONE_REQUIRED_SPREADSHEET 40226
#define IDS_OMNIBOX_PEDAL_SYNONYMS_CREATE_GOOGLE_SLIDE_ONE_REQUIRED_CREATE_GOOGLE_SLIDE 40227
#define IDS_OMNIBOX_PEDAL_SYNONYMS_CREATE_GOOGLE_SLIDE_ONE_REQUIRED_CREATE 40228
#define IDS_OMNIBOX_PEDAL_SYNONYMS_CREATE_GOOGLE_SLIDE_ONE_REQUIRED_PRESENTATION 40229
#define IDS_OMNIBOX_PEDAL_SYNONYMS_CREATE_GOOGLE_SLIDE_ANY_REQUIRED_WORKSPACE 40230
#define IDS_OMNIBOX_PEDAL_SYNONYMS_CREATE_GOOGLE_CALENDAR_EVENT_ONE_REQUIRED_CREATE_GOOGLE_CAL_INVITE 40231
#define IDS_OMNIBOX_PEDAL_SYNONYMS_CREATE_GOOGLE_CALENDAR_EVENT_ANY_REQUIRED_SCHEDULE 40232
#define IDS_OMNIBOX_PEDAL_SYNONYMS_CREATE_GOOGLE_CALENDAR_EVENT_ANY_REQUIRED_WORKSPACE 40233
#define IDS_OMNIBOX_PEDAL_SYNONYMS_CREATE_GOOGLE_CALENDAR_EVENT_ONE_REQUIRED_MEETING 40234
#define IDS_OMNIBOX_PEDAL_SYNONYMS_CREATE_GOOGLE_SITE_ONE_REQUIRED_CREATE_GOOGLE_SITE 40235
#define IDS_OMNIBOX_PEDAL_SYNONYMS_CREATE_GOOGLE_SITE_ANY_REQUIRED_CREATE 40236
#define IDS_OMNIBOX_PEDAL_SYNONYMS_CREATE_GOOGLE_SITE_ANY_REQUIRED_WORKSPACE 40237
#define IDS_OMNIBOX_PEDAL_SYNONYMS_CREATE_GOOGLE_SITE_ONE_REQUIRED_WEBSITE 40238
#define IDS_OMNIBOX_PEDAL_SYNONYMS_CREATE_GOOGLE_KEEP_NOTE_ONE_REQUIRED_CREATE_GOOGLE_KEEP_NOTE 40239
#define IDS_OMNIBOX_PEDAL_SYNONYMS_CREATE_GOOGLE_KEEP_NOTE_ANY_REQUIRED_CREATE 40240
#define IDS_OMNIBOX_PEDAL_SYNONYMS_CREATE_GOOGLE_KEEP_NOTE_ANY_REQUIRED_WORKSPACE 40241
#define IDS_OMNIBOX_PEDAL_SYNONYMS_CREATE_GOOGLE_KEEP_NOTE_ONE_REQUIRED_NOTES 40242
#define IDS_OMNIBOX_PEDAL_SYNONYMS_CREATE_GOOGLE_FORM_ONE_REQUIRED_CREATE_GOOGLE_FORM 40243
#define IDS_OMNIBOX_PEDAL_SYNONYMS_CREATE_GOOGLE_FORM_ANY_REQUIRED_CREATE 40244
#define IDS_OMNIBOX_PEDAL_SYNONYMS_CREATE_GOOGLE_FORM_ANY_REQUIRED_WORKSPACE 40245
#define IDS_OMNIBOX_PEDAL_SYNONYMS_CREATE_GOOGLE_FORM_ONE_REQUIRED_SURVEY 40246
#define IDS_OMNIBOX_PEDAL_SYNONYMS_SEE_CHROME_TIPS_ONE_REQUIRED_CHROME_FEATURES 40247
#define IDS_OMNIBOX_PEDAL_SYNONYMS_SEE_CHROME_TIPS_ONE_OPTIONAL_MAKE_THE_MOST_OF 40248
#define IDS_OMNIBOX_PEDAL_SYNONYMS_SEE_CHROME_TIPS_ANY_OPTIONAL_BROWSER 40249
#define IDS_OMNIBOX_PEDAL_SYNONYMS_SEE_CHROME_TIPS_ONE_REQUIRED_NEW_CHROME_FEATURES 40250
#define IDS_OMNIBOX_PEDAL_SYNONYMS_MANAGE_GOOGLE_ACCOUNT_ONE_REQUIRED_CONTROL_MY_GOOGLE_ACCOUNT 40251
#define IDS_OMNIBOX_PEDAL_SYNONYMS_MANAGE_GOOGLE_ACCOUNT_ONE_REQUIRED_GOOGLE_ACCOUNT 40252
#define IDS_OMNIBOX_PEDAL_SYNONYMS_MANAGE_GOOGLE_ACCOUNT_ANY_OPTIONAL_BROWSER 40253
#define IDS_OMNIBOX_PEDAL_SYNONYMS_MANAGE_GOOGLE_ACCOUNT_ONE_REQUIRED_CONTROL 40254
#define IDS_OMNIBOX_PEDAL_SYNONYMS_CHANGE_GOOGLE_PASSWORD_ONE_REQUIRED_CHANGE_GMAIL_PASSWORD 40255
#define IDS_OMNIBOX_PEDAL_SYNONYMS_CHANGE_GOOGLE_PASSWORD_ONE_REQUIRED_GOOGLE_ACCOUNT_PASSWORD 40256
#define IDS_OMNIBOX_PEDAL_SYNONYMS_CHANGE_GOOGLE_PASSWORD_ANY_OPTIONAL_BROWSER 40257
#define IDS_OMNIBOX_PEDAL_SYNONYMS_CHANGE_GOOGLE_PASSWORD_ONE_REQUIRED_CHANGE 40258
#define IDS_OMNIBOX_PEDAL_SYNONYMS_CLOSE_INCOGNITO_WINDOWS_ONE_REQUIRED_CLOSE_INCOGNITO_WINDOW 40259
#define IDS_OMNIBOX_PEDAL_SYNONYMS_CLOSE_INCOGNITO_WINDOWS_ONE_REQUIRED_DELETE 40260
#define IDS_OMNIBOX_PEDAL_SYNONYMS_CLOSE_INCOGNITO_WINDOWS_ONE_REQUIRED_INCOGNITO_WINDOW 40261
#define IDS_OMNIBOX_PEDAL_SYNONYMS_PLAY_CHROME_DINO_GAME_ONE_REQUIRED_CHROME_DINO 40262
#define IDS_OMNIBOX_PEDAL_SYNONYMS_PLAY_CHROME_DINO_GAME_ONE_REQUIRED_PLAY_CHROME_DINO_GAME 40263
#define IDS_OMNIBOX_PEDAL_SYNONYMS_FIND_MY_PHONE_ONE_REQUIRED_FIND_LOST_PHONE 40264
#define IDS_OMNIBOX_PEDAL_SYNONYMS_FIND_MY_PHONE_ONE_REQUIRED_HELP_ME_LOCATE 40265
#define IDS_OMNIBOX_PEDAL_SYNONYMS_FIND_MY_PHONE_ONE_REQUIRED_LOST_DEVICE 40266
#define IDS_OMNIBOX_PEDAL_SYNONYMS_MANAGE_GOOGLE_PRIVACY_ONE_REQUIRED_CHANGE_GOOGLE_PRIVACY_SETTINGS 40267
#define IDS_OMNIBOX_PEDAL_SYNONYMS_MANAGE_GOOGLE_PRIVACY_ONE_REQUIRED_MANAGE_GOOGLE_PRIVACY_SETTINGS 40268
#define IDS_OMNIBOX_PEDAL_SYNONYMS_MANAGE_CHROME_SETTINGS_ONE_REQUIRED_CHANGE_CHROME_SETTINGS 40269
#define IDS_OMNIBOX_PEDAL_SYNONYMS_MANAGE_CHROME_SETTINGS_ONE_OPTIONAL_CONTROL 40270
#define IDS_OMNIBOX_PEDAL_SYNONYMS_MANAGE_CHROME_SETTINGS_ONE_REQUIRED_CHROME_BROWSER_SETTINGS 40271
#define IDS_OMNIBOX_PEDAL_SYNONYMS_MANAGE_CHROME_DOWNLOADS_ONE_REQUIRED_MANAGE_CHROME_DOWNLOADS 40272
#define IDS_OMNIBOX_PEDAL_SYNONYMS_MANAGE_CHROME_DOWNLOADS_ONE_REQUIRED_CONTROL 40273
#define IDS_OMNIBOX_PEDAL_SYNONYMS_MANAGE_CHROME_DOWNLOADS_ONE_REQUIRED_CHROME_BROWSER_DOWNLOADS 40274
#define IDS_OMNIBOX_PEDAL_SYNONYMS_VIEW_CHROME_HISTORY_ONE_REQUIRED_SEE_CHROME_HISTORY 40275
#define IDS_OMNIBOX_PEDAL_SYNONYMS_VIEW_CHROME_HISTORY_ONE_REQUIRED_REVISIT 40276
#define IDS_OMNIBOX_PEDAL_SYNONYMS_VIEW_CHROME_HISTORY_ONE_REQUIRED_GOOGLE_CHROME_BROWSING_HISTORY 40277
#define IDS_OMNIBOX_PEDAL_SYNONYMS_SHARE_THIS_PAGE_ONE_REQUIRED_SHARE_THIS_PAGE 40278
#define IDS_OMNIBOX_PEDAL_SYNONYMS_SHARE_THIS_PAGE_ONE_REQUIRED_SHARE_LINK_WITH_QR_CODE 40279
#define IDS_OMNIBOX_PEDAL_SYNONYMS_MANAGE_CHROME_ACCESSIBILITY_ONE_REQUIRED_CUSTOMIZE_CHROME_ACCESSIBILITY 40280
#define IDS_OMNIBOX_PEDAL_SYNONYMS_MANAGE_CHROME_ACCESSIBILITY_ONE_OPTIONAL_CUSTOMIZE 40281
#define IDS_OMNIBOX_PEDAL_SYNONYMS_MANAGE_CHROME_ACCESSIBILITY_ONE_REQUIRED_ACCESSIBILITY_SETTINGS 40282
#define IDS_OMNIBOX_PEDAL_SYNONYMS_CUSTOMIZE_CHROME_FONTS_ONE_REQUIRED_CHANGE_BROWSER_FONT 40283
#define IDS_OMNIBOX_PEDAL_SYNONYMS_CUSTOMIZE_CHROME_FONTS_ONE_OPTIONAL_CUSTOMIZE 40284
#define IDS_OMNIBOX_PEDAL_SYNONYMS_CUSTOMIZE_CHROME_FONTS_ONE_REQUIRED_GOOGLE_CHROME 40285
#define IDS_OMNIBOX_PEDAL_SYNONYMS_CUSTOMIZE_CHROME_FONTS_ONE_REQUIRED_FONT_SIZING 40286
#define IDS_OMNIBOX_PEDAL_SYNONYMS_MANAGE_CHROME_THEMES_ONE_REQUIRED_CUSTOMIZE_CHROME_APPEARANCE 40287
#define IDS_OMNIBOX_PEDAL_SYNONYMS_MANAGE_CHROME_THEMES_ONE_OPTIONAL_CUSTOMIZE 40288
#define IDS_OMNIBOX_PEDAL_SYNONYMS_MANAGE_CHROME_THEMES_ONE_REQUIRED_CHROME_BACKGROUNDS 40289
#define IDS_OMNIBOX_PEDAL_SYNONYMS_CUSTOMIZE_SEARCH_ENGINES_ONE_REQUIRED_ADD_CUSTOM_SEARCH 40290
#define IDS_OMNIBOX_PEDAL_SYNONYMS_CUSTOMIZE_SEARCH_ENGINES_ONE_OPTIONAL_CUSTOMIZE 40291
#define IDS_OMNIBOX_PEDAL_SYNONYMS_CUSTOMIZE_SEARCH_ENGINES_ONE_REQUIRED_CUSTOM_SEARCH_ENGINES 40292
#define IDS_OMNIBOX_PEDAL_SYNONYMS_CUSTOMIZE_SEARCH_ENGINES_ONE_OPTIONAL_GOOGLE_CHROME 40293
#define IDS_OMNIBOX_PEDAL_SYNONYMS_MANAGE_CHROMEOS_ACCESSIBILITY_ONE_REQUIRED_CUSTOMIZE_CHROMEOS_ACCESSIBILITY 40294
#define IDS_OMNIBOX_PEDAL_SYNONYMS_MANAGE_CHROMEOS_ACCESSIBILITY_ONE_OPTIONAL_CUSTOMIZE 40295
#define IDS_OMNIBOX_PEDAL_SYNONYMS_MANAGE_CHROMEOS_ACCESSIBILITY_ONE_REQUIRED_ACCESSIBILITY_SETTINGS 40296
#define IDS_OMNIBOX_PEDAL_SYNONYMS_SET_CHROME_AS_DEFAULT_BROWSER_ONE_REQUIRED_ALWAYS_OPEN_LINKS_IN_CHROME 40297
#define IDS_OMNIBOX_PEDAL_SYNONYMS_SET_CHROME_AS_DEFAULT_BROWSER_ONE_REQUIRED_HOW_TO_MAKE_CHROME_MY_DEFAULT_BROWSER 40298
#define IDS_OMNIBOX_PEDAL_SYNONYMS_SET_CHROME_AS_DEFAULT_BROWSER_ONE_OPTIONAL_SELECT 40299
#define IDS_OMNIBOX_PEDAL_SYNONYMS_SET_CHROME_AS_DEFAULT_BROWSER_ONE_OPTIONAL_DEFAULT_BROWSER 40300

// ---------------------------------------------------------------------------
// From permission_element_generated_strings.h:

#define IDS_PERMISSION_REQUEST_LAUNCH_af 55820
#define IDS_PERMISSION_REQUEST_CAMERA_af 55821
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ALLOWED_af 55822
#define IDS_PERMISSION_REQUEST_INSTALL_WITH_HOST_af 55823
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ALLOWED_af 55824
#define IDS_PERMISSION_REQUEST_INSTALL_af 55825
#define IDS_PERMISSION_REQUEST_MICROPHONE_ALLOWED_af 55826
#define IDS_PERMISSION_REQUEST_CAMERA_ALLOWED_af 55827
#define IDS_PERMISSION_REQUEST_MICROPHONE_af 55828
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ALLOWED_af 55829
#define IDS_PERMISSION_REQUEST_GEOLOCATION_af 55830
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_af 55831
#define IDS_PERMISSION_REQUEST_USING_LOCATION_af 55832
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_af 55833
#define IDS_PERMISSION_REQUEST_LAUNCH_am 55834
#define IDS_PERMISSION_REQUEST_CAMERA_am 55835
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ALLOWED_am 55836
#define IDS_PERMISSION_REQUEST_INSTALL_WITH_HOST_am 55837
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ALLOWED_am 55838
#define IDS_PERMISSION_REQUEST_INSTALL_am 55839
#define IDS_PERMISSION_REQUEST_MICROPHONE_ALLOWED_am 55840
#define IDS_PERMISSION_REQUEST_CAMERA_ALLOWED_am 55841
#define IDS_PERMISSION_REQUEST_MICROPHONE_am 55842
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ALLOWED_am 55843
#define IDS_PERMISSION_REQUEST_GEOLOCATION_am 55844
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_am 55845
#define IDS_PERMISSION_REQUEST_USING_LOCATION_am 55846
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_am 55847
#define IDS_PERMISSION_REQUEST_LAUNCH_ar 55848
#define IDS_PERMISSION_REQUEST_CAMERA_ar 55849
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ALLOWED_ar 55850
#define IDS_PERMISSION_REQUEST_INSTALL_WITH_HOST_ar 55851
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ALLOWED_ar 55852
#define IDS_PERMISSION_REQUEST_INSTALL_ar 55853
#define IDS_PERMISSION_REQUEST_MICROPHONE_ALLOWED_ar 55854
#define IDS_PERMISSION_REQUEST_CAMERA_ALLOWED_ar 55855
#define IDS_PERMISSION_REQUEST_MICROPHONE_ar 55856
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ALLOWED_ar 55857
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ar 55858
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ar 55859
#define IDS_PERMISSION_REQUEST_USING_LOCATION_ar 55860
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ar 55861
#define IDS_PERMISSION_REQUEST_LAUNCH_as 55862
#define IDS_PERMISSION_REQUEST_CAMERA_as 55863
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ALLOWED_as 55864
#define IDS_PERMISSION_REQUEST_INSTALL_WITH_HOST_as 55865
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ALLOWED_as 55866
#define IDS_PERMISSION_REQUEST_INSTALL_as 55867
#define IDS_PERMISSION_REQUEST_MICROPHONE_ALLOWED_as 55868
#define IDS_PERMISSION_REQUEST_CAMERA_ALLOWED_as 55869
#define IDS_PERMISSION_REQUEST_MICROPHONE_as 55870
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ALLOWED_as 55871
#define IDS_PERMISSION_REQUEST_GEOLOCATION_as 55872
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_as 55873
#define IDS_PERMISSION_REQUEST_USING_LOCATION_as 55874
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_as 55875
#define IDS_PERMISSION_REQUEST_LAUNCH_az 55876
#define IDS_PERMISSION_REQUEST_CAMERA_az 55877
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ALLOWED_az 55878
#define IDS_PERMISSION_REQUEST_INSTALL_WITH_HOST_az 55879
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ALLOWED_az 55880
#define IDS_PERMISSION_REQUEST_INSTALL_az 55881
#define IDS_PERMISSION_REQUEST_MICROPHONE_ALLOWED_az 55882
#define IDS_PERMISSION_REQUEST_CAMERA_ALLOWED_az 55883
#define IDS_PERMISSION_REQUEST_MICROPHONE_az 55884
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ALLOWED_az 55885
#define IDS_PERMISSION_REQUEST_GEOLOCATION_az 55886
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_az 55887
#define IDS_PERMISSION_REQUEST_USING_LOCATION_az 55888
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_az 55889
#define IDS_PERMISSION_REQUEST_LAUNCH_be 55890
#define IDS_PERMISSION_REQUEST_CAMERA_be 55891
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ALLOWED_be 55892
#define IDS_PERMISSION_REQUEST_INSTALL_WITH_HOST_be 55893
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ALLOWED_be 55894
#define IDS_PERMISSION_REQUEST_INSTALL_be 55895
#define IDS_PERMISSION_REQUEST_MICROPHONE_ALLOWED_be 55896
#define IDS_PERMISSION_REQUEST_CAMERA_ALLOWED_be 55897
#define IDS_PERMISSION_REQUEST_MICROPHONE_be 55898
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ALLOWED_be 55899
#define IDS_PERMISSION_REQUEST_GEOLOCATION_be 55900
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_be 55901
#define IDS_PERMISSION_REQUEST_USING_LOCATION_be 55902
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_be 55903
#define IDS_PERMISSION_REQUEST_LAUNCH_bg 55904
#define IDS_PERMISSION_REQUEST_CAMERA_bg 55905
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ALLOWED_bg 55906
#define IDS_PERMISSION_REQUEST_INSTALL_WITH_HOST_bg 55907
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ALLOWED_bg 55908
#define IDS_PERMISSION_REQUEST_INSTALL_bg 55909
#define IDS_PERMISSION_REQUEST_MICROPHONE_ALLOWED_bg 55910
#define IDS_PERMISSION_REQUEST_CAMERA_ALLOWED_bg 55911
#define IDS_PERMISSION_REQUEST_MICROPHONE_bg 55912
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ALLOWED_bg 55913
#define IDS_PERMISSION_REQUEST_GEOLOCATION_bg 55914
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_bg 55915
#define IDS_PERMISSION_REQUEST_USING_LOCATION_bg 55916
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_bg 55917
#define IDS_PERMISSION_REQUEST_LAUNCH_bn 55918
#define IDS_PERMISSION_REQUEST_CAMERA_bn 55919
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ALLOWED_bn 55920
#define IDS_PERMISSION_REQUEST_INSTALL_WITH_HOST_bn 55921
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ALLOWED_bn 55922
#define IDS_PERMISSION_REQUEST_INSTALL_bn 55923
#define IDS_PERMISSION_REQUEST_MICROPHONE_ALLOWED_bn 55924
#define IDS_PERMISSION_REQUEST_CAMERA_ALLOWED_bn 55925
#define IDS_PERMISSION_REQUEST_MICROPHONE_bn 55926
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ALLOWED_bn 55927
#define IDS_PERMISSION_REQUEST_GEOLOCATION_bn 55928
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_bn 55929
#define IDS_PERMISSION_REQUEST_USING_LOCATION_bn 55930
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_bn 55931
#define IDS_PERMISSION_REQUEST_LAUNCH_bs 55932
#define IDS_PERMISSION_REQUEST_CAMERA_bs 55933
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ALLOWED_bs 55934
#define IDS_PERMISSION_REQUEST_INSTALL_WITH_HOST_bs 55935
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ALLOWED_bs 55936
#define IDS_PERMISSION_REQUEST_INSTALL_bs 55937
#define IDS_PERMISSION_REQUEST_MICROPHONE_ALLOWED_bs 55938
#define IDS_PERMISSION_REQUEST_CAMERA_ALLOWED_bs 55939
#define IDS_PERMISSION_REQUEST_MICROPHONE_bs 55940
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ALLOWED_bs 55941
#define IDS_PERMISSION_REQUEST_GEOLOCATION_bs 55942
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_bs 55943
#define IDS_PERMISSION_REQUEST_USING_LOCATION_bs 55944
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_bs 55945
#define IDS_PERMISSION_REQUEST_LAUNCH_ca 55946
#define IDS_PERMISSION_REQUEST_CAMERA_ca 55947
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ALLOWED_ca 55948
#define IDS_PERMISSION_REQUEST_INSTALL_WITH_HOST_ca 55949
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ALLOWED_ca 55950
#define IDS_PERMISSION_REQUEST_INSTALL_ca 55951
#define IDS_PERMISSION_REQUEST_MICROPHONE_ALLOWED_ca 55952
#define IDS_PERMISSION_REQUEST_CAMERA_ALLOWED_ca 55953
#define IDS_PERMISSION_REQUEST_MICROPHONE_ca 55954
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ALLOWED_ca 55955
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ca 55956
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ca 55957
#define IDS_PERMISSION_REQUEST_USING_LOCATION_ca 55958
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ca 55959
#define IDS_PERMISSION_REQUEST_LAUNCH_cs 55960
#define IDS_PERMISSION_REQUEST_CAMERA_cs 55961
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ALLOWED_cs 55962
#define IDS_PERMISSION_REQUEST_INSTALL_WITH_HOST_cs 55963
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ALLOWED_cs 55964
#define IDS_PERMISSION_REQUEST_INSTALL_cs 55965
#define IDS_PERMISSION_REQUEST_MICROPHONE_ALLOWED_cs 55966
#define IDS_PERMISSION_REQUEST_CAMERA_ALLOWED_cs 55967
#define IDS_PERMISSION_REQUEST_MICROPHONE_cs 55968
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ALLOWED_cs 55969
#define IDS_PERMISSION_REQUEST_GEOLOCATION_cs 55970
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_cs 55971
#define IDS_PERMISSION_REQUEST_USING_LOCATION_cs 55972
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_cs 55973
#define IDS_PERMISSION_REQUEST_LAUNCH_cy 55974
#define IDS_PERMISSION_REQUEST_CAMERA_cy 55975
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ALLOWED_cy 55976
#define IDS_PERMISSION_REQUEST_INSTALL_WITH_HOST_cy 55977
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ALLOWED_cy 55978
#define IDS_PERMISSION_REQUEST_INSTALL_cy 55979
#define IDS_PERMISSION_REQUEST_MICROPHONE_ALLOWED_cy 55980
#define IDS_PERMISSION_REQUEST_CAMERA_ALLOWED_cy 55981
#define IDS_PERMISSION_REQUEST_MICROPHONE_cy 55982
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ALLOWED_cy 55983
#define IDS_PERMISSION_REQUEST_GEOLOCATION_cy 55984
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_cy 55985
#define IDS_PERMISSION_REQUEST_USING_LOCATION_cy 55986
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_cy 55987
#define IDS_PERMISSION_REQUEST_LAUNCH_da 55988
#define IDS_PERMISSION_REQUEST_CAMERA_da 55989
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ALLOWED_da 55990
#define IDS_PERMISSION_REQUEST_INSTALL_WITH_HOST_da 55991
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ALLOWED_da 55992
#define IDS_PERMISSION_REQUEST_INSTALL_da 55993
#define IDS_PERMISSION_REQUEST_MICROPHONE_ALLOWED_da 55994
#define IDS_PERMISSION_REQUEST_CAMERA_ALLOWED_da 55995
#define IDS_PERMISSION_REQUEST_MICROPHONE_da 55996
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ALLOWED_da 55997
#define IDS_PERMISSION_REQUEST_GEOLOCATION_da 55998
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_da 55999
#define IDS_PERMISSION_REQUEST_USING_LOCATION_da 56000
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_da 56001
#define IDS_PERMISSION_REQUEST_LAUNCH_de 56002
#define IDS_PERMISSION_REQUEST_CAMERA_de 56003
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ALLOWED_de 56004
#define IDS_PERMISSION_REQUEST_INSTALL_WITH_HOST_de 56005
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ALLOWED_de 56006
#define IDS_PERMISSION_REQUEST_INSTALL_de 56007
#define IDS_PERMISSION_REQUEST_MICROPHONE_ALLOWED_de 56008
#define IDS_PERMISSION_REQUEST_CAMERA_ALLOWED_de 56009
#define IDS_PERMISSION_REQUEST_MICROPHONE_de 56010
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ALLOWED_de 56011
#define IDS_PERMISSION_REQUEST_GEOLOCATION_de 56012
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_de 56013
#define IDS_PERMISSION_REQUEST_USING_LOCATION_de 56014
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_de 56015
#define IDS_PERMISSION_REQUEST_LAUNCH_el 56016
#define IDS_PERMISSION_REQUEST_CAMERA_el 56017
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ALLOWED_el 56018
#define IDS_PERMISSION_REQUEST_INSTALL_WITH_HOST_el 56019
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ALLOWED_el 56020
#define IDS_PERMISSION_REQUEST_INSTALL_el 56021
#define IDS_PERMISSION_REQUEST_MICROPHONE_ALLOWED_el 56022
#define IDS_PERMISSION_REQUEST_CAMERA_ALLOWED_el 56023
#define IDS_PERMISSION_REQUEST_MICROPHONE_el 56024
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ALLOWED_el 56025
#define IDS_PERMISSION_REQUEST_GEOLOCATION_el 56026
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_el 56027
#define IDS_PERMISSION_REQUEST_USING_LOCATION_el 56028
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_el 56029
#define IDS_PERMISSION_REQUEST_LAUNCH_en_GB 56030
#define IDS_PERMISSION_REQUEST_CAMERA_en_GB 56031
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ALLOWED_en_GB 56032
#define IDS_PERMISSION_REQUEST_INSTALL_WITH_HOST_en_GB 56033
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ALLOWED_en_GB 56034
#define IDS_PERMISSION_REQUEST_INSTALL_en_GB 56035
#define IDS_PERMISSION_REQUEST_MICROPHONE_ALLOWED_en_GB 56036
#define IDS_PERMISSION_REQUEST_CAMERA_ALLOWED_en_GB 56037
#define IDS_PERMISSION_REQUEST_MICROPHONE_en_GB 56038
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ALLOWED_en_GB 56039
#define IDS_PERMISSION_REQUEST_GEOLOCATION_en_GB 56040
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_en_GB 56041
#define IDS_PERMISSION_REQUEST_USING_LOCATION_en_GB 56042
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_en_GB 56043
#define IDS_PERMISSION_REQUEST_LAUNCH_es_419 56044
#define IDS_PERMISSION_REQUEST_CAMERA_es_419 56045
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ALLOWED_es_419 56046
#define IDS_PERMISSION_REQUEST_INSTALL_WITH_HOST_es_419 56047
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ALLOWED_es_419 56048
#define IDS_PERMISSION_REQUEST_INSTALL_es_419 56049
#define IDS_PERMISSION_REQUEST_MICROPHONE_ALLOWED_es_419 56050
#define IDS_PERMISSION_REQUEST_CAMERA_ALLOWED_es_419 56051
#define IDS_PERMISSION_REQUEST_MICROPHONE_es_419 56052
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ALLOWED_es_419 56053
#define IDS_PERMISSION_REQUEST_GEOLOCATION_es_419 56054
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_es_419 56055
#define IDS_PERMISSION_REQUEST_USING_LOCATION_es_419 56056
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_es_419 56057
#define IDS_PERMISSION_REQUEST_LAUNCH_es 56058
#define IDS_PERMISSION_REQUEST_CAMERA_es 56059
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ALLOWED_es 56060
#define IDS_PERMISSION_REQUEST_INSTALL_WITH_HOST_es 56061
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ALLOWED_es 56062
#define IDS_PERMISSION_REQUEST_INSTALL_es 56063
#define IDS_PERMISSION_REQUEST_MICROPHONE_ALLOWED_es 56064
#define IDS_PERMISSION_REQUEST_CAMERA_ALLOWED_es 56065
#define IDS_PERMISSION_REQUEST_MICROPHONE_es 56066
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ALLOWED_es 56067
#define IDS_PERMISSION_REQUEST_GEOLOCATION_es 56068
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_es 56069
#define IDS_PERMISSION_REQUEST_USING_LOCATION_es 56070
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_es 56071
#define IDS_PERMISSION_REQUEST_LAUNCH_et 56072
#define IDS_PERMISSION_REQUEST_CAMERA_et 56073
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ALLOWED_et 56074
#define IDS_PERMISSION_REQUEST_INSTALL_WITH_HOST_et 56075
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ALLOWED_et 56076
#define IDS_PERMISSION_REQUEST_INSTALL_et 56077
#define IDS_PERMISSION_REQUEST_MICROPHONE_ALLOWED_et 56078
#define IDS_PERMISSION_REQUEST_CAMERA_ALLOWED_et 56079
#define IDS_PERMISSION_REQUEST_MICROPHONE_et 56080
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ALLOWED_et 56081
#define IDS_PERMISSION_REQUEST_GEOLOCATION_et 56082
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_et 56083
#define IDS_PERMISSION_REQUEST_USING_LOCATION_et 56084
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_et 56085
#define IDS_PERMISSION_REQUEST_LAUNCH_eu 56086
#define IDS_PERMISSION_REQUEST_CAMERA_eu 56087
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ALLOWED_eu 56088
#define IDS_PERMISSION_REQUEST_INSTALL_WITH_HOST_eu 56089
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ALLOWED_eu 56090
#define IDS_PERMISSION_REQUEST_INSTALL_eu 56091
#define IDS_PERMISSION_REQUEST_MICROPHONE_ALLOWED_eu 56092
#define IDS_PERMISSION_REQUEST_CAMERA_ALLOWED_eu 56093
#define IDS_PERMISSION_REQUEST_MICROPHONE_eu 56094
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ALLOWED_eu 56095
#define IDS_PERMISSION_REQUEST_GEOLOCATION_eu 56096
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_eu 56097
#define IDS_PERMISSION_REQUEST_USING_LOCATION_eu 56098
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_eu 56099
#define IDS_PERMISSION_REQUEST_LAUNCH_fa 56100
#define IDS_PERMISSION_REQUEST_CAMERA_fa 56101
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ALLOWED_fa 56102
#define IDS_PERMISSION_REQUEST_INSTALL_WITH_HOST_fa 56103
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ALLOWED_fa 56104
#define IDS_PERMISSION_REQUEST_INSTALL_fa 56105
#define IDS_PERMISSION_REQUEST_MICROPHONE_ALLOWED_fa 56106
#define IDS_PERMISSION_REQUEST_CAMERA_ALLOWED_fa 56107
#define IDS_PERMISSION_REQUEST_MICROPHONE_fa 56108
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ALLOWED_fa 56109
#define IDS_PERMISSION_REQUEST_GEOLOCATION_fa 56110
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_fa 56111
#define IDS_PERMISSION_REQUEST_USING_LOCATION_fa 56112
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_fa 56113
#define IDS_PERMISSION_REQUEST_LAUNCH_fi 56114
#define IDS_PERMISSION_REQUEST_CAMERA_fi 56115
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ALLOWED_fi 56116
#define IDS_PERMISSION_REQUEST_INSTALL_WITH_HOST_fi 56117
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ALLOWED_fi 56118
#define IDS_PERMISSION_REQUEST_INSTALL_fi 56119
#define IDS_PERMISSION_REQUEST_MICROPHONE_ALLOWED_fi 56120
#define IDS_PERMISSION_REQUEST_CAMERA_ALLOWED_fi 56121
#define IDS_PERMISSION_REQUEST_MICROPHONE_fi 56122
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ALLOWED_fi 56123
#define IDS_PERMISSION_REQUEST_GEOLOCATION_fi 56124
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_fi 56125
#define IDS_PERMISSION_REQUEST_USING_LOCATION_fi 56126
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_fi 56127
#define IDS_PERMISSION_REQUEST_LAUNCH_fil 56128
#define IDS_PERMISSION_REQUEST_CAMERA_fil 56129
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ALLOWED_fil 56130
#define IDS_PERMISSION_REQUEST_INSTALL_WITH_HOST_fil 56131
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ALLOWED_fil 56132
#define IDS_PERMISSION_REQUEST_INSTALL_fil 56133
#define IDS_PERMISSION_REQUEST_MICROPHONE_ALLOWED_fil 56134
#define IDS_PERMISSION_REQUEST_CAMERA_ALLOWED_fil 56135
#define IDS_PERMISSION_REQUEST_MICROPHONE_fil 56136
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ALLOWED_fil 56137
#define IDS_PERMISSION_REQUEST_GEOLOCATION_fil 56138
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_fil 56139
#define IDS_PERMISSION_REQUEST_USING_LOCATION_fil 56140
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_fil 56141
#define IDS_PERMISSION_REQUEST_LAUNCH_fr_CA 56142
#define IDS_PERMISSION_REQUEST_CAMERA_fr_CA 56143
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ALLOWED_fr_CA 56144
#define IDS_PERMISSION_REQUEST_INSTALL_WITH_HOST_fr_CA 56145
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ALLOWED_fr_CA 56146
#define IDS_PERMISSION_REQUEST_INSTALL_fr_CA 56147
#define IDS_PERMISSION_REQUEST_MICROPHONE_ALLOWED_fr_CA 56148
#define IDS_PERMISSION_REQUEST_CAMERA_ALLOWED_fr_CA 56149
#define IDS_PERMISSION_REQUEST_MICROPHONE_fr_CA 56150
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ALLOWED_fr_CA 56151
#define IDS_PERMISSION_REQUEST_GEOLOCATION_fr_CA 56152
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_fr_CA 56153
#define IDS_PERMISSION_REQUEST_USING_LOCATION_fr_CA 56154
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_fr_CA 56155
#define IDS_PERMISSION_REQUEST_LAUNCH_fr 56156
#define IDS_PERMISSION_REQUEST_CAMERA_fr 56157
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ALLOWED_fr 56158
#define IDS_PERMISSION_REQUEST_INSTALL_WITH_HOST_fr 56159
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ALLOWED_fr 56160
#define IDS_PERMISSION_REQUEST_INSTALL_fr 56161
#define IDS_PERMISSION_REQUEST_MICROPHONE_ALLOWED_fr 56162
#define IDS_PERMISSION_REQUEST_CAMERA_ALLOWED_fr 56163
#define IDS_PERMISSION_REQUEST_MICROPHONE_fr 56164
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ALLOWED_fr 56165
#define IDS_PERMISSION_REQUEST_GEOLOCATION_fr 56166
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_fr 56167
#define IDS_PERMISSION_REQUEST_USING_LOCATION_fr 56168
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_fr 56169
#define IDS_PERMISSION_REQUEST_LAUNCH_gl 56170
#define IDS_PERMISSION_REQUEST_CAMERA_gl 56171
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ALLOWED_gl 56172
#define IDS_PERMISSION_REQUEST_INSTALL_WITH_HOST_gl 56173
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ALLOWED_gl 56174
#define IDS_PERMISSION_REQUEST_INSTALL_gl 56175
#define IDS_PERMISSION_REQUEST_MICROPHONE_ALLOWED_gl 56176
#define IDS_PERMISSION_REQUEST_CAMERA_ALLOWED_gl 56177
#define IDS_PERMISSION_REQUEST_MICROPHONE_gl 56178
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ALLOWED_gl 56179
#define IDS_PERMISSION_REQUEST_GEOLOCATION_gl 56180
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_gl 56181
#define IDS_PERMISSION_REQUEST_USING_LOCATION_gl 56182
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_gl 56183
#define IDS_PERMISSION_REQUEST_LAUNCH_gu 56184
#define IDS_PERMISSION_REQUEST_CAMERA_gu 56185
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ALLOWED_gu 56186
#define IDS_PERMISSION_REQUEST_INSTALL_WITH_HOST_gu 56187
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ALLOWED_gu 56188
#define IDS_PERMISSION_REQUEST_INSTALL_gu 56189
#define IDS_PERMISSION_REQUEST_MICROPHONE_ALLOWED_gu 56190
#define IDS_PERMISSION_REQUEST_CAMERA_ALLOWED_gu 56191
#define IDS_PERMISSION_REQUEST_MICROPHONE_gu 56192
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ALLOWED_gu 56193
#define IDS_PERMISSION_REQUEST_GEOLOCATION_gu 56194
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_gu 56195
#define IDS_PERMISSION_REQUEST_USING_LOCATION_gu 56196
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_gu 56197
#define IDS_PERMISSION_REQUEST_LAUNCH_hi 56198
#define IDS_PERMISSION_REQUEST_CAMERA_hi 56199
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ALLOWED_hi 56200
#define IDS_PERMISSION_REQUEST_INSTALL_WITH_HOST_hi 56201
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ALLOWED_hi 56202
#define IDS_PERMISSION_REQUEST_INSTALL_hi 56203
#define IDS_PERMISSION_REQUEST_MICROPHONE_ALLOWED_hi 56204
#define IDS_PERMISSION_REQUEST_CAMERA_ALLOWED_hi 56205
#define IDS_PERMISSION_REQUEST_MICROPHONE_hi 56206
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ALLOWED_hi 56207
#define IDS_PERMISSION_REQUEST_GEOLOCATION_hi 56208
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_hi 56209
#define IDS_PERMISSION_REQUEST_USING_LOCATION_hi 56210
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_hi 56211
#define IDS_PERMISSION_REQUEST_LAUNCH_hr 56212
#define IDS_PERMISSION_REQUEST_CAMERA_hr 56213
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ALLOWED_hr 56214
#define IDS_PERMISSION_REQUEST_INSTALL_WITH_HOST_hr 56215
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ALLOWED_hr 56216
#define IDS_PERMISSION_REQUEST_INSTALL_hr 56217
#define IDS_PERMISSION_REQUEST_MICROPHONE_ALLOWED_hr 56218
#define IDS_PERMISSION_REQUEST_CAMERA_ALLOWED_hr 56219
#define IDS_PERMISSION_REQUEST_MICROPHONE_hr 56220
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ALLOWED_hr 56221
#define IDS_PERMISSION_REQUEST_GEOLOCATION_hr 56222
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_hr 56223
#define IDS_PERMISSION_REQUEST_USING_LOCATION_hr 56224
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_hr 56225
#define IDS_PERMISSION_REQUEST_LAUNCH_hu 56226
#define IDS_PERMISSION_REQUEST_CAMERA_hu 56227
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ALLOWED_hu 56228
#define IDS_PERMISSION_REQUEST_INSTALL_WITH_HOST_hu 56229
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ALLOWED_hu 56230
#define IDS_PERMISSION_REQUEST_INSTALL_hu 56231
#define IDS_PERMISSION_REQUEST_MICROPHONE_ALLOWED_hu 56232
#define IDS_PERMISSION_REQUEST_CAMERA_ALLOWED_hu 56233
#define IDS_PERMISSION_REQUEST_MICROPHONE_hu 56234
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ALLOWED_hu 56235
#define IDS_PERMISSION_REQUEST_GEOLOCATION_hu 56236
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_hu 56237
#define IDS_PERMISSION_REQUEST_USING_LOCATION_hu 56238
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_hu 56239
#define IDS_PERMISSION_REQUEST_LAUNCH_hy 56240
#define IDS_PERMISSION_REQUEST_CAMERA_hy 56241
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ALLOWED_hy 56242
#define IDS_PERMISSION_REQUEST_INSTALL_WITH_HOST_hy 56243
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ALLOWED_hy 56244
#define IDS_PERMISSION_REQUEST_INSTALL_hy 56245
#define IDS_PERMISSION_REQUEST_MICROPHONE_ALLOWED_hy 56246
#define IDS_PERMISSION_REQUEST_CAMERA_ALLOWED_hy 56247
#define IDS_PERMISSION_REQUEST_MICROPHONE_hy 56248
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ALLOWED_hy 56249
#define IDS_PERMISSION_REQUEST_GEOLOCATION_hy 56250
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_hy 56251
#define IDS_PERMISSION_REQUEST_USING_LOCATION_hy 56252
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_hy 56253
#define IDS_PERMISSION_REQUEST_LAUNCH_id 56254
#define IDS_PERMISSION_REQUEST_CAMERA_id 56255
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ALLOWED_id 56256
#define IDS_PERMISSION_REQUEST_INSTALL_WITH_HOST_id 56257
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ALLOWED_id 56258
#define IDS_PERMISSION_REQUEST_INSTALL_id 56259
#define IDS_PERMISSION_REQUEST_MICROPHONE_ALLOWED_id 56260
#define IDS_PERMISSION_REQUEST_CAMERA_ALLOWED_id 56261
#define IDS_PERMISSION_REQUEST_MICROPHONE_id 56262
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ALLOWED_id 56263
#define IDS_PERMISSION_REQUEST_GEOLOCATION_id 56264
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_id 56265
#define IDS_PERMISSION_REQUEST_USING_LOCATION_id 56266
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_id 56267
#define IDS_PERMISSION_REQUEST_LAUNCH_is 56268
#define IDS_PERMISSION_REQUEST_CAMERA_is 56269
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ALLOWED_is 56270
#define IDS_PERMISSION_REQUEST_INSTALL_WITH_HOST_is 56271
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ALLOWED_is 56272
#define IDS_PERMISSION_REQUEST_INSTALL_is 56273
#define IDS_PERMISSION_REQUEST_MICROPHONE_ALLOWED_is 56274
#define IDS_PERMISSION_REQUEST_CAMERA_ALLOWED_is 56275
#define IDS_PERMISSION_REQUEST_MICROPHONE_is 56276
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ALLOWED_is 56277
#define IDS_PERMISSION_REQUEST_GEOLOCATION_is 56278
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_is 56279
#define IDS_PERMISSION_REQUEST_USING_LOCATION_is 56280
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_is 56281
#define IDS_PERMISSION_REQUEST_LAUNCH_it 56282
#define IDS_PERMISSION_REQUEST_CAMERA_it 56283
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ALLOWED_it 56284
#define IDS_PERMISSION_REQUEST_INSTALL_WITH_HOST_it 56285
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ALLOWED_it 56286
#define IDS_PERMISSION_REQUEST_INSTALL_it 56287
#define IDS_PERMISSION_REQUEST_MICROPHONE_ALLOWED_it 56288
#define IDS_PERMISSION_REQUEST_CAMERA_ALLOWED_it 56289
#define IDS_PERMISSION_REQUEST_MICROPHONE_it 56290
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ALLOWED_it 56291
#define IDS_PERMISSION_REQUEST_GEOLOCATION_it 56292
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_it 56293
#define IDS_PERMISSION_REQUEST_USING_LOCATION_it 56294
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_it 56295
#define IDS_PERMISSION_REQUEST_LAUNCH_iw 56296
#define IDS_PERMISSION_REQUEST_CAMERA_iw 56297
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ALLOWED_iw 56298
#define IDS_PERMISSION_REQUEST_INSTALL_WITH_HOST_iw 56299
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ALLOWED_iw 56300
#define IDS_PERMISSION_REQUEST_INSTALL_iw 56301
#define IDS_PERMISSION_REQUEST_MICROPHONE_ALLOWED_iw 56302
#define IDS_PERMISSION_REQUEST_CAMERA_ALLOWED_iw 56303
#define IDS_PERMISSION_REQUEST_MICROPHONE_iw 56304
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ALLOWED_iw 56305
#define IDS_PERMISSION_REQUEST_GEOLOCATION_iw 56306
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_iw 56307
#define IDS_PERMISSION_REQUEST_USING_LOCATION_iw 56308
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_iw 56309
#define IDS_PERMISSION_REQUEST_LAUNCH_ja 56310
#define IDS_PERMISSION_REQUEST_CAMERA_ja 56311
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ALLOWED_ja 56312
#define IDS_PERMISSION_REQUEST_INSTALL_WITH_HOST_ja 56313
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ALLOWED_ja 56314
#define IDS_PERMISSION_REQUEST_INSTALL_ja 56315
#define IDS_PERMISSION_REQUEST_MICROPHONE_ALLOWED_ja 56316
#define IDS_PERMISSION_REQUEST_CAMERA_ALLOWED_ja 56317
#define IDS_PERMISSION_REQUEST_MICROPHONE_ja 56318
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ALLOWED_ja 56319
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ja 56320
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ja 56321
#define IDS_PERMISSION_REQUEST_USING_LOCATION_ja 56322
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ja 56323
#define IDS_PERMISSION_REQUEST_LAUNCH_ka 56324
#define IDS_PERMISSION_REQUEST_CAMERA_ka 56325
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ALLOWED_ka 56326
#define IDS_PERMISSION_REQUEST_INSTALL_WITH_HOST_ka 56327
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ALLOWED_ka 56328
#define IDS_PERMISSION_REQUEST_INSTALL_ka 56329
#define IDS_PERMISSION_REQUEST_MICROPHONE_ALLOWED_ka 56330
#define IDS_PERMISSION_REQUEST_CAMERA_ALLOWED_ka 56331
#define IDS_PERMISSION_REQUEST_MICROPHONE_ka 56332
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ALLOWED_ka 56333
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ka 56334
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ka 56335
#define IDS_PERMISSION_REQUEST_USING_LOCATION_ka 56336
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ka 56337
#define IDS_PERMISSION_REQUEST_LAUNCH_kk 56338
#define IDS_PERMISSION_REQUEST_CAMERA_kk 56339
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ALLOWED_kk 56340
#define IDS_PERMISSION_REQUEST_INSTALL_WITH_HOST_kk 56341
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ALLOWED_kk 56342
#define IDS_PERMISSION_REQUEST_INSTALL_kk 56343
#define IDS_PERMISSION_REQUEST_MICROPHONE_ALLOWED_kk 56344
#define IDS_PERMISSION_REQUEST_CAMERA_ALLOWED_kk 56345
#define IDS_PERMISSION_REQUEST_MICROPHONE_kk 56346
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ALLOWED_kk 56347
#define IDS_PERMISSION_REQUEST_GEOLOCATION_kk 56348
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_kk 56349
#define IDS_PERMISSION_REQUEST_USING_LOCATION_kk 56350
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_kk 56351
#define IDS_PERMISSION_REQUEST_LAUNCH_km 56352
#define IDS_PERMISSION_REQUEST_CAMERA_km 56353
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ALLOWED_km 56354
#define IDS_PERMISSION_REQUEST_INSTALL_WITH_HOST_km 56355
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ALLOWED_km 56356
#define IDS_PERMISSION_REQUEST_INSTALL_km 56357
#define IDS_PERMISSION_REQUEST_MICROPHONE_ALLOWED_km 56358
#define IDS_PERMISSION_REQUEST_CAMERA_ALLOWED_km 56359
#define IDS_PERMISSION_REQUEST_MICROPHONE_km 56360
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ALLOWED_km 56361
#define IDS_PERMISSION_REQUEST_GEOLOCATION_km 56362
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_km 56363
#define IDS_PERMISSION_REQUEST_USING_LOCATION_km 56364
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_km 56365
#define IDS_PERMISSION_REQUEST_LAUNCH_kn 56366
#define IDS_PERMISSION_REQUEST_CAMERA_kn 56367
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ALLOWED_kn 56368
#define IDS_PERMISSION_REQUEST_INSTALL_WITH_HOST_kn 56369
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ALLOWED_kn 56370
#define IDS_PERMISSION_REQUEST_INSTALL_kn 56371
#define IDS_PERMISSION_REQUEST_MICROPHONE_ALLOWED_kn 56372
#define IDS_PERMISSION_REQUEST_CAMERA_ALLOWED_kn 56373
#define IDS_PERMISSION_REQUEST_MICROPHONE_kn 56374
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ALLOWED_kn 56375
#define IDS_PERMISSION_REQUEST_GEOLOCATION_kn 56376
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_kn 56377
#define IDS_PERMISSION_REQUEST_USING_LOCATION_kn 56378
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_kn 56379
#define IDS_PERMISSION_REQUEST_LAUNCH_ko 56380
#define IDS_PERMISSION_REQUEST_CAMERA_ko 56381
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ALLOWED_ko 56382
#define IDS_PERMISSION_REQUEST_INSTALL_WITH_HOST_ko 56383
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ALLOWED_ko 56384
#define IDS_PERMISSION_REQUEST_INSTALL_ko 56385
#define IDS_PERMISSION_REQUEST_MICROPHONE_ALLOWED_ko 56386
#define IDS_PERMISSION_REQUEST_CAMERA_ALLOWED_ko 56387
#define IDS_PERMISSION_REQUEST_MICROPHONE_ko 56388
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ALLOWED_ko 56389
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ko 56390
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ko 56391
#define IDS_PERMISSION_REQUEST_USING_LOCATION_ko 56392
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ko 56393
#define IDS_PERMISSION_REQUEST_LAUNCH_ky 56394
#define IDS_PERMISSION_REQUEST_CAMERA_ky 56395
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ALLOWED_ky 56396
#define IDS_PERMISSION_REQUEST_INSTALL_WITH_HOST_ky 56397
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ALLOWED_ky 56398
#define IDS_PERMISSION_REQUEST_INSTALL_ky 56399
#define IDS_PERMISSION_REQUEST_MICROPHONE_ALLOWED_ky 56400
#define IDS_PERMISSION_REQUEST_CAMERA_ALLOWED_ky 56401
#define IDS_PERMISSION_REQUEST_MICROPHONE_ky 56402
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ALLOWED_ky 56403
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ky 56404
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ky 56405
#define IDS_PERMISSION_REQUEST_USING_LOCATION_ky 56406
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ky 56407
#define IDS_PERMISSION_REQUEST_LAUNCH_lo 56408
#define IDS_PERMISSION_REQUEST_CAMERA_lo 56409
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ALLOWED_lo 56410
#define IDS_PERMISSION_REQUEST_INSTALL_WITH_HOST_lo 56411
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ALLOWED_lo 56412
#define IDS_PERMISSION_REQUEST_INSTALL_lo 56413
#define IDS_PERMISSION_REQUEST_MICROPHONE_ALLOWED_lo 56414
#define IDS_PERMISSION_REQUEST_CAMERA_ALLOWED_lo 56415
#define IDS_PERMISSION_REQUEST_MICROPHONE_lo 56416
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ALLOWED_lo 56417
#define IDS_PERMISSION_REQUEST_GEOLOCATION_lo 56418
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_lo 56419
#define IDS_PERMISSION_REQUEST_USING_LOCATION_lo 56420
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_lo 56421
#define IDS_PERMISSION_REQUEST_LAUNCH_lt 56422
#define IDS_PERMISSION_REQUEST_CAMERA_lt 56423
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ALLOWED_lt 56424
#define IDS_PERMISSION_REQUEST_INSTALL_WITH_HOST_lt 56425
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ALLOWED_lt 56426
#define IDS_PERMISSION_REQUEST_INSTALL_lt 56427
#define IDS_PERMISSION_REQUEST_MICROPHONE_ALLOWED_lt 56428
#define IDS_PERMISSION_REQUEST_CAMERA_ALLOWED_lt 56429
#define IDS_PERMISSION_REQUEST_MICROPHONE_lt 56430
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ALLOWED_lt 56431
#define IDS_PERMISSION_REQUEST_GEOLOCATION_lt 56432
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_lt 56433
#define IDS_PERMISSION_REQUEST_USING_LOCATION_lt 56434
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_lt 56435
#define IDS_PERMISSION_REQUEST_LAUNCH_lv 56436
#define IDS_PERMISSION_REQUEST_CAMERA_lv 56437
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ALLOWED_lv 56438
#define IDS_PERMISSION_REQUEST_INSTALL_WITH_HOST_lv 56439
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ALLOWED_lv 56440
#define IDS_PERMISSION_REQUEST_INSTALL_lv 56441
#define IDS_PERMISSION_REQUEST_MICROPHONE_ALLOWED_lv 56442
#define IDS_PERMISSION_REQUEST_CAMERA_ALLOWED_lv 56443
#define IDS_PERMISSION_REQUEST_MICROPHONE_lv 56444
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ALLOWED_lv 56445
#define IDS_PERMISSION_REQUEST_GEOLOCATION_lv 56446
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_lv 56447
#define IDS_PERMISSION_REQUEST_USING_LOCATION_lv 56448
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_lv 56449
#define IDS_PERMISSION_REQUEST_LAUNCH_mk 56450
#define IDS_PERMISSION_REQUEST_CAMERA_mk 56451
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ALLOWED_mk 56452
#define IDS_PERMISSION_REQUEST_INSTALL_WITH_HOST_mk 56453
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ALLOWED_mk 56454
#define IDS_PERMISSION_REQUEST_INSTALL_mk 56455
#define IDS_PERMISSION_REQUEST_MICROPHONE_ALLOWED_mk 56456
#define IDS_PERMISSION_REQUEST_CAMERA_ALLOWED_mk 56457
#define IDS_PERMISSION_REQUEST_MICROPHONE_mk 56458
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ALLOWED_mk 56459
#define IDS_PERMISSION_REQUEST_GEOLOCATION_mk 56460
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_mk 56461
#define IDS_PERMISSION_REQUEST_USING_LOCATION_mk 56462
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_mk 56463
#define IDS_PERMISSION_REQUEST_LAUNCH_ml 56464
#define IDS_PERMISSION_REQUEST_CAMERA_ml 56465
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ALLOWED_ml 56466
#define IDS_PERMISSION_REQUEST_INSTALL_WITH_HOST_ml 56467
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ALLOWED_ml 56468
#define IDS_PERMISSION_REQUEST_INSTALL_ml 56469
#define IDS_PERMISSION_REQUEST_MICROPHONE_ALLOWED_ml 56470
#define IDS_PERMISSION_REQUEST_CAMERA_ALLOWED_ml 56471
#define IDS_PERMISSION_REQUEST_MICROPHONE_ml 56472
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ALLOWED_ml 56473
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ml 56474
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ml 56475
#define IDS_PERMISSION_REQUEST_USING_LOCATION_ml 56476
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ml 56477
#define IDS_PERMISSION_REQUEST_LAUNCH_mn 56478
#define IDS_PERMISSION_REQUEST_CAMERA_mn 56479
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ALLOWED_mn 56480
#define IDS_PERMISSION_REQUEST_INSTALL_WITH_HOST_mn 56481
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ALLOWED_mn 56482
#define IDS_PERMISSION_REQUEST_INSTALL_mn 56483
#define IDS_PERMISSION_REQUEST_MICROPHONE_ALLOWED_mn 56484
#define IDS_PERMISSION_REQUEST_CAMERA_ALLOWED_mn 56485
#define IDS_PERMISSION_REQUEST_MICROPHONE_mn 56486
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ALLOWED_mn 56487
#define IDS_PERMISSION_REQUEST_GEOLOCATION_mn 56488
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_mn 56489
#define IDS_PERMISSION_REQUEST_USING_LOCATION_mn 56490
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_mn 56491
#define IDS_PERMISSION_REQUEST_LAUNCH_mr 56492
#define IDS_PERMISSION_REQUEST_CAMERA_mr 56493
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ALLOWED_mr 56494
#define IDS_PERMISSION_REQUEST_INSTALL_WITH_HOST_mr 56495
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ALLOWED_mr 56496
#define IDS_PERMISSION_REQUEST_INSTALL_mr 56497
#define IDS_PERMISSION_REQUEST_MICROPHONE_ALLOWED_mr 56498
#define IDS_PERMISSION_REQUEST_CAMERA_ALLOWED_mr 56499
#define IDS_PERMISSION_REQUEST_MICROPHONE_mr 56500
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ALLOWED_mr 56501
#define IDS_PERMISSION_REQUEST_GEOLOCATION_mr 56502
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_mr 56503
#define IDS_PERMISSION_REQUEST_USING_LOCATION_mr 56504
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_mr 56505
#define IDS_PERMISSION_REQUEST_LAUNCH_ms 56506
#define IDS_PERMISSION_REQUEST_CAMERA_ms 56507
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ALLOWED_ms 56508
#define IDS_PERMISSION_REQUEST_INSTALL_WITH_HOST_ms 56509
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ALLOWED_ms 56510
#define IDS_PERMISSION_REQUEST_INSTALL_ms 56511
#define IDS_PERMISSION_REQUEST_MICROPHONE_ALLOWED_ms 56512
#define IDS_PERMISSION_REQUEST_CAMERA_ALLOWED_ms 56513
#define IDS_PERMISSION_REQUEST_MICROPHONE_ms 56514
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ALLOWED_ms 56515
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ms 56516
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ms 56517
#define IDS_PERMISSION_REQUEST_USING_LOCATION_ms 56518
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ms 56519
#define IDS_PERMISSION_REQUEST_LAUNCH_my 56520
#define IDS_PERMISSION_REQUEST_CAMERA_my 56521
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ALLOWED_my 56522
#define IDS_PERMISSION_REQUEST_INSTALL_WITH_HOST_my 56523
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ALLOWED_my 56524
#define IDS_PERMISSION_REQUEST_INSTALL_my 56525
#define IDS_PERMISSION_REQUEST_MICROPHONE_ALLOWED_my 56526
#define IDS_PERMISSION_REQUEST_CAMERA_ALLOWED_my 56527
#define IDS_PERMISSION_REQUEST_MICROPHONE_my 56528
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ALLOWED_my 56529
#define IDS_PERMISSION_REQUEST_GEOLOCATION_my 56530
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_my 56531
#define IDS_PERMISSION_REQUEST_USING_LOCATION_my 56532
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_my 56533
#define IDS_PERMISSION_REQUEST_LAUNCH_ne 56534
#define IDS_PERMISSION_REQUEST_CAMERA_ne 56535
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ALLOWED_ne 56536
#define IDS_PERMISSION_REQUEST_INSTALL_WITH_HOST_ne 56537
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ALLOWED_ne 56538
#define IDS_PERMISSION_REQUEST_INSTALL_ne 56539
#define IDS_PERMISSION_REQUEST_MICROPHONE_ALLOWED_ne 56540
#define IDS_PERMISSION_REQUEST_CAMERA_ALLOWED_ne 56541
#define IDS_PERMISSION_REQUEST_MICROPHONE_ne 56542
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ALLOWED_ne 56543
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ne 56544
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ne 56545
#define IDS_PERMISSION_REQUEST_USING_LOCATION_ne 56546
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ne 56547
#define IDS_PERMISSION_REQUEST_LAUNCH_nl 56548
#define IDS_PERMISSION_REQUEST_CAMERA_nl 56549
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ALLOWED_nl 56550
#define IDS_PERMISSION_REQUEST_INSTALL_WITH_HOST_nl 56551
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ALLOWED_nl 56552
#define IDS_PERMISSION_REQUEST_INSTALL_nl 56553
#define IDS_PERMISSION_REQUEST_MICROPHONE_ALLOWED_nl 56554
#define IDS_PERMISSION_REQUEST_CAMERA_ALLOWED_nl 56555
#define IDS_PERMISSION_REQUEST_MICROPHONE_nl 56556
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ALLOWED_nl 56557
#define IDS_PERMISSION_REQUEST_GEOLOCATION_nl 56558
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_nl 56559
#define IDS_PERMISSION_REQUEST_USING_LOCATION_nl 56560
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_nl 56561
#define IDS_PERMISSION_REQUEST_LAUNCH_no 56562
#define IDS_PERMISSION_REQUEST_CAMERA_no 56563
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ALLOWED_no 56564
#define IDS_PERMISSION_REQUEST_INSTALL_WITH_HOST_no 56565
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ALLOWED_no 56566
#define IDS_PERMISSION_REQUEST_INSTALL_no 56567
#define IDS_PERMISSION_REQUEST_MICROPHONE_ALLOWED_no 56568
#define IDS_PERMISSION_REQUEST_CAMERA_ALLOWED_no 56569
#define IDS_PERMISSION_REQUEST_MICROPHONE_no 56570
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ALLOWED_no 56571
#define IDS_PERMISSION_REQUEST_GEOLOCATION_no 56572
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_no 56573
#define IDS_PERMISSION_REQUEST_USING_LOCATION_no 56574
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_no 56575
#define IDS_PERMISSION_REQUEST_LAUNCH_or 56576
#define IDS_PERMISSION_REQUEST_CAMERA_or 56577
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ALLOWED_or 56578
#define IDS_PERMISSION_REQUEST_INSTALL_WITH_HOST_or 56579
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ALLOWED_or 56580
#define IDS_PERMISSION_REQUEST_INSTALL_or 56581
#define IDS_PERMISSION_REQUEST_MICROPHONE_ALLOWED_or 56582
#define IDS_PERMISSION_REQUEST_CAMERA_ALLOWED_or 56583
#define IDS_PERMISSION_REQUEST_MICROPHONE_or 56584
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ALLOWED_or 56585
#define IDS_PERMISSION_REQUEST_GEOLOCATION_or 56586
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_or 56587
#define IDS_PERMISSION_REQUEST_USING_LOCATION_or 56588
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_or 56589
#define IDS_PERMISSION_REQUEST_LAUNCH_pa 56590
#define IDS_PERMISSION_REQUEST_CAMERA_pa 56591
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ALLOWED_pa 56592
#define IDS_PERMISSION_REQUEST_INSTALL_WITH_HOST_pa 56593
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ALLOWED_pa 56594
#define IDS_PERMISSION_REQUEST_INSTALL_pa 56595
#define IDS_PERMISSION_REQUEST_MICROPHONE_ALLOWED_pa 56596
#define IDS_PERMISSION_REQUEST_CAMERA_ALLOWED_pa 56597
#define IDS_PERMISSION_REQUEST_MICROPHONE_pa 56598
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ALLOWED_pa 56599
#define IDS_PERMISSION_REQUEST_GEOLOCATION_pa 56600
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_pa 56601
#define IDS_PERMISSION_REQUEST_USING_LOCATION_pa 56602
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_pa 56603
#define IDS_PERMISSION_REQUEST_LAUNCH_pl 56604
#define IDS_PERMISSION_REQUEST_CAMERA_pl 56605
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ALLOWED_pl 56606
#define IDS_PERMISSION_REQUEST_INSTALL_WITH_HOST_pl 56607
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ALLOWED_pl 56608
#define IDS_PERMISSION_REQUEST_INSTALL_pl 56609
#define IDS_PERMISSION_REQUEST_MICROPHONE_ALLOWED_pl 56610
#define IDS_PERMISSION_REQUEST_CAMERA_ALLOWED_pl 56611
#define IDS_PERMISSION_REQUEST_MICROPHONE_pl 56612
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ALLOWED_pl 56613
#define IDS_PERMISSION_REQUEST_GEOLOCATION_pl 56614
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_pl 56615
#define IDS_PERMISSION_REQUEST_USING_LOCATION_pl 56616
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_pl 56617
#define IDS_PERMISSION_REQUEST_LAUNCH_pt_BR 56618
#define IDS_PERMISSION_REQUEST_CAMERA_pt_BR 56619
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ALLOWED_pt_BR 56620
#define IDS_PERMISSION_REQUEST_INSTALL_WITH_HOST_pt_BR 56621
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ALLOWED_pt_BR 56622
#define IDS_PERMISSION_REQUEST_INSTALL_pt_BR 56623
#define IDS_PERMISSION_REQUEST_MICROPHONE_ALLOWED_pt_BR 56624
#define IDS_PERMISSION_REQUEST_CAMERA_ALLOWED_pt_BR 56625
#define IDS_PERMISSION_REQUEST_MICROPHONE_pt_BR 56626
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ALLOWED_pt_BR 56627
#define IDS_PERMISSION_REQUEST_GEOLOCATION_pt_BR 56628
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_pt_BR 56629
#define IDS_PERMISSION_REQUEST_USING_LOCATION_pt_BR 56630
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_pt_BR 56631
#define IDS_PERMISSION_REQUEST_LAUNCH_pt_PT 56632
#define IDS_PERMISSION_REQUEST_CAMERA_pt_PT 56633
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ALLOWED_pt_PT 56634
#define IDS_PERMISSION_REQUEST_INSTALL_WITH_HOST_pt_PT 56635
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ALLOWED_pt_PT 56636
#define IDS_PERMISSION_REQUEST_INSTALL_pt_PT 56637
#define IDS_PERMISSION_REQUEST_MICROPHONE_ALLOWED_pt_PT 56638
#define IDS_PERMISSION_REQUEST_CAMERA_ALLOWED_pt_PT 56639
#define IDS_PERMISSION_REQUEST_MICROPHONE_pt_PT 56640
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ALLOWED_pt_PT 56641
#define IDS_PERMISSION_REQUEST_GEOLOCATION_pt_PT 56642
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_pt_PT 56643
#define IDS_PERMISSION_REQUEST_USING_LOCATION_pt_PT 56644
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_pt_PT 56645
#define IDS_PERMISSION_REQUEST_LAUNCH_ro 56646
#define IDS_PERMISSION_REQUEST_CAMERA_ro 56647
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ALLOWED_ro 56648
#define IDS_PERMISSION_REQUEST_INSTALL_WITH_HOST_ro 56649
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ALLOWED_ro 56650
#define IDS_PERMISSION_REQUEST_INSTALL_ro 56651
#define IDS_PERMISSION_REQUEST_MICROPHONE_ALLOWED_ro 56652
#define IDS_PERMISSION_REQUEST_CAMERA_ALLOWED_ro 56653
#define IDS_PERMISSION_REQUEST_MICROPHONE_ro 56654
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ALLOWED_ro 56655
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ro 56656
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ro 56657
#define IDS_PERMISSION_REQUEST_USING_LOCATION_ro 56658
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ro 56659
#define IDS_PERMISSION_REQUEST_LAUNCH_ru 56660
#define IDS_PERMISSION_REQUEST_CAMERA_ru 56661
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ALLOWED_ru 56662
#define IDS_PERMISSION_REQUEST_INSTALL_WITH_HOST_ru 56663
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ALLOWED_ru 56664
#define IDS_PERMISSION_REQUEST_INSTALL_ru 56665
#define IDS_PERMISSION_REQUEST_MICROPHONE_ALLOWED_ru 56666
#define IDS_PERMISSION_REQUEST_CAMERA_ALLOWED_ru 56667
#define IDS_PERMISSION_REQUEST_MICROPHONE_ru 56668
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ALLOWED_ru 56669
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ru 56670
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ru 56671
#define IDS_PERMISSION_REQUEST_USING_LOCATION_ru 56672
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ru 56673
#define IDS_PERMISSION_REQUEST_LAUNCH_si 56674
#define IDS_PERMISSION_REQUEST_CAMERA_si 56675
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ALLOWED_si 56676
#define IDS_PERMISSION_REQUEST_INSTALL_WITH_HOST_si 56677
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ALLOWED_si 56678
#define IDS_PERMISSION_REQUEST_INSTALL_si 56679
#define IDS_PERMISSION_REQUEST_MICROPHONE_ALLOWED_si 56680
#define IDS_PERMISSION_REQUEST_CAMERA_ALLOWED_si 56681
#define IDS_PERMISSION_REQUEST_MICROPHONE_si 56682
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ALLOWED_si 56683
#define IDS_PERMISSION_REQUEST_GEOLOCATION_si 56684
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_si 56685
#define IDS_PERMISSION_REQUEST_USING_LOCATION_si 56686
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_si 56687
#define IDS_PERMISSION_REQUEST_LAUNCH_sk 56688
#define IDS_PERMISSION_REQUEST_CAMERA_sk 56689
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ALLOWED_sk 56690
#define IDS_PERMISSION_REQUEST_INSTALL_WITH_HOST_sk 56691
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ALLOWED_sk 56692
#define IDS_PERMISSION_REQUEST_INSTALL_sk 56693
#define IDS_PERMISSION_REQUEST_MICROPHONE_ALLOWED_sk 56694
#define IDS_PERMISSION_REQUEST_CAMERA_ALLOWED_sk 56695
#define IDS_PERMISSION_REQUEST_MICROPHONE_sk 56696
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ALLOWED_sk 56697
#define IDS_PERMISSION_REQUEST_GEOLOCATION_sk 56698
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_sk 56699
#define IDS_PERMISSION_REQUEST_USING_LOCATION_sk 56700
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_sk 56701
#define IDS_PERMISSION_REQUEST_LAUNCH_sl 56702
#define IDS_PERMISSION_REQUEST_CAMERA_sl 56703
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ALLOWED_sl 56704
#define IDS_PERMISSION_REQUEST_INSTALL_WITH_HOST_sl 56705
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ALLOWED_sl 56706
#define IDS_PERMISSION_REQUEST_INSTALL_sl 56707
#define IDS_PERMISSION_REQUEST_MICROPHONE_ALLOWED_sl 56708
#define IDS_PERMISSION_REQUEST_CAMERA_ALLOWED_sl 56709
#define IDS_PERMISSION_REQUEST_MICROPHONE_sl 56710
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ALLOWED_sl 56711
#define IDS_PERMISSION_REQUEST_GEOLOCATION_sl 56712
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_sl 56713
#define IDS_PERMISSION_REQUEST_USING_LOCATION_sl 56714
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_sl 56715
#define IDS_PERMISSION_REQUEST_LAUNCH_sq 56716
#define IDS_PERMISSION_REQUEST_CAMERA_sq 56717
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ALLOWED_sq 56718
#define IDS_PERMISSION_REQUEST_INSTALL_WITH_HOST_sq 56719
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ALLOWED_sq 56720
#define IDS_PERMISSION_REQUEST_INSTALL_sq 56721
#define IDS_PERMISSION_REQUEST_MICROPHONE_ALLOWED_sq 56722
#define IDS_PERMISSION_REQUEST_CAMERA_ALLOWED_sq 56723
#define IDS_PERMISSION_REQUEST_MICROPHONE_sq 56724
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ALLOWED_sq 56725
#define IDS_PERMISSION_REQUEST_GEOLOCATION_sq 56726
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_sq 56727
#define IDS_PERMISSION_REQUEST_USING_LOCATION_sq 56728
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_sq 56729
#define IDS_PERMISSION_REQUEST_LAUNCH_sr_Latn 56730
#define IDS_PERMISSION_REQUEST_CAMERA_sr_Latn 56731
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ALLOWED_sr_Latn 56732
#define IDS_PERMISSION_REQUEST_INSTALL_WITH_HOST_sr_Latn 56733
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ALLOWED_sr_Latn 56734
#define IDS_PERMISSION_REQUEST_INSTALL_sr_Latn 56735
#define IDS_PERMISSION_REQUEST_MICROPHONE_ALLOWED_sr_Latn 56736
#define IDS_PERMISSION_REQUEST_CAMERA_ALLOWED_sr_Latn 56737
#define IDS_PERMISSION_REQUEST_MICROPHONE_sr_Latn 56738
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ALLOWED_sr_Latn 56739
#define IDS_PERMISSION_REQUEST_GEOLOCATION_sr_Latn 56740
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_sr_Latn 56741
#define IDS_PERMISSION_REQUEST_USING_LOCATION_sr_Latn 56742
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_sr_Latn 56743
#define IDS_PERMISSION_REQUEST_LAUNCH_sr 56744
#define IDS_PERMISSION_REQUEST_CAMERA_sr 56745
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ALLOWED_sr 56746
#define IDS_PERMISSION_REQUEST_INSTALL_WITH_HOST_sr 56747
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ALLOWED_sr 56748
#define IDS_PERMISSION_REQUEST_INSTALL_sr 56749
#define IDS_PERMISSION_REQUEST_MICROPHONE_ALLOWED_sr 56750
#define IDS_PERMISSION_REQUEST_CAMERA_ALLOWED_sr 56751
#define IDS_PERMISSION_REQUEST_MICROPHONE_sr 56752
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ALLOWED_sr 56753
#define IDS_PERMISSION_REQUEST_GEOLOCATION_sr 56754
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_sr 56755
#define IDS_PERMISSION_REQUEST_USING_LOCATION_sr 56756
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_sr 56757
#define IDS_PERMISSION_REQUEST_LAUNCH_sv 56758
#define IDS_PERMISSION_REQUEST_CAMERA_sv 56759
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ALLOWED_sv 56760
#define IDS_PERMISSION_REQUEST_INSTALL_WITH_HOST_sv 56761
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ALLOWED_sv 56762
#define IDS_PERMISSION_REQUEST_INSTALL_sv 56763
#define IDS_PERMISSION_REQUEST_MICROPHONE_ALLOWED_sv 56764
#define IDS_PERMISSION_REQUEST_CAMERA_ALLOWED_sv 56765
#define IDS_PERMISSION_REQUEST_MICROPHONE_sv 56766
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ALLOWED_sv 56767
#define IDS_PERMISSION_REQUEST_GEOLOCATION_sv 56768
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_sv 56769
#define IDS_PERMISSION_REQUEST_USING_LOCATION_sv 56770
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_sv 56771
#define IDS_PERMISSION_REQUEST_LAUNCH_sw 56772
#define IDS_PERMISSION_REQUEST_CAMERA_sw 56773
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ALLOWED_sw 56774
#define IDS_PERMISSION_REQUEST_INSTALL_WITH_HOST_sw 56775
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ALLOWED_sw 56776
#define IDS_PERMISSION_REQUEST_INSTALL_sw 56777
#define IDS_PERMISSION_REQUEST_MICROPHONE_ALLOWED_sw 56778
#define IDS_PERMISSION_REQUEST_CAMERA_ALLOWED_sw 56779
#define IDS_PERMISSION_REQUEST_MICROPHONE_sw 56780
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ALLOWED_sw 56781
#define IDS_PERMISSION_REQUEST_GEOLOCATION_sw 56782
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_sw 56783
#define IDS_PERMISSION_REQUEST_USING_LOCATION_sw 56784
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_sw 56785
#define IDS_PERMISSION_REQUEST_LAUNCH_ta 56786
#define IDS_PERMISSION_REQUEST_CAMERA_ta 56787
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ALLOWED_ta 56788
#define IDS_PERMISSION_REQUEST_INSTALL_WITH_HOST_ta 56789
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ALLOWED_ta 56790
#define IDS_PERMISSION_REQUEST_INSTALL_ta 56791
#define IDS_PERMISSION_REQUEST_MICROPHONE_ALLOWED_ta 56792
#define IDS_PERMISSION_REQUEST_CAMERA_ALLOWED_ta 56793
#define IDS_PERMISSION_REQUEST_MICROPHONE_ta 56794
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ALLOWED_ta 56795
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ta 56796
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ta 56797
#define IDS_PERMISSION_REQUEST_USING_LOCATION_ta 56798
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ta 56799
#define IDS_PERMISSION_REQUEST_LAUNCH_te 56800
#define IDS_PERMISSION_REQUEST_CAMERA_te 56801
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ALLOWED_te 56802
#define IDS_PERMISSION_REQUEST_INSTALL_WITH_HOST_te 56803
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ALLOWED_te 56804
#define IDS_PERMISSION_REQUEST_INSTALL_te 56805
#define IDS_PERMISSION_REQUEST_MICROPHONE_ALLOWED_te 56806
#define IDS_PERMISSION_REQUEST_CAMERA_ALLOWED_te 56807
#define IDS_PERMISSION_REQUEST_MICROPHONE_te 56808
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ALLOWED_te 56809
#define IDS_PERMISSION_REQUEST_GEOLOCATION_te 56810
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_te 56811
#define IDS_PERMISSION_REQUEST_USING_LOCATION_te 56812
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_te 56813
#define IDS_PERMISSION_REQUEST_LAUNCH_th 56814
#define IDS_PERMISSION_REQUEST_CAMERA_th 56815
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ALLOWED_th 56816
#define IDS_PERMISSION_REQUEST_INSTALL_WITH_HOST_th 56817
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ALLOWED_th 56818
#define IDS_PERMISSION_REQUEST_INSTALL_th 56819
#define IDS_PERMISSION_REQUEST_MICROPHONE_ALLOWED_th 56820
#define IDS_PERMISSION_REQUEST_CAMERA_ALLOWED_th 56821
#define IDS_PERMISSION_REQUEST_MICROPHONE_th 56822
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ALLOWED_th 56823
#define IDS_PERMISSION_REQUEST_GEOLOCATION_th 56824
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_th 56825
#define IDS_PERMISSION_REQUEST_USING_LOCATION_th 56826
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_th 56827
#define IDS_PERMISSION_REQUEST_LAUNCH_tr 56828
#define IDS_PERMISSION_REQUEST_CAMERA_tr 56829
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ALLOWED_tr 56830
#define IDS_PERMISSION_REQUEST_INSTALL_WITH_HOST_tr 56831
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ALLOWED_tr 56832
#define IDS_PERMISSION_REQUEST_INSTALL_tr 56833
#define IDS_PERMISSION_REQUEST_MICROPHONE_ALLOWED_tr 56834
#define IDS_PERMISSION_REQUEST_CAMERA_ALLOWED_tr 56835
#define IDS_PERMISSION_REQUEST_MICROPHONE_tr 56836
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ALLOWED_tr 56837
#define IDS_PERMISSION_REQUEST_GEOLOCATION_tr 56838
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_tr 56839
#define IDS_PERMISSION_REQUEST_USING_LOCATION_tr 56840
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_tr 56841
#define IDS_PERMISSION_REQUEST_LAUNCH_uk 56842
#define IDS_PERMISSION_REQUEST_CAMERA_uk 56843
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ALLOWED_uk 56844
#define IDS_PERMISSION_REQUEST_INSTALL_WITH_HOST_uk 56845
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ALLOWED_uk 56846
#define IDS_PERMISSION_REQUEST_INSTALL_uk 56847
#define IDS_PERMISSION_REQUEST_MICROPHONE_ALLOWED_uk 56848
#define IDS_PERMISSION_REQUEST_CAMERA_ALLOWED_uk 56849
#define IDS_PERMISSION_REQUEST_MICROPHONE_uk 56850
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ALLOWED_uk 56851
#define IDS_PERMISSION_REQUEST_GEOLOCATION_uk 56852
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_uk 56853
#define IDS_PERMISSION_REQUEST_USING_LOCATION_uk 56854
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_uk 56855
#define IDS_PERMISSION_REQUEST_LAUNCH_ur 56856
#define IDS_PERMISSION_REQUEST_CAMERA_ur 56857
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ALLOWED_ur 56858
#define IDS_PERMISSION_REQUEST_INSTALL_WITH_HOST_ur 56859
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ALLOWED_ur 56860
#define IDS_PERMISSION_REQUEST_INSTALL_ur 56861
#define IDS_PERMISSION_REQUEST_MICROPHONE_ALLOWED_ur 56862
#define IDS_PERMISSION_REQUEST_CAMERA_ALLOWED_ur 56863
#define IDS_PERMISSION_REQUEST_MICROPHONE_ur 56864
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ALLOWED_ur 56865
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ur 56866
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ur 56867
#define IDS_PERMISSION_REQUEST_USING_LOCATION_ur 56868
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ur 56869
#define IDS_PERMISSION_REQUEST_LAUNCH_uz 56870
#define IDS_PERMISSION_REQUEST_CAMERA_uz 56871
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ALLOWED_uz 56872
#define IDS_PERMISSION_REQUEST_INSTALL_WITH_HOST_uz 56873
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ALLOWED_uz 56874
#define IDS_PERMISSION_REQUEST_INSTALL_uz 56875
#define IDS_PERMISSION_REQUEST_MICROPHONE_ALLOWED_uz 56876
#define IDS_PERMISSION_REQUEST_CAMERA_ALLOWED_uz 56877
#define IDS_PERMISSION_REQUEST_MICROPHONE_uz 56878
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ALLOWED_uz 56879
#define IDS_PERMISSION_REQUEST_GEOLOCATION_uz 56880
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_uz 56881
#define IDS_PERMISSION_REQUEST_USING_LOCATION_uz 56882
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_uz 56883
#define IDS_PERMISSION_REQUEST_LAUNCH_vi 56884
#define IDS_PERMISSION_REQUEST_CAMERA_vi 56885
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ALLOWED_vi 56886
#define IDS_PERMISSION_REQUEST_INSTALL_WITH_HOST_vi 56887
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ALLOWED_vi 56888
#define IDS_PERMISSION_REQUEST_INSTALL_vi 56889
#define IDS_PERMISSION_REQUEST_MICROPHONE_ALLOWED_vi 56890
#define IDS_PERMISSION_REQUEST_CAMERA_ALLOWED_vi 56891
#define IDS_PERMISSION_REQUEST_MICROPHONE_vi 56892
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ALLOWED_vi 56893
#define IDS_PERMISSION_REQUEST_GEOLOCATION_vi 56894
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_vi 56895
#define IDS_PERMISSION_REQUEST_USING_LOCATION_vi 56896
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_vi 56897
#define IDS_PERMISSION_REQUEST_LAUNCH_zh_CN 56898
#define IDS_PERMISSION_REQUEST_CAMERA_zh_CN 56899
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ALLOWED_zh_CN 56900
#define IDS_PERMISSION_REQUEST_INSTALL_WITH_HOST_zh_CN 56901
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ALLOWED_zh_CN 56902
#define IDS_PERMISSION_REQUEST_INSTALL_zh_CN 56903
#define IDS_PERMISSION_REQUEST_MICROPHONE_ALLOWED_zh_CN 56904
#define IDS_PERMISSION_REQUEST_CAMERA_ALLOWED_zh_CN 56905
#define IDS_PERMISSION_REQUEST_MICROPHONE_zh_CN 56906
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ALLOWED_zh_CN 56907
#define IDS_PERMISSION_REQUEST_GEOLOCATION_zh_CN 56908
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_zh_CN 56909
#define IDS_PERMISSION_REQUEST_USING_LOCATION_zh_CN 56910
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_zh_CN 56911
#define IDS_PERMISSION_REQUEST_LAUNCH_zh_HK 56912
#define IDS_PERMISSION_REQUEST_CAMERA_zh_HK 56913
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ALLOWED_zh_HK 56914
#define IDS_PERMISSION_REQUEST_INSTALL_WITH_HOST_zh_HK 56915
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ALLOWED_zh_HK 56916
#define IDS_PERMISSION_REQUEST_INSTALL_zh_HK 56917
#define IDS_PERMISSION_REQUEST_MICROPHONE_ALLOWED_zh_HK 56918
#define IDS_PERMISSION_REQUEST_CAMERA_ALLOWED_zh_HK 56919
#define IDS_PERMISSION_REQUEST_MICROPHONE_zh_HK 56920
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ALLOWED_zh_HK 56921
#define IDS_PERMISSION_REQUEST_GEOLOCATION_zh_HK 56922
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_zh_HK 56923
#define IDS_PERMISSION_REQUEST_USING_LOCATION_zh_HK 56924
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_zh_HK 56925
#define IDS_PERMISSION_REQUEST_LAUNCH_zh_TW 56926
#define IDS_PERMISSION_REQUEST_CAMERA_zh_TW 56927
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ALLOWED_zh_TW 56928
#define IDS_PERMISSION_REQUEST_INSTALL_WITH_HOST_zh_TW 56929
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ALLOWED_zh_TW 56930
#define IDS_PERMISSION_REQUEST_INSTALL_zh_TW 56931
#define IDS_PERMISSION_REQUEST_MICROPHONE_ALLOWED_zh_TW 56932
#define IDS_PERMISSION_REQUEST_CAMERA_ALLOWED_zh_TW 56933
#define IDS_PERMISSION_REQUEST_MICROPHONE_zh_TW 56934
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ALLOWED_zh_TW 56935
#define IDS_PERMISSION_REQUEST_GEOLOCATION_zh_TW 56936
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_zh_TW 56937
#define IDS_PERMISSION_REQUEST_USING_LOCATION_zh_TW 56938
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_zh_TW 56939
#define IDS_PERMISSION_REQUEST_LAUNCH_zu 56940
#define IDS_PERMISSION_REQUEST_CAMERA_zu 56941
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ALLOWED_zu 56942
#define IDS_PERMISSION_REQUEST_INSTALL_WITH_HOST_zu 56943
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ALLOWED_zu 56944
#define IDS_PERMISSION_REQUEST_INSTALL_zu 56945
#define IDS_PERMISSION_REQUEST_MICROPHONE_ALLOWED_zu 56946
#define IDS_PERMISSION_REQUEST_CAMERA_ALLOWED_zu 56947
#define IDS_PERMISSION_REQUEST_MICROPHONE_zu 56948
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ALLOWED_zu 56949
#define IDS_PERMISSION_REQUEST_GEOLOCATION_zu 56950
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_zu 56951
#define IDS_PERMISSION_REQUEST_USING_LOCATION_zu 56952
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_zu 56953

// ---------------------------------------------------------------------------
// From permission_element_strings.h:

#define IDS_PERMISSION_REQUEST_CAMERA_ALLOWED 46110
#define IDS_PERMISSION_REQUEST_CAMERA 46111
#define IDS_PERMISSION_REQUEST_MICROPHONE_ALLOWED 46112
#define IDS_PERMISSION_REQUEST_MICROPHONE 46113
#define IDS_PERMISSION_REQUEST_GEOLOCATION_ALLOWED 46114
#define IDS_PERMISSION_REQUEST_GEOLOCATION 46115
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE_ALLOWED 46116
#define IDS_PERMISSION_REQUEST_CAMERA_MICROPHONE 46117
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION_ALLOWED 46118
#define IDS_PERMISSION_REQUEST_PRECISE_GEOLOCATION 46119
#define IDS_PERMISSION_REQUEST_USING_LOCATION 46120
#define IDS_PERMISSION_REQUEST_INSTALL 46121
#define IDS_PERMISSION_REQUEST_INSTALL_WITH_HOST 46122
#define IDS_PERMISSION_REQUEST_LAUNCH 46123

// ---------------------------------------------------------------------------
// From platform_locale_settings.h:

#define IDS_STANDARD_FONT_FAMILY 15100
#define IDS_FIXED_FONT_FAMILY 15101
#define IDS_SERIF_FONT_FAMILY 15102
#define IDS_SANS_SERIF_FONT_FAMILY 15103
#define IDS_NTP_FONT_FAMILY 15104
#define IDS_CURSIVE_FONT_FAMILY 15105
#define IDS_FANTASY_FONT_FAMILY 15106
#define IDS_MATH_FONT_FAMILY 15107
#define IDS_FIXED_FONT_FAMILY_JAPANESE 15108
#define IDS_STANDARD_FONT_FAMILY_JAPANESE 15109
#define IDS_SERIF_FONT_FAMILY_JAPANESE 15110
#define IDS_SANS_SERIF_FONT_FAMILY_JAPANESE 15111
#define IDS_STANDARD_FONT_FAMILY_KOREAN 15112
#define IDS_SERIF_FONT_FAMILY_KOREAN 15113
#define IDS_SANS_SERIF_FONT_FAMILY_KOREAN 15114
#define IDS_STANDARD_FONT_FAMILY_SIMPLIFIED_HAN 15115
#define IDS_SERIF_FONT_FAMILY_SIMPLIFIED_HAN 15116
#define IDS_SANS_SERIF_FONT_FAMILY_SIMPLIFIED_HAN 15117
#define IDS_STANDARD_FONT_FAMILY_TRADITIONAL_HAN 15118
#define IDS_SERIF_FONT_FAMILY_TRADITIONAL_HAN 15119
#define IDS_SANS_SERIF_FONT_FAMILY_TRADITIONAL_HAN 15120
#define IDS_STANDARD_FONT_FAMILY_DEVANAGARI 15121
#define IDS_FIXED_FONT_FAMILY_DEVANAGARI 15122
#define IDS_SERIF_FONT_FAMILY_DEVANAGARI 15123
#define IDS_SANS_SERIF_FONT_FAMILY_DEVANAGARI 15124
#define IDS_MINIMUM_FONT_SIZE 15125
#define IDS_MINIMUM_LOGICAL_FONT_SIZE 15126

// ---------------------------------------------------------------------------
// From services_strings.h:

#define IDS_PROXY_RESOLVER_DISPLAY_NAME 45930
#define IDS_WINDOWS_SYSTEM_PROXY_RESOLVER_DISPLAY_NAME 45931
#define IDS_PDF_OCR_RESULT_BEGIN 45932
#define IDS_PDF_OCR_RESULT_END 45933

// ---------------------------------------------------------------------------
// From ui_strings.h:

#define IDS_TIME_SECS 49900
#define IDS_TIME_LONG_SECS 49901
#define IDS_TIME_LONG_SECS_2ND 49902
#define IDS_TIME_MINS 49903
#define IDS_TIME_LONG_MINS 49904
#define IDS_TIME_LONG_MINS_1ST 49905
#define IDS_TIME_LONG_MINS_2ND 49906
#define IDS_TIME_HOURS 49907
#define IDS_TIME_HOURS_1ST 49908
#define IDS_TIME_HOURS_2ND 49909
#define IDS_TIME_DAYS 49910
#define IDS_TIME_DAYS_1ST 49911
#define IDS_TIME_MONTHS 49912
#define IDS_TIME_YEARS 49913
#define IDS_TIME_REMAINING_SECS 49914
#define IDS_TIME_REMAINING_LONG_SECS 49915
#define IDS_TIME_REMAINING_MINS 49916
#define IDS_TIME_REMAINING_LONG_MINS 49917
#define IDS_TIME_REMAINING_HOURS 49918
#define IDS_TIME_REMAINING_DAYS 49919
#define IDS_TIME_REMAINING_MONTHS 49920
#define IDS_TIME_REMAINING_YEARS 49921
#define IDS_TIME_ELAPSED_SECS 49922
#define IDS_TIME_ELAPSED_LONG_SECS 49923
#define IDS_TIME_ELAPSED_MINS 49924
#define IDS_TIME_ELAPSED_LONG_MINS 49925
#define IDS_TIME_ELAPSED_HOURS 49926
#define IDS_TIME_ELAPSED_DAYS 49927
#define IDS_TIME_ELAPSED_MONTHS 49928
#define IDS_TIME_ELAPSED_YEARS 49929
#define IDS_TIME_TITLE_CASE_ELAPSED_LONG_SECS 49930
#define IDS_TIME_TITLE_CASE_ELAPSED_LONG_MINS 49931
#define IDS_TIME_TITLE_CASE_ELAPSED_HOURS 49932
#define IDS_TIME_TITLE_CASE_ELAPSED_DAYS 49933
#define IDS_TIME_TITLE_CASE_ELAPSED_MONTHS 49934
#define IDS_TIME_TITLE_CASE_ELAPSED_YEARS 49935
#define IDS_PAST_TIME_TODAY 49936
#define IDS_PAST_TIME_YESTERDAY 49937
#define IDS_TIME_TOMORROW 49938
#define IDS_APP_MENU_EMPTY_SUBMENU 49939
#define IDS_APP_MENU_AX_ANNOUNCE_EMPTY_SUBMENU 49940
#define IDS_CLIPBOARD_MENU_HTML 49941
#define IDS_CLIPBOARD_MENU_IMAGE 49942
#define IDS_CLIPBOARD_MENU_RTF_CONTENT 49943
#define IDS_CLIPBOARD_MENU_WEB_SMART_PASTE 49944
#define IDS_CLIPBOARD_MENU_CLIPBOARD 49945
#define IDS_CLIPBOARD_MENU_DELETE_ALL 49946
#define IDS_EXTENSION_PINNED 49947
#define IDS_EXTENSION_UNPINNED 49948
#define IDS_NEW_BADGE 49949
#define IDS_NEW_BADGE_SCREEN_READER_MESSAGE 49950
#define IDS_PREVIEW_BADGE 49951
#define IDS_PREVIEW_BADGE_SCREEN_READER_MESSAGE 49952
#define IDS_SENTENCE_END 49953
#define IDS_CONCAT_TWO_STRINGS_WITH_COMMA 49954
#define IDS_CONCAT_THREE_STRINGS_WITH_COMMA 49955
#define IDS_CONCAT_TWO_STRINGS_WITH_PERIODS 49956
#define IDS_SELECT_UPLOAD_FOLDER_DIALOG_TITLE 49960
#define IDS_CONTENT_CONTEXT_WRITING_DIRECTION_MENU 49968
#define IDS_CONTENT_CONTEXT_WRITING_DIRECTION_DEFAULT 49969
#define IDS_CONTENT_CONTEXT_WRITING_DIRECTION_LTR 49970
#define IDS_CONTENT_CONTEXT_WRITING_DIRECTION_RTL 49971
#define IDS_SELECT_FOLDER_DIALOG_TITLE 49972
#define IDS_SAVE_AS_DIALOG_TITLE 49973
#define IDS_OPEN_FILE_DIALOG_TITLE 49974
#define IDS_OPEN_FILES_DIALOG_TITLE 49975
#define IDS_SAVEAS_ALL_FILES 49976
#define IDS_SELECT_UPLOAD_FOLDER_DIALOG_UPLOAD_BUTTON 49977
#define IDS_APP_ACCNAME_BACK 49978
#define IDS_APP_ACCNAME_CENTER 49979
#define IDS_APP_ACCNAME_CLOSE 49980
#define IDS_APP_ACCNAME_MINIMIZE 49981
#define IDS_APP_ACCNAME_MAXIMIZE 49982
#define IDS_APP_ACCNAME_RESTORE 49983
#define IDS_APP_ACCNAME_MENU 49984
#define IDS_APP_ACCNAME_COLOR_CHOOSER_HEX_INPUT 49985
#define IDS_APP_SCROLLBAR_CXMENU_SCROLLHERE 49986
#define IDS_APP_SCROLLBAR_CXMENU_SCROLLLEFTEDGE 49987
#define IDS_APP_SCROLLBAR_CXMENU_SCROLLRIGHTEDGE 49988
#define IDS_APP_SCROLLBAR_CXMENU_SCROLLHOME 49989
#define IDS_APP_SCROLLBAR_CXMENU_SCROLLEND 49990
#define IDS_APP_SCROLLBAR_CXMENU_SCROLLPAGEUP 49991
#define IDS_APP_SCROLLBAR_CXMENU_SCROLLPAGEDOWN 49992
#define IDS_APP_SCROLLBAR_CXMENU_SCROLLLEFT 49993
#define IDS_APP_SCROLLBAR_CXMENU_SCROLLRIGHT 49994
#define IDS_APP_SCROLLBAR_CXMENU_SCROLLUP 49995
#define IDS_APP_SCROLLBAR_CXMENU_SCROLLDOWN 49996
#define IDS_APP_TABLE_COLUMN_NOT_SORTED_ACCNAME 49997
#define IDS_APP_TABLE_COLUMN_SORTED_ASC_ACCNAME 49998
#define IDS_APP_TABLE_COLUMN_SORTED_DESC_ACCNAME 49999
#define IDS_APP_TABLE_HEADER_NOT_SORTED_ACCNAME 50000
#define IDS_APP_TABLE_HEADER_NOT_SORTED_ACCNAME_MAC 50001
#define IDS_APP_TABLE_HEADER_SORTED_ASC_ACCNAME 50002
#define IDS_APP_TABLE_HEADER_SORTED_DESC_ACCNAME 50003
#define IDS_APP_UNDO 50004
#define IDS_APP_CUT 50005
#define IDS_APP_COPY 50006
#define IDS_APP_PASTE 50007
#define IDS_APP_DELETE 50008
#define IDS_APP_SELECT_ALL 50009
#define IDS_APP_SELECT 50010
#define IDS_CONTENT_CONTEXT_EMOJI 50011
#define IDS_APP_OK 50014
#define IDS_APP_CANCEL 50015
#define IDS_APP_CLOSE 50016
#define IDS_APP_CONTINUE 50017
#define IDS_APP_TURN_ON 50018
#define IDS_APP_ESC_KEY 50019
#define IDS_APP_TAB_KEY 50020
#define IDS_APP_INSERT_KEY 50021
#define IDS_APP_HOME_KEY 50022
#define IDS_APP_DELETE_KEY 50023
#define IDS_APP_END_KEY 50024
#define IDS_APP_PAGEUP_KEY 50025
#define IDS_APP_PAGEDOWN_KEY 50026
#define IDS_APP_LEFT_ARROW_KEY 50027
#define IDS_APP_RIGHT_ARROW_KEY 50028
#define IDS_APP_UP_ARROW_KEY 50029
#define IDS_APP_DOWN_ARROW_KEY 50030
#define IDS_APP_ENTER_KEY 50031
#define IDS_APP_SPACE_KEY 50032
#define IDS_APP_F1_KEY 50033
#define IDS_APP_F4_KEY 50034
#define IDS_APP_F5_KEY 50035
#define IDS_APP_F6_KEY 50036
#define IDS_APP_F11_KEY 50037
#define IDS_APP_BACKSPACE_KEY 50038
#define IDS_APP_COMMA_KEY 50039
#define IDS_APP_PERIOD_KEY 50040
#define IDS_APP_MEDIA_NEXT_TRACK_KEY 50041
#define IDS_APP_MEDIA_PLAY_PAUSE_KEY 50042
#define IDS_APP_MEDIA_PREV_TRACK_KEY 50043
#define IDS_APP_MEDIA_STOP_KEY 50044
#define IDS_APP_ALT_KEY 50045
#define IDS_APP_COMMAND_KEY 50046
#define IDS_APP_CTRL_KEY 50047
#define IDS_APP_SEARCH_KEY 50048
#define IDS_APP_SHIFT_KEY 50049
#define IDS_APP_SUPER_KEY 50051
#define IDS_APP_BYTES 50055
#define IDS_APP_KIBIBYTES 50056
#define IDS_APP_MEBIBYTES 50057
#define IDS_APP_GIBIBYTES 50058
#define IDS_APP_TEBIBYTES 50059
#define IDS_APP_PEBIBYTES 50060
#define IDS_APP_BYTES_PER_SECOND 50061
#define IDS_APP_KIBIBYTES_PER_SECOND 50062
#define IDS_APP_MEBIBYTES_PER_SECOND 50063
#define IDS_APP_GIBIBYTES_PER_SECOND 50064
#define IDS_APP_TEBIBYTES_PER_SECOND 50065
#define IDS_APP_PEBIBYTES_PER_SECOND 50066
#define IDS_MESSAGE_CENTER_ACCESSIBLE_NAME 50067
#define IDS_MESSAGE_CENTER_NOTIFICATION_ACCESSIBLE_NAME 50068
#define IDS_MESSAGE_CENTER_NOTIFICATION_ACCESSIBLE_NAME_PLURAL 50069
#define IDS_MESSAGE_CENTER_EXPAND_NOTIFICATION 50070
#define IDS_MESSAGE_CENTER_COLLAPSE_NOTIFICATION 50071
#define IDS_MESSAGE_CENTER_LIST_NOTIFICATION_MESSAGE_WITH_DIVIDER 50072
#define IDS_MESSAGE_CENTER_LIST_NOTIFICATION_HEADER_OVERFLOW_INDICATOR 50073
#define IDS_MESSAGE_CENTER_NOTIFICATION_PROGRESS_PERCENTAGE 50074
#define IDS_MESSAGE_CENTER_NOTIFICATION_CHROMEOS_SYSTEM 50075
#define IDS_MESSAGE_CENTER_NOTIFICATION_INLINE_REPLY_PLACEHOLDER 50076
#define IDS_MESSAGE_CENTER_NOTIFICATION_INLINE_REPLY_ACCESSIBLE_NAME 50077
#define IDS_MESSAGE_NOTIFICATION_NOW_STRING_SHORTEST 50078
#define IDS_MESSAGE_NOTIFICATION_DURATION_MINUTES_SHORTEST 50079
#define IDS_MESSAGE_NOTIFICATION_DURATION_HOURS_SHORTEST 50080
#define IDS_MESSAGE_NOTIFICATION_DURATION_DAYS_SHORTEST 50081
#define IDS_MESSAGE_NOTIFICATION_DURATION_YEARS_SHORTEST 50082
#define IDS_MESSAGE_NOTIFICATION_DURATION_MINUTES_SHORTEST_FUTURE 50083
#define IDS_MESSAGE_NOTIFICATION_DURATION_HOURS_SHORTEST_FUTURE 50084
#define IDS_MESSAGE_NOTIFICATION_DURATION_DAYS_SHORTEST_FUTURE 50085
#define IDS_MESSAGE_NOTIFICATION_DURATION_YEARS_SHORTEST_FUTURE 50086
#define IDS_MESSAGE_CENTER_BLOCK_ALL_NOTIFICATIONS_SITE 50087
#define IDS_MESSAGE_CENTER_BLOCK_ALL_NOTIFICATIONS_APP 50088
#define IDS_MESSAGE_CENTER_BLOCK_ALL_NOTIFICATIONS 50089
#define IDS_MESSAGE_CENTER_DONT_BLOCK_NOTIFICATIONS 50090
#define IDS_MESSAGE_CENTER_SETTINGS_DONE 50091
#define IDS_MESSAGE_CENTER_CLOSE_NOTIFICATION_BUTTON_ACCESSIBLE_NAME 50092
#define IDS_MESSAGE_CENTER_CLOSE_NOTIFICATION_BUTTON_TOOLTIP 50093
#define IDS_MESSAGE_CENTER_NOTIFICATION_SNOOZE_BUTTON_TOOLTIP 50094
#define IDS_MESSAGE_NOTIFICATION_SETTINGS_BUTTON_ACCESSIBLE_NAME 50095
#define IDS_MESSAGE_NOTIFICATION_ACCESSIBLE_NAME 50096
#define IDS_MESSAGE_NOTIFICATION_SEND_TAB_TO_SELF_CONFIRMATION_FAILURE_TITLE 50097
#define IDS_MESSAGE_NOTIFICATION_SEND_TAB_TO_SELF_CONFIRMATION_FAILURE_MESSAGE 50098
#define IDS_CLIPBOARD_HISTORY_MENU_PNG_IMAGE 50100
#define IDS_CLIPBOARD_HISTORY_MENU_HTML_IMAGE 50101
#define IDS_CLIPBOARD_HISTORY_MENU_TITLE 50102
#define IDS_CLIPBOARD_HISTORY_DELETE_ITEM_TEXT 50103
#define IDS_CLIPBOARD_HISTORY_DELETE_BUTTON_HOVER_TEXT 50104
#define IDS_CLIPBOARD_HISTORY_ITEM_DELETION 50105
#define IDS_DISPLAY_TOUCH_CALIBRATION_EXIT_LABEL 50106
#define IDS_DISPLAY_TOUCH_CALIBRATION_SECONDARY_SKIP_LABEL 50107
#define IDS_DISPLAY_TOUCH_CALIBRATION_PRIMARY_SKIP_LABEL 50108
#define IDS_DISPLAY_TOUCH_CALIBRATION_HINT_LABEL_TEXT 50109
#define IDS_DISPLAY_TOUCH_CALIBRATION_HINT_SUBLABEL_TEXT 50110
#define IDS_DISPLAY_TOUCH_CALIBRATION_TAP_HERE_LABEL 50111
#define IDS_DISPLAY_TOUCH_CALIBRATION_FINISH_LABEL 50112
#define IDS_DISPLAY_TOUCH_CALIBRATION_A11Y_TARGET_MOVED_TOP_RIGHT 50113
#define IDS_DISPLAY_TOUCH_CALIBRATION_A11Y_TARGET_MOVED_BOTTOM_LEFT 50114
#define IDS_DISPLAY_TOUCH_CALIBRATION_A11Y_TARGET_MOVED_BOTTOM_RIGHT 50115
#define IDS_DISPLAY_NAME_UNKNOWN 50116
#define IDS_DISPLAY_NAME_INTERNAL 50117
#define IDS_SATURATED_BADGE_CONTENT 50118
#define IDS_BADGE_UNREAD_NOTIFICATIONS_SATURATED 50119
#define IDS_BADGE_UNREAD_NOTIFICATIONS_UNSPECIFIED 50120
#define IDS_BADGE_UNREAD_NOTIFICATIONS 50121
#define IDS_BROWSER_SHARING_CLICK_TO_CALL_DIALOG_TITLE_LABEL 50122
#define IDS_BROWSER_SHARING_CLICK_TO_CALL_DIALOG_TITLE_NO_DEVICES 50123
#define IDS_BROWSER_SHARING_CLICK_TO_CALL_DIALOG_CALL_BUTTON_LABEL 50124
#define IDS_BROWSER_SHARING_OMNIBOX_SENDING_LABEL 50125
#define IDS_BROWSER_SHARING_DIALOG_DEVICE_SUBTITLE_LAST_ACTIVE_DAYS 50126
#define IDS_BROWSER_SHARING_CLICK_TO_CALL_DIALOG_INITIATING_ORIGIN 50127
#define IDS_BROWSER_SHARING_CONTENT_TYPE_TEXT 50128
#define IDS_BROWSER_SHARING_CONTENT_TYPE_NUMBER 50129
#define IDS_BROWSER_SHARING_ERROR_DIALOG_TITLE_GENERIC_ERROR 50130
#define IDS_BROWSER_SHARING_ERROR_DIALOG_TITLE_INTERNAL_ERROR 50131
#define IDS_BROWSER_SHARING_ERROR_DIALOG_TEXT_NETWORK_ERROR 50132
#define IDS_BROWSER_SHARING_ERROR_DIALOG_TEXT_DEVICE_ACK_TIMEOUT 50133
#define IDS_BROWSER_SHARING_ERROR_DIALOG_TEXT_INTERNAL_ERROR 50134
#define IDS_BROWSER_SHARING_SHARED_CLIPBOARD_ERROR_DIALOG_TITLE_PAYLOAD_TOO_LARGE 50135
#define IDS_BROWSER_SHARING_SHARED_CLIPBOARD_ERROR_DIALOG_TEXT_PAYLOAD_TOO_LARGE 50136
#define IDS_SETTINGS_PASSWORD_SHOW 50140
#define IDS_SETTINGS_PASSWORD_HIDE 50141
#define IDS_TABLE_VIEW_AX_ANNOUNCE_ROW_SELECTED 50142
#define IDS_LINK_COPIED 50143
#define IDS_INCOGNITO_NAVIGATION_BLOCKED_PAGE_TITLE 50144
#define IDS_INCOGNITO_NAVIGATION_MESSAGE 50145
#define IDS_INCOGNITO_NAVIGATION_INSTRUCTIONS 50146
#define IDS_INCOGNITO_NAVIGATION_INSTRUCTIONS_STEP_1 50147
#define IDS_INCOGNITO_NAVIGATION_INSTRUCTIONS_STEP_2 50148
#define IDS_INCOGNITO_NAVIGATION_INSTRUCTIONS_STEP_3 50149
#define IDS_INCOGNITO_NAVIGATION_INSTRUCTIONS_STEP_4 50150
#define IDS_INCOGNITO_NAVIGATION_MISSING_TITLE 50151
#define IDS_INCOGNITO_NAVIGATION_MISSING_EXTENSIONS 50152

#endif  // CEF_INCLUDE_CEF_PACK_STRINGS_H_
